Type: Package
Package: scDiagnostics
Title: Cell type annotation diagnostics
Version: 1.4.0
Authors@R: c(
    person("Anthony", "Christidis", role = c("aut", "cre"), 
            email = "anthony-alexander_christidis@hms.harvard.edu", 
            comment = c(ORCID = "0000-0002-4565-6279")),
    person("Andrew", "Ghazi", role = "aut"),
    person("Smriti", "Chawla", role = "aut"),
    person("Nitesh", "Turaga", role = "ctb"),
    person("Ludwig", "Geistlinger", role = "aut"),
    person("Robert", "Gentleman", role = "aut")
  )
Description: The scDiagnostics package provides diagnostic plots to
    assess the quality of cell type assignments from single cell gene
    expression profiles. The implemented functionality allows to
    assess the reliability of cell type annotations, investigate gene
    expression patterns, and explore relationships between different
    cell types in query and reference datasets allowing users to
    detect potential misalignments between reference and query
    datasets. The package also provides visualization capabilities for
    diagnostics purposes.    
License: Artistic-2.0
URL: https://github.com/ccb-hms/scDiagnostics
BugReports: https://github.com/ccb-hms/scDiagnostics/issues
Depends: R (>= 4.4.0)
Imports: SingleCellExperiment, methods, isotree, FNN, igraph, ggplot2,
        GGally, ggridges, SummarizedExperiment, ranger, transport,
        cramer, rlang, bluster, scales, MASS, stringr, Matrix,
        grDevices
Suggests: AUCell, BiocStyle, knitr, rmarkdown, scran, scRNAseq,
        SingleR, celldex, scuttle, scater, dplyr, ComplexHeatmap, grid,
        testthat (>= 3.0.0)
VignetteBuilder: knitr
biocViews: Annotation, Classification, Clustering, GeneExpression,
        RNASeq, SingleCell, Software, Transcriptomics
Encoding: UTF-8
LazyDataCompression: xz
RoxygenNote: 7.3.2
Config/testthat/edition: 3
LazyData: true
git_url: https://git.bioconductor.org/packages/scDiagnostics
git_branch: RELEASE_3_22
git_last_commit: 7dca061
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 06:21:07 UTC; biocbuild
Author: Anthony Christidis [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4565-6279>),
  Andrew Ghazi [aut],
  Smriti Chawla [aut],
  Nitesh Turaga [ctb],
  Ludwig Geistlinger [aut],
  Robert Gentleman [aut]
Maintainer: Anthony Christidis <anthony-alexander_christidis@hms.harvard.edu>

# Generated by roxygen2: do not edit by hand

S3method(plot,calculateAveragePairwiseCorrelationObject)
S3method(plot,calculateCellDistancesObject)
S3method(plot,calculateCellSimilarityPCAObject)
S3method(plot,calculateDiscriminantSpaceObject)
S3method(plot,calculateGraphIntegrationObject)
S3method(plot,calculateSIRSpaceObject)
S3method(plot,calculateTopLoadingGeneShiftsObject)
S3method(plot,calculateWassersteinDistanceObject)
S3method(plot,compareMarkersObject)
S3method(plot,comparePCAObject)
S3method(plot,comparePCASubspaceObject)
S3method(plot,detectAnomalyObject)
S3method(plot,regressPCObject)
export(boxplotPCA)
export(calculateAveragePairwiseCorrelation)
export(calculateCategorizationEntropy)
export(calculateCellDistances)
export(calculateCellDistancesSimilarity)
export(calculateCellSimilarityPCA)
export(calculateCramerPValue)
export(calculateDiscriminantSpace)
export(calculateGraphIntegration)
export(calculateHVGOverlap)
export(calculateHotellingPValue)
export(calculateMMDPValue)
export(calculateSIRSpace)
export(calculateTopLoadingGeneShifts)
export(calculateVarImpOverlap)
export(calculateWassersteinDistance)
export(compareMarkers)
export(comparePCA)
export(comparePCASubspace)
export(detectAnomaly)
export(histQCvsAnnotation)
export(plotCellTypeMDS)
export(plotCellTypePCA)
export(plotGeneExpressionDimred)
export(plotGeneSetScores)
export(plotMarkerExpression)
export(plotPairwiseDistancesDensity)
export(plotQCvsAnnotation)
export(processPCA)
export(projectPCA)
export(projectSIR)
export(regressPC)
import(SingleCellExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,unit)
importFrom(ggplot2,ylim)
importFrom(ggridges,geom_density_ridges)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(methods,is)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,approxfun)
importFrom(stats,as.formula)
importFrom(stats,cancor)
importFrom(stats,cmdscale)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,density)
importFrom(stats,dist)
importFrom(stats,formula)
importFrom(stats,mahalanobis)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,reshape)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(tools,toTitleCase)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,tail)

# `scDiagnostics`: diagnostic functions to assess the quality of cell type annotations in single-cell RNA-seq data

[![BioC Release Years in BioC](https://www.bioconductor.org/shields/years-in-bioc/scDiagnostics.svg)](https://www.bioconductor.org/packages/release/bioc/html/scDiagnostics.html) 

# Overview

The accurate annotation of cell types is a critical step in single-cell RNA-sequencing (scRNA-seq) analysis. While annotation transfer from a reference dataset offers a convenient and automated approach, it can also introduce biases and errors if not performed carefully.

`scDiagnostics` is an R package designed to address this challenge by providing a comprehensive set of diagnostic tools for evaluating the quality of cell type annotations in scRNA-seq data. With `scDiagnostics`, researchers can systematically assess the compatibility and accuracy of annotations, ensuring reliable and reproducible results in their scRNA-seq analysis workflow.

# Installation

## Installation from Bioconductor (Release)

Users interested in using the stable release version of the `scDiagnostics` package: please follow the installation instructions [**here**](https://bioconductor.org/packages/release/bioc/html/scDiagnostics.html). This is the recommended way of installing the package.

## Installation from GitHub (Development)

To install the development version of the package from GitHub use the following command:

``` r
BiocManager::install("ccb-hms/scDiagnostics")
```

To build the package vignettes upon installation use:

``` r
BiocManager::install("ccb-hms/scDiagnostics",
                     build_vignettes = TRUE,
                     dependencies = TRUE)
```

# Usage

See the [**scDiagnostics website**](https://ccb-hms.github.io/scDiagnostics/) for an overview of the functionality of the package. The individual documentation of each available function incl. usage details and executable examples is available in the [**reference tab**](https://ccb-hms.github.io/scDiagnostics/reference/index.html).

## Key Features

See the [**Getting Started with scDiagnostics**](https://ccb-hms.github.io/scDiagnostics/articles/scDiagnostics.html) vignette for an overview of the main functionality of the package. As listed below, additional vignettes are available for specific diagnostic tasks.

-   [**Visualization of Cell Type Annotations**](https://ccb-hms.github.io/scDiagnostics/articles/VisualizationTools.html): Illustrates the distributions of cell type annotations of the query and reference dataset, allowing the user to identify potential differences in the cell type composition between datasets. Also provides functionality for assessing the impact of frequently used QC criteria on the cell type annotation confidence, allowing the user to identify systematic relationships between QC metrics and the predicted cell type categories.

-   [**Evaluation of Dataset and Marker Gene Alignment**](https://ccb-hms.github.io/scDiagnostics/articles/DatasetAlignment.html): Provides functionality for assessing dataset alignment through quantitative comparison of query-to-reference projections in reduced dimension space. Additional functionality for assessing marker gene expression across datasets allows the user to identify potential misalignments between reference and query on the level of individual genes.

-   [**Detection and Analysis of Annotation Anomalies**](https://ccb-hms.github.io/scDiagnostics/articles/AnnotationAnomalies.html): Focuses on identifying inconsistencies or anomalies in cell type annotations between the query and reference datasets through comparison of expert annotations with annotations derived from automated methods. By highlighting discrepancies that could be indicative of potential errors, this feature aids in refining and improving the accuracy and reliability of cell type classifications.

#' @title Histograms: QC Stats and Annotation Scores Visualization
#'
#' @description
#' This function generates histograms for visualizing the distribution of quality control (QC) statistics and
#' annotation scores associated with cell types in single-cell genomic data.
#'
#' @details The particularly useful in the analysis of data from single-cell experiments,
#' where understanding the distribution of these metrics is crucial for quality assessment and
#' interpretation of cell type annotations.
#'
#' @param sce_object  A \code{\linkS4class{SingleCellExperiment}} containing the single-cell
#' expression data and metadata.
#' @param cell_type_col The column name in the \code{colData} of \code{sce_object}
#' that contains the cell type labels.
#' @param cell_types A vector of cell types to plot (e.g., c("T-cell", "B-cell")).
#' Defaults to \code{NULL}, which will include all the cells.
#' @param qc_col A column name in the \code{colData} of \code{sce_object} that
#' contains the QC stats of interest.
#' @param score_col The column name in the \code{colData} of \code{sce_object} that
#' contains the cell type scores.
#' @param max_cells Maximum number of cells to retain. If the object has fewer cells, it is returned unchanged.
#'                  Default is NULL.
#'
#' @return A object containing two histograms displayed side by side.
#' The first histogram represents the distribution of QC stats,
#' and the second histogram represents the distribution of annotation scores.
#'
#' @examples
# Load data
#' data("query_data")
#'
#' # Generate histograms
#' histQCvsAnnotation(sce_object = query_data,
#'                    cell_type_col = "SingleR_annotation",
#'                    cell_types = c("CD4", "CD8"),
#'                    qc_col = "percent_mito",
#'                    score_col = "annotation_scores")
#'
#' histQCvsAnnotation(sce_object = query_data,
#'                    cell_type_col = "SingleR_annotation",
#'                    cell_types = NULL,
#'                    qc_col = "percent_mito",
#'                    score_col = "annotation_scores")
#'
#' @export
#'
# Function to plot histogram of QC scores and annotation scores
histQCvsAnnotation <- function(sce_object,
                               cell_type_col,
                               cell_types = NULL,
                               qc_col,
                               score_col,
                               max_cells = NULL) {

    # Check standard input arguments
    argumentCheck(query_data = sce_object,
                  query_cell_type_col = cell_type_col,
                  max_cells_query = max_cells)

    # Convert cell type columns to character if needed
    sce_object <- convertColumnsToCharacter(sce_object = sce_object,
                                            convert_cols = cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = sce_object,
                                  query_cell_type_col = cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = 10)

    # Downsample SCE object
    sce_object <- downsampleSCE(sce_object = sce_object,
                               max_cells = max_cells,
                               cell_type_col = cell_type_col,
                               cell_types = cell_types)

    # Check if qc_col is a valid column name in sce_object
    if (!qc_col %in% names(colData(sce_object))) {
        stop("qc_col: '",
             qc_col,
             "' is not a valid column name in sce_object.")
    }

    # Check if score_col is a valid column name in sce_object
    if (!score_col %in% names(colData(sce_object))) {
        stop("score_col: '",
             score_col,
             "' is not a valid column name in sce_object.")
    }

    # Filter cells based on cell_types if specified
    if (!is.null(cell_types)) {
        index <- which(sce_object[[cell_type_col]] %in% cell_types)
        sce_object <- sce_object[, index]
    }

    # Extract QC stats, scores, and cell_types
    qc_stats <- sce_object[[qc_col]]
    cell_type_scores <- sce_object[[score_col]]

    # Combine QC stats, scores, and cell_types into a data frame
    data <- data.frame(stats = c(qc_stats, cell_type_scores),
                       Metric = c(rep("QC Statistics", length(qc_stats)),
                                  rep("Annotation Scores",
                                      length(cell_type_scores))))
    data[["Metric"]] <- factor(data[["Metric"]],
                               levels = c("QC Statistics", "Annotation Scores"))

    # Create histogram plots
    hist_plot <- ggplot2::ggplot(data, ggplot2::aes(x = .data[["stats"]],
                                                    fill = .data[["Metric"]])) +
        ggplot2::geom_histogram(bins = 30, alpha = 0.5, position = "identity",
                                color = "gray50") +
        ggplot2::facet_wrap(~ .data[["Metric"]], scales = "free") +
        ggplot2::labs(x = "", y = "Frequency", fill = "Metric") +
        ggplot2::theme_bw() +
        ggplot2::theme(
            strip.background = ggplot2::element_rect(
                fill = "white", color = "black", linewidth = 0.5),
            legend.position = "none",
            panel.grid.minor = ggplot2::element_blank(),
            panel.grid.major = ggplot2::element_line(color = "gray",
                                                     linetype = "dotted"),
            plot.title = ggplot2::element_text(size = 14,
                                               face = "bold",
                                               hjust = 0.5),
            axis.title = ggplot2::element_text(size = 12),
            axis.text = ggplot2::element_text(size = 10))

    # Return the list of plots
    return(hist_plot)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("query_data")

test_that("histQCvsAnnotation works correctly with default parameters", {
    # Generate histograms
    histograms <- histQCvsAnnotation(
        sce_object = query_data,
        cell_type_col = "SingleR_annotation",
        qc_col = "percent_mito",
        score_col = "annotation_scores"
    )

    # Check if the output is a ggplot object
    expect_true(inherits(histograms, "ggplot"))
})

test_that("histQCvsAnnotation works correctly with specific cell types", {
    # Generate histograms for specific cell types
    histograms <- histQCvsAnnotation(
        sce_object = query_data,
        cell_type_col = "SingleR_annotation",
        cell_types = c("CD4", "CD8"),
        qc_col = "percent_mito",
        score_col = "annotation_scores"
    )

    # Check if the output is a ggplot object
    expect_true(inherits(histograms, "ggplot"))
})

test_that("histQCvsAnnotation handles incorrect parameters", {
    expect_error(histQCvsAnnotation(
        sce_object = query_data,
        cell_type_col = "SingleR_annotation",
        qc_col = "invalid_column",
        score_col = "annotation_scores"
    ), "qc_col: 'invalid_column' is not a valid column name in sce_object.")

    expect_error(histQCvsAnnotation(
        sce_object = query_data,
        cell_type_col = "SingleR_annotation",
        qc_col = "percent_mito",
        score_col = "invalid_column"
    ), "score_col: 'invalid_column' is not a valid column name in sce_object.")
})

test_that("histQCvsAnnotation works with all cell types", {
    # Generate histograms for all cell types
    histograms <- histQCvsAnnotation(
        sce_object = query_data,
        cell_type_col = "SingleR_annotation",
        cell_types = NULL,
        qc_col = "percent_mito",
        score_col = "annotation_scores"
    )

    # Check if the output is a ggplot object
    expect_true(inherits(histograms, "ggplot"))
})

#' scDiagnostics: Single-Cell Diagnostics Package
#'
#' @description
#' `scDiagnostics` is a comprehensive toolkit designed for the analysis and diagnostics of single-cell RNA sequencing (scRNA-seq) data. This package provides functionalities for comparing principal components, visualizing canonical correlation analysis (CCA) outputs, and plotting cell type-specific MDS and PCA projections.
#'
#' @details
#' The package includes the following key functionalities, organized by their specific purposes:
#'
#' @section Visualization of Cell Type Annotations:
#' Functions for visualizing differences between query and reference datasets across multiple cell types.
#' \itemize{
#'   \item \code{\link{boxplotPCA}}: Boxplots of PCA scores for cell types.
#'   \item \code{\link{calculateDiscriminantSpace}}: Calculates discriminant space, with a plot method for visualization.
#'   \item \code{\link{plotCellTypeMDS}}: Creates MDS plots for cell types using query and reference datasets.
#'   \item \code{\link{plotCellTypePCA}}: Plots principal components for different cell types.
#' }
#'
#' @section Evaluation of Dataset Alignment:
#' Functions for visualizing differences between query and reference datasets for a specific cell type.
#' \itemize{
#'   \item \code{\link{calculateGraphIntegration}}: Calculate a graph network and form communities of cells.
#'   \item \code{\link{calculateWassersteinDistance}}: Wasserstein distance for different cell types.
#'   \item \code{\link{comparePCA}}: Compares PCA results, with a plot method for visualization.
#'   \item \code{\link{comparePCASubspace}}: Compares PCA subspace, with a plot method for visualization.
#'   \item \code{\link{plotPairwiseDistancesDensity}}: Plots the density of pairwise distances.
#' }
#'
#' @section Calculation of Statistical Measures to Compare Two Datasets:
#' Functions to compute statistical measures to compare two datasets.
#' \itemize{
#'   \item \code{\link{calculateAveragePairwiseCorrelation}}: Calculates average pairwise correlation, with a plot method for visualization.
#'   \item \code{\link{calculateCramerPValue}}: Calculates the p-value using Cramer's test.
#'   \item \code{\link{calculateHotellingPValue}}: Calculates the p-value using Hotelling's T-squared test.
#'   \item \code{\link{calculateMMDPValue}}: Calculates the p-value using maximum mean discrepancy.
#'   \item \code{\link{regressPC}}: Performs regression on principal components, with a plot method for visualization.
#' }
#'
#' @section Evaluation of Marker Gene Alignment:
#' Functions for calculating overlap measures of genes between two datasets.
#' \itemize{
#'   \item \code{\link{calculateHVGOverlap}}: Calculates overlap of highly variable genes (HVG) between datasets.
#'   \item \code{\link{calculateVarImpOverlap}}: Calculates overlap of variable importance measures between datasets.
#'   \item \code{\link{compareMarkers}}: Compares marker genes between datasets.
#' }
#'
#' @section Visualization of Marker Expressions:
#' Functions for to visualize and compare the expression of markers between a reference and a query dataset.
#' \itemize{
#'   \item \code{\link{plotGeneExpressionDimred}}: Plots gene expression in a dimensionality reduction space.
#'   \item \code{\link{plotMarkerExpression}}: Plots marker expression levels.
#' }
#'
#' @section Anomaly Detection (Global and Cell Type-Specific):
#' Functions for detecting anomalies at both the global and cell type-specific levels.
#' \itemize{
#'   \item \code{\link{detectAnomaly}}: Detects anomalies in the data, with a plot method for visualization.
#'   \item \code{\link{calculateCellSimilarityPCA}}: Calculates cell similarity in PCA space, with a plot method for visualization.
#' }
#'
#' @section Calculation of Distances Between Specific Cells and Cell Populations:
#' Functions for calculating distances between specific cells and cell populations.
#' \itemize{
#'   \item \code{\link{calculateCellDistances}}: Calculates distances between cells, with a plot method for visualization.
#'   \item \code{\link{calculateCellDistancesSimilarity}}: Calculates similarity based on cell distances, with a plot method for visualization.
#' }
#'
#' @section Visualization of QC and Annotation Scores:
#' Functions for visualizing quality control (QC) metrics or other characteristics of the data.
#' \itemize{
#'   \item \code{\link{histQCvsAnnotation}}: Plots histograms of QC metrics versus annotations.
#'   \item \code{\link{plotGeneSetScores}}: Plots scores of gene sets.
#'   \item \code{\link{plotQCvsAnnotation}}: Plots QC metrics versus annotations.
#' }
#'
#' @section Misc:
#' Miscellaneous functions for various tasks.
#' \itemize{
#'   \item \code{\link{processPCA}}: Process \code{\linkS4class{SingleCellExperiment}} objects to compute PCA.
#'   \item \code{\link{projectPCA}}: Projects new data into PCA space.
#'   \item \code{\link{projectSIR}}: Projects new data into SIR space.
#'   \item \code{\link{calculateCategorizationEntropy}}: Calculates categorization entropy for clusters.
#' }
#'
#' The package is built to facilitate in-depth analysis and visualization of single-cell data, enhancing the understanding of cell type similarities and differences across datasets.
#'
#' @name scDiagnostics-package
#' @aliases scDiagnostics
#' @keywords package internal
"_PACKAGE"

#' @title Principal Component Regression
#'
#' @description
#' This function performs linear regression of a covariate of interest onto one
#' or more principal components, based on the data in a \code{\linkS4class{SingleCellExperiment}}
#' object.
#'
#' @details
#' Principal component regression, derived from PCA, can be used to quantify the
#' variance explained by a covariate of interest. Applications for single-cell
#' analysis include quantification of batch effects, assessing clustering
#' homogeneity, and evaluating alignment of query and reference datasets in cell
#' type annotation settings.
#'
#' The function supports multiple regression scenarios:
#' \itemize{
#'   \item Query only, no batch: PC  cell_type
#'   \item Query only, with batch: PC  cell_type * batch
#'   \item Query + Reference, no batch: PC  cell_type * dataset
#'   \item Query + Reference, with batch: PC  cell_type * batch (where batch includes Reference)
#' }
#'
#' When batch information is provided with reference data, batches are labeled as
#' "Reference" for reference data and "Query_BatchName" for query batches, with
#' Reference set as the first factor level for interpretation.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' If NULL, the PC scores are regressed against the cell types of the query data.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param query_batch_col The column name in the \code{colData} of \code{query_data} that identifies the batch or sample.
#' If provided, performs interaction analysis with cell types. Default is NULL.
#' @param cell_types A character vector specifying the cell types to include in the analysis. If NULL, all cell types are included.
#' @param pc_subset A numeric vector specifying which principal components to include in the analysis. Default is PC1 to PC10.
#' @param adjust_method A character string specifying the method to adjust the p-values.
#'   Options include "BH", "holm", "hochberg", "hommel", "bonferroni", "BY", "fdr", or "none".
#'   Default is "BH" (Benjamini-Hochberg).
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#'
#' @return
#' A \code{list} containing \itemize{ \item summaries of the linear
#' regression models for each specified principal component, \item the
#' corresponding R-squared (R2) values, \item the variance contributions for
#' each principal component, and \item the total variance explained.}
#'
#' @references Luecken et al. Benchmarking atlas-level data integration in
#' single-cell genomics. Nature Methods, 19:41-50, 2022.
#'
#' @export
#'
#' @author
#' Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{plot.regressPCObject}}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Query only analysis
#' regress_res <- regressPC(query_data = query_data,
#'                          query_cell_type_col = "expert_annotation",
#'                          cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
#'                          pc_subset = 1:10)
#' # Visualize results
#' plot(regress_res, plot_type = "r_squared")
#' plot(regress_res, plot_type = "variance_contribution")
#' plot(regress_res, plot_type = "coefficient_heatmap")
#'
#' # Query + Reference analysis
#' regress_res <- regressPC(query_data = query_data,
#'                          reference_data = reference_data,
#'                          query_cell_type_col = "SingleR_annotation",
#'                          ref_cell_type_col = "expert_annotation",
#'                          cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
#'                          pc_subset = 1:10)
#' # Visualize results
#' plot(regress_res, plot_type = "r_squared")
#' plot(regress_res, plot_type = "variance_contribution")
#' plot(regress_res, plot_type = "coefficient_heatmap")
#'
#' @importFrom rlang .data
#' @import SingleCellExperiment
#'
# Function to regress PCs against cell types and batches
regressPC <- function(query_data,
                      reference_data = NULL,
                      query_cell_type_col,
                      ref_cell_type_col = NULL,
                      query_batch_col = NULL,
                      cell_types = NULL,
                      pc_subset = 1:10,
                      adjust_method = c("BH", "holm",
                                        "hochberg", "hommel",
                                        "bonferroni", "BY",
                                        "fdr", "none"),
                      assay_name = "logcounts",
                      max_cells_ref = 5000,
                      max_cells_query = 5000) {

    # Match argument for adjustment method
    adjust_method <- match.arg(adjust_method)

    # Check standard input arguments
    argumentCheck(reference_data = reference_data,
                  query_data = query_data,
                  ref_cell_type_col = ref_cell_type_col,
                  query_cell_type_col = query_cell_type_col,
                  pc_subset_query = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_cols_to_convert <- c(query_cell_type_col)
    if (!is.null(query_batch_col)) {
        query_cols_to_convert <- c(query_cols_to_convert, query_batch_col)
    }
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cols_to_convert)

    if(!is.null(reference_data)){
        reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                    convert_cols = ref_cell_type_col)
    }

    # Additional check for batch column
    if(!is.null(query_batch_col)){
        if(!query_batch_col %in% colnames(colData(query_data))){
            stop("query_batch_col '", query_batch_col,
                 "' not found in query_data colData.")
        }
    }

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = NULL)

    # Sort cell types for consistent reference category
    cell_types <- sort(cell_types)

    # Store reference cell type (first alphabetically)
    reference_cell_type <- cell_types[1]

    # Set dependent variables
    dep_vars <- paste0("PC", pc_subset)

    # Case 1: Query data only
    if(is.null(reference_data)){

        # Get query PCA variance for plotting
        query_pca_var <- attr(reducedDim(query_data, "PCA"), "percentVar")

        # Downsample reference and query data
        query_data <- downsampleSCE(sce_object = query_data,
                                    max_cells = max_cells_query,
                                    cell_types = cell_types,
                                    cell_type_col = query_cell_type_col)

        # Case 1a: No batch - PC ~ cell_type
        if(is.null(query_batch_col)){

            regress_data <- data.frame(
                reducedDim(query_data, "PCA")[, dep_vars],
                cell_type = factor(query_data[[query_cell_type_col]],
                                   levels = cell_types))

            # Regress PCs against cell types
            summaries <- lapply(dep_vars, regressFastCustom,
                                indep_var = "cell_type",
                                df = regress_data)
            names(summaries) <- dep_vars

            # Decompose R² by components
            r2_decomposed <- lapply(dep_vars, decomposeR2,
                                    indep_var = "cell_type",
                                    df = regress_data)
            names(r2_decomposed) <- dep_vars

            # Calculate variance contributions
            r_squared <- vapply(summaries, `[[`, numeric(1), x = "r_squared")
            var_expl <- query_pca_var[pc_subset]
            var_contr <- var_expl * r_squared
            total_var_expl <- sum(var_contr)

            # Calculate component-specific variance contributions
            r2_cell_type <- vapply(r2_decomposed, function(x) x$cell_type, numeric(1))
            var_contr_cell_type <- var_expl * r2_cell_type

            regress_res <- list(regression_summaries = summaries,
                                r_squared = r_squared,
                                r_squared_components = list(cell_type = r2_cell_type),
                                var_contributions = var_contr,
                                var_contributions_components = list(cell_type = var_contr_cell_type),
                                total_variance_explained = total_var_expl,
                                query_pca_var = query_pca_var,
                                indep_var = "cell_type",
                                reference_cell_type = reference_cell_type)

            regress_res <- adjustPValues(regress_res,
                                         adjust_method = adjust_method,
                                         indep_var = "cell_type")

            # Case 1b: With batch - PC ~ cell_type * batch
        } else {


            regress_data <- data.frame(
                reducedDim(query_data, "PCA")[, dep_vars],
                cell_type = factor(query_data[[query_cell_type_col]],
                                   levels = cell_types),
                batch = factor(query_data[[query_batch_col]]))

            # Get reference batch (first alphabetically)
            batch_levels <- levels(regress_data[["batch"]])
            reference_batch <- batch_levels[1]

            # Regress PCs against cell_type * batch interaction
            summaries <- lapply(dep_vars, regressFastCustom,
                                indep_var = "cell_type * batch",
                                df = regress_data)
            names(summaries) <- dep_vars

            # Decompose R² by components
            r2_decomposed <- lapply(dep_vars, decomposeR2,
                                    indep_var = "cell_type * batch",
                                    df = regress_data)
            names(r2_decomposed) <- dep_vars

            # Calculate variance contributions
            r_squared <- vapply(summaries, `[[`, numeric(1), x = "r_squared")
            var_expl <- query_pca_var[pc_subset]
            var_contr <- var_expl * r_squared
            total_var_expl <- sum(var_contr)

            # Calculate component-specific variance contributions
            r2_cell_type <- vapply(r2_decomposed, function(x) x$cell_type, numeric(1))
            r2_batch <- vapply(r2_decomposed, function(x) x$batch, numeric(1))
            r2_interaction <- vapply(r2_decomposed, function(x) x$interaction, numeric(1))

            var_contr_cell_type <- var_expl * r2_cell_type
            var_contr_batch <- var_expl * r2_batch
            var_contr_interaction <- var_expl * r2_interaction

            regress_res <- list(regression_summaries = summaries,
                                r_squared = r_squared,
                                r_squared_components = list(cell_type = r2_cell_type,
                                                            batch = r2_batch,
                                                            interaction = r2_interaction),
                                var_contributions = var_contr,
                                var_contributions_components = list(cell_type = var_contr_cell_type,
                                                                    batch = var_contr_batch,
                                                                    interaction = var_contr_interaction),
                                total_variance_explained = total_var_expl,
                                query_pca_var = query_pca_var,
                                indep_var = "cell_type_batch_interaction",
                                reference_cell_type = reference_cell_type,
                                reference_batch = reference_batch)

            regress_res <- adjustPValues(regress_res,
                                         adjust_method = adjust_method,
                                         indep_var = "cell_type_batch_interaction")
        }

        # Case 2: Query + Reference
    } else {

        # Get reference PCA variance for plotting
        reference_pca_var <- attr(reducedDim(reference_data, "PCA"), "percentVar")

        # Get the projected PCA data
        pca_output <- projectPCA(reference_data = reference_data,
                                 query_data = query_data,
                                 ref_cell_type_col = ref_cell_type_col,
                                 query_cell_type_col = query_cell_type_col,
                                 cell_types = cell_types,
                                 pc_subset = pc_subset,
                                 assay_name = assay_name,
                                 max_cells_ref = max_cells_ref,
                                 max_cells_query = max_cells_query)

        # Case 2a: No batch - PC ~ cell_type * dataset
        if(is.null(query_batch_col)){

            pca_output[["dataset"]] <- factor(pca_output[["dataset"]],
                                              levels = c("Reference", "Query"))
            pca_output[["cell_type"]] <- factor(pca_output[["cell_type"]],
                                                levels = cell_types)

            # Unified interaction model: PC ~ cell_type * dataset
            summaries <- lapply(dep_vars, regressFastCustom,
                                indep_var = "cell_type * dataset",
                                df = pca_output)
            names(summaries) <- dep_vars

            # Decompose R² by components
            r2_decomposed <- lapply(dep_vars, decomposeR2,
                                    indep_var = "cell_type * dataset",
                                    df = pca_output)
            names(r2_decomposed) <- dep_vars

            # Calculate R-squared and variance contributions
            r_squared <- vapply(summaries, `[[`, numeric(1), x = "r_squared")
            var_expl <- reference_pca_var[pc_subset]
            var_contr <- var_expl * r_squared
            total_var_expl <- sum(var_contr)

            # Calculate component-specific variance contributions
            r2_cell_type <- vapply(r2_decomposed, function(x) x$cell_type, numeric(1))
            r2_dataset <- vapply(r2_decomposed, function(x) x$dataset, numeric(1))
            r2_interaction <- vapply(r2_decomposed, function(x) x$interaction, numeric(1))

            var_contr_cell_type <- var_expl * r2_cell_type
            var_contr_dataset <- var_expl * r2_dataset
            var_contr_interaction <- var_expl * r2_interaction

            regress_res <- list(regression_summaries = summaries,
                                r_squared = r_squared,
                                r_squared_components = list(cell_type = r2_cell_type,
                                                            dataset = r2_dataset,
                                                            interaction = r2_interaction),
                                var_contributions = var_contr,
                                var_contributions_components = list(cell_type = var_contr_cell_type,
                                                                    dataset = var_contr_dataset,
                                                                    interaction = var_contr_interaction),
                                total_variance_explained = total_var_expl,
                                reference_pca_var = reference_pca_var,
                                indep_var = "cell_type_dataset_interaction",
                                reference_cell_type = reference_cell_type)

            regress_res <- adjustPValues(regress_res,
                                         adjust_method = adjust_method,
                                         indep_var = "cell_type_dataset_interaction")

            # Case 2b: With batch - PC ~ cell_type * batch (where batch includes Reference)
        } else {

            # Create batch labels: "Reference" for reference, "Query_BatchName" for query
            batch_labels <- rep(NA, nrow(pca_output))
            ref_indices <- pca_output[["dataset"]] == "Reference"
            query_indices <- pca_output[["dataset"]] == "Query"

            # Reference gets "Reference" label
            batch_labels[ref_indices] <- "Reference"

            # Query gets "Query_BatchName" labels
            query_cell_names <- rownames(pca_output)[query_indices]
            query_batch_data <- query_data[[query_batch_col]]
            names(query_batch_data) <- colnames(query_data)
            query_batch_values <- query_batch_data[query_cell_names]
            batch_labels[query_indices] <- paste0("Query_", query_batch_values)

            # Add batch information to pca_output
            pca_output[["batch"]] <- factor(
                batch_labels,
                levels = c("Reference",
                           sort(unique(batch_labels[query_indices]))))
            pca_output[["cell_type"]] <- factor(pca_output[["cell_type"]],
                                                levels = cell_types)

            # Remove any rows with missing batch info
            pca_output <- pca_output[!is.na(pca_output[["batch"]]),]

            # Check if this reduces to Case 2a (only one query batch)
            unique_batches <- unique(pca_output[["batch"]])
            if(length(unique_batches) == 2 && "Reference" %in% unique_batches){
                # Fallback to dataset interaction analysis
                pca_output[["dataset"]] <- ifelse(pca_output[["batch"]] == "Reference",
                                                  "Reference", "Query")
                pca_output[["dataset"]] <- factor(pca_output[["dataset"]],
                                                  levels = c("Reference", "Query"))

                # Unified interaction model: PC ~ cell_type * dataset
                summaries <- lapply(dep_vars, regressFastCustom,
                                    indep_var = "cell_type * dataset",
                                    df = pca_output)
                names(summaries) <- dep_vars

                # Decompose R² by components
                r2_decomposed <- lapply(dep_vars, decomposeR2,
                                        indep_var = "cell_type * dataset",
                                        df = pca_output)
                names(r2_decomposed) <- dep_vars

                # Calculate R-squared and variance contributions
                r_squared <- vapply(summaries, `[[`, numeric(1), x = "r_squared")
                var_expl <- reference_pca_var[pc_subset]
                var_contr <- var_expl * r_squared
                total_var_expl <- sum(var_contr)

                # Calculate component-specific variance contributions
                r2_cell_type <- vapply(r2_decomposed, function(x) x$cell_type, numeric(1))
                r2_dataset <- vapply(r2_decomposed, function(x) x$dataset, numeric(1))
                r2_interaction <- vapply(r2_decomposed, function(x) x$interaction, numeric(1))

                var_contr_cell_type <- var_expl * r2_cell_type
                var_contr_dataset <- var_expl * r2_dataset
                var_contr_interaction <- var_expl * r2_interaction

                regress_res <- list(regression_summaries = summaries,
                                    r_squared = r_squared,
                                    r_squared_components = list(cell_type = r2_cell_type,
                                                                dataset = r2_dataset,
                                                                interaction = r2_interaction),
                                    var_contributions = var_contr,
                                    var_contributions_components = list(cell_type = var_contr_cell_type,
                                                                        dataset = var_contr_dataset,
                                                                        interaction = var_contr_interaction),
                                    total_variance_explained = total_var_expl,
                                    reference_pca_var = reference_pca_var,
                                    indep_var = "cell_type_dataset_interaction",
                                    reference_cell_type = reference_cell_type)

                regress_res <- adjustPValues(regress_res,
                                             adjust_method = adjust_method,
                                             indep_var = "cell_type_dataset_interaction")
            } else {
                # True multi-batch case: PC ~ cell_type * batch
                summaries <- lapply(dep_vars, regressFastCustom,
                                    indep_var = "cell_type * batch",
                                    df = pca_output)
                names(summaries) <- dep_vars

                # Decompose R² by components
                r2_decomposed <- lapply(dep_vars, decomposeR2,
                                        indep_var = "cell_type * batch",
                                        df = pca_output)
                names(r2_decomposed) <- dep_vars

                # Calculate R-squared and variance contributions
                r_squared <- vapply(summaries, `[[`, numeric(1), x = "r_squared")
                var_expl <- reference_pca_var[pc_subset]
                var_contr <- var_expl * r_squared
                total_var_expl <- sum(var_contr)

                # Calculate component-specific variance contributions
                r2_cell_type <- vapply(r2_decomposed, function(x) x$cell_type, numeric(1))
                r2_batch <- vapply(r2_decomposed, function(x) x$batch, numeric(1))
                r2_interaction <- vapply(r2_decomposed, function(x) x$interaction, numeric(1))

                var_contr_cell_type <- var_expl * r2_cell_type
                var_contr_batch <- var_expl * r2_batch
                var_contr_interaction <- var_expl * r2_interaction

                regress_res <- list(regression_summaries = summaries,
                                    r_squared = r_squared,
                                    r_squared_components = list(cell_type = r2_cell_type,
                                                                batch = r2_batch,
                                                                interaction = r2_interaction),
                                    var_contributions = var_contr,
                                    var_contributions_components = list(cell_type = var_contr_cell_type,
                                                                        batch = var_contr_batch,
                                                                        interaction = var_contr_interaction),
                                    total_variance_explained = total_var_expl,
                                    reference_pca_var = reference_pca_var,
                                    indep_var = "cell_type_batch_interaction",
                                    reference_cell_type = reference_cell_type)

                regress_res <- adjustPValues(regress_res,
                                             adjust_method = adjust_method,
                                             indep_var = "cell_type_batch_interaction")
            }
        }
    }

    # Return regression output
    class(regress_res) <- c(class(regress_res), "regressPCObject")
    return(regress_res)
}

#' @title Fast Custom Linear Regression for Principal Components
#'
#' @description
#' Performs efficient linear regression of principal component scores against categorical
#' predictors (cell types, batches, datasets, or their interactions) using QR decomposition
#' for numerical stability and computational efficiency.
#'
#' @details
#' This function implements a custom linear regression optimized for categorical predictors
#' commonly used in single-cell RNA sequencing analysis. It uses QR decomposition instead
#' of normal equations for improved numerical stability and handles rank-deficient design
#' matrices gracefully. The function supports various model specifications including:
#' \itemize{
#'   \item Simple cell type effects: \code{PC ~ cell_type}
#'   \item Cell type and batch interactions: \code{PC ~ cell_type * batch}
#'   \item Cell type and dataset interactions: \code{PC ~ cell_type * dataset}
#' }
#'
#' The output format is compatible with \code{speedglm} results to maintain consistency
#' with existing plotting and analysis workflows.
#'
#' @param pc A character string specifying the principal component column name in the data frame.
#' @param indep_var A character string specifying the independent variable specification.
#'   Options include "cell_type", "cell_type * batch", "cell_type * dataset", or other
#'   interaction specifications.
#' @param df A data frame containing the principal component scores and categorical predictors.
#'   Must include columns for the specified PC and predictor variables.
#'
#' @keywords internal
#'
#' @return A list containing:
#'   \item{coefficients}{A data frame with columns \code{coef}, \code{se}, \code{t}, and \code{p.value}
#'     containing regression coefficients, standard errors, t-statistics, and p-values.}
#'   \item{r_squared}{A numeric value representing the R-squared of the model.}
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @importFrom stats model.matrix pt
#'
# Perform linear regression for each principal component
regressFastCustom <- function(pc, indep_var, df) {

    # Extract the response variable
    y <- df[[pc]]
    n <- length(y)

    # Create design matrix based on independent variable specification
    if (indep_var == "cell_type") {
        X <- model.matrix(~ cell_type, data = df)
    } else if (indep_var == "cell_type * batch") {
        X <- model.matrix(~ cell_type * batch, data = df)
    } else if (indep_var == "cell_type * dataset") {
        X <- model.matrix(~ cell_type * dataset, data = df)
    } else {
        # Handle other interaction cases generically
        formula_str <- paste("~", gsub("_", " * ", gsub("_interaction$", "", indep_var)))
        X <- model.matrix(as.formula(formula_str), data = df)
    }

    # Use QR decomposition for numerical stability
    qr_decomp <- qr(X)

    # Handle rank deficiency (shouldn't happen with your data, but for safety)
    if (qr_decomp$rank < ncol(X)) {
        X <- X[, qr_decomp$pivot[1:qr_decomp$rank], drop = FALSE]
        qr_decomp <- qr(X)
    }

    # Solve for coefficients using QR decomposition
    coefficients <- qr.coef(qr_decomp, y)

    # Handle any NAs (shouldn't happen but for completeness)
    if (any(is.na(coefficients))) {
        # Fall back to normal equations
        XtX <- crossprod(X)
        Xty <- crossprod(X, y)
        coefficients <- solve(XtX, Xty)
    }

    # Compute fitted values and residuals
    fitted_values <- X %*% coefficients
    residuals <- y - fitted_values

    # Sum of squares calculations
    y_mean <- mean(y)
    ss_total <- sum((y - y_mean)^2)
    ss_residual <- sum(residuals^2)
    ss_regression <- ss_total - ss_residual

    # R-squared
    r_squared <- ss_regression / ss_total

    # Degrees of freedom
    p <- ncol(X)  # number of parameters including intercept
    df_residual <- n - p

    # Residual standard error
    mse <- ss_residual / df_residual

    # Variance-covariance matrix of coefficients
    # Var(beta) = sigma^2 * (X'X)^(-1)
    XtX_inv <- chol2inv(qr.R(qr_decomp))
    var_coef <- mse * XtX_inv
    se_coef <- sqrt(diag(var_coef))

    # t-statistics and p-values
    t_values <- coefficients / se_coef
    p_values <- 2 * pt(abs(t_values), df_residual, lower.tail = FALSE)

    # Create coefficients data.frame in exact format as speedlm
    coef_df <- data.frame(
        coef = as.numeric(coefficients),
        se = se_coef,
        t = t_values,
        p.value = p_values,
        stringsAsFactors = FALSE
    )

    # Set row names to match coefficient names
    rownames(coef_df) <- names(coefficients)

    # Return in same format as speedlm summary
    model_summary <- list(
        coefficients = coef_df,
        r_squared = r_squared
    )

    return(model_summary)
}

#' @title Decompose R-squared by Model Components
#'
#' @description
#' Decomposes the total R-squared from a linear model into individual components
#' representing the variance explained by main effects (cell type, batch/dataset)
#' and their interaction using sequential sum of squares.
#'
#' @details
#' This function performs R-squared decomposition using a sequential sum of squares approach,
#' which partitions the total explained variance into additive components. The decomposition
#' follows the hierarchical structure:
#' \enumerate{
#'   \item Cell type main effect
#'   \item Batch/dataset main effect (after accounting for cell type)
#'   \item Cell type × batch/dataset interaction (after accounting for main effects)
#' }
#'
#' For simple cell type models, only the cell type component is returned. For interaction
#' models, all three components are computed. The method uses group means and residual
#' analysis to avoid computationally expensive matrix operations while maintaining
#' mathematical accuracy equivalent to ANOVA decomposition.
#'
#' @param pc A character string specifying the principal component column name in the data frame.
#' @param indep_var A character string specifying the independent variable specification.
#'   Options are "cell_type", "cell_type * batch", or "cell_type * dataset".
#' @param df A data frame containing the principal component scores and categorical predictors.
#'   Must include columns for the specified PC and predictor variables.
#'
#' @keywords internal
#'
#' @return A named list containing R-squared components:
#'   \item{cell_type}{Numeric value representing the R-squared explained by cell type main effect.}
#'   \item{batch/dataset}{Numeric value representing the R-squared explained by batch or dataset
#'     main effect (only for interaction models).}
#'   \item{interaction}{Numeric value representing the R-squared explained by the interaction term
#'     (only for interaction models).}
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
# Function to decompose R-squared using sequential group means
decomposeR2 <- function(pc, indep_var, df) {
    y <- df[[pc]]
    y_mean <- mean(y)
    ss_total <- sum((y - y_mean)^2)

    if (indep_var == "cell_type") {
        # Simple case
        group_means <- tapply(y, df$cell_type, mean)
        y_pred <- group_means[df$cell_type]
        ss_explained <- sum((y_pred - y_mean)^2)

        return(list(cell_type = ss_explained / ss_total))

    } else if (indep_var %in% c("cell_type * batch", "cell_type * dataset")) {
        second_var <- if (indep_var == "cell_type * batch") "batch" else "dataset"

        # Sequential sum of squares approach
        # 1. Cell type only
        ct_means <- tapply(y, df$cell_type, mean)
        y_pred_ct <- ct_means[df$cell_type]
        ss_cell_type <- sum((y_pred_ct - y_mean)^2)

        # 2. Add second variable
        # Fit additive model by computing residuals and group means
        residuals_1 <- y - y_pred_ct
        sv_residual_means <- tapply(residuals_1, df[[second_var]], mean)
        y_pred_sv_adj <- sv_residual_means[df[[second_var]]]
        ss_second_var <- sum(y_pred_sv_adj^2)

        # 3. Add interaction (full model with all combinations)
        interaction_means <- tapply(y, list(df$cell_type, df[[second_var]]), mean)

        # Predict using full interaction model
        y_pred_full <- numeric(length(y))
        for (i in seq_along(y)) {
            ct_level <- as.character(df$cell_type[i])
            sv_level <- as.character(df[[second_var]][i])
            y_pred_full[i] <- interaction_means[ct_level, sv_level]
        }

        ss_full <- sum((y_pred_full - y_mean)^2)
        ss_interaction <- ss_full - ss_cell_type - ss_second_var

        result <- list(
            cell_type = ss_cell_type / ss_total,
            interaction = max(0, ss_interaction / ss_total)  # Ensure non-negative due to numerical precision
        )
        result[[second_var]] <- ss_second_var / ss_total

        return(result)
    }

    # Fallback
    return(list(total = 0))
}

#' @title Adjust P-Values in Regression Results
#'
#' @description
#' Adjusts the p-values in the regression results using a specified adjustment method.
#' The adjustment is performed for different regression types including cell type,
#' dataset, and cell type-batch interaction analyses.
#'
#' @details
#' This function adjusts p-values from regression results stored in a list. The adjustment
#' can be applied across different regression structures depending on the analysis type.
#' The method for adjusting p-values can be selected from various options such as
#' Benjamini-Hochberg (BH), Holm, and others, which are supported by the `p.adjust` function in R.
#'
#' @param regress_res A list containing regression results. The structure varies by analysis type.
#' @param adjust_method A character string specifying the method to adjust the p-values.
#'   Options include "BH", "holm", "hochberg", "hommel", "bonferroni", "BY", "fdr", or "none".
#'   Default is "BH" (Benjamini-Hochberg).
#' @param indep_var A character string specifying the independent variable for the adjustment.
#'   Options are "cell_type", "cell_type_dataset_interaction", or "cell_type_batch_interaction".
#'
#' @keywords internal
#'
#' @return A list similar to \code{regress_res}, but with an added column for adjusted p-values
#'   in the coefficients tables.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @importFrom stats p.adjust
#'
# Function to compute adjusted p-values for PC regression
adjustPValues <- function(regress_res,
                          adjust_method = c("BH", "holm",
                                            "hochberg", "hommel",
                                            "bonferroni", "BY",
                                            "fdr", "none"),
                          indep_var = c("cell_type", "cell_type_dataset_interaction",
                                        "cell_type_batch_interaction")){

    # Match arguments
    adjust_method <- match.arg(adjust_method)
    indep_var <- match.arg(indep_var)

    # All interaction models have the same structure
    if(indep_var %in% c("cell_type", "cell_type_dataset_interaction", "cell_type_batch_interaction")){
        # Add adjusted p-values for unified model structure
        for (pc in names(regress_res[["regression_summaries"]])) {
            coeffs <- regress_res[["regression_summaries"]][[pc]][["coefficients"]]
            p.adjusted <- p.adjust(coeffs[["p.value"]], method = adjust_method)
            coeffs[["p.adjusted"]] <- p.adjusted
            regress_res[["regression_summaries"]][[pc]][["coefficients"]] <- coeffs
        }
    }

    # Return regression object
    return(regress_res)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

test_that("regressPC works correctly with only query data", {
    # Perform regression with only query data
    regress_res <- regressPC(
        query_data = query_data,
        query_cell_type_col = "expert_annotation",
        cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
        pc_subset = 1:5
    )

    # Check if the output is a list
    expect_true(is.list(regress_res))

    # Check if the list contains the correct elements
    expect_true("regression_summaries" %in% names(regress_res))
    expect_true("r_squared" %in% names(regress_res))
    expect_true("var_contributions" %in% names(regress_res))
    expect_true("total_variance_explained" %in% names(regress_res))
    expect_true("query_pca_var" %in% names(regress_res))
    expect_equal(regress_res$indep_var, "cell_type")

    # Check if R-squared values are numeric and of correct length
    expect_true(is.numeric(regress_res$r_squared))
    expect_equal(length(regress_res$r_squared), 5)

    # Check if total variance explained is a numeric value
    expect_true(is.numeric(regress_res$total_variance_explained))

    # Check if object has correct class
    expect_true("regressPCObject" %in% class(regress_res))
})

test_that("regressPC works correctly with reference and query data", {
    # Perform regression with reference and query data
    regress_res <- regressPC(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
        pc_subset = 1:5
    )

    # Check if the output is a list
    expect_true(is.list(regress_res))

    # Check if the list contains the correct elements
    expect_true("regression_summaries" %in% names(regress_res))
    expect_true("r_squared" %in% names(regress_res))
    expect_true("reference_pca_var" %in% names(regress_res))
    expect_equal(regress_res$indep_var, "cell_type_dataset_interaction")

    # Check if R-squared values are numeric and of correct length
    expect_true(is.numeric(regress_res$r_squared))
    expect_equal(length(regress_res$r_squared), 5)

    # Check if object has correct class
    expect_true("regressPCObject" %in% class(regress_res))
})

test_that("regressPC works with batch information", {
    # Add batch information to query data for testing
    query_data$batch <- sample(c("batch1", "batch2"), ncol(query_data), replace = TRUE)

    # Perform regression with batch information
    regress_res <- regressPC(
        query_data = query_data,
        query_cell_type_col = "expert_annotation",
        query_batch_col = "batch",
        cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
        pc_subset = 1:5
    )

    # Check if the output is a list
    expect_true(is.list(regress_res))
    expect_equal(regress_res$indep_var, "cell_type_batch_interaction")

    # Check if object has correct class
    expect_true("regressPCObject" %in% class(regress_res))
})

test_that("regressPC handles incorrect parameters", {
    # Test for invalid query data column
    expect_error(regressPC(
        query_data = query_data,
        query_cell_type_col = "invalid_column",
        cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
        pc_subset = 1:5
    ))

    # Test for invalid reference data column
    expect_error(regressPC(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "invalid_column",
        cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
        pc_subset = 1:5
    ))

    # Test for invalid batch column
    expect_error(regressPC(
        query_data = query_data,
        query_cell_type_col = "expert_annotation",
        query_batch_col = "invalid_batch_column",
        pc_subset = 1:5
    ))
})

test_that("regressPC works with all cell types and default PC subset", {
    # Perform regression with all cell types and default PC subset
    regress_res <- regressPC(
        query_data = query_data,
        query_cell_type_col = "expert_annotation"
    )

    # Check if the output is a list
    expect_true(is.list(regress_res))

    # Check if the list contains the correct elements
    expect_true("regression_summaries" %in% names(regress_res))
    expect_true("r_squared" %in% names(regress_res))
    expect_true("var_contributions" %in% names(regress_res))
    expect_true("total_variance_explained" %in% names(regress_res))

    # Check if R-squared values are numeric and of correct length
    expect_true(is.numeric(regress_res$r_squared))
    expect_equal(length(regress_res$r_squared), 10)

    # Check if total variance explained is a numeric value
    expect_true(is.numeric(regress_res$total_variance_explained))

    # Check if object has correct class
    expect_true("regressPCObject" %in% class(regress_res))
})

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Create test regression results for different scenarios
regress_res_query_only <- regressPC(query_data = query_data,
                                    query_cell_type_col = "expert_annotation",
                                    cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                                    pc_subset = 1:5)

regress_res_combined <- regressPC(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = "expert_annotation",
                                  ref_cell_type_col = "expert_annotation",
                                  cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                                  pc_subset = 1:5)

# Add batch information for batch testing
query_data_batch <- query_data
query_data_batch$batch <- sample(c("batch1", "batch2"), ncol(query_data_batch), replace = TRUE)

regress_res_batch <- regressPC(query_data = query_data_batch,
                               query_cell_type_col = "expert_annotation",
                               query_batch_col = "batch",
                               cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                               pc_subset = 1:5)

# Add multiple batch scenario for query+reference
reference_data_batch <- reference_data
query_data_multi_batch <- query_data
query_data_multi_batch$batch <- sample(c("batch1", "batch2", "batch3"), ncol(query_data_multi_batch), replace = TRUE)

regress_res_multi_batch <- regressPC(query_data = query_data_multi_batch,
                                     reference_data = reference_data_batch,
                                     query_cell_type_col = "expert_annotation",
                                     ref_cell_type_col = "expert_annotation",
                                     query_batch_col = "batch",
                                     cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                                     pc_subset = 1:5)

test_that("plot.regressPCObject generates r_squared plots correctly", {
    # Test query-only analysis
    p1 <- plot(regress_res_query_only, plot_type = "r_squared")
    expect_s3_class(p1, "ggplot")
    expect_true("GeomCol" %in% class(p1$layers[[1]]$geom))

    # Test combined analysis (should have stacked bars with components)
    p2 <- plot(regress_res_combined, plot_type = "r_squared")
    expect_s3_class(p2, "ggplot")
    expect_true("GeomCol" %in% class(p2$layers[[1]]$geom))

    # Test batch analysis (should have stacked bars with components)
    p3 <- plot(regress_res_batch, plot_type = "r_squared")
    expect_s3_class(p3, "ggplot")
    expect_true("GeomCol" %in% class(p3$layers[[1]]$geom))

    # Test multi-batch analysis
    p4 <- plot(regress_res_multi_batch, plot_type = "r_squared")
    expect_s3_class(p4, "ggplot")
    expect_true("GeomCol" %in% class(p4$layers[[1]]$geom))
})

test_that("plot.regressPCObject generates variance_contribution plots correctly", {
    # Test query-only analysis
    p1 <- plot(regress_res_query_only, plot_type = "variance_contribution")
    expect_s3_class(p1, "ggplot")
    expect_true("GeomCol" %in% class(p1$layers[[1]]$geom))

    # Test combined analysis (should have stacked bars with components)
    p2 <- plot(regress_res_combined, plot_type = "variance_contribution")
    expect_s3_class(p2, "ggplot")
    expect_true("GeomCol" %in% class(p2$layers[[1]]$geom))

    # Test batch analysis (should have stacked bars with components)
    p3 <- plot(regress_res_batch, plot_type = "variance_contribution")
    expect_s3_class(p3, "ggplot")
    expect_true("GeomCol" %in% class(p3$layers[[1]]$geom))

    # Test multi-batch analysis
    p4 <- plot(regress_res_multi_batch, plot_type = "variance_contribution")
    expect_s3_class(p4, "ggplot")
    expect_true("GeomCol" %in% class(p4$layers[[1]]$geom))
})

test_that("plot.regressPCObject generates coefficient_heatmap plots correctly", {
    # Test query-only analysis
    p1 <- plot(regress_res_query_only, plot_type = "coefficient_heatmap")
    expect_s3_class(p1, "ggplot")
    expect_true("GeomTile" %in% class(p1$layers[[1]]$geom))

    # Test combined analysis
    p2 <- plot(regress_res_combined, plot_type = "coefficient_heatmap")
    expect_s3_class(p2, "ggplot")
    expect_true("GeomTile" %in% class(p2$layers[[1]]$geom))

    # Test batch analysis
    p3 <- plot(regress_res_batch, plot_type = "coefficient_heatmap")
    expect_s3_class(p3, "ggplot")
    expect_true("GeomTile" %in% class(p3$layers[[1]]$geom))

    # Test multi-batch analysis
    p4 <- plot(regress_res_multi_batch, plot_type = "coefficient_heatmap")
    expect_s3_class(p4, "ggplot")
    expect_true("GeomTile" %in% class(p4$layers[[1]]$geom))
})

test_that("plot.regressPCObject handles default plot type correctly", {
    # Test default plot type (should be r_squared)
    p1 <- plot(regress_res_query_only)
    expect_s3_class(p1, "ggplot")

    p2 <- plot(regress_res_combined)
    expect_s3_class(p2, "ggplot")

    p3 <- plot(regress_res_batch)
    expect_s3_class(p3, "ggplot")
})

test_that("plot.regressPCObject handles invalid plot types", {
    # Test invalid plot type
    expect_error(plot(regress_res_query_only, plot_type = "invalid_type"))
    expect_error(plot(regress_res_combined, plot_type = "invalid_type"))
    expect_error(plot(regress_res_batch, plot_type = "nonexistent_plot"))
})

test_that("plot.regressPCObject respects alpha parameter for heatmaps", {
    # Test with different alpha values
    p1 <- plot(regress_res_query_only, plot_type = "coefficient_heatmap", alpha = 0.01)
    expect_s3_class(p1, "ggplot")

    p2 <- plot(regress_res_combined, plot_type = "coefficient_heatmap", alpha = 0.1)
    expect_s3_class(p2, "ggplot")

    p3 <- plot(regress_res_batch, plot_type = "coefficient_heatmap", alpha = 0.001)
    expect_s3_class(p3, "ggplot")
})

test_that("plot.regressPCObject generates correct plot elements for stacked bars", {
    # Check that component breakdown plots have legends for stacked bars
    p1 <- plot(regress_res_combined, plot_type = "r_squared")
    expect_s3_class(p1, "ggplot")

    # Extract plot data to verify stacking
    built_plot <- ggplot2::ggplot_build(p1)
    expect_true(length(built_plot$data) >= 1)

    # Test variance contribution stacked bars
    p2 <- plot(regress_res_batch, plot_type = "variance_contribution")
    expect_s3_class(p2, "ggplot")

    built_plot2 <- ggplot2::ggplot_build(p2)
    expect_true(length(built_plot2$data) >= 1)
})

test_that("plot.regressPCObject handles edge cases gracefully", {
    # Test with minimal PC subset
    regress_res_minimal <- regressPC(query_data = query_data,
                                     query_cell_type_col = "expert_annotation",
                                     cell_types = c("CD4", "CD8"),
                                     pc_subset = 1:2)

    p1 <- plot(regress_res_minimal, plot_type = "r_squared")
    expect_s3_class(p1, "ggplot")

    p2 <- plot(regress_res_minimal, plot_type = "variance_contribution")
    expect_s3_class(p2, "ggplot")

    p3 <- plot(regress_res_minimal, plot_type = "coefficient_heatmap")
    expect_s3_class(p3, "ggplot")
})

test_that("plot.regressPCObject component decomposition works correctly", {
    # Test that objects with component decomposition create stacked bars
    if (!is.null(regress_res_combined[["r_squared_components"]]) &&
        length(regress_res_combined[["r_squared_components"]]) > 1) {

        p1 <- plot(regress_res_combined, plot_type = "r_squared")
        expect_s3_class(p1, "ggplot")

        # Check that the plot has fill aesthetic (indicating stacked bars)
        expect_true("fill" %in% names(p1$mapping))
    }

    # Test batch interaction component decomposition
    if (!is.null(regress_res_batch[["r_squared_components"]]) &&
        length(regress_res_batch[["r_squared_components"]]) > 1) {

        p2 <- plot(regress_res_batch, plot_type = "variance_contribution")
        expect_s3_class(p2, "ggplot")

        # Check that the plot has fill aesthetic (indicating stacked bars)
        expect_true("fill" %in% names(p2$mapping))
    }
})

test_that("plot.regressPCObject handles different model types in heatmaps", {
    # Test cell type only model
    p1 <- plot(regress_res_query_only, plot_type = "coefficient_heatmap")
    expect_s3_class(p1, "ggplot")

    # Test cell type * dataset interaction
    p2 <- plot(regress_res_combined, plot_type = "coefficient_heatmap")
    expect_s3_class(p2, "ggplot")

    # Test cell type * batch interaction
    p3 <- plot(regress_res_batch, plot_type = "coefficient_heatmap")
    expect_s3_class(p3, "ggplot")

    # Test multi-batch scenario
    p4 <- plot(regress_res_multi_batch, plot_type = "coefficient_heatmap")
    expect_s3_class(p4, "ggplot")
})

test_that("plot.regressPCObject produces plots with correct titles and labels", {
    # Test titles for different plot types
    p1 <- plot(regress_res_query_only, plot_type = "r_squared")
    expect_true(grepl("R-squared Values", p1$labels$title))

    p2 <- plot(regress_res_combined, plot_type = "variance_contribution")
    expect_true(grepl("Variance Contribution", p2$labels$title))

    p3 <- plot(regress_res_batch, plot_type = "coefficient_heatmap")
    expect_true(grepl("Regression Coefficients", p3$labels$title))

    # Test that y-axis labels are correct
    expect_true(grepl("R", as.character(p1$labels$y)) || is.expression(p1$labels$y))
    expect_true(grepl("Variance", p2$labels$y))
})

test_that("plot.regressPCObject handles missing or NULL components gracefully", {
    # Create a minimal regression object without component decomposition
    minimal_regress <- list(
        r_squared = c(PC1 = 0.5, PC2 = 0.3),
        var_contributions = c(PC1 = 2.5, PC2 = 1.5),
        total_variance_explained = 4.0,
        query_pca_var = c(5.0, 5.0, 4.0, 4.0, 3.0),
        indep_var = "cell_type",
        reference_cell_type = "CD4",
        regression_summaries = list(
            PC1 = list(coefficients = data.frame(coef = 0.1, se = 0.05, t = 2.0, p.value = 0.05, p.adjusted = 0.1)),
            PC2 = list(coefficients = data.frame(coef = 0.2, se = 0.06, t = 3.0, p.value = 0.01, p.adjusted = 0.02))
        )
    )
    class(minimal_regress) <- c("list", "regressPCObject")

    # Test that plots work even without component decomposition
    p1 <- plot(minimal_regress, plot_type = "r_squared")
    expect_s3_class(p1, "ggplot")

    p2 <- plot(minimal_regress, plot_type = "variance_contribution")
    expect_s3_class(p2, "ggplot")
})

test_that("plot.regressPCObject additional arguments are passed correctly", {
    # Test that additional arguments don't break the function
    p1 <- plot(regress_res_query_only, plot_type = "r_squared", extra_arg = "test")
    expect_s3_class(p1, "ggplot")

    p2 <- plot(regress_res_combined, plot_type = "coefficient_heatmap", alpha = 0.05, another_arg = TRUE)
    expect_s3_class(p2, "ggplot")
})

#' @title Convert Specified Columns to Character in SingleCellExperiment Objects
#'
#' @description
#' This function converts specified columns in the \code{colData} of a \code{\linkS4class{SingleCellExperiment}}
#' object to character type. It checks that the specified columns exist and only performs conversion
#' when necessary (i.e., when columns are not already character type).
#'
#' @details
#' The function performs the following operations:
#' \itemize{
#'  \item Validates that the input is a \code{\linkS4class{SingleCellExperiment}} object.
#'  \item Checks that all specified columns exist in the \code{colData} of the object.
#'  \item Converts each specified column to character type if it is not already character.
#'  \item Returns the modified \code{\linkS4class{SingleCellExperiment}} object.
#'  \item If all specified columns are already character type, returns the object unchanged.
#' }
#'
#' This function is particularly useful for handling factor columns that need to be converted
#' to character for downstream analysis functions that expect character input.
#'
#' @param sce_object A \code{\linkS4class{SingleCellExperiment}} object containing single-cell data.
#' @param convert_cols A character vector specifying the column names in \code{colData} to convert
#' to character type. All specified columns must exist in the \code{colData}.
#'
#' @keywords internal
#'
#' @return A \code{\linkS4class{SingleCellExperiment}} object with the specified columns converted
#' to character type in the \code{colData}.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
# Function to convert specified columns to character in SingleCellExperiment objects
convertColumnsToCharacter <- function(sce_object, convert_cols) {

    # Validate input object
    if (!is(sce_object, "SingleCellExperiment")) {
        stop("'sce_object' must be a SingleCellExperiment object.")
    }

    # Validate convert_cols parameter
    if (!is.character(convert_cols) || length(convert_cols) == 0) {
        stop("'convert_cols' must be a non-empty character vector.")
    }

    # Check if all specified columns exist in colData
    existing_cols <- names(SummarizedExperiment::colData(sce_object))
    missing_cols <- convert_cols[!convert_cols %in% existing_cols]

    if (length(missing_cols) > 0) {
        stop("The following columns are not found in colData: ",
             paste(missing_cols, collapse = ", "))
    }

    # Convert columns to character if they are not already character
    col_data <- SummarizedExperiment::colData(sce_object)
    conversion_needed <- FALSE

    for (col in convert_cols) {
        if (!is.character(col_data[[col]])) {
            col_data[[col]] <- as.character(col_data[[col]])
            conversion_needed <- TRUE
        }
    }

    # Update the object only if conversion was needed
    if (conversion_needed) {
        SummarizedExperiment::colData(sce_object) <- col_data
    }

    return(sce_object)
}

#' @title Calculate Cell Similarity Using PCA Loadings
#'
#' @description
#' This function calculates the cosine similarity between cells based on the principal components (PCs)
#' obtained from PCA (Principal Component Analysis) loadings.
#'
#' @details
#' This function calculates the cosine similarity between cells based on the loadings of the selected
#' principal components obtained from PCA. It extracts the rotation matrix from the PCA results of the
#' \code{\linkS4class{SingleCellExperiment}} object and identifies the high-loading variables for each selected PC.
#' Then, it computes the cosine similarity between cells using the high-loading variables for each PC.
#'
#' @param sce_object A \code{\linkS4class{SingleCellExperiment}} object containing expression data.
#' @param cell_names A character vector specifying the cell names for which to compute the similarity.
#' @param pc_subset A numeric vector specifying the subset of principal components to consider. Default is 1:5..
#' @param n_top_vars An integer indicating the number of top loading variables to consider for each PC. Default is 50.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#'
#' @return A data frame containing cosine similarity values between cells for each selected principal component.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{plot.calculateCellSimilarityPCAObject}}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Store PCA anomaly data and plots
#' anomaly_output <- detectAnomaly(reference_data = reference_data,
#'                                 query_data = query_data,
#'                                 ref_cell_type_col = "expert_annotation",
#'                                 query_cell_type_col = "SingleR_annotation",
#'                                 pc_subset = 1:10,
#'                                 n_tree = 500,
#'                                 anomaly_threshold = 0.5)
#' top6_anomalies <- names(sort(anomaly_output$Combined$reference_anomaly_scores,
#'                              decreasing = TRUE)[1:6])
#'
#' # Compute cosine similarity between anomalies and top PCs
#' cosine_similarities <- calculateCellSimilarityPCA(reference_data,
#'                                                   cell_names = top6_anomalies,
#'                                                   pc_subset = 1:25,
#'                                                   n_top_vars = 50)
#' cosine_similarities
#'
#' # Plot similarities
#' plot(cosine_similarities, pc_subset = 15:25)
#'
# Function to calculate cosine similarities between cells and PCs
calculateCellSimilarityPCA <- function(sce_object,
                                       cell_names,
                                       pc_subset = 1:5,
                                       n_top_vars = 50,
                                       assay_name = "logcounts"){

    # Format the cell names - remove "Query_" prefix if present
    cell_names <- gsub("^Query_", "", cell_names)
    cell_names <- gsub("^Reference_", "", cell_names)

    # Check standard input arguments
    argumentCheck(query_data = sce_object,
                  cell_names_query = cell_names,
                  pc_subset_query = pc_subset,
                  assay_name = assay_name)

    # Check if n_top_vars is a positive integer
    if (!is.numeric(n_top_vars) || n_top_vars <= 0 ||
        n_top_vars != as.integer(n_top_vars)) {
        stop("\'n_top_vars\' must be a positive integer.")
    }
    if(is.null(n_top_vars)){
        n_top_vars <- nrow(sce_object)
    }

    # Extract rotation matrix for SingleCellExperiment object
    rotation_mat <- attributes(
        reducedDim(sce_object, "PCA"))$rotation[, pc_subset]

    # Function to identify high-loading variables for each PC
    .getHighLoadingVars <- function(rotation_mat, n_top_vars) {
        high_loading_vars <- lapply(
            seq_len(ncol(rotation_mat)), function(pc) {
                abs_loadings <- abs(rotation_mat[, pc])
                top_vars <-
                    names(sort(abs_loadings,
                               decreasing = TRUE))[seq_len(n_top_vars)]
                return(top_vars)
            })
        return(high_loading_vars)
    }

    # Extract high-loading variables
    high_loading_vars <- .getHighLoadingVars(rotation_mat, n_top_vars)

    # Function to compute cosine similarity
    .cosine_similarity <- function(vector1, vector2) {
        sum(vector1 * vector2) / (sqrt(sum(vector1^2)) *
                                      sqrt(sum(vector2^2)))
    }

    # Function to compute cosine similarity for each PC using high-loading variables
    .computeCosineSimilarity <- function(cell_names,
                                         rotation_mat,
                                         high_loading_vars) {
        similarities <- lapply(
            seq_len(length(high_loading_vars)), function(pc) {
                vars <- high_loading_vars[[pc]]
                cell_subset <- cell_names[, vars, drop = FALSE]
                pc_vector <- rotation_mat[vars, pc]
                apply(cell_subset, 1, .cosine_similarity,
                      vector2 = pc_vector)
            })
        return(similarities)
    }

    # Calculate similarities
    assay_mat <- t(as.matrix(assay(sce_object[, cell_names, drop = FALSE],
                                   assay_name)))
    similarities <- .computeCosineSimilarity(assay_mat, rotation_mat,
                                             high_loading_vars)

    # Format the result into a data frame for easy interpretation
    similarity_df <- do.call(cbind, similarities)
    colnames(similarity_df) <- paste0("PC", seq_len(ncol(rotation_mat)))

    # Update class of output
    class(similarity_df) <- c(class(similarity_df),
                              "calculateCellSimilarityPCAObject")
    return(similarity_df)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load data
data("reference_data")
data("query_data")

# Store PCA anomaly data and plots
anomaly_output <- detectAnomaly(reference_data = reference_data,
                                query_data = query_data,
                                ref_cell_type_col = "expert_annotation",
                                query_cell_type_col = "SingleR_annotation",
                                pc_subset = 1:10,
                                n_tree = 500,
                                anomaly_threshold = 0.5)
top6_anomalies <- names(sort(anomaly_output$Combined$reference_anomaly_scores, decreasing = TRUE)[1:6])

# Compute cosine similarity between anomalies and top PCs
cosine_similarities <- calculateCellSimilarityPCA(sce_object = reference_data,
                                                  cell_names = top6_anomalies,
                                                  pc_subset = 1:25,
                                                  n_top_vars = 50)

test_that("plot.calculateCellSimilarityPCA generates plots correctly", {
    # Generate plot using the function
    p1 <- plot(cosine_similarities, pc_subset = 15:25)

    # Check if output is a ggplot object
    expect_s3_class(p1, "ggplot")
})

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Test cases
test_that("calculateCellSimilarityPCA computes cosine similarity correctly", {

    # Test with typical input
    cosine_similarities <- calculateCellSimilarityPCA(sce_object = reference_data,
                                                      cell_names = colnames(reference_data)[1:5],
                                                      pc_subset = 1:5,
                                                      n_top_vars = 50)

    # Check if the result is a data frame
    expect_type(cosine_similarities, "double")

    # Check dimensions of the result
    expect_equal(nrow(cosine_similarities), length(colnames(reference_data)[1:5]))
    expect_equal(ncol(cosine_similarities), 5)  # Check against pc_subset

    # Check if column names correspond to PCs
    expect_true(all(grepl("^PC", colnames(cosine_similarities))))

    # Check range of cosine similarity values
    expect_true(all(cosine_similarities >= -1 & cosine_similarities <= 1))

})

test_that("calculateCellSimilarityPCA handles NULL n_top_vars gracefully.", {

    expect_error(cosine_similarities <-
                     calculateCellSimilarityPCA(sce_object = reference_data,
                                                cell_names = rownames(reference_data)[1:5],
                                                pc_subset = 1:5,
                                                n_top_vars = NULL))

})

## ----setup, include = FALSE, fig.show='hide'----------------------------------
knitr::knit_hooks$set(pngquant = knitr::hook_pngquant)

knitr::opts_chunk$set(
collapse = TRUE,
comment = "#>",
dev = "ragg_png",
dpi = 72,
fig.retina = 2,
fig.align = "center",
out.width = "100%",
pngquant = "--speed=1 --quality=1-5"
)

## ----message=FALSE, fig.show='hide'-------------------------------------------
# Load library
library(scDiagnostics)

# Load datasets
data("reference_data")
data("query_data")

# Set seed for reproducibility
set.seed(0)

## ----message=FALSE, fig.show='hide'-------------------------------------------
# Load library
library(scran)
library(scater)

# Subset to CD4 cells
ref_data_cd4 <- reference_data[, which(
    reference_data$expert_annotation == "CD4")]
query_data_cd4 <- query_data_cd4 <- query_data[, which(
    query_data$expert_annotation == "CD4")]

# Select highly variable genes
ref_top_genes <- getTopHVGs(ref_data_cd4, n = 500)
query_top_genes <- getTopHVGs(query_data_cd4, n = 500)
common_genes <- intersect(ref_top_genes, query_top_genes)

# Subset data by common genes
ref_data_cd4 <- ref_data_cd4[common_genes,]
query_data_cd4 <- query_data_cd4[common_genes,]

# Run PCA on both datasets
ref_data_cd4 <- runPCA(ref_data_cd4)
query_data_cd4 <- runPCA(query_data_cd4)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Perform PCA 
pca_comparison <- comparePCA(query_data = query_data_cd4, 
                             reference_data = ref_data_cd4, 
                             query_cell_type_col = "expert_annotation", 
                             ref_cell_type_col = "expert_annotation", 
                             pc_subset = 1:5, 
                             metric = "cosine")
plot(pca_comparison)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Compare PCA subspaces between query and reference data
subspace_comparison <- comparePCASubspace(
    query_data = query_data_cd4,
    reference_data = ref_data_cd4, 
    query_cell_type_col = "expert_annotation", 
    ref_cell_type_col = "expert_annotation", 
    pc_subset = 1:5
)

# View weighted cosine similarity score
subspace_comparison$weighted_cosine_similarity

# Plot output for PCA subspace comparison (if a plot method is available)
plot(subspace_comparison)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Example usage of the function
plotPairwiseDistancesDensity(query_data = query_data, 
                             reference_data = reference_data, 
                             query_cell_type_col = "expert_annotation", 
                             ref_cell_type_col = "expert_annotation", 
                             cell_type = "CD4", 
                             pc_subset = 1:5,
                             distance_metric = "euclidean")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Generate the Wasserstein distance density plot
wasserstein_data <- calculateWassersteinDistance(
    query_data = query_data_cd4,
    reference_data = ref_data_cd4, 
    query_cell_type_col = "expert_annotation", 
    ref_cell_type_col = "expert_annotation", 
    pc_subset = 1:10,
)
plot(wasserstein_data)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Perform Cramer test for the specified cell types and principal components
cramer_test <- calculateCramerPValue(
    reference_data = reference_data,
    query_data = query_data,
    ref_cell_type_col = "expert_annotation", 
    query_cell_type_col = "SingleR_annotation",
    cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
    pc_subset = 1:5
)

# Display the Cramer test p-values
print(cramer_test)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Calculate Hotelling's T-squared test p-values
p_values <- calculateHotellingPValue(
    query_data = query_data, 
    reference_data = reference_data, 
    query_cell_type_col = "SingleR_annotation", 
    ref_cell_type_col = "expert_annotation",
    pc_subset = 1:10
)

# Display the p-values rounded to five decimal places
print(round(p_values, 5))

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Compute pairwise correlations between specified cell types
cor_matrix_avg <- calculateAveragePairwiseCorrelation(
  query_data = query_data, 
  reference_data = reference_data, 
  query_cell_type_col = "SingleR_annotation", 
  ref_cell_type_col = "expert_annotation", 
  cell_types = c("CD4", "CD8", "B_and_plasma"), 
  pc_subset = 1:5,
  correlation_method = "spearman"
)

# Visualize the average pairwise correlation matrix
plot(cor_matrix_avg)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Query-only analysis: How well do cell types separate in PCA space?
regress_res_query <- regressPC(
    query_data = query_data,
    query_cell_type_col = "expert_annotation",
    cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
    pc_subset = 1:10
)

# Visualize results
plot(regress_res_query, plot_type = "r_squared")
plot(regress_res_query, plot_type = "variance_contribution")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Plot results showing p-values
plot(regress_res_query, plot_type = "coefficient_heatmap")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Perform regression analysis using both reference and query data
regress_res <- regressPC(
    query_data = query_data,
    reference_data = reference_data,
    query_cell_type_col = "SingleR_annotation",
    ref_cell_type_col = "expert_annotation",
    cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
    pc_subset = 1:10
)

# Visualize results
plot(regress_res, plot_type = "r_squared")
plot(regress_res, plot_type = "variance_contribution")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Plot results showing p-values
plot(regress_res, plot_type = "coefficient_heatmap")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Load library to get top HVGs
library(scran)

# Select the top 500 highly variable genes from both datasets
ref_var_genes <- getTopHVGs(reference_data, n = 500)
query_var_genes <- getTopHVGs(query_data, n = 500)

# Calculate the overlap coefficient between the reference and query HVGs
overlap_coefficient <- calculateHVGOverlap(reference_genes = ref_var_genes, 
                                           query_genes = query_var_genes)

# Display the overlap coefficient
overlap_coefficient

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# RF function to compare which genes are best at differentiating cell types
rf_output <- calculateVarImpOverlap(reference_data = reference_data, 
                                    query_data = query_data, 
                                    query_cell_type_col = "expert_annotation", 
                                    ref_cell_type_col = "expert_annotation", 
                                    n_tree = 500,
                                    n_top = 50)

# Comparison table
rf_output$var_imp_comparison

## ----SessionInfo, echo=FALSE, message=FALSE, warning=FALSE, comment=NA, fig.show='hide'----
options(width = 80) 
sessionInfo()

#' @title Process PCA for SingleCellExperiment Objects
#'
#' @description
#' This function ensures that a \code{\linkS4class{SingleCellExperiment}} object has valid PCA computed using
#' highly variable genes when needed. It only performs downsampling when PCA computation
#' is required, preserving existing valid PCA computations without modification.
#'
#' @details
#' The function performs the following operations:
#' \itemize{
#'   \item Checks if PCA exists and is valid in the provided \code{\linkS4class{SingleCellExperiment}} object
#'   \item Validates PCA integrity including rotation matrix, percentVar, gene consistency, and dimensions
#'   \item If PCA is valid, returns the object unchanged (no downsampling)
#'   \item If PCA is missing or invalid and dataset is large, downsamples before computing PCA
#'   \item Computes PCA using highly variable genes when PCA is missing or invalid
#'   \item Utilizes scran for HVG selection and scater for PCA computation (soft dependencies)
#' }
#'
#' The downsampling strategy uses random sampling without replacement and only occurs
#' when PCA computation is necessary. This preserves expensive pre-computed PCA results
#' while ensuring computational efficiency for new PCA computations.
#'
#' PCA validation includes checking for:
#' \itemize{
#'   \item Presence of PCA in reducedDims
#'   \item Existence of rotation matrix and percentVar attributes
#'   \item Gene consistency between rotation matrix and current assay
#'   \item Dimension consistency between PCA coordinates and cell count
#' }
#'
#' @param sce_object A \code{\linkS4class{SingleCellExperiment}} object to process.
#' @param assay_name Name of the assay to use for HVG selection and PCA computation.
#' Should contain log-normalized expression values. Default is "logcounts".
#' @param n_hvgs Number of highly variable genes to select for PCA computation. Default is 2000.
#' @param max_cells Maximum number of cells to retain if downsampling is needed for PCA computation.
#' If NULL, no downsampling is performed. Default is NULL.
#'
#' @return A \code{\linkS4class{SingleCellExperiment}} object with valid PCA in the reducedDims slot,
#' including rotation matrix and percentVar attributes. Will have original cell count if PCA was valid,
#' or at most max_cells if PCA was computed.
#'
#' @note
#' This function requires the scran and scater packages for HVG selection and PCA computation.
#' These packages should be installed via BiocManager::install(c("scran", "scater")).
#'
#' Objects with existing valid PCA are returned unchanged to preserve expensive pre-computations.
#' Only datasets requiring PCA computation are subject to downsampling.
#'
#' @examples
#' library(SingleCellExperiment)
#'
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Example 1: Dataset without PCA (will compute PCA)
#' query_no_pca <- query_data
#' reducedDims(query_no_pca) <- list()  # Remove existing PCA
#'
#' processed_query <- processPCA(sce_object = query_no_pca, n_hvgs = 500)
#' "PCA" %in% reducedDimNames(processed_query)  # Should be TRUE
#' ncol(processed_query)  # Should be 503 (unchanged)
#'
#' # Example 2: Dataset with existing valid PCA (will be preserved)
#' processed_existing <- processPCA(sce_object = query_data, n_hvgs = 500)
#' ncol(processed_existing)  # Should be 503 (unchanged, no downsampling)
#'
#' # Example 3: Large dataset requiring downsampling for PCA computation
#' ref_no_pca <- reference_data
#' reducedDims(ref_no_pca) <- list()  # Remove existing PCA
#'
#' processed_large <- processPCA(sce_object = ref_no_pca,
#'                               n_hvgs = 800,
#'                               max_cells = 1000)
#' ncol(processed_large)  # Should be 1000 (downsampled for PCA computation)
#'
#' # Example 4: Large dataset with existing PCA (no downsampling)
#' processed_large_existing <- processPCA(sce_object = reference_data,
#'                                        n_hvgs = 800,
#'                                        max_cells = 1000)
#' ncol(processed_large_existing)  # Should be 1500 (preserved, no downsampling)
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
# Function to process SCE objects with PCA computation
processPCA <- function(sce_object,
                       assay_name = "logcounts",
                       n_hvgs = 2000,
                       max_cells = NULL) {

    # Validate input
    if (!is(sce_object, "SingleCellExperiment")) {
        stop("'sce_object' must be a SingleCellExperiment object.")
    }

    if (!is.null(assay_name) &&
        !(assay_name %in% SummarizedExperiment::assayNames(sce_object))) {
        stop("'sce_object' does not contain the specified assay.")
    }

    # Check max_cells
    if (!is.null(max_cells)) {
        if (!is.numeric(max_cells) || max_cells <= 0 ||
            max_cells != as.integer(max_cells)) {
            stop("'max_cells' must be a positive integer.")
        }
    }

    # Check n_hvgs
    if (!is.numeric(n_hvgs) || length(n_hvgs) != 1 ||
        n_hvgs <= 0 || n_hvgs != as.integer(n_hvgs)) {
        stop("'n_hvgs' must be a single positive integer.")
    }
    n_hvgs <- as.integer(n_hvgs)

    # Helper function to validate PCA integrity
    .validatePCA <- function(sce, assay_name) {
        # Check if PCA exists
        if (!"PCA" %in% reducedDimNames(sce)) {
            return(FALSE)
        }

        # Check if rotation matrix exists
        pca_attrs <- attributes(reducedDim(sce, "PCA"))
        rotation_mat <- pca_attrs[["rotation"]]
        if (is.null(rotation_mat)) {
            return(FALSE)
        }

        # Check if percentVar exists
        percent_var <- pca_attrs[["percentVar"]]
        if (is.null(percent_var)) {
            return(FALSE)
        }

        # Check if genes in rotation matrix exist in the assay
        pca_genes <- rownames(rotation_mat)
        assay_genes <- rownames(assay(sce, assay_name))
        if (!all(pca_genes %in% assay_genes)) {
            return(FALSE)
        }

        # Check dimension consistency
        pca_coords <- reducedDim(sce, "PCA")
        if (nrow(pca_coords) != ncol(sce)) {
            return(FALSE)
        }

        return(TRUE)
    }

    # Helper function to compute PCA with HVGs
    .computePCAWithHvgs <- function(sce, n_hvgs, assay_name) {

        # Check if required packages are available
        if (!requireNamespace("scran", quietly = TRUE)) {
            stop("Package 'scran' is required but not installed. Please install it with: BiocManager::install('scran')")
        }
        if (!requireNamespace("scater", quietly = TRUE)) {
            stop("Package 'scater' is required but not installed. Please install it with: BiocManager::install('scater')")
        }

        # Get HVGs
        var_stats <- scran::modelGeneVar(sce, assay.type = assay_name)
        hvg_genes <- scran::getTopHVGs(var_stats, n = n_hvgs)

        message("Using ", length(hvg_genes), " highly variable genes for PCA computation")

        # Compute PCA on HVGs
        sce <- scater::runPCA(sce,
                              assay.type = assay_name,
                              subset_row = hvg_genes)

        return(sce)
    }

    # Check if PCA is valid
    n_cells <- ncol(sce_object)
    has_valid_pca <- .validatePCA(sce_object, assay_name)

    if (has_valid_pca) {
        # PCA exists AND is valid - return unchanged (no downsampling)
        message("Data already has valid PCA - returning unchanged (", n_cells, " cells)")
        return(sce_object)
    } else {
        # PCA is missing or invalid - needs to be computed
        if ("PCA" %in% reducedDimNames(sce_object)) {
            message("Data has invalid PCA - recomputing...")
        } else {
            message("Data missing PCA - computing...")
        }

        # Downsample if needed
        processed_sce <- sce_object
        if (!is.null(max_cells) && n_cells > max_cells) {
            message("Downsampling data from ", n_cells, " to ",
                    max_cells, " cells before PCA computation")
            cell_indices <- sample(n_cells, max_cells)
            processed_sce <- sce_object[, cell_indices]
        }

        # Compute PCA
        message("Computing PCA...")
        processed_sce <- .computePCAWithHvgs(processed_sce,
                                             n_hvgs,
                                             assay_name)

        return(processed_sce)
    }
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load the datasets that come with the package
data("query_data", package = "scDiagnostics")
data("reference_data", package = "scDiagnostics")

test_that("processPCA works with single dataset without existing PCA", {
    skip_if_not_installed("scater")
    skip_if_not_installed("scran")

    # Create test dataset without PCA
    test_data <- query_data
    reducedDims(test_data) <- list()  # Remove all existing PCA

    # Test single dataset without PCA
    result <- processPCA(sce_object = test_data, n_hvgs = 300)

    expect_s4_class(result, "SingleCellExperiment")
    expect_true("PCA" %in% reducedDimNames(result))
    expect_equal(ncol(result), 503)  # Should preserve all cells

    # Check PCA attributes exist
    pca_attrs <- attributes(reducedDim(result, "PCA"))
    expect_true("rotation" %in% names(pca_attrs))
    expect_true("percentVar" %in% names(pca_attrs))
})

test_that("processPCA preserves existing valid PCA", {
    skip_if_not_installed("scater")
    skip_if_not_installed("scran")

    # Use dataset that already has PCA
    test_data <- query_data
    original_pca <- reducedDim(test_data, "PCA")

    # Process data that already has valid PCA
    result <- processPCA(sce_object = test_data, n_hvgs = 300)

    expect_s4_class(result, "SingleCellExperiment")
    expect_true("PCA" %in% reducedDimNames(result))
    expect_equal(ncol(result), 503)  # Should preserve all cells

    # Should return the same object unchanged
    expect_identical(reducedDim(result, "PCA"), original_pca)
})

test_that("processPCA works with larger dataset", {
    skip_if_not_installed("scater")
    skip_if_not_installed("scran")

    # Use reference dataset (larger) without PCA
    test_data <- reference_data
    reducedDims(test_data) <- list()  # Remove existing PCA

    # Test with max_cells limit
    result <- processPCA(sce_object = test_data, n_hvgs = 500, max_cells = 800)

    expect_s4_class(result, "SingleCellExperiment")
    expect_true("PCA" %in% reducedDimNames(result))
    expect_equal(ncol(result), 800)  # Should be downsampled to max_cells
})

test_that("processPCA preserves large dataset with valid PCA", {
    skip_if_not_installed("scater")
    skip_if_not_installed("scran")

    # Use reference dataset that already has PCA
    test_data <- reference_data

    # Process data that already has valid PCA - should NOT downsample
    result <- processPCA(sce_object = test_data, n_hvgs = 500, max_cells = 800)

    expect_s4_class(result, "SingleCellExperiment")
    expect_true("PCA" %in% reducedDimNames(result))
    expect_equal(ncol(result), 1500)  # Should preserve all 1500 cells (no downsampling)
})

test_that("processPCA input validation", {
    # Test invalid input
    expect_error(processPCA("not_an_sce"),
                 "'sce_object' must be a SingleCellExperiment object")

    expect_error(processPCA(query_data, assay_name = "nonexistent"),
                 "'sce_object' does not contain the specified assay")

    expect_error(processPCA(query_data, n_hvgs = -1),
                 "'n_hvgs' must be a single positive integer")

    expect_error(processPCA(query_data, max_cells = -1),
                 "'max_cells' must be a positive integer")
})

#' @title Plot Top Loading Gene Expression Shifts
#'
#' @description
#' This function creates visualizations showing expression distributions for top loading genes
#' that exhibit distributional differences between query and reference datasets. Can display
#' results as elegant complex heatmaps, information-rich summary boxplots, or pseudo-bulk fold
#' change barplots. Optionally displays anomaly status when available.
#'
#' @details
#' This function visualizes the results from \code{calculateTopLoadingGeneShifts}.
#' The "heatmap" option displays a hierarchically clustered set of genes.
#' The "boxplot" option creates a two-panel plot using `ggplot2`: the left panel shows
#' horizontal expression boxplots for up to 5 PCs, while the right panel displays their
#' corresponding PC loadings and adjusted p-values.
#' The "barplot" option creates horizontal barplots showing log2 fold changes between
#' pseudo-bulk expression profiles (query vs reference), with genes ordered identically
#' to the heatmap clustering. Bars show comparisons for query non-anomaly (green),
#' optionally all query cells (yellow), and query anomaly cells (red) versus reference.
#' When anomaly detection results are available and \code{show_anomalies} is TRUE,
#' additional annotation bars or visual cues highlight anomalous cells.
#'
#' @param x An object of class \code{calculateTopLoadingGeneShiftsObject}.
#' @param cell_type A character string specifying the cell type to plot (must be exactly one).
#' @param pc_subset A numeric vector specifying which principal components to plot. Default is 1:3.
#' @param plot_type A character string specifying visualization type. Either "heatmap", "barplot", or "boxplot".
#'                  Default is "heatmap".
#' @param plot_by A character string specifying gene selection method when `n_genes` is not NULL.
#'                Either "top_loading" or "p_adjusted". Default is "p_adjusted".
#' @param n_genes Number of top genes to show per PC. Can be NULL if `significance_threshold` is set.
#'                Default is 10.
#' @param significance_threshold If not NULL, a numeric value between 0 and 1. Used for gene
#'   selection or annotation. Default is 0.05.
#' @param show_anomalies Logical indicating whether to display anomaly status annotations.
#'                      Default is FALSE. Requires anomaly results to be present in the object.
#' @param show_all_query Logical indicating whether to show the yellow bar representing all
#'                       query cells vs reference in barplot visualization. Only applies when
#'                       \code{plot_type = "barplot"} and anomaly data is available. Default is TRUE.
#' @param pseudo_bulk Logical indicating whether to create pseudo-bulk profiles instead of
#'                    showing individual cells. When TRUE, expression values are averaged within groups
#'                    (dataset and optionally anomaly status). Not compatible with boxplot visualization.
#'                    Required for barplot visualization. Default is FALSE.
#' @param cluster_cols Logical indicating whether to cluster columns in the heatmap when
#'                    `pseudo_bulk = TRUE`. When TRUE, columns (pseudo-bulk profiles) will be
#'                    hierarchically clustered. When FALSE, columns maintain their original ordering
#'                    (Query groups followed by Reference groups). Only applicable when
#'                    `pseudo_bulk = TRUE` and `plot_type = "heatmap"`. Default is FALSE.
#' @param draw_plot Logical indicating whether to draw the plot immediately (TRUE) or return
#'                  the undrawn plot object (FALSE). For heatmaps, FALSE returns a ComplexHeatmap
#'                  object that can be further customized before drawing. Default is TRUE.
#' @param show_all_query Logical indicating whether to show the yellow bar for all query vs reference
#'                       comparison. Default is TRUE. When FALSE, only green and red bars are shown.
#' @param max_cells_ref Maximum number of reference cells to include in the plot. If NULL,
#' all available reference cells are plotted. Default is NULL.
#' @param max_cells_query Maximum number of query cells to include in the plot. If NULL,
#' all available query cells are plotted. Default is NULL.
#' @param ... Additional arguments passed to \code{\link[ComplexHeatmap]{draw}} or not used for other plot types.
#'
#' @return A plot object. For heatmaps when \code{draw_plot = FALSE}, returns a ComplexHeatmap object.
#' For boxplots and barplots, returns a ggplot2 object.
#'
#' @export
#'
#' @author
#' Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{calculateTopLoadingGeneShifts}}
#'
#' @rdname calculateTopLoadingGeneShifts
#'
#' @importFrom stats wilcox.test var p.adjust na.omit setNames aggregate reshape
#'
# Function to visualize genes with top loadings
plot.calculateTopLoadingGeneShiftsObject <- function(x,
                                                     cell_type,
                                                     pc_subset = 1:3,
                                                     plot_type = c("heatmap", "barplot", "boxplot"),
                                                     plot_by = c("p_adjusted", "top_loading"),
                                                     n_genes = 10,
                                                     significance_threshold = 0.05,
                                                     show_anomalies = FALSE,
                                                     pseudo_bulk = FALSE,
                                                     cluster_cols = FALSE,
                                                     draw_plot = TRUE,
                                                     show_all_query = TRUE,
                                                     max_cells_ref = NULL,
                                                     max_cells_query = NULL,
                                                     ...) {

    #Input Validation
    if (missing(cell_type) || length(cell_type) != 1) {
        stop("cell_type must be specified and be a single character string.")
    }

    # Match arguments first
    plot_type <- match.arg(plot_type)
    plot_by <- match.arg(plot_by)

    # Validate show_anomalies parameter
    if (!is.logical(show_anomalies) || length(show_anomalies) != 1) {
        stop("show_anomalies must be a logical value.")
    }

    # Validate max_cells parameters
    if (!is.null(max_cells_ref)) {
        if (!is.numeric(max_cells_ref) || max_cells_ref <= 0 || max_cells_ref != as.integer(max_cells_ref)) {
            stop("'max_cells_ref' must be a positive integer.")
        }
    }

    if (!is.null(max_cells_query)) {
        if (!is.numeric(max_cells_query) || max_cells_query <= 0 || max_cells_query != as.integer(max_cells_query)) {
            stop("'max_cells_query' must be a positive integer.")
        }
    }

    # Add after existing validation
    if (!is.logical(pseudo_bulk) || length(pseudo_bulk) != 1) {
        stop("pseudo_bulk must be a logical value.")
    }

    if (!is.logical(cluster_cols) || length(cluster_cols) != 1) {
        stop("cluster_cols must be a logical value.")
    }

    # Restrict boxplot with pseudo_bulk
    if (pseudo_bulk && plot_type == "boxplot") {
        stop("Boxplot visualization is not compatible with pseudo_bulk = TRUE. ",
             "Please use plot_type = 'heatmap' or set pseudo_bulk = FALSE.")
    }

    # Require pseudo_bulk for barplot
    if (plot_type == "barplot" && !pseudo_bulk) {
        stop("Barplot visualization requires pseudo_bulk = TRUE. ",
             "Please set pseudo_bulk = TRUE for barplot.")
    }

    # Restrict cluster_cols without pseudo_bulk
    if (cluster_cols && !pseudo_bulk) {
        stop("cluster_cols = TRUE requires pseudo_bulk = TRUE.")
    }

    # Check if anomaly data is available when requested
    if (show_anomalies) {
        if (!"anomaly_status" %in% names(x[["cell_metadata"]])) {
            stop("Anomaly visualization requested but anomaly data not found in object. ",
                 "Please re-run calculateTopLoadingGeneShifts() with detect_anomalies = TRUE.")
        }
    }

    # Validate plot-type specific requirements
    if (plot_type %in% c("boxplot", "barplot") && is.null(n_genes)) {
        stop("For '", plot_type, "' plot type, 'n_genes' must be specified.")
    }
    if (plot_type == "heatmap" &&
        is.null(n_genes) &&
        is.null(significance_threshold)) {
        stop("For 'heatmap' plot type, at least one of 'n_genes' or 'significance_threshold' must be specified.")
    }
    if (!is.null(n_genes) &&
        (!is.numeric(n_genes) ||
         length(n_genes) != 1 ||
         n_genes <= 0)) {
        stop("If not NULL, 'n_genes' must be a positive integer.")
    }
    if (!is.null(significance_threshold) &&
        (!is.numeric(significance_threshold) ||
         length(significance_threshold) != 1 ||
         significance_threshold < 0 ||
         significance_threshold > 1)) {
        stop("If not NULL, 'significance_threshold' must be a numeric value between 0 and 1.")
    }

    # Check for package dependencies
    if (plot_type == "heatmap") {
        if (!requireNamespace("ComplexHeatmap", quietly = TRUE)) {
            stop("Packages 'ComplexHeatmap' and 'circlize' are required. Please install them.",
                 call. = FALSE)
        }
    }

    # Check object structure
    required_elements <- c("expression_data", "cell_metadata", "percent_var")
    if (!all(required_elements %in% names(x))) {
        stop("Input object 'x' is missing required elements.", call. = FALSE)
    }

    # Get available PCs from the object
    pc_names <- paste0("PC", pc_subset)
    available_pcs <- intersect(pc_names, names(x))
    if (length(available_pcs) == 0) {
        stop("None of the requested PCs were found in the results object.")
    }
    available_pcs <- pc_names[pc_names %in% available_pcs]

    # Limit PCs for boxplot
    if (plot_type == "boxplot" && length(available_pcs) > 5) {
        warning("Boxplot type can display at most 5 PCs. Using the first 5.")
        available_pcs <- available_pcs[1:5]
    }

    # Check cell type existence
    if (!cell_type %in% x[["cell_metadata"]][["cell_type"]]) {
        stop(paste("Cell type '", cell_type, "' not found in results.", sep = ""))
    }

    # Downsample cells if max_cells parameters are specified
    cell_metadata_filtered <- x[["cell_metadata"]]
    cell_subset <- cell_metadata_filtered[cell_metadata_filtered[["cell_type"]] == cell_type, ]

    # Separate reference and query data for potential downsampling
    ref_cells <- cell_subset[cell_subset[["dataset"]] == "Reference", ]
    query_cells <- cell_subset[cell_subset[["dataset"]] == "Query", ]

    # Downsample reference cells if specified
    if (!is.null(max_cells_ref) && nrow(ref_cells) > max_cells_ref) {
        sampled_indices <- sample(nrow(ref_cells), max_cells_ref)
        ref_cells <- ref_cells[sampled_indices, ]
    }

    # Downsample query cells if specified
    if (!is.null(max_cells_query) && nrow(query_cells) > max_cells_query) {
        sampled_indices <- sample(nrow(query_cells), max_cells_query)
        query_cells <- query_cells[sampled_indices, ]
    }

    # Combine the potentially downsampled cells back together
    cell_subset_final <- rbind(ref_cells, query_cells)

    # Update the object with downsampled data for plotting
    x_plotting <- x
    x_plotting[["cell_metadata"]] <- cell_subset_final
    x_plotting[["expression_data"]] <- x[["expression_data"]][, cell_subset_final[["cell_id"]],
                                                              drop = FALSE]

    # Route to Plotting Functions
    if (plot_type == "heatmap") {
        ht <- plotHeatmap(x_plotting, cell_type,
                          available_pcs, plot_by, n_genes,
                          significance_threshold, show_anomalies,
                          pseudo_bulk, cluster_cols)

        if (is.null(ht)) {
            message("No data available to generate a heatmap for the specified parameters.")
            return(invisible(NULL))
        }

        # Return drawn or undrawn heatmap based on draw_plot parameter
        if (draw_plot) {
            # Draw the heatmap and return the drawn object
            return(
                ComplexHeatmap::draw(ht,
                                     heatmap_legend_side = "right",
                                     annotation_legend_side = "right",
                                     merge_legends = TRUE,
                                     ...)
            )
        } else {
            # Return the undrawn Heatmap object for further customization
            return(ht)
        }
    } else if (plot_type == "barplot") {
        return(plotBarplot(x_plotting, cell_type,
                           available_pcs, plot_by,
                           n_genes, significance_threshold,
                           show_anomalies, show_all_query))
    } else {
        return(plotBoxplot(x_plotting, cell_type,
                           available_pcs, plot_by,
                           n_genes, significance_threshold,
                           show_anomalies))
    }
}

#' @title Plot Heatmaps for Top Loading Gene Shifts (Simplified Single Heatmap)
#'
#' @description
#' This internal helper function creates a single, hierarchically clustered heatmap
#' displaying expression data for top loading genes from principal component analysis.
#' The function handles gene selection, data preprocessing, and visualization formatting.
#' Optionally includes anomaly status annotations with proper cell ordering.
#'
#' @details
#' This function generates a ComplexHeatmap visualization showing scaled gene expression
#' data across cells. Genes are selected based on either their loading values or
#' statistical significance. The function automatically handles data filtering,
#' scaling, and visual formatting including color schemes and annotations.
#'
#' Cell ordering (left to right):
#' 1. Query Anomalous cells (leftmost)
#' 2. Query Normal cells
#' 3. Reference Anomalous cells
#' 4. Reference Normal cells (rightmost)
#'
#' This ensures clear dataset separation while grouping anomalous cells together
#' within each dataset for easy visual identification.
#'
#' @param x An object of class \code{calculateTopLoadingGeneShiftsObject} containing
#'          expression data and analysis results.
#' @param cell_type A character string specifying the cell type to visualize.
#' @param available_pcs A character vector of principal components to include in analysis.
#' @param plot_by A character string indicating gene selection criterion ("top_loading" or "p_adjusted").
#' @param n_genes An integer specifying the number of top genes to display per PC. Can be NULL.
#' @param significance_threshold A numeric value between 0 and 1 for significance filtering. Can be NULL.
#' @param show_anomalies Logical indicating whether to show anomaly annotations.
#' @param pseudo_bulk Logical indicating whether to create pseudo-bulk profiles instead of
#'                    showing individual cells. When TRUE, expression values are averaged within groups
#'                    (dataset and optionally anomaly status). Not compatible with boxplot visualization.
#'                    Default is FALSE.
#' @param cluster_cols Logical indicating whether to cluster columns in the heatmap when
#'                    `pseudo_bulk = TRUE`. When TRUE, columns (pseudo-bulk profiles) will be
#'                    hierarchically clustered. When FALSE, columns maintain their original ordering
#'                    (Query groups followed by Reference groups). Only applicable when
#'                    `pseudo_bulk = TRUE` and `plot_type = "heatmap"`. Default is FALSE.
#'
#' @keywords internal
#'
#' @return A ComplexHeatmap object ready for plotting, or NULL if no genes meet selection criteria.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
# Helper heatmap function
plotHeatmap <- function(x, cell_type,
                        available_pcs, plot_by,
                        n_genes, significance_threshold,
                        show_anomalies, pseudo_bulk = FALSE,
                        cluster_cols = FALSE) {

    # Initialize gene collection
    all_genes_to_plot <- c()

    # Gene selection based on n_genes criterion
    if (!is.null(n_genes)) {
        gene_list <- lapply(available_pcs, function(pc_name) {
            pc_results <- x[[pc_name]]
            if (is.null(pc_results) || nrow(pc_results) == 0) {
                return(NULL)
            }

            pc_cell_data <- pc_results[pc_results[["cell_type"]] == cell_type, ]
            if (nrow(pc_cell_data) == 0) {
                return(NULL)
            }

            # Order genes by selection criterion
            if (plot_by == "top_loading") {
                gene_order <- order(abs(pc_cell_data[["loading"]]),
                                    decreasing = TRUE)
            } else {
                gene_order <- order(pc_cell_data[["p_adjusted"]])
            }

            selected_indices <- gene_order[1:min(n_genes, nrow(pc_cell_data))]
            return(pc_cell_data[selected_indices, "gene"])
        })
        all_genes_to_plot <- unique(unlist(gene_list))

    } else if (!is.null(significance_threshold)) {
        # Gene selection based on significance threshold
        sig_gene_list <- lapply(available_pcs, function(pc_name) {
            pc_results <- x[[pc_name]]
            if (is.null(pc_results) || nrow(pc_results) == 0) {
                return(NULL)
            }

            sig_df <- pc_results[pc_results[["cell_type"]] == cell_type &
                                     pc_results[["p_adjusted"]] < significance_threshold, ]
            return(sig_df[["gene"]])
        })
        all_genes_to_plot <- unique(unlist(sig_gene_list))
    }

    # Check if any genes were selected
    if (length(all_genes_to_plot) == 0) {
        return(NULL)  # Return NULL instead of warning
    }

    # Identify significant genes for annotation
    significant_genes <- c()
    if (!is.null(significance_threshold)) {
        sig_gene_list <- lapply(available_pcs, function(pc_name) {
            pc_results <- x[[pc_name]]
            if (is.null(pc_results) || nrow(pc_results) == 0) {
                return(NULL)
            }

            sig_df <- pc_results[pc_results[["cell_type"]] == cell_type &
                                     pc_results[["p_adjusted"]] < significance_threshold, ]
            return(sig_df[["gene"]])
        })
        significant_genes <- unique(unlist(sig_gene_list))
    }

    # Prepare cell subset
    cell_subset <- x[["cell_metadata"]][x[["cell_metadata"]][["cell_type"]] == cell_type, ]

    # Handle pseudo-bulk aggregation
    if (pseudo_bulk) {
        # Create grouping categories
        if (show_anomalies && "anomaly_status" %in% names(cell_subset)) {
            # Only show anomalies for query cells, treat all reference as normal
            cell_subset_modified <- cell_subset
            cell_subset_modified[["anomaly_status"]][cell_subset_modified[["dataset"]] == "Reference"] <- "Normal"

            # 3 categories: Query_Normal, Query_Anomaly, Reference_Normal (no Reference_Anomaly)
            cell_subset_modified[["group"]] <- paste(cell_subset_modified[["dataset"]],
                                                     cell_subset_modified[["anomaly_status"]],
                                                     sep = "_")
            group_order <- c("Query_Normal", "Query_Anomaly", "Reference_Normal")

            # Update cell_subset to use modified version
            cell_subset <- cell_subset_modified
            cell_subset[["group"]] <- cell_subset_modified[["group"]]
        } else {
            # 2 categories: Query, Reference
            cell_subset[["group"]] <- cell_subset[["dataset"]]
            group_order <- c("Query", "Reference")
        }

        # Aggregate expression data by groups
        expr_matrix_unfiltered <- as.matrix(
            x[["expression_data"]][all_genes_to_plot, cell_subset[["cell_id"]], drop = FALSE]
        )

        # Create pseudo-bulk profiles
        pseudo_bulk_matrix <- matrix(0, nrow = nrow(expr_matrix_unfiltered), ncol = length(group_order))
        rownames(pseudo_bulk_matrix) <- rownames(expr_matrix_unfiltered)
        colnames(pseudo_bulk_matrix) <- group_order

        for (group in group_order) {
            group_cells <- cell_subset[cell_subset[["group"]] == group, "cell_id"]
            if (length(group_cells) > 0) {
                if (length(group_cells) == 1) {
                    pseudo_bulk_matrix[, group] <- expr_matrix_unfiltered[, group_cells]
                } else {
                    pseudo_bulk_matrix[, group] <- rowMeans(expr_matrix_unfiltered[, group_cells, drop = FALSE])
                }
            }
        }

        # Update expression matrix for plotting
        expr_matrix_unfiltered <- pseudo_bulk_matrix

        # Create new cell_subset for annotations
        cell_subset_pseudo <- data.frame(
            cell_id = group_order,
            group = group_order,
            stringsAsFactors = FALSE
        )

        # Parse group names back to dataset and anomaly status
        if (show_anomalies && "anomaly_status" %in% names(cell_subset)) {
            cell_subset_pseudo[["dataset"]] <- gsub("_.*", "", cell_subset_pseudo[["group"]])
            cell_subset_pseudo[["anomaly_status"]] <- gsub(".*_", "", cell_subset_pseudo[["group"]])

            # Ensure reference cells are marked as Normal (redundant safety check)
            cell_subset_pseudo[["anomaly_status"]][cell_subset_pseudo[["dataset"]] == "Reference"] <- "Normal"
        } else {
            cell_subset_pseudo[["dataset"]] <- cell_subset_pseudo[["group"]]
        }

        cell_subset <- cell_subset_pseudo
    } else {
        # Original ordering logic for individual cells
        if (show_anomalies && "anomaly_status" %in% names(cell_subset)) {
            # Only show anomalies for query cells, treat all reference as normal
            cell_subset[["anomaly_status"]][cell_subset[["dataset"]] == "Reference"] <- "Normal"

            cell_subset <- cell_subset[order(
                factor(cell_subset[["dataset"]], levels = c("Query", "Reference")),
                factor(cell_subset[["anomaly_status"]], levels = c("Anomaly", "Normal"))
            ), ]
        } else {
            cell_subset <- cell_subset[order(
                factor(cell_subset[["dataset"]], levels = c("Query", "Reference"))
            ), ]
        }

        # Extract expression matrix for individual cells
        expr_matrix_unfiltered <- as.matrix(
            x[["expression_data"]][all_genes_to_plot, cell_subset[["cell_id"]], drop = FALSE]
        )
    }

    # Remove zero-variance genes
    row_variances <- apply(expr_matrix_unfiltered, 1, stats::var)
    genes_to_keep <- row_variances > .Machine$double.eps

    if (sum(genes_to_keep) == 0) {
        return(NULL)
    }

    # Create final expression matrix and scale
    expr_matrix <- expr_matrix_unfiltered[genes_to_keep, , drop = FALSE]
    scaled_matrix <- t(scale(t(expr_matrix)))

    # Clamp z-scores to the range [-2, 2] for consistent visualization
    scaled_matrix[scaled_matrix < -2] <- -2
    scaled_matrix[scaled_matrix > 2] <- 2

    # Define color schemes
    dataset_colors <- c("Query" = "#B565D8", "Reference" = "#5A9BD8")
    anomaly_colors <- c("Non-Anomalous" = "#C8C8C8", "Anomalous" = "#D2314C")

    # Create custom color mapping function
    .createColorMapping <- function(breaks, colors) {
        function(x) {
            # Handle NA values
            if (any(is.na(x))) {
                return(rep("grey", length(x)))
            }

            # Clamp values to break range
            x[x < min(breaks)] <- min(breaks)
            x[x > max(breaks)] <- max(breaks)

            # Create color ramp and normalize values
            col_ramp <- grDevices::colorRamp(colors)
            x_norm <- (x - min(breaks)) / (max(breaks) - min(breaks))

            # Convert to hex colors
            rgb_vals <- col_ramp(x_norm)
            grDevices::rgb(rgb_vals[,1], rgb_vals[,2], rgb_vals[,3], maxColorValue = 255)
        }
    }
    zscore_col_fun <- .createColorMapping(c(-2, 0, 2),
                                          c("#313695", "white", "#A50026"))

    # Create top annotation with query-only anomaly status
    annotation_list <- list()
    color_list <- list()
    legend_params <- list()

    if (show_anomalies && "anomaly_status" %in% names(cell_subset)) {
        # Map anomaly status to new labels (reference cells are already set to "Normal")
        anomaly_labels <- ifelse(cell_subset[["anomaly_status"]] == "Anomaly",
                                 "Anomalous", "Non-Anomalous")
        annotation_list[["Status"]] <- anomaly_labels
        color_list[["Status"]] <- anomaly_colors
        legend_params[["Status"]] <- list(title = "Query Anomaly Status")
    }

    # Add dataset annotation below anomaly status
    annotation_list[["Dataset"]] <- cell_subset[["dataset"]]
    color_list[["Dataset"]] <- dataset_colors
    legend_params[["Dataset"]] <- list(title = "Dataset")

    top_ha <- ComplexHeatmap::HeatmapAnnotation(
        df = as.data.frame(annotation_list),
        col = color_list,
        show_legend = TRUE,
        show_annotation_name = FALSE,
        annotation_legend_param = legend_params
    )

    # Prepare gene labels with significance indicators
    final_genes <- rownames(expr_matrix)
    row_labels <- paste0(final_genes,
                         ifelse(final_genes %in% significant_genes, "*", ""))

    # Smart dependency checking for grid package
    if (requireNamespace("grid", quietly = TRUE)) {
        # Use smaller font size for better readability when grid is available
        row_gp <- grid::gpar(fontsize = 8)
        ht <- ComplexHeatmap::Heatmap(
            matrix = scaled_matrix,
            name = "Z-Score",
            col = zscore_col_fun,
            heatmap_legend_param = list(
                title = "Z-Score",
                at = c(-2, -1, 0, 1, 2),
                labels = c("-2", "-1", "0", "1", "2")
            ),
            show_row_names = TRUE,
            row_labels = row_labels,
            row_names_gp = row_gp,
            cluster_rows = TRUE,
            show_column_names = FALSE,
            cluster_columns = if(pseudo_bulk && cluster_cols) TRUE else FALSE,
            column_order = if(pseudo_bulk && cluster_cols) NULL else cell_subset[["cell_id"]],
            top_annotation = top_ha,
            border = TRUE,
            use_raster = TRUE,
            raster_quality = 2
        )
    } else {
        # Use default font size and inform user about grid package
        message("Note: For optimal gene name readability, consider installing the 'grid' package: install.packages('grid')")
        ht <- ComplexHeatmap::Heatmap(
            matrix = scaled_matrix,
            name = "Z-Score",
            col = zscore_col_fun,
            heatmap_legend_param = list(
                title = "Z-Score",
                at = c(-2, -1, 0, 1, 2),
                labels = c("-2", "-1", "0", "1", "2")
            ),
            show_row_names = TRUE,
            row_labels = row_labels,
            row_names_gp = row_gp,
            cluster_rows = TRUE,
            show_column_names = FALSE,
            cluster_columns = if(pseudo_bulk && cluster_cols) TRUE else FALSE,
            column_order = if(pseudo_bulk && cluster_cols) NULL else cell_subset[["cell_id"]],
            top_annotation = top_ha,
            border = TRUE,
            use_raster = TRUE,
            raster_quality = 2
        )
    }

    return(ht)
}

#' @title Plot Barplots for Top Loading Gene Fold Changes (Pseudo-Bulk)
#'
#' @description
#' This internal helper function creates a barplot visualization showing pseudo-bulk fold changes
#' between different cell groups for top loading genes.
#'
#' @details
#' This function generates a ggplot2 barplot where genes are arranged vertically
#' in hierarchically clustered order (identical to heatmap gene ordering), and horizontal bars show log2 fold changes
#' for different pseudo-bulk comparisons vs reference. The function creates an internal heatmap
#' object with the same parameters to extract the exact gene clustering order, ensuring
#' consistency between plot types.
#'
#' Bar colors:
#' \itemize{
#'   \item Green: Query non-anomaly (pseudo-bulk) vs Reference (pseudo-bulk)
#'   \item Yellow: All Query (pseudo-bulk) vs Reference (pseudo-bulk) (shown when show_all_query = TRUE and anomaly data available)
#'   \item Red: Query anomaly (pseudo-bulk) vs Reference (pseudo-bulk)
#' }
#'
#' @param x An object of class \code{calculateTopLoadingGeneShiftsObject} containing
#'          expression data and analysis results.
#' @param cell_type A character string specifying the cell type to visualize.
#' @param available_pcs A character vector of principal components to include in analysis.
#' @param plot_by A character string indicating gene selection criterion ("top_loading" or "p_adjusted").
#' @param n_genes An integer specifying the number of top genes to display per PC.
#' @param significance_threshold A numeric value between 0 and 1 for significance annotation.
#' @param show_anomalies Logical indicating whether to show anomaly-related bars.
#' @param show_all_query Logical indicating whether to show the yellow bar for all query vs reference
#'                       comparison. Default is TRUE. When FALSE, only green and red bars are shown.
#'
#' @keywords internal
#'
#' @return A ggplot2 object ready for display, or NULL if no genes meet selection criteria.
#'
#' @author
#' Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
plotBarplot <- function(x, cell_type, available_pcs, plot_by,
                        n_genes, significance_threshold, show_anomalies,
                        show_all_query = TRUE) {

    # Validate required parameters
    if (is.null(n_genes)) {
        stop("'plot_type = \"barplot\"' requires 'n_genes' to be set.")
    }

    # Validate show_all_query parameter
    if (!is.logical(show_all_query) || length(show_all_query) != 1) {
        stop("show_all_query must be a logical value.")
    }

    # Create internal heatmap object to extract gene order
    # Use the same parameters as would be used for heatmap plotting
    internal_heatmap <- tryCatch({
        plotHeatmap(x, cell_type, available_pcs, plot_by, n_genes,
                    significance_threshold, show_anomalies,
                    pseudo_bulk = TRUE, cluster_cols = FALSE)
    }, error = function(e) {
        stop("Failed to create internal heatmap for gene ordering: ", e$message)
    })

    if (is.null(internal_heatmap)) {
        warning("No genes met the selection criteria for the barplot.")
        return(NULL)
    }

    # Extract gene order from the heatmap
    gene_order_clustered <- extractGeneOrder(internal_heatmap)

    # Get expression data for these genes
    cell_subset <- x[["cell_metadata"]][x[["cell_metadata"]][["cell_type"]] == cell_type, ]
    expr_matrix <- as.matrix(
        x[["expression_data"]][gene_order_clustered, cell_subset[["cell_id"]], drop = FALSE]
    )

    # Check if anomaly data is available
    has_anomaly_data <- show_anomalies && "anomaly_status" %in% names(cell_subset)

    # Create pseudo-bulk profiles for each group
    if (has_anomaly_data) {
        # Only show anomalies for query cells, treat all reference as normal
        cell_subset_modified <- cell_subset
        cell_subset_modified[["anomaly_status"]][cell_subset_modified[["dataset"]] == "Reference"] <- "Normal"

        # 3 categories: Query_Normal, Query_Anomaly, Reference_Normal
        cell_subset_modified[["group"]] <- paste(cell_subset_modified[["dataset"]],
                                                 cell_subset_modified[["anomaly_status"]],
                                                 sep = "_")
        group_order <- c("Query_Normal", "Query_Anomaly", "Reference_Normal")
        cell_subset <- cell_subset_modified
    } else {
        # 2 categories: Query, Reference
        cell_subset[["group"]] <- cell_subset[["dataset"]]
        group_order <- c("Query", "Reference")
    }

    # Calculate pseudo-bulk expression profiles
    pseudo_bulk_matrix <- matrix(0, nrow = nrow(expr_matrix), ncol = length(group_order))
    rownames(pseudo_bulk_matrix) <- rownames(expr_matrix)
    colnames(pseudo_bulk_matrix) <- group_order

    for (group in group_order) {
        group_cells <- cell_subset[cell_subset[["group"]] == group, "cell_id"]
        if (length(group_cells) > 0) {
            if (length(group_cells) == 1) {
                pseudo_bulk_matrix[, group] <- expr_matrix[, group_cells]
            } else {
                pseudo_bulk_matrix[, group] <- rowMeans(expr_matrix[, group_cells, drop = FALSE])
            }
        }
    }

    # Calculate fold changes for different comparisons
    fold_change_data <- list()

    # Reference pseudo-bulk (always Reference_Normal or Reference)
    ref_column <- if (has_anomaly_data) "Reference_Normal" else "Reference"

    for (gene in gene_order_clustered) {
        ref_expr <- pseudo_bulk_matrix[gene, ref_column]

        # Query non-anomaly vs Reference (always calculated)
        query_normal_column <- if (has_anomaly_data) "Query_Normal" else "Query"
        query_normal_expr <- pseudo_bulk_matrix[gene, query_normal_column]
        query_normal_fc <- query_normal_expr - ref_expr  # Log2 fold change

        # All Query vs Reference (calculated when anomaly data available AND show_all_query is TRUE)
        if (has_anomaly_data && show_all_query) {
            # Calculate overall query mean from both normal and anomaly
            query_normal_cells <- cell_subset[cell_subset[["group"]] == "Query_Normal", "cell_id"]
            query_anomaly_cells <- cell_subset[cell_subset[["group"]] == "Query_Anomaly", "cell_id"]
            all_query_cells <- c(query_normal_cells, query_anomaly_cells)

            if (length(all_query_cells) > 0) {
                query_all_expr <- mean(expr_matrix[gene, all_query_cells], na.rm = TRUE)
                query_all_fc <- query_all_expr - ref_expr
            } else {
                query_all_fc <- NA
            }
        } else {
            query_all_fc <- NA
        }

        # Query anomaly vs Reference (calculated when anomaly data available)
        if (has_anomaly_data) {
            query_anomaly_expr <- pseudo_bulk_matrix[gene, "Query_Anomaly"]
            query_anomaly_fc <- query_anomaly_expr - ref_expr
        } else {
            query_anomaly_fc <- NA
        }

        # Store fold change data
        gene_fc_data <- data.frame(
            gene = gene,
            query_normal_fc = query_normal_fc,
            query_all_fc = query_all_fc,
            query_anomaly_fc = query_anomaly_fc,
            stringsAsFactors = FALSE
        )

        fold_change_data[[length(fold_change_data) + 1]] <- gene_fc_data
    }

    # Combine fold change data
    fc_df <- do.call(rbind, fold_change_data)

    # Identify significant genes for annotation
    significant_genes <- c()
    if (!is.null(significance_threshold)) {
        sig_gene_list <- lapply(available_pcs, function(pc_name) {
            pc_results <- x[[pc_name]]
            if (is.null(pc_results) || nrow(pc_results) == 0) {
                return(NULL)
            }

            sig_df <- pc_results[pc_results[["cell_type"]] == cell_type &
                                     pc_results[["p_adjusted"]] < significance_threshold, ]
            return(sig_df[["gene"]])
        })
        significant_genes <- unique(unlist(sig_gene_list))
    }

    # Reshape data for plotting
    plot_data_list <- list()

    # Query non-anomaly vs Reference (always shown)
    normal_data <- data.frame(
        gene = fc_df[["gene"]],
        fold_change = fc_df[["query_normal_fc"]],
        comparison = "Query Non-Anomaly vs Reference",
        color_group = "normal",
        stringsAsFactors = FALSE
    )
    plot_data_list[[length(plot_data_list) + 1]] <- normal_data

    # All Query vs Reference (shown when anomaly data available AND show_all_query is TRUE)
    if (has_anomaly_data && show_all_query && !all(is.na(fc_df[["query_all_fc"]]))) {
        all_data <- data.frame(
            gene = fc_df[["gene"]],
            fold_change = fc_df[["query_all_fc"]],
            comparison = "All Query vs Reference",
            color_group = "all",
            stringsAsFactors = FALSE
        )
        plot_data_list[[length(plot_data_list) + 1]] <- all_data
    }

    # Query anomaly vs Reference (shown when anomaly data available)
    if (has_anomaly_data && !all(is.na(fc_df[["query_anomaly_fc"]]))) {
        anomaly_data <- data.frame(
            gene = fc_df[["gene"]],
            fold_change = fc_df[["query_anomaly_fc"]],
            comparison = "Query Anomaly vs Reference",
            color_group = "anomaly",
            stringsAsFactors = FALSE
        )
        plot_data_list[[length(plot_data_list) + 1]] <- anomaly_data
    }

    # Combine plot data
    if (length(plot_data_list) == 0) {
        warning("No fold change data available for plotting.")
        return(NULL)
    }

    plot_df <- do.call(rbind, plot_data_list)
    plot_df <- plot_df[!is.na(plot_df[["fold_change"]]), ]

    if (nrow(plot_df) == 0) {
        warning("No valid fold change data for plotting.")
        return(NULL)
    }

    # Set gene factor levels in clustered order (reversed for y-axis display)
    plot_df[["gene"]] <- factor(plot_df[["gene"]], levels = rev(gene_order_clustered))

    # Set comparison factor levels for proper ordering
    comparison_levels <- c("All Query vs Reference",
                           "Query Non-Anomaly vs Reference",
                           "Query Anomaly vs Reference")
    plot_df[["comparison"]] <- factor(plot_df[["comparison"]], levels = comparison_levels)

    # Set factor levels on color_group to control bar order
    plot_df[["color_group"]] <- factor(plot_df[["color_group"]],
                                       levels = c("anomaly", "normal", "all"))

    bar_colors <- c("normal" = "#BEBEBE",
                    "all" = "#666666",
                    "anomaly" = "#DC2F41")

    # Prepare gene labels with significance indicators
    gene_labels <- sapply(gene_order_clustered, function(g) {
        paste0(g, ifelse(g %in% significant_genes, "*", ""))
    })
    names(gene_labels) <- gene_order_clustered

    # Create the plot
    p <- ggplot2::ggplot(data = plot_df,
                         ggplot2::aes(x = .data[["fold_change"]],
                                      y = .data[["gene"]],
                                      fill = .data[["color_group"]])) +
        ggplot2::geom_col(position = ggplot2::position_dodge(width = 0.8),
                          alpha = 0.8, width = 0.4) +
        ggplot2::geom_vline(xintercept = 0, linetype = "dashed",
                            color = "grey50", alpha = 0.7) +
        ggplot2::scale_fill_manual(name = "Comparison",
                                   values = bar_colors,
                                   labels = c("normal" = "Query Non-Anomaly vs Ref",
                                              "all" = "All Query vs Ref",
                                              "anomaly" = "Query Anomaly vs Ref"),
                                   guide = ggplot2::guide_legend(reverse = TRUE)) +
        ggplot2::scale_y_discrete(labels = rev(gene_labels), name = NULL) +
        ggplot2::scale_x_continuous(name = "Log2 Fold Change (Pseudo-Bulk)",
                                    expand = ggplot2::expansion(mult = c(0.05, 0.05))) +
        ggplot2::labs(title = NULL, subtitle = NULL) +
        ggplot2::theme_bw() +
        ggplot2::theme(
            axis.text.y = ggplot2::element_text(face = "bold", size = 10),
            axis.text.x = ggplot2::element_text(size = 9),
            axis.title.x = ggplot2::element_text(face = "bold", size = 11),
            legend.position = "right",
            legend.title = ggplot2::element_text(face = "bold", size = 10),
            legend.text = ggplot2::element_text(size = 9),
            panel.grid.minor = ggplot2::element_blank(),
            panel.grid.major.y = ggplot2::element_line(color = "grey90", linewidth = 0.3),
            panel.border = ggplot2::element_rect(color = "grey70", linewidth = 0.5)
        )

    return(p)
}

#' @title Plot Boxplots for Top Loading Gene Shifts (Two-Panel Summary using ggplot2)
#'
#' @description
#' This internal helper function creates a comprehensive two-panel summary plot
#' displaying gene expression distributions and principal component loadings.
#' The visualization uses ggplot2 faceting to create side-by-side panels.
#' Optionally includes anomaly status information using visual cues.
#'
#' @details
#' This function generates a dual-panel ggplot2 visualization where the left panel
#' shows horizontal boxplots of gene expression distributions comparing Reference
#' and Query datasets, while the right panel displays PC loading values as points
#' with adjusted p-values. Gene selection is based on the union of top genes
#' across specified principal components.
#'
#' When anomaly information is available and requested, anomalous cells are
#' distinguished using dashed boxplot borders (normal cells have solid borders).
#' This approach avoids color conflicts between dataset identification (fill colors)
#' and PC identification (point colors/shapes).
#'
#' Visual encoding:
#' \itemize{
#'   \item Dataset: Fill colors (Reference = blue, Query = red)
#'   \item Anomaly status: Line types (Normal = solid, Anomaly = dashed borders)
#'   \item PC identity: Point colors and shapes in loading panel
#' }
#'
#' @param x An object of class \code{calculateTopLoadingGeneShiftsObject} containing
#'   expression data and analysis results.
#' @param cell_type A character string specifying the cell type to visualize.
#' @param available_pcs A character vector of principal components to include in analysis.
#' @param plot_by A character string indicating gene selection criterion ("top_loading" or "p_adjusted").
#' @param n_genes An integer specifying the number of top genes to display per PC.
#' @param significance_threshold A numeric value between 0 and 1 for significance annotation.
#' @param show_anomalies Logical indicating whether to show anomaly annotations using
#'   line type differences in boxplot borders.
#'
#' @keywords internal
#'
#' @return A ggplot2 object ready for display, or NULL if no genes meet selection criteria.
#'   The returned plot contains:
#'   \itemize{
#'     \item Left panel: Expression boxplots with dataset-specific fill colors
#'     \item Right panel: PC loading scatter points with PC-specific colors/shapes
#'     \item Gene labels on y-axis with significance indicators (*)
#'     \item P-value annotations on secondary y-axis
#'     \item Legend showing dataset, PC information, and anomaly status (if applicable)
#'   }
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{plotHeatmap}}, \code{\link{plot.calculateTopLoadingGeneShiftsObject}}
#'
# Helper boxplot function
plotBoxplot <- function(x, cell_type, available_pcs, plot_by,
                        n_genes, significance_threshold, show_anomalies) {

    # Validate required parameters
    if (is.null(n_genes)) {
        stop("'plot_type = \"boxplot\"' requires 'n_genes' to be set.")
    }

    # Gene selection across principal components
    gene_df_list <- lapply(available_pcs, function(pc_name) {
        pc_results <- x[[pc_name]]
        if (is.null(pc_results) || nrow(pc_results) == 0) {
            return(NULL)
        }

        pc_cell_data <- pc_results[pc_results[["cell_type"]] == cell_type, ]
        if (nrow(pc_cell_data) == 0) {
            return(NULL)
        }

        # Order genes by selection criterion
        if (plot_by == "top_loading") {
            gene_order <- order(abs(pc_cell_data[["loading"]]),
                                decreasing = TRUE)
        } else {
            gene_order <- order(pc_cell_data[["p_adjusted"]])
        }

        selected_indices <- gene_order[1:min(n_genes, nrow(pc_cell_data))]
        return(pc_cell_data[selected_indices, ])
    })

    # Combine results and remove duplicates
    final_gene_data <- do.call(rbind, gene_df_list)
    final_gene_data <- final_gene_data[!duplicated(final_gene_data[["gene"]]), ]

    if (nrow(final_gene_data) == 0) {
        warning("No genes met the selection criteria for the boxplot.")
        return(NULL)
    }

    # Order final gene set by selection criterion
    if (plot_by == "top_loading") {
        final_gene_data <- final_gene_data[
            order(abs(final_gene_data[["loading"]]), decreasing = TRUE),
        ]
    } else {
        final_gene_data <- final_gene_data[
            order(final_gene_data[["p_adjusted"]]),
        ]
    }

    # Prepare expression data for plotting
    cell_subset_meta <- x[["cell_metadata"]][
        x[["cell_metadata"]][["cell_type"]] == cell_type,
    ]
    expr_subset <- x[["expression_data"]][
        final_gene_data[["gene"]],
        cell_subset_meta[["cell_id"]],
        drop = FALSE
    ]

    # Create expression data frame with cell-level metadata
    expr_plot_list <- lapply(final_gene_data[["gene"]], function(g) {
        gene_data <- data.frame(
            gene = g,
            value = as.numeric(expr_subset[g, ]),
            dataset = cell_subset_meta[["dataset"]],
            metric = "Expression",
            pc_group = NA_character_,
            stringsAsFactors = FALSE
        )

        # Add anomaly status if available and requested
        if (show_anomalies && "anomaly_status" %in% names(cell_subset_meta)) {
            gene_data[["anomaly_status"]] <- cell_subset_meta[["anomaly_status"]]
        }

        return(gene_data)
    })
    expr_plot_df <- do.call(rbind, expr_plot_list)

    # Prepare loading data for plotting (PC-specific information)
    loading_data <- x[["gene_metadata"]][
        x[["gene_metadata"]][["gene"]] %in% final_gene_data[["gene"]] &
            paste0("PC", x[["gene_metadata"]][["pc"]]) %in% available_pcs,
    ]

    loading_plot_df <- data.frame(
        gene = loading_data[["gene"]],
        value = loading_data[["loading"]],
        dataset = factor("Reference", levels = c("Reference", "Query")),
        metric = "PC Loading",
        pc_group = paste0("PC", loading_data[["pc"]]),
        stringsAsFactors = FALSE
    )

    # Add anomaly status column to loading data (not applicable for PC loadings)
    if (show_anomalies && "anomaly_status" %in% names(cell_subset_meta)) {
        loading_plot_df[["anomaly_status"]] <- NA_character_
    }

    # Combine plotting data
    full_plot_df <- rbind(expr_plot_df, loading_plot_df)

    # Prepare gene labels with significance indicators
    is_significant <- if (!is.null(significance_threshold)) {
        final_gene_data[["p_adjusted"]] < significance_threshold
    } else {
        FALSE
    }
    gene_labels <- paste0(final_gene_data[["gene"]],
                          ifelse(is_significant, "*", ""))
    full_plot_df[["gene"]] <- factor(full_plot_df[["gene"]],
                                     levels = rev(final_gene_data[["gene"]]))

    # Format p-value labels for secondary axis
    p_value_data <- final_gene_data[, c("gene", "p_adjusted")]
    p_value_labels <- paste0("p = ",
                             sprintf("%.2g", p_value_data[["p_adjusted"]]))
    p_value_labels[p_value_data[["p_adjusted"]] < 0.001] <- "p < .001"
    names(p_value_labels) <- p_value_data[["gene"]]

    # Create PC legend labels with variance explained
    pc_legend_labels <- paste0(available_pcs, " (",
                               sprintf("%.1f%%", x[["percent_var"]][available_pcs]),
                               ")")
    names(pc_legend_labels) <- available_pcs

    # Define color and shape palettes
    dataset_colors <- c("Reference" = "#377EB8", "Query" = "#E41A1C")
    num_pcs <- length(available_pcs)
    pc_color_palette <- grDevices::hcl.colors(num_pcs, palette = "Dark 3")
    pc_shape_palette <- c(16, 17, 15, 18, 8)  # circle, triangle, square, diamond, star

    pc_colors <- pc_color_palette[1:num_pcs]
    pc_shapes <- pc_shape_palette[1:num_pcs]
    names(pc_colors) <- available_pcs
    names(pc_shapes) <- available_pcs

    # Create facet labeller for panel titles
    facet_labeller <- ggplot2::labeller(
        metric = c("Expression" = "Log-Normalized Expression",
                   "PC Loading" = "PC Loading")
    )

    # Build the base ggplot object
    p <- ggplot2::ggplot(
        data = full_plot_df,
        mapping = ggplot2::aes(y = .data[["gene"]], x = .data[["value"]])
    )

    # Add expression panel elements with conditional anomaly visualization
    if (show_anomalies && "anomaly_status" %in% names(full_plot_df)) {
        # Boxplots with anomaly-aware border styling using linetype
        p <- p + ggplot2::geom_boxplot(
            data = ~subset(., metric == "Expression"),
            ggplot2::aes(fill = .data[["dataset"]],
                         linetype = .data[["anomaly_status"]]),
            alpha = 0.8,
            outlier.size = 0.5,
            size = 0.8  # Slightly thicker borders for better linetype visibility
        ) +
            ggplot2::scale_linetype_manual(
                name = "Status",
                values = c("Normal" = "solid", "Anomaly" = "dashed"),
                na.value = "solid",
                guide = ggplot2::guide_legend(
                    order = 3,
                    override.aes = list(color = "black", fill = "grey80")
                )
            )
    } else {
        # Standard boxplots without anomaly information
        p <- p + ggplot2::geom_boxplot(
            data = ~subset(., metric == "Expression"),
            ggplot2::aes(fill = .data[["dataset"]]),
            alpha = 0.8,
            outlier.size = 0.5
        )
    }

    # Add PC loading panel elements
    p <- p +
        # Reference line at zero for PC loadings
        ggplot2::geom_vline(
            xintercept = 0,
            linetype = "dashed",
            color = "grey50",
            alpha = 0.7
        ) +
        # PC loading points with color and shape encoding
        ggplot2::geom_point(
            data = ~subset(., metric == "PC Loading"),
            ggplot2::aes(color = .data[["pc_group"]],
                         shape = .data[["pc_group"]]),
            size = 3,
            alpha = 0.8
        ) +

        # Panel configuration with custom labelling
        ggplot2::facet_grid(
            ~ metric,
            scales = "free_x",
            switch = "x",
            labeller = facet_labeller
        ) +

        # Define all aesthetic scales
        ggplot2::scale_fill_manual(
            name = "Dataset",
            values = dataset_colors,
            guide = ggplot2::guide_legend(
                order = 1,
                override.aes = list(linetype = "solid")
            )
        ) +
        ggplot2::scale_color_manual(
            name = "PC",
            values = pc_colors,
            labels = pc_legend_labels,
            guide = ggplot2::guide_legend(order = 2)
        ) +
        ggplot2::scale_shape_manual(
            name = "PC",
            values = pc_shapes,
            labels = pc_legend_labels,
            guide = ggplot2::guide_legend(order = 2)
        ) +
        ggplot2::scale_y_discrete(
            labels = rev(gene_labels),
            name = NULL
        ) +
        ggplot2::scale_x_continuous(
            sec.axis = ggplot2::dup_axis(
                name = "Adj. P-value",
                breaks = NULL,
                labels = rev(p_value_labels)
            ),
            name = NULL,
            expand = ggplot2::expansion(mult = c(0.05, 0.15))
        ) +

        # Labels and legend configuration
        ggplot2::labs(
            title = NULL,
            subtitle = NULL
        ) +
        ggplot2::guides(
            fill = ggplot2::guide_legend(order = 1),
            color = ggplot2::guide_legend(order = 2),
            shape = ggplot2::guide_legend(order = 2),
            linetype = if(show_anomalies && "anomaly_status" %in% names(full_plot_df)) {
                ggplot2::guide_legend(order = 3)
            } else {
                "none"
            }
        ) +

        # Theme customization for publication-ready appearance
        ggplot2::theme_bw() +
        ggplot2::theme(
            # Remove x-axis title (shown via facet labels)
            axis.title.x = ggplot2::element_blank(),
            # Bold gene names for emphasis
            axis.text.y.left = ggplot2::element_text(face = "bold", size = 10),
            # P-value labels styling
            axis.text.y.right = ggplot2::element_text(size = 8),
            # Legend positioning and styling
            legend.position = "right",
            legend.box = "vertical",
            legend.key.size = ggplot2::unit(0.8, "lines"),
            legend.text = ggplot2::element_text(size = 9),
            legend.title = ggplot2::element_text(size = 10, face = "bold"),
            # Panel styling
            strip.background = ggplot2::element_blank(),
            strip.placement = "outside",
            strip.text.x = ggplot2::element_text(face = "bold", size = 11),
            # Grid and panel borders
            panel.grid.minor = ggplot2::element_blank(),
            panel.border = ggplot2::element_rect(color = "grey70", linewidth = 0.5)
        )

    return(p)
}

#' @title Extract Gene Order from ComplexHeatmap Object
#'
#' @description
#' Extracts the hierarchically clustered gene order from a ComplexHeatmap Heatmap object.
#'
#' @details
#' This function initializes a ComplexHeatmap object (without displaying it) to perform
#' hierarchical clustering, then extracts the resulting gene order. This allows matching
#' gene ordering between heatmap and other plot types. The function uses a null graphics
#' device to initialize the heatmap clustering without actually drawing the plot.
#'
#' @param heatmap_object A ComplexHeatmap Heatmap object (undrawn or drawn).
#'
#' @keywords internal
#'
#' @return A character vector of gene names in the clustered order (top to bottom as they appear in heatmap).
#'
#' @author
#' Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @importFrom grDevices dev.off pdf
#'
# Function to extract genes used by heatmap plot
extractGeneOrder <- function(heatmap_object) {

    # Check if it's a ComplexHeatmap object
    if (!inherits(heatmap_object, "Heatmap")) {
        stop("Input must be a ComplexHeatmap Heatmap object")
    }

    tryCatch({
        # Open a null device to draw without displaying
        pdf(NULL)
        on.exit(dev.off(), add = TRUE)

        # Draw to initialize the heatmap clustering
        drawn_ht <- ComplexHeatmap::draw(heatmap_object, show_heatmap_legend = FALSE,
                                         show_annotation_legend = FALSE)

        # Extract row order from the drawn heatmap
        gene_order_indices <- ComplexHeatmap::row_order(drawn_ht)

        # Get the matrix and extract gene names in clustered order
        heatmap_matrix <- heatmap_object@matrix
        if (is.null(rownames(heatmap_matrix))) {
            stop("Heatmap matrix has no row names (gene names)")
        }

        # Return genes in clustered order
        clustered_genes <- rownames(heatmap_matrix)[gene_order_indices]
        return(clustered_genes)

    }, error = function(e) {
        stop("Failed to extract gene order from heatmap object. Error: ", e$message)
    })
}

#' @title Plot Reference and Query Cell Types using MDS
#'
#' @description
#' This function facilitates the assessment of similarity between reference and query datasets
#' through Multidimensional Scaling (MDS) scatter plots. It allows the visualization of cell types,
#' color-coded with user-defined custom colors, based on a dissimilarity matrix computed from a
#' user-selected gene set. If MDS coordinates are precomputed in reducedDims, they will be used;
#' otherwise, MDS will be computed from scratch.
#'
#' @details
#' The function first checks if MDS coordinates are available in the reducedDims of both datasets.
#' If precomputed MDS is found, it uses those coordinates directly for visualization.
#'
#' If MDS is not precomputed, the function selects specific subsets of cells from
#' both reference and query datasets. It then calculates Spearman correlations between gene expression profiles,
#' deriving a dissimilarity matrix. This matrix undergoes Classical Multidimensional Scaling (MDS) for
#' visualization, presenting cell types in a scatter plot, distinguished by colors defined by the user.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} containing the single-cell
#' expression data and metadata.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing the single-cell
#' expression data and metadata.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#'
#' @return A ggplot object representing the MDS scatter plot with cell type coloring.
#'
#' @references
#' \itemize{
#' \item Kruskal, J. B. (1964). "Multidimensional scaling by optimizing goodness of fit to a nonmetric hypothesis". *Psychometrika*, 29(1), 1-27. doi:10.1007/BF02289565.
#' \item Borg, I., & Groenen, P. J. F. (2005). *Modern multidimensional scaling: Theory and applications* (2nd ed.). Springer Science & Business Media. doi:10.1007/978-0-387-25975-1.
#' }
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Generate the MDS scatter plot with cell type coloring
#' mds_plot <- plotCellTypeMDS(query_data = query_data,
#'                             reference_data = reference_data,
#'                             cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid")[1:4],
#'                             query_cell_type_col = "SingleR_annotation",
#'                             ref_cell_type_col = "expert_annotation")
#' mds_plot
#'
#' @importFrom stats cmdscale cor
#' @importFrom SummarizedExperiment assay
#'
# Function to visualize cell types in MDS plot
plotCellTypeMDS <- function(query_data,
                            reference_data,
                            query_cell_type_col,
                            ref_cell_type_col,
                            cell_types = NULL,
                            assay_name = "logcounts",
                            max_cells_query = 5000,
                            max_cells_ref = 5000) {

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = 10)

    # Check if MDS is precomputed in both datasets
    query_has_mds <- "MDS" %in% reducedDimNames(query_data)
    ref_has_mds <- "MDS" %in% reducedDimNames(reference_data)
    use_precomputed_mds <- query_has_mds && ref_has_mds

    if (use_precomputed_mds) {
        message("Using precomputed MDS coordinates from reducedDims.")

        # Downsample using precomputed MDS (coordinates will be preserved through subsetting)
        query_data <- downsampleSCE(sce_object = query_data,
                                    max_cells = max_cells_query,
                                    cell_types = cell_types,
                                    cell_type_col = query_cell_type_col)
        reference_data <- downsampleSCE(sce_object = reference_data,
                                        max_cells = max_cells_ref,
                                        cell_types = cell_types,
                                        cell_type_col = ref_cell_type_col)

        # Extract precomputed MDS coordinates
        query_mds <- reducedDim(query_data, "MDS")
        ref_mds <- reducedDim(reference_data, "MDS")

        # Validate MDS dimensions (ensure at least 2D)
        if (ncol(query_mds) < 2 || ncol(ref_mds) < 2) {
            warning("Precomputed MDS has fewer than 2 dimensions. Computing MDS from scratch.")
            use_precomputed_mds <- FALSE
        }

        if (use_precomputed_mds) {
            # Create data frame from precomputed coordinates
            cmd <- data.frame(
                Dim1 = c(ref_mds[, 1], query_mds[, 1]),
                Dim2 = c(ref_mds[, 2], query_mds[, 2]),
                dataset = c(rep("Reference", nrow(ref_mds)), rep("Query", nrow(query_mds))),
                cellType = c(reference_data[[ref_cell_type_col]], query_data[[query_cell_type_col]]),
                stringsAsFactors = FALSE
            )
        }
    }

    # Fall back to computing MDS if not precomputed or validation failed
    if (!use_precomputed_mds) {
        if (query_has_mds || ref_has_mds) {
            message("MDS not available in both datasets or validation failed. Computing MDS from expression data.")
        } else {
            message("Computing MDS from expression data.")
        }

        # Downsample query and reference data (with cell type filtering)
        query_data <- downsampleSCE(sce_object = query_data,
                                    max_cells = max_cells_query,
                                    cell_types = cell_types,
                                    cell_type_col = query_cell_type_col)
        reference_data <- downsampleSCE(sce_object = reference_data,
                                        max_cells = max_cells_ref,
                                        cell_types = cell_types,
                                        cell_type_col = ref_cell_type_col)

        # Extract assay matrices
        query_assay <- as.matrix(assay(query_data, assay_name))
        ref_assay <- as.matrix(assay(reference_data, assay_name))

        # Compute correlation and dissimilarity matrix
        df <- cbind(query_assay, ref_assay)
        corMat <- cor(df, method = "spearman")
        disMat <- (1 - corMat)

        # Compute MDS coordinates
        mds_coords <- cmdscale(disMat)

        # Create data frame
        cmd <- data.frame(
            Dim1 = mds_coords[, 1],
            Dim2 = mds_coords[, 2],
            dataset = c(rep("Query", ncol(query_assay)), rep("Reference", ncol(ref_assay))),
            cellType = c(query_data[[query_cell_type_col]], reference_data[[ref_cell_type_col]]),
            stringsAsFactors = FALSE
        )
    }

    # Clean up data and create combined cell type-dataset labels
    cmd <- na.omit(cmd)
    cmd[["cell_type_dataset"]] <- paste(cmd[["dataset"]], cmd[["cellType"]], sep = " ")

    # Define the order of cell type and dataset combinations
    order_combinations <- paste(rep(c("Reference", "Query"), length(cell_types)),
                                rep(sort(cell_types), each = 2))
    cmd[["cell_type_dataset"]] <- factor(cmd[["cell_type_dataset"]],
                                         levels = order_combinations)

    # Define the colors for cell types
    cell_type_colors <- generateColors(order_combinations, paired = TRUE)

    # Create the plot
    mds_plot <- ggplot2::ggplot(cmd,
                                ggplot2::aes(
                                    x = .data[["Dim1"]],
                                    y = .data[["Dim2"]],
                                    color = .data[["cell_type_dataset"]])) +
        ggplot2::geom_point(alpha = 0.5, size = 1) +
        ggplot2::scale_color_manual(values = cell_type_colors, name = "Cell Types") +
        ggplot2::theme_bw() +
        ggplot2::theme(
            panel.grid.minor = ggplot2::element_blank(),
            panel.grid.major = ggplot2::element_line(color = "gray",
                                                     linetype = "dotted"),
            plot.title = ggplot2::element_text(size = 14, face = "bold",
                                               hjust = 0.5),
            axis.title = ggplot2::element_text(size = 12),
            axis.text = ggplot2::element_text(size = 10)) +
        ggplot2::guides(color = ggplot2::guide_legend(title = "Cell Types"))

    return(mds_plot)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

test_that("plotCellTypeMDS generates plots correctly", {
    # Generate plot using the function
    p1 <- plotCellTypeMDS(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid")[1:4]
    )
    
    # Check if output is a ggplot object
    expect_s3_class(p1, "ggplot")

})

test_that("plotCellTypeMDS handles invalid input gracefully", {
    # Test with invalid cell type column names
    expect_error(plotCellTypeMDS(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "InvalidColumn",
        ref_cell_type_col = "InvalidColumn",
        cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid")[1:4]
    ))
    
    # Test with non-existent cell types
    expect_error(plotCellTypeMDS(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_types = "NonExistentCellType"
    ))
})

#' @title Compare Gene Importance Across Datasets Using Random Forest
#'
#' @description
#' This function identifies and compares the most important genes for differentiating cell types between a query dataset
#' and a reference dataset using Random Forest.
#'
#' @details This function uses the Random Forest algorithm to calculate the importance of genes in differentiating between cell types
#' within both a reference dataset and a query dataset. The function then compares the top genes identified in both datasets to determine
#' the overlap in their importance scores.
#'
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' If NULL, then the variable importance scores are only computed for the reference data. Default is NULL.
#' @param ref_cell_type_col A character string specifying the column name in the reference dataset containing cell type annotations.
#' @param query_cell_type_col A character string specifying the column name in the query dataset containing cell type annotations.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.
#' @param n_tree An integer specifying the number of trees to grow in the Random Forest. Default is 500.
#' @param n_top An integer specifying the number of top genes to consider when comparing variable importance scores. Default is 50.
#' @param assay_name Name of the assay on which to perform computations. Defaults to \code{"logcounts"}.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#'
#' @return A list containing three elements:
#' \item{var_imp_ref}{A list of data frames containing variable importance scores for each combination of cell types in the reference
#' dataset.}
#' \item{var_imp_query}{A list of data frames containing variable importance scores for each combination of cell types in the query
#' dataset.}
#' \item{var_imp_comparison}{A named vector indicating the proportion of top genes that overlap between the reference and query
#' datasets for each combination of cell types.}
#'
#' @export
#'
#' @references
#' Breiman, L. (2001). "Random forests". *Machine Learning*, 45(1), 5-32. doi:10.1023/A:1010933404324.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Compute important variables for all pairwise cell comparisons
#' rf_output <- calculateVarImpOverlap(reference_data = reference_data,
#'                                     query_data = query_data,
#'                                     query_cell_type_col = "SingleR_annotation",
#'                                     ref_cell_type_col = "expert_annotation",
#'                                     n_tree = 500,
#'                                     n_top = 50)
#'
#' # Comparison table
#' rf_output$var_imp_comparison
#'
# RF function to compare (between datasets) which genes are best at differentiating cell types from each
calculateVarImpOverlap <- function(reference_data,
                                   query_data = NULL,
                                   ref_cell_type_col,
                                   query_cell_type_col = NULL,
                                   cell_types = NULL,
                                   n_tree = 500,
                                   n_top = 50,
                                   assay_name = "logcounts",
                                   max_cells_ref = 5000,
                                   max_cells_query = 5000){

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  assay_name = assay_name,
                  max_cells_ref = max_cells_ref,
                  max_cells_query = max_cells_query)

    # Convert cell type columns to character if needed
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)
    if(!is.null(query_data)){
        query_data <- convertColumnsToCharacter(sce_object = query_data,
                                                convert_cols = query_cell_type_col)
    }

    # Check if n_tree is a positive integer
    if (!is.numeric(n_tree) || n_tree <= 0 || n_tree != as.integer(n_tree)) {
        stop("\'n_tree\' must be a positive integer.")
    }

    # Check if n_top is a positive integer
    if (!is.numeric(n_top) || n_top <= 0 || n_top != as.integer(n_top)) {
        stop("\'n_top\' must be a positive integer.")
    }

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = NULL)

    # Downsample query and reference data (with cell type filtering)
    reference_data <- downsampleSCE(sce_object = reference_data,
                                    max_cells = max_cells_ref,
                                    cell_types = cell_types,
                                    cell_type_col = ref_cell_type_col)
    if(!is.null(query_data)){
        query_data <- downsampleSCE(sce_object = query_data,
                                    max_cells = max_cells_query,
                                    cell_types = cell_types,
                                    cell_type_col = query_cell_type_col)
    }

    # Extract assay data for reference and query datasets
    ref_x <- t(as.matrix(assay(reference_data, assay_name)))

    # Extract labels from reference and query datasets
    ref_y <- reference_data[[ref_cell_type_col]]

    # Remove NA from reference
    ref_x <- ref_x[which(!is.na(ref_y)),]
    ref_y <- na.omit(ref_y)

    # Finding importance scores for each cell type in reference dataset
    var_imp_ref <- list()
    cell_types <- unique(ref_y)
    cell_types_combn <- combn(length(cell_types), 2)
    for(combn_id in seq_len(ncol(cell_types_combn))){

        ref_x_subset <- ref_x[which(
            ref_y %in% c(cell_types[cell_types_combn[1, combn_id]],
                         cell_types[cell_types_combn[2, combn_id]])),]
        ref_y_subset <- ref_y[which(
            ref_y %in% c(cell_types[cell_types_combn[1, combn_id]],
                         cell_types[cell_types_combn[2, combn_id]]))]

        # Using x/y interface instead of formula interface
        rf_binary <- ranger::ranger(x = ref_x_subset,
                                    y = factor(ref_y_subset),
                                    num.trees = n_tree,
                                    importance = "impurity")

        var_importance_name <- paste0(cell_types[cell_types_combn[1, combn_id]],
                                      "-",
                                      cell_types[cell_types_combn[2, combn_id]])
        var_imp_ref[[var_importance_name]] <- rf_binary$variable.importance
        names(var_imp_ref[[var_importance_name]]) <- colnames(ref_x_subset)
        var_imp_ref[[var_importance_name]] <-
            data.frame(Gene = names(
                var_imp_ref[[var_importance_name]])[order(
                    var_imp_ref[[var_importance_name]],
                    decreasing = TRUE)],
                RF_Importance = var_imp_ref[[var_importance_name]][order(
                    var_imp_ref[[var_importance_name]],
                    decreasing = TRUE)])
        rownames(var_imp_ref[[var_importance_name]]) <- NULL
    }

    if(!is.null(query_data)){

        # Extract assay data for query dataset
        query_x <- t(as.matrix(assay(query_data, assay_name)))

        # Extract labels from query dataset
        query_y <- query_data[[query_cell_type_col]]

        # Remove NA from query
        query_x <- query_x[which(!is.na(query_y)),]
        query_y <- na.omit(query_y)

        # Finding importance scores for each cell type in query dataset
        cell_types <- unique(intersect(ref_y, query_y))
        var_imp_query <- list()
        for(combn_id in seq_len(ncol(cell_types_combn))){

            query_x_subset <- query_x[which(
                query_y %in% c(cell_types[cell_types_combn[1, combn_id]],
                               cell_types[cell_types_combn[2, combn_id]])),]
            query_y_subset <- query_y[which(
                query_y %in% c(cell_types[cell_types_combn[1, combn_id]],
                               cell_types[cell_types_combn[2, combn_id]]))]

            # Using x/y interface instead of formula interface
            rf_binary <- ranger::ranger(x = query_x_subset,
                                        y = factor(query_y_subset),
                                        num.trees = n_tree,
                                        importance = "impurity")

            var_importance_name <- paste0(
                cell_types[cell_types_combn[1, combn_id]], "-",
                cell_types[cell_types_combn[2, combn_id]])
            var_imp_query[[var_importance_name]] <-
                rf_binary$variable.importance
            names(var_imp_query[[var_importance_name]]) <-
                colnames(query_x_subset)
            var_imp_query[[var_importance_name]] <-
                data.frame(Gene = names(
                    var_imp_query[[var_importance_name]])[order(
                        var_imp_query[[var_importance_name]],
                        decreasing = TRUE)],
                    RF_Importance = var_imp_query[[var_importance_name]][order(
                        var_imp_query[[var_importance_name]],
                        decreasing = TRUE)])
            rownames(var_imp_query[[var_importance_name]]) <- NULL
        }

        # Comparison vector
        var_imp_comparison <- rep(NA, length(var_imp_ref))
        names(var_imp_comparison) <- names(var_imp_ref)
        for(cells in names(var_imp_comparison)){
            var_imp_comparison[cells] <- length(
                intersect(var_imp_ref[[cells]][["Gene"]][seq_len(n_top)],
                          var_imp_query[[cells]][["Gene"]][seq_len(n_top)])) /
                n_top
        }

        # Return variable importance scores for each combination of cell types in each dataset and the comparison
        return(list(var_imp_ref = var_imp_ref,
                    var_imp_query = var_imp_query,
                    var_imp_comparison = var_imp_comparison))
    } else{
        return(list(var_imp_ref = var_imp_ref))
    }
}

# Load necessary libraries
library(testthat)
library(scDiagnostics) 

# Load example datasets
data("reference_data")
data("query_data")

# Tests for the calculateVarImpOverlap function
test_that("calculateVarImpOverlap handles incorrect input arguments", {
    expect_error(calculateVarImpOverlap(reference_data = NULL, ref_cell_type_col = "expert_annotation"))
    expect_error(calculateVarImpOverlap(reference_data = reference_data, ref_cell_type_col = NULL))
    expect_error(calculateVarImpOverlap(reference_data = reference_data, ref_cell_type_col = "expert_annotation", n_tree = -1))
    expect_error(calculateVarImpOverlap(reference_data = reference_data, ref_cell_type_col = "expert_annotation", n_top = -1))
})

test_that("calculateVarImpOverlap calculates variable importance scores correctly for reference data", {
    result <- calculateVarImpOverlap(reference_data = reference_data, ref_cell_type_col = "expert_annotation", n_tree = 50, n_top = 10)
    expect_type(result, "list")
    expect_true("var_imp_ref" %in% names(result))
    expect_type(result$var_imp_ref, "list")
    for (cell_comb in names(result$var_imp_ref)) {
        expect_true(all(c("Gene", "RF_Importance") %in% colnames(result$var_imp_ref[[cell_comb]])))
    }
})

test_that("calculateVarImpOverlap calculates variable importance scores correctly for both reference and query data", {
    result <- calculateVarImpOverlap(reference_data = reference_data, query_data = query_data, ref_cell_type_col = "expert_annotation", 
                                     query_cell_type_col = "expert_annotation", n_tree = 50, n_top = 10)
    expect_type(result, "list")
    expect_true(all(c("var_imp_ref", "var_imp_query", "var_imp_comparison") %in% names(result)))
    expect_type(result$var_imp_ref, "list")
    expect_type(result$var_imp_query, "list")
    expect_type(result$var_imp_comparison, "double")
    for (cell_comb in names(result$var_imp_ref)) {
        expect_true(all(c("Gene", "RF_Importance") %in% colnames(result$var_imp_ref[[cell_comb]])))
        expect_true(all(c("Gene", "RF_Importance") %in% colnames(result$var_imp_query[[cell_comb]])))
    }
    expect_equal(length(result$var_imp_comparison), length(result$var_imp_ref))
})

test_that("calculateVarImpOverlap correctly compares top genes between reference and query data", {
    result <- calculateVarImpOverlap(reference_data = reference_data, query_data = query_data, ref_cell_type_col = "expert_annotation", 
                                     query_cell_type_col = "expert_annotation", n_tree = 50, n_top = 10)
    for (cell_comb in names(result$var_imp_ref)) {
        top_genes_ref <- result$var_imp_ref[[cell_comb]]$Gene[1:10]
        top_genes_query <- result$var_imp_query[[cell_comb]]$Gene[1:10]
        overlap <- length(intersect(top_genes_ref, top_genes_query)) / 10
        expect_equal(as.numeric(result$var_imp_comparison[cell_comb]), overlap)
    }
})

#' @title Plot Distance Density Comparison for a Specific Cell Type and Selected Cells
#'
#' @description
#' The S3 plot method plots the density functions for the reference data and the distances from a specified query cells
#' to all reference cell within a specified cell type.
#'
#' @details
#' The S3 plot method first checks if the specified cell type and cell names are present in the object. If the
#' specified cell type or cell name is not found, an error is thrown. It then extracts the distances within the reference dataset
#' and the distances from the specified query cell to the reference cells The function creates a density plot using \code{ggplot2}
#' to compare the distance distributions. The density plot will show two distributions: one for the pairwise distances within the
#' reference dataset and one for the distances from the specified query cell to each reference cell. These distributions are
#' plotted in different colors to visually assess how similar the query cell is to the reference cells of the specified cell type.
#'
#' @param x A list containing the distance data computed by \code{calculatecellDistances}.
#' @param ref_cell_type A string specifying the reference cell type.
#' @param cell_names A string specifying the query cell name for which to plot the distances.
#' @param ... Additional arguments passed to the plotting function.
#'
#' @keywords internal
#'
#' @return The S3 plot method returns a \code{ggplot} density plot comparing the reference distances and the distances from the specified cell to the reference cells.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{calculateCellDistances}}
#'
#' @rdname calculateCellDistances
#'
# Function to plot density functions for the reference data and the specified cell
plot.calculateCellDistancesObject <- function(x, ref_cell_type, cell_names, ...) {

    # Check if cell type is available
    if(length(ref_cell_type) != 1 || !(ref_cell_type %in% names(x)))
        stop("The specified \'ref_cell_type\' is not available.")

    # Filter distance data for the specified cell type
    distance_data <- x[[ref_cell_type]]

    # Check if cells are available in data for that cell type
    if(!all(cell_names %in% rownames(
        distance_data[["query_to_ref_distances"]])))
        stop("One or more specified 'cell_names' are not available for that cell type.")

    # Extract distances within the reference dataset
    ref_distances <- distance_data[["ref_distances"]]

    # Initialize an empty list to store data frames for each cell
    plot_data_list <- vector("list", length = length(cell_names))
    names(plot_data_list) <- cell_names

    # Loop through each cell to create the combined data frame
    for(s in cell_names) {
        # Extract distances for the current cell
        cell_distances <- distance_data[["query_to_ref_distances"]][s, ]

        # Create a data frame for the current cell and reference distances
        cell_data <- data.frame(cell = s, Distance = cell_distances,
                                Distance_Type = "Query")
        ref_data <- data.frame(cell = s, Distance = ref_distances,
                               Distance_Type = "Reference")

        # Combine the reference and cell data frames
        combined_data <- rbind(ref_data, cell_data)

        # Append the combined data frame to the list
        plot_data_list[[s]] <- combined_data
    }

    # Combine all data frames into one data frame
    plot_data <- do.call(rbind, plot_data_list)

    # Keep order of cell names
    plot_data[["Query"]] <- factor(plot_data[["cell"]], levels = cell_names)

    # Plot density comparison with facets for each cell
    density_plot <- ggplot2::ggplot(plot_data, ggplot2::aes(
        x = .data[["Distance"]], fill = .data[["Distance_Type"]])) +
        ggplot2::geom_density(alpha = 0.5) +
        ggplot2::labs(title = NULL,
                      x = "Distance", y = "Density",
                      fill = "Distance Type") +
        ggplot2::facet_wrap(~ .data[["Query"]], scales = "free_y") +
        ggplot2::theme_bw() +
        ggplot2::theme(
            strip.background = ggplot2::element_rect(fill = "white",
                                                     color = "black",
                                                     linewidth = 0.5),
            panel.grid.minor = ggplot2::element_blank(),
            panel.grid.major = ggplot2::element_line(
                color = "gray",
                linetype = "dotted"),
            plot.title = ggplot2::element_text(size = 14,
                                               face = "bold",
                                               hjust = 0.5),
            axis.title = ggplot2::element_text(size = 12),
            axis.text = ggplot2::element_text(size = 10),
            legend.position = "bottom")
    return(density_plot)
}

#' Scatter plot: QC stats vs Cell Type Annotation Scores
#'
#' Creates a scatter plot to visualize the relationship between QC stats (e.g., library size)
#' and cell type annotation scores for one or more cell types.
#'
#' @details This function generates a scatter plot to explore the relationship between various quality
#' control (QC) statistics, such as library size and mitochondrial percentage, and cell type
#' annotation scores. By examining these relationships, users can assess whether specific QC
#' metrics, systematically influence the confidence in cell type annotations,
#' which is essential for ensuring reliable cell type annotation.
#'
#' @param sce_object A \code{\linkS4class{SingleCellExperiment}} containing the single-cell
#' expression data and metadata.
#' @param cell_type_col The column name in the \code{colData} of \code{sce_object}
#' that contains the cell type labels.
#' @param cell_types A vector of cell type labels to plot (e.g., c("T-cell", "B-cell")).
#' Defaults to \code{NULL}, which will include all the cells.
#' @param qc_col A column name in the \code{colData} of \code{sce_object} that
#' contains the QC stats of interest.
#' @param score_col The column name in the \code{colData} of \code{sce_object} that
#' contains the cell type annotation scores.
#' @param max_cells Maximum number of cells to retain. If the object has fewer cells, it is returned unchanged.
#'                  Default is 5000.
#'
#' @return A ggplot object displaying a scatter plot of QC stats vs annotation scores,
#'         where each point represents a cell, color-coded by its cell type.
#'
#' @examples
#' # Load data
#' data("qc_data")
#'
#' # Remove cell types with very few cells
#' qc_data_subset <- qc_data[, !(qc_data$SingleR_annotation
#'                               %in% c("Chondrocytes", "DC",
#'                                      "Neurons","Platelets"))]
#'
# Create a scatter plot between library size and annotation scores
#' p1 <- plotQCvsAnnotation(sce_object = qc_data_subset,
#'                          cell_type_col = "SingleR_annotation",
#'                          cell_types = NULL,
#'                          qc_col = "total",
#'                          score_col = "annotation_scores")
#'p1 + ggplot2::xlab("Library Size")
#'
#' @export
#'
# Function to plot QC score against annotation
plotQCvsAnnotation <- function(sce_object,
                               cell_type_col,
                               cell_types = NULL,
                               qc_col,
                               score_col,
                               max_cells = 5000) {

    # Check standard input arguments
    argumentCheck(query_data = sce_object,
                  query_cell_type_col = cell_type_col,
                  max_cells_query = max_cells)

    # Convert cell type columns to character if needed
    sce_object <- convertColumnsToCharacter(sce_object = sce_object,
                                            convert_cols = cell_type_col)

    # Downsample SCE object
    sce_object <- downsampleSCE(sce_object = sce_object,
                                max_cells = max_cells,
                                cell_types = cell_types,
                                cell_type_col = cell_type_col)

    # Check if qc_col is a valid column name in sce_object
    if (!qc_col %in% names(colData(sce_object))) {
        stop("qc_col: '", qc_col, "' is not a valid column name in sce_object.")
    }

    # Check if score_col is a valid column name in sce_object
    if (!score_col %in% names(colData(sce_object))) {
        stop("score_col: '", score_col, "' is not a valid column name in sce_object.")
    }

    # Select cell types
    cell_types <- selectCellTypes(query_data = sce_object,
                                  reference_data = NULL,
                                  query_cell_type_col = cell_type_col,
                                  ref_cell_type_col = NULL,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = 10)

    # Extract QC stats, scores, and cell_typess
    qc_stats <- sce_object[[qc_col]]
    cell_types_scores <- sce_object[[score_col]]
    cell_labels <- sce_object[[cell_type_col]]

    # Combine QC stats, scores, and labels into a data frame
    data <- data.frame(QCStats = qc_stats,
                       Scores = cell_types_scores,
                       cell_type = cell_labels)

    # Define the colors for cell types
    cell_type_colors <- generateColors(sort(unique(cell_labels)),
                                       paired = FALSE)

    # Create a scatter plot with color-coded points based on cell types or labels
    qc_plot <- ggplot2::ggplot(data,
                               ggplot2::aes(x = .data[["QCStats"]],
                                            y = .data[["Scores"]],
                                            color = .data[["cell_type"]])) +
        ggplot2::geom_point() +
        ggplot2::scale_color_manual(values = cell_type_colors,
                                    name = "Cell Type") +
        ggplot2::xlab("QC stats") +
        ggplot2::ylab("Annotation Scores") +
        ggplot2::labs(color = "Cell Type") +
        ggplot2::theme_bw() +
        ggplot2::theme(
            panel.grid.minor = ggplot2::element_blank(),
            panel.grid.major = ggplot2::element_line(color = "gray",
                                                     linetype = "dotted"),
            plot.title = ggplot2::element_text(size = 14,
                                               face = "bold", hjust = 0.5),
            axis.title = ggplot2::element_text(size = 12),
            axis.text = ggplot2::element_text(size = 10))

    return(qc_plot)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("qc_data")

# Remove cell types with very few cells
qc_data_subset <- qc_data[, !(qc_data$SingleR_annotation
                              %in% c("Chondrocytes", "DC",
                                     "Neurons","Platelets"))]

test_that("plotQCvsAnnotation generates scatter plot correctly", {
    # Generate plot using the function
    p1 <- plotQCvsAnnotation(
        sce_object = qc_data_subset,
        cell_type_col = "SingleR_annotation",
        cell_types = NULL,
        qc_col = "total",
        score_col = "annotation_scores"
    )

    # Check if output is a ggplot object
    expect_s3_class(p1, "ggplot")

})

test_that("plotQCvsAnnotation handles invalid input gracefully", {
    # Test with non-existent column names in sce_object
    expect_error(plotQCvsAnnotation(
        sce_object = qc_data_subset,
        cell_type_col = "invalid_column",
        cell_types = NULL,
        qc_col = "total",
        score_col = "annotation_scores"
    ))

    # Test with non-existent qc_col
    expect_error(plotQCvsAnnotation(
        sce_object = qc_data_subset,
        cell_type_col = "SingleR_annotation",
        cell_types = NULL,
        qc_col = "invalid_column",
        score_col = "annotation_scores"
    ))

    # Test with non-existent score_col
    expect_error(plotQCvsAnnotation(
        sce_object = qc_data_subset,
        cell_type_col = "SingleR_annotation",
        cell_types = NULL,
        qc_col = "total",
        score_col = "invalid_column"
    ))
})

#' @title Visualize gene expression on a dimensional reduction plot
#'
#' @description
#' This function plots gene expression on a dimensional reduction plot using methods like t-SNE, UMAP, or PCA. Each single cell is color-coded based on the expression of a specific gene or feature.
#'
#' @param sce_object An object of class \code{\linkS4class{SingleCellExperiment}} containing log-transformed expression matrix and other metadata.
#'        It can be either a reference or query dataset.
#' @param method The reduction method to use for visualization. It should be one of the supported methods: "TSNE", "UMAP", or "PCA".
#' @param pc_subset An optional vector specifying the principal components (PCs) to include in the plot if method = "PCA".
#'        Default is 1:5.
#' @param feature A character string representing the name of the gene or feature to be visualized.
#' @param cell_type_col The column name in the \code{colData} of \code{sce_object} that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells Maximum number of cells to retain. If the object has fewer cells, it is returned unchanged.
#'                  Default is 2000.
#'
#' @importFrom SummarizedExperiment assay
#' @import SingleCellExperiment
#'
#' @return A ggplot object representing the dimensional reduction plot with gene expression.
#'
#' @export
#'
#' @examples
#' # Load data
#' data("query_data")
#'
#' # Plot gene expression on PCA plot
#' plotGeneExpressionDimred(sce_object = query_data,
#'                          cell_type_col = "SingleR_annotation",
#'                          method = "PCA",
#'                          pc_subset = 1:5,
#'                          feature = "CD8A",
#'                          cell_types = "CD4")
#'
# Function to plot the gene expression of a gene for one or more cell types
plotGeneExpressionDimred <- function(sce_object,
                                     method = c("TSNE", "UMAP", "PCA"),
                                     pc_subset = 1:5,
                                     feature,
                                     cell_type_col,
                                     cell_types = NULL,
                                     assay_name = "logcounts",
                                     max_cells = 2000) {

    # Check standard input arguments
    argumentCheck(query_data = sce_object,
                  pc_subset_query = pc_subset,
                  assay_name = assay_name,
                  query_cell_type_col = cell_type_col,
                  max_cells_query = max_cells)

    # Convert cell type columns to character if needed
    sce_object <- convertColumnsToCharacter(sce_object = sce_object,
                                            convert_cols = cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = sce_object,
                                  query_cell_type_col = cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = NULL)

    # Match arguments
    method <- match.arg(method)

    # Store PCA attributes BEFORE downsampling (if method is PCA)
    pca_percent_var <- NULL
    if (method == "PCA" && "PCA" %in% reducedDimNames(sce_object)) {
        pca_attrs <- attributes(reducedDim(sce_object, "PCA"))
        if (!is.null(pca_attrs[["percentVar"]])) {
            pca_percent_var <- pca_attrs[["percentVar"]]
        }
    }

    # Downsample SCE object
    sce_object <- downsampleSCE(sce_object = sce_object,
                                max_cells = max_cells,
                                cell_types = cell_types,
                                cell_type_col = cell_type_col)

    # Check if feature is available
    if (!feature %in% rownames(assay(sce_object, assay_name))) {
        stop("Specified feature does not exist in the expression matrix.")
    }

    # Handle cell type filtering
    filtered_to_specific_types <- FALSE
    if (!is.null(cell_type_col)) {
        # Check if cell type column exists
        if (!cell_type_col %in% colnames(colData(sce_object))) {
            stop("Specified cell_type_col does not exist in colData.")
        }

        # Get available cell types
        available_cell_types <- unique(colData(sce_object)[[cell_type_col]])

        # If specific cell types are requested, filter to those
        if (!is.null(cell_types)) {
            # Check if requested cell types exist
            missing_types <- setdiff(cell_types, available_cell_types)
            if (length(missing_types) > 0) {
                warning("The following cell types were not found: ",
                        paste(missing_types, collapse = ", "))
            }

            # Filter to cells of specified types
            cell_mask <- colData(sce_object)[[cell_type_col]] %in% cell_types
            filtered_to_specific_types <- TRUE
        } else {
            # Use all cells if no specific types requested
            cell_mask <- rep(TRUE, ncol(sce_object))
            cell_types <- available_cell_types
        }

        # Filter the object
        sce_object <- sce_object[, cell_mask]
    }

    # Extract gene expression vector (after potential filtering)
    expression <- assay(sce_object, assay_name)[feature, ]

    if(method %in% c("TSNE", "UMAP")){

        # Extract dimension reduction coordinates from SingleCellExperiment object
        reduction <- reducedDim(sce_object, method)

        # Prepare data for plotting
        df <- data.frame(Dim1 = reduction[, 1], Dim2 = reduction[, 2],
                         Expression = expression)

        # Add cell type information if available
        if (!is.null(cell_type_col)) {
            df[["CellType"]] <- colData(sce_object)[[cell_type_col]]
        }

        # Create the plot object with better color gradient
        plot_obj <- ggplot2::ggplot(df, ggplot2::aes(
            x = .data[["Dim1"]],
            y = .data[["Dim2"]])) +
            ggplot2::geom_point(ggplot2::aes(color = .data[["Expression"]]),
                                alpha = 0.7, size = 1.2) +
            ggplot2::scale_color_gradient(low = "lightgray", high = "red",
                                          name = paste(feature, "\nExpression")) +
            ggplot2::xlab("Dimension 1") +
            ggplot2::ylab("Dimension 2") +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                strip.background = ggplot2::element_rect(fill = "grey85",
                                                         color = "grey70"),
                strip.text = ggplot2::element_text(size = 10,
                                                   face = "bold",
                                                   color = "black"),
                axis.title = ggplot2::element_text(size = 12),
                axis.text = ggplot2::element_text(size = 10),
                panel.grid = ggplot2::element_blank(),
                panel.background = ggplot2::element_rect(fill = "white",
                                                         color = "black"),
                legend.position = "right",
                plot.title = ggplot2::element_text(size = 14, hjust = 0.5),
                plot.background = ggplot2::element_rect(fill = "white"))

        # Add title with cell type info if filtered
        if (!is.null(cell_type_col) && !is.null(cell_types)) {
            plot_obj <- plot_obj +
                ggplot2::ggtitle(paste0(feature, " Expression in ",
                                        paste(cell_types, collapse = ", "),
                                        " Cells"))
        }

    } else if (method == "PCA"){

        # Check input for pc_subset
        if(!all(pc_subset %in% seq_len(ncol(reducedDim(sce_object, "PCA")))))
            stop("\'pc_subset\' is out of range.")

        # PCA data
        plot_mat <- reducedDim(sce_object, "PCA")[, pc_subset]

        # Create PC column names with variance explained (always show percentages if available)
        if (!is.null(pca_percent_var) && length(pca_percent_var) >= max(pc_subset)) {
            plot_names <- paste0(
                "PC", pc_subset, " (",
                sprintf("%.1f%%", pca_percent_var[pc_subset]), ")")
        } else {
            # Fallback if percentVar is not available
            plot_names <- paste0("PC", pc_subset)
        }

        # Create a new data frame with selected PCs
        pc_df <- data.frame(matrix(0, nrow = nrow(plot_mat),
                                   ncol = length(pc_subset)))
        colnames(pc_df) <- plot_names

        for (i in 1:length(pc_subset)) {
            pc_df[, i] <- plot_mat[, i]
        }

        # Add expression data
        pc_df[["Expression"]] <- expression

        # Add cell type information if available
        if (!is.null(cell_type_col)) {
            pc_df[["CellType"]] <- colData(sce_object)[[cell_type_col]]
        }

        # Create a simple plot to extract the legend with better color gradient
        legend_plot <- ggplot2::ggplot(pc_df,
                                       ggplot2::aes(x = pc_df[,1],
                                                    y = pc_df[,2])) +
            ggplot2::geom_point(ggplot2::aes(color = .data[["Expression"]])) +
            ggplot2::scale_color_gradient(low = "gray", high = "blue",
                                          name = paste(feature, "\nExpression")) +
            ggplot2::theme(
                legend.position = "right",
                legend.box = "vertical",
                legend.key = ggplot2::element_rect(fill = "white"),
                legend.background = ggplot2::element_blank()
            )

        # Lower facet - always scatter with expression coloring
        .expressionScatterFunc <- function(data, mapping, ...) {
            ggplot2::ggplot(data = data, mapping = mapping) +
                ggplot2::geom_point(alpha = 0.7, size = 1.2,
                                    ggplot2::aes(color = .data[["Expression"]])) +
                ggplot2::scale_color_gradient(low = "gray", high = "blue") +
                ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(
                        color = "black", fill = NA, linewidth = 0.5),
                    legend.position = "none")
        }

        # Blank facet function for diagonal and upper
        .blankFunc <- function(data, mapping, ...) {
            ggplot2::ggplot() +
                ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(
                        color = "black", fill = NA, linewidth = 0.5),
                    legend.position = "none",
                    axis.text = ggplot2::element_blank(),
                    axis.ticks = ggplot2::element_blank(),
                    axis.title = ggplot2::element_blank(),
                    panel.grid = ggplot2::element_blank())
        }

        # Create pairs plot using GGally
        plot_obj <- suppressMessages(
            GGally::ggpairs(
                pc_df,
                columns = seq_len(length(pc_subset)),
                mapping = ggplot2::aes(color = .data[["Expression"]]),
                lower = list(continuous = .expressionScatterFunc),
                upper = list(continuous = .blankFunc),
                diag = list(continuous = .blankFunc),
                progress = FALSE,
                legend = GGally::grab_legend(legend_plot)
            )
        )

        # Add consistent theming
        plot_obj <- plot_obj +
            ggplot2::theme(
                strip.background = ggplot2::element_rect(
                    fill = "white", color = "black", linewidth = 0.5),
                strip.text = ggplot2::element_text(color = "black")
            )

        # Add title with cell type info if filtered
        if (!is.null(cell_type_col) && !is.null(cell_types)) {
            plot_obj <- plot_obj +
                ggplot2::labs(title = paste0(feature, " Expression in ",
                                             paste(cell_types, collapse = ", "),
                                             " Cells"))
        }
    }

    return(plot_obj)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example dataset
data("query_data")

test_that("plotGeneExpressionDimred generates plots correctly", {
    # Test PCA method - returns ggmatrix object
    p1 <- plotGeneExpressionDimred(
        sce_object = query_data,
        cell_type_col = "SingleR_annotation",
        method = "PCA",
        pc_subset = 1:5,
        feature = "VPREB3"
    )

    # Check if output is a ggmatrix object (from GGally) for PCA
    expect_s3_class(p1, "ggmatrix")

    # Test UMAP method - should return ggplot object
    # (assuming query_data has UMAP coordinates)
    if ("UMAP" %in% reducedDimNames(query_data)) {
        p2 <- plotGeneExpressionDimred(
            sce_object = query_data,
            cell_type_col = "SingleR_annotation",
            method = "UMAP",
            feature = "VPREB3"
        )
        expect_s3_class(p2, "ggplot")
    }

    # Test TSNE method - should return ggplot object
    # (assuming query_data has TSNE coordinates)
    if ("TSNE" %in% reducedDimNames(query_data)) {
        p3 <- plotGeneExpressionDimred(
            sce_object = query_data,
            cell_type_col = "SingleR_annotation",
            method = "TSNE",
            feature = "VPREB3"
        )
        expect_s3_class(p3, "ggplot")
    }
})

test_that("plotGeneExpressionDimred handles different assay names", {
    # Test with different assay name (if available)
    available_assays <- assayNames(query_data)

    if ("counts" %in% available_assays) {
        p1 <- plotGeneExpressionDimred(
            sce_object = query_data,
            cell_type_col = "SingleR_annotation",
            method = "PCA",
            pc_subset = 1:3,
            feature = "VPREB3",
            assay_name = "counts"
        )
        expect_s3_class(p1, "ggmatrix")
    }

    # Test with invalid assay name
    expect_error(
        plotGeneExpressionDimred(
            sce_object = query_data,
            cell_type_col = "SingleR_annotation",
            method = "PCA",
            pc_subset = 1:3,
            feature = "VPREB3",
            assay_name = "invalid_assay"
        )
    )
})

#' @title Plot SIR Components for Different Cell Types
#'
#' @description
#' This function plots the Sliced Inverse Regression (SIR) components for different cell types in query and reference datasets.
#'
#' @details
#' This function visualizes the SIR projections for specified cell types, providing a pairs plot of the SIR components.
#' It offers various visualization options for different facets of the plot including scatter plots, contours, ellipses, and density plots.
#' When plot_type is "loadings", it creates horizontal bar plots showing the n_top contributing variables for each SIR component.
#'
#' @param x An object of class \code{calculateSIRSpaceObject} containing SIR projections.
#' @param plot_type A character string specifying the type of plot. Either "scores" (default) for SIR projections or "loadings" for variable loadings.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are included. Only used when plot_type = "scores".
#' @param sir_subset A numeric vector specifying which SIR components to include in the plot. Default is 1:5.
#' @param lower_facet Type of plot to use for the lower panels. Either "scatter" (default), "contour", "ellipse", or "blank". Only used when plot_type = "scores".
#' @param diagonal_facet Type of plot to use for the diagonal panels. Either "ridge" (default), "density", "boxplot" or "blank". Only used when plot_type = "scores".
#' @param upper_facet Type of plot to use for the upper panels. Either "blank" (default), "scatter", "contour", or "ellipse". Only used when plot_type = "scores".
#' @param n_top A numeric value specifying the number of n_top variables (by absolute loading value) to display. Default is 10 Only used when plot_type = "loadings".
#' @param max_cells_ref Maximum number of reference cells to include in the plot. If NULL,
#' all available reference cells are plotted. Default is NULL. Only used when plot_type = "scores".
#' @param max_cells_query Maximum number of query cells to include in the plot. If NULL,
#' all available query cells are plotted. Default is NULL. Only used when plot_type = "scores".
#' @param ... Additional arguments passed to the plotting function.
#'
#' @return A ggmatrix object representing a pairs plot of specified SIR components for the given cell types and datasets when plot_type = "scores", or a ggplot object showing loadings when plot_type = "loadings".
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{calculateSIRSpace}}
#'
#' @rdname calculateSIRSpace
#'
# Function to plot data projected onto SIR space
plot.calculateSIRSpaceObject <- function(x,
                                         plot_type = c("scores", "loadings"),
                                         cell_types = NULL,
                                         sir_subset = 1:5,
                                         lower_facet = c("scatter", "contour", "ellipse", "blank"),
                                         diagonal_facet = c("ridge", "density", "boxplot", "blank"),
                                         upper_facet = c("blank", "scatter", "contour", "ellipse"),
                                         n_top = 10,
                                         max_cells_ref = NULL,
                                         max_cells_query = NULL,
                                         ...) {

    # Match arguments
    plot_type <- match.arg(plot_type)
    lower_facet <- match.arg(lower_facet)
    diagonal_facet <- match.arg(diagonal_facet)
    upper_facet <- match.arg(upper_facet)

    # Check sir_subset against available SIR components
    if (any(!(sir_subset %in% seq_len(ncol(x[["rotation_mat"]]))))) {
        stop("sir_subset contains values outside the range of available SIR components")
    }

    # Check n_top parameter for loadings plot
    if (plot_type == "loadings") {
        if (!is.numeric(n_top) || n_top <= 0 || n_top != as.integer(n_top)) {
            stop("n_top must be a positive integer.")
        }
    }

    # Validate max_cells parameters
    if (!is.null(max_cells_ref)) {
        if (!is.numeric(max_cells_ref) || max_cells_ref <= 0 || max_cells_ref != as.integer(max_cells_ref)) {
            stop("'max_cells_ref' must be a positive integer.")
        }
    }

    if (!is.null(max_cells_query)) {
        if (!is.numeric(max_cells_query) || max_cells_query <= 0 || max_cells_query != as.integer(max_cells_query)) {
            stop("'max_cells_query' must be a positive integer.")
        }
    }

    # Helper function to plot scores (original functionality with added downsampling)
    .plotScores <- function(x, cell_types,
                            sir_subset,
                            lower_facet,
                            diagonal_facet,
                            upper_facet,
                            max_cells_ref,
                            max_cells_query) {

        # Get SIR projections data
        sir_projections <- na.omit(x[["sir_projections"]])

        # Get available cell types if not specified by user
        if (is.null(cell_types)) {
            cell_types <- unique(sir_projections[["cell_type"]])
        } else {
            # Check if specified cell types exist in the data
            if (!all(cell_types %in% sir_projections[["cell_type"]])) {
                stop("One or more specified cell types not found in the data")
            }
            # Filter projections to include only specified cell types
            sir_projections <- sir_projections[sir_projections[["cell_type"]] %in%
                                                   cell_types, ]
        }

        # Separate reference and query data
        ref_data <- sir_projections[sir_projections[["dataset"]] == "Reference", ]
        query_data <- sir_projections[sir_projections[["dataset"]] == "Query", ]

        # Downsample reference data if max_cells_ref is specified
        if(!is.null(max_cells_ref)){
            # Input validation for max_cells_ref
            if (!is.numeric(max_cells_ref) || max_cells_ref <= 0 || max_cells_ref != as.integer(max_cells_ref)) {
                stop("'max_cells_ref' must be a positive integer.")
            }

            if(nrow(ref_data) > max_cells_ref){
                # Stratified sampling by cell type
                ref_data_list <- list()
                for(ct in cell_types){
                    ct_data <- ref_data[ref_data[["cell_type"]] == ct, ]
                    if(nrow(ct_data) > 0){
                        # Calculate proportional allocation
                        n_cells_ct <- min(nrow(ct_data),
                                          max(1, round(max_cells_ref * nrow(ct_data) / nrow(ref_data))))
                        if(nrow(ct_data) > n_cells_ct){
                            sampled_indices <- sample(nrow(ct_data), n_cells_ct)
                            ct_data <- ct_data[sampled_indices, ]
                        }
                        ref_data_list[[ct]] <- ct_data
                    }
                }
                ref_data <- do.call(rbind, ref_data_list)
                rownames(ref_data) <- NULL
            }
        }

        # Downsample query data if max_cells_query is specified
        if(!is.null(max_cells_query)){
            # Input validation for max_cells_query
            if (!is.numeric(max_cells_query) || max_cells_query <= 0 || max_cells_query != as.integer(max_cells_query)) {
                stop("'max_cells_query' must be a positive integer.")
            }

            if(nrow(query_data) > max_cells_query){
                # Stratified sampling by cell type
                query_data_list <- list()
                for(ct in cell_types){
                    ct_data <- query_data[query_data[["cell_type"]] == ct, ]
                    if(nrow(ct_data) > 0){
                        # Calculate proportional allocation
                        n_cells_ct <- min(nrow(ct_data),
                                          max(1, round(max_cells_query * nrow(ct_data) / nrow(query_data))))
                        if(nrow(ct_data) > n_cells_ct){
                            sampled_indices <- sample(nrow(ct_data), n_cells_ct)
                            ct_data <- ct_data[sampled_indices, ]
                        }
                        query_data_list[[ct]] <- ct_data
                    }
                }
                query_data <- do.call(rbind, query_data_list)
                rownames(query_data) <- NULL
            }
        }

        # Combine the potentially downsampled reference and query data
        sir_projections <- rbind(ref_data, query_data)

        # Create SIR column names with variance explained
        plot_names <- paste0(
            "SIR", sir_subset, " (",
            sprintf("%.1f%%", x[["percent_var"]][sir_subset]), ")")

        # Create a new data frame with selected SIR components
        sir_df <- data.frame(matrix(0, nrow = nrow(sir_projections),
                                    ncol = length(sir_subset)))
        colnames(sir_df) <- plot_names

        for (i in 1:length(sir_subset)) {
            sir_df[, i] <- sir_projections[, sir_subset[i]]
        }

        # Create a cell type dataset column for coloring
        cell_type_dataset <- paste(sir_projections[["dataset"]],
                                   sir_projections[["cell_type"]], sep = " ")

        # Define the order of cell type and dataset combinations
        order_combinations <- paste(
            rep(c("Reference", "Query"),
                length(cell_types)),
            rep(sort(cell_types), each = 2))

        cell_type_dataset <- factor(cell_type_dataset,
                                    levels = order_combinations)

        # Generate colors for cell types and datasets
        cell_type_colors <- generateColors(order_combinations, paired = TRUE)

        # Add the cell type dataset and colors columns to the SIR data frame
        sir_df[["cell_type_dataset"]] <- cell_type_dataset

        # Create a simple plot to extract the legend using GGally::grab_legend
        legend_plot <- ggplot2::ggplot(sir_df,
                                       ggplot2::aes(x = sir_df[, 1],
                                                    y = sir_df[, 2])) +
            ggplot2::geom_point(ggplot2::aes(color = .data[["cell_type_dataset"]])) +
            ggplot2::scale_color_manual(values = cell_type_colors,
                                        name = "Cell Type") +
            ggplot2::theme(
                legend.position = "right",
                legend.box = "vertical",
                legend.key = ggplot2::element_rect(fill = "white"),
                legend.background = ggplot2::element_blank()
            ) +
            ggplot2::guides(color = ggplot2::guide_legend(ncol = 1))

        # Scatterplot facet function
        .scatterFunc <- function(data, mapping, ...) {

            ggplot2::ggplot(data = data, mapping = mapping) +
                ggplot2::geom_point(alpha = 0.5, size = 1,
                                    ggplot2::aes(color = cell_type_dataset)) +
                ggplot2::scale_color_manual(values = cell_type_colors) +
                ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(
                        color = "black",
                        fill = NA,
                        linewidth = 0.5))
        }

        # Contour facet function
        .smoothContourFunc <- function(data, mapping, ...) {

            x_name <- rlang::as_name(mapping[["x"]])
            y_name <- rlang::as_name(mapping[["y"]])

            # Start with empty plot
            p <- ggplot2::ggplot() +
                ggplot2::theme_minimal()

            # Process each cell type separately
            for (ct in unique(data[["cell_type_dataset"]])) {
                subset_data <- data[data[["cell_type_dataset"]] == ct, ]

                # Skip if too few points
                if (nrow(subset_data) < 10) next

                # Use larger adjustment factor for smoother contours
                adjust_factor <- 1.5

                # Add smoother contours with fewer levels
                p <- p + ggplot2::stat_density_2d(
                    data = subset_data,
                    mapping = ggplot2::aes(
                        x = !!rlang::sym(x_name),
                        y = !!rlang::sym(y_name)
                    ),
                    contour = TRUE,
                    adjust = adjust_factor,
                    bins = 5,
                    color = cell_type_colors[which(
                        levels(data[["cell_type_dataset"]]) == ct)],
                    linewidth = 0.5,
                    na.rm = TRUE
                )
            }

            # Apply theme elements
            p + ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(
                        color = "black", fill = NA, linewidth = 0.5),
                    legend.position = "none",
                    axis.text = ggplot2::element_blank(),
                    axis.ticks = ggplot2::element_blank(),
                    axis.title = ggplot2::element_blank()
                )
        }

        # Ellipse facet function
        .robustEllipseFunc <- function(data, mapping, ...) {

            # Function to calculate robust ellipses through bootstrapping
            createEllipse <- function(d) {
                if (nrow(d) < 10) return(NULL)

                x_var <- rlang::as_name(mapping[["x"]])
                y_var <- rlang::as_name(mapping[["y"]])

                # Extract variables
                x <- d[[x_var]]
                y <- d[[y_var]]

                # Calculate robust center and covariance
                cov_mat <- cov(cbind(x, y), use = "pairwise.complete.obs")
                center <- c(mean(x, na.rm = TRUE), mean(y, na.rm = TRUE))

                # Calculate ellipse points
                theta <- seq(0, 2 * pi, length.out = 100)
                ellipse <- MASS::cov.trob(cbind(x, y))

                # Get ellipse coordinates
                ev <- eigen(ellipse[["cov"]])
                a <- sqrt(ev[["values"]][1]) * 2.45
                b <- sqrt(ev[["values"]][2]) * 2.45

                # Create ellipse coordinates
                angle <- atan2(ev[["vectors"]][2,1], ev[["vectors"]][1,1])
                ellipse_x <- center[1] + a * cos(theta) * cos(angle) -
                    b * sin(theta) * sin(angle)
                ellipse_y <- center[2] + a * cos(theta) * sin(angle) +
                    b * sin(theta) * cos(angle)

                return(data.frame(x = ellipse_x, y = ellipse_y))
            }

            p <- ggplot2::ggplot(data, mapping)

            # Split by cell type and create robust ellipses
            for (ct in unique(data[["cell_type_dataset"]])) {
                subset_data <- data[data[["cell_type_dataset"]] == ct,]
                ellipse_data <- createEllipse(subset_data)

                if (!is.null(ellipse_data)) {
                    p <- p + ggplot2::geom_path(
                        data = ellipse_data,
                        ggplot2::aes(x = .data[["x"]], y = .data[["y"]]),
                        color = cell_type_colors[
                            which(levels(data[["cell_type_dataset"]]) == ct)],
                        linewidth = 0.7
                    )
                }
            }

            p + ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(
                        color = "black", fill = NA,
                        linewidth = 0.5),
                    legend.position = "none",
                    axis.text = ggplot2::element_blank(),
                    axis.ticks = ggplot2::element_blank(),
                    axis.title = ggplot2::element_blank()
                )
        }

        # Blank facet function
        .blankFunc <- function(data, mapping, ...) {

            ggplot2::ggplot() +
                ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(
                        color = "black", fill = NA,
                        linewidth = 0.5),
                    legend.position = "none",
                    axis.text = ggplot2::element_blank(),
                    axis.ticks = ggplot2::element_blank(),
                    axis.title = ggplot2::element_blank(),
                    panel.grid = ggplot2::element_blank()
                )
        }

        # Ridge diagonal facet
        .ridgeFunc <- function(data, mapping, ...) {

            # Get current mapping info
            x_var_name <- rlang::as_name(mapping[["x"]])

            # Create a long-format data frame for ridge plot
            plot_data <- data.frame(
                value = data[[x_var_name]],
                group = data[["cell_type_dataset"]]
            )

            # Create ridge plot with ggridges
            suppressMessages({
                p <- ggplot2::ggplot(plot_data,
                                     ggplot2::aes(x = .data[["value"]],
                                                  y = .data[["group"]],
                                                  fill = .data[["group"]])) +
                    ggridges::geom_density_ridges(
                        alpha = 0.7,
                        scale = 2,
                        rel_min_height = 0.01,
                        quantile_lines = FALSE
                    ) +
                    ggplot2::scale_fill_manual(values = cell_type_colors) +
                    ggplot2::scale_y_discrete(
                        limits = rev(levels(cell_type_dataset))) +
                    ggplot2::theme_minimal() +
                    ggplot2::theme(
                        panel.border = ggplot2::element_rect(color = "black",
                                                             fill = NA,
                                                             linewidth = 0.5),
                        axis.title = ggplot2::element_blank(),
                        axis.text.y = ggplot2::element_blank(),
                        axis.ticks.y = ggplot2::element_blank(),
                        legend.position = "none"
                    )
            })

            return(p)
        }

        # Density diagonal facet
        .densityFunc <- function(data, mapping, ...) {

            # Get current mapping info
            x_var_name <- rlang::as_name(mapping[["x"]])

            ggplot2::ggplot(data = data, mapping = mapping) +
                ggplot2::geom_density(ggplot2::aes(fill = cell_type_dataset,
                                                   color = cell_type_dataset),
                                      alpha = 0.5) +
                ggplot2::scale_fill_manual(values = cell_type_colors) +
                ggplot2::scale_color_manual(values = cell_type_colors) +
                ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(color = "black",
                                                         fill = NA,
                                                         linewidth = 0.5),
                    axis.title.y = ggplot2::element_blank(),
                    axis.text.y = ggplot2::element_blank(),
                    axis.ticks.y = ggplot2::element_blank()
                )
        }

        # Boxplot diagonal facet
        .boxplotFunc <- function(data, mapping, ...) {

            # Extract the x variable name for the boxplot title
            x_name <- rlang::as_name(mapping[["x"]])

            # Create a long-format data frame for horizontal boxplot
            plot_data <- data.frame(
                value = data[[x_name]],
                group = data[["cell_type_dataset"]]
            )

            # Create horizontal boxplot
            ggplot2::ggplot(plot_data, ggplot2::aes(x = .data[["value"]],
                                                    y = .data[["group"]],
                                                    fill = .data[["group"]])) +
                ggplot2::geom_boxplot(alpha = 0.7,
                                      outlier.size = 0.5,
                                      width = 0.6) +
                ggplot2::scale_fill_manual(values = cell_type_colors) +
                ggplot2::scale_y_discrete(limits = rev(
                    levels(cell_type_dataset))) +
                ggplot2::labs(x = "", y = "") +
                ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(color = "black",
                                                         fill = NA,
                                                         linewidth = 0.5),
                    axis.title.x = ggplot2::element_blank(),
                    legend.position = "none",
                    axis.title.y = ggplot2::element_blank(),
                    axis.text.y = ggplot2::element_blank(),
                    axis.ticks.y = ggplot2::element_blank()
                )
        }

        # Select the lower facet plot type based on user input
        if (lower_facet == "scatter") {
            lower_plot <- .scatterFunc
        } else if (lower_facet == "contour") {
            lower_plot <- .smoothContourFunc
        } else if (lower_facet == "ellipse") {
            lower_plot <- .robustEllipseFunc
        } else if (lower_facet == "blank") {
            lower_plot <- .blankFunc
        }

        # Select the diagonal plot type based on user input
        if (diagonal_facet == "density") {
            diag_plot <- .densityFunc
        } else if (diagonal_facet == "boxplot") {
            diag_plot <- .boxplotFunc
        } else if (diagonal_facet == "ridge") {
            diag_plot <- .ridgeFunc
        }

        # Select the upper facet plot type based on user input
        if (upper_facet == "blank") {
            upper_plot <- .blankFunc
        } else if (upper_facet == "scatter") {
            upper_plot <- .scatterFunc
        } else if (upper_facet == "contour") {
            upper_plot <- .smoothContourFunc
        } else if (upper_facet == "ellipse") {
            upper_plot <- .robustEllipseFunc
        }

        # Create pairs plot using GGally
        plot_obj <- suppressMessages(
            GGally::ggpairs(
                sir_df,
                columns = seq_len(length(sir_subset)),
                mapping = ggplot2::aes(color = cell_type_dataset),
                lower = list(continuous = lower_plot),
                upper = list(continuous = upper_plot),
                diag = list(continuous = diag_plot),
                progress = FALSE,
                legend = GGally::grab_legend(legend_plot)
            )
        )

        # Add black frame around facet titles
        plot_obj <- plot_obj +
            ggplot2::theme(
                strip.background = ggplot2::element_rect(
                    fill = "white", color = "black", linewidth = 0.5),
                strip.text = ggplot2::element_text(color = "black")
            )

        # Return the plot
        return(plot_obj)
    }

    # Helper function to plot loadings (unchanged)
    .plotLoadings <- function(x, sir_subset, n_top) {

        # Get rotation matrix (loadings)
        rotation_mat <- x[["rotation_mat"]]

        # Check if rotation matrix exists
        if (is.null(rotation_mat)) {
            stop("No rotation matrix found in the SIR object.")
        }

        # Get variable names
        var_names <- rownames(rotation_mat)
        if (is.null(var_names)) {
            var_names <- paste0("Var_", seq_len(nrow(rotation_mat)))
        }

        # Create a combined data frame for all SIR components
        all_loadings_list <- list()

        for (i in seq_along(sir_subset)) {
            sir_comp <- sir_subset[i]

            # Get loadings for this component
            loadings <- rotation_mat[, sir_comp]

            # Create data frame with variable names and loadings
            loading_df <- data.frame(
                variable = var_names,
                loading = loadings,
                abs_loading = abs(loadings),
                sir_component = paste0("SIR", sir_comp, " (",
                                       sprintf("%.1f%%",
                                               x[["percent_var"]][sir_comp]),
                                       ")"),
                sir_order = i,
                stringsAsFactors = FALSE
            )

            # Sort by absolute loading and take n_top variables
            loading_df <- loading_df[order(loading_df[["abs_loading"]],
                                           decreasing = TRUE), ]
            loading_df <- loading_df[seq_len(min(n_top,
                                                 nrow(loading_df))), ]

            # Order variables for plotting (highest absolute loading at n_top of each facet)
            loading_df <- loading_df[order(loading_df[["abs_loading"]]), ]
            loading_df[["variable_facet"]] <- factor(loading_df[["variable"]],
                                                     levels = loading_df[["variable"]])

            # Create color based on sign of loading
            loading_df[["color"]] <- ifelse(loading_df[["loading"]] >= 0,
                                            "Positive", "Negative")

            all_loadings_list[[i]] <- loading_df
        }

        # Combine all data frames
        all_loadings <- do.call(rbind, all_loadings_list)

        # Create factor for SIR components to control facet order
        sir_component_levels <- unique(all_loadings[["sir_component"]][
            order(all_loadings[["sir_order"]])])
        all_loadings[["sir_component"]] <- factor(
            all_loadings[["sir_component"]],
            levels = sir_component_levels)

        # Create the faceted plot
        p <- ggplot2::ggplot(all_loadings, ggplot2::aes(
            x = .data[["loading"]],
            y = .data[["variable_facet"]],
            fill = .data[["color"]])) +
            ggplot2::geom_col(alpha = 0.8, width = 0.6) +
            ggplot2::facet_wrap(~ sir_component, scales = "free_y",
                                ncol = length(sir_subset)) +
            ggplot2::scale_fill_manual(values = c("Positive" = "#2166ac",
                                                  "Negative" = "#d6604d"),
                                       name = "Loading") +
            ggplot2::labs(
                title = "SIR Component Loadings",
                x = "Loading Value",
                y = "Variables"
            ) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.grid.major.y = ggplot2::element_blank(),
                panel.grid.minor = ggplot2::element_blank(),
                panel.border = ggplot2::element_rect(color = "black",
                                                     fill = NA,
                                                     linewidth = 0.5),
                strip.background = ggplot2::element_rect(fill = "white",
                                                         color = "black",
                                                         linewidth = 0.5),
                strip.text = ggplot2::element_text(size = 10,
                                                   face = "bold"),
                plot.title = ggplot2::element_text(hjust = 0.5,
                                                   size = 14,
                                                   face = "bold"),
                axis.text.y = ggplot2::element_text(size = 8),
                legend.position = "bottom"
            ) +
            ggplot2::geom_vline(xintercept = 0,
                                linetype = "dashed",
                                color = "gray50",
                                alpha = 0.7)

        return(p)
    }


    # Branch based on plot type
    if (plot_type == "scores") {
        return(.plotScores(x,
                           cell_types,
                           sir_subset,
                           lower_facet,
                           diagonal_facet,
                           upper_facet,
                           max_cells_ref,
                           max_cells_query))
    } else {
        return(.plotLoadings(x,
                             sir_subset,
                             n_top))
    }
}

#' @title Ridgeline Plot of Pairwise Distance Analysis
#'
#' @description
#' This function calculates pairwise distances or correlations between query and reference cells of a specified cell type
#' and visualizes the results using ridgeline plots, displaying the density distribution for each comparison.
#'
#' @details
#' Designed for \code{\linkS4class{SingleCellExperiment}} objects, this function subsets data for the specified cell type,
#' computes pairwise distances or correlations, and visualizes these measurements through ridgeline plots.
#' The plots help evaluate the consistency and differentiation of annotated cell types within single-cell datasets.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} containing the single-cell
#' expression data and metadata.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing the single-cell
#' expression data and metadata.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param cell_type The cell type for which distances or correlations are calculated.
#' @param pc_subset A numeric vector specifying which principal components to use in the analysis. Default is 1:5.
#' If set to \code{NULL}, the assay data is used directly for computations without dimensionality reduction.
#' @param distance_metric The distance metric to use for calculating pairwise distances, such as euclidean, manhattan, etc.
#'                        Set to "correlation" to calculate correlation coefficients.
#' @param correlation_method The correlation method to use when \code{distance_metric} is "correlation".
#'                           Possible values are "pearson" and "spearman".
#' @param bandwidth Numeric value controlling the smoothness of the density estimate; smaller values create more detailed curves. Default is 0.25.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#'
#' @return A ggplot2 object showing ridgeline plots of calculated distances or correlations.
#'
#' @export
#'
#' @seealso \code{\link{calculateWassersteinDistance}}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Example usage of the function
#' plotPairwiseDistancesDensity(query_data = query_data,
#'                              reference_data = reference_data,
#'                              query_cell_type_col = "SingleR_annotation",
#'                              ref_cell_type_col = "expert_annotation",
#'                              cell_type = "CD8",
#'                              pc_subset = 1:5,
#'                              distance_metric = "euclidean",
#'                              correlation_method = "pearson")
#'
#' @importFrom stats cor dist
#' @import SingleCellExperiment
#' @importFrom SummarizedExperiment assay
#' @importFrom ggridges geom_density_ridges
#'
# Plot density curves for distances between cells
plotPairwiseDistancesDensity <- function(
        query_data,
        reference_data,
        query_cell_type_col,
        ref_cell_type_col,
        cell_type,
        pc_subset = 1:5,
        distance_metric = c("correlation", "euclidean"),
        correlation_method = c("spearman", "pearson"),
        bandwidth = 0.25,
        assay_name = "logcounts",
        max_cells_query = 5000,
        max_cells_ref = 5000) {

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Match argument for distance_metric
    distance_metric <- match.arg(distance_metric)

    # Match argument for correlation method
    correlation_method <- match.arg(correlation_method)

    # Check if bandwidth is valid
    if (!is.numeric(bandwidth) || length(bandwidth) != 1 || bandwidth <= 0 || bandwidth > 2) {
        stop("\'bandwidth\' must be a single positive numeric value between 0 and 2.")
    }

    # Check if cell_type only contains one cell type
    if(length(cell_type) > 1){
        stop("\'cell_type\' should only contain one cell type.")
    }

    # Select cell types
    cell_type <- selectCellTypes(query_data = query_data,
                                 reference_data = reference_data,
                                 query_cell_type_col = query_cell_type_col,
                                 ref_cell_type_col = ref_cell_type_col,
                                 cell_types = cell_type,
                                 dual_only = TRUE,
                                 n_cell_types = NULL)

    # Convert to matrix and potentially applied PCA dimensionality reduction
    if(!is.null(pc_subset)){
        # Project query data onto PCA space of reference data
        pca_output <- projectPCA(query_data = query_data,
                                 reference_data = reference_data,
                                 query_cell_type_col = query_cell_type_col,
                                 ref_cell_type_col = ref_cell_type_col,
                                 cell_types = cell_type,
                                 pc_subset = pc_subset,
                                 assay_name = assay_name,
                                 max_cells_ref = max_cells_ref,
                                 max_cells_query = max_cells_query)
        ref_mat <- pca_output[pca_output[["dataset"]] == "Reference",
                              paste0("PC", pc_subset)]
        query_mat <- pca_output[pca_output[["dataset"]] == "Query",
                                paste0("PC", pc_subset)]
    } else{
        # Subset query and reference data to the specified cell type
        reference_data <- downsampleSCE(sce_object = reference_data,
                                        max_cells = max_cells_ref,
                                        cell_types =  cell_type,
                                        cell_type_col = ref_cell_type_col)
        ref_mat <- t(as.matrix(assay(reference_data, assay_name)))
        query_data <- downsampleSCE(sce_object = query_data,
                                    max_cells = max_cells_ref,
                                    cell_types =  cell_type,
                                    cell_type_col = query_cell_type_col)
        query_mat <- t(as.matrix(assay(query_data, assay_name)))
    }

    # Combine query and reference matrices
    combined_mat <- rbind(query_mat, ref_mat)

    # Calculate pairwise distances or correlations for all comparisons
    if (distance_metric == "correlation") {
        dist_matrix <- cor(t(combined_mat), method = correlation_method)
    } else {
        dist_matrix <- as.matrix(dist(combined_mat, method = distance_metric))
    }

    # Extract the distances or correlations for the different pairwise comparisons
    num_query_cells <- nrow(query_mat)
    num_ref_cells <- nrow(ref_mat)
    query_indices <- seq_len(num_query_cells)
    ref_indices <- seq_len(num_ref_cells) + num_query_cells
    dist_query_query <- dist_matrix[query_indices, query_indices]
    dist_ref_ref <- dist_matrix[ref_indices, ref_indices]
    dist_query_ref <- dist_matrix[query_indices, ref_indices]

    # Create data frame for plotting
    ref_ref_name <- paste("Ref", cell_type,
                          "vs", "Ref", cell_type)
    query_query_name <- paste("Query", cell_type,
                              "vs", "Query", cell_type)
    query_ref_name <- paste("Query", cell_type,
                            "vs", "Ref", cell_type)
    dist_df <- data.frame(
        Comparison = factor(c(
            rep(ref_ref_name, length(dist_ref_ref)),
            rep(query_query_name, length(dist_query_query)),
            rep(query_ref_name, length(dist_query_ref))),
            levels = c(ref_ref_name,
                       query_query_name,
                       query_ref_name)),
        Distance = c(as.vector(dist_ref_ref),
                     as.vector(dist_query_query),
                     as.vector(dist_query_ref))
    )

    # Plot ridge line plots
    ridgeline_plot <- ggplot2::ggplot(
        dist_df, ggplot2::aes(
            x = .data[["Distance"]], y = .data[["Comparison"]],
            fill = .data[["Comparison"]])) +
        ggridges::geom_density_ridges(scale = 1.5,
                                      bandwidth = bandwidth) +
        ggplot2::scale_fill_manual(values = setNames(
            c("#5DADE2", "#BA55D3", "#D9534F"),
            c(ref_ref_name, query_query_name, query_ref_name)
        )) +
        ggplot2::labs(
            x = ifelse(distance_metric == "correlation",
                       ifelse(correlation_method == "spearman",
                              "Spearman Correlation",
                              "Pearson Correlation"),
                       "Distance"),
            y = NULL,
            title = "Pairwise Distance Analysis") +
        ggplot2::theme_bw() +
        ggplot2::theme(
            axis.text.y = ggplot2::element_blank(),
            axis.ticks.y = ggplot2::element_blank(),
            panel.grid.minor = ggplot2::element_blank(),
            panel.grid.major = ggplot2::element_line(color = "gray",
                                                     linetype = "dotted"),
            plot.title = ggplot2::element_text(size = 14,
                                               face = "bold",
                                               hjust = 0.5),
            axis.title = ggplot2::element_text(size = 12),
            axis.text = ggplot2::element_text(size = 10),
            legend.position = "bottom") +
        ggplot2::guides(fill = ggplot2::guide_legend(reverse = TRUE,
                                                     nrow = 3))

    # Adjust x-axis if correlation is used
    if(distance_metric == "correlation"){
        ridgeline_plot <- ridgeline_plot + ggplot2::xlim(-1, 1)
    }

    # Return the plot
    return(ridgeline_plot)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

test_that("plotPairwiseDistancesDensity generates density plot correctly", {
    # Generate plot using the function
    p1 <- plotPairwiseDistancesDensity(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_type = "CD8",
        pc_subset = 1:5,
        distance_metric = "euclidean",
        correlation_method = "pearson"
    )

    # Check if output is a ggplot object
    expect_s3_class(p1, "ggplot")

})

test_that("plotPairwiseDistancesDensity handles invalid input gracefully", {
    # Test with non-existent column names in query_data or reference_data
    expect_error(plotPairwiseDistancesDensity(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "invalid_column",
        ref_cell_type_col = "expert_annotation",
        cell_type = "CD8",
        pc_subset = 1:5,
        distance_metric = "euclidean",
        correlation_method = "pearson"
    ))

    # Test with non-existent cell types
    expect_error(plotPairwiseDistancesDensity(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_type = "invalid_cell_type",
        pc_subset = 1:5,
        distance_metric = "euclidean",
        correlation_method = "pearson"
    ))

    # Test with invalid distance_metric
    expect_error(plotPairwiseDistancesDensity(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_type = "CD8",
        pc_subset = 1:5,
        distance_metric = "invalid_metric",
        correlation_method = "pearson"
    ))

    # Test with invalid correlation_method
    expect_error(plotPairwiseDistancesDensity(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_type = "CD8",
        pc_subset = 1:5,
        distance_metric = "correlation",
        correlation_method = "invalid_method"
    ))
})

#' @title Plot the output of the `calculateAveragePairwiseCorrelation` function
#'
#' @description
#' The S3 plot method takes the output of the `calculateAveragePairwiseCorrelation` function,
#' which should be a matrix of pairwise correlations, and plots it as a heatmap.
#'
#' @details
#' The S3 plot method converts the correlation matrix into a dataframe, creates a heatmap using ggplot2,
#' and customizes the appearance of the heatmap with updated colors and improved aesthetics.
#'
#' @param x Output matrix from `calculateAveragePairwiseCorrelation` function.
#' @param ... Additional arguments to be passed to the plotting function.
#'
#' @keywords internal
#'
#' @return The S3 plot method returns a \code{ggplot} object representing the heatmap plot.
#'
#' @export
#'
#' @seealso \code{\link{calculateAveragePairwiseCorrelation}}
#'
#' @rdname calculateAveragePairwiseCorrelation
#'
# Function to plot the output of the calculateAveragePairwiseCorrelation function
plot.calculateAveragePairwiseCorrelationObject <- function(x, ...){

    # Convert matrix to dataframe
    cor_df <- as.data.frame(as.table(x))
    cor_df[["Var1"]] <- factor(cor_df[["Var1"]], levels = rownames(x))
    cor_df[["Var2"]] <- factor(cor_df[["Var2"]], levels = rev(colnames(x)))

    # Create the heatmap with updated colors and improved aesthetics
    heatmap_plot <- ggplot2::ggplot(cor_df, ggplot2::aes(x = .data[["Var2"]],
                                                         y = .data[["Var1"]])) +
        ggplot2::geom_tile(ggplot2::aes(fill = .data[["Freq"]]), color = "white") +
        ggplot2::geom_text(ggplot2::aes(label = round(.data[["Freq"]], 2)),
                           color = "black", size = 3, family = "sans") +
        ggplot2::scale_fill_gradient2(low = "blue", mid = "white", high = "red",
                                      midpoint = 0,
                                      limits = c(min(cor_df[["Freq"]]),
                                                 max(cor_df[["Freq"]])),
                                      name = "Correlation",
                                      breaks = seq(-1, 1, by = 0.2)) +
        ggplot2::labs(title = "Correlation Heatmap", x = "", y = "") +
        ggplot2::theme_minimal() +
        ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45,
                                                           hjust = 1),
                       axis.text.y = ggplot2::element_text(family = "sans"),
                       plot.title = ggplot2::element_text(face = "bold"),
                       legend.position = "right",
                       legend.title = ggplot2::element_text(face = "italic"))

    # Print the plot
    return(heatmap_plot)
}

#' @title Function to Calculate Bhattacharyya Coefficients and Hellinger Distances
#'
#' @description
#' This function computes Bhattacharyya coefficients and Hellinger distances to quantify the similarity of density
#' distributions between query cells and reference data for each cell type.
#'
#' @details
#' This function first computes distance data using the \code{calculateCellDistances} function, which calculates
#' pairwise distances between cells within the reference data and between query cells and reference cells in the PCA space.
#' Bhattacharyya coefficients and Hellinger distances are calculated to quantify the similarity of density distributions between query
#' cells and reference data for each cell type. Bhattacharyya coefficient measures the similarity of two probability distributions,
#' while Hellinger distance measures the distance between two probability distributions.
#'
#' Bhattacharyya coefficients range between 0 and 1. A value closer to 1 indicates higher similarity between distributions, while a value
#' closer to 0 indicates lower similarity
#'
#' Hellinger distances range between 0 and 1. A value closer to 0 indicates higher similarity between distributions, while a value
#' closer to 1 indicates lower similarity.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data}
#' that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data}
#' that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are
#'                   included.
#' @param cell_names_query A character vector specifying the names of the query cells for which to compute distance measures.
#' @param pc_subset A numeric vector specifying which principal components to include in the plot. Default is 1:5.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#'
#' @return A list containing distance data for each cell type. Each entry in the list contains:
#' \describe{
#'   \item{ref_distances}{A vector of all pairwise distances within the reference subset for the cell type.}
#'   \item{query_to_ref_distances}{A matrix of distances from each query cell to all reference cells for the cell type.}
#' }
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Plot the PC data
#' distance_data <- calculateCellDistances(query_data = query_data,
#'                                         reference_data = reference_data,
#'                                         query_cell_type_col = "SingleR_annotation",
#'                                         ref_cell_type_col = "expert_annotation",
#'                                         pc_subset = 1:10)
#'
#' # Identify outliers for CD4
#' cd4_anomalies <- detectAnomaly(reference_data = reference_data,
#'                                query_data = query_data,
#'                                query_cell_type_col = "SingleR_annotation",
#'                                ref_cell_type_col = "expert_annotation",
#'                                pc_subset = 1:10,
#'                                n_tree = 500,
#'                                anomaly_threshold = 0.5)
#' cd4_top6_anomalies <- names(sort(cd4_anomalies$CD4$query_anomaly_scores, decreasing = TRUE)[1:6])
#'
#' # Get overlap measures
#' overlap_measures <- calculateCellDistancesSimilarity(query_data = query_data,
#'                                                      reference_data = reference_data,
#'                                                      cell_names_query = cd4_top6_anomalies,
#'                                                      query_cell_type_col = "SingleR_annotation",
#'                                                      ref_cell_type_col = "expert_annotation",
#'                                                      pc_subset = 1:10)
#' overlap_measures
#'
# Function to compute Bhattacharyya coefficients and Hellinger distances
calculateCellDistancesSimilarity <- function(query_data,
                                             reference_data,
                                             query_cell_type_col,
                                             ref_cell_type_col,
                                             cell_types = NULL,
                                             cell_names_query,
                                             pc_subset = 1:5,
                                             assay_name = "logcounts",
                                             max_cells_ref = 5000) {

    # Format the query cell names - remove "Query_" prefix if present
    cell_names_query <- gsub("^Query_", "", cell_names_query)

    # Check standard input arguments (now with proper cell_types)
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  cell_names_query = cell_names_query,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = NULL)

    # Subset query data first
    query_data_subset <- query_data[, cell_names_query, drop = FALSE]

    # Compute distance data
    distance_data <- calculateCellDistances(
        query_data = query_data_subset,
        reference_data = reference_data,
        query_cell_type_col = query_cell_type_col,
        ref_cell_type_col = ref_cell_type_col,
        cell_types = cell_types,
        pc_subset = pc_subset,
        assay_name = assay_name,
        max_cells_ref = max_cells_ref)

    # Update the names of the query cells
    cell_names_query <- paste0("Query_", cell_names_query)

    # Initialize empty lists to store results
    bhattacharyya_list <- hellinger_list <-
        vector("list", length = length(distance_data))
    names(bhattacharyya_list) <- names(hellinger_list) <- names(distance_data)

    # Iterate over each cell type
    for (cell_type in names(distance_data)) {

        # Extract distances within the reference dataset for the current cell type
        ref_distances <- distance_data[[cell_type]][["ref_distances"]]

        # Compute density of reference distances
        ref_density <- density(ref_distances)

        # Initialize an empty vector to store overlap measures for the current cell type
        bhattacharyya_coef <- numeric(length(cell_names_query))
        hellinger_dist <- numeric(length(cell_names_query))

        # Iterate over each cell
        for (i in seq_len(length(cell_names_query))) {

            # Extract distances from the current cell to reference cells
            cell_distances <-
                distance_data[[cell_type]][["query_to_ref_distances"]][cell_names_query[i], , drop = FALSE]

            # Compute density of cell distances
            cell_density <- density(cell_distances)

            # Create a common grid for evaluating densities
            common_grid <- seq(min(min(ref_density[["x"]]),
                                   min(cell_density[["x"]]), 0),
                               max(max(ref_density[["x"]]),
                                   max(cell_density[["x"]])),
                               length.out = 1000)

            # Interpolate densities onto the common grid
            ref_density_interp <- approxfun(ref_density[["x"]],
                                            ref_density[["y"]])(common_grid)
            ref_density_interp[is.na(ref_density_interp)] <- 0
            cell_density_interp <- approxfun(cell_density[["x"]],
                                             cell_density[["y"]])(common_grid)
            cell_density_interp[is.na(cell_density_interp)] <- 0

            # Compute and store Bhattacharyya coefficient/Hellinger distance
            bhattacharyya_coef[i] <- sum(
                sqrt(ref_density_interp * cell_density_interp) *
                    mean(diff(common_grid)))
            hellinger_dist[i] <- sqrt(
                1 - sum(sqrt(ref_density_interp * cell_density_interp)) *
                    mean(diff(common_grid)))
        }

        # Store overlap measures for the current cell type
        bhattacharyya_list[[cell_type]] <- bhattacharyya_coef
        hellinger_list[[cell_type]] <- hellinger_dist
    }

    # Return list with overlap measures
    bhattacharyya_coef <- data.frame(Cell = cell_names_query, bhattacharyya_list)
    hellinger_dist <- data.frame(Cell = cell_names_query, hellinger_list)
    return(list(bhattacharyya_coef = bhattacharyya_coef,
                hellinger_dist = hellinger_dist))
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Test cases
test_that("calculateCellDistancesSimilarity computes similarity measures correctly", {

    # Test with typical input
    overlap_measures <- calculateCellDistancesSimilarity(query_data = query_data,
                                                         reference_data = reference_data,
                                                         query_cell_type_col = "SingleR_annotation",
                                                         ref_cell_type_col = "expert_annotation",
                                                         cell_names = colnames(query_data)[1:5],
                                                         pc_subset = 1:10)

    # Check if the result is a list
    expect_true(is.list(overlap_measures))

    # Check if the list contains entries for Bhattacharyya coefficients and Hellinger distances
    expect_named(overlap_measures, c("bhattacharyya_coef", "hellinger_dist"))

})

test_that("calculateCellDistancesSimilarity handles NULL cell_types gracefully", {

    # Test with NULL cell_types
    overlap_measures <- calculateCellDistancesSimilarity(query_data = query_data,
                                                         reference_data = reference_data,
                                                         query_cell_type_col = "SingleR_annotation",
                                                         ref_cell_type_col = "expert_annotation",
                                                         cell_names = colnames(query_data)[1:5],
                                                         pc_subset = 1:10)

    # Check if the result is a list
    expect_true(is.list(overlap_measures))

    # Check if the list contains entries for Bhattacharyya coefficients and Hellinger distances
    expect_named(overlap_measures, c("bhattacharyya_coef", "hellinger_dist"))

    # Check structure of Bhattacharyya coefficients and Hellinger distances entries
    expect_equal(ncol(overlap_measures$bhattacharyya_coef) - 1, length(unique(c(reference_data$expert_annotation, query_data$SingleR_annotation))))
    expect_equal(ncol(overlap_measures$hellinger_dist) - 1, length(unique(c(reference_data$expert_annotation, query_data$SingleR_annotation))))

})

test_that("calculateCellDistancesSimilarity handles single cell names gracefully", {

    # Test with a single cell name
    overlap_measures <- calculateCellDistancesSimilarity(query_data = query_data,
                                                         reference_data = reference_data,
                                                         query_cell_type_col = "SingleR_annotation",
                                                         ref_cell_type_col = "expert_annotation",
                                                         cell_names = colnames(query_data)[1:5],
                                                         pc_subset = 1:10)

    # Check if the result is a list
    expect_true(is.list(overlap_measures))

    # Check if the list contains entries for Bhattacharyya coefficients and Hellinger distances
    expect_named(overlap_measures, c("bhattacharyya_coef", "hellinger_dist"))

})

#' @title Plot Wasserstein Distance Distributions for Multiple Cell Types
#'
#' @description
#' This function generates ridge plots comparing reference-reference
#' and reference-query Wasserstein distance distributions for each cell type.
#'
#' @details
#' The function creates faceted ridge plots showing two clearly separated density curves for each
#' cell type: one for the reference-reference distribution (null) and one for the
#' reference-query distribution.
#'
#' @param x A list object containing the Wasserstein distance results from the \code{calculateWassersteinDistance} function.
#' @param cell_types A character vector specifying which cell types to plot. If NULL, all cell types are plotted.
#' @param bandwidth A numeric value specifying the bandwidth for density estimation. If NULL (default), automatic bandwidth selection is used.
#' @param ... Additional arguments for future extensions.
#'
#' @keywords internal
#'
#' @return A ggplot2 object representing the comparison of Wasserstein distance distributions.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{calculateWassersteinDistance}}
#'
#' @rdname calculateWassersteinDistance
#'
# Function to plot densities of Wasserstein distances
plot.calculateWassersteinDistanceObject <- function(
        x,
        cell_types = NULL,
        bandwidth = NULL,
        ...){

    # Determine which cell types to plot
    if (is.null(cell_types)) {
        cell_types <- x[["cell_types"]]
    } else {
        # Check that requested cell types are available
        missing_types <- setdiff(cell_types, x[["cell_types"]])
        if (length(missing_types) > 0) {
            warning(paste("Cell types not found in data:",
                          paste(missing_types, collapse = ", ")))
        }
        cell_types <- intersect(cell_types, x[["cell_types"]])
    }

    if (length(cell_types) == 0) {
        stop("No valid cell types to plot.")
    }

    # Prepare data for plotting
    plot_data <- data.frame()

    for (cell_type in cell_types) {
        # Reference-reference distribution
        ref_ref_data <- data.frame(
            wasserstein_dist = x[["ref_ref_dist"]][[cell_type]],
            distribution = "Reference-Reference",
            cell_type = cell_type
        )

        # Reference-query distribution
        ref_query_data <- data.frame(
            wasserstein_dist = x[["ref_query_dist"]][[cell_type]],
            distribution = "Reference-Query",
            cell_type = cell_type
        )

        plot_data <- rbind(plot_data, ref_ref_data, ref_query_data)
    }

    # Set factor levels to control order (Reference-Reference at bottom, Reference-Query at top)
    plot_data[["distribution"]] <- factor(plot_data[["distribution"]],
                                     levels = c("Reference-Query",
                                                "Reference-Reference"))

    # Set factor levels for cell_type to preserve the order specified in cell_types parameter
    plot_data[["cell_type"]] <- factor(plot_data[["cell_type"]], levels = cell_types)

    # Setting up color data
    plot_data[["cell_type_distribution"]] <-
        paste(plot_data[["cell_type"]],
              plot_data[["distribution"]])
    plot_data[["cell_type_distribution"]] <-
        factor(plot_data[["cell_type_distribution"]],
               levels = unique(plot_data[["cell_type_distribution"]]))
    cell_type_distribution_colors <-
        generateColors(levels(plot_data[["cell_type_distribution"]]),
                       paired = TRUE)

    # Build geom_density_ridges arguments conditionally
    ridge_args <- list(
        alpha = 0.7,
        scale = 1.25,
        rel_min_height = 0.01,
        jittered_points = FALSE,
        position = "identity"
    )

    # Add bandwidth if provided
    if (!is.null(bandwidth)) {
        ridge_args[["bandwidth"]] <- bandwidth
    }

    # Create the ridge plot
    ridge_plot <- ggplot2::ggplot(plot_data,
                                  ggplot2::aes(
                                      x = .data[["wasserstein_dist"]],
                                      y = .data[["distribution"]],
                                      fill = .data[["cell_type_distribution"]])) +
        do.call(ggridges::geom_density_ridges, ridge_args) +
        ggplot2::facet_wrap(~ .data[["cell_type"]], scales = "free_x", ncol = 2) +
        ggplot2::scale_fill_manual(
            name = "Distribution",
            values = cell_type_distribution_colors,
            labels = names(cell_type_distribution_colors)
        ) +
        ggplot2::scale_y_discrete(
            expand = ggplot2::expansion(mult = c(0.02, 0.02))
        ) +
        ggplot2::labs(
            title = "Comparison of Wasserstein Distance Distributions by Cell Type",
            x = "Wasserstein Distance",
            y = "Distribution"
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
            panel.border = ggplot2::element_rect(color = "black",
                                                 fill = NA,
                                                 linewidth = 0.5),
            axis.title.x = ggplot2::element_text(size = 12),
            legend.position = "bottom",
            axis.title.y = ggplot2::element_blank(),
            axis.text.y = ggplot2::element_text(size = 10),
            axis.ticks.y = ggplot2::element_blank(),
            plot.title = ggplot2::element_text(size = 14,
                                               face = "bold", hjust = 0.5),
            strip.text = ggplot2::element_text(size = 11,
                                               face = "bold"),
            strip.background = ggplot2::element_rect(fill = "white",
                                                     color = "black",
                                                     linewidth = 0.5),
            panel.grid.minor = ggplot2::element_blank(),
            panel.grid.major.x = ggplot2::element_line(color = "gray",
                                                       linetype = "dotted"),
            panel.grid.major.y = ggplot2::element_blank()
        ) +
        ggplot2::guides(
            fill = ggplot2::guide_legend(override.aes = list(alpha = 0.8))
        )

    return(ridge_plot)
}

#' @title Calculate Categorization Entropy
#'
#' @description
#' This function takes a matrix of category scores (cell type by
#' cells) and calculates the entropy of the category probabilities for each
#' cell. This gives a sense of how confident the cell type assignments are.
#' High entropy = lots of plausible category assignments = low confidence. Low
#' entropy = only one or two plausible categories = high confidence. This is
#' confidence in the vernacular sense, not in the "confidence interval"
#' statistical sense. Also note that the entropy tells you nothing about
#' whether or not the assignments are correct -- see the other functionality
#' in the package for that. This functionality can be used for assessing how
#' comparatively confident different sets of assignments are (given that the
#' number of categories is the same).
#'
#' @details
#' The function checks if X is already on the probability scale.
#' Otherwise, it applies softmax columnwise.
#'
#' You can think about entropies on a scale from 0 to a maximum that depends
#' on the number of categories. This is the function for entropy (minus input
#' checking): \code{entropy(p) = -sum(p*log(p))} . If that input vector p is a
#' uniform distribution over the \code{length(p)} categories, the entropy will
#' be a high as possible.
#
#' @param X A matrix of category scores.
#' @param inverseNormalTransformationform If TRUE, apply inverse normal transformation to X. Default is FALSE.
#' @param verbose If TRUE, display messages about the calculations. Default is TRUE.
#' @param plot If TRUE, plot a histogram of the entropies. Default is TRUE.
#'
#' @returns A vector of entropy values for each column in X.
#'
#' @export
#'
#' @author Andrew Ghazi, \email{andrew_ghazi@hms.harvard.edu}
#'
#' @examples
#' # Simulate 500 cells with scores on 4 possible cell types
#' X <- rnorm(500 * 4) |> matrix(nrow = 4)
#' X[1, 1:250] <- X[1, 1:250] + 5 # Make the first category highly scored in the first 250 cells
#'
#' # The function will issue a message about softmaxing the scores, and the entropy histogram will be
#' # bimodal since we made half of the cells clearly category 1 while the other half are roughly even.
#' entropy_scores <- calculateCategorizationEntropy(X)
#'
# Function to calculate categorization entropy
calculateCategorizationEntropy <- function(X,
                                           inverseNormalTransformationform = FALSE,
                                           plot = TRUE,
                                           verbose = TRUE) {

    if (inverseNormalTransformationform) {

        # https://cran.r-project.org/web/packages/RNOmni/vignettes/RNOmni.html#inverse-normal-transformation
        if (verbose) message("Applying global inverse normal transformation.")
        # You can't do the INT column-wise (by cell) because it will set a
        # constant "range" to the probabilities, eliminating the differences in
        # confidence across methods we're trying to quantify.

        # You can't do the INT row-wise (by cell-type) because even though
        # different cell types exhibit different marginal distributions of
        # scores (in SingleR at least), doing the transformation row-wise would
        # eliminate any differences in which cell types are "hard to predict".
        # You don't want a score of .5 for cytotoxic T cells (hard to predict
        # type) to overwhelm a score of .62 from erythroid type 2 (easy to
        # predict), even though the first would be extraordinary within its cell
        # type and the latter unexceptional within its cell type.

        X <- inverseNormalTransformation(X)
    }

    colSumsX <- colSums(X)

    X_is_probabilities <- all(X >= 0 & X <= 1) &
        all((colSumsX - 1) <= 1e-8)

    if (!X_is_probabilities) {
        if (verbose) message("X doesn't seem to be on the probability scale, applying column-wise softmax.")
        expX <- exp(X)

        X <- sweep(expX, MARGIN = 2, STATS = colSums(expX), FUN = "/")
    }

    ncat <- nrow(X)

    max_ent <- calculateEntropy(rep(1 / ncat, ncat))

    if (verbose) {
        message(
            "Max possible entropy given ", ncat, " categories: ",
            round(max_ent,
                  digits = 2
            )
        )
    }

    entropies <- apply(X, 2, calculateEntropy)

    if (plot) {
        p <- data.frame(entropies = entropies) |>
            ggplot2::ggplot(ggplot2::aes(entropies)) +
            ggplot2::geom_histogram(
                color = "black", fill = "white",
                bins = 30,
                boundary = 0
            ) +
            ggplot2::theme_bw()
        p
    }

    return(entropies)
}

#' @title Calculate Entropy
#'
#' @description
#' This function calculates the entropy of a probability distribution.
#'
#' @details
#' The entropy is calculated using the formula \eqn{-\sum p \log(p)}, where the sum is over all non-zero elements of \code{p}.
#'
#' @param p A numeric vector representing a probability distribution. The elements should sum to 1.
#'
#' @keywords internal
#'
#' @return A numeric value representing the entropy of the probability distribution.
#'
# Function to calculate entropy
calculateEntropy <- function(p) {

    nonzeros <- p != 0

    -sum(p[nonzeros] * log(p[nonzeros]))
}

#' @title Number of Elements
#'
#' @description
#' This function returns the number of elements in a matrix or vector.
#'
#' @details If \code{X} is a matrix, the function returns the product of its dimensions. If \code{X} is a vector, the function returns
#' its length.
#'
#' @param X A matrix or vector.
#'
#' @keywords internal
#'
#' @return An integer representing the number of elements in \code{X}.
#'
#' @author Andrew Ghazi, \email{andrew_ghazi@hms.harvard.edu}
#'
# Function to return the number of elements
nElements <- function(X){

    return(ifelse(is.matrix(X), prod(dim(X)), length(X)))
}

#' @title Inverse Normal Transformation
#'
#' @description
#' This function performs an inverse normal transformation on a matrix or vector.
#'
#' @details
#' The function ranks the elements of \code{X} and then applies the inverse normal transformation using the formula \eqn{qnorm((rank - constant) / (n - 2 * constant + 1))}.
#'
#' @param X A numeric matrix or vector.
#' @param constant A numeric value used in the transformation. Default is \code{3 / 8}.
#'
#' @keywords internal
#'
#' @author Andrew Ghazi, \email{andrew_ghazi@hms.harvard.edu}
#'
#' @return A matrix or vector with the same dimensions as \code{X}, with values transformed using the inverse normal transformation.
#'
# Function to compute the inverse normal rank transformation
inverseNormalTransformation <- function(X,
                                        constant = 3 / 8) {

    n <- nElements(X)

    rankX <- rank(X)

    intX <- qnorm((rankX - constant) / (n - 2 * constant + 1))

    if (is.matrix(X)) {
        intX <- matrix(intX, nrow = nrow(X))
    }

    return(intX)
}

#' @title Plot Regression Results on Principal Components
#'
#' @description
#' The S3 plot method generates plots to visualize the results of regression analyses
#' performed on principal components (PCs) against cell types, datasets, or their interactions.
#'
#' @param x An object of class \code{regressPCObject} containing the output of the \code{regressPC} function.
#' @param plot_type Type of plot to generate. Available options:
#'   "r_squared", "variance_contribution", "coefficient_heatmap".
#' @param alpha Significance threshold for p-values. Default is 0.05.
#' @param coefficients_include Character vector specifying which coefficient types to include
#'   in the coefficient heatmap. Options are \code{c("cell_type", "batch", "interaction")}.
#'   Default is \code{NULL}, which includes all available coefficient types. Only applies
#'   to \code{plot_type = "coefficient_heatmap"}.
#' @param ... Additional arguments to be passed to the plotting functions.
#'
#' @return The S3 plot method returns a \code{ggplot} object representing the specified plot type.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{regressPC}}
#'
#' @rdname regressPC
#'
#' @importFrom stats aggregate formula complete.cases as.formula
#'
# Plot the results of regressPC
plot.regressPCObject <- function(x,
                                 plot_type = c("r_squared", "variance_contribution",
                                               "coefficient_heatmap"),
                                 alpha = 0.05,
                                 coefficients_include = NULL,
                                 ...) {

    # Match plot_type argument
    plot_type <- match.arg(plot_type)

    # Validate coefficients_include parameter
    if (!is.null(coefficients_include)) {
        # Check if coefficients_include is valid
        valid_coefficients <- c("cell_type", "batch", "interaction")
        if (!all(coefficients_include %in% valid_coefficients)) {
            stop("coefficients_include must be one or more of: ",
                 paste(valid_coefficients, collapse = ", "))
        }

        # Check if at least one coefficient type is specified
        if (length(coefficients_include) == 0) {
            stop("coefficients_include must contain at least one coefficient type")
        }

        # Check if the specified coefficients are available in the object
        available_coefficients <- .getAvailableCoefficients(x)
        unavailable <- coefficients_include[!coefficients_include %in% available_coefficients]
        if (length(unavailable) > 0) {
            stop("The following coefficient types are not available in the regression object: ",
                 paste(unavailable, collapse = ", "),
                 ". Available types: ", paste(available_coefficients, collapse = ", "))
        }
    }

    # Generate plot based on type
    switch(plot_type,
           "r_squared" = plotRSquared(x, ...),
           "variance_contribution" = plotVarianceContribution(x, ...),
           "coefficient_heatmap" = plotCoefficientHeatmap(x, alpha, coefficients_include, ...)
    )
}

#' @title Determine Available Coefficient Types
#'
#' @description
#' Helper function to determine which coefficient types are available in a regressPCObject
#' based on the model type and data structure.
#'
#' @param x An object of class \code{regressPCObject} containing regression results
#'   from \code{regressPC} function.
#'
#' @keywords internal
#'
#' @return A character vector of available coefficient types.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
.getAvailableCoefficients <- function(x) {
    available <- c("cell_type")  # cell_type is always available

    # Check for batch coefficients
    if (x[["indep_var"]] %in% c("cell_type_batch_interaction", "cell_type_dataset_interaction")) {
        available <- c(available, "batch")
    }

    # Check for interaction coefficients
    if (x[["indep_var"]] %in% c("cell_type_batch_interaction", "cell_type_dataset_interaction")) {
        available <- c(available, "interaction")
    }

    return(available)
}

# [plotRSquared and plotVarianceContribution functions remain unchanged]
#' @title Generate R-squared Bar Plot with Component Breakdown
#'
#' @description
#' Creates a bar plot visualization of R-squared values for each principal component,
#' with optional stacked bars showing the contribution of individual model components
#' (cell type, batch/dataset, and interaction effects) when component decomposition
#' is available.
#'
#' @details
#' This function generates either a simple bar plot or a stacked bar plot depending
#' on the availability of component-wise R-squared decomposition in the input object.
#' When component breakdown is available, the bars are stacked to show:
#' \itemize{
#'   \item Cell type main effect (blue)
#'   \item Batch or dataset main effect (orange)
#'   \item Interaction effect (green)
#' }
#'
#' Principal component labels include the percentage of total variance explained by
#' each PC. The total R-squared value is displayed above each bar. For query-only
#' analyses, the function uses query PCA variance; for query+reference analyses,
#' it uses reference PCA variance.
#'
#' @param x An object of class \code{regressPCObject} containing regression results
#'   from \code{regressPC} function.
#' @param ... Additional arguments passed to the plotting function (currently unused).
#'
#' @keywords internal
#'
#' @return A \code{ggplot2} object representing the R-squared bar plot with optional
#'   component breakdown.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @importFrom ggplot2 ggplot aes geom_col geom_text scale_fill_manual labs theme_minimal theme element_text element_blank element_line margin ylim
#' @importFrom tools toTitleCase
#'
# Helper function: R-squared barplot with component breakdown
plotRSquared <- function(x, ...) {

    # Helper function to get model text
    .getModelText <- function(indep_var) {
        switch(indep_var,
               "cell_type" = "PC ~ Cell Type",
               "cell_type_batch_interaction" = "PC ~ Cell Type * Batch",
               "cell_type_dataset_interaction" = "PC ~ Cell Type * Dataset",
               "Unknown Model")
    }

    # Determine which PCA variance to use based on data type
    if (!is.null(x[["reference_pca_var"]])) {
        # Reference + Query data
        pca_var <- x[["reference_pca_var"]]
    } else {
        # Query only data
        pca_var <- x[["query_pca_var"]]
    }

    # Get PC indices for variance lookup
    pc_indices <- as.numeric(gsub("PC", "", names(x[["r_squared"]])))

    # Create PC names with variance labels
    pc_names <- names(x[["r_squared"]])
    pc_labels <- paste0(pc_names, "\n(", sprintf("%.1f%%", pca_var[pc_indices]), ")")

    # Check if component breakdown is available
    if (!is.null(x[["r_squared_components"]]) && length(x[["r_squared_components"]]) > 1) {
        # Create stacked bar plot with components

        # Prepare component data
        component_data <- list()
        for (component_name in names(x[["r_squared_components"]])) {
            component_data[[component_name]] <- data.frame(
                PC = factor(pc_labels, levels = pc_labels),
                Component = component_name,
                R_squared = x[["r_squared_components"]][[component_name]],
                stringsAsFactors = FALSE
            )
        }

        # Combine all components
        plot_data <- do.call(rbind, component_data)

        # Create proper component labels and colors
        plot_data[["Component"]] <- factor(plot_data[["Component"]],
                                           levels = names(x[["r_squared_components"]]))

        # Set up colors based on available components
        component_colors <- c(
            "cell_type" = "#1f77b4",     # Blue
            "batch" = "#ff7f0e",         # Orange
            "dataset" = "#ff7f0e",       # Orange (same as batch)
            "interaction" = "#2ca02c"    # Green
        )

        # Get colors for available components
        available_colors <- component_colors[names(x[["r_squared_components"]])]

        # Create component labels for legend
        component_labels <- names(x[["r_squared_components"]])
        component_labels <- gsub("_", " ", component_labels)
        component_labels <- tools::toTitleCase(component_labels)

        # Add total R² labels on top of bars
        total_r_squared <- aggregate(R_squared ~ PC, data = plot_data, FUN = sum)

        # Create plot
        p <- ggplot2::ggplot(plot_data, ggplot2::aes(x = .data[["PC"]],
                                                     y = .data[["R_squared"]],
                                                     fill = .data[["Component"]])) +
            ggplot2::geom_col(alpha = 0.8, width = 0.6) +
            ggplot2::geom_text(data = total_r_squared,
                               ggplot2::aes(x = .data[["PC"]],
                                            y = .data[["R_squared"]],
                                            label = sprintf("%.3f", .data[["R_squared"]])),
                               vjust = -0.5, size = 3.5, fontface = "bold",
                               inherit.aes = FALSE) +
            ggplot2::scale_fill_manual(values = available_colors,
                                       labels = component_labels,
                                       name = "Component") +
            ggplot2::labs(
                title = "R-squared Values by Principal Component",
                subtitle = paste("Model:", .getModelText(x[["indep_var"]])),
                x = NULL,
                y = expression(R^2)
            ) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                plot.title = ggplot2::element_text(size = 16, face = "bold", margin = ggplot2::margin(b = 10)),
                plot.subtitle = ggplot2::element_text(size = 12, color = "gray40", margin = ggplot2::margin(b = 20)),
                axis.text.x = ggplot2::element_text(size = 10, color = "gray20"),
                axis.text.y = ggplot2::element_text(size = 10, color = "gray20"),
                axis.title.y = ggplot2::element_text(size = 12, face = "bold", color = "gray20"),
                legend.position = "right",
                legend.title = ggplot2::element_text(size = 11, face = "bold"),
                panel.grid.major.x = ggplot2::element_blank(),
                panel.grid.minor = ggplot2::element_blank(),
                panel.grid.major.y = ggplot2::element_line(color = "gray90", linewidth = 0.5),
                plot.margin = ggplot2::margin(20, 20, 20, 20)
            ) +
            ggplot2::ylim(0, max(total_r_squared[["R_squared"]]) * 1.1)

    } else {
        # Fallback to simple bar plot (original behavior)
        plot_data <- data.frame(
            PC = factor(pc_labels, levels = pc_labels),
            R_squared = x[["r_squared"]]
        )

        p <- ggplot2::ggplot(plot_data, ggplot2::aes(x = .data[["PC"]], y = .data[["R_squared"]])) +
            ggplot2::geom_col(fill = "steelblue", alpha = 0.8, width = 0.6) +
            ggplot2::geom_text(ggplot2::aes(label = sprintf("%.3f", .data[["R_squared"]])),
                               vjust = -0.5, size = 3.5, fontface = "bold") +
            ggplot2::labs(
                title = "R-squared Values by Principal Component",
                subtitle = paste("Model:", .getModelText(x[["indep_var"]])),
                x = NULL,
                y = expression(R^2)
            ) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                plot.title = ggplot2::element_text(size = 16, face = "bold", margin = ggplot2::margin(b = 10)),
                plot.subtitle = ggplot2::element_text(size = 12, color = "gray40", margin = ggplot2::margin(b = 20)),
                axis.text.x = ggplot2::element_text(size = 10, color = "gray20"),
                axis.text.y = ggplot2::element_text(size = 10, color = "gray20"),
                axis.title.y = ggplot2::element_text(size = 12, face = "bold", color = "gray20"),
                panel.grid.major.x = ggplot2::element_blank(),
                panel.grid.minor = ggplot2::element_blank(),
                panel.grid.major.y = ggplot2::element_line(color = "gray90", linewidth = 0.5),
                plot.margin = ggplot2::margin(20, 20, 20, 20)
            ) +
            ggplot2::ylim(0, max(plot_data[["R_squared"]]) * 1.1)
    }

    return(p)
}

#' @title Generate Variance Contribution Bar Plot with Component Breakdown
#'
#' @description
#' Creates a bar plot visualization of variance contributions for each principal component,
#' showing how much total dataset variance is explained by the regression model.
#' When available, displays stacked bars showing individual component contributions
#' (cell type, batch/dataset, and interaction effects).
#'
#' @details
#' This function visualizes the variance contribution of each principal component,
#' calculated as the product of PC variance and R-squared values. The variance
#' contribution represents the percentage of total dataset variance explained by
#' the regression model for each PC.
#'
#' When component decomposition is available, the function creates stacked bars with:
#' \itemize{
#'   \item Cell type main effect contribution (blue)
#'   \item Batch or dataset main effect contribution (orange)
#'   \item Interaction effect contribution (green)
#' }
#'
#' The plot subtitle includes the total variance explained across all principal
#' components. PC labels show the individual variance percentage for each component.
#' The function automatically selects appropriate PCA variance values based on
#' analysis type (query-only vs. query+reference).
#'
#' @param x An object of class \code{regressPCObject} containing regression results
#'   from \code{regressPC} function.
#' @param ... Additional arguments passed to the plotting function (currently unused).
#'
#' @keywords internal
#'
#' @return A \code{ggplot2} object representing the variance contribution bar plot
#'   with optional component breakdown.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @importFrom ggplot2 ggplot aes geom_col geom_text scale_fill_manual labs theme_minimal theme element_text element_blank element_line margin ylim
#' @importFrom tools toTitleCase
#'
# Helper function: Variance contribution barplot with component breakdown
plotVarianceContribution <- function(x, ...) {

    # Helper function to get model text
    .getModelText <- function(indep_var) {
        switch(indep_var,
               "cell_type" = "PC ~ Cell Type",
               "cell_type_batch_interaction" = "PC ~ Cell Type * Batch",
               "cell_type_dataset_interaction" = "PC ~ Cell Type * Dataset",
               "Unknown Model")
    }

    # Determine which PCA variance to use based on data type
    if (!is.null(x[["reference_pca_var"]])) {
        # Reference + Query data
        pca_var <- x[["reference_pca_var"]]
    } else {
        # Query only data
        pca_var <- x[["query_pca_var"]]
    }

    # Get PC indices for variance lookup
    pc_indices <- as.numeric(gsub("PC", "", names(x[["var_contributions"]])))

    # Create PC names with variance labels
    pc_names <- names(x[["var_contributions"]])
    pc_labels <- paste0(pc_names, "\n(", sprintf("%.1f%%", pca_var[pc_indices]), ")")

    # Check if component breakdown is available
    if (!is.null(x[["var_contributions_components"]]) && length(x[["var_contributions_components"]]) > 1) {
        # Create stacked bar plot with components

        # Prepare component data
        component_data <- list()
        for (component_name in names(x[["var_contributions_components"]])) {
            component_data[[component_name]] <- data.frame(
                PC = factor(pc_labels, levels = pc_labels),
                Component = component_name,
                Variance_Contribution = x[["var_contributions_components"]][[component_name]],
                stringsAsFactors = FALSE
            )
        }

        # Combine all components
        plot_data <- do.call(rbind, component_data)

        # Create proper component labels and colors
        plot_data[["Component"]] <- factor(plot_data[["Component"]],
                                           levels = names(x[["var_contributions_components"]]))

        # Set up colors based on available components
        component_colors <- c(
            "cell_type" = "#1f77b4",     # Blue
            "batch" = "#ff7f0e",         # Orange
            "dataset" = "#ff7f0e",       # Orange (same as batch)
            "interaction" = "#2ca02c"    # Green
        )

        # Get colors for available components
        available_colors <- component_colors[names(x[["var_contributions_components"]])]

        # Create component labels for legend
        component_labels <- names(x[["var_contributions_components"]])
        component_labels <- gsub("_", " ", component_labels)
        component_labels <- tools::toTitleCase(component_labels)

        # Add total variance contribution labels on top of bars
        total_var_contrib <- aggregate(Variance_Contribution ~ PC, data = plot_data, FUN = sum)

        # Create plot
        p <- ggplot2::ggplot(plot_data, ggplot2::aes(x = .data[["PC"]],
                                                     y = .data[["Variance_Contribution"]],
                                                     fill = .data[["Component"]])) +
            ggplot2::geom_col(alpha = 0.8, width = 0.6) +
            ggplot2::geom_text(data = total_var_contrib,
                               ggplot2::aes(x = .data[["PC"]],
                                            y = .data[["Variance_Contribution"]],
                                            label = sprintf("%.2f%%", .data[["Variance_Contribution"]])),
                               vjust = -0.5, size = 3.5, fontface = "bold",
                               inherit.aes = FALSE) +
            ggplot2::scale_fill_manual(values = available_colors,
                                       labels = component_labels,
                                       name = "Component") +
            ggplot2::labs(
                title = "Variance Contribution by Principal Component",
                subtitle = paste("Total variance explained:",
                                 sprintf("%.2f%%", x[["total_variance_explained"]]),
                                 "| Model:", .getModelText(x[["indep_var"]])),
                x = NULL,
                y = "Variance Contribution (%)"
            ) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                plot.title = ggplot2::element_text(size = 16, face = "bold", margin = ggplot2::margin(b = 10)),
                plot.subtitle = ggplot2::element_text(size = 12, color = "gray40", margin = ggplot2::margin(b = 20)),
                axis.text.x = ggplot2::element_text(size = 10, color = "gray20"),
                axis.text.y = ggplot2::element_text(size = 10, color = "gray20"),
                axis.title.y = ggplot2::element_text(size = 12, face = "bold", color = "gray20"),
                legend.position = "right",
                legend.title = ggplot2::element_text(size = 11, face = "bold"),
                panel.grid.major.x = ggplot2::element_blank(),
                panel.grid.minor = ggplot2::element_blank(),
                panel.grid.major.y = ggplot2::element_line(color = "gray90", linewidth = 0.5),
                plot.margin = ggplot2::margin(20, 20, 20, 20)
            ) +
            ggplot2::ylim(0, max(total_var_contrib[["Variance_Contribution"]]) * 1.1)

    } else {
        # Fallback to simple bar plot (original behavior)
        plot_data <- data.frame(
            PC = factor(pc_labels, levels = pc_labels),
            Variance_Contribution = x[["var_contributions"]],
            PCA_Variance = pca_var[pc_indices],
            R_squared = x[["r_squared"]]
        )

        p <- ggplot2::ggplot(plot_data, ggplot2::aes(x = .data[["PC"]], y = .data[["Variance_Contribution"]])) +
            ggplot2::geom_col(fill = "darkgreen", alpha = 0.8, width = 0.6) +
            ggplot2::geom_text(ggplot2::aes(label = sprintf("%.2f%%", .data[["Variance_Contribution"]])),
                               vjust = -0.5, size = 3.5, fontface = "bold") +
            ggplot2::labs(
                title = "Variance Contribution by Principal Component",
                subtitle = paste("Total variance explained:",
                                 sprintf("%.2f%%", x[["total_variance_explained"]]),
                                 "| Model:", .getModelText(x[["indep_var"]])),
                x = NULL,
                y = "Variance Contribution (%)"
            ) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                plot.title = ggplot2::element_text(size = 16, face = "bold", margin = ggplot2::margin(b = 10)),
                plot.subtitle = ggplot2::element_text(size = 12, color = "gray40", margin = ggplot2::margin(b = 20)),
                axis.text.x = ggplot2::element_text(size = 10, color = "gray20"),
                axis.text.y = ggplot2::element_text(size = 10, color = "gray20"),
                axis.title.y = ggplot2::element_text(size = 12, face = "bold", color = "gray20"),
                panel.grid.major.x = ggplot2::element_blank(),
                panel.grid.minor = ggplot2::element_blank(),
                panel.grid.major.y = ggplot2::element_line(color = "gray90", linewidth = 0.5),
                plot.margin = ggplot2::margin(20, 20, 20, 20)
            ) +
            ggplot2::ylim(0, max(plot_data[["Variance_Contribution"]]) * 1.1)
    }

    return(p)
}

#' @title Generate Regression Coefficients Heatmap
#'
#' @description
#' Creates a heatmap visualization of regression coefficients from principal component
#' regression analysis, organized by coefficient type (cell type, batch, interaction)
#' and annotated with significance indicators.
#'
#' @details
#' This function generates a comprehensive heatmap showing regression coefficients
#' for each principal component and model term. The visualization includes:
#' \itemize{
#'   \item Color-coded coefficient values (blue = negative, red = positive)
#'   \item Significance indicators (asterisks) for adjusted p-values below threshold
#'   \item Faceted organization by coefficient category (Cell Type, Batch, Interaction)
#'   \item Clean term labels with proper formatting and reference category information
#' }
#'
#' The function handles different model types automatically:
#' \itemize{
#'   \item Simple cell type models: \code{PC ~ cell_type}
#'   \item Batch interaction models: \code{PC ~ cell_type * batch}
#'   \item Dataset interaction models: \code{PC ~ cell_type * dataset}
#' }
#'
#' Term labels are cleaned and formatted for better readability, with batch/dataset
#' terms converted to consistent "Query Batch" terminology. The plot includes
#' comprehensive subtitle information showing model specification, significance
#' threshold, and reference categories.
#'
#' @param x An object of class \code{regressPCObject} containing regression results
#'   from \code{regressPC} function.
#' @param alpha Numeric value specifying the significance threshold for p-value
#'   adjustment. Default is 0.05.
#' @param coefficients_include Character vector specifying which coefficient types to include.
#'   Options are \code{c("cell_type", "batch", "interaction")}. Default is \code{NULL},
#'   which includes all available coefficient types.
#' @param ... Additional arguments passed to the plotting function (currently unused).
#'
#' @keywords internal
#'
#' @return A \code{ggplot2} object representing the regression coefficients heatmap
#'   with faceted organization and significance annotations.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @importFrom ggplot2 ggplot aes geom_tile geom_point scale_fill_gradient2 facet_grid labs theme_minimal theme element_text element_blank element_rect unit scale_y_discrete
#'
# Helper function: Coefficient heatmap
plotCoefficientHeatmap <- function(x, alpha = 0.05, coefficients_include = NULL, ...) {

    # Determine which PCA variance to use based on data type
    if (!is.null(x[["reference_pca_var"]])) {
        # Reference + Query data
        pca_var <- x[["reference_pca_var"]]
    } else {
        # Query only data
        pca_var <- x[["query_pca_var"]]
    }

    # Get PC indices for variance lookup
    pc_names <- names(x[["regression_summaries"]])
    pc_indices <- as.numeric(gsub("PC", "", pc_names))

    # Create PC labels with variance percentages
    pc_labels <- paste0(pc_names, "\n(", sprintf("%.1f%%", pca_var[pc_indices]), ")")

    # Extract coefficient data
    coef_list <- lapply(pc_names, function(pc) {
        coef_data <- x[["regression_summaries"]][[pc]][["coefficients"]]
        coef_data[["PC"]] <- pc
        coef_data[["Term"]] <- rownames(coef_data)
        return(coef_data)
    })

    # Combine into single dataframe
    plot_data <- do.call(rbind, coef_list)

    # Remove intercept for cleaner visualization
    plot_data <- plot_data[!grepl("Intercept", plot_data[["Term"]]), ]

    # Remove any rows with NA values
    plot_data <- plot_data[complete.cases(plot_data), ]

    # Add significance indicator
    plot_data[["Significant"]] <- plot_data[["p.adjusted"]] < alpha

    # Initialize category and clean term names
    plot_data[["Category"]] <- "Cell Type"
    plot_data[["Term_clean"]] <- plot_data[["Term"]]

    # Handle different model types
    if (x[["indep_var"]] == "cell_type_batch_interaction") {

        # Identify main effect cell types (no "batch" and no ":" and no "dataset")
        celltype_main <- !grepl("batch", plot_data[["Term"]]) &
            !grepl(":", plot_data[["Term"]]) &
            !grepl("dataset", plot_data[["Term"]])
        plot_data[["Category"]][celltype_main] <- "Cell Type"

        # Identify main effect batch (starts with "batch" but no ":")
        batch_main <- grepl("^batch", plot_data[["Term"]]) & !grepl(":", plot_data[["Term"]])
        plot_data[["Category"]][batch_main] <- "Batch"

        # Identify interactions (contains ":")
        interactions <- grepl(":", plot_data[["Term"]])
        plot_data[["Category"]][interactions] <- "Cell Type : Batch"

        # Clean up term names
        plot_data[["Term_clean"]] <- gsub("cell_type", "", plot_data[["Term_clean"]])

        # Special handling for batch names
        batch_pattern <- grepl("^batch", plot_data[["Term_clean"]])
        if (any(batch_pattern)) {
            batch_names <- gsub("^batch", "", plot_data[["Term_clean"]][batch_pattern])
            # Remove any remaining "batch" or "sample" prefixes
            batch_names <- gsub("^batch", "", batch_names, ignore.case = TRUE)
            batch_names <- gsub("^sample", "", batch_names, ignore.case = TRUE)
            # Remove Query_ prefix if present (fixes the main effect double Query issue)
            batch_names <- gsub("^Query_", "", batch_names)
            # Remove leading non-alphanumeric characters
            batch_names <- gsub("^[^A-Za-z0-9]+", "", batch_names)
            plot_data[["Term_clean"]][batch_pattern] <- paste("Query Batch", batch_names)
        }

        # Clean up remaining terms
        plot_data[["Term_clean"]] <- gsub("^:", "", plot_data[["Term_clean"]])

        # Special handling for interactions
        interaction_indices <- grepl(":", plot_data[["Term_clean"]])
        if (any(interaction_indices)) {
            interaction_terms <- plot_data[["Term_clean"]][interaction_indices]
            interaction_parts <- strsplit(interaction_terms, ":")
            formatted_interactions <- sapply(interaction_parts, function(parts) {
                if (length(parts) == 2) {
                    cell_type_part <- trimws(parts[1])
                    batch_part <- trimws(parts[2])

                    # Clean batch part thoroughly
                    batch_part <- gsub("^batch", "", batch_part, ignore.case = TRUE)
                    batch_part <- gsub("^sample", "", batch_part, ignore.case = TRUE)
                    batch_part <- gsub("^[^A-Za-z0-9]+", "", batch_part)

                    # Remove Query_ prefix if present (fixes the double Query issue)
                    batch_part <- gsub("^Query_", "", batch_part)
                    batch_part <- trimws(batch_part)

                    # Add "Query Batch" prefix if not already there
                    if (!grepl("Query Batch", batch_part)) {
                        batch_part <- paste("Query Batch", batch_part)
                    }
                    return(paste("Cell Type", cell_type_part, ":", batch_part))
                }
                return(paste(parts, collapse = " : "))
            })
            plot_data[["Term_clean"]][interaction_indices] <- formatted_interactions
        }

    } else if (x[["indep_var"]] == "cell_type_dataset_interaction") {

        # Identify main effect cell types (no "dataset" and no ":")
        celltype_main <- !grepl("dataset", plot_data[["Term"]]) & !grepl(":", plot_data[["Term"]])
        plot_data[["Category"]][celltype_main] <- "Cell Type"

        # Identify main effect dataset (starts with "dataset" but no ":")
        dataset_main <- grepl("^dataset", plot_data[["Term"]]) & !grepl(":", plot_data[["Term"]])
        plot_data[["Category"]][dataset_main] <- "Batch"

        # Identify interactions (contains ":")
        interactions <- grepl(":", plot_data[["Term"]])
        plot_data[["Category"]][interactions] <- "Cell Type : Batch"

        # Clean up term names
        plot_data[["Term_clean"]] <- gsub("cell_type", "", plot_data[["Term_clean"]])

        # Special handling for dataset names - convert to batch terminology
        dataset_pattern <- grepl("^dataset", plot_data[["Term_clean"]])
        if (any(dataset_pattern)) {
            dataset_names <- gsub("^dataset", "", plot_data[["Term_clean"]][dataset_pattern])
            # For dataset interaction, "Query" becomes "Query Batch"
            dataset_names <- gsub("Query", "Query Batch", dataset_names)
            plot_data[["Term_clean"]][dataset_pattern] <- dataset_names
        }

        # Clean up remaining terms
        plot_data[["Term_clean"]] <- gsub("^:", "", plot_data[["Term_clean"]])

        # Special handling for interactions
        interaction_indices <- grepl(":", plot_data[["Term_clean"]])
        if (any(interaction_indices)) {
            interaction_terms <- plot_data[["Term_clean"]][interaction_indices]
            interaction_parts <- strsplit(interaction_terms, ":")
            formatted_interactions <- sapply(interaction_parts, function(parts) {
                if (length(parts) == 2) {
                    cell_type_part <- trimws(parts[1])
                    dataset_part <- trimws(parts[2])

                    # Clean dataset part
                    dataset_part <- gsub("^dataset", "", dataset_part, ignore.case = TRUE)
                    dataset_part <- trimws(dataset_part)

                    # Convert "Query" to "Query Batch"
                    if (dataset_part == "Query") {
                        dataset_part <- "Query Batch"
                    }

                    return(paste("Cell Type", cell_type_part, ":", dataset_part))
                }
                return(paste(parts, collapse = " : "))
            })
            plot_data[["Term_clean"]][interaction_indices] <- formatted_interactions
        }

    } else if (x[["indep_var"]] == "cell_type") {
        # For cell type only model
        plot_data[["Term_clean"]] <- gsub("cell_type", "", plot_data[["Term_clean"]])
    }

    # Filter coefficients based on coefficients_include parameter
    if (!is.null(coefficients_include)) {
        # Map coefficient types to categories
        category_mapping <- list(
            "cell_type" = "Cell Type",
            "batch" = "Batch",
            "interaction" = "Cell Type : Batch"
        )

        # Get categories to include
        categories_to_include <- unlist(category_mapping[coefficients_include])

        # Filter plot data
        plot_data <- plot_data[plot_data[["Category"]] %in% categories_to_include, ]

        # If no data remains after filtering, stop with informative message
        if (nrow(plot_data) == 0) {
            stop("No coefficients found for the specified types: ",
                 paste(coefficients_include, collapse = ", "))
        }
    }

    # Create proper factor ordering for facets (only include categories present in data)
    available_categories <- unique(plot_data[["Category"]])
    all_category_levels <- c("Cell Type", "Batch", "Cell Type : Batch")
    category_levels <- all_category_levels[all_category_levels %in% available_categories]

    plot_data[["Category"]] <- factor(plot_data[["Category"]], levels = category_levels)

    # Create PC factor with labels
    existing_pc_names <- unique(plot_data[["PC"]])
    existing_pc_indices <- match(existing_pc_names, pc_names)
    existing_pc_labels <- pc_labels[existing_pc_indices]

    plot_data[["PC_labeled"]] <- factor(plot_data[["PC"]],
                                        levels = existing_pc_names,
                                        labels = existing_pc_labels)

    # Create y-axis ordering - special handling for interactions
    if (x[["indep_var"]] %in% c("cell_type_batch_interaction", "cell_type_dataset_interaction")) {
        # For interactions, we want to order by batch first, then by cell type
        interaction_data <- plot_data[plot_data[["Category"]] == "Cell Type : Batch", ]

        if (nrow(interaction_data) > 0) {
            # Extract batch and cell type parts from interaction terms
            interaction_parts <- strsplit(interaction_data[["Term_clean"]], " : ")
            batch_parts <- sapply(interaction_parts, function(x) if(length(x) == 2) trimws(x[2]) else "")
            cell_type_parts <- sapply(interaction_parts, function(x) if(length(x) == 2) gsub("Cell Type ", "", trimws(x[1])) else "")

            # Create ordering data frame with unique terms only
            ordering_df <- data.frame(
                Term_clean = interaction_data[["Term_clean"]],
                batch_part = batch_parts,
                cell_type_part = cell_type_parts,
                stringsAsFactors = FALSE
            )

            # Remove duplicates
            ordering_df <- unique(ordering_df)

            # Order by batch first, then by cell type
            ordering_df <- ordering_df[order(ordering_df[["batch_part"]], ordering_df[["cell_type_part"]]), ]

            # Get the ordered interaction terms
            ordered_interaction_terms <- ordering_df[["Term_clean"]]

            # Get unique non-interaction terms
            non_interaction_terms <- unique(plot_data[plot_data[["Category"]] != "Cell Type : Batch", "Term_clean"])

            # Combine all unique terms in desired order
            all_terms <- unique(c(non_interaction_terms, ordered_interaction_terms))

            # Create factor with custom ordering
            plot_data[["Term_factor"]] <- factor(plot_data[["Term_clean"]], levels = all_terms)
        } else {
            # No interactions, use default ordering
            plot_data[["Term_factor"]] <- factor(plot_data[["Term_clean"]])
        }
    } else {
        # For non-interaction models, use default ordering
        plot_data[["Term_factor"]] <- factor(plot_data[["Term_clean"]])
    }

    # Check if Batch category has only one term and add padding if needed
    if (x[["indep_var"]] %in% c("cell_type_batch_interaction", "cell_type_dataset_interaction") &&
        "Batch" %in% available_categories) {
        batch_terms <- unique(plot_data[plot_data[["Category"]] == "Batch", "Term_clean"])
        if (length(batch_terms) == 1) {
            # Add a dummy row for better spacing
            dummy_row <- plot_data[plot_data[["Category"]] == "Batch", ][1, ]
            dummy_row[["Term_clean"]] <- ""
            dummy_row[["Term_factor"]] <- factor("", levels = c(levels(plot_data[["Term_factor"]]), ""))
            dummy_row[["coef"]] <- NA
            dummy_row[["Significant"]] <- FALSE
            plot_data <- rbind(plot_data, dummy_row)
        }
    }

    # Create main subtitle
    model_text <- switch(x[["indep_var"]],
                         "cell_type" = "PC ~ Cell Type",
                         "cell_type_batch_interaction" = "PC ~ Cell Type * Batch",
                         "cell_type_dataset_interaction" = "PC ~ Cell Type * Batch")

    subtitle_text1 <- paste("Model:", model_text, "| * indicates significance at alpha =", alpha)

    # Add information about coefficient filtering if applied
    if (!is.null(coefficients_include)) {
        coefficient_text <- paste("Showing coefficients:", paste(coefficients_include, collapse = ", "))
        subtitle_text1 <- paste(subtitle_text1, "|", coefficient_text)
    }

    # Create reference category subtitle
    reference_text_parts <- c()

    # Always include cell type reference
    if (!is.null(x[["reference_cell_type"]])) {
        reference_text_parts <- c(reference_text_parts, paste("Cell Type:", x[["reference_cell_type"]]))
    }

    # Include batch/dataset reference based on model type
    if (x[["indep_var"]] == "cell_type_batch_interaction") {
        # For query+reference with multiple batches OR query-only with batches
        if (!is.null(x[["reference_batch"]])) {
            reference_text_parts <- c(reference_text_parts, paste("Batch: Query Batch", x[["reference_batch"]]))
        } else if (!is.null(x[["reference_pca_var"]])) {
            # Query+reference with multiple batches (Reference is reference category)
            reference_text_parts <- c(reference_text_parts, "Batch: Reference")
        }
    } else if (x[["indep_var"]] == "cell_type_dataset_interaction") {
        # Query + Reference (Reference is always the reference category)
        reference_text_parts <- c(reference_text_parts, "Batch: Reference")
    }

    # Create subtitle
    if (length(reference_text_parts) > 0) {
        subtitle_text2 <- paste("Reference categories -", paste(reference_text_parts, collapse = "; "))
    } else {
        subtitle_text2 <- "Reference category is first alphabetically"
    }

    # Create plot with facets
    p <- ggplot2::ggplot(plot_data, ggplot2::aes(x = .data[["PC_labeled"]],
                                                 y = .data[["Term_factor"]],
                                                 fill = .data[["coef"]])) +
        ggplot2::geom_tile(color = "white", linewidth = 0.5) +
        ggplot2::geom_point(data = plot_data[plot_data[["Significant"]] & !is.na(plot_data[["coef"]]), ],
                            ggplot2::aes(x = .data[["PC_labeled"]],
                                         y = .data[["Term_factor"]]),
                            shape = 8, size = 2, color = "black") +
        ggplot2::scale_fill_gradient2(
            low = "blue", mid = "white", high = "red",
            midpoint = 0, name = "Coefficient",
            na.value = "white"
        ) +
        ggplot2::facet_grid(Category ~ ., scales = "free_y", space = "free_y") +
        ggplot2::labs(
            title = "Regression Coefficients Heatmap",
            subtitle = paste(subtitle_text1, "\n", subtitle_text2),
            x = NULL,
            y = NULL
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
            axis.text.y = ggplot2::element_text(size = 9),
            axis.text.x = ggplot2::element_text(size = 10),
            plot.title = ggplot2::element_text(size = 16, face = "bold", margin = ggplot2::margin(b = 10)),
            plot.subtitle = ggplot2::element_text(size = 11, color = "gray40", margin = ggplot2::margin(b = 20)),
            legend.position = "right",
            legend.title = ggplot2::element_text(size = 11, face = "bold"),
            panel.grid = ggplot2::element_blank(),
            plot.margin = ggplot2::margin(20, 20, 20, 20),
            strip.text.y = ggplot2::element_text(size = 10, face = "bold", color = "black"),
            strip.background = ggplot2::element_rect(fill = "gray92", color = "black", linewidth = 0.5),
            panel.spacing = ggplot2::unit(0.8, "lines"),
            panel.background = ggplot2::element_rect(fill = "white", color = "black", linewidth = 0.5)
        ) +
        ggplot2::scale_y_discrete(labels = function(x) ifelse(x == "", "", x))

    return(p)
}

#' @title Calculate the Overlap Coefficient for Highly Variable Genes
#'
#' @description
#' Calculates the overlap coefficient between the sets of highly variable genes from a reference dataset and a query dataset.
#'
#' @details
#' The overlap coefficient measures the similarity between two gene sets, indicating how well-aligned
#' reference and query datasets are in terms of their highly variable genes. This metric is
#' useful in single-cell genomics to understand the correspondence between different datasets.
#'
#' The coefficient is calculated using the formula:
#'
#' \deqn{Coefficient(X, Y) = \frac{|X \cap Y|}{min(|X|, |Y|)}}
#'
#' where X and Y are the sets of highly variable genes from the reference and query datasets, respectively,
#' \eqn{|X \cap Y|} is the number of genes common to both \eqn{X} and \eqn{Y}, and \eqn{min(|X|, |Y|)} is the size of the
#' smaller set among \eqn{X} and \eqn{Y}.
#'
#' @param reference_genes A character vector of highly variable genes from the reference dataset.
#' @param query_genes A character vector of highly variable genes from the query dataset.
#'
#' @return Overlap coefficient, a value between 0 and 1, where 0 indicates no overlap
#' and 1 indicates complete overlap of highly variable genes between datasets.
#'
#' @export
#'
#' @references
#' Luecken et al. Benchmarking atlas-level data integration in single-cell genomics. Nature Methods, 19:41-50, 2022.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Selecting highly variable genes
#' ref_var <- scran::getTopHVGs(reference_data, n = 500)
#' query_var <- scran::getTopHVGs(query_data, n = 500)
#' overlap_coefficient <- calculateHVGOverlap(reference_genes = ref_var,
#'                                            query_genes = query_var)
#' overlap_coefficient
#'
# Function to calculate overlap between HVGs of reference and query datasets
calculateHVGOverlap <- function(reference_genes,
                                query_genes) {

    # Sanity checks
    if (!is.vector(reference_genes) || !is.character(reference_genes)) {
        stop("reference_genes must be a character vector.")
    }
    if (!is.vector(query_genes) || !is.character(query_genes)) {
        stop("query_genes must be a character vector.")
    }
    if (length(reference_genes) == 0 || length(query_genes) == 0) {
        stop("Input vectors must not be empty.")
    }

    # Calculate the intersection of highly variable genes
    common_genes <- intersect(reference_genes, query_genes)

    # Calculate the size of the intersection
    intersection_size <- length(common_genes)

    # Calculate the size of the smaller set
    min_size <- min(length(reference_genes), length(query_genes))

    # Compute the overlap coefficient
    overlap_coefficient <- intersection_size / min_size
    overlap_coefficient <- round(overlap_coefficient, 2)

    # Return the overlap coefficient
    return(overlap_coefficient)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Unit tests for calculateHVGOverlap function
test_that("calculateHVGOverlap function tests", {
    
    # Test case 1: Check if function returns a numeric value
    reference_genes <- c("Gene1", "Gene2", "Gene3", "Gene4", "Gene5")
    query_genes <- c("Gene3", "Gene4", "Gene5", "Gene6", "Gene7")
    overlap_coefficient <- calculateHVGOverlap(reference_genes, query_genes)
    expect_type(overlap_coefficient, "double")
    
    # Test case 2: Check if function returns a value between 0 and 1
    expect_true(overlap_coefficient >= 0 && overlap_coefficient <= 1)
    
    # Test case 3: Check if function handles complete overlap correctly
    reference_genes <- c("Gene1", "Gene2", "Gene3")
    query_genes <- c("Gene1", "Gene2", "Gene3")
    overlap_coefficient <- calculateHVGOverlap(reference_genes, query_genes)
    expect_equal(overlap_coefficient, 1)
    
    # Test case 4: Check if function handles no overlap correctly
    reference_genes <- c("Gene1", "Gene2", "Gene3")
    query_genes <- c("Gene4", "Gene5", "Gene6")
    overlap_coefficient <- calculateHVGOverlap(reference_genes, query_genes)
    expect_equal(overlap_coefficient, 0)
    
    # Test case 5: Check if function handles partial overlap correctly
    reference_genes <- c("Gene1", "Gene2", "Gene3")
    query_genes <- c("Gene2", "Gene3", "Gene4")
    overlap_coefficient <- calculateHVGOverlap(reference_genes, query_genes)

    # Test case 6: Check if function handles different sizes of input vectors
    reference_genes <- c("Gene1", "Gene2", "Gene3", "Gene4", "Gene5")
    query_genes <- c("Gene3", "Gene4")
    overlap_coefficient <- calculateHVGOverlap(reference_genes, query_genes)
    expect_equal(overlap_coefficient, 2 / 2)
    
    # Test case 7: Check for correct error handling of non-character input
    expect_error(calculateHVGOverlap(reference_genes = 1:5, query_genes = c("Gene1", "Gene2")),
                 "reference_genes must be a character vector.")
    expect_error(calculateHVGOverlap(reference_genes = c("Gene1", "Gene2"), query_genes = 1:5),
                 "query_genes must be a character vector.")
    
    # Test case 8: Check for correct error handling of empty input vectors
    expect_error(calculateHVGOverlap(reference_genes = character(0), query_genes = c("Gene1", "Gene2")),
                 "Input vectors must not be empty.")
    expect_error(calculateHVGOverlap(reference_genes = c("Gene1", "Gene2"), query_genes = character(0)),
                 "Input vectors must not be empty.")
})

#' @title Compare Marker Gene Expression between Query and Reference Data
#'
#' @description
#' This function identifies marker genes for each cell type in both query and reference datasets
#' using the standard Bioconductor approach (Wilcoxon rank-sum test), and compares their expression
#' patterns to assess annotation quality. It can optionally filter query cells based on anomaly detection
#' results and restrict analysis to specific cell types.
#'
#' @details
#' The function performs the following steps:
#' 1. Optionally performs anomaly detection and filters query cells based on results.
#' 2. Identifies marker genes for each cell type in both datasets using \code{findMarkers} approach.
#' 3. Reference markers are always computed using all reference cells for each cell type.
#' 4. Query markers are computed using filtered cells (anomalous/non-anomalous) if specified.
#' 5. Compares the overlap of top marker genes between corresponding cell types.
#' 6. Evaluates the expression consistency of reference markers in query data.
#' 7. Provides quality scores based on marker gene concordance.
#'
#' Marker genes are identified using Wilcoxon rank-sum tests comparing each cell type against all others.
#' High overlap and consistent expression of markers indicate good annotation quality.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are
#'                   included.
#' @param n_markers Number of top marker genes to consider for each cell type. Default is 50.
#' @param min_cells Minimum number of cells required per cell type for marker identification. Default is 10.
#' @param anomaly_filter Character string specifying how to filter query cells based on anomaly detection.
#'                       Options: "none" (default), "anomalous_only", "non_anomalous_only".
#' @param assay_name Name of the assay to use for computations. Default is "logcounts".
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#' @param ... Additional arguments passed to the \code{detectAnomaly} function.
#'
#' @return A list containing the following elements:
#' \describe{
#'   \item{marker_overlap}{Matrix showing overlap of top markers between query and reference for each cell type.}
#'   \item{expression_consistency}{Matrix showing expression consistency of reference markers in query data.}
#'   \item{quality_scores}{Named vector of quality assessments for each cell type.}
#'   \item{markers_query}{List of marker gene results for each cell type in query data.}
#'   \item{markers_ref}{List of marker gene results for each cell type in reference data.}
#'   \item{common_cell_types}{Vector of cell types present in both datasets.}
#'   \item{n_cells_query}{Named vector of cell counts per type in query data.}
#'   \item{n_cells_ref}{Named vector of cell counts per type in reference data.}
#'   \item{anomaly_filter_used}{Character string indicating the anomaly filter applied.}
#'   \item{selected_cell_types}{Character vector of cell types analyzed.}
#'   \item{anomaly_output}{Output from anomaly detection if performed.}
#' }
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{plot.compareMarkersObject}}, \code{\link{detectAnomaly}}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Compare marker genes
#' marker_comparison <- compareMarkers(query_data = query_data,
#'                                     reference_data = reference_data,
#'                                     query_cell_type_col = "expert_annotation",
#'                                     ref_cell_type_col = "expert_annotation")
#'
#' # With anomaly filtering
#' marker_comparison_filtered <- compareMarkers(query_data = query_data,
#'                                             reference_data = reference_data,
#'                                             query_cell_type_col = "expert_annotation",
#'                                             ref_cell_type_col = "expert_annotation",
#'                                             anomaly_filter = "non_anomalous_only")
#'
#' # Visualize results
#' plot(marker_comparison)
#'
#' @importFrom stats var wilcox.test
#' @importFrom utils head
#'
# Function to compare marker genes between query and reference datasets
compareMarkers <- function(query_data,
                           reference_data,
                           query_cell_type_col,
                           ref_cell_type_col,
                           cell_types = NULL,
                           n_markers = 50,
                           min_cells = 10,
                           anomaly_filter = c("none", "anomalous_only", "non_anomalous_only"),
                           assay_name = "logcounts",
                           max_cells_query = 5000,
                           max_cells_ref = 5000,
                           ...){

    # Match arguments
    anomaly_filter <- match.arg(anomaly_filter)

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = TRUE,
                                  n_cell_types = NULL)

    # Ensure cell names exist for anomaly detection mapping
    original_ref_names <- colnames(reference_data)
    original_query_names <- colnames(query_data)

    if (is.null(original_ref_names) || any(is.na(original_ref_names)) ||
        length(unique(original_ref_names)) != length(original_ref_names)) {
        original_ref_names <- paste0("REF_CELL_", seq_len(ncol(reference_data)))
        colnames(reference_data) <- original_ref_names
    }

    if (is.null(original_query_names) || any(is.na(original_query_names)) ||
        length(unique(original_query_names)) != length(original_query_names)) {
        original_query_names <- paste0("QUERY_CELL_", seq_len(ncol(query_data)))
        colnames(query_data) <- original_query_names
    }

    # Perform anomaly detection FIRST (before any downsampling) if needed
    anomaly_output <- NULL
    if (anomaly_filter != "none") {
        tryCatch({
            anomaly_output <- detectAnomaly(reference_data = reference_data,
                                            query_data = query_data,
                                            ref_cell_type_col = ref_cell_type_col,
                                            query_cell_type_col = query_cell_type_col,
                                            cell_types = cell_types,
                                            max_cells_ref = NULL,      # No downsampling in anomaly detection
                                            max_cells_query = NULL,    # No downsampling in anomaly detection
                                            ...)
        }, error = function(e) {
            warning("Anomaly detection failed: ", e[["message"]], ". Proceeding without anomaly filtering.")
            anomaly_output <<- NULL
        })
    }

    # Downsample query and reference data (with cell type filtering)
    query_data <- downsampleSCE(sce_object = query_data,
                                max_cells = max_cells_query,
                                cell_types = cell_types,
                                cell_type_col = query_cell_type_col)
    reference_data <- downsampleSCE(sce_object = reference_data,
                                    max_cells = max_cells_ref,
                                    cell_types = cell_types,
                                    cell_type_col = ref_cell_type_col)

    # Get cell type information after downsampling
    query_cell_types_orig <- colData(query_data)[[query_cell_type_col]]
    ref_cell_types_orig <- colData(reference_data)[[ref_cell_type_col]]

    # Reference data is NEVER filtered - always use all reference cells
    reference_data_filtered <- reference_data
    ref_cell_types <- ref_cell_types_orig

    # Query data filtering based on anomaly detection results
    query_data_filtered <- query_data
    query_cell_types <- query_cell_types_orig

    if (anomaly_filter != "none" && !is.null(anomaly_output)) {

        # Function to filter ONLY query cells based on anomaly results using cell names
        .filterQueryCellsByAnomalyNames <- function(data, anomaly_data, filter_type) {
            current_cell_names <- colnames(data)
            cells_to_keep <- rep(TRUE, ncol(data))
            current_cell_types <- colData(data)[[query_cell_type_col]]

            for (cell_type in unique(current_cell_types)) {
                if (cell_type %in% names(anomaly_data)) {
                    # Get indices of cells of this type in current (downsampled) data
                    cell_indices <- which(current_cell_types == cell_type)
                    cell_names_this_type <- current_cell_names[cell_indices]

                    if ("query_anomaly" %in% names(anomaly_data[[cell_type]])) {
                        # Get anomaly status from original detection
                        anomaly_names <- names(anomaly_data[[cell_type]][["query_anomaly"]])
                        anomaly_status <- anomaly_data[[cell_type]][["query_anomaly"]]

                        if (!is.null(anomaly_names) && length(anomaly_names) > 0) {
                            # Map current cells to their anomaly status
                            for (i in seq_along(cell_indices)) {
                                cell_name <- cell_names_this_type[i]
                                cell_idx <- cell_indices[i]

                                # Find this cell in the anomaly results
                                anomaly_idx <- match(cell_name, anomaly_names)
                                if (!is.na(anomaly_idx)) {
                                    is_anomalous <- anomaly_status[anomaly_idx]

                                    if (filter_type == "anomalous_only") {
                                        cells_to_keep[cell_idx] <- is_anomalous
                                    } else if (filter_type == "non_anomalous_only") {
                                        cells_to_keep[cell_idx] <- !is_anomalous
                                    }
                                } else {
                                    # Cell not found in anomaly results (shouldn't happen)
                                    # Keep it by default
                                    if (filter_type == "anomalous_only") {
                                        cells_to_keep[cell_idx] <- FALSE
                                    }
                                }
                            }
                        }
                    }
                }
            }
            return(cells_to_keep)
        }

        # Filter ONLY query data using cell names
        query_keep <- .filterQueryCellsByAnomalyNames(query_data_filtered, anomaly_output, anomaly_filter)

        if (sum(query_keep) == 0) {
            warning("Anomaly filtering removed all query cells. Proceeding without filtering.")
        } else {
            query_data_filtered <- query_data_filtered[, query_keep]
            query_cell_types <- query_cell_types[query_keep]
        }
    }

    # Get unique cell types with sufficient cells
    query_types <- names(table(query_cell_types))[table(query_cell_types) >= min_cells]
    ref_types <- names(table(ref_cell_types))[table(ref_cell_types) >= min_cells]
    common_cell_types <- intersect(query_types, ref_types)

    # Filter by user-specified cell types if provided
    if (!is.null(cell_types)) {
        common_cell_types <- intersect(common_cell_types, cell_types)
    }

    if (length(common_cell_types) == 0) {
        stop("No common cell types with sufficient cells found between query and reference data")
    }

    # Function to find markers using Wilcoxon test (standard Bioconductor approach)
    .findMarkers <- function(expr_matrix, cell_types, target_type) {
        target_cells <- cell_types == target_type
        other_cells <- cell_types != target_type

        if (sum(target_cells) < 3 || sum(other_cells) < 3) {
            return(data.frame(gene = character(0), pval = numeric(0),
                              logFC = numeric(0), stringsAsFactors = FALSE))
        }

        # Perform Wilcoxon test for each gene
        pvals <- rep(1, nrow(expr_matrix))
        logFCs <- rep(0, nrow(expr_matrix))

        for (i in seq_len(nrow(expr_matrix))) {
            target_expr <- expr_matrix[i, target_cells]
            other_expr <- expr_matrix[i, other_cells]

            # Skip if no variation
            if (var(target_expr) == 0 && var(other_expr) == 0) {
                next
            }

            # Wilcoxon test with warning suppression
            test_result <- tryCatch({
                wilcox.test(target_expr, other_expr, alternative = "greater", exact = FALSE)
            }, error = function(e) {
                list(p.value = 1)
            })

            pvals[i] <- test_result[["p.value"]]
            logFCs[i] <- mean(target_expr) - mean(other_expr)
        }

        # Adjust p-values
        adj_pvals <- p.adjust(pvals, method = "BH")

        # Create results data frame
        results <- data.frame(
            gene = rownames(expr_matrix),
            pval = pvals,
            adj_pval = adj_pvals,
            logFC = logFCs,
            stringsAsFactors = FALSE
        )

        # Filter and sort
        results <- results[results[["adj_pval"]] < 0.05 & results[["logFC"]] > 0, ]
        results <- results[order(results[["adj_pval"]], decreasing = FALSE), ]

        return(results)
    }

    # Extract expression matrices
    query_matrix <- as.matrix(assay(query_data_filtered, assay_name))
    ref_matrix <- as.matrix(assay(reference_data_filtered, assay_name))  # Uses ALL reference cells

    # Find markers for each cell type
    markers_query <- list()
    markers_ref <- list()

    for (cell_type in common_cell_types) {
        # Query markers: use filtered cells (anomalous/non-anomalous if specified)
        markers_query[[cell_type]] <- .findMarkers(query_matrix, query_cell_types, cell_type)

        # Reference markers: ALWAYS use all reference cells
        markers_ref[[cell_type]] <- .findMarkers(ref_matrix, ref_cell_types, cell_type)
    }

    # Calculate marker overlap
    marker_overlap <- rep(0, length(common_cell_types))
    names(marker_overlap) <- common_cell_types

    # Calculate expression consistency
    expression_consistency <- rep(0, length(common_cell_types))
    names(expression_consistency) <- common_cell_types

    # Cell counts - convert table to named numeric vector
    query_table <- table(query_cell_types)  # Filtered query counts
    ref_table <- table(ref_cell_types)      # All reference counts

    n_cells_query <- as.numeric(query_table[common_cell_types])
    names(n_cells_query) <- common_cell_types

    n_cells_ref <- as.numeric(ref_table[common_cell_types])
    names(n_cells_ref) <- common_cell_types

    for (cell_type in common_cell_types) {
        # Get top markers
        query_top <- head(markers_query[[cell_type]][["gene"]], n_markers)
        ref_top <- head(markers_ref[[cell_type]][["gene"]], n_markers)

        # Calculate overlap (Jaccard index)
        if (length(query_top) > 0 && length(ref_top) > 0) {
            overlap <- length(intersect(query_top, ref_top))
            union_size <- length(union(query_top, ref_top))
            marker_overlap[cell_type] <- overlap / union_size
        }

        # Calculate expression consistency of reference markers in query
        if (length(ref_top) > 0) {
            ref_markers_available <- intersect(ref_top, rownames(query_matrix))
            if (length(ref_markers_available) > 0) {
                target_cells <- query_cell_types == cell_type
                other_cells <- query_cell_types != cell_type

                # Check if reference markers are still upregulated in query
                consistent_markers <- 0
                for (marker in ref_markers_available) {
                    target_mean <- mean(query_matrix[marker, target_cells])
                    other_mean <- mean(query_matrix[marker, other_cells])
                    if (target_mean > other_mean) {
                        consistent_markers <- consistent_markers + 1
                    }
                }
                expression_consistency[cell_type] <- consistent_markers / length(ref_markers_available)
            }
        }
    }

    # Quality assessment based on MINIMUM of both metrics (most conservative)
    overlap_quality <- rep("Poor", length(common_cell_types))
    overlap_quality[marker_overlap >= 0.7] <- "Good"
    overlap_quality[marker_overlap >= 0.4 & marker_overlap < 0.7] <- "Moderate"

    consistency_quality <- rep("Poor", length(common_cell_types))
    consistency_quality[expression_consistency >= 0.7] <- "Good"
    consistency_quality[expression_consistency >= 0.4 & expression_consistency < 0.7] <- "Moderate"

    # Overall quality is the WORST of the two
    quality_scores <- rep("Poor", length(common_cell_types))
    names(quality_scores) <- common_cell_types

    for (i in seq_along(common_cell_types)) {
        overlap_qual <- overlap_quality[i]
        consistency_qual <- consistency_quality[i]

        # Take the worst quality (Poor > Moderate > Good in terms of "badness")
        if (overlap_qual == "Poor" || consistency_qual == "Poor") {
            quality_scores[i] <- "Poor"
        } else if (overlap_qual == "Moderate" || consistency_qual == "Moderate") {
            quality_scores[i] <- "Moderate"
        } else {
            quality_scores[i] <- "Good"
        }
    }

    # Prepare output
    output <- list(
        marker_overlap = marker_overlap,
        expression_consistency = expression_consistency,
        quality_scores = quality_scores,
        markers_query = markers_query,
        markers_ref = markers_ref,
        common_cell_types = common_cell_types,
        n_cells_query = n_cells_query,
        n_cells_ref = n_cells_ref,
        anomaly_filter_used = anomaly_filter,
        selected_cell_types = common_cell_types,
        anomaly_output = anomaly_output
    )

    class(output) <- c(class(output), "compareMarkersObject")
    return(output)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Test cases
test_that("compareMarkers correctly handles typical inputs", {

    # Basic functionality test
    result <- compareMarkers(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = "expert_annotation",
                             ref_cell_type_col = "expert_annotation")

    # Check that result is a list with correct class
    expect_true(is.list(result))
    expect_true("compareMarkersObject" %in% class(result))

    # Check all expected components are present
    expected_components <- c("marker_overlap", "expression_consistency", "quality_scores",
                             "markers_query", "markers_ref", "common_cell_types",
                             "n_cells_query", "n_cells_ref", "anomaly_filter_used",
                             "selected_cell_types", "anomaly_output")
    expect_true(all(expected_components %in% names(result)))

    # Check that marker_overlap is numeric and properly named
    expect_true(is.numeric(result$marker_overlap))
    expect_true(all(result$marker_overlap >= 0 & result$marker_overlap <= 1))
    expect_true(!is.null(names(result$marker_overlap)))

    # Check that expression_consistency is numeric and properly named
    expect_true(is.numeric(result$expression_consistency))
    expect_true(all(result$expression_consistency >= 0 & result$expression_consistency <= 1))
    expect_true(!is.null(names(result$expression_consistency)))

    # Check quality_scores are valid categories
    expect_true(all(result$quality_scores %in% c("Good", "Moderate", "Poor")))
    expect_true(!is.null(names(result$quality_scores)))

    # Check that markers are lists
    expect_true(is.list(result$markers_query))
    expect_true(is.list(result$markers_ref))

    # Check cell counts are numeric and positive
    expect_true(is.numeric(result$n_cells_query))
    expect_true(is.numeric(result$n_cells_ref))
    expect_true(all(result$n_cells_query > 0))
    expect_true(all(result$n_cells_ref > 0))

    # Check anomaly_filter_used defaults to "none"
    expect_equal(result$anomaly_filter_used, "none")
    expect_null(result$anomaly_output)
})

test_that("compareMarkers works with different parameter settings", {

    # Test with different n_markers
    result <- compareMarkers(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = "expert_annotation",
                             ref_cell_type_col = "expert_annotation",
                             n_markers = 20)

    expect_true("compareMarkersObject" %in% class(result))
    expect_true(length(result$common_cell_types) > 0)

    # Test with different min_cells
    result2 <- compareMarkers(query_data = query_data,
                              reference_data = reference_data,
                              query_cell_type_col = "expert_annotation",
                              ref_cell_type_col = "expert_annotation",
                              min_cells = 5)

    expect_true("compareMarkersObject" %in% class(result2))

    # Test with specific cell_types
    # First get available cell types
    query_types <- unique(colData(query_data)[["expert_annotation"]])
    ref_types <- unique(colData(reference_data)[["expert_annotation"]])
    common_types <- intersect(query_types, ref_types)

    if (length(common_types) > 1) {
        selected_types <- common_types[1:min(2, length(common_types))]
        result3 <- compareMarkers(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = "expert_annotation",
                                  ref_cell_type_col = "expert_annotation",
                                  cell_types = selected_types)

        expect_true("compareMarkersObject" %in% class(result3))
        expect_true(all(result3$selected_cell_types %in% selected_types))
    }
})

test_that("compareMarkers handles edge cases appropriately", {

    # Test error when no common cell types exist
    # Create a modified query with different cell type names
    query_modified <- query_data
    colData(query_modified)[["different_annotation"]] <- paste0("type_",
                                                                colData(query_data)[["expert_annotation"]])

    expect_error(
        compareMarkers(query_data = query_modified,
                       reference_data = reference_data,
                       query_cell_type_col = "different_annotation",
                       ref_cell_type_col = "expert_annotation"),
        "No common cell types"
    )

    # Test with very high min_cells requirement
    expect_error(
        compareMarkers(query_data = query_data,
                       reference_data = reference_data,
                       query_cell_type_col = "expert_annotation",
                       ref_cell_type_col = "expert_annotation",
                       min_cells = 10000),
        "No common cell types"
    )
})

test_that("compareMarkers anomaly filtering works correctly", {

    # Test that anomaly_filter argument is properly handled
    result_none <- compareMarkers(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = "expert_annotation",
                                  ref_cell_type_col = "expert_annotation",
                                  anomaly_filter = "none")

    expect_equal(result_none$anomaly_filter_used, "none")
    expect_null(result_none$anomaly_output)

    # Note: Testing actual anomaly filtering would require the detectAnomaly function
    # to work properly, so we mainly test that the parameter is handled correctly
})

test_that("compareMarkers produces consistent marker gene results", {

    result <- compareMarkers(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = "expert_annotation",
                             ref_cell_type_col = "expert_annotation")

    # Check that each cell type has corresponding markers
    for (cell_type in result$common_cell_types) {
        expect_true(cell_type %in% names(result$markers_query))
        expect_true(cell_type %in% names(result$markers_ref))

        # Check marker data frame structure
        query_markers <- result$markers_query[[cell_type]]
        ref_markers <- result$markers_ref[[cell_type]]

        if (nrow(query_markers) > 0) {
            expect_true(all(c("gene", "pval", "adj_pval", "logFC") %in% colnames(query_markers)))
            expect_true(all(query_markers$logFC > 0))
            expect_true(all(query_markers$adj_pval < 0.05))
        }

        if (nrow(ref_markers) > 0) {
            expect_true(all(c("gene", "pval", "adj_pval", "logFC") %in% colnames(ref_markers)))
            expect_true(all(ref_markers$logFC > 0))
            expect_true(all(ref_markers$adj_pval < 0.05))
        }
    }
})

test_that("compareMarkers handles different assay names", {

    # Test with default assay
    result <- compareMarkers(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = "expert_annotation",
                             ref_cell_type_col = "expert_annotation",
                             assay_name = "logcounts")

    expect_true("compareMarkersObject" %in% class(result))

    # Test error with non-existent assay
    expect_error(
        compareMarkers(query_data = query_data,
                       reference_data = reference_data,
                       query_cell_type_col = "expert_annotation",
                       ref_cell_type_col = "expert_annotation",
                       assay_name = "nonexistent_assay")
    )
})

test_that("compareMarkers argument validation works", {

    # Test invalid anomaly_filter argument
    expect_error(
        compareMarkers(query_data = query_data,
                       reference_data = reference_data,
                       query_cell_type_col = "expert_annotation",
                       ref_cell_type_col = "expert_annotation",
                       anomaly_filter = "invalid_option")
    )

    # Test with non-existent cell type column
    expect_error(
        compareMarkers(query_data = query_data,
                       reference_data = reference_data,
                       query_cell_type_col = "nonexistent_column",
                       ref_cell_type_col = "expert_annotation")
    )
})

## ----setup, include = FALSE, fig.show='hide'----------------------------------
knitr::knit_hooks$set(pngquant = knitr::hook_pngquant)

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dev = "ragg_png",
  dpi = 72,
  fig.retina = 2,
  fig.align = "center",
  out.width = "100%",
  pngquant = "--speed=1 --quality=1-5"
)

## ----eval = FALSE, fig.show='hide'--------------------------------------------
# BiocManager::install("ccb-hms/scDiagnostics")

## ----eval=FALSE, fig.show='hide'----------------------------------------------
# BiocManager::install("ccb-hms/scDiagnostics",
#                      build_vignettes = TRUE,
#                      dependencies = TRUE)

## ----message=FALSE, fig.show='hide'-------------------------------------------
library(scDiagnostics)

## ----message=FALSE, fig.show='hide'-------------------------------------------
# Load datasets
data("reference_data")
data("query_data")
data("qc_data")

# Set seed for reproducibility
set.seed(0)

## ----message=FALSE, fig.show='hide'-------------------------------------------
# Load library
library(scran)
library(scater)

# Subset to CD4 cells
ref_data_cd4 <- reference_data[, which(
    reference_data$expert_annotation == "CD4")]
query_data_cd4 <- query_data_cd4 <- query_data[, which(
    query_data$expert_annotation == "CD4")]

# Select highly variable genes
ref_top_genes <- getTopHVGs(ref_data_cd4, n = 500)
query_top_genes <- getTopHVGs(query_data_cd4, n = 500)
common_genes <- intersect(ref_top_genes, query_top_genes)

# Subset data by common genes
ref_data_cd4 <- ref_data_cd4[common_genes,]
query_data_cd4 <- query_data_cd4[common_genes,]

# Run PCA on both datasets
ref_data_cd4 <- runPCA(ref_data_cd4)
query_data_cd4 <- runPCA(query_data_cd4)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Plot PCA data
pc_plot <- plotCellTypePCA(
    query_data = query_data, 
    reference_data = reference_data,
    cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
    query_cell_type_col = "expert_annotation", 
    ref_cell_type_col = "expert_annotation",
    pc_subset = 1:3
)
# Display plot
pc_plot

## ----fig.show='hide'----------------------------------------------------------
disc_output <- calculateDiscriminantSpace(
    reference_data = reference_data,
    query_data = query_data, 
    ref_cell_type_col = "expert_annotation",
    query_cell_type_col = "SingleR_annotation"
)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
plot(disc_output, plot_type = "scatterplot")

## ----fig.height=5, fig.width=10, fig.show='hide', fig.show='hide'-------------
plotMarkerExpression(reference_data = reference_data, 
                     query_data = query_data, 
                     ref_cell_type_col = "expert_annotation", 
                     query_cell_type_col = "SingleR_annotation", 
                     gene_name = "VPREB3", 
                     cell_type = "B_and_plasma")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Remove cell types with very few cells
qc_data_subset <- qc_data[, !(qc_data$SingleR_annotation 
                              %in% c("Chondrocytes", "DC", 
                                     "Neurons","Platelets"))]

# Generate scatter plot
library(ggplot2)
p1 <- plotQCvsAnnotation(sce_object = qc_data_subset,
                         cell_type_col = "SingleR_annotation",
                         qc_col = "total",
                         score_col = "annotation_scores")
p1 + xlab("Library Size")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Compare PCA subspaces between query and reference data
subspace_comparison <- comparePCASubspace(
    query_data = query_data_cd4,
    reference_data = ref_data_cd4, 
    query_cell_type_col = "expert_annotation", 
    ref_cell_type_col = "expert_annotation", 
    pc_subset = 1:5
)

# View weighted cosine similarity score
subspace_comparison$weighted_cosine_similarity

# Plot output for PCA subspace comparison (if a plot method is available)
plot(subspace_comparison)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Example usage of the function
plotPairwiseDistancesDensity(query_data = query_data, 
                             reference_data = reference_data, 
                             query_cell_type_col = "expert_annotation", 
                             ref_cell_type_col = "expert_annotation", 
                             cell_type = "CD8", 
                             pc_subset = 1:10,
                             distance_metric = "correlation",
                             correlation_method = "pearson")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Generate the Wasserstein distance density plot
wasserstein_data <- calculateWassersteinDistance(
    query_data = query_data_cd4,
    reference_data = ref_data_cd4, 
    query_cell_type_col = "expert_annotation", 
    ref_cell_type_col = "expert_annotation", 
    pc_subset = 1:10,
)
plot(wasserstein_data)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# RF function to compare (between datasets) which genes are best at differentiating cell types
rf_output <- calculateVarImpOverlap(reference_data = reference_data, 
                                    query_data = query_data, 
                                    query_cell_type_col = "SingleR_annotation", 
                                    ref_cell_type_col = "expert_annotation", 
                                    n_tree = 500,
                                    n_top = 50)

# Comparison table
rf_output$var_imp_comparison

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Compute pairwise correlations between specified cell types
cor_matrix_avg <- calculateAveragePairwiseCorrelation(
  query_data = query_data, 
  reference_data = reference_data, 
  query_cell_type_col = "SingleR_annotation", 
  ref_cell_type_col = "expert_annotation", 
  cell_types = c("CD4", "CD8", "B_and_plasma"), 
  pc_subset = 1:5,
  correlation_method = "spearman"
)

# Visualize the average pairwise correlation matrix
plot(cor_matrix_avg)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Perform anomaly detection
anomaly_output <- detectAnomaly(reference_data = reference_data, 
                                query_data = query_data, 
                                ref_cell_type_col = "expert_annotation", 
                                query_cell_type_col = "SingleR_annotation",
                                pc_subset = 1:5)
# Plot the results for a specific cell type
plot(anomaly_output, 
     cell_type = "CD4", 
     data_type = "query",
     pc_subset = 1:5)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Identify outliers for CD4
cd4_anomalies <- detectAnomaly(reference_data = reference_data, 
                               query_data = query_data, 
                               query_cell_type_col = "SingleR_annotation", 
                               ref_cell_type_col = "expert_annotation")
cd4_top6_anomalies <- names(sort(cd4_anomalies$CD4$query_anomaly_scores, 
                                 decreasing = TRUE)[1:6])

# Plot the PC data
distance_data <- calculateCellDistances(
    query_data = query_data, 
    reference_data = reference_data, 
    query_cell_type_col = "SingleR_annotation", 
    ref_cell_type_col = "expert_annotation"
) 

# Plot the densities of the distances
plot(distance_data, ref_cell_type = "CD4", cell_names = cd4_top6_anomalies)

## ----SessionInfo, echo=FALSE, message=FALSE, warning=FALSE, comment=NA, fig.show='hide'----
options(width = 80) 
sessionInfo()

#' @title PCA Anomaly Scores via Isolation Forests with Visualization
#'
#' @description
#' This function detects anomalies in single-cell data by projecting the data onto a PCA space and using an isolation forest
#' algorithm to identify anomalies.
#'
#' @details
#' This function projects the query data onto the PCA space of the reference data. An isolation forest is then built on the
#' reference data to identify anomalies in the query data based on their PCA projections. If no query dataset is provided by the user,
#' the anomaly scores are computed on the reference data itself. Anomaly scores for the data with all combined cell types are also
#' provided as part of the output.
#'
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_data An optional \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' If NULL, then the isolation forest anomaly scores are computed for the reference data. Default is NULL.
#' @param ref_cell_type_col A character string specifying the column name in the reference dataset containing cell type annotations.
#' @param query_cell_type_col A character string specifying the column name in the query dataset containing cell type annotations.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.
#' @param pc_subset A numeric vector specifying which principal components to use in the analysis. Default is 1:5
#' If set to \code{NULL} then no dimensionality reduction is performed and the assay data is used directly for computations.
#' @param n_tree An integer specifying the number of trees for the isolation forest. Default is 500
#' @param anomaly_threshold A numeric value specifying the threshold for identifying anomalies, Default is 0.6.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#' @param ... Additional arguments passed to the `isolation.forest` function.
#'
#' @return A list containing the following components for each cell type and the combined data:
#' \item{anomaly_scores}{Anomaly scores for each cell in the query data.}
#' \item{anomaly}{Logical vector indicating whether each cell is classified as an anomaly.}
#' \item{reference_mat_subset}{PCA projections of the reference data.}
#' \item{query_mat_subset}{PCA projections of the query data (if provided).}
#' \item{var_explained}{Proportion of variance explained by the retained principal components.}
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{plot.detectAnomalyObject}}
#'
#' @references
#' \itemize{
#'   \item Liu, F. T., Ting, K. M., & Zhou, Z. H. (2008). Isolation forest. In 2008 Eighth IEEE International Conference on Data Mining (pp. 413-422). IEEE.
#'   \item \href{https://cran.r-project.org/web/packages/isotree/isotree.pdf}{isotree: Isolation-Based Outlier Detection}
#' }
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Store PCA anomaly data
#' anomaly_output <- detectAnomaly(reference_data = reference_data,
#'                                 query_data = query_data,
#'                                 ref_cell_type_col = "expert_annotation",
#'                                 query_cell_type_col = "SingleR_annotation",
#'                                 pc_subset = 1:5,
#'                                 n_tree = 500,
#'                                 anomaly_threshold = 0.6)
#'
#' # Plot the output for a cell type
#' plot(anomaly_output,
#'      cell_type = "CD4",
#'      pc_subset = 1:3,
#'      data_type = "query")
#'
#' @importFrom methods is
#' @importFrom stats na.omit predict qnorm
#' @importFrom utils tail
#'
# Function to perform diagnostics using isolation forest with PCA and visualization
detectAnomaly <- function(reference_data,
                          query_data = NULL,
                          ref_cell_type_col,
                          query_cell_type_col = NULL,
                          cell_types = NULL,
                          pc_subset = 1:5,
                          n_tree = 500,
                          anomaly_threshold = 0.6,
                          assay_name = "logcounts",
                          max_cells_query = 5000,
                          max_cells_ref = 5000,
                          ...) {

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)
    if(!is.null(query_data)){
        query_data <- convertColumnsToCharacter(sce_object = query_data,
                                                convert_cols = query_cell_type_col)
    }

    # Check if n_tree is a positive integer
    if (!is.numeric(n_tree) || n_tree <= 0 || n_tree != as.integer(n_tree)) {
        stop("\'n_tree\' must be a positive integer.")
    }

    # Input check for anomaly_threshold
    if (!is.numeric(anomaly_threshold) || anomaly_threshold <= 0 ||
        anomaly_threshold >= 1) {
        stop("\'anomaly_threshold\' must be a positive number greater than 0 and less than 1.")
    }

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = NULL)

    # Get data from reference and query datasets
    if(!is.null(pc_subset)){
        if(!is.null(query_data)){
            pca_output <- projectPCA(query_data = query_data,
                                     reference_data = reference_data,
                                     query_cell_type_col = query_cell_type_col,
                                     ref_cell_type_col = ref_cell_type_col,
                                     cell_types = cell_types,
                                     pc_subset = pc_subset,
                                     assay_name = assay_name,
                                     max_cells_ref = max_cells_ref,
                                     max_cells_query = max_cells_query)
            query_mat <- pca_output[pca_output[["dataset"]] == "Query",]
            reference_mat <- pca_output[pca_output[["dataset"]] == "Reference",]

            # Extract cell type information from PCA output
            query_cell_types <- query_mat[["cell_type"]]
            reference_cell_types <- reference_mat[["cell_type"]]

            # Remove the metadata columns to keep only PCA coordinates
            pc_cols <- grep("^PC", colnames(pca_output), value = TRUE)
            query_mat <- query_mat[, pc_cols, drop = FALSE]
            reference_mat <- reference_mat[, pc_cols, drop = FALSE]
        } else {
            reference_data <- downsampleSCE(sce_object = reference_data,
                                            max_cells = max_cells_ref,
                                            cell_types =  cell_types,
                                            cell_type_col = ref_cell_type_col)
            reference_mat <- reducedDim(reference_data, "PCA")[, pc_subset]
            reference_cell_types <- reference_data[[ref_cell_type_col]]
        }
    } else{
        reference_data <- downsampleSCE(sce_object = reference_data,
                                        max_cells = max_cells_ref,
                                        cell_types =  cell_types,
                                        cell_type_col = ref_cell_type_col)
        reference_mat <- t(as.matrix(assay(reference_data, assay_name)))
        reference_cell_types <- reference_data[[ref_cell_type_col]]
        if(!is.null(query_data)){
            query_data <- downsampleSCE(sce_object = query_data,
                                        max_cells = max_cells_ref,
                                        cell_types =  cell_types,
                                        cell_type_col = query_cell_type_col)
            query_mat <- t(as.matrix(assay(query_data, assay_name)))
            query_cell_types <- query_data[[query_cell_type_col]]
        }
    }

    # List to store output
    output <- list()

    # Cell types list for anomaly detection
    cell_types_list <- as.list(cell_types)
    cell_types_list[["Combined"]] <- cell_types

    for (cell_type in cell_types_list) {

        # Filter reference and query PCA data for the current cell type
        reference_mat_subset <- na.omit(reference_mat[which(
            reference_cell_types %in% cell_type),])

        # Build isolation forest on reference PCA data for this cell type
        isolation_forest <- isotree::isolation.forest(reference_mat_subset,
                                                      ntree = n_tree,)

        # Calculate anomaly scores for query data (scaled by reference path length)
        reference_anomaly_scores <- predict(isolation_forest,
                                            newdata = reference_mat_subset,
                                            type = "score")
        if(!is.null(query_data)){
            query_mat_subset <- na.omit(query_mat[which(
                query_cell_types %in% cell_type),])
            query_anomaly_scores <- predict(isolation_forest,
                                            newdata = query_mat_subset,
                                            type = "score")
        }

        # Store cell type anomaly scores and PCA data
        list_name <- ifelse(length(cell_type) == 1, cell_type, "Combined")
        output[[list_name]] <- list()
        output[[list_name]][["reference_anomaly_scores"]] <-
            reference_anomaly_scores
        output[[list_name]][["reference_anomaly"]] <-
            reference_anomaly_scores > anomaly_threshold
        output[[list_name]][["reference_mat_subset"]] <- reference_mat_subset
        if(!is.null(query_data)){
            output[[list_name]][["query_mat_subset"]] <- query_mat_subset
            output[[list_name]][["query_anomaly_scores"]] <-
                query_anomaly_scores
            output[[list_name]][["query_anomaly"]] <-
                query_anomaly_scores > anomaly_threshold
        }
        if(!is.null(pc_subset))
            output[[list_name]][["var_explained"]] <-
            attributes(reducedDim(
                reference_data, "PCA"))[["percentVar"]][pc_subset]
    }

    # Set the class of the output
    class(output) <- c(class(output), "detectAnomalyObject")

    # Return anomaly, PCA data and optional PCA anomaly plots for each cell type
    return(output)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load data
data("reference_data")
data("query_data")

# Plot the PC data
distance_data <- calculateCellDistances(query_data = query_data,
                                        reference_data = reference_data,
                                        query_cell_type_col = "SingleR_annotation",
                                        ref_cell_type_col = "expert_annotation",
                                        pc_subset = 1:10)

# Identify outliers for CD4
cd4_anomalies <- detectAnomaly(reference_data = reference_data,
                               query_data = query_data,
                               query_cell_type_col = "SingleR_annotation",
                               ref_cell_type_col = "expert_annotation",
                               pc_subset = 1:10,
                               n_tree = 500,
                               anomaly_threshold = 0.5)$CD4
cd4_top6_anomalies <- names(sort(cd4_anomalies$query_anomaly_scores, decreasing = TRUE)[1:6])

test_that("plot.calculateCellDistances generates plots correctly", {
    # Generate plot using the function
    p1 <- plot(distance_data, ref_cell_type = "CD4", cell_names = cd4_top6_anomalies)
    p2 <- plot(distance_data, ref_cell_type = "CD8", cell_names = cd4_top6_anomalies)

    # Check if output is a ggplot object
    expect_s3_class(p1, "ggplot")
    expect_s3_class(p2, "ggplot")
})

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Store PCA anomaly data
anomaly_output <- detectAnomaly(reference_data = reference_data,
                                query_data = query_data,
                                ref_cell_type_col = "expert_annotation",
                                query_cell_type_col = "SingleR_annotation",
                                pc_subset = 1:10,
                                n_tree = 500,
                                anomaly_threshold = 0.5)

test_that("plot.detectAnomalyObject generates plots correctly", {
    # Generate plot using the function (query data)
    p1 <- plot(anomaly_output,
               cell_type = "CD4",
               pc_subset = 1:3,
               data_type = "query")

    # Check if output is a ggmatrix object (from GGally::ggpairs)
    expect_s3_class(p1, "ggmatrix")

    # Generate plot using the function (reference data)
    p2 <- plot(anomaly_output,
               cell_type = "CD4",
               pc_subset = 1:3,
               data_type = "reference")

    # Check if output is a ggmatrix object
    expect_s3_class(p2, "ggmatrix")
})

test_that("plot works with different facet options", {
    # Test different diagonal facet options
    p1 <- plot(anomaly_output,
               cell_type = "CD4",
               pc_subset = 1:2,
               data_type = "query",
               diagonal_facet = "density",
               upper_facet = "blank")

    expect_s3_class(p1, "ggmatrix")

    # Test with ridge diagonal and contour upper
    p2 <- plot(anomaly_output,
               cell_type = "CD4",
               pc_subset = 1:2,
               data_type = "query",
               diagonal_facet = "ridge",
               upper_facet = "contour")

    expect_s3_class(p2, "ggmatrix")

    # Test with boxplot diagonal and ellipse upper
    p3 <- plot(anomaly_output,
               cell_type = "CD4",
               pc_subset = 1:2,
               data_type = "query",
               diagonal_facet = "boxplot",
               upper_facet = "ellipse")

    expect_s3_class(p3, "ggmatrix")
})

test_that("plot works with Combined cell type and default parameters", {
    # Test with Combined cell type (default)
    p1 <- plot(anomaly_output,
               pc_subset = 1:2,
               data_type = "query")

    expect_s3_class(p1, "ggmatrix")

    # Test with explicit Combined cell type
    p2 <- plot(anomaly_output,
               cell_type = "Combined",
               pc_subset = 1:2,
               data_type = "reference")

    expect_s3_class(p2, "ggmatrix")
})

test_that("plot handles error cases correctly", {
    # Test with invalid cell type
    expect_error(plot(anomaly_output,
                      cell_type = "NonexistentType",
                      pc_subset = 1:2),
                 "cell_type.*not available")

    # Test with out of range pc_subset
    max_pc <- ncol(anomaly_output$CD4$reference_mat_subset)
    expect_error(plot(anomaly_output,
                      cell_type = "CD4",
                      pc_subset = 1:(max_pc + 5)),
                 "pc_subset.*out of range")
})

test_that("plot works with different n_tree parameter", {
    # Test with different n_tree parameter
    p1 <- plot(anomaly_output,
               cell_type = "CD4",
               pc_subset = 1:2,
               data_type = "query",
               n_tree = 100)

    expect_s3_class(p1, "ggmatrix")
})

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

test_that("detectAnomaly works correctly with default parameters", {
    # Store PCA anomaly data
    anomaly_output <- detectAnomaly(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = "SingleR_annotation"
    )

    # Check if the output is a list and has correct class
    expect_true(is.list(anomaly_output))
    expect_s3_class(anomaly_output, "detectAnomalyObject")

    # Check if the output contains the expected elements
    expect_true(length(anomaly_output) > 0)
    expect_true("Combined" %in% names(anomaly_output))

    # Check the structure of the elements for the first cell type
    first_cell_type <- names(anomaly_output)[1]
    expect_true(all(c("reference_anomaly_scores", "reference_anomaly", "reference_mat_subset") %in% names(anomaly_output[[first_cell_type]])))

    # Check query-related components if query data is provided
    if (!is.null(query_data)) {
        expect_true(all(c("query_mat_subset", "query_anomaly_scores", "query_anomaly") %in% names(anomaly_output[[first_cell_type]])))
    }
})

test_that("detectAnomaly works correctly without query data", {
    # Store PCA anomaly data without query data
    anomaly_output <- detectAnomaly(
        reference_data = reference_data,
        ref_cell_type_col = "expert_annotation"
    )

    # Check if the output is a list and has correct class
    expect_true(is.list(anomaly_output))
    expect_s3_class(anomaly_output, "detectAnomalyObject")

    # Check if the output contains the expected elements
    expect_true(length(anomaly_output) > 0)
    expect_true("Combined" %in% names(anomaly_output))

    # Check the structure of the elements for the first cell type
    first_cell_type <- names(anomaly_output)[1]
    expect_true(all(c("reference_anomaly_scores", "reference_anomaly", "reference_mat_subset") %in% names(anomaly_output[[first_cell_type]])))

    # Check that query-related components are not present
    expect_false("query_anomaly_scores" %in% names(anomaly_output[[first_cell_type]]))
    expect_false("query_anomaly" %in% names(anomaly_output[[first_cell_type]]))
})

test_that("detectAnomaly handles parameter validation correctly", {
    # Test invalid n_tree
    expect_error(detectAnomaly(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = "SingleR_annotation",
        n_tree = -10
    ), "n_tree.*must be a positive integer")

    expect_error(detectAnomaly(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = "SingleR_annotation",
        n_tree = 0
    ), "n_tree.*must be a positive integer")

    # Test invalid anomaly_threshold
    expect_error(detectAnomaly(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = "SingleR_annotation",
        anomaly_threshold = 1.5
    ), "anomaly_threshold.*must be a positive number.*less than 1")

    expect_error(detectAnomaly(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = "SingleR_annotation",
        anomaly_threshold = 0
    ), "anomaly_threshold.*must be a positive number.*greater than 0")
})

test_that("detectAnomaly works with custom parameters", {
    # Store PCA anomaly data with custom parameters
    anomaly_output <- detectAnomaly(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = "SingleR_annotation",
        cell_types = c("CD4", "CD8"),
        pc_subset = 1:3,
        n_tree = 100,
        anomaly_threshold = 0.7
    )

    # Check if the output is a list and has correct class
    expect_true(is.list(anomaly_output))
    expect_s3_class(anomaly_output, "detectAnomalyObject")

    # Check that it contains the specified cell types plus Combined
    expect_true("CD4" %in% names(anomaly_output))
    expect_true("CD8" %in% names(anomaly_output))
})

test_that("detectAnomaly works without pc_subset (using raw data)", {
    # Test with pc_subset = NULL
    anomaly_output <- detectAnomaly(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = "SingleR_annotation",
        pc_subset = NULL,
        n_tree = 50
    )

    # Check basic structure
    expect_true(is.list(anomaly_output))
    expect_s3_class(anomaly_output, "detectAnomalyObject")

    # When pc_subset is NULL, var_explained should not be present
    first_cell_type <- names(anomaly_output)[1]
    expect_false("var_explained" %in% names(anomaly_output[[first_cell_type]]))
})

#' @title Plot Heatmap of Similarities Between Principal Components
#'
#' @description
#' The S3 plot method generates a heatmap to visualize the similarities between
#' principal components from the output of the \code{comparePCA} function.
#'
#' @details
#' The S3 plot method creates an enhanced heatmap visualization with options to display
#' statistical significance and similarity values. The heatmap uses a blue-white-red
#' color gradient for similarity values, and optionally overlays significance indicators.
#'
#' @param x A \code{comparePCAObject} output from the \code{comparePCA} function.
#' @param show_values Logical, whether to display similarity values on the heatmap. Default is TRUE.
#' @param show_significance Logical, whether to display significance indicators (requires permutation test). Default is TRUE.
#' @param significance_threshold Numeric, p-value threshold for significance. Default is 0.05.
#' @param color_limits Numeric vector of length 2 specifying color scale limits. If NULL, uses data range.
#' @param ... Additional arguments passed to the plotting function.
#'
#' @return A \code{ggplot} object representing the heatmap of similarities.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{comparePCA}}
#'
#' @rdname comparePCA
#'
# Function to plot the comparison of the PCs of the SCE objects
plot.comparePCAObject <- function(x,
                                  show_values = TRUE,
                                  show_significance = TRUE,
                                  significance_threshold = 0.05,
                                  color_limits = NULL,
                                  ...) {

    # Extract similarity matrix from the object
    if (is.list(x) && "similarity_matrix" %in% names(x)) {
        similarity_matrix <- x[["similarity_matrix"]]
        p_values <- x[["p_values"]]
        metric_name <- x[["metric"]]
    } else {
        # Backward compatibility with old format
        similarity_matrix <- x
        p_values <- NULL
        metric_name <- "cosine"
    }

    # Convert the matrix to a data frame
    similarity_df <- data.frame(
        Ref = factor(rep(rownames(similarity_matrix),
                         each = ncol(similarity_matrix)),
                     levels = rev(rownames(similarity_matrix))),
        Query = factor(rep(colnames(similarity_matrix),
                           times = nrow(similarity_matrix)),
                       levels = colnames(similarity_matrix)),
        Similarity = as.vector(similarity_matrix)
    )

    # Add significance information if available
    if (!is.null(p_values) && show_significance) {
        similarity_df[["P_Value"]] <- as.vector(p_values)
        similarity_df[["Significant"]] <- similarity_df[["P_Value"]] <
            significance_threshold
        similarity_df[["Significance_Symbol"]] <-
            ifelse(similarity_df[["Significant"]], "*", "")

        # Create combined labels with values and significance BEFORE ggplot
        if (show_values) {
            similarity_df[["Label"]] <- paste0(
                sprintf("%.2f", similarity_df[["Similarity"]]),
                similarity_df[["Significance_Symbol"]]
            )
        }
    } else {
        similarity_df[["Significant"]] <- FALSE
        similarity_df[["Significance_Symbol"]] <- ""

        # Create simple labels if just showing values
        if (show_values) {
            similarity_df[["Label"]] <-
                sprintf("%.2f", similarity_df[["Similarity"]])
        }
    }

    # Determine color limits
    if (is.null(color_limits)) {
        max_abs <- max(abs(range(similarity_matrix, na.rm = TRUE)))
        color_limits <- c(-max_abs, max_abs)
    }

    # Create appropriate title based on metric
    title_text <- switch(metric_name,
                         "cosine" = "Heatmap of Cosine Similarities Between PCs",
                         "correlation" = "Heatmap of Correlations Between PCs",
                         "Heatmap of Similarities Between PCs")

    # Create the base heatmap
    pc_plot <- ggplot2::ggplot(
        similarity_df,
        ggplot2::aes(x = .data[["Query"]],
                     y = .data[["Ref"]],
                     fill = .data[["Similarity"]])
    ) +
        ggplot2::geom_tile(color = "white", linewidth = 0.5) +
        ggplot2::scale_fill_gradient2(
            low = "blue",
            high = "red",
            mid = "white",
            midpoint = 0,
            limit = color_limits,
            space = "Lab",
            name = paste(stringr::str_to_title(metric_name),
                         "\nSimilarity")
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
            axis.text.x = ggplot2::element_text(angle = 45,
                                                vjust = 1,
                                                size = 10,
                                                hjust = 1),
            axis.text.y = ggplot2::element_text(size = 10),
            plot.title = ggplot2::element_text(hjust = 0.5,
                                               size = 14),
            legend.title = ggplot2::element_text(size = 10),
            panel.grid = ggplot2::element_blank()
        ) +
        ggplot2::labs(
            x = "Query Dataset PCs",
            y = "Reference Dataset PCs",
            title = title_text
        )

    # Add similarity values if requested
    if (show_values) {
        if (!is.null(p_values) && show_significance) {
            # Use the pre-created Label column and add color mapping
            pc_plot <- pc_plot +
                ggplot2::geom_text(
                    ggplot2::aes(label = .data[["Label"]],
                                 color = .data[["Significant"]]),
                    size = 3, fontface = "bold"
                ) +
                ggplot2::scale_color_manual(
                    values = c("TRUE" = "black", "FALSE" = "gray60"),
                    guide = "none"
                )
        } else {
            # Use the simple Label column
            pc_plot <- pc_plot +
                ggplot2::geom_text(
                    ggplot2::aes(label = .data[["Label"]]),
                    size = 3, color = "black"
                )
        }
    }

    # Add significance legend if applicable
    if (!is.null(p_values) && show_significance && show_values) {
        pc_plot <- pc_plot +
            ggplot2::labs(
                caption = paste0("* indicates p < ",
                                 significance_threshold,
                                 " (based on ",
                                 x[["n_permutations"]],
                                 " permutations)")
            ) +
            ggplot2::theme(
                plot.caption = ggplot2::element_text(hjust = 0,
                                                     size = 8,
                                                     color = "gray50")
            )
    }

    return(pc_plot)
}

#' @title Plot Visualization of Output from comparePCASubspace Function
#'
#' @description
#' The S3 plot method generates a visualization of the output from the \code{comparePCASubspace} function.
#' The plot shows the cosine of principal angles between reference and query principal components,
#' with point sizes representing the variance explained and colors showing the difference in variance between datasets.
#'
#' @details
#' The S3 plot method converts the input list into a data frame suitable for plotting with \code{ggplot2}.
#' Each point in the scatter plot represents the cosine of a principal angle, with the size of the point
#' indicating the average variance explained by the corresponding principal components. The color represents
#' the difference in variance explained between reference and query datasets.
#'
#' @param x A numeric matrix output from the \code{comparePCASubspace} function, representing
#' cosine similarities between query and reference principal components.
#' @param ... Additional arguments passed to the plotting function.
#'
#' @keywords internal
#'
#' @return The S3 plot method returns a \code{ggplot} object representing the cosine similarities with variance information.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{comparePCASubspace}}
#'
#' @rdname comparePCASubspace
#'
# Function to produce the visualization of output from comparePCASubspace function
plot.comparePCASubspaceObject <- function(x,
                                          ...){

    # Create a data frame for plotting
    plot_data <- data.frame(
        PC = paste0("Ref PC", x[["cosine_id"]][, 1], " - Query PC", x[["cosine_id"]][, 2]),
        PC_Label = paste0("Ref PC", x[["cosine_id"]][, 1],
                          " (", round(x[["var_explained_ref"]], 1), "%) - Query PC",
                          x[["cosine_id"]][, 2], " (", round(x[["var_explained_query"]], 1), "%)"),
        Cosine = x[["cosine_similarity"]],
        VarianceExplained = x[["var_explained_avg"]],
        RefVariance = x[["var_explained_ref"]],
        QueryVariance = x[["var_explained_query"]],
        VarianceDiff = x[["var_explained_ref"]] - x[["var_explained_query"]]
    )
    plot_data[["PC_Label"]] <- factor(plot_data[["PC_Label"]], levels = plot_data[["PC_Label"]])

    # Create plot
    pc_plot <- ggplot2::ggplot(
        plot_data, ggplot2::aes(x = .data[["PC_Label"]],
                                y = .data[["Cosine"]],
                                size = .data[["VarianceExplained"]],
                                color = .data[["VarianceDiff"]])) +
        ggplot2::geom_point(alpha = 0.8, stroke = 1) +
        ggplot2::scale_size_continuous(range = c(3, 10), name = "Avg Variance\nExplained (%)") +
        ggplot2::scale_color_gradient2(
            low = "blue", mid = "gray", high = "red", midpoint = 0,
            name = "Variance Diff\n(Ref - Query)"
        ) +
        ggplot2::labs(
            title = "Principal Angles Cosines with Variance Explained",
            subtitle = paste0("Weighted Cosine Similarity: ", sprintf("%.3f", x[["weighted_cosine_similarity"]])),
            x = "",
            y = "Cosine Similarity of Principal Angle"
        ) +
        ggplot2::theme_bw() +
        ggplot2::theme(
            axis.text.x = ggplot2::element_text(angle = 45, vjust = 1,
                                                size = 10, hjust = 1),
            axis.title = ggplot2::element_text(size = 12),
            axis.text = ggplot2::element_text(size = 10),
            panel.grid.minor = ggplot2::element_blank(),
            panel.grid.major = ggplot2::element_line(color = "gray",
                                                     linetype = "dotted"),
            plot.title = ggplot2::element_text(hjust = 0.5),
            plot.subtitle = ggplot2::element_text(hjust = 0.5)
        )
    return(pc_plot)
}

#' @title Argument Validation for SingleCellExperiment Analysis
#'
#' @description
#' This function validates the input arguments for functions that analyze \code{\linkS4class{SingleCellExperiment}}
#' objects. It checks that the inputs are of the correct types and formats, and that required columns and cell types
#' are present in the data.
#'
#' @details
#' The function performs a series of checks to ensure that:
#' \itemize{
#'  \item `query_data` and `reference_data` are \code{\linkS4class{SingleCellExperiment}} objects.
#'  \item `query_cell_type_col` and `ref_cell_type_col` exist in the column data of their respective \code{\linkS4class{SingleCellExperiment}} objects.
#'  \item If `unique_cell_type` is `TRUE`, there should only be one cell type in the \code{\linkS4class{SingleCellExperiment}} objects.
#'  \item `cell_names_query` are valid cell names in the provided query dataset.
#'  \item `cell_names_ref` are valid cell names in the provided reference dataset.
#'  \item The PCA subsets specified by `pc_subset_query` and `pc_subset_ref` are valid.
#'  \item `max_cells_ref` and `max_cells_query` are positive integers when not NULL.
#' }
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' If `NULL`, no check is performed.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' If `NULL`, no check is performed.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data}
#' that identifies the cell types. If `NULL`, no check is performed.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data}
#' that identifies the cell types. If `NULL`, no check is performed.
#' @param unique_cell_type If `TRUE`, there should only be one cell type in the provided \code{\linkS4class{SingleCellExperiment}} objects.
#' Default is `FALSE`.
#' @param plot_function A logical value indicating whether the function is being called to generate a plot. Default is `FALSE`.
#' @param cell_names_query A character vector of cell names in query data to be analyzed. If `NULL`, no check is performed.
#' @param cell_names_ref A character vector of cell names in reference data to be analyzed. If `NULL`, no check is performed.
#' @param pc_subset_query A numeric vector specifying the principal components to be used for the query data. If `NULL`, no check is performed.
#' @param pc_subset_ref A numeric vector specifying the principal components to be used for the reference data. If `NULL`, no check is performed.
#' @param common_rotation_genes If TRUE, check the rotation matrices of the reference and query data and ensure they have the same genes.
#' Default is FALSE.
#' @param assay_name Name of the assay on which to perform computations. If `NULL`, no check is performed.
#' @param max_cells_ref Maximum number of reference cells to retain. If `NULL`, no check is performed.
#' @param max_cells_query Maximum number of query cells to retain. If `NULL`, no check is performed.
#'
#' @keywords internal
#'
#' @return None.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
# Function to check standard arguments for functions in the package
argumentCheck <- function(query_data = NULL,
                          reference_data = NULL,
                          query_cell_type_col = NULL,
                          ref_cell_type_col = NULL,
                          unique_cell_type = FALSE,
                          plot_function = FALSE,
                          cell_names_query = NULL,
                          cell_names_ref = NULL,
                          pc_subset_query = NULL,
                          pc_subset_ref = NULL,
                          common_rotation_genes = FALSE,
                          assay_name = NULL,
                          max_cells_ref = NULL,
                          max_cells_query = NULL) {

    # Check if query_data is a SingleCellExperiment object
    if (!is.null(query_data)) {

        if (!is(query_data, "SingleCellExperiment")) {
            stop("'query_data' must be a SingleCellExperiment object.")
        }

        if(!is.null(assay_name) && !(assay_name %in% SummarizedExperiment::assayNames(query_data)))
            stop("'query_data' does not contain the specified assay.")
    }

    # Check if reference_data is a SingleCellExperiment object
    if (!is.null(reference_data)) {

        if (!is(reference_data, "SingleCellExperiment")) {
            stop("'reference_data' must be a SingleCellExperiment object.")
        }

        if(!is.null(assay_name) && !(assay_name %in% SummarizedExperiment::assayNames(reference_data)))
            stop("'reference_data' does not contain the specified assay.")
    }

    # Check if query_cell_type_col is a character string of length 1 and exists in query_data
    if (!is.null(query_cell_type_col)) {

        if (!is.null(query_data)) {
            if (!is.character(query_cell_type_col) ||
                length(query_cell_type_col) != 1) {
                stop("'query_cell_type_col' must be a character string of length 1")
            }

            if (!(query_cell_type_col %in% names(colData(query_data)))) {
                stop("'query_cell_type_col' is not an existing column of 'query_data'.")
            }
        }
    }

    # Check if ref_cell_type_col is a character string of length 1 and exists in reference_data
    if (!is.null(ref_cell_type_col)) {

        if (!is.null(reference_data)) {
            if (!is.character(ref_cell_type_col) ||
                length(ref_cell_type_col) != 1) {
                stop("'ref_cell_type_col' must be a character string of length 1")
            }

            if (!(ref_cell_type_col %in% names(colData(reference_data)))) {
                stop("'ref_cell_type_col' is not an existing column of 'reference_data'.")
            }
        }
    }

    # Check that the SingleCellExperiment object(s) have a unique cell type
    if (isTRUE(unique_cell_type)) {

        if (!is.null(query_data)) {
            if (length(unique(query_data[[query_cell_type_col]])) > 1) {
                stop("This function should be used when there is only one cell type in 'query_data'.")
            }
        }

        if (!is.null(reference_data)) {

            if (length(unique(reference_data[[ref_cell_type_col]])) > 1) {
                stop("This function should be used when there is only one cell type in 'reference_data'.")
            }
        }

        if (!is.null(reference_data) && !is.null(query_data)) {
            if (unique(query_data[[query_cell_type_col]]) != unique(reference_data[[ref_cell_type_col]])) {
                stop("The cell type of the query data does not match the cell type of the reference data.")
            }
        }
    }

    # Check cell_names contain valid cell names in query_data
    if (!is.null(cell_names_query)) {

        if (!all(cell_names_query %in% colnames(query_data))) {
            stop("'cell_names' contains one or more cells that are not available in 'query_data'.")
        }
    }

    # Check cell_names contain valid cell names in reference_data
    if (!is.null(cell_names_ref)) {

        if (!all(cell_names_ref %in% colnames(reference_data))) {
            stop("'cell_names' contains one or more cells that are not available in 'reference_data'.")
        }
    }

    # Check PC subset for query_data
    if (!is.null(pc_subset_query)) {

        # Check if "PCA" is present in query's reduced dimensions
        if (!"PCA" %in% names(reducedDims(query_data))) {
            stop("'query_data' must have pre-computed PCA in 'reducedDims'. ",
                 "Use processPCA() to compute PCA: ",
                 "query_data <- processPCA(query_data = query_data)")
        }

        # Check input if PC subset is valid
        if (!all(pc_subset_query %in% seq_len(ncol(reducedDim(query_data, "PCA"))))) {
            stop("'pc_subset' is out of range for 'query_data'.")
        }
    }

    # Check PC subset for reference_data
    if (!is.null(pc_subset_ref)) {

        # Check if "PCA" is present in reference's reduced dimensions
        if (!"PCA" %in% names(reducedDims(reference_data))) {
            stop("'reference_data' must have pre-computed PCA in 'reducedDims'. ",
                 "Use processPCA() to compute PCA: ",
                 "reference_data <- processPCA(reference_data = reference_data)")
        }

        # Check input if PC subset is valid
        if (!all(pc_subset_ref %in% seq_len(ncol(reducedDim(reference_data, "PCA"))))) {
            stop("'pc_subset' is out of range for 'reference_data'.")
        }
    }

    # Check if the rotation matrices have the same genes in the same order
    if (common_rotation_genes == TRUE) {

        # Check if both datasets have PCA before comparing rotation matrices
        if (!"PCA" %in% names(reducedDims(query_data))) {
            stop("'query_data' must have pre-computed PCA in 'reducedDims' for rotation matrix comparison. ",
                 "Use processPCA() to compute PCA for both datasets: ",
                 "result <- processPCA(query_data = query_data, reference_data = reference_data)")
        }

        if (!"PCA" %in% names(reducedDims(reference_data))) {
            stop("'reference_data' must have pre-computed PCA in 'reducedDims' for rotation matrix comparison. ",
                 "Use processPCA() to compute PCA for both datasets: ",
                 "result <- processPCA(query_data = query_data, reference_data = reference_data)")
        }

        # Check if the rotation matrices have the same number of genes
        if (ncol(attributes(reducedDim(query_data, "PCA"))[["rotation"]]) !=
            ncol(attributes(reducedDim(reference_data, "PCA"))[["rotation"]])) {
            stop("The number of genes in the rotation matrices differ.")
        }

        # Check if genes in both rotation matrices are the same
        if (!all(rownames(attributes(reducedDim(query_data, "PCA"))[["rotation"]]) %in%
                 rownames(attributes(reducedDim(reference_data, "PCA"))[["rotation"]]))) {
            stop("The genes in the rotation matrices differ.")
        }
    }

    # Check max_cells_ref parameter
    if (!is.null(max_cells_ref)) {
        # Check if max_cells_ref is a positive integer
        if (!is.numeric(max_cells_ref) || max_cells_ref <= 0 || max_cells_ref != as.integer(max_cells_ref)) {
            stop("'max_cells_ref' must be a positive integer.")
        }

        # Warning for plot functions when max_cells_ref > 50000 and reference_data exists
        if (plot_function && !is.null(reference_data) && max_cells_ref > 50000) {
            warning("'max_cells_ref' is set to ", max_cells_ref,
                    " which is greater than 50,000. For better plot performance, ",
                    "consider using a smaller value for 'max_cells_ref' to downsample the reference data.",
                    call. = FALSE)
        }
    }

    # Check max_cells_query parameter
    if (!is.null(max_cells_query)) {
        # Check if max_cells_query is a positive integer
        if (!is.numeric(max_cells_query) || max_cells_query <= 0 || max_cells_query != as.integer(max_cells_query)) {
            stop("'max_cells_query' must be a positive integer.")
        }

        # Warning for plot functions when max_cells_query > 50000 and query_data exists
        if (plot_function && !is.null(query_data) && max_cells_query > 50000) {
            warning("'max_cells_query' is set to ", max_cells_query,
                    " which is greater than 50,000. For better plot performance, ",
                    "consider using a smaller value for 'max_cells_query' to downsample the query data.",
                    call. = FALSE)
        }
    }
}

#' @title Calculate Graph Community Integration Diagnostics
#'
#' @description
#' This function performs graph-based community detection to identify annotation inconsistencies
#' by detecting query-only communities, true cross-cell-type mixing patterns, and local
#' annotation inconsistencies based on immediate neighborhood analysis.
#'
#' @details
#' The function performs three types of analysis: (1) Communities containing only query cells,
#' (2) Communities where query cells are mixed with reference cells of different cell types
#' WITHOUT any reference cells of the same type, and (3) Local analysis of each query cell's
#' immediate neighbors to detect annotation inconsistencies even within mixed communities.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col A character string specifying the column name in the query dataset containing cell type annotations.
#' @param ref_cell_type_col A character string specifying the column name in the reference dataset containing cell type annotations.
#' @param cell_types A character vector specifying the cell types to include in the analysis. If NULL, all cell types are included.
#' @param pc_subset A vector specifying the subset of principal components to use in the analysis. Default is 1:10.
#' @param k_neighbors An integer specifying the number of nearest neighbors for graph construction. Default is 30.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param resolution Resolution parameter for Leiden clustering. Default is 0.15 for fewer, larger communities.
#' @param min_cells_per_community Minimum number of cells required for a community to be analyzed. Default is 10.
#' @param min_cells_per_celltype Minimum number of cells required per cell type for inclusion. Default is 20.
#' @param high_query_prop_threshold Minimum proportion of query cells to consider a community "query-only". Default is 0.9.
#' @param cross_type_threshold Minimum proportion needed to flag cross-cell-type mixing. Default is 0.1.
#' @param local_consistency_threshold Minimum proportion of reference neighbors that should support a query cell's annotation. Default is 0.6.
#' @param local_confidence_threshold Minimum confidence difference needed to suggest re-annotation. Default is 0.2.
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#'
#' @return A list containing:
#' \item{high_query_prop_analysis}{Analysis of communities with only query cells}
#' \item{cross_type_mixing}{Analysis of communities with true query-reference cross-cell-type mixing}
#' \item{local_annotation_inconsistencies}{Local neighborhood-based annotation inconsistencies}
#' \item{local_inconsistency_summary}{Summary of local inconsistencies by cell type}
#' \item{community_composition}{Detailed composition of each community}
#' \item{annotation_consistency}{Summary of annotation consistency issues}
#' \item{overall_metrics}{Overall diagnostic metrics}
#' \item{graph_info}{Graph structure information for plotting}
#' \item{parameters}{Analysis parameters used}
#' The list is assigned the class \code{"calculateGraphIntegration"}.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Remove a cell type (Myeloid)
#' library(scater)
#' library(SingleR)
#' reference_data <- reference_data[, reference_data$expert_annotation != "Myeloid"]
#' reference_data <- runPCA(reference_data, ncomponents = 50)
#' SingleR_annotation <- SingleR(query_data, reference_data,
#'                               labels = reference_data$expert_annotation)
#' query_data$SingleR_annotation <- SingleR_annotation$labels
#'
#' # Check annotation data
#' table(Expert = query_data$expert_annotation, SingleR = query_data$SingleR_annotation)
#'
#' # Run comprehensive annotation consistency diagnostics
#' graph_diagnostics <- calculateGraphIntegration(
#'     query_data = query_data,
#'     reference_data = reference_data,
#'     query_cell_type_col = "SingleR_annotation",
#'     ref_cell_type_col = "expert_annotation",
#'    pc_subset = 1:10,
#'    k_neighbors = 30,
#'     resolution = 0.1,
#'     high_query_prop_threshold = 0.9,
#'     cross_type_threshold = 0.15,
#'     local_consistency_threshold = 0.6,
#'     local_confidence_threshold = 0.2
#' )
#'
#' # Look at main output
#' graph_diagnostics$overall_metrics
#'
#' # Network graph showing all issue types (color by cell type)
#' plot(graph_diagnostics, plot_type = "community_network", color_by = "cell_type")
#'
#' # Network graph showing all issue types
#' plot(graph_diagnostics, plot_type = "cell_network",
#'      max_nodes = 2000, color_by = "community_type")
#'
#' # Network graph showing all issue types
#' plot(graph_diagnostics, plot_type = "community_data")
#'
#' # Summary bar chart of all issues by cell type
#' plot(graph_diagnostics, plot_type = "summary")
#'
#' # Focus on local annotation inconsistencies
#' plot(graph_diagnostics, plot_type = "local_issues")
#'
#' # Overall annotation issues overview
#' plot(graph_diagnostics, plot_type = "annotation_issues")
#'
#' @importFrom stats dist
#'
# Function to calculate graph integration diagnostics
calculateGraphIntegration <- function(query_data,
                                      reference_data,
                                      query_cell_type_col,
                                      ref_cell_type_col,
                                      cell_types = NULL,
                                      pc_subset = 1:10,
                                      k_neighbors = 30,
                                      assay_name = "logcounts",
                                      resolution = 0.1,
                                      min_cells_per_community = 10,
                                      min_cells_per_celltype = 20,
                                      high_query_prop_threshold = 0.9,
                                      cross_type_threshold = 0.15,
                                      local_consistency_threshold = 0.6,
                                      local_confidence_threshold = 0.2,
                                      max_cells_query = 5000,
                                      max_cells_ref = 5000) {

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Check additional parameters
    if (!is.numeric(k_neighbors) || k_neighbors <= 0 ||
        k_neighbors != as.integer(k_neighbors)) {
        stop("\'k_neighbors\' must be a positive integer.")
    }

    if (!is.numeric(resolution) || resolution <= 0) {
        stop("\'resolution\' must be a positive number.")
    }

    if (!is.numeric(high_query_prop_threshold) || high_query_prop_threshold <= 0.5 ||
        high_query_prop_threshold > 1) {
        stop("\'high_query_prop_threshold\' must be between 0.5 and 1.")
    }

    if (!is.numeric(cross_type_threshold) || cross_type_threshold <= 0 ||
        cross_type_threshold >= 0.5) {
        stop("\'cross_type_threshold\' must be between 0 and 0.5.")
    }

    if (!is.numeric(local_consistency_threshold) || local_consistency_threshold <= 0 ||
        local_consistency_threshold > 1) {
        stop("\'local_consistency_threshold\' must be between 0 and 1.")
    }

    if (!is.numeric(local_confidence_threshold) || local_confidence_threshold <= 0 ||
        local_confidence_threshold > 1) {
        stop("\'local_confidence_threshold\' must be between 0 and 1.")
    }

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = TRUE,
                                  n_cell_types = NULL)

    # Filter cell types by minimum cell count
    ref_counts <- table(reference_data[[ref_cell_type_col]])
    query_counts <- table(query_data[[query_cell_type_col]])

    valid_cell_types <- intersect(
        names(ref_counts)[ref_counts >= min_cells_per_celltype],
        names(query_counts)[query_counts >= min_cells_per_celltype]
    )

    cell_types <- intersect(cell_types, valid_cell_types)

    if (length(cell_types) == 0) {
        stop("No cell types meet the minimum cell count requirement.")
    }

    # Get PCA projection
    pca_output <- projectPCA(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = query_cell_type_col,
                             ref_cell_type_col = ref_cell_type_col,
                             cell_types = cell_types,
                             pc_subset = pc_subset,
                             assay_name = assay_name,
                             max_cells_ref = max_cells_ref,
                             max_cells_query = max_cells_query)

    # Extract PCA coordinates
    pc_coords <- as.matrix(pca_output[, paste0("PC", pc_subset)])

    # Construct k-nearest neighbor graph
    .constructKNNGraph <- function(coords, k) {
        knn_result <- FNN::get.knn(coords, k = k)
        edges <- do.call(rbind, lapply(seq_len(nrow(coords)), function(i) {
            neighbors <- knn_result[["nn.index"]][i, ]
            cbind(i, neighbors)
        }))
        graph <- igraph::graph_from_edgelist(edges, directed = FALSE)
        graph <- igraph::simplify(graph, remove.multiple = TRUE, remove.loops = TRUE)
        return(list(graph = graph, edges = edges))
    }

    # Perform Leiden clustering
    .performLeidenClustering <- function(graph, resolution) {
        clustering <- igraph::cluster_leiden(graph, resolution = resolution)
        membership_vec <- igraph::membership(clustering)
        mod_value <- igraph::modularity(graph, membership_vec)
        return(list(membership = membership_vec, modularity = mod_value))
    }

    # Main analysis function for annotation consistency
    .analyzeAnnotationConsistency <- function(membership, cell_info,
                                              query_threshold, cross_threshold) {
        communities <- unique(membership)

        # Initialize results
        high_query_prop_communities <- list()
        cross_type_communities <- list()
        all_communities <- list()

        for (comm in communities) {
            comm_cells <- which(membership == comm)
            comm_info <- cell_info[comm_cells, ]

            # Skip small communities
            if (nrow(comm_info) < min_cells_per_community) next

            # Basic counts
            dataset_counts <- table(comm_info[["dataset"]])
            celltype_counts <- table(comm_info[["cell_type"]])

            total_cells <- nrow(comm_info)
            n_reference <- ifelse("Reference" %in% names(dataset_counts),
                                  as.numeric(dataset_counts[["Reference"]]), 0)
            n_query <- ifelse("Query" %in% names(dataset_counts),
                              as.numeric(dataset_counts[["Query"]]), 0)

            query_prop <- n_query / total_cells

            # Community summary
            comm_summary <- list(
                community = comm,
                total_cells = total_cells,
                n_reference = n_reference,
                n_query = n_query,
                query_proportion = query_prop,
                cell_types = names(celltype_counts),
                cell_type_counts = as.list(celltype_counts),
                dominant_celltype = names(celltype_counts)[which.max(celltype_counts)]
            )

            all_communities[[length(all_communities) + 1]] <- comm_summary

            # Check for query-only communities
            if (query_prop >= query_threshold) {
                query_analysis <- list(
                    community = comm,
                    total_query_cells = n_query,
                    query_cell_types = names(
                        table(comm_info[["cell_type"]][comm_info[["dataset"]] == "Query"])),
                    query_celltype_counts = table(
                        comm_info[["cell_type"]][comm_info[["dataset"]] == "Query"]),
                    is_homogeneous = length(unique(comm_info[["cell_type"]])) == 1,
                    mixed_types = if(length(unique(comm_info[["cell_type"]])) > 1) {
                        paste(names(table(comm_info[["cell_type"]])), collapse = ", ")
                    } else { NA }
                )
                high_query_prop_communities[[length(high_query_prop_communities) + 1]] <-
                    query_analysis
            }

            # Check for TRUE cross-cell-type mixing only
            if (n_reference > 0 && n_query > 0) {
                # Get cell types for each dataset
                ref_celltypes <- table(
                    comm_info[["cell_type"]][comm_info[["dataset"]] == "Reference"])
                query_celltypes <- table(
                    comm_info[["cell_type"]][comm_info[["dataset"]] == "Query"])

                ref_types <- names(ref_celltypes)
                query_types <- names(query_celltypes)

                # Only flag TRUE mismatches: query cells with NO matching reference type
                cross_mixing_detected <- FALSE
                cross_details <- list()

                for (qt in query_types) {
                    # Check if this query type has NO corresponding reference cells
                    if (!(qt %in% ref_types)) {
                        # This query type is ONLY mixing with different reference types
                        query_count <- as.numeric(query_celltypes[[qt]])
                        query_prop_in_community <- query_count / n_query

                        # Only flag if significant proportion
                        if (query_prop_in_community >= cross_threshold) {
                            cross_mixing_detected <- TRUE

                            # Find which reference types this query type is mixing with
                            ref_mixing_with <- ref_types
                            ref_counts_mixing <- ref_celltypes[ref_mixing_with]

                            cross_details[[paste("Query", qt, "with Ref",
                                                 paste(ref_mixing_with,
                                                       collapse = "+"))]] <- list(
                                                           query_type = qt,
                                                           reference_types_mixing_with = ref_mixing_with,
                                                           reference_counts_mixing_with = ref_counts_mixing,
                                                           query_cells = query_count,
                                                           query_proportion = query_prop_in_community,
                                                           is_true_mismatch = TRUE
                                                       )
                        }
                    }
                }

                if (cross_mixing_detected) {
                    cross_analysis <- list(
                        community = comm,
                        total_cells = total_cells,
                        n_query = n_query,
                        n_reference = n_reference,
                        cross_mixing_details = cross_details,
                        query_types_with_no_ref_match = setdiff(query_types,
                                                                ref_types),
                        reference_types_in_community = ref_types,
                        query_types_in_community = query_types
                    )
                    cross_type_communities[[length(cross_type_communities) + 1]] <-
                        cross_analysis
                }
            }
        }

        return(list(
            high_query_prop = high_query_prop_communities,
            cross_type = cross_type_communities,
            all_communities = all_communities
        ))
    }

    # NEW: Local annotation consistency analysis
    .analyzeLocalAnnotationConsistency <- function(
        membership, cell_info, knn_edges,
        local_threshold, confidence_threshold) {
        # For each query cell, check if its immediate neighbors support its annotation
        query_cells <- which(cell_info[["dataset"]] == "Query")
        local_inconsistencies <- list()

        for (query_idx in query_cells) {
            query_type <- cell_info[["cell_type"]][query_idx]
            query_community <- cell_info[["community"]][query_idx]

            # Find this query cell's direct neighbors from KNN graph
            neighbors_from <- knn_edges[knn_edges[, 1] == query_idx, 2]
            neighbors_to <- knn_edges[knn_edges[, 2] == query_idx, 1]
            all_neighbors <- unique(c(neighbors_from, neighbors_to))

            if (length(all_neighbors) == 0) next

            # Get neighbor information
            neighbor_info <- cell_info[all_neighbors, ]

            # Focus on reference neighbors only
            ref_neighbors <- neighbor_info[neighbor_info[["dataset"]] == "Reference", ]

            if (nrow(ref_neighbors) == 0) next

            # Calculate support for query cell's annotation among reference neighbors
            ref_neighbor_types <- table(ref_neighbors[["cell_type"]])
            total_ref_neighbors <- nrow(ref_neighbors)

            # Support for query's annotation
            support_for_query_type <- ifelse(query_type %in% names(ref_neighbor_types),
                                             as.numeric(ref_neighbor_types[[query_type]]),
                                             0)
            support_proportion <- support_for_query_type / total_ref_neighbors

            # Flag as inconsistent if support is below threshold
            if (support_proportion < local_threshold) {
                # Find the most supported type among reference neighbors
                most_supported_type <- names(ref_neighbor_types)[which.max(ref_neighbor_types)]
                most_supported_count <- max(ref_neighbor_types)
                most_supported_proportion <- most_supported_count / total_ref_neighbors

                # Only flag if there's a clear alternative with stronger support
                if (most_supported_proportion > support_proportion + confidence_threshold) {
                    inconsistency <- list(
                        query_cell_idx = query_idx,
                        query_annotation = query_type,
                        suggested_annotation = most_supported_type,
                        community = query_community,
                        ref_neighbors_total = total_ref_neighbors,
                        support_for_query = support_for_query_type,
                        support_proportion = support_proportion,
                        suggested_support = most_supported_count,
                        suggested_proportion = most_supported_proportion,
                        confidence_score = most_supported_proportion - support_proportion
                    )
                    local_inconsistencies[[length(local_inconsistencies) + 1]] <- inconsistency
                }
            }
        }

        return(local_inconsistencies)
    }

    # Build graph and perform clustering
    if (k_neighbors >= nrow(pc_coords)) {
        k_neighbors <- nrow(pc_coords) - 1
        warning(paste("k_neighbors reduced to", k_neighbors, "due to insufficient cells"))
    }

    graph_result <- .constructKNNGraph(pc_coords, k_neighbors)
    knn_graph <- graph_result[["graph"]]
    clustering_result <- .performLeidenClustering(knn_graph, resolution)

    # Prepare cell information
    cell_info <- data.frame(
        cell_id = seq_len(nrow(pca_output)),
        dataset = pca_output[["dataset"]],
        cell_type = pca_output[["cell_type"]],
        community = clustering_result[["membership"]],
        stringsAsFactors = FALSE
    )

    # Perform main community analysis
    analysis_results <- .analyzeAnnotationConsistency(
        clustering_result[["membership"]],
        cell_info,
        high_query_prop_threshold,
        cross_type_threshold
    )

    # Perform local annotation consistency analysis
    local_inconsistencies <- .analyzeLocalAnnotationConsistency(
        clustering_result[["membership"]],
        cell_info,
        graph_result[["edges"]],
        local_consistency_threshold,
        local_confidence_threshold
    )

    # Convert results to data frames for easier handling
    if (length(analysis_results[["high_query_prop"]]) > 0) {
        high_query_prop_df <- do.call(rbind, lapply(analysis_results[["high_query_prop"]], function(x) {
            data.frame(
                community = x[["community"]],
                total_query_cells = x[["total_query_cells"]],
                query_cell_types = paste(x[["query_cell_types"]], collapse = ", "),
                n_query_cell_types = length(x[["query_cell_types"]]),
                is_homogeneous = x[["is_homogeneous"]],
                mixed_types = ifelse(is.na(x[["mixed_types"]]), NA, x[["mixed_types"]]),
                dominant_query_type = names(x[["query_celltype_counts"]])[which.max(x[["query_celltype_counts"]])],
                stringsAsFactors = FALSE
            )
        }))
    } else {
        high_query_prop_df <- data.frame(
            community = integer(0), total_query_cells = integer(0),
            query_cell_types = character(0), n_query_cell_types = integer(0),
            is_homogeneous = logical(0), mixed_types = character(0),
            dominant_query_type = character(0)
        )
    }

    if (length(analysis_results[["cross_type"]]) > 0) {
        cross_type_df <- do.call(rbind, lapply(analysis_results[["cross_type"]], function(x) {
            # Summarize cross-mixing details - only TRUE mismatches
            cross_summary <- sapply(
                x[["cross_mixing_details"]], function(detail) {
                    ref_summary <- paste(
                        names(detail[["reference_counts_mixing_with"]]),
                        paste0("(", detail[["reference_counts_mixing_with"]], ")"),
                        collapse = "+")
                    paste0("Query ", detail[["query_type"]], " (",
                           detail[["query_cells"]], ") with Ref ",
                           ref_summary)
                })

            data.frame(
                community = x[["community"]],
                total_cells = x[["total_cells"]],
                n_query = x[["n_query"]],
                n_reference = x[["n_reference"]],
                query_types_no_ref_match = paste(x[["query_types_with_no_ref_match"]],
                                                 collapse = ", "),
                reference_types_available = paste(x[["reference_types_in_community"]],
                                                  collapse = ", "),
                cross_mixing_summary = paste(cross_summary, collapse = "; "),
                n_true_mismatches = length(x[["cross_mixing_details"]]),
                stringsAsFactors = FALSE
            )
        }))
    } else {
        cross_type_df <- data.frame(
            community = integer(0),
            total_cells = integer(0),
            n_query = integer(0),
            n_reference = integer(0),
            query_types_no_ref_match = character(0),
            reference_types_available = character(0),
            cross_mixing_summary = character(0),
            n_true_mismatches = integer(0)
        )
    }

    # Convert local inconsistencies to data frame
    if (length(local_inconsistencies) > 0) {
        local_inconsistency_df <- do.call(rbind, lapply(local_inconsistencies, function(x) {
            data.frame(
                query_cell_idx = x[["query_cell_idx"]],
                current_annotation = x[["query_annotation"]],
                suggested_annotation = x[["suggested_annotation"]],
                community = x[["community"]],
                ref_neighbors_total = x[["ref_neighbors_total"]],
                current_support = x[["support_for_query"]],
                current_support_prop = x[["support_proportion"]],
                suggested_support = x[["suggested_support"]],
                suggested_support_prop = x[["suggested_proportion"]],
                confidence_score = x[["confidence_score"]],
                stringsAsFactors = FALSE
            )
        }))

        # Add cell type information
        local_inconsistency_df[["cell_type"]] <-
            cell_info[["cell_type"]][local_inconsistency_df[["query_cell_idx"]]]
    } else {
        local_inconsistency_df <- data.frame(
            query_cell_idx = integer(0), current_annotation = character(0),
            suggested_annotation = character(0), community = integer(0),
            ref_neighbors_total = integer(0), current_support = integer(0),
            current_support_prop = numeric(0), suggested_support = integer(0),
            suggested_support_prop = numeric(0), confidence_score = numeric(0),
            cell_type = character(0)
        )
    }

    # Create community composition summary
    community_composition <-
        do.call(rbind, lapply(analysis_results[["all_communities"]], function(x) {
            data.frame(
                community = x[["community"]],
                total_cells = x[["total_cells"]],
                n_reference = x[["n_reference"]],
                n_query = x[["n_query"]],
                query_proportion = x[["query_proportion"]],
                cell_types = paste(x[["cell_types"]], collapse = ", "),
                n_cell_types = length(x[["cell_types"]]),
                dominant_celltype = x[["dominant_celltype"]],
                is_high_query_prop = x[["query_proportion"]] >= high_query_prop_threshold,
                has_true_cross_mixing = x[["community"]] %in% cross_type_df[["community"]],
                stringsAsFactors = FALSE
            )
        }))

    # Calculate annotation consistency summary
    annotation_summary <- data.frame(
        cell_type = cell_types,
        total_query_cells = sapply(
            cell_types, function(ct) sum(cell_info[["cell_type"]] == ct &
                                             cell_info[["dataset"]] == "Query")),
        high_query_prop_cells = sapply(cell_types, function(ct) {
            if (nrow(high_query_prop_df) == 0) return(0)
            sum(sapply(seq_len(nrow(high_query_prop_df)), function(i) {
                comm_id <- high_query_prop_df[["community"]][i]
                comm_cells <- cell_info[cell_info[["community"]] == comm_id, ]
                sum(comm_cells[["cell_type"]] == ct & comm_cells[["dataset"]] == "Query")
            }))
        }),
        true_cross_mixing_cells = sapply(cell_types, function(ct) {
            if (nrow(cross_type_df) == 0) return(0)
            sum(sapply(seq_len(nrow(cross_type_df)), function(i) {
                comm_id <- cross_type_df[["community"]][i]
                comm_cells <- cell_info[cell_info[["community"]] == comm_id, ]
                sum(comm_cells[["cell_type"]] == ct & comm_cells[["dataset"]] == "Query")
            }))
        }),
        stringsAsFactors = FALSE
    )

    annotation_summary[["query_isolation_rate"]] <-
        annotation_summary[["high_query_prop_cells"]] /
        annotation_summary[["total_query_cells"]]
    annotation_summary[["true_cross_mixing_rate"]] <-
        annotation_summary[["true_cross_mixing_cells"]] /
        annotation_summary[["total_query_cells"]]

    # Replace NaN with 0
    annotation_summary[["query_isolation_rate"]][
        is.nan(annotation_summary[["query_isolation_rate"]])] <- 0
    annotation_summary[["true_cross_mixing_rate"]][
        is.nan(annotation_summary[["true_cross_mixing_rate"]])] <- 0

    # Add local inconsistency summary by cell type
    local_inconsistency_summary <- data.frame(
        cell_type = cell_types,
        total_query_cells = sapply(cell_types, function(ct) sum(cell_info[["cell_type"]] == ct &
                                                                    cell_info[["dataset"]] == "Query")),
        locally_inconsistent_cells = sapply(cell_types, function(ct) {
            sum(local_inconsistency_df[["current_annotation"]] == ct)
        }),
        stringsAsFactors = FALSE
    )
    local_inconsistency_summary[["local_inconsistency_rate"]] <-
        local_inconsistency_summary[["locally_inconsistent_cells"]] /
        local_inconsistency_summary[["total_query_cells"]]
    local_inconsistency_summary[["local_inconsistency_rate"]][
        is.nan(local_inconsistency_summary[["local_inconsistency_rate"]])] <- 0

    # Calculate layout for graph plotting
    graph_layout <- igraph::layout_with_fr(knn_graph, dim = 2)
    graph_info <- list(layout = graph_layout, edges = graph_result[["edges"]],
                       n_nodes = nrow(pc_coords))

    # Overall metrics
    overall_metrics <- list(
        total_communities = nrow(community_composition),
        high_query_prop_communities = nrow(high_query_prop_df),
        true_cross_type_communities = nrow(cross_type_df),
        total_high_query_prop_cells = sum(high_query_prop_df[["total_query_cells"]]),
        total_true_cross_mixing_cells = sum(cross_type_df[["n_query"]]),
        total_locally_inconsistent_cells = nrow(local_inconsistency_df),
        modularity = clustering_result[["modularity"]],
        mean_query_isolation_rate = mean(annotation_summary[["query_isolation_rate"]]),
        mean_true_cross_mixing_rate = mean(annotation_summary[["true_cross_mixing_rate"]]),
        mean_local_inconsistency_rate = mean(local_inconsistency_summary[["local_inconsistency_rate"]])
    )

    # Parameters
    parameters <- list(
        pc_subset = pc_subset,
        k_neighbors = k_neighbors,
        resolution = resolution,
        min_cells_per_community = min_cells_per_community,
        min_cells_per_celltype = min_cells_per_celltype,
        high_query_prop_threshold = high_query_prop_threshold,
        cross_type_threshold = cross_type_threshold,
        local_consistency_threshold = local_consistency_threshold,
        local_confidence_threshold = local_confidence_threshold,
        cell_types_analyzed = cell_types
    )

    # Create output
    result <- list(
        high_query_prop_analysis = high_query_prop_df,
        cross_type_mixing = cross_type_df,
        local_annotation_inconsistencies = local_inconsistency_df,
        local_inconsistency_summary = local_inconsistency_summary,
        community_composition = community_composition,
        annotation_consistency = annotation_summary,
        overall_metrics = overall_metrics,
        graph_info = graph_info,
        parameters = parameters,
        cell_info = cell_info
    )

    class(result) <- c(class(result), "calculateGraphIntegrationObject")
    return(result)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)
library(ggplot2)
library(scater)
library(SingleR)

# Load example datasets
data("reference_data")
data("query_data")

test_that("plot.calculateGraphIntegrationObject", {

    # Create modified reference data (remove Myeloid)
    ref_data_mod <- reference_data[, reference_data$expert_annotation != "Myeloid"]
    ref_data_mod <- runPCA(ref_data_mod, ncomponents = 50)

    # Create SingleR annotations
    SingleR_annotation <- SingleR(query_data, ref_data_mod,
                                  labels = ref_data_mod$expert_annotation)
    query_data_mod <- query_data
    query_data_mod$SingleR_annotation <- SingleR_annotation$labels

    # Create diagnostic results for testing
    diag_result <- calculateGraphIntegration(
        query_data = query_data_mod,
        reference_data = ref_data_mod,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation"
    )

    # Test 1: Default plot (community_network)
    p1 <- plot(diag_result)
    expect_s3_class(p1, "ggplot")
    expect_true(grepl("Community Network", p1$labels$title))

    # Test 2: Community network plot with cell_type coloring
    p2 <- plot(diag_result, plot_type = "community_network", color_by = "cell_type")
    expect_s3_class(p2, "ggplot")
    expect_equal(p2$labels$title, "Community Network: Cell Type Composition")

    # Test 3: Community network plot with community_type coloring
    p3 <- plot(diag_result, plot_type = "community_network", color_by = "community_type")
    expect_s3_class(p3, "ggplot")
    expect_equal(p3$labels$title, "Community Network: Inter-Community Connection Strength")

    # Test 4: Cell network plot with cell_type coloring
    p4 <- plot(diag_result, plot_type = "cell_network", color_by = "cell_type")
    expect_s3_class(p4, "ggplot")
    expect_equal(p4$labels$title, "Cell Network: Cell Type Distribution")

    # Test 5: Cell network plot with community_type coloring
    p5 <- plot(diag_result, plot_type = "cell_network", color_by = "community_type")
    expect_s3_class(p5, "ggplot")
    expect_equal(p5$labels$title, "Cell Network: Annotation Consistency Diagnostics")

    # Test 6: Community data plot
    p6 <- plot(diag_result, plot_type = "community_data")
    expect_s3_class(p6, "ggplot")
    expect_equal(p6$labels$title, "Community Data Overview")
    expect_equal(p6$labels$x, "Query Proportion")
    expect_equal(p6$labels$y, "Total Cells in Community")

    # Test 7: Summary plot
    p7 <- plot(diag_result, plot_type = "summary")
    expect_s3_class(p7, "ggplot")
    expect_equal(p7$labels$title, "Total Annotation Issues by Cell Type")
    expect_true("GeomCol" %in% class(p7$layers[[1]]$geom))

    # Test 8: Local issues plot
    p8 <- plot(diag_result, plot_type = "local_issues")
    expect_s3_class(p8, "ggplot")
    expect_true(grepl("Local", p8$labels$title))

    # Test 9: Annotation issues plot
    p9 <- plot(diag_result, plot_type = "annotation_issues")
    expect_s3_class(p9, "ggplot")
    expect_true(grepl("Annotation", p9$labels$title))

    # Test 10: Invalid plot_type
    expect_error(
        plot(diag_result, plot_type = "invalid_type"),
        "'arg' should be one of"
    )

    # Test 11: Invalid color_by
    expect_error(
        plot(diag_result, plot_type = "community_network", color_by = "invalid_color"),
        "'arg' should be one of"
    )

    # Test 12: Custom parameters
    p12 <- plot(diag_result, plot_type = "cell_network",
                max_nodes = 500, point_size = 1.5)
    expect_s3_class(p12, "ggplot")

    # Test 13: exclude_reference_only parameter
    p13 <- plot(diag_result, plot_type = "community_data",
                exclude_reference_only = TRUE)
    expect_s3_class(p13, "ggplot")

    p14 <- plot(diag_result, plot_type = "cell_network",
                exclude_reference_only = TRUE)
    expect_s3_class(p14, "ggplot")

    # Test 14: Test color schemes are applied correctly
    p_colors <- plot(diag_result, plot_type = "community_data")
    expect_true("ScaleDiscrete" %in% class(p_colors$scales$get_scales("colour")))

    # Test 15: Test with large dataset (triggering sampling)
    p_large <- plot(diag_result, plot_type = "cell_network", max_nodes = 100)
    expect_s3_class(p_large, "ggplot")

    # Test 16: Test subtitle generation
    p_subtitle <- plot(diag_result, plot_type = "cell_network")
    expect_true(!is.null(p_subtitle$labels$subtitle))

    # Test 17: Test coordinate systems
    p_coord <- plot(diag_result, plot_type = "summary")
    expect_true("CoordCartesian" %in% class(p_coord$coordinates))

    # Test 18: Test themes
    p_void <- plot(diag_result, plot_type = "cell_network")
    expect_true(!is.null(p_void$theme))
    expect_s3_class(p_void$theme, "theme")

    # Test 19: Test scale transformations
    p_scales <- plot(diag_result, plot_type = "community_data")
    size_scale <- p_scales$scales$get_scales("size")
    expect_true("ScaleContinuous" %in% class(size_scale))

    # Test 20: Test with exclude_reference_only filtering
    p_with_ref <- plot(diag_result, plot_type = "community_network",
                       exclude_reference_only = FALSE)
    p_without_ref <- plot(diag_result, plot_type = "community_network",
                          exclude_reference_only = TRUE)
    expect_s3_class(p_with_ref, "ggplot")
    expect_s3_class(p_without_ref, "ggplot")

    # Test 21: Test legend positioning
    p_legend <- plot(diag_result, plot_type = "community_data")
    expect_equal(p_legend$theme$legend.position, "right")

    p_legend_bottom <- plot(diag_result, plot_type = "summary")
    expect_equal(p_legend_bottom$theme$legend.position, "bottom")

    # Test 22: Test coordinate flipping in annotation_issues plot
    if (nrow(diag_result$high_query_prop_analysis) > 0 ||
        nrow(diag_result$cross_type_mixing) > 0 ||
        nrow(diag_result$local_annotation_inconsistencies) > 0) {

        p_flip <- plot(diag_result, plot_type = "annotation_issues")
        expect_true("CoordFlip" %in% class(p_flip$coordinates))
    }

    # Test 23: Test no local issues scenario
    diag_no_local <- diag_result
    diag_no_local$local_annotation_inconsistencies <-
        diag_no_local$local_annotation_inconsistencies[0, ]

    p_no_local <- plot(diag_no_local, plot_type = "local_issues")
    expect_s3_class(p_no_local, "ggplot")

    # Test 24: Test no annotation issues scenario
    diag_no_issues <- diag_result
    diag_no_issues$high_query_prop_analysis <- diag_no_issues$high_query_prop_analysis[0, ]
    diag_no_issues$cross_type_mixing <- diag_no_issues$cross_type_mixing[0, ]
    diag_no_issues$local_annotation_inconsistencies <-
        diag_no_issues$local_annotation_inconsistencies[0, ]

    p_no_issues <- plot(diag_no_issues, plot_type = "annotation_issues")
    expect_s3_class(p_no_issues, "ggplot")

    # Test 25: Test plot building without errors
    expect_silent(ggplot_build(plot(diag_result, plot_type = "community_data")))
    expect_silent(ggplot_build(plot(diag_result, plot_type = "summary")))

    # Test 26: Test different point sizes
    p_small <- plot(diag_result, plot_type = "cell_network", point_size = 0.5)
    p_large <- plot(diag_result, plot_type = "cell_network", point_size = 2.0)
    expect_s3_class(p_small, "ggplot")
    expect_s3_class(p_large, "ggplot")

    # Test 27: Test all plot types work
    plot_types <- c("community_network", "cell_network", "community_data",
                    "summary", "local_issues", "annotation_issues")

    for (plot_type in plot_types) {
        p_test <- plot(diag_result, plot_type = plot_type)
        expect_s3_class(p_test, "ggplot")
    }

    # Test 28: Test both color_by options for network plots
    color_options <- c("cell_type", "community_type")

    for (color_opt in color_options) {
        p_comm <- plot(diag_result, plot_type = "community_network", color_by = color_opt)
        p_cell <- plot(diag_result, plot_type = "cell_network", color_by = color_opt)
        expect_s3_class(p_comm, "ggplot")
        expect_s3_class(p_cell, "ggplot")
    }

    # Test 29: Test additional arguments
    p_extra <- plot(diag_result, plot_type = "summary",
                    environment = environment())
    expect_s3_class(p_extra, "ggplot")
})

# Load necessary libraries
library(testthat)
library(scDiagnostics)
library(scater)
library(SingleR)

# Load example datasets
data("reference_data")
data("query_data")

# Function to run all tests
test_that("calculateGraphIntegration", {

    # Create modified reference data (remove Myeloid)
    ref_data_mod <- reference_data[, reference_data$expert_annotation != "Myeloid"]
    ref_data_mod <- runPCA(ref_data_mod, ncomponents = 50)

    # Create SingleR annotations
    SingleR_annotation <- SingleR(query_data, ref_data_mod,
                                  labels = ref_data_mod$expert_annotation)
    query_data_mod <- query_data
    query_data_mod$SingleR_annotation <- SingleR_annotation$labels

    # Test 1: Basic functionality with default parameters
    result <- calculateGraphIntegration(
        query_data = query_data_mod,
        reference_data = ref_data_mod,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation"
    )

    # Check return object structure
    expect_s3_class(result, "calculateGraphIntegrationObject")
    expected_names <- c("high_query_prop_analysis", "cross_type_mixing",
                        "local_annotation_inconsistencies", "local_inconsistency_summary",
                        "community_composition", "annotation_consistency",
                        "overall_metrics", "graph_info", "parameters", "cell_info")
    expect_equal(names(result), expected_names)

    # Check data frame structures
    expect_true(is.data.frame(result$high_query_prop_analysis))
    expect_true(is.data.frame(result$cross_type_mixing))
    expect_true(is.data.frame(result$local_annotation_inconsistencies))
    expect_true(is.data.frame(result$community_composition))
    expect_true(is.data.frame(result$annotation_consistency))
    expect_true(is.data.frame(result$cell_info))

    # Check overall metrics structure
    expect_true(is.list(result$overall_metrics))
    expect_true(all(c("total_communities", "modularity", "mean_query_isolation_rate") %in%
                        names(result$overall_metrics)))

    # Check parameters are stored correctly
    expect_equal(result$parameters$pc_subset, 1:10)
    expect_equal(result$parameters$k_neighbors, 30)
    expect_equal(result$parameters$resolution, 0.1)

    # Test 2: Custom parameters
    result_custom <- calculateGraphIntegration(
        query_data = query_data_mod,
        reference_data = ref_data_mod,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:5,
        k_neighbors = 15,
        resolution = 0.2,
        high_query_prop_threshold = 0.8,
        cross_type_threshold = 0.15,
        local_consistency_threshold = 0.5,
        local_confidence_threshold = 0.3
    )

    expect_equal(result_custom$parameters$pc_subset, 1:5)
    expect_equal(result_custom$parameters$k_neighbors, 15)
    expect_equal(result_custom$parameters$resolution, 0.2)
    expect_equal(result_custom$parameters$high_query_prop_threshold, 0.8)

    # Test 3: Specific cell types
    specific_types <- c("CD4", "CD8", "B_and_plasma")
    result_specific <- calculateGraphIntegration(
        query_data = query_data_mod,
        reference_data = ref_data_mod,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_types = specific_types
    )

    expect_true(all(result_specific$parameters$cell_types_analyzed %in% specific_types))

    # Test 4: Different assay name
    # Add logcounts assay if it doesn't exist
    if (!"logcounts" %in% assayNames(query_data_mod)) {
        assay(query_data_mod, "logcounts") <- assay(query_data_mod, "counts")
    }
    if (!"logcounts" %in% assayNames(ref_data_mod)) {
        assay(ref_data_mod, "logcounts") <- assay(ref_data_mod, "counts")
    }

    result_logcounts <- calculateGraphIntegration(
        query_data = query_data_mod,
        reference_data = ref_data_mod,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        assay_name = "logcounts"
    )

    expect_s3_class(result_logcounts, "calculateGraphIntegrationObject")

    # Test 5: Error handling - invalid parameters

    # Invalid k_neighbors
    expect_error(
        calculateGraphIntegration(
            query_data = query_data_mod,
            reference_data = ref_data_mod,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            k_neighbors = -5
        ),
        "k_neighbors.*must be a positive integer"
    )

    expect_error(
        calculateGraphIntegration(
            query_data = query_data_mod,
            reference_data = ref_data_mod,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            k_neighbors = 1.5
        ),
        "k_neighbors.*must be a positive integer"
    )

    # Invalid resolution
    expect_error(
        calculateGraphIntegration(
            query_data = query_data_mod,
            reference_data = ref_data_mod,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            resolution = -0.1
        ),
        "resolution.*must be a positive number"
    )

    # Invalid high_query_prop_threshold
    expect_error(
        calculateGraphIntegration(
            query_data = query_data_mod,
            reference_data = ref_data_mod,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            high_query_prop_threshold = 0.3
        ),
        "high_query_prop_threshold.*must be between 0.5 and 1"
    )

    expect_error(
        calculateGraphIntegration(
            query_data = query_data_mod,
            reference_data = ref_data_mod,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            high_query_prop_threshold = 1.2
        ),
        "high_query_prop_threshold.*must be between 0.5 and 1"
    )

    # Invalid cross_type_threshold
    expect_error(
        calculateGraphIntegration(
            query_data = query_data_mod,
            reference_data = ref_data_mod,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            cross_type_threshold = 0.6
        ),
        "cross_type_threshold.*must be between 0 and 0.5"
    )

    # Invalid local_consistency_threshold
    expect_error(
        calculateGraphIntegration(
            query_data = query_data_mod,
            reference_data = ref_data_mod,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            local_consistency_threshold = 1.5
        ),
        "local_consistency_threshold.*must be between 0 and 1"
    )

    # Invalid local_confidence_threshold
    expect_error(
        calculateGraphIntegration(
            query_data = query_data_mod,
            reference_data = ref_data_mod,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            local_confidence_threshold = -0.1
        ),
        "local_confidence_threshold.*must be between 0 and 1"
    )

    # Test 6: Edge case - very small dataset
    # Create small subset
    small_query <- query_data_mod[, 1:50]
    small_ref <- ref_data_mod[, 1:50]
    small_ref <- runPCA(small_ref, ncomponents = 10)

    # Should handle small datasets gracefully
    result_small <- calculateGraphIntegration(
        query_data = small_query,
        reference_data = small_ref,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        min_cells_per_celltype = 5,
        min_cells_per_community = 3
    )

    expect_s3_class(result_small, "calculateGraphIntegrationObject")

    # Test 7: Edge case - k_neighbors larger than dataset
    # Should automatically adjust k_neighbors
    expect_warning(
        result_large_k <- calculateGraphIntegration(
            query_data = small_query,
            reference_data = small_ref,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            k_neighbors = 1000,
            min_cells_per_celltype = 5
        ),
        "k_neighbors reduced"
    )

    expect_s3_class(result_large_k, "calculateGraphIntegrationObject")

    # Test 8: Invalid cell type columns
    expect_error(
        calculateGraphIntegration(
            query_data = query_data_mod,
            reference_data = ref_data_mod,
            query_cell_type_col = "nonexistent_column",
            ref_cell_type_col = "expert_annotation"
        ),
        # This should trigger the argumentCheck function
        class = "error"
    )

    # Test 9: No common cell types after filtering
    # Create data with no overlapping cell types that meet minimum requirements
    query_small_types <- query_data_mod[, 1:10]  # Very small subset
    ref_small_types <- ref_data_mod[, 1:10]      # Very small subset

    expect_error(
        calculateGraphIntegration(
            query_data = query_small_types,
            reference_data = ref_small_types,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            min_cells_per_celltype = 50  # Very high threshold
        ),
        "No cell types meet the minimum cell count requirement"
    )

    # Test 10: Check numerical results make sense
    expect_true(result$overall_metrics$total_communities >= 0)
    expect_true(result$overall_metrics$modularity >= -1 && result$overall_metrics$modularity <= 1)
    expect_true(result$overall_metrics$mean_query_isolation_rate >= 0 &&
                    result$overall_metrics$mean_query_isolation_rate <= 1)
    expect_true(result$overall_metrics$mean_local_inconsistency_rate >= 0 &&
                    result$overall_metrics$mean_local_inconsistency_rate <= 1)

    # Test 11: Check community composition consistency
    if (nrow(result$community_composition) > 0) {
        expect_true(all(result$community_composition$total_cells >=
                            (result$community_composition$n_reference + result$community_composition$n_query)))
        expect_true(all(result$community_composition$query_proportion >= 0 &
                            result$community_composition$query_proportion <= 1))
    }

    # Test 12: Check annotation consistency metrics
    if (nrow(result$annotation_consistency) > 0) {
        expect_true(all(result$annotation_consistency$query_isolation_rate >= 0 &
                            result$annotation_consistency$query_isolation_rate <= 1))
        expect_true(all(result$annotation_consistency$true_cross_mixing_rate >= 0 &
                            result$annotation_consistency$true_cross_mixing_rate <= 1))
    }

    # Test 13: Check cell_info consistency
    expect_equal(nrow(result$cell_info),
                 sum(result$annotation_consistency$total_query_cells) +
                     sum(table(ref_data_mod$expert_annotation)[result$parameters$cell_types_analyzed]))

    expect_true(all(c("Query", "Reference") %in% result$cell_info$dataset))
    expect_true(all(result$cell_info$cell_type %in% result$parameters$cell_types_analyzed))

    # Test 14: Graph info structure
    expect_true(is.list(result$graph_info))
    expect_true(all(c("layout", "edges", "n_nodes") %in% names(result$graph_info)))
    expect_true(is.matrix(result$graph_info$layout))
    expect_equal(ncol(result$graph_info$layout), 2)  # 2D layout
    expect_equal(result$graph_info$n_nodes, nrow(result$cell_info))

    # Test 15: Local inconsistencies structure
    if (nrow(result$local_annotation_inconsistencies) > 0) {
        expect_true(all(result$local_annotation_inconsistencies$confidence_score >= 0))
        expect_true(all(result$local_annotation_inconsistencies$current_support_prop >= 0 &
                            result$local_annotation_inconsistencies$current_support_prop <= 1))
        expect_true(all(result$local_annotation_inconsistencies$suggested_support_prop >= 0 &
                            result$local_annotation_inconsistencies$suggested_support_prop <= 1))
    }

    # Test 16: Check that different thresholds produce different results
    result_strict <- calculateGraphIntegration(
        query_data = query_data_mod,
        reference_data = ref_data_mod,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        high_query_prop_threshold = 0.95,
        local_consistency_threshold = 0.8
    )

    result_lenient <- calculateGraphIntegration(
        query_data = query_data_mod,
        reference_data = ref_data_mod,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        high_query_prop_threshold = 0.7,
        local_consistency_threshold = 0.4
    )

    # Strict thresholds should generally detect fewer issues than lenient ones
    expect_true(nrow(result_strict$high_query_prop_analysis) <= nrow(result_lenient$high_query_prop_analysis))

    # Test 17: Different PC subsets
    result_fewer_pcs <- calculateGraphIntegration(
        query_data = query_data_mod,
        reference_data = ref_data_mod,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:5
    )

    result_more_pcs <- calculateGraphIntegration(
        query_data = query_data_mod,
        reference_data = ref_data_mod,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:20
    )

    expect_equal(result_fewer_pcs$parameters$pc_subset, 1:5)
    expect_equal(result_more_pcs$parameters$pc_subset, 1:20)

    # Both should produce valid results
    expect_s3_class(result_fewer_pcs, "calculateGraphIntegrationObject")
    expect_s3_class(result_more_pcs, "calculateGraphIntegrationObject")
})

#' @title Plot Principal Components for Different Cell Types
#'
#' @description
#' This function plots the principal components for different cell types in the query and reference datasets.
#'
#' @details
#' This function projects the query dataset onto the principal component space of the reference dataset and then plots the
#' specified principal components for the specified cell types.
#' It uses the `projectPCA` function to perform the projection and \code{GGally} to create the pairs plot.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.
#' @param pc_subset A numeric vector specifying which principal components to include in the plot. Default is 1:5.
#' @param lower_facet Type of plot to use for the lower panels. Either "scatter" (default), "contour", "ellipse", or "blank".
#' @param diagonal_facet Type of plot to use for the diagonal panels. Either "ridge" (default), "density", or "boxplot".
#' @param upper_facet Type of plot to use for the upper panels. Either "blank" (default), "scatter", "contour", or "ellipse".
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 2000.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 2000.
#'
#' @return A ggmatrix object representing a pairs plot of specified principal components for the given cell types and datasets.
#'
#' @export
#'
# Function to plot PC for different cell types
plotCellTypePCA <- function(query_data,
                            reference_data,
                            query_cell_type_col,
                            ref_cell_type_col,
                            cell_types = NULL,
                            pc_subset = 1:5,
                            assay_name = "logcounts",
                            lower_facet = c("scatter", "contour", "ellipse", "blank"),
                            diagonal_facet = c("ridge", "density", "boxplot"),
                            upper_facet = c("blank", "scatter", "contour", "ellipse"),
                            max_cells_query = 2000,
                            max_cells_ref = 2000){

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Match diagonal_facet and upper_facet arguments
    lower_facet <- match.arg(lower_facet)
    diagonal_facet <- match.arg(diagonal_facet)
    upper_facet <- match.arg(upper_facet)

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = 10)

    # Get the projected PCA data
    pca_output <- projectPCA(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = query_cell_type_col,
                             ref_cell_type_col = ref_cell_type_col,
                             cell_types = cell_types,
                             pc_subset = pc_subset,
                             assay_name = assay_name,
                             max_cells_ref = max_cells_ref,
                             max_cells_query = max_cells_query)

    # Create PC column names with variance explained
    plot_names <- paste0(
        "PC", pc_subset, " (",
        sprintf("%.1f%%", attributes(
            reducedDim(reference_data, "PCA"))[["percentVar"]][pc_subset]), ")")

    # Create a new data frame with selected PCs
    pc_df <- data.frame(matrix(0, nrow = nrow(pca_output),
                               ncol = length(pc_subset)))
    colnames(pc_df) <- plot_names

    for (i in 1:length(pc_subset)) {
        pc_df[, i] <- pca_output[, paste0("PC", pc_subset[i])]
    }

    # Create a cell type dataset column for coloring
    cell_type_dataset <- paste(pca_output[["dataset"]],
                               pca_output[["cell_type"]], sep = " ")

    # Define the order of cell type and dataset combinations
    order_combinations <- paste(
        rep(c("Reference", "Query"),
            length(cell_types)),
        rep(sort(cell_types), each = 2))

    cell_type_dataset <- factor(cell_type_dataset,
                                levels = order_combinations)

    # Generate colors for cell types and datasets
    cell_type_colors <- generateColors(order_combinations, paired = TRUE)

    # Add the cell type dataset column to the PC data frame
    pc_df[["cell_type_dataset"]] <- cell_type_dataset

    # Create a simple plot to extract the legend using GGally::grab_legend
    legend_plot <- ggplot2::ggplot(pc_df,
                                   ggplot2::aes(x = pc_df[,1],
                                                y = pc_df[,2])) +
        ggplot2::geom_point(ggplot2::aes(color = cell_type_dataset)) +
        ggplot2::scale_color_manual(values = cell_type_colors,
                                    name = "Cell Type") +
        ggplot2::theme(
            legend.position = "right",
            legend.box = "vertical",
            legend.key = ggplot2::element_rect(fill = "white"),
            legend.background = ggplot2::element_blank()
        ) +
        ggplot2::guides(color = ggplot2::guide_legend(ncol = 1))

    # Scatterplot facet function
    .scatterFunc <- function(data, mapping, ...) {

        ggplot2::ggplot(data = data, mapping = mapping) +
            ggplot2::geom_point(alpha = 0.5, size = 1,
                                ggplot2::aes(color = cell_type_dataset)) +
            ggplot2::scale_color_manual(values = cell_type_colors) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(
                    color = "black",
                    fill = NA,
                    linewidth = 0.5))
    }

    # Contour facet function
    .smoothContourFunc <- function(data, mapping, ...) {

        x_name <- rlang::as_name(mapping[["x"]])
        y_name <- rlang::as_name(mapping[["y"]])

        # Start with empty plot
        p <- ggplot2::ggplot() +
            ggplot2::theme_minimal()

        # Process each cell type separately
        for (ct in unique(data[["cell_type_dataset"]])) {
            subset_data <- data[data[["cell_type_dataset"]] == ct, ]

            # Skip if too few points
            if (nrow(subset_data) < 10) next

            # Use larger adjustment factor for smoother contours
            adjust_factor <- 1.5

            # Add smoother contours with fewer levels
            p <- p + ggplot2::stat_density_2d(
                data = subset_data,
                mapping = ggplot2::aes(
                    x = !!rlang::sym(x_name),
                    y = !!rlang::sym(y_name)
                ),
                contour = TRUE,
                adjust = adjust_factor,
                bins = 5,
                color = cell_type_colors[which(
                    levels(data[["cell_type_dataset"]]) == ct)],
                linewidth = 0.5,
                na.rm = TRUE
            )
        }

        # Apply theme elements
        p + ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(
                    color = "black", fill = NA, linewidth = 0.5),
                legend.position = "none",
                axis.text = ggplot2::element_blank(),
                axis.ticks = ggplot2::element_blank(),
                axis.title = ggplot2::element_blank()
            )
    }

    # Ellipse facet function
    .robustEllipseFunc <- function(data, mapping, ...) {

        # Function to calculate robust ellipses through bootstrapping
        createEllipse <- function(d) {
            if (nrow(d) < 10) return(NULL)

            x_var <- rlang::as_name(mapping[["x"]])
            y_var <- rlang::as_name(mapping[["y"]])

            # Extract variables
            x <- d[[x_var]]
            y <- d[[y_var]]

            # Calculate robust center and covariance
            cov_mat <- cov(cbind(x, y), use = "pairwise.complete.obs")
            center <- c(mean(x, na.rm = TRUE), mean(y, na.rm = TRUE))

            # Calculate ellipse points
            theta <- seq(0, 2 * pi, length.out = 100)
            ellipse <- MASS::cov.trob(cbind(x, y))

            # Get ellipse coordinates
            ev <- eigen(ellipse[["cov"]])
            a <- sqrt(ev[["values"]][1]) * 2.45
            b <- sqrt(ev[["values"]][2]) * 2.45

            # Create ellipse coordinates
            angle <- atan2(ev[["vectors"]][2,1], ev[["vectors"]][1,1])
            ellipse_x <- center[1] + a * cos(theta) * cos(angle) -
                b * sin(theta) * sin(angle)
            ellipse_y <- center[2] + a * cos(theta) * sin(angle) +
                b * sin(theta) * cos(angle)

            return(data.frame(x = ellipse_x, y = ellipse_y))
        }

        p <- ggplot2::ggplot(data, mapping)

        # Split by cell type and create robust ellipses
        for (ct in unique(data[["cell_type_dataset"]])) {
            subset_data <- data[data[["cell_type_dataset"]] == ct,]
            ellipse_data <- createEllipse(subset_data)

            if (!is.null(ellipse_data)) {
                p <- p + ggplot2::geom_path(
                    data = ellipse_data,
                    ggplot2::aes(x = .data[["x"]], y = .data[["y"]]),
                    color = cell_type_colors[
                        which(levels(data[["cell_type_dataset"]]) == ct)],
                    linewidth = 0.7
                )
            }
        }

        p + ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(
                    color = "black", fill = NA,
                    linewidth = 0.5),
                legend.position = "none",
                axis.text = ggplot2::element_blank(),
                axis.ticks = ggplot2::element_blank(),
                axis.title = ggplot2::element_blank()
            )
    }

    # Blank facet function
    .blankFunc <- function(data, mapping, ...) {

        ggplot2::ggplot() +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(
                    color = "black", fill = NA,
                    linewidth = 0.5),
                legend.position = "none",
                axis.text = ggplot2::element_blank(),
                axis.ticks = ggplot2::element_blank(),
                axis.title = ggplot2::element_blank(),
                panel.grid = ggplot2::element_blank()
            )
    }

    # Ridge diagonal facet
    .ridgeFunc <- function(data, mapping, ...) {

        # Get current mapping info
        x_var_name <- rlang::as_name(mapping[["x"]])

        # Create a long-format data frame for ridge plot
        plot_data <- data.frame(
            value = data[[x_var_name]],
            group = data[["cell_type_dataset"]]
        )

        # Create ridge plot with ggridges
        suppressMessages({
            p <- ggplot2::ggplot(plot_data,
                                 ggplot2::aes(x = .data[["value"]],
                                              y = .data[["group"]],
                                              fill = .data[["group"]])) +
                ggridges::geom_density_ridges(
                    alpha = 0.7,
                    scale = 2,
                    rel_min_height = 0.01,
                    quantile_lines = FALSE
                ) +
                ggplot2::scale_fill_manual(values = cell_type_colors) +
                ggplot2::scale_y_discrete(
                    limits = rev(levels(cell_type_dataset))) +
                ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(color = "black",
                                                         fill = NA,
                                                         linewidth = 0.5),
                    axis.title = ggplot2::element_blank(),
                    axis.text.y = ggplot2::element_blank(),
                    axis.ticks.y = ggplot2::element_blank(),
                    legend.position = "none"
                )
        })

        return(p)
    }

    # Density diagonal facet
    .densityFunc <- function(data, mapping, ...) {

        # Get current mapping info
        x_var_name <- rlang::as_name(mapping[["x"]])

        ggplot2::ggplot(data = data, mapping = mapping) +
            ggplot2::geom_density(ggplot2::aes(fill = cell_type_dataset,
                                               color = cell_type_dataset),
                                  alpha = 0.5) +
            ggplot2::scale_fill_manual(values = cell_type_colors) +
            ggplot2::scale_color_manual(values = cell_type_colors) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(color = "black",
                                                     fill = NA,
                                                     linewidth = 0.5),
                axis.title.y = ggplot2::element_blank(),
                axis.text.y = ggplot2::element_blank(),
                axis.ticks.y = ggplot2::element_blank()
            )
    }

    # Boxplot diagonal facet
    .boxplotFunc <- function(data, mapping, ...) {

        # Extract the x variable name for the boxplot title
        x_name <- rlang::as_name(mapping[["x"]])

        # Create a long-format data frame for horizontal boxplot
        plot_data <- data.frame(
            value = data[[x_name]],
            group = data[["cell_type_dataset"]]
        )

        # Create horizontal boxplot
        ggplot2::ggplot(plot_data, ggplot2::aes(x = .data[["value"]],
                                                y = .data[["group"]],
                                                fill = .data[["group"]])) +
            ggplot2::geom_boxplot(alpha = 0.7,
                                  outlier.size = 0.5,
                                  width = 0.6) +
            ggplot2::scale_fill_manual(values = cell_type_colors) +
            ggplot2::scale_y_discrete(limits = rev(
                levels(cell_type_dataset))) +
            ggplot2::labs(x = "", y = "") +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(color = "black",
                                                     fill = NA,
                                                     linewidth = 0.5),
                axis.title.x = ggplot2::element_blank(),
                legend.position = "none",
                axis.title.y = ggplot2::element_blank(),
                axis.text.y = ggplot2::element_blank(),
                axis.ticks.y = ggplot2::element_blank()
            )
    }

    # Select the upper facet plot type based on user input
    if (lower_facet == "scatter") {
        lower_plot <- .scatterFunc
    } else if (lower_facet == "contour") {
        lower_plot <- .smoothContourFunc
    } else if (lower_facet == "ellipse") {
        lower_plot <- .robustEllipseFunc
    } else if (lower_facet == "blank") {
        lower_plot <- .blankFunc
    }

    # Select the diagonal plot type based on user input
    if (diagonal_facet == "density") {
        diag_plot <- .densityFunc
    } else if (diagonal_facet == "boxplot") {
        diag_plot <- .boxplotFunc
    } else if (diagonal_facet == "ridge") {
        diag_plot <- .ridgeFunc
    }

    # Select the upper facet plot type based on user input
    if (upper_facet == "blank") {
        upper_plot <- .blankFunc
    } else if (upper_facet == "scatter") {
        upper_plot <- .scatterFunc
    } else if (upper_facet == "contour") {
        upper_plot <- .smoothContourFunc
    } else if (upper_facet == "ellipse") {
        upper_plot <- .robustEllipseFunc
    }

    # Create pairs plot using GGally
    plot_obj <- suppressMessages(
        GGally::ggpairs(
            pc_df,
            columns = seq_len(length(pc_subset)),
            mapping = ggplot2::aes(color = cell_type_dataset),
            lower = list(continuous = lower_plot),
            upper = list(continuous = upper_plot),
            diag = list(continuous = diag_plot),
            progress = FALSE,
            legend = GGally::grab_legend(legend_plot)
        )
    )

    # Add black frame around facet titles
    plot_obj <- plot_obj +
        ggplot2::theme(
            strip.background = ggplot2::element_rect(
                fill = "white", color = "black", linewidth = 0.5),
            strip.text = ggplot2::element_text(color = "black")
        )

    # Return the plot
    return(plot_obj)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

test_that("plotCellTypePCA generates plots correctly", {
    # Generate plot using the function
    p1 <- plotCellTypePCA(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
        pc_subset = 1:5
    )

    # Check if output is a ggmatrix object (from GGally::ggpairs)
    expect_s3_class(p1, "ggmatrix")
})

test_that("plotCellTypePCA works with different facet options", {
    # Test different facet combinations
    p1 <- plotCellTypePCA(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_types = c("CD4", "CD8"),
        pc_subset = 1:3,
        lower_facet = "scatter",
        diagonal_facet = "density",
        upper_facet = "blank"
    )

    expect_s3_class(p1, "ggmatrix")

    # Test with different options
    p2 <- plotCellTypePCA(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_types = c("CD4", "CD8"),
        pc_subset = 1:3,
        lower_facet = "contour",
        diagonal_facet = "boxplot",
        upper_facet = "ellipse"
    )

    expect_s3_class(p2, "ggmatrix")
})

test_that("plotCellTypePCA works with default parameters", {
    # Test with minimal parameters (should use defaults)
    p1 <- plotCellTypePCA(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "expert_annotation"
    )

    expect_s3_class(p1, "ggmatrix")
})

test_that("plotCellTypePCA handles invalid input gracefully", {
    # Test with invalid column names for cell types
    expect_error(plotCellTypePCA(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "InvalidColumn",
        ref_cell_type_col = "InvalidColumn",
        cell_types = c("CD4", "CD8"),
        pc_subset = 1:3
    ))

    # Test with invalid principal component subset (too high)
    expect_error(plotCellTypePCA(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_types = c("CD4", "CD8"),
        pc_subset = c(100, 200, 300)
    ))
})

#' @title Plot Principal Components for Different Cell Types
#'
#' @description This function generates a \code{ggplot2} visualization of principal components (PCs) for different
#' cell types across two datasets (query and reference), using either boxplots or violin plots.
#'
#' @details
#' The function \code{boxplotPCA} is designed to provide a visualization of principal component analysis (PCA) results. It projects
#' the query dataset onto the principal components obtained from the reference dataset. The results are then visualized
#' as boxplots or violin plots, grouped by cell types and datasets (query and reference). This allows for a comparative analysis of the
#' distributions of the principal components across different cell types and datasets. The function internally calls \code{projectPCA}
#' to perform the PCA projection. It then reshapes the output data into a long format suitable for ggplot2 plotting.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the
#'                   query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for
#'                       the reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are
#'                   included.
#' @param pc_subset A numeric vector specifying which principal components to include in the plot. Default is PC1 to PC5.
#' @param shape Character string indicating the plot type: "box" for boxplots or "violin" for violin plots.
#'              Default is "box".
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is NULL.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is NULL.
#'
#' @return A ggplot object representing the boxplots or violin plots of specified principal components for the given
#'         cell types and datasets.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Plot the PC data with boxplots (default)
#' pc_plot <- boxplotPCA(query_data = query_data,
#'                       reference_data = reference_data,
#'                       cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
#'                       query_cell_type_col = "SingleR_annotation",
#'                       ref_cell_type_col = "expert_annotation",
#'                       pc_subset = 1:6)
#' pc_plot
#'
#' # Plot the PC data with violin plots
#' pc_violin <- boxplotPCA(query_data = query_data,
#'                         reference_data = reference_data,
#'                         cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
#'                         query_cell_type_col = "SingleR_annotation",
#'                         ref_cell_type_col = "expert_annotation",
#'                         pc_subset = 1:6,
#'                         shape = "violin")
#' pc_violin
#'
#' @importFrom stats approxfun cancor density setNames
#' @importFrom utils combn
#'
# Function to plot PC for different cell types
boxplotPCA <- function(query_data,
                       reference_data,
                       query_cell_type_col,
                       ref_cell_type_col,
                       cell_types = NULL,
                       pc_subset = 1:5,
                       shape = c("box", "violin"),
                       assay_name = "logcounts",
                       max_cells_query = NULL,
                       max_cells_ref = NULL){

    # Match the shape argument
    shape <- match.arg(shape)

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = 10)

    # Get the projected PCA data
    pca_output <- projectPCA(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = query_cell_type_col,
                             ref_cell_type_col = ref_cell_type_col,
                             cell_types = cell_types,
                             pc_subset = pc_subset,
                             assay_name = assay_name,
                             max_cells_ref = max_cells_ref,
                             max_cells_query = max_cells_query)

    # Get variance explained percentages
    pca_data <- SingleCellExperiment::reducedDim(reference_data, "PCA")
    percent_var <- attr(pca_data, "percentVar")[pc_subset]

    # Create PC labels with variance explained
    pc_labels <- paste0("PC", pc_subset, " (", round(percent_var, 1), "%)")
    names(pc_labels) <- paste0("pc", pc_subset)

    # Create the long format data frame manually
    pca_long <- data.frame(PC = rep(paste0("pc", pc_subset),
                                    each = nrow(pca_output)),
                           Value = unlist(c(pca_output[, pc_subset])),
                           dataset = rep(pca_output[["dataset"]],
                                         length(pc_subset)),
                           cell_type = rep(pca_output[["cell_type"]],
                                           length(pc_subset)))

    # Create properly ordered PC factor with variance explained labels
    pc_order <- paste0("pc", pc_subset)
    pca_long[["PC"]] <- factor(pca_long[["PC"]], levels = pc_order)

    # Create a new variable representing the combination of cell type and dataset
    pca_long[["cell_type_dataset"]] <- paste(pca_long[["dataset"]],
                                             pca_long[["cell_type"]],
                                             sep = " ")

    # Define the order of cell type and dataset combinations
    order_combinations <- paste(rep(c("Reference", "Query"),
                                    length(unique(pca_long[["cell_type"]]))),
                                rep(sort(unique(pca_long[["cell_type"]])),
                                    each = 2))

    # Reorder the levels of cell type and dataset factor
    pca_long[["cell_type_dataset"]] <- factor(pca_long[["cell_type_dataset"]],
                                              levels = order_combinations)

    # Define the colors for cell types
    cell_type_colors <- generateColors(order_combinations,
                                       paired = TRUE)

    # Create the ggplot
    plot <- ggplot2::ggplot(pca_long, ggplot2::aes(
        x = .data[["cell_type"]],
        y = .data[["Value"]],
        fill = .data[["cell_type_dataset"]]))

    # Add either boxplot or violin plot based on the shape parameter
    if(shape == "box") {
        plot <- plot + ggplot2::geom_boxplot(alpha = 0.7,
                                             outlier.shape = NA,
                                             width = 0.7)
    } else { # shape == "violin"
        plot <- plot + ggplot2::geom_violin(alpha = 0.7,
                                            trim = FALSE,
                                            width = 0.7)
    }

    # Continue with common plot elements
    plot <- plot +
        ggplot2::facet_wrap(~ .data[["PC"]], scales = "free",
                            labeller = ggplot2::labeller(PC = pc_labels)) +
        ggplot2::scale_fill_manual(values = cell_type_colors,
                                   name = "Cell Types") +
        ggplot2::labs(x = "", y = "PCA Score") +
        ggplot2::theme_bw() +
        ggplot2::theme(
            strip.background = ggplot2::element_rect(
                fill = "white", color = "black", linewidth = 0.5),
            panel.grid.minor = ggplot2::element_blank(),
            panel.grid.major = ggplot2::element_line(color = "gray",
                                                     linetype = "dotted"),
            plot.title = ggplot2::element_text(size = 14,
                                               face = "bold", hjust = 0.5),
            axis.title = ggplot2::element_text(size = 12),
            axis.text = ggplot2::element_text(size = 10),
            axis.text.x = ggplot2::element_text(angle = 45,
                                                hjust = 1,
                                                size = 10))

    # Return the plot
    return(plot)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics) 

# Load example datasets
data("reference_data")
data("query_data")

# Test cases
test_that("boxplotPCA correctly handles typical inputs", {
    
    # Test with typical input
    plot <- boxplotPCA(query_data = query_data, 
                       reference_data = reference_data,
                       query_cell_type_col = "SingleR_annotation",
                       ref_cell_type_col = "expert_annotation",
                       cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                       pc_subset = 1:5)
    
    # Check if the result is a ggplot object
    expect_true("ggplot" %in% class(plot))
    
})

test_that("boxplotPCA handles NULL cell_types gracefully", {
    
    # Test with NULL cell_types
    plot <- boxplotPCA(query_data = query_data, 
                       reference_data = reference_data,
                       query_cell_type_col = "SingleR_annotation",
                       ref_cell_type_col = "expert_annotation",
                       pc_subset = 1:5)
    
    # Check if the result is a ggplot object
    expect_true("ggplot" %in% class(plot))
    
})

test_that("boxplotPCA handles invalid cell_types gracefully", {
    
    # Test with invalid cell_types
    expect_error(boxplotPCA(query_data = query_data, 
                            reference_data = reference_data,
                            query_cell_type_col = "SingleR_annotation",
                            ref_cell_type_col = "expert_annotation",
                            cell_types = c("InvalidCellType"),
                            pc_subset = 1:5))
    
})

test_that("boxplotPCA correctly handles single principal component subset", {
    
    # Test with single principal component subset
    plot <- boxplotPCA(query_data = query_data, 
                       reference_data = reference_data,
                       query_cell_type_col = "SingleR_annotation",
                       ref_cell_type_col = "expert_annotation",
                       cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                       pc_subset = 1)
    
    # Check if the result is a ggplot object
    expect_true("ggplot" %in% class(plot))

})

#' @title Plot gene expression distribution from overall and cell type-specific perspective
#'
#' @description
#' This function generates density plots to visualize the distribution of gene expression values
#' for a specific gene across the overall dataset and within a specified cell type.
#'
#' @details
#' This function generates density plots to compare the distribution of a specific marker
#' gene between reference and query datasets. The aim is to inspect the alignment of gene expression
#' levels as a surrogate for dataset similarity. Similar distributions suggest a good alignment,
#' while differences may indicate discrepancies or incompatibilities between the datasets.
#'
#' Multiple normalization options are available:
#' - "z_score": Standard z-score normalization within each dataset
#' - "min_max": Min-max scaling to [0,1] range within each dataset
#' - "rank": Maps values to quantile ranks (0-100 scale)
#' - "none": No transformation (preserves original scale differences)
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param cell_type A cell type to plot (e.g., c("T-cell", "B-cell")).
#' @param gene_name The gene name for which the distribution is to be visualized.
#' @param normalization Method for normalizing expression values. Options: "z_score" (default), "min_max", "rank", "none".
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is NULL.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is NULL.
#'
#' @return A ggplot object containing density plots comparing reference and query distributions.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Note: Users can use SingleR or any other method to obtain the cell type annotations.
#' plotMarkerExpression(reference_data = reference_data,
#'                      query_data = query_data,
#'                      ref_cell_type_col = "expert_annotation",
#'                      query_cell_type_col = c("expert_annotation", "SingleR_annotation")[1],
#'                      gene_name = "CD8A",
#'                      cell_type = "CD4",
#'                      normalization = "z_score")
#'
#' @importFrom SummarizedExperiment assay
#' @importFrom stats sd
#' @import SingleCellExperiment
#'
# Function to plot the expression of a marker
plotMarkerExpression <- function(query_data,
                                 reference_data,
                                 ref_cell_type_col,
                                 query_cell_type_col,
                                 cell_type,
                                 gene_name,
                                 assay_name = "logcounts",
                                 normalization = c("z_score", "min_max", "rank", "none"),
                                 max_cells_query = NULL,
                                 max_cells_ref = NULL) {

    # Match normalization argument
    normalization <- match.arg(normalization)

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Select cell types
    cell_type <- selectCellTypes(query_data = query_data,
                                 reference_data = reference_data,
                                 query_cell_type_col = query_cell_type_col,
                                 ref_cell_type_col = ref_cell_type_col,
                                 cell_types = cell_type,
                                 dual_only = TRUE,
                                 n_cell_types = NULL)

    # Downsample query and reference data
    query_data <- downsampleSCE(sce_object = query_data,
                                cell_type_col = query_cell_type_col,
                                max_cells = max_cells_query)
    reference_data <- downsampleSCE(sce_object = reference_data,
                                    cell_type_col = ref_cell_type_col,
                                    max_cells = max_cells_ref)

    # Check if gene_name is present in both query_data and reference_data
    if (!(gene_name %in% rownames(assay(query_data)) &&
          gene_name %in% rownames(assay(reference_data)))) {
        stop("gene_name: \'",
             gene_name,
             "\' is not present in the row names of both \'query_data\' and \'reference_data\'.")
    }

    # Get expression of the specified gene for reference and query datasets
    ref_gene_expression <- assay(reference_data, assay_name)[gene_name, ]
    query_gene_expression <- assay(query_data, assay_name)[gene_name, ]
    ref_gene_expression_specific <- assay(
        reference_data, assay_name)[gene_name,
                                    which(reference_data[[ref_cell_type_col]] %in%
                                              cell_type)]
    query_gene_expression_specific <- assay(
        query_data, assay_name)[gene_name,
                                which(query_data[[query_cell_type_col]] %in%
                                          cell_type)]

    # Transformation functions
    .quantileTransformation <- function(x) {
        return(100 * rank(x, ties.method = "average") / length(x))
    }

    .zScoreTransformation <- function(x) {
        if(sd(x) == 0) return(rep(0, length(x)))
        return((x - mean(x)) / sd(x))
    }

    .minMaxTransformation <- function(x) {
        if(max(x) == min(x)) return(rep(0.5, length(x)))
        return((x - min(x)) / (max(x) - min(x)))
    }

    # Apply selected normalization
    if (normalization == "rank") {
        ref_gene_expression_norm <-
            .quantileTransformation(ref_gene_expression)
        query_gene_expression_norm <-
            .quantileTransformation(query_gene_expression)
        ref_gene_expression_specific_norm <-
            .quantileTransformation(ref_gene_expression_specific)
        query_gene_expression_specific_norm <-
            .quantileTransformation(query_gene_expression_specific)
        x_label <-
            paste("Quantile Rank Normalized Gene Expression:", gene_name)

    } else if (normalization == "z_score") {
        ref_gene_expression_norm <-
            .zScoreTransformation(ref_gene_expression)
        query_gene_expression_norm <-
            .zScoreTransformation(query_gene_expression)
        ref_gene_expression_specific_norm <-
            .zScoreTransformation(ref_gene_expression_specific)
        query_gene_expression_specific_norm <-
            .zScoreTransformation(query_gene_expression_specific)
        x_label <-
            paste("Z-Score Normalized Gene Expression:", gene_name)

    } else if (normalization == "min_max") {
        ref_gene_expression_norm <-
            .minMaxTransformation(ref_gene_expression)
        query_gene_expression_norm <-
            .minMaxTransformation(query_gene_expression)
        ref_gene_expression_specific_norm <-
            .minMaxTransformation(ref_gene_expression_specific)
        query_gene_expression_specific_norm <-
            .minMaxTransformation(query_gene_expression_specific)
        x_label <- paste("Min-Max Normalized Gene Expression:", gene_name)
    } else if (normalization == "none") {
        ref_gene_expression_norm <-
            ref_gene_expression
        query_gene_expression_norm <-
            query_gene_expression
        ref_gene_expression_specific_norm <-
            ref_gene_expression_specific
        query_gene_expression_specific_norm <-
            query_gene_expression_specific
        x_label <-
            paste("Log-Normalized Gene Expression:", gene_name)

    }

    # Create a combined vector of gene expression values
    combined_gene_expression <- c(
        ref_gene_expression_norm,
        query_gene_expression_norm,
        ref_gene_expression_specific_norm,
        query_gene_expression_specific_norm)

    # Create a grouping vector for dataset types
    dataset_types <- rep(c("Reference", "Query", "Reference", "Query"),
                         times = c(
                             length(ref_gene_expression),
                             length(query_gene_expression),
                             length(ref_gene_expression_specific),
                             length(query_gene_expression_specific)))

    # Combine the gene expression values and dataset types
    marker_data <- data.frame(
        GeneExpression = combined_gene_expression,
        Dataset = dataset_types,
        plot_type = rep(c("Overall Distribution", "Cell Type-Specific Distribution"),
                        times = c(length(ref_gene_expression) +
                                      length(query_gene_expression),
                                  length(ref_gene_expression_specific) +
                                      length(query_gene_expression_specific))))
    marker_data[["Dataset"]] <- factor(marker_data[["Dataset"]],
                                       levels = c("Query", "Reference"))


    # Create a stacked density plot
    plot_obj <- ggplot2::ggplot(
        marker_data,
        ggplot2::aes(x = .data[["GeneExpression"]],
                     y = .data[["Dataset"]],
                     fill = .data[["Dataset"]])) +
        ggridges::geom_density_ridges(
            alpha = 0.7,
            scale = 1,
            rel_min_height = 0.01
        ) +
        ggplot2::scale_y_discrete(
            expand = ggplot2::expansion(mult = c(0.02, 0.02))
        ) +
        ggplot2::scale_fill_manual(
            values = c("Query" = "#B565D8", "Reference" = "#5A9BD8"),
            name = "Dataset"
        ) +
        ggplot2::facet_wrap(~ .data[["plot_type"]], scales = "free") +
        ggplot2::labs(title = NULL,
                      x = x_label,
                      y = "") +
        ggplot2::theme_bw() +
        ggplot2::theme(
            panel.grid.minor = ggplot2::element_blank(),
            panel.grid.major = ggplot2::element_line(color = "gray",
                                                     linetype = "dotted"),
            panel.grid.major.y = ggplot2::element_blank(),
            strip.background = ggplot2::element_rect(fill = "white",
                                                     color = "black"),
            axis.title = ggplot2::element_text(size = 12),
            axis.text = ggplot2::element_text(size = 10),
            axis.text.y = ggplot2::element_blank(),
            axis.ticks.y = ggplot2::element_blank())
    return(plot_obj)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

test_that("plotMarkerExpression generates density plots correctly", {
    # Generate plot using the function
    p1 <- plotMarkerExpression(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = "SingleR_annotation",
        gene_name = "VPREB3",
        cell_type = c("B_and_plasma")
    )
    
    # Check if output is a ggplot object
    expect_s3_class(p1, "ggplot")
    
})

test_that("plotMarkerExpression handles invalid input gracefully", {
    # Test with non-existent column names in query_data or reference_data
    expect_error(plotMarkerExpression(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "invalid_column",
        query_cell_type_col = "SingleR_annotation",
        gene_name = "VPREB3",
        cell_type = c("B_and_plasma")
    ))
    
    # Test with non-existent gene_name
    expect_error(plotMarkerExpression(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = "SingleR_annotation",
        gene_name = "invalid_gene_name",
        cell_type = c("B_and_plasma")
    ))
})

## ----setup, include = FALSE, fig.show='hide'----------------------------------
knitr::knit_hooks$set(pngquant = knitr::hook_pngquant)

knitr::opts_chunk$set(
collapse = TRUE,
comment = "#>",
dev = "ragg_png",
dpi = 72,
fig.retina = 2,
fig.align = "center",
out.width = "100%",
pngquant = "--speed=1 --quality=1-5"
)

## ----message=FALSE, fig.show='hide'-------------------------------------------
# Load library
library(scDiagnostics)

# Load datasets
data("reference_data")
data("query_data")

# Set seed for reproducibility
set.seed(0)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Perform anomaly detection
anomaly_output <- detectAnomaly(reference_data = reference_data, 
                                query_data = query_data, 
                                ref_cell_type_col = "expert_annotation", 
                                query_cell_type_col = "SingleR_annotation",
                                pc_subset = 1:5,
                                n_tree = 500,
                                anomaly_threshold = 0.6)

# Plot the output for the "CD4" cell type
plot(anomaly_output, 
     cell_type = "CD8", 
     pc_subset = 1:5, 
     data_type = "query")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Perform anomaly detection
anomaly_output <- detectAnomaly(reference_data = reference_data, 
                                query_data = query_data, 
                                ref_cell_type_col = "expert_annotation", 
                                query_cell_type_col = "SingleR_annotation",
                                pc_subset = 1:5,
                                n_tree = 500,
                                anomaly_threshold = 0.6)

# Plot the global anomaly scores
plot(anomaly_output, 
     cell_type = NULL,  # Plot all cell types
     pc_subset = 1:5, 
     data_type = "query")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Detect anomalies and select top anomalies for further analysis
anomaly_output <- detectAnomaly(reference_data = reference_data, 
                                query_data = query_data,
                                ref_cell_type_col = "expert_annotation", 
                                query_cell_type_col = "SingleR_annotation",
                                pc_subset = 1:10,
                                n_tree = 500,
                                anomaly_threshold = 0.5)

# Select the top 6 anomalies based on the anomaly scores
top6_anomalies <- names(sort(anomaly_output$Combined$reference_anomaly_scores, 
                             decreasing = TRUE)[1:6])

# Calculate cosine similarity between the top anomalies and top 25 PCs
cosine_similarities <- calculateCellSimilarityPCA(reference_data, 
                                                  cell_names = top6_anomalies, 
                                                  pc_subset = 1:25, 
                                                  n_top_vars = 50)

# Plot the cosine similarities across PCs
round(cosine_similarities[, paste0("PC", 15:25)], 2)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Compute cell distances
distance_data <- calculateCellDistances(
    query_data = query_data,
    reference_data = reference_data,
    query_cell_type_col = "SingleR_annotation",
    ref_cell_type_col = "expert_annotation",
    pc_subset = 1:10
)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Identify top 6 anomalies for CD4 cells
cd4_anomalies <- detectAnomaly(
  reference_data = reference_data,
  query_data = query_data,
  query_cell_type_col = "SingleR_annotation",
  ref_cell_type_col = "expert_annotation",
  pc_subset = 1:10,
  n_tree = 500,
  anomaly_threshold = 0.5)

# Top 6 CD4 anomalies
cd4_top6_anomalies <- names(sort(cd4_anomalies$CD4$query_anomaly_scores, decreasing = TRUE)[1:6])

# Plot distances for CD4 and CD8 cells
plot(distance_data, ref_cell_type = "CD4", cell_names = cd4_top6_anomalies)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Plot distances for CD8 cells
plot(distance_data, ref_cell_type = "CD8", cell_names = cd4_top6_anomalies)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Compute similarity measures for top 6 CD4 anomalies
overlap_measures <- calculateCellDistancesSimilarity(
  query_data = query_data,
  reference_data = reference_data,
  cell_names = cd4_top6_anomalies,
  query_cell_type_col = "SingleR_annotation",
  ref_cell_type_col = "expert_annotation",
  pc_subset = 1:10)
overlap_measures

## ----SessionInfo, echo=FALSE, message=FALSE, warning=FALSE, comment=NA, fig.show='hide'----
options(width = 80) 
sessionInfo()

## ----setup, include = FALSE, fig.show='hide'----------------------------------
knitr::knit_hooks$set(pngquant = knitr::hook_pngquant)

knitr::opts_chunk$set(
collapse = TRUE,
comment = "#>",
dev = "ragg_png",
dpi = 72,
fig.retina = 2,
fig.align = "center",
out.width = "100%",
pngquant = "--speed=1 --quality=5-10"
)

## ----message=FALSE, fig.show='hide'-------------------------------------------
# Load library
library(scDiagnostics)

# Load datasets
data("reference_data")
data("query_data")
data("qc_data")

# Set seed for reproducibility
set.seed(0)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Generate the MDS scatter plot with cell type coloring
plotCellTypeMDS(query_data = query_data, 
                reference_data = reference_data, 
                cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                query_cell_type_col = "SingleR_annotation", 
                ref_cell_type_col = "expert_annotation")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Generate the MDS scatter plot with cell type coloring
plotCellTypePCA(query_data = query_data, 
                reference_data = reference_data,
                cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                query_cell_type_col = "SingleR_annotation", 
                ref_cell_type_col = "expert_annotation", 
                pc_subset = 1:3)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Plot the PCA data as boxplots
boxplotPCA(query_data = query_data, 
           reference_data = reference_data,
           cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
           query_cell_type_col = "SingleR_annotation", 
           ref_cell_type_col = "expert_annotation", 
           pc_subset = 1:3)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Compute important variables for all pairwise cell comparisons
disc_output <- calculateDiscriminantSpace(
  reference_data = reference_data,
  query_data = query_data, 
  query_cell_type_col = "SingleR_annotation", 
  ref_cell_type_col = "expert_annotation")

# Generate scatter and boxplot
plot(disc_output, plot_type = "scatterplot")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Perform discriminant analysis and projection
discriminant_results <- calculateDiscriminantSpace(
  reference_data = reference_data,
  query_data = query_data,
  ref_cell_type_col = "expert_annotation",
  query_cell_type_col = "SingleR_annotation",
  calculate_metrics = TRUE,  # Compute Mahalanobis distance and cosine similarity
  alpha = 0.01  # Significance level for Mahalanobis distance cutoff
)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Extract Mahalanobis distances
mahalanobis_distances <- discriminant_results$query_mahalanobis_dist

# Identify anomalies based on a threshold
threshold <- discriminant_results$mahalanobis_crit  # Use the computed cutoff value
anomalies <- mahalanobis_distances[mahalanobis_distances > threshold]

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Load ggplot2 for visualization
library(ggplot2)

# Convert distances to a data frame
distances_df <- data.frame(Distance = mahalanobis_distances,
                           Anomaly = ifelse(mahalanobis_distances > threshold, "Anomaly", "Normal"))

# Plot histogram of Mahalanobis distances
ggplot(distances_df, aes(x = Distance, fill = Anomaly)) +
  geom_histogram(binwidth = 0.5, position = "identity", alpha = 0.7) +
  labs(title = "Histogram of Mahalanobis Distances",
       x = "Mahalanobis Distance",
       y = "Frequency") +
  theme_bw()

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Calculate the SIR space projections
sir_output <- calculateSIRSpace(
  reference_data = reference_data,
  query_data = query_data,
  query_cell_type_col = "SingleR_annotation",
  ref_cell_type_col = "expert_annotation",
  multiple_cond_means = TRUE
)

# Plot the SIR space projections
plot(sir_output,
     sir_subset = 1:5,
     cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
     lower_facet = "scatter",
     diagonal_facet = "boxplot",
     upper_facet = "blank")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Plot the top contributing markers
plot(sir_output, 
     plot_type = "loadings", 
     sir_subset = 1:5,
     n_top = 10)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Visualize VPREB3 gene on a PCA plot
plotGeneExpressionDimred(sce_object = query_data,
                         method = "PCA",
                         pc_subset = 1:5,
                         feature = "NKG7",
                         cell_types = "CD4",
                         cell_type_col = "SingleR_annotation")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
plotMarkerExpression(
    reference_data = reference_data,
    query_data = query_data,
    ref_cell_type_col = "expert_annotation",
    query_cell_type_col = "SingleR_annotation",
    gene_name = "NKG7",
    cell_type = "CD4",
    normalization = "z_score"
)

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Remove cell types with very few cells
qc_data_subset <- qc_data[, !(qc_data$SingleR_annotation 
                              %in% c("Chondrocytes", "DC", 
                                     "Neurons","Platelets"))]

# Generate scatter plot
p1 <- plotQCvsAnnotation(sce_object = qc_data_subset,
                         cell_type_col = "SingleR_annotation",
                         qc_col = "total",
                         score_col = "annotation_scores")
p1 + ggplot2::xlab("Library Size")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Generate histograms
histQCvsAnnotation(sce_object = query_data, 
                   cell_type_col = "SingleR_annotation", 
                   qc_col = "percent_mito", 
                   score_col = "annotation_scores")

## ----fig.height=5, fig.width=10, fig.show='hide'------------------------------
# Plot gene set scores on PCA
plotGeneSetScores(sce_object = query_data, 
                  cell_type_col = "SingleR_annotation",
                  cell_types = "CD8",
                  method = "PCA", 
                  score_col = "gene_set_scores", 
                  pc_subset = 1:3)

## ----SessionInfo, echo=FALSE, message=FALSE, warning=FALSE, comment=NA, fig.show='hide'----
options(width = 80) 
sessionInfo()

#' @title Compare Subspaces Spanned by Top Principal Components
#'
#' @description
#' This function compares the subspace spanned by the top principal components (PCs) in a reference dataset to that
#' in a query dataset. It computes the cosine similarity between the loadings of the top variables for each PC in
#' both datasets and provides a weighted cosine similarity score.
#'
#' @details
#' This function compares the subspace spanned by the top principal components (PCs) in a reference dataset
#' to that in a query dataset. It first computes the cosine similarity between the loadings of the top variables
#' for each PC in both datasets. The top cosine similarity scores are then selected, and their corresponding PC
#' indices are stored. Additionally, the function calculates the average percentage of variance explained by the
#' selected top PCs. Finally, it computes a weighted cosine similarity score based on the top cosine similarities
#' and the average percentage of variance explained.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param pc_subset A numeric vector specifying the subset of principal components (PCs) to compare. Default is the first five PCs.
#' @param n_top_vars An integer indicating the number of top loading variables to consider for each PC. Default is 50.
#'
#' @return A list containing the following components:
#'   \item{cosine_similarity}{A numeric vector of cosine values of principal angles.}
#'   \item{cosine_id}{A matrix showing which reference and query PCs were matched.}
#'   \item{var_explained_ref}{A numeric vector of variance explained by reference PCs.}
#'   \item{var_explained_query}{A numeric vector of variance explained by query PCs.}
#'   \item{var_explained_avg}{A numeric vector of average variance explained by each PC pair.}
#'   \item{weighted_cosine_similarity}{A numeric value representing the weighted cosine similarity.}
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{plot.comparePCASubspaceObject}}
#'
#' @examples
#' # Load libraries
#' library(scran)
#' library(scater)
#'
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Extract CD4 cells
#' ref_data_subset <- reference_data[, which(reference_data$expert_annotation == "CD4")]
#' query_data_subset <- query_data[, which(query_data$expert_annotation == "CD4")]
#'
#' # Selecting highly variable genes (can be customized by the user)
#' ref_top_genes <- getTopHVGs(ref_data_subset, n = 500)
#' query_top_genes <- getTopHVGs(query_data_subset, n = 500)
#'
#' # Intersect the gene symbols to obtain common genes
#' common_genes <- intersect(ref_top_genes, query_top_genes)
#' ref_data_subset <- ref_data_subset[common_genes,]
#' query_data_subset <- query_data_subset[common_genes,]
#'
#' # Run PCA on datasets separately
#' ref_data_subset <- runPCA(ref_data_subset)
#' query_data_subset <- runPCA(query_data_subset)
#'
#' # Compare PCA subspaces
#' subspace_comparison <- comparePCASubspace(query_data = query_data_subset,
#'                                           reference_data = ref_data_subset,
#'                                           query_cell_type_col = "expert_annotation",
#'                                           ref_cell_type_col = "expert_annotation",
#'                                           n_top_vars = 50,
#'                                           pc_subset = 1:5)
#'
#' # Plot output for PCA subspace comparison
#' plot(subspace_comparison)
#'
# Function to compare subspace spanned by top PCs in reference and query datasets
comparePCASubspace <- function(query_data,
                               reference_data,
                               query_cell_type_col,
                               ref_cell_type_col,
                               pc_subset = 1:5,
                               n_top_vars = 50){

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  unique_cell_type = TRUE,
                  pc_subset_query = pc_subset,
                  pc_subset_ref = pc_subset,
                  common_rotation_genes = TRUE)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Check if n_top_vars is a positive integer
    if (!is.numeric(n_top_vars) || n_top_vars <= 0 ||
        n_top_vars != as.integer(n_top_vars)) {
        stop("\'n_top_vars\' must be a positive integer.")
    }

    # Compute the cosine similarity (cosine of principal angle)
    cosine_similarity_result <- comparePCA(query_data = query_data,
                                           reference_data = reference_data,
                                           query_cell_type_col = query_cell_type_col,
                                           ref_cell_type_col = ref_cell_type_col,
                                           pc_subset = pc_subset,
                                           n_top_vars = n_top_vars,
                                           metric = "cosine")

    # Extract similarity matrix from the result
    cosine_similarity <- cosine_similarity_result[["similarity_matrix"]]

    # Vector to store top cosine similarities
    top_cosine <- numeric(length(pc_subset))
    # Matrix to store PC IDs for each top cosine similarity
    cosine_id <- matrix(NA, nrow = length(pc_subset), ncol = 2)
    colnames(cosine_id) <- c("Ref", "Query")

    # Looping to store top cosine similarities and PC IDs
    for(id in seq_len(length(pc_subset))){

        # Store data for top cosine
        top_ref <- which.max(apply(abs(cosine_similarity), 1, max))
        top_query <- which.max(abs(cosine_similarity)[top_ref,])
        top_cosine[id] <- abs(cosine_similarity)[top_ref, top_query]
        cosine_id[id,] <- c(top_ref, top_query)

        # Remove as candidate
        cosine_similarity[top_ref,] <- 0
        cosine_similarity[, top_query] <- 0
    }

    # Vector of variance explained - FIXED BUG AND ADDED INDIVIDUAL VALUES
    var_explained_ref_all <- attributes(
        reducedDim(reference_data, "PCA"))[["percentVar"]][pc_subset]
    var_explained_query_all <- attributes(
        reducedDim(query_data, "PCA"))[["percentVar"]][pc_subset]

    # Get variance explained for the matched PCs
    var_explained_ref <- var_explained_ref_all[cosine_id[, 1]]
    var_explained_query <- var_explained_query_all[cosine_id[, 2]]
    var_explained_avg <- (var_explained_ref + var_explained_query) / 2

    # Weighted cosine similarity score
    weighted_cosine_similarity <- sum(top_cosine * var_explained_avg)/100

    # Update class of return output
    output <- list(cosine_similarity = top_cosine,
                   cosine_id = cosine_id,
                   var_explained_ref = var_explained_ref,
                   var_explained_query = var_explained_query,
                   var_explained_avg = var_explained_avg,
                   weighted_cosine_similarity = weighted_cosine_similarity)
    class(output) <- c(class(output), "comparePCASubspaceObject")

    # Return cosine similarity output
    return(output)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Extract CD4 cells
reference_data_subset <- reference_data[, which(reference_data$expert_annotation == "CD4")]
query_data_subset <- query_data[, which(query_data$expert_annotation == "CD4")]

# Selecting highly variable genes (can be customized by the user)
ref_top_genes <- scran::getTopHVGs(reference_data_subset, n = 500)
query_top_genes <- scran::getTopHVGs(query_data_subset, n = 500)

# Intersect the gene symbols to obtain common genes
common_genes <- intersect(ref_top_genes, query_top_genes)
reference_data_subset <- reference_data_subset[common_genes,]
query_data_subset <- query_data_subset[common_genes,]

# Run PCA on datasets separately
reference_data_subset <- scater::runPCA(reference_data_subset)
query_data_subset <- scater::runPCA(query_data_subset)

# Unit tests for comparePCASubspace
test_that("comparePCASubspace returns expected output structure", {
    # Run the comparePCASubspace function
    subspace_comparison <- comparePCASubspace(query_data = query_data_subset,
                                              reference_data = reference_data_subset,
                                              query_cell_type_col = "expert_annotation",
                                              ref_cell_type_col = "expert_annotation",
                                              pc_subset = 1:5,
                                              n_top_vars = 50)

    # Check the class of the output
    expect_s3_class(subspace_comparison, "comparePCASubspaceObject")

    # Check the structure of the output list
    expect_true(is.list(subspace_comparison))
    expect_named(subspace_comparison, c("cosine_similarity", "cosine_id", "var_explained_ref", "var_explained_query", "var_explained_avg", "weighted_cosine_similarity"))
})

test_that("comparePCASubspace handles invalid n_top_vars argument", {
    # Check if function stops with invalid n_top_vars
    expect_error(comparePCASubspace(query_data = query_data_subset,
                                    reference_data = reference_data_subset,
                                    query_cell_type_col = "expert_annotation",
                                    ref_cell_type_col = "expert_annotation",
                                    pc_subset = 1:5,
                                    n_top_vars = -10))
})

test_that("comparePCASubspace handles missing common genes", {
    # Create a subset without common genes
    ref_subset_no_common <- reference_data[setdiff(rownames(reference_data), common_genes), ]
    query_subset_no_common <- query_data[setdiff(rownames(query_data), common_genes), ]

    # Check if function handles no common genes
    expect_error(comparePCASubspace(query_data = query_subset_no_common,
                                    reference_data = ref_subset_no_common,
                                    query_cell_type_col = "expert_annotation",
                                    ref_cell_type_col = "expert_annotation",
                                    pc_subset = 1:5,
                                    n_top_vars = 50))
})

test_that("comparePCASubspace calculates cosine similarities correctly", {
    # Run the comparePCASubspace function
    subspace_comparison <- comparePCASubspace(query_data = query_data_subset,
                                              reference_data = reference_data_subset,
                                              query_cell_type_col = "expert_annotation",
                                              ref_cell_type_col = "expert_annotation",
                                              pc_subset = 1:5,
                                              n_top_vars = 50)

    # Check that cosine_similarity contains numeric values
    expect_type(subspace_comparison$cosine_similarity, "double")
    # Check that cosine_similarity values are within range [0, 1]
    expect_true(all(subspace_comparison$cosine_similarity >= 0 & subspace_comparison$cosine_similarity <= 1))
})

test_that("comparePCASubspace returns correct dimensions for cosine_id", {
    # Run the comparePCASubspace function
    subspace_comparison <- comparePCASubspace(query_data = query_data_subset,
                                              reference_data = reference_data_subset,
                                              query_cell_type_col = "expert_annotation",
                                              ref_cell_type_col = "expert_annotation",
                                              pc_subset = 1:5,
                                              n_top_vars = 50)

    # Check the dimensions of the cosine_id matrix
    expect_equal(dim(subspace_comparison$cosine_id), c(5, 2))
})

test_that("comparePCASubspace returns variance explained components correctly", {
    # Run the comparePCASubspace function
    subspace_comparison <- comparePCASubspace(query_data = query_data_subset,
                                              reference_data = reference_data_subset,
                                              query_cell_type_col = "expert_annotation",
                                              ref_cell_type_col = "expert_annotation",
                                              pc_subset = 1:5,
                                              n_top_vars = 50)

    # Check that var_explained_ref is a numeric vector
    expect_type(subspace_comparison$var_explained_ref, "double")
    expect_equal(length(subspace_comparison$var_explained_ref), length(1:5))

    # Check that var_explained_query is a numeric vector
    expect_type(subspace_comparison$var_explained_query, "double")
    expect_equal(length(subspace_comparison$var_explained_query), length(1:5))

    # Check that var_explained_avg is a numeric vector
    expect_type(subspace_comparison$var_explained_avg, "double")
    expect_equal(length(subspace_comparison$var_explained_avg), length(1:5))
})

test_that("comparePCASubspace returns a valid weighted cosine similarity score", {
    # Run the comparePCASubspace function
    subspace_comparison <- comparePCASubspace(query_data = query_data_subset,
                                              reference_data = reference_data_subset,
                                              query_cell_type_col = "expert_annotation",
                                              ref_cell_type_col = "expert_annotation",
                                              pc_subset = 1:5,
                                              n_top_vars = 50)

    # Check that weighted_cosine_similarity is a numeric value
    expect_type(subspace_comparison$weighted_cosine_similarity, "double")
    # Check that weighted_cosine_similarity is within range [0, 1]
    expect_true(subspace_comparison$weighted_cosine_similarity >= 0 & subspace_comparison$weighted_cosine_similarity <= 1)
})

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Extract CD4 cells
reference_data_subset <- reference_data[, which(reference_data$expert_annotation == "CD4")]
query_data_subset <- query_data[, which(query_data$expert_annotation == "CD4")]

# Selecting highly variable genes (can be customized by the user)
ref_top_genes <- scran::getTopHVGs(reference_data_subset, n = 500)
query_top_genes <- scran::getTopHVGs(query_data_subset, n = 500)

# Intersect the gene symbols to obtain common genes
common_genes <- intersect(ref_top_genes, query_top_genes)
reference_data_subset <- reference_data_subset[common_genes,]
query_data_subset <- query_data_subset[common_genes,]

# Run PCA on datasets separately
reference_data_subset <- scater::runPCA(reference_data_subset)
query_data_subset <- scater::runPCA(query_data_subset)

# Compare PCA subspaces
subspace_comparison <- comparePCASubspace(reference_data = reference_data_subset,
                                          query_data = query_data_subset,
                                          query_cell_type_col = "expert_annotation",
                                          ref_cell_type_col = "expert_annotation",
                                          n_top_vars = 50,
                                          pc_subset = 1:5)

test_that("plot.comparePCASubspaceObject generates plots correctly", {
    # Generate plot using the function
    p1 <- plot(subspace_comparison)

    # Check if output is a ggplot object
    expect_s3_class(p1, "ggplot")
})

test_that("plot.comparePCASubspaceObject creates expected plot elements", {
    # Generate plot using the function
    p1 <- plot(subspace_comparison)

    # Check if output is a ggplot object
    expect_s3_class(p1, "ggplot")

    # Check that the plot has the expected layers
    expect_true(length(p1$layers) > 0)

    # Check that the plot has appropriate labels
    expect_true(!is.null(p1$labels$title))
    expect_true(!is.null(p1$labels$subtitle))
    expect_true(!is.null(p1$labels$x))
    expect_true(!is.null(p1$labels$y))
})

#' @title Calculate Maximum Mean Discrepancy P-Values for Two-Sample Comparison
#'
#' @description
#' This function performs the Maximum Mean Discrepancy (MMD) test for comparing
#' distributions between two samples in PCA space using a custom implementation
#' with permutation testing for better sensitivity.
#'
#' @details
#' The function performs the following steps:
#' \enumerate{
#'   \item Projects the data into the PCA space.
#'   \item Subsets the data to the specified cell types and principal components.
#'   \item Performs a custom MMD test with permutation-based p-values for each cell type.
#' }
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.
#' @param pc_subset A numeric vector specifying which principal components to include in the plot. Default is PC1 to PC5.
#' @param n_permutation Number of permutations for p-value calculation. Default is 100.
#' @param kernel_type Type of kernel to use. Options are "gaussian" (default) or "linear".
#' @param sigma Bandwidth parameter for Gaussian kernel. If NULL, uses median heuristic.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#'
#' @return A named vector of p-values from the MMD test for each cell type.
#'
#' @references Gretton, A., Borgwardt, K. M., Rasch, M. J., Schölkopf, B., & Smola, A. (2012).
#' "A kernel two-sample test". Journal of Machine Learning Research, 13(1), 723-773.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Calculate MMD p-values (with query data)
#' mmd_test <- calculateMMDPValue(reference_data = reference_data,
#'                               query_data = query_data,
#'                               ref_cell_type_col = "expert_annotation",
#'                               query_cell_type_col = "SingleR_annotation",
#'                               cell_types = c("CD4", "CD8"),
#'                               pc_subset = 1:5,
#'                               n_permutation = 30)
#' mmd_test
#'
#' @importFrom stats median
#'
# Function to perform MMD test for two-sample comparison
calculateMMDPValue <- function(query_data,
                               reference_data,
                               query_cell_type_col,
                               ref_cell_type_col,
                               cell_types = NULL,
                               pc_subset = 1:5,
                               n_permutation = 100,
                               kernel_type = "gaussian",
                               sigma = NULL,
                               assay_name = "logcounts",
                               max_cells_query = 5000,
                               max_cells_ref = 5000) {

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = TRUE,
                                  n_cell_types = NULL)

    # Get the projected PCA data
    pca_output <- projectPCA(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = query_cell_type_col,
                             ref_cell_type_col = ref_cell_type_col,
                             cell_types = cell_types,
                             pc_subset = pc_subset,
                             assay_name = assay_name,
                             max_cells_ref = max_cells_ref,
                             max_cells_query = max_cells_query)

    # Set data for MMD test
    cell_list <- split(pca_output, pca_output[["cell_type"]])

    # Set the PC variables
    pc_vars <- paste0("PC", pc_subset)

    # Calculate MMD p-values for each cell type
    p_values <- vector("numeric", length = length(cell_types))
    names(p_values) <- cell_types

    for(cell_type in cell_types){
        dataset_ind <- cell_list[[cell_type]][, "dataset"] == "Reference"

        X <- as.matrix(cell_list[[cell_type]][dataset_ind, pc_vars])
        Y <- as.matrix(cell_list[[cell_type]][!dataset_ind, pc_vars])

        # Skip if either group is too small
        if(nrow(X) < 3 || nrow(Y) < 3) {
            p_values[cell_type] <- NA
            next
        }

        # Adaptive permutation testing - stop early if clear result
        observed_mmd <- computeMMDStatistic(X, Y, kernel_type, sigma)

        combined_data <- rbind(X, Y)
        n_total <- nrow(combined_data)
        n_X <- nrow(X)

        # Adaptive permutation with early stopping
        min_perms <- min(100, n_permutation)
        extreme_count <- 0

        for(i in seq_len(n_permutation)) {
            perm_indices <- sample.int(n_total)
            X_perm <-
                combined_data[perm_indices[seq_len(n_X)], , drop = FALSE]
            Y_perm <-
                combined_data[perm_indices[seq(n_X + 1, n_total)], , drop = FALSE]
            perm_stat <-
                computeMMDStatistic(X_perm, Y_perm, kernel_type, sigma)

            if(perm_stat >= observed_mmd) {
                extreme_count <- extreme_count + 1
            }

            # Early stopping if we have enough evidence
            if(i >= min_perms) {
                current_pval <- (extreme_count + 1) / (i + 1)
                # Stop early if p-value is clearly < 0.01 or > 0.1
                if(current_pval < 0.005 || current_pval > 0.15) {
                    p_values[cell_type] <- (extreme_count + 1) / (i + 1)
                    break
                }
            }

            # If we've done all permutations
            if(i == n_permutation) {
                p_values[cell_type] <- (extreme_count + 1) /
                    (n_permutation + 1)
            }
        }
    }

    return(p_values)
}

#' @title Compute Maximum Mean Discrepancy Statistic
#'
#' @description
#' Compute the Maximum Mean Discrepancy (MMD) statistic between two datasets
#' using either Gaussian or linear kernels for distribution comparison.
#'
#' @details
#' This function calculates the MMD statistic, which measures the distance between
#' two probability distributions by comparing their embeddings in a reproducing
#' kernel Hilbert space (RKHS). For the Gaussian kernel, an optimized median
#' heuristic is used to estimate the bandwidth parameter sigma when not provided.
#' The linear kernel provides a computationally faster alternative.
#'
#' The MMD statistic is computed as:
#' MMD^2 = E[k(X,X')] + E[k(Y,Y')] - 2*E[k(X,Y)]
#' where k is the chosen kernel function.
#'
#' @param X A numeric matrix representing the first dataset, where rows are observations
#' and columns are features.
#' @param Y A numeric matrix representing the second dataset, where rows are observations
#' and columns are features.
#' @param kernel_type A character string specifying the kernel type. Options are
#' "gaussian" for RBF kernel or "linear" for linear kernel. Default is "gaussian".
#' @param sigma A numeric value specifying the bandwidth parameter for the Gaussian kernel.
#' If NULL, it is estimated using the median heuristic. Default is NULL.
#'
#' @keywords internal
#'
#' @return A numeric value representing the MMD^2 statistic between the two datasets.
#'
#' @author
#' Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
# Function to compute MMD statistic
computeMMDStatistic <- function(X, Y,
                                kernel_type = "gaussian",
                                sigma = NULL) {

    # Get sample sizes
    n <- nrow(X)
    m <- nrow(Y)

    # Gaussian kernel computation
    if (kernel_type == "gaussian") {

        # Estimate sigma using median heuristic if not provided
        if (is.null(sigma)) {
            # Efficient median heuristic using sampled subset of distances
            combined <- rbind(X, Y)
            n_combined <- nrow(combined)

            # Sample maximum of 1000 pairs to avoid computational burden
            max_pairs <- min(1000, n_combined * (n_combined - 1) / 2)
            sampled_dists <- numeric(max_pairs)

            for(k in seq_len(max_pairs)) {
                i <- sample.int(n_combined, 1)
                j <- sample.int(n_combined, 1)
                if(i != j) {
                    sampled_dists[k] <- sqrt(sum((combined[i, ] -
                                                      combined[j, ])^2))
                }
            }
            sigma <- median(sampled_dists[sampled_dists > 0])
        }

        # Pre-compute kernel parameter for efficiency
        sigma_sq_2_inv <- 1 / (2 * sigma^2)

        # Initialize kernel sums
        K_XX_sum <- 0
        K_YY_sum <- 0
        K_XY_sum <- 0

        # Compute K(X,X) sum - kernel evaluations within first dataset
        for(i in seq_len(n - 1)) {
            X_i <- X[i, ]
            for(j in seq(i + 1, n)) {
                dist_sq <- sum((X_i - X[j, ])^2)
                K_XX_sum <- K_XX_sum + 2 * exp(-dist_sq * sigma_sq_2_inv)
            }
        }

        # Compute K(Y,Y) sum - kernel evaluations within second dataset
        for(i in seq_len(m - 1)) {
            Y_i <- Y[i, ]
            for(j in seq(i + 1, m)) {
                dist_sq <- sum((Y_i - Y[j, ])^2)
                K_YY_sum <- K_YY_sum + 2 * exp(-dist_sq * sigma_sq_2_inv)
            }
        }

        # Compute K(X,Y) sum - kernel evaluations between datasets
        for(i in seq_len(n)) {
            X_i <- X[i, ]
            for(j in seq_len(m)) {
                dist_sq <- sum((X_i - Y[j, ])^2)
                K_XY_sum <- K_XY_sum + exp(-dist_sq * sigma_sq_2_inv)
            }
        }

    } else {
        # Linear kernel - computationally efficient alternative
        XX <- tcrossprod(X)
        YY <- tcrossprod(Y)
        XY <- tcrossprod(X, Y)

        # Compute kernel sums excluding diagonal elements for within-dataset terms
        K_XX_sum <- sum(XX) - sum(diag(XX))
        K_YY_sum <- sum(YY) - sum(diag(YY))
        K_XY_sum <- sum(XY)
    }

    # Calculate MMD^2 statistic using unbiased estimator
    mmd_stat <- K_XX_sum/(n*(n-1)) + K_YY_sum/(m*(m-1)) - 2*K_XY_sum/(n*m)

    return(mmd_stat)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Test cases
test_that("calculateMMDPValue correctly handles typical inputs", {

    # Basic functionality test
    result <- calculateMMDPValue(reference_data = reference_data,
                                 query_data = query_data,
                                 ref_cell_type_col = "expert_annotation",
                                 query_cell_type_col = "expert_annotation",
                                 n_permutation = 5)

    # Check that result is a named numeric vector
    expect_true(is.numeric(result))
    expect_true(!is.null(names(result)))

    # Check that p-values are in valid range [0, 1] or NA
    valid_pvals <- result[!is.na(result)]
    expect_true(all(valid_pvals >= 0 & valid_pvals <= 1))

    # Check that cell type names are present
    expect_true(length(names(result)) > 0)
})

test_that("calculateMMDPValue works with different parameters", {

    # Test with specific cell types
    query_types <- unique(colData(query_data)[["expert_annotation"]])
    ref_types <- unique(colData(reference_data)[["expert_annotation"]])
    common_types <- intersect(query_types, ref_types)

    if (length(common_types) > 1) {
        selected_types <- common_types[1:min(2, length(common_types))]

        result <- calculateMMDPValue(reference_data = reference_data,
                                     query_data = query_data,
                                     ref_cell_type_col = "expert_annotation",
                                     query_cell_type_col = "expert_annotation",
                                     cell_types = selected_types,
                                     n_permutation = 5)

        expect_true(is.numeric(result))
        expect_true(all(names(result) %in% selected_types))
    }

    # Test with different PC subset
    result2 <- calculateMMDPValue(reference_data = reference_data,
                                  query_data = query_data,
                                  ref_cell_type_col = "expert_annotation",
                                  query_cell_type_col = "expert_annotation",
                                  pc_subset = 1:3,
                                  n_permutation = 5)

    expect_true(is.numeric(result2))

    # Test with fewer permutations
    result3 <- calculateMMDPValue(reference_data = reference_data,
                                  query_data = query_data,
                                  ref_cell_type_col = "expert_annotation",
                                  query_cell_type_col = "expert_annotation",
                                  n_permutation = 5)

    expect_true(is.numeric(result3))
})

test_that("calculateMMDPValue handles edge cases", {

    # Test with very small permutation number
    result <- calculateMMDPValue(reference_data = reference_data,
                                 query_data = query_data,
                                 ref_cell_type_col = "expert_annotation",
                                 query_cell_type_col = "expert_annotation",
                                 n_permutation = 5)

    expect_true(is.numeric(result))

    # Test error with invalid column names
    expect_error(
        calculateMMDPValue(reference_data = reference_data,
                           query_data = query_data,
                           ref_cell_type_col = "nonexistent_column",
                           query_cell_type_col = "expert_annotation",
                           n_permutation = 5)
    )

    # Test error with invalid assay name
    expect_error(
        calculateMMDPValue(reference_data = reference_data,
                           query_data = query_data,
                           ref_cell_type_col = "expert_annotation",
                           query_cell_type_col = "expert_annotation",
                           assay_name = "nonexistent_assay",
                           n_permutation = 5)
    )
})

#' @title Compute Average Pairwise Correlation between Cell Types
#'
#' @description
#' Computes the average pairwise correlations between specified cell types
#' in single-cell gene expression data.
#'
#' @details
#' This function operates on \code{\linkS4class{SingleCellExperiment}} objects,
#' ideal for single-cell analysis workflows. It calculates pairwise correlations between query and
#' reference cells using a specified correlation method, then averages these correlations for each
#' cell type pair. This function aids in assessing the similarity between cells in reference and query datasets,
#' providing insights into the reliability of cell type annotations in single-cell gene expression data.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.
#' @param pc_subset A numeric vector specifying which principal components to use in the analysis. Default is 1:10.
#' If set to \code{NULL} then no dimensionality reduction is performed and the assay data is used directly for computations.
#' @param correlation_method The correlation method to use for calculating pairwise correlations.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000
#'
#' @return A matrix containing the average pairwise correlation values.
#'         Rows and columns are labeled with the cell types. Each element
#'         in the matrix represents the average correlation between a pair
#'         of cell types.
#'
#' @seealso \code{\link{plot.calculateAveragePairwiseCorrelationObject}}
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Compute pairwise correlations
#' cor_matrix_avg <- calculateAveragePairwiseCorrelation(query_data = query_data,
#'                                                       reference_data = reference_data,
#'                                                       query_cell_type_col = "SingleR_annotation",
#'                                                       ref_cell_type_col = "expert_annotation",
#'                                                       cell_types = c("CD4", "CD8", "B_and_plasma"),
#'                                                       pc_subset = 1:10,
#'                                                       correlation_method = "spearman")
#'
#' # Visualize correlation output
#' plot(cor_matrix_avg)
#'
#' @import SingleCellExperiment
#' @importFrom stats cor
#'
#' @export
#'
# Function to calculate average pairwise correlation between cell types
calculateAveragePairwiseCorrelation <- function(
        query_data,
        reference_data,
        query_cell_type_col,
        ref_cell_type_col,
        cell_types = NULL,
        pc_subset = 1:10,
        correlation_method = c("spearman", "pearson"),
        assay_name = "logcounts",
        max_cells_query = 5000,
        max_cells_ref = 5000) {

    # Match correlation method argument
    correlation_method <- match.arg(correlation_method)

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = TRUE,
                                  n_cell_types = NULL)

    # Function to compute correlation between two cell types
    .computeCorrelation <- function(type1, type2) {

        if(!is.null(pc_subset)){
            # Project query data onto PCA space of reference data
            pca_output <- projectPCA(
                query_data = query_data,
                reference_data = reference_data,
                query_cell_type_col = query_cell_type_col,
                ref_cell_type_col = ref_cell_type_col,
                cell_types = cell_types,
                pc_subset = pc_subset,
                assay_name = assay_name,
                max_cells_ref = max_cells_ref,
                max_cells_query = max_cells_query)
            ref_mat <- pca_output[which(
                pca_output[["dataset"]] == "Reference" &
                    pca_output[["cell_type"]] == type2),
                paste0("PC", pc_subset)]
            query_mat <- pca_output[which(
                pca_output[["dataset"]] == "Query" &
                    pca_output[["cell_type"]] == type1),
                paste0("PC", pc_subset)]
        } else{

            # Subset query and reference data to the specified cell type
            query_subset <- query_data[, which(
                query_data[[query_cell_type_col]] == type1), drop = FALSE]
            ref_subset <- reference_data[, which(
                reference_data[[ref_cell_type_col]] == type2), drop = FALSE]

            query_mat <- t(as.matrix(assay(query_subset, assay_name)))
            ref_mat <- t(as.matrix(assay(ref_subset, assay_name)))
        }

        cor_matrix <- cor(t(query_mat), t(ref_mat),
                          method = correlation_method)
        return(mean(cor_matrix))
    }

    # Use outer to compute pairwise correlations
    cor_matrix_avg <- outer(cell_types, cell_types,
                            Vectorize(.computeCorrelation))

    # Assign cell type names to rows and columns
    rownames(cor_matrix_avg) <- paste0("Query-", cell_types)
    colnames(cor_matrix_avg) <- paste0("Ref-", cell_types)

    # Update class of output
    class(cor_matrix_avg) <- c(class(cor_matrix_avg),
                               "calculateAveragePairwiseCorrelationObject")

    return(cor_matrix_avg)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics) 

# Load data
data("reference_data")
data("query_data")

# Compute pairwise correlations
cor_matrix_avg <- calculateAveragePairwiseCorrelation(query_data = query_data, 
                                                      reference_data = reference_data, 
                                                      query_cell_type_col = "expert_annotation", 
                                                      ref_cell_type_col = "expert_annotation", 
                                                      cell_types = c("CD4", "CD8", "B_and_plasma"), 
                                                      pc_subset = 1:5,
                                                      correlation_method = "spearman")

test_that("plot.calculateAveragePairwiseCorrelation generates plots correctly", {
    # Generate plot using the function
    p1 <- plot(cor_matrix_avg)
    
    # Check if output is a ggplot object
    expect_s3_class(p1, "ggplot")
})

# Load necessary libraries
library(testthat)
library(scDiagnostics) 

# Load example datasets
data("reference_data")
data("query_data")

# Test cases
test_that("calculateAveragePairwiseCorrelation computes average pairwise correlations", {
    
    # Test with typical input
    cor_matrix_avg <- calculateAveragePairwiseCorrelation(query_data = query_data, 
                                                          reference_data = reference_data,
                                                          query_cell_type_col = "SingleR_annotation",
                                                          ref_cell_type_col = "expert_annotation",
                                                          cell_types = c("CD4", "CD8", "B_and_plasma"),
                                                          pc_subset = 1:10,
                                                          correlation_method = "pearson")
    
    # Check if the result is a matrix
    expect_true(is.matrix(cor_matrix_avg))
    
    # Check dimensions of the output matrix
    expect_equal(dim(cor_matrix_avg), c(length(c("Query-CD4", "Query-CD8", "Query-B_and_plasma")), 
                                        length(c("Ref-CD4", "Ref-CD8", "Ref-B_and_plasma"))))
    
    # Check specific values if needed
    
})

test_that("calculateAveragePairwiseCorrelation handles NULL cell_types gracefully", {
    
    # Test with NULL cell_types
    cor_matrix_avg <- calculateAveragePairwiseCorrelation(query_data = query_data, 
                                                          reference_data = reference_data,
                                                          query_cell_type_col = "SingleR_annotation",
                                                          ref_cell_type_col = "expert_annotation",
                                                          pc_subset = 1:10,
                                                          correlation_method = "spearman")
    
    # Check if the result is a matrix
    expect_true(is.matrix(cor_matrix_avg))
    
    # Check dimensions of the output matrix
    expect_equal(dim(cor_matrix_avg)[1], dim(cor_matrix_avg)[2])  # Square matrix
    
})

test_that("calculateAveragePairwiseCorrelation handles single cell type gracefully", {
    
    # Test with a single cell type
    cor_matrix_avg <- calculateAveragePairwiseCorrelation(query_data = query_data, 
                                                          reference_data = reference_data,
                                                          query_cell_type_col = "SingleR_annotation",
                                                          ref_cell_type_col = "expert_annotation",
                                                          cell_types = "CD4",
                                                          pc_subset = 1:10,
                                                          correlation_method = "pearson")
    
    # Check if the result is a matrix
    expect_true(is.matrix(cor_matrix_avg))
    
    # Check dimensions of the output matrix
    expect_equal(dim(cor_matrix_avg), c(1, 1))
    
})

test_that("calculateAveragePairwiseCorrelation handles invalid correlation methods", {
    
    # Test with an invalid correlation method
    expect_error(calculateAveragePairwiseCorrelation(query_data = query_data, 
                                                     reference_data = reference_data,
                                                     query_cell_type_col = "SingleR_annotation",
                                                     ref_cell_type_col = "expert_annotation",
                                                     cell_types = c("CD4", "CD8", "B_and_plasma"),
                                                     pc_subset = 1:10,
                                                     correlation_method = "invalid_method"))
    
})

#' @title Project Query Data onto a Unified Discriminant Space of Reference Data
#'
#' @description
#' This function projects query single-cell RNA-seq data onto a unified discriminant space defined by reference data. The reference data
#' is used to identify important variables across all cell types and compute discriminant vectors, which are then used to project both reference and query
#' data. Similarity between the query and reference projections can be assessed using cosine similarity and Mahalanobis distance.
#'
#' @details
#' The function performs the following steps:
#' \itemize{
#'   \item Identifies the top important variables to distinguish cell types from the reference data by taking the union of important variables from pairwise comparisons.
#'   \item Computes the Ledoit-Wolf shrinkage estimate of the covariance matrix for each cell type using these important genes.
#'   \item Constructs within-class and between-class scatter matrices.
#'   \item Solves the generalized eigenvalue problem to obtain discriminant vectors.
#'   \item Projects both reference and query data onto the unified discriminant space.
#'   \item Assesses similarity of the query data projection to the reference data using cosine similarity and Mahalanobis distance.
#' }
#'
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' If NULL, only the projected reference data is returned. Default is NULL.
#' @param ref_cell_type_col The column name in \code{reference_data} indicating cell type labels.
#' @param query_cell_type_col The column name in \code{query_data} indicating cell type labels.
#' @param cell_types A character vector specifying the cell types to include in the analysis. If NULL, all cell types are included.
#' @param n_tree An integer specifying the number of trees for the random forest used in variable importance calculation.
#' @param n_top An integer specifying the number of top variables to select based on importance scores from each pairwise comparison.
#' @param eigen_threshold A numeric value specifying the threshold for retaining eigenvalues in discriminant analysis.
#' @param calculate_metrics Parameter to determine if cosine similarity and Mahalanobis distance metrics should be computed. Default is FALSE.
#' @param alpha A numeric value specifying the significance level for Mahalanobis distance cutoff.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is NULL.
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is NULL.
#'
#' @return A list with the following components:
#' \item{discriminant_eigenvalues}{Eigenvalues from the discriminant analysis.}
#' \item{discriminant_eigenvectors}{Eigenvectors from the discriminant analysis.}
#' \item{ref_proj}{Reference data projected onto the discriminant space.}
#' \item{query_proj}{Query data projected onto the discriminant space (if query_data is provided).}
#' \item{query_mahalanobis_dist}{Mahalanobis distances of query projections (if calculate_metrics is TRUE).}
#' \item{mahalanobis_crit}{Cutoff value for Mahalanobis distance significance (if calculate_metrics is TRUE).}
#' \item{query_cosine_similarity}{Cosine similarity scores of query projections (if calculate_metrics is TRUE).}
#'
#' @references
#' \itemize{
#' \item Fisher, R. A. (1936). "The Use of Multiple Measurements in Taxonomic Problems". *Annals of Eugenics*. 7 (2): 179–188. doi:10.1111/j.1469-1809.1936.tb02137.x.
#' \item Hastie, T., Tibshirani, R., & Friedman, J. (2009). *The Elements of Statistical Learning: Data Mining, Inference, and Prediction*. Springer. Chapter 4: Linear Methods for Classification.
#' \item Ledoit, O., & Wolf, M. (2004). "A well-conditioned estimator for large-dimensional covariance matrices". *Journal of Multivariate Analysis*. 88 (2): 365–411. doi:10.1016/S0047-259X(03)00096-4.
#' \item De Maesschalck, R., Jouan-Rimbaud, D., & Massart, D. L. (2000). "The Mahalanobis distance". *Chemometrics and Intelligent Laboratory Systems*. 50 (1): 1–18. doi:10.1016/S0169-7439(99)00047-7.
#' \item Breiman, L. (2001). "Random Forests". *Machine Learning*. 45 (1): 5–32. doi:10.1023/A:1010933404324.
#' }
#'
#' @export
#'
#' @author
#' Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{plot.calculateDiscriminantSpaceObject}}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Compute discriminant space using unified model across all cell types
#' disc_output <- calculateDiscriminantSpace(reference_data = reference_data,
#'                                           query_data = query_data,
#'                                           query_cell_type_col = "SingleR_annotation",
#'                                           ref_cell_type_col = "expert_annotation",
#'                                           n_tree = 500,
#'                                           n_top = 50,
#'                                           eigen_threshold  = 1e-1,
#'                                           calculate_metrics = FALSE,
#'                                           alpha = 0.01)
#'
#' # Generate scatter and boxplot
#' plot(disc_output, plot_type = "scatterplot")
#' plot(disc_output, cell_types = c("CD4", "CD8"), plot_type = "boxplot")
#'
#' # Check comparison
#' table(Expert_Annotation = query_data$expert_annotation,
#'       SingleR = query_data$SingleR_annotation)
#'
#' @importFrom stats cov qchisq mahalanobis
#'
# Function to get the discriminant spaces and the projected reference/query data on the discriminant space.
# Similarity measures (cosine similarity/Mahalanobis distance) for the projected query data are also available.
calculateDiscriminantSpace <- function(reference_data,
                                       query_data = NULL,
                                       ref_cell_type_col,
                                       query_cell_type_col = NULL,
                                       cell_types = NULL,
                                       n_tree = 500,
                                       n_top = 20,
                                       eigen_threshold  = 1e-1,
                                       calculate_metrics = FALSE,
                                       alpha = 0.01,
                                       assay_name = "logcounts",
                                       max_cells_ref = NULL,
                                       max_cells_query = NULL){

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  assay_name = assay_name,
                  max_cells_ref = max_cells_ref,
                  max_cells_query = max_cells_query)

    # Convert cell type columns to character if needed
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)
    if(!is.null(query_data)){
        query_data <- convertColumnsToCharacter(sce_object = query_data,
                                                convert_cols = query_cell_type_col)
    }

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = NULL)

    # Downsample query and reference data (with cell type filtering)
    reference_data <- downsampleSCE(sce_object = reference_data,
                                    max_cells = max_cells_ref,
                                    cell_types = cell_types,
                                    cell_type_col = ref_cell_type_col)
    if(!is.null(query_data)){
        query_data <- downsampleSCE(sce_object = query_data,
                                    max_cells = max_cells_query,
                                    cell_types = cell_types,
                                    cell_type_col = query_cell_type_col)
    }

    # Check if n_tree is a positive integer
    if (!is.numeric(n_tree) || n_tree <= 0 || n_tree != as.integer(n_tree)) {
        stop("\'n_tree\' must be a positive integer.")
    }

    # Check if n_top is a positive integer
    if (!is.numeric(n_top) || n_top <= 0 || n_top != as.integer(n_top)) {
        stop("\'n_top\' must be a positive integer.")
    }

    # Input check for eigen_threshold
    if (!is.numeric(eigen_threshold) || eigen_threshold <= 0) {
        stop("\'eigen_threshold\' must be a positive number greater than 0.")
    }

    # Input check for alpha
    if (!is.numeric(alpha) || alpha <= 0 || alpha >= 1) {
        stop("\'alpha\' must be a positive number greater than 0 and less than 1.")
    }

    # Getting top variables
    var_imp <- calculateVarImpOverlap(reference_data = reference_data,
                                      ref_cell_type_col = ref_cell_type_col,
                                      n_tree = n_tree,
                                      n_top = n_top)

    # Get union of top n_top genes from all pairwise combinations
    all_top_genes <- unique(unlist(
        lapply(var_imp[["var_imp_ref"]], function(x) {
            x[["Gene"]][seq_len(n_top)]
        })))

    # Create a single discriminant model using all_top_genes
    # Extract reference matrix using top genes
    ref_mat <- t(as.matrix(assay(reference_data, assay_name)))[, all_top_genes]

    # Compute within-class and between-class scatter matrix
    sw <- sb <- matrix(0, length(all_top_genes), length(all_top_genes))
    overall_mean <- colMeans(ref_mat)

    for(cell_type in cell_types){
        # Extract matrix for cell type
        ref_mat_subset <- ref_mat[which(
            reference_data[[ref_cell_type_col]] == cell_type),]

        # Ledoit-Wolf estimation for the current class
        lw_cov <- ledoitWolf(ref_mat_subset)
        # Update within-class scatter matrix
        sw <- sw + (nrow(ref_mat_subset) - 1) * lw_cov

        # Update between-class scatter matrix
        class_mean <- colMeans(ref_mat_subset)
        sb <- sb + nrow(ref_mat_subset) * (class_mean - overall_mean) %*%
            t(class_mean - overall_mean)
    }

    # Ensure symmetry of sw
    sw <- (sw + t(sw)) / 2

    # Solve generalized eigenvalue problem
    eig <- eigen(solve(sw) %*% sb)

    # Sort eigenvectors by eigenvalues
    discriminant_eigenvalues <- Re(
        eig[["values"]][which(Re(eig[["values"]]) > eigen_threshold),
                        drop = FALSE])
    discriminant_eigenvectors <- Re(
        eig[["vectors"]][, which(Re(eig[["values"]]) > eigen_threshold),
                         drop = FALSE])
    rownames(discriminant_eigenvectors) <- all_top_genes
    colnames(discriminant_eigenvectors) <- paste0(
        "DV", seq_len(ncol(discriminant_eigenvectors)))

    # Compute projected data for reference
    ref_proj <- data.frame(
        ref_mat[which(reference_data[[ref_cell_type_col]] %in% cell_types),] %*%
            discriminant_eigenvectors,
        reference_data[[ref_cell_type_col]][reference_data[[ref_cell_type_col]] %in%
                                                cell_types])
    colnames(ref_proj) <- c(
        paste0("DV", seq_len(ncol(discriminant_eigenvectors))),
        "cell_type")

    # Create a single entry in discriminant_output
    discriminant_output <- list()
    discriminant_output <- list()
    discriminant_output[["discriminant_eigenvalues"]] <- discriminant_eigenvalues
    discriminant_output[["discriminant_eigenvectors"]] <- discriminant_eigenvectors
    discriminant_output[["ref_proj"]] <- ref_proj

    # Computations for query data
    if(!is.null(query_data)){

        # Projection on discriminant space
        query_mat <- t(as.matrix(assay(query_data, assay_name)))[, all_top_genes]
        query_proj <- data.frame(
            query_mat[which(query_data[[query_cell_type_col]] %in%
                                cell_types),] %*% discriminant_eigenvectors,
            query_data[[query_cell_type_col]][query_data[[query_cell_type_col]] %in%
                                                  cell_types])
        colnames(query_proj) <- c(
            paste0("DV", seq_len(ncol(discriminant_eigenvectors))), "cell_type")
        discriminant_output[["query_proj"]] <- query_proj

        if(calculate_metrics){

            # Cosine similarity between mean vector of reference projection
            # Mahalanobis distance between each query cell projected on reference discriminant space
            # and reference data projected on reference discriminant space
            cosine_similarity <- mahalanobis_dist <- numeric(nrow(query_proj))
            mahalanobis_crit <- numeric(length(cell_types))
            for(type_idx in seq_along(cell_types)){
                type <- cell_types[type_idx]
                query_cells_of_type <- query_proj[
                    query_proj[, "cell_type"] == type,
                    paste0("DV",
                           seq_len(length(discriminant_eigenvalues)))]
                ref_cells_of_type <- ref_proj[
                    ref_proj[, "cell_type"] == type,
                    paste0("DV",
                           seq_len(length(discriminant_eigenvalues)))]

                # Skip if we have no cells of this type
                if(nrow(query_cells_of_type) == 0 || nrow(ref_cells_of_type) == 0) {
                    next
                }

                # Calculate mean of reference projection for this cell type
                ref_mean <- colMeans(ref_cells_of_type)

                # Calculate covariance of reference projection for this cell type
                ref_cov <- cov(ref_cells_of_type)

                # Check if covariance matrix is invertible
                if(any(is.na(ref_cov)) || determinant(ref_cov)[["modulus"]][1] <= 0) {
                    # If not invertible, use a regularized version
                    ref_cov <- ledoitWolf(ref_cells_of_type)
                }

                # Calculate Mahalanobis distance
                mahalanobis_dist[query_proj[, "cell_type"] == type] <- mahalanobis(
                    query_cells_of_type, ref_mean, ref_cov)

                # Calculate cosine similarity
                cosine_similarity[query_proj[, "cell_type"] == type] <-
                    apply(query_cells_of_type, 1,
                          function(x, y) return(sum(x * y) / (sqrt(sum(x^2)) * sqrt(sum(y^2)))),
                          y = ref_mean)
            }
            discriminant_output[["query_mahalanobis_dist"]] <-
                mahalanobis_dist
            discriminant_output[["mahalanobis_crit"]] <-
                qchisq(1 - alpha, df = length(cell_types))
            discriminant_output[["query_cosine_similarity"]] <-
                cosine_similarity
        }
    }

    # Return data projected onto (reference) discriminant space for single combined model
    class(discriminant_output) <- c(class(discriminant_output),
                                    "calculateDiscriminantSpaceObject")
    return(discriminant_output)
}

#' @title Ledoit-Wolf Covariance Matrix Estimation
#'
#' @description
#' Estimate the covariance matrix using the Ledoit-Wolf shrinkage method.
#'
#' @details
#' This function computes the Ledoit-Wolf shrinkage covariance matrix estimator,
#' which improves the accuracy of the sample covariance matrix by shrinking it
#' towards a structured estimator, typically the diagonal matrix with the mean
#' of variances as its diagonal elements.
#'
#' @param class_data A numeric matrix or data frame containing the data for covariance estimation,
#' where rows represent observations and columns represent variables.
#'
#' @keywords internal
#'
#' @return A numeric matrix representing the Ledoit-Wolf estimated covariance matrix.
#'
#' @author
#' Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
# Function to compute Ledoit-Wolf covariance matrix
ledoitWolf <- function(class_data) {

    # Sample covariance matrix
    sample_cov <- cov(class_data)

    # Check for zero column means and replace them with a small constant if necessary
    col_means <- colMeans(class_data)
    col_means[col_means == 0] <- 1e-10

    # Calculate the shrinkage target (identity matrix scaled by the average variance)
    mean_variance <- mean(diag(sample_cov))
    shrinkage_target <- diag(mean_variance, ncol(class_data),
                             ncol(class_data))

    # Calculate the shrinkage intensity
    phi_hat <- sum((class_data - col_means) ^ 2) /
        (nrow(class_data) - 1)
    shrinkage_intensity <- (1 / nrow(class_data)) *
        min(phi_hat, mean_variance ^ 2)

    # Ledoit-Wolf estimated covariance matrix
    lw_cov <- (1 - shrinkage_intensity) *
        sample_cov + shrinkage_intensity * shrinkage_target

    # Ensure symmetry
    lw_cov <- (lw_cov + t(lw_cov)) / 2

    # Return Ledoit-Wolf estimated covariance matrix
    return(lw_cov)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Unit tests for calculateDiscriminantSpace function
test_that("calculateDiscriminantSpace function works as expected", {

    # Test Case 1: Basic Functionality - Reference data only
    output <- calculateDiscriminantSpace(reference_data = reference_data,
                                         query_data = NULL,
                                         ref_cell_type_col = "expert_annotation",
                                         n_tree = 500,
                                         n_top = 20,
                                         eigen_threshold = 0.1,
                                         calculate_metrics = FALSE,
                                         alpha = 0.01)

    expect_type(output, "list")
    expect_s3_class(output, "calculateDiscriminantSpaceObject")
    expect_true(all(c("discriminant_eigenvalues", "discriminant_eigenvectors", "ref_proj") %in% names(output)))

    # Test Case 2: With query data but no metrics
    output_with_query <- calculateDiscriminantSpace(reference_data = reference_data,
                                                    query_data = query_data,
                                                    ref_cell_type_col = "expert_annotation",
                                                    query_cell_type_col = "SingleR_annotation",
                                                    n_tree = 500,
                                                    n_top = 20,
                                                    eigen_threshold = 0.1,
                                                    calculate_metrics = FALSE,
                                                    alpha = 0.01)

    expect_type(output_with_query, "list")
    expect_s3_class(output_with_query, "calculateDiscriminantSpaceObject")
    expect_true(all(c("discriminant_eigenvalues", "discriminant_eigenvectors", "ref_proj", "query_proj") %in% names(output_with_query)))

    # Test Case 3: With query data and metrics
    output_with_metrics <- calculateDiscriminantSpace(reference_data = reference_data,
                                                      query_data = query_data,
                                                      ref_cell_type_col = "expert_annotation",
                                                      query_cell_type_col = "SingleR_annotation",
                                                      n_tree = 100,
                                                      n_top = 10,
                                                      eigen_threshold = 0.1,
                                                      calculate_metrics = TRUE,
                                                      alpha = 0.01)

    expect_type(output_with_metrics, "list")
    expect_s3_class(output_with_metrics, "calculateDiscriminantSpaceObject")
    expect_true(all(c("discriminant_eigenvalues", "discriminant_eigenvectors", "ref_proj", "query_proj",
                      "query_mahalanobis_dist", "mahalanobis_crit", "query_cosine_similarity") %in% names(output_with_metrics)))
})

test_that("calculateDiscriminantSpace parameter validation works", {
    # Test invalid n_tree
    expect_error(calculateDiscriminantSpace(reference_data = reference_data,
                                            ref_cell_type_col = "expert_annotation",
                                            n_tree = -10),
                 "n_tree.*must be a positive integer")

    # Test invalid n_top
    expect_error(calculateDiscriminantSpace(reference_data = reference_data,
                                            ref_cell_type_col = "expert_annotation",
                                            n_top = 0),
                 "n_top.*must be a positive integer")

    # Test invalid eigen_threshold
    expect_error(calculateDiscriminantSpace(reference_data = reference_data,
                                            ref_cell_type_col = "expert_annotation",
                                            eigen_threshold = -0.1),
                 "eigen_threshold.*must be a positive number")

    # Test invalid alpha
    expect_error(calculateDiscriminantSpace(reference_data = reference_data,
                                            ref_cell_type_col = "expert_annotation",
                                            alpha = 1.5),
                 "alpha.*must be a positive number.*less than 1")
})

test_that("calculateDiscriminantSpace works with specific cell types", {
    output <- calculateDiscriminantSpace(reference_data = reference_data,
                                         query_data = query_data,
                                         ref_cell_type_col = "expert_annotation",
                                         query_cell_type_col = "SingleR_annotation",
                                         cell_types = c("CD4", "CD8"),
                                         n_tree = 100,
                                         n_top = 10,
                                         eigen_threshold = 0.1)

    expect_type(output, "list")
    expect_s3_class(output, "calculateDiscriminantSpaceObject")
    expect_true("ref_proj" %in% names(output))
    expect_true("query_proj" %in% names(output))
})

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Compute important variables for all pairwise cell comparisons
disc_output <- calculateDiscriminantSpace(reference_data = reference_data,
                                          query_data = query_data,
                                          query_cell_type_col = "expert_annotation",
                                          ref_cell_type_col = "expert_annotation",
                                          eigen_threshold  = 1e-1,
                                          n_tree = 500,
                                          n_top = 50,
                                          calculate_metrics = FALSE,
                                          alpha = 0.01)

test_that("plot.calculateDiscriminantSpaceObject generates plots correctly", {
    # Generate plot using the function (returns ggmatrix, not ggplot)
    p1 <- plot(disc_output)

    # Check if output is a ggmatrix object (from GGally::ggpairs)
    expect_s3_class(p1, "ggmatrix")
})

test_that("plot works with different facet options", {
    # Test different facet combinations
    p1 <- plot(disc_output,
               lower_facet = "scatter",
               diagonal_facet = "density",
               upper_facet = "blank")

    expect_s3_class(p1, "ggmatrix")

    # Test with different options
    p2 <- plot(disc_output,
               lower_facet = "contour",
               diagonal_facet = "boxplot",
               upper_facet = "ellipse")

    expect_s3_class(p2, "ggmatrix")
})

test_that("plot works with specific cell types and dv_subset", {
    # Test with specific cell types
    available_cell_types <- unique(c(disc_output$ref_proj$cell_type,
                                     disc_output$query_proj$cell_type))
    selected_cell_types <- available_cell_types[1:min(2, length(available_cell_types))]

    p1 <- plot(disc_output, cell_types = selected_cell_types)
    expect_s3_class(p1, "ggmatrix")

    # Test with dv_subset
    max_dv <- ncol(disc_output$discriminant_eigenvectors)
    dv_subset <- 1:min(2, max_dv)

    p2 <- plot(disc_output, dv_subset = dv_subset)
    expect_s3_class(p2, "ggmatrix")
})

test_that("plot handles error cases correctly", {
    # Create a disc_output without query data
    disc_output_no_query <- calculateDiscriminantSpace(reference_data = reference_data,
                                                       query_data = NULL,
                                                       ref_cell_type_col = "expert_annotation",
                                                       n_tree = 100,
                                                       n_top = 10)

    # Should error when no query data is available
    expect_error(plot(disc_output_no_query), "There is no query data to plot")
})

#' @title Plot Visualization of Output from `compareMarkers` Function
#'
#' @description
#' The S3 plot method generates a comprehensive visualization of the output from the `compareMarkers` function.
#' The plot shows marker gene overlap and expression consistency between query and reference cell types,
#' with quality assessment and detailed annotations.
#'
#' @details
#' The S3 plot method creates a scatter plot showing the relationship between marker overlap (x-axis)
#' and expression consistency (y-axis) for each cell type. Points are colored by quality score and
#' sized by the minimum number of cells. Quality zones provide visual guidance for interpretation.
#'
#' @param x A list containing the output from the \code{compareMarkers} function.
#' @param cell_types Character vector specifying which cell types to plot. If NULL, all cell types are plotted.
#' @param ... Additional arguments passed to the plotting function.
#'
#' @keywords internal
#'
#' @return The S3 plot method returns a \code{ggplot} object representing the marker gene comparison results.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{compareMarkers}}
#'
#' @rdname compareMarkers
#'
# Plot visualization of output from compareMarkers function
plot.compareMarkersObject <- function(x, cell_types = NULL, ...){

    # Filter data by cell types if specified
    if (!is.null(cell_types)) {
        available_types <- intersect(cell_types, names(x[["marker_overlap"]]))
        if (length(available_types) == 0) {
            stop("None of the specified cell types are available in the data")
        }

        # Filter all relevant components
        x[["marker_overlap"]] <-
            x[["marker_overlap"]][available_types]
        x[["expression_consistency"]] <-
            x[["expression_consistency"]][available_types]
        x[["quality_scores"]] <-
            x[["quality_scores"]][available_types]
        x[["n_cells_query"]] <-
            x[["n_cells_query"]][available_types]
        x[["n_cells_ref"]] <-
            x[["n_cells_ref"]][available_types]
        x[["selected_cell_types"]] <- available_types
    }

    # Create data frame for plotting
    plot_data <- data.frame(
        Cell_Type = as.character(names(x[["marker_overlap"]])),
        Marker_Overlap = as.numeric(x[["marker_overlap"]]),
        Expression_Consistency = as.numeric(x[["expression_consistency"]]),
        Quality = as.character(x[["quality_scores"]]),
        stringsAsFactors = FALSE
    )

    # Handle cell counts properly
    query_counts <- x[["n_cells_query"]]
    ref_counts <- x[["n_cells_ref"]]

    # Ensure we have counts for all cell types
    plot_data[["N_Query"]] <-
        as.numeric(query_counts[plot_data[["Cell_Type"]]])
    plot_data[["N_Ref"]] <-
        as.numeric(ref_counts[plot_data[["Cell_Type"]]])

    # Calculate minimum cell count
    plot_data[["Min_Cells"]] <- pmin(plot_data[["N_Query"]],
                                     plot_data[["N_Ref"]])

    # Define colors
    quality_colors <- c("Good" = "#2E8B57",
                        "Moderate" = "#FF8C00",
                        "Poor" = "#DC143C")

    # Create formal filter description
    filter_description <- switch(
        x[["anomaly_filter_used"]],
        "none" = "",
        "anomalous_only" = " (Anomalous Cells Only)",
        "non_anomalous_only" = " (Non-Anomalous Cells Only)")

    # Create subtitle with filtering information
    subtitle_text <- paste0("Analysis of ",
                            length(x[["selected_cell_types"]]),
                            " cell types",
                            filter_description)

    # Create the main scatter plot
    marker_plot <- ggplot2::ggplot(plot_data,
                                   ggplot2::aes(
                                       x = .data[["Marker_Overlap"]],
                                       y = .data[["Expression_Consistency"]])) +

        # Add quality zone backgrounds
        ggplot2::annotate("rect", xmin = 0.7, xmax = 1,
                          ymin = 0.7, ymax = 1,
                          fill = "#2E8B57", alpha = 0.1) +
        ggplot2::annotate("rect", xmin = 0.4, xmax = 0.7,
                          ymin = 0.4, ymax = 0.7,
                          fill = "#FF8C00", alpha = 0.1) +
        ggplot2::annotate("rect", xmin = 0, xmax = 0.4,
                          ymin = 0, ymax = 0.4,
                          fill = "#DC143C", alpha = 0.1) +

        # Add threshold lines
        ggplot2::geom_hline(yintercept = 0.7,
                            linetype = "dashed",
                            color = "#2E8B57",
                            alpha = 0.5) +
        ggplot2::geom_vline(xintercept = 0.7,
                            linetype = "dashed",
                            color = "#2E8B57",
                            alpha = 0.5) +
        ggplot2::geom_hline(yintercept = 0.4,
                            linetype = "dashed",
                            color = "#FF8C00",
                            alpha = 0.5) +
        ggplot2::geom_vline(xintercept = 0.4,
                            linetype = "dashed",
                            color = "#FF8C00",
                            alpha = 0.5) +

        # Add points
        ggplot2::geom_point(ggplot2::aes(
            color = .data[["Quality"]],
            size = .data[["Min_Cells"]]),
            alpha = 0.8) +

        # Add text labels for cell types
        ggplot2::geom_text(ggplot2::aes(
            label = .data[["Cell_Type"]]),
            size = 3,
            hjust = -0.1,
            vjust = -0.1,
            alpha = 0.8) +

        # Color and size scales
        ggplot2::scale_color_manual(values = quality_colors,
                                    name = "Quality\n(Worst of Both)") +
        ggplot2::scale_size_continuous(range = c(3, 8),
                                       name = "Min Cells\nper Type") +

        # Set axis limits
        ggplot2::xlim(0, 1) +
        ggplot2::ylim(0, 1) +

        # Labels and theme
        ggplot2::labs(
            title = "Marker Gene Comparison Between Query and Reference",
            subtitle = subtitle_text,
            x = "Marker Gene Overlap (Jaccard Index)",
            y = "Expression Consistency of Reference Markers"
        ) +
        ggplot2::theme_bw() +
        ggplot2::theme(
            axis.title = ggplot2::element_text(size = 12),
            axis.text = ggplot2::element_text(size = 10),
            panel.grid.minor = ggplot2::element_blank(),
            panel.grid.major = ggplot2::element_line(color = "gray",
                                                     linetype = "dotted"),
            plot.title = ggplot2::element_text(size = 14, face = "bold"),
            plot.subtitle = ggplot2::element_text(size = 11),
            legend.position = "right"
        )

    # Add text annotations for zones
    marker_plot <- marker_plot +
        ggplot2::annotate("text", x = 0.85, y = 0.95, label = "Good",
                          size = 4, color = "#2E8B57", fontface = "bold") +
        ggplot2::annotate("text", x = 0.55, y = 0.55, label = "Moderate",
                          size = 4, color = "#FF8C00", fontface = "bold") +
        ggplot2::annotate("text", x = 0.2, y = 0.2, label = "Poor",
                          size = 4, color = "#DC143C", fontface = "bold")

    return(marker_plot)
}

#' @title Calculate Cramer Test P-Values for Two-Sample Comparison of Multivariate ECDFs
#'
#' @description
#' This function performs the Cramer test for comparing multivariate empirical cumulative distribution functions (ECDFs)
#' between two samples.
#'
#' @details
#' The function performs the following steps:
#' \enumerate{
#'   \item Projects the data into the PCA space.
#'   \item Subsets the data to the specified cell types and principal components.
#'   \item Performs the Cramer test for each cell type using the \code{cramer.test} function in the \code{cramer} package.
#' }
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.
#' @param pc_subset A numeric vector specifying which principal components to include in the plot. Default is PC1 to PC5.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#'
#' @return A named vector of p-values from the Cramer test for each cell type.
#'
#' @references Baringhaus, L., & Franz, C. (2004). "On a new multivariate two-sample test".
#' Journal of Multivariate Analysis, 88(1), 190-206.
#'
#' @export
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Plot the PC data (with query data)
#' cramer_test <- calculateCramerPValue(reference_data = reference_data,
#'                                      query_data = query_data,
#'                                      ref_cell_type_col = "expert_annotation",
#'                                      query_cell_type_col = "SingleR_annotation",
#'                                      cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
#'                                      pc_subset = 1:5)
#' cramer_test
#'
# Function to perform Cramer test for two-sample comparison of multivariate ECDFs
calculateCramerPValue <- function(query_data,
                                  reference_data,
                                  query_cell_type_col,
                                  ref_cell_type_col,
                                  cell_types = NULL,
                                  pc_subset = 1:5,
                                  assay_name = "logcounts",
                                  max_cells_query = 5000,
                                  max_cells_ref = 5000) {

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = TRUE,
                                  n_cell_types = NULL)

    # Get the projected PCA data
    pca_output <- projectPCA(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = query_cell_type_col,
                             ref_cell_type_col = ref_cell_type_col,
                             cell_types = cell_types,
                             pc_subset = pc_subset,
                             assay_name = assay_name,
                             max_cells_ref = max_cells_ref,
                             max_cells_query = max_cells_query)

    # Set data for Cramer test
    cell_list <- split(pca_output, pca_output[["cell_type"]])

    # Set the PC variables
    pc_vars <- paste0("PC", pc_subset)

    # Adding regression summaries for each cell type
    cramer_test <- vector("list", length = length(cell_list))
    names(cramer_test) <- cell_types
    for(cell_type in cell_types){

        dataset_ind <- cell_list[[cell_type]][, "dataset"] == "Reference"
        cramer_test[[cell_type]] <- cramer::cramer.test(
            as.matrix(cell_list[[cell_type]][dataset_ind, pc_vars]),
            as.matrix(cell_list[[cell_type]][!dataset_ind, pc_vars]),
            kernel = "phiBahr")
    }
    p_values <- unlist(lapply(cramer_test, function(t) t[["p.value"]]))

    # Return Cramer test output
    return(p_values)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics) 

# Load example datasets
data("reference_data")
data("query_data")

## Unit tests for calculateCramerPValue function
test_that("calculateCramerPValue handles NULL query_data gracefully", {
    
    # Test with NULL query_data
    expect_error(calculateCramerPValue(
        reference_data = reference_data,
        query_data = NULL,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = "SingleR_annotation",
        cell_types = c("CD4", "CD8"),
        pc_subset = 1:5
    ))
    
})

test_that("calculateCramerPValue handles NULL query_cell_type_col gracefully", {
    
    # Test with NULL query_cell_type_col
    expect_error(calculateCramerPValue(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = NULL,
        cell_types = c("CD4", "CD8"),
        pc_subset = 1:5
    ))
    
})


test_that("calculateCramerPValue handles NULL cell_types gracefully", {
    
    # Test with NULL cell_types
    p_values <- calculateCramerPValue(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = "SingleR_annotation",
        cell_types = NULL,
        pc_subset = 1:5
    )
    
    # Check if the result is a named vector of p-values
    expect_type(p_values, "double")
    expect_true(length(p_values) > 0)  # Ensure some p-values are returned
    
    # Check the range of p-values
    expect_true(all(p_values >= 0 & p_values <= 1))
    
})

test_that("calculateCramerPValue returns correct p-values for specified cell types", {
    
    # Test with specific cell_types
    p_values <- calculateCramerPValue(
        reference_data = reference_data,
        query_data = query_data,
        ref_cell_type_col = "expert_annotation",
        query_cell_type_col = "SingleR_annotation",
        cell_types = c("CD4", "CD8"),
        pc_subset = 1:5
    )
    
    # Check if the result is a named vector of p-values
    expect_type(p_values, "double")
    expect_true(all(names(p_values) %in% c("CD4", "CD8")))
    
    # Check the range of p-values
    expect_true(all(p_values >= 0 & p_values <= 1))
    
    # Add more specific checks as needed based on the expected behavior
    
})

#' @title Project Query Data Onto SIR Space of Reference Data
#'
#' @description
#' This function projects a query \code{SingleCellExperiment} object onto the SIR (supervised independent
#' component) space of a reference \code{SingleCellExperiment} object. The SVD of the reference data is
#' computed on conditional means per cell type, and the query data is projected based on these reference
#' components.
#'
#' @details
#' The genes used for the projection (SVD) must be present in both the reference and query datasets.
#' The function first computes conditional means for each cell type in the reference data, then performs
#' SVD on these conditional means to obtain the rotation matrix used for projecting both the reference
#' and query datasets. The query data is centered and scaled based on the reference data.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col A character string specifying the column in the \code{colData} of \code{query_data}
#' that identifies the cell types.
#' @param ref_cell_type_col A character string specifying the column in the \code{colData} of \code{reference_data}
#' that identifies the cell types.
#' @param cell_types A character vector of cell types for which to compute conditional means in the reference data.
#' @param multiple_cond_means A logical value indicating whether to compute multiple conditional means per cell type
#' (through PCA and clustering). Defaults to \code{TRUE}.
#' @param cumulative_variance_threshold A numeric value between 0 and 1 specifying the variance threshold for PCA
#' when computing multiple conditional means. Defaults to \code{0.7}.
#' @param n_neighbor An integer specifying the number of nearest neighbors for clustering when computing multiple
#' conditional means. Defaults to \code{1}.
#' @param assay_name A character string specifying the assay name on which to perform computations. Defaults to \code{"logcounts"}.
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#'
#' @return A list containing:
#' \item{cond_means}{A matrix of the conditional means computed for the reference data.}
#' \item{rotation_mat}{The rotation matrix obtained from the SVD of the conditional means.}
#' \item{sir_projections}{A \code{data.frame} containing the SIR projections for both the reference and query datasets.}
#' \item{percent_var}{The percentage of variance explained by each component of the SIR projection.}
#'
#' @export
#'
#' @author
#' Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Project the query data onto SIR space of reference
#' sir_output <- projectSIR(query_data = query_data,
#'                          reference_data = reference_data,
#'                          query_cell_type_col = "SingleR_annotation",
#'                          ref_cell_type_col = "expert_annotation")
#'
# Function to project query data onto PCA space of reference data
projectSIR <- function(query_data,
                       reference_data,
                       query_cell_type_col,
                       ref_cell_type_col,
                       cell_types = NULL,
                       multiple_cond_means = TRUE,
                       cumulative_variance_threshold = 0.7,
                       n_neighbor = 1,
                       assay_name = "logcounts",
                       max_cells_query = 5000,
                       max_cells_ref = 5000){

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Downsample query and reference data
    query_data <- downsampleSCE(sce_object = query_data,
                                cell_type_col = query_cell_type_col,
                                max_cells = max_cells_query)
    reference_data <- downsampleSCE(sce_object = reference_data,
                                    cell_type_col = ref_cell_type_col,
                                    max_cells = max_cells_ref)

    # Check if cumulative_variance_threshold is between 0 and 1
    if (!is.numeric(cumulative_variance_threshold) ||
        cumulative_variance_threshold < 0 || cumulative_variance_threshold > 1) {
        stop("cumulative_variance_threshold must be a numeric value between 0 and 1.")
    }

    # Check if n_neighbor is a positive integer
    if (!is.numeric(n_neighbor) || n_neighbor <= 0 || n_neighbor != as.integer(n_neighbor)) {
        stop("n_neighbor must be a positive integer.")
    }

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = NULL)

    # Compute conditional means for each cell type of reference data
    cond_means <- conditionalMeans(reference_data = reference_data,
                                   ref_cell_type_col = ref_cell_type_col,
                                   cell_types = cell_types,
                                   multiple_cond_means = multiple_cond_means,
                                   assay_name = assay_name,
                                   cumulative_variance_threshold = cumulative_variance_threshold,
                                   n_neighbor = n_neighbor)

    # Extract reference SVD components and rotation matrix
    SVD_genes <- rownames(reference_data)
    centering_vec <- apply(cond_means, 2, mean)[SVD_genes]
    svd_ref <- svd(scale(cond_means, center = centering_vec, scale = FALSE))
    rotation_mat <- svd_ref[["v"]]
    rownames(rotation_mat) <- SVD_genes

    # Check if genes used for PCA are available in query data
    if (!all(SVD_genes %in% rownames(assay(query_data, assay_name)))) {
        stop("Genes in reference SVD are not found in query data.")
    }

    # Compute percentage of variance explained
    percent_var <- svd_ref[["d"]]^2 /
        sum(svd_ref[["d"]]^2) * 100

    # Center and scale reference query data based on reference for projection
    ref_mat <- scale(t(as.matrix(
        assay(reference_data, assay_name)))[, SVD_genes, drop = FALSE],
        center = centering_vec, scale = FALSE) %*% rotation_mat
    query_mat <- scale(t(as.matrix(
        assay(query_data, assay_name)))[, SVD_genes, drop = FALSE],
        center = centering_vec, scale = FALSE) %*% rotation_mat
    sir_projections <- data.frame(
        rbind(ref_mat, query_mat),
        dataset = c(rep("Reference", nrow(ref_mat)),
                    rep("Query", nrow(query_mat))),
        cell_type = c(ifelse(rep(is.null(ref_cell_type_col),
                                 nrow(ref_mat)),
                             rep(NA, nrow(ref_mat)),
                             reference_data[[ref_cell_type_col]]),
                      ifelse(rep(is.null(query_cell_type_col),
                                 nrow(query_mat)),
                             rep(NA, nrow(query_mat)),
                             query_data[[query_cell_type_col]])))
    colnames(sir_projections)[seq_len(ncol(ref_mat))] <-
        paste0("SIR", seq_len(ncol(ref_mat)), " (",
               sprintf("%.1f%%", percent_var[seq_len(ncol(ref_mat))]), ")")

    # Returning output as a data frame
    return(list(cond_means = cond_means,
                rotation_mat = rotation_mat,
                sir_projections = sir_projections,
                percent_var = percent_var))
}

#' @title Compute Conditional Means for Cell Types
#'
#' @description
#' This function computes conditional means for each cell type in the reference data. It can compute
#' either a single conditional mean per cell type or multiple conditional means, depending on the
#' specified settings. Principal component analysis (PCA) is used for dimensionality reduction before
#' clustering when computing multiple conditional means.
#'
#' @details
#' The function offers two modes of operation:
#' - **Single conditional mean per cell type**: For each cell type, it computes the mean expression
#'   across all observations.
#' - **Multiple conditional means per cell type**: For each cell type, the function performs PCA to
#'   reduce dimensionality, followed by clustering to compute multiple conditional means.
#'
#' @param reference_data A \code{SingleCellExperiment} object containing the reference data, where rows
#' represent genes and columns represent cells.
#' @param ref_cell_type_col A character string specifying the column in \code{colData(reference_data)} that contains the cell type labels.
#' @param cell_types A character vector of cell types for which to compute conditional means.
#' @param multiple_cond_means A logical value indicating whether to compute multiple conditional means per cell type.
#' Defaults to \code{FALSE}.
#' @param assay_name A character string specifying the name of the assay to use for the computation. Defaults to \code{"logcounts"}.
#' @param cumulative_variance_threshold A numeric value between 0 and 1 specifying the variance threshold
#' for PCA when computing multiple conditional means. Defaults to \code{0.7}.
#' @param n_neighbor An integer specifying the number of nearest neighbors for clustering when computing multiple conditional means.
#' Defaults to \code{1}.
#'
#' @return A numeric matrix with the conditional means for each cell type. If \code{multiple_cond_means = TRUE}, the matrix
#' will contain multiple rows for each cell type, representing the different conditional means computed via clustering.
#'
#' @keywords internal
#'
#' @author
#' Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
# Function to compute Ledoit-Wolf covariance matrix
conditionalMeans <- function(reference_data,
                             ref_cell_type_col,
                             cell_types,
                             multiple_cond_means = FALSE,
                             assay_name = "logcounts",
                             cumulative_variance_threshold = 0.7,
                             n_neighbor = 1) {

    # Compute conditional means for each cell type of reference data
    if (multiple_cond_means) {

        # Matrix to store results
        cond_means <- matrix(nrow = 0, ncol = nrow(reference_data))
        colnames(cond_means) <- rownames(reference_data)

        for(cell_type in cell_types){

            # Compute multiple conditional means per cell type
            assay_mat <- scale(t(as.matrix(assay(
                reference_data[, which(reference_data[[ref_cell_type_col]] == cell_type)],
                assay_name))), center = TRUE, scale = FALSE)
            assay_svd <- svd(assay_mat)
            cumulative_variance <- cumsum(assay_svd$d^2) / sum(assay_svd$d^2)
            n_components <- min(which(cumulative_variance >= cumulative_variance_threshold))
            projections <- assay_mat %*%
                assay_svd$v[, seq_len(n_components)]
            clusters <- suppressWarnings(
                bluster::clusterRows(
                    projections,
                    BLUSPARAM = bluster::TwoStepParam(
                        second = bluster::NNGraphParam(
                            k = n_neighbor))))
            cluster_means <- do.call(
                rbind, lapply(unique(clusters),
                              function(cl) colMeans(assay_mat[clusters == cl,, drop = FALSE])))
            rownames(cluster_means) <- rep(cell_type, nrow(cluster_means))
            cond_means  <- rbind(cond_means, cluster_means)
        }

    } else {

        # Compute a single conditional mean per cell type
        cond_means <- lapply(cell_types, function(x)
            apply(as.matrix(assay(
                reference_data[, which(reference_data[[ref_cell_type_col]] == x)],
                assay_name)), 1, mean))
        cond_means <- do.call(rbind, cond_means)
        rownames(cond_means) <- cell_types

    }

    # Return conditional means
    return(cond_means)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

test_that("projectSIR returns expected output structure", {
    # Test that the function returns a list with the correct components
    result <- projectSIR(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation"
    )

    expect_type(result, "list")
    expect_named(result, c("cond_means", "rotation_mat", "sir_projections", "percent_var"))
})

test_that("projectSIR handles genes not found in query data", {
    # Modify query_data to exclude certain genes
    modified_query_data <- query_data[-1, ] # Remove the first gene

    expect_error(
        projectSIR(
            query_data = modified_query_data,
            reference_data = reference_data,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation"
        ),
        "Genes in reference SVD are not found in query data."
    )
})

test_that("projectSIR returns projections with correct number of components", {
    result <- projectSIR(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation"
    )

    expect_equal(ncol(result$sir_projections), length(result$percent_var) + 2)
})

#' @title Perform Hotelling's T-squared Test on PCA Scores for Single-cell RNA-seq Data
#'
#' @description
#' Computes Hotelling's T-squared test statistic and p-values for each specified cell type
#' based on PCA-projected data from query and reference datasets.
#'
#' @details
#' This function calculates Hotelling's T-squared statistic for comparing multivariate means
#' between reference and query datasets, projected onto a subset of principal components (PCs).
#' It performs a permutation test to obtain p-values for each cell type specified.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col character. The column name in the \code{colData} of \code{query_data}
#' that identifies the cell types.
#' @param ref_cell_type_col character. The column name in the \code{colData} of \code{reference_data}
#' that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.
#' @param pc_subset A numeric vector specifying which principal components to include in the plot. Default is PC1 to PC5.
#' @param n_permutation Number of permutations to perform for p-value calculation. Default is 500.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#'
#' @return A named numeric vector of p-values from Hotelling's T-squared test for each cell type.
#'
#' @references
#' Hotelling, H. (1931). "The generalization of Student's ratio". *Annals of Mathematical Statistics*. 2 (3): 360–378.
#' doi:10.1214/aoms/1177732979.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Get the p-values
#' p_values <- calculateHotellingPValue(query_data = query_data,
#'                                      reference_data = reference_data,
#'                                      query_cell_type_col = "SingleR_annotation",
#'                                      ref_cell_type_col = "expert_annotation",
#'                                      pc_subset = 1:10)
#' round(p_values, 5)
#'
# Function to perform Hotelling T^2 test for each cell type
# The test is performed on the PCA space of the reference data The query data projected onto PCA space of reference
calculateHotellingPValue <- function(query_data,
                                     reference_data,
                                     query_cell_type_col,
                                     ref_cell_type_col,
                                     cell_types = NULL,
                                     pc_subset = 1:5,
                                     n_permutation = 500,
                                     assay_name = "logcounts",
                                     max_cells_query = 5000,
                                     max_cells_ref = 5000) {

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = TRUE,
                                  n_cell_types = NULL)

    # Get the projected PCA data
    pca_output <- projectPCA(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = query_cell_type_col,
                             ref_cell_type_col = ref_cell_type_col,
                             cell_types = cell_types,
                             pc_subset = pc_subset,
                             assay_name = assay_name,
                             max_cells_ref = max_cells_ref,
                             max_cells_query = max_cells_query)

    # Set data for Hotelling permutation test
    cell_list <- split(pca_output, pca_output[["cell_type"]])

    # Perform permutation test with p-values
    p_values <- numeric(length(cell_types))
    names(p_values) <- cell_types
    for (cell_type in cell_types) {

        ref_ind <- cell_list[[cell_type]][["dataset"]] == "Reference"
        observed_t2_data <- as.matrix(cell_list[[cell_type]][, paste0("PC", pc_subset)])
        observed_t2 <- hotellingT2(observed_t2_data[ref_ind,],
                                   observed_t2_data[!ref_ind,])

        perm_t2 <- numeric(n_permutation)
        for(perm_id in seq_len(n_permutation)){

            ref_sample_id <- sample(seq_len(nrow(cell_list[[cell_type]])), sum(ref_ind), replace = FALSE)
            perm_t2_data <- as.matrix(cell_list[[cell_type]][, paste0("PC", pc_subset)])
            perm_t2[perm_id] <- hotellingT2(perm_t2_data[ref_sample_id,],
                                            perm_t2_data[-ref_sample_id,])
        }
        p_values[cell_type] <- mean(observed_t2 < perm_t2)
    }

    # Return p-values
    return(p_values)
}

#' @title Calculate Hotelling's T^2 Statistic
#'
#' @description
#' Calculates the Hotelling's T^2 statistic for comparing means of multivariate data.
#'
#' @param sample1 A numeric matrix or data frame of multivariate observations for sample 1, where rows are observations and columns
#' are variables.
#' @param sample2 A numeric matrix or data frame of multivariate observations for sample 2, with the same structure as sample 1.
#'
#' @keywords internal
#'
#' @return The Hotelling's T^2 statistic.
#'
# Function to calculate hotelling T2 statistic between two groups
hotellingT2 <- function(sample1, sample2) {

    # Number of observations in each sample
    n1 <- nrow(sample1)
    n2 <- nrow(sample2)

    # Number of variables (columns) in each sample
    p <- ncol(sample1)

    # Compute mean vectors for each sample
    mean1 <- colMeans(sample1)
    mean2 <- colMeans(sample2)

    # Compute covariance matrices for each sample
    cov1 <- cov(sample1)
    cov2 <- cov(sample2)

    # Pooled covariance matrix
    pooled_cov <- ((n1 - 1) * cov1 + (n2 - 1) * cov2) / (n1 + n2 - 2)

    # Compute Hotelling's T^2 statistic
    # T^2 = n1 * n2 / (n1 + n2) * (mean1 - mean2)' * pooled_cov^-1 * (mean1 - mean2)
    t2 <- n1 * n2 / (n1 + n2) * t(mean1 - mean2) %*% solve(pooled_cov) %*%
        (mean1 - mean2)

    # Return the computed T^2 statistic
    return(as.numeric(t2))
}

# Load necessary libraries
library(testthat)
library(scDiagnostics) 

# Load example datasets
data("reference_data")
data("query_data")

# Unit tests for calculateHotellingPValue function
test_that("calculateHotellingPValue function tests", {
    
    # Test case 1: Check if function returns a named numeric vector
    p_values <- calculateHotellingPValue(query_data = query_data,
                                         reference_data = reference_data,
                                         query_cell_type_col = "SingleR_annotation",
                                         ref_cell_type_col = "expert_annotation",
                                         pc_subset = 1:5,
                                         n_permutation = 100)
    
    expect_type(p_values, "double")
    expect_true(length(p_values) > 0)

    # Test case 2: Check if p-values are between 0 and 1
    expect_true(all(p_values >= 0 & p_values <= 1))
    
})

#' @title Cell Type Selection and Validation for SingleCellExperiment Analysis
#'
#' @description
#' This function selects and validates cell types for functions that analyze \code{\linkS4class{SingleCellExperiment}}
#' objects. It determines which cell types to include based on availability in datasets, applies filtering
#' criteria, and optionally selects the top cell types by cell count.
#'
#' @details
#' The function performs the following selection and validation steps:
#' \itemize{
#'  \item Validates that at least one of \code{query_data} or \code{reference_data} is provided.
#'  \item When \code{dual_only} is TRUE, ensures both datasets are provided.
#'  \item Determines available cell types based on dataset availability and \code{dual_only} setting.
#'  \item If \code{cell_types} is NULL and both datasets are available, includes cell types based on \code{dual_only}.
#'  \item If \code{cell_types} is NULL and only one dataset is available, includes all cell types from that dataset.
#'  \item If \code{cell_types} is provided, filters to include only valid types.
#'  \item If \code{n_cell_types} is specified, selects the top cell types by total cell count.
#'  \item Returns the selected and validated cell types as character strings.
#' }
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' Can be \code{NULL} if only reference data is available.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' Can be \code{NULL} if only query data is available.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data}
#' that identifies the cell types. Should be \code{NULL} if \code{query_data} is \code{NULL}.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data}
#' that identifies the cell types. Should be \code{NULL} if \code{reference_data} is \code{NULL}.
#' @param cell_types A character vector specifying the cell types to validate. If \code{NULL},
#' cell types will be automatically selected based on dataset availability and \code{dual_only} setting.
#' @param dual_only A logical value indicating whether cell types must be present in both datasets.
#' If \code{TRUE}, both \code{query_data} and \code{reference_data} must be provided, and only
#' cell types present in both datasets will be considered. Default is \code{FALSE}.
#' @param n_cell_types An integer specifying the maximum number of cell types to select based on
#' highest cell count. If \code{NULL}, all valid cell types are returned. Default is \code{NULL}.
#'
#' @keywords internal
#'
#' @return A character vector of selected cell types that meet the specified criteria.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
# Function to select and validate cell types
selectCellTypes <- function(query_data = NULL,
                            reference_data = NULL,
                            query_cell_type_col = NULL,
                            ref_cell_type_col = NULL,
                            cell_types = NULL,
                            dual_only = FALSE,
                            n_cell_types = NULL) {

    # Check that at least one dataset is provided
    if (is.null(query_data) && is.null(reference_data)) {
        stop("At least one of 'query_data' or 'reference_data' must be provided.")
    }

    # Check dual_only requirements
    if (dual_only && (is.null(query_data) || is.null(reference_data))) {
        stop("When 'dual_only' is TRUE, both 'query_data' and 'reference_data' must be provided.")
    }

    # Validate n_cell_types if provided
    if (!is.null(n_cell_types)) {
        if (!is.numeric(n_cell_types) || length(n_cell_types) != 1 ||
            n_cell_types <= 0 || n_cell_types != as.integer(n_cell_types)) {
            stop("'n_cell_types' must be a positive integer.")
        }
    }

    # Extract available cell types from each dataset and convert to character
    available_query_types <- NULL
    available_ref_types <- NULL

    if (!is.null(query_data) && !is.null(query_cell_type_col)) {
        if (!query_cell_type_col %in% names(colData(query_data))) {
            stop("'query_cell_type_col' is not found in query_data colData.")
        }
        available_query_types <- as.character(na.omit(unique(query_data[[query_cell_type_col]])))
    }

    if (!is.null(reference_data) && !is.null(ref_cell_type_col)) {
        if (!ref_cell_type_col %in% names(colData(reference_data))) {
            stop("'ref_cell_type_col' is not found in reference_data colData.")
        }
        available_ref_types <- as.character(na.omit(unique(reference_data[[ref_cell_type_col]])))
    }

    # Determine valid cell types based on input and availability
    if (is.null(cell_types)) {
        # Auto-select cell types based on dataset availability
        if (!is.null(available_query_types) && !is.null(available_ref_types)) {
            # Both datasets available - use dual_only setting
            if (dual_only) {
                all_available_types <- intersect(available_query_types, available_ref_types)
            } else {
                all_available_types <- unique(c(available_query_types, available_ref_types))
            }
        } else {
            # Only one dataset available - use all cell types from that dataset
            all_available_types <- unique(c(available_query_types, available_ref_types))
        }
    } else {
        # User provided cell types - convert to character and validate against availability
        cell_types <- as.character(cell_types)

        # Determine which types are valid based on dual_only setting
        if (dual_only) {
            if (is.null(available_query_types) || is.null(available_ref_types)) {
                stop("When 'dual_only' is TRUE, both datasets must have valid cell type columns.")
            }
            valid_types_pool <- intersect(available_query_types, available_ref_types)
        } else {
            valid_types_pool <- unique(c(available_query_types, available_ref_types))
        }

        # Filter to only those present in valid pool
        all_available_types <- cell_types[cell_types %in% valid_types_pool]

        # Check if any requested cell types were found
        if (length(all_available_types) == 0) {
            missing_types <- setdiff(cell_types, valid_types_pool)
            if (dual_only) {
                stop("None of the specified cell types are present in both datasets. ",
                     "Missing types: ", paste(missing_types, collapse = ", "))
            } else {
                stop("None of the specified cell types are present in the provided datasets. ",
                     "Missing types: ", paste(missing_types, collapse = ", "))
            }
        }

        # Warn about missing cell types if some but not all were found
        missing_types <- setdiff(cell_types, all_available_types)
        if (length(missing_types) > 0) {
            if (dual_only) {
                warning("Some specified cell types are not present in both datasets and will be excluded: ",
                        paste(missing_types, collapse = ", "))
            } else {
                warning("Some specified cell types are not present in the provided datasets and will be excluded: ",
                        paste(missing_types, collapse = ", "))
            }
        }
    }

    # Ensure all_available_types is character and remove any NA values
    all_available_types <- as.character(all_available_types)
    all_available_types <- all_available_types[!is.na(all_available_types)]

    # If no cell types are available, return error
    if (length(all_available_types) == 0) {
        if (dual_only) {
            stop("No common cell types found between query and reference datasets.")
        } else {
            stop("No valid cell types found in the provided datasets.")
        }
    }

    # Apply n_cell_types selection if specified
    if (!is.null(n_cell_types) && length(all_available_types) > n_cell_types) {
        # Count cells for each cell type
        cell_counts <- numeric(length(all_available_types))
        names(cell_counts) <- all_available_types

        for (ct in all_available_types) {
            count <- 0

            # Count cells in query data if available and cell type is present
            if (!is.null(available_query_types) && ct %in% available_query_types) {
                count <- count + sum(as.character(query_data[[query_cell_type_col]]) == ct, na.rm = TRUE)
            }

            # Count cells in reference data if available and cell type is present
            if (!is.null(available_ref_types) && ct %in% available_ref_types) {
                count <- count + sum(as.character(reference_data[[ref_cell_type_col]]) == ct, na.rm = TRUE)
            }

            cell_counts[ct] <- count
        }

        # Select top n_cell_types by count
        top_indices <- order(cell_counts, decreasing = TRUE)[1:n_cell_types]
        selected_types <- names(cell_counts)[top_indices]

        # Warn about selection
        excluded_types <- setdiff(all_available_types, selected_types)
        warning("Selected top ", n_cell_types, " cell types by cell count. ",
                "Excluded types: ", paste(excluded_types, collapse = ", "))

        final_cell_types <- selected_types
    } else {
        final_cell_types <- all_available_types
    }

    # Ensure final result is character and sort for consistent output
    final_cell_types <- as.character(final_cell_types)
    final_cell_types <- sort(final_cell_types)

    return(final_cell_types)
}

#' @title Plot Cosine Similarities Between Cells and PCs
#'
#' @description
#' The S3 plot method  creates a heatmap plot to visualize the cosine similarities between cells and principal components (PCs).
#'
#' @details
#' The S3 plot method reshapes the input data frame to create a long format suitable for plotting as a heatmap. It then
#' creates a heatmap plot using ggplot2, where the x-axis represents the PCs, the y-axis represents the cells, and the
#' color intensity represents the cosine similarity values.
#'
#' @param x An object of class `calculateCellSimilarityPCA` containing a dataframe of cosine similarity values
#' between cells and PCs.
#' @param pc_subset A numeric vector specifying the subset of principal components to include in the plot. Default is 1:5.
#' @param ... Additional arguments passed to the plotting function.
#'
#' @keywords internal
#'
#' @return The S3 plot method returns a \code{ggplot} object representing the cosine similarity heatmap.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{calculateCellSimilarityPCA}}
#'
#' @rdname calculateCellSimilarityPCA
#'
# Function to plot cosine similarities between cells and PCs
plot.calculateCellSimilarityPCAObject <- function(x,
                                            pc_subset = 1:5,
                                            ...){

    # Subset data
    x <- x[, paste0("PC", pc_subset)]

    # Initialize empty vectors for reshaped data
    cell_names <- c()
    pc_names <- c()
    cosine_values <- c()

    # Loop through the data frame to manually reshape it
    for (cell in rownames(x)) {
        for (pc in colnames(x)) {
            cell_names <- c(cell_names, cell)
            pc_names <- c(pc_names, pc)
            cosine_values <- c(cosine_values, x[cell, pc])
        }
    }

    # Create a data frame with the reshaped data
    cosine_long <- data.frame(Cell = factor(cell_names,
                                            levels = rev(rownames(x))),
                              PC = pc_names, CosineSimilarity = cosine_values)

    # Create the heatmap plot
    plot <- ggplot2::ggplot(cosine_long, ggplot2::aes(
        x = .data[["PC"]],
        y = .data[["Cell"]],
        fill = .data[["CosineSimilarity"]])) +
        ggplot2::geom_tile(color = "white") +
        ggplot2::geom_text(
            ggplot2::aes(label = sprintf("%.2f", .data[["CosineSimilarity"]])),
            size = 3) +
        ggplot2::scale_fill_gradient2(low = "blue", mid = "white",
                                      high = "red", midpoint = 0,
                             limits = c(-1, 1), space = "Lab",
                             name = "Cosine Similarity") +
        ggplot2::labs(title = "Cosine Similarity Heatmap", x = "", y = "") +
        ggplot2::theme_minimal() +
        ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45,
                                                           hjust = 1),
                       plot.title = ggplot2::element_text(hjust = 0.5))
    return(plot)
}

#' @title Calculate Top Loading Gene Expression Shifts
#'
#' @description
#' This function identifies genes with the highest loadings for specified principal components
#' and performs statistical tests to detect distributional differences between query and reference data.
#' It also calculates the proportion of variance explained by each principal component within
#' specific cell types. Optionally, it can detect anomalous cells using isolation forests.
#'
#' @details
#' This function extracts the top loading genes for each specified principal component from the reference
#' PCA space and performs distributional comparisons between query and reference data. For each gene,
#' it performs statistical tests to identify genes that may be causing PC-specific alignment issues
#' between datasets. A key feature is the calculation of cell-type-specific variance explained by
#' global PCs, providing a more nuanced view of how major biological axes affect individual populations.
#' When anomaly detection is enabled, isolation forests are used to identify anomalous cells based on
#' their PCA projections.
#'
#' When \code{anomaly_comparison = TRUE}, the statistical analysis focuses specifically on
#' comparing non-anomalous reference cells against anomalous query cells. This can help
#' identify genes that are differentially expressed between "normal" reference cells and
#' potentially problematic query cells, providing insights into what makes certain query
#' cells anomalous.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to analyze. If NULL, all common cell types are used.
#' @param pc_subset A numeric vector specifying which principal components to analyze. Default is 1:5.
#' @param n_top_loadings Number of top loading genes to analyze per PC. Default is 50.
#' @param p_value_threshold P-value threshold for statistical significance. Default is 0.05.
#' @param adjust_method Method for multiple testing correction. Default is "fdr".
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param detect_anomalies Logical indicating whether to perform anomaly detection using isolation forests.
#'                        Default is FALSE.
#' @param anomaly_comparison Logical indicating whether to perform statistical comparisons
#'                           between non-anomalous reference cells and anomalous query cells instead of all-vs-all
#'                           comparisons. When TRUE, only non-anomalous reference cells are compared against only
#'                           anomalous query cells for each cell type. Requires detect_anomalies = TRUE. Default is FALSE.
#' @param anomaly_threshold A numeric value specifying the threshold for identifying anomalies when
#'                         \code{detect_anomalies} is TRUE. Default is 0.6.
#' @param n_tree An integer specifying the number of trees for the isolation forest when
#'              \code{detect_anomalies} is TRUE. Default is 500.
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#'
#' @return A list containing:
#' \itemize{
#'   \item PC results: Named elements for each PC (e.g., "PC1", "PC2") containing data frames with gene-level analysis results.
#'   \item expression_data: Matrix of expression values for all analyzed genes (genes × cells).
#'   \item cell_metadata: Data frame with columns: cell_id, dataset, cell_type, original_index, and optionally anomaly_status.
#'   \item gene_metadata: Data frame with columns: gene, pc, loading for all analyzed genes.
#'   \item percent_var: Named numeric vector of global percent variance explained for each analyzed PC.
#'   \item cell_type_variance: A data frame detailing the percent of variance a global PC explains within specific cell types for both query and reference datasets.
#'   \item anomaly_results: If \code{detect_anomalies} is TRUE, contains the full output from \code{detectAnomaly}.
#' }
#'
#' The `cell_type_variance` data frame contains columns: pc, cell_type, dataset, percent_variance.
#' When anomaly detection is enabled, `cell_metadata` includes an additional `anomaly_status` column.
#'
#' @export
#'
#' @author
#' Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{plot.calculateTopLoadingGeneShiftsObject}}, \code{\link{detectAnomaly}}
#'
#' @importFrom stats wilcox.test var p.adjust na.omit setNames
#'
# Function to calculate expression shifts for genes with top loadings
calculateTopLoadingGeneShifts <- function(query_data,
                                          reference_data,
                                          query_cell_type_col,
                                          ref_cell_type_col,
                                          cell_types = NULL,
                                          pc_subset = 1:5,
                                          n_top_loadings = 50,
                                          p_value_threshold = 0.05,
                                          adjust_method = "fdr",
                                          assay_name = "logcounts",
                                          detect_anomalies = FALSE,
                                          anomaly_comparison = FALSE,
                                          anomaly_threshold = 0.6,
                                          n_tree = 500,
                                          max_cells_query = 5000,
                                          max_cells_ref = 5000) {

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  ref_cell_type_col = ref_cell_type_col,
                  query_cell_type_col = query_cell_type_col,
                  pc_subset_query = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Input validation
    if (!is.numeric(n_top_loadings) || length(n_top_loadings) != 1 || n_top_loadings <= 0) {
        stop("n_top_loadings must be a positive integer")
    }

    if (!is.numeric(p_value_threshold) || length(p_value_threshold) != 1 ||
        p_value_threshold < 0 || p_value_threshold > 1) {
        stop("p_value_threshold must be between 0 and 1")
    }

    # Validate anomaly detection parameters
    if (!is.logical(detect_anomalies) || length(detect_anomalies) != 1) {
        stop("detect_anomalies must be a logical value")
    }

    if (detect_anomalies) {
        if (!is.numeric(anomaly_threshold) || length(anomaly_threshold) != 1 ||
            anomaly_threshold <= 0 || anomaly_threshold >= 1) {
            stop("anomaly_threshold must be a numeric value between 0 and 1")
        }

        if (!is.numeric(n_tree) || length(n_tree) != 1 || n_tree <= 0 || n_tree != as.integer(n_tree)) {
            stop("n_tree must be a positive integer")
        }
    }

    # Validate anomaly comparison parameter
    if (!is.logical(anomaly_comparison) || length(anomaly_comparison) != 1) {
        stop("anomaly_comparison must be a logical value")
    }

    if (anomaly_comparison && !detect_anomalies) {
        stop("anomaly_comparison = TRUE requires detect_anomalies = TRUE")
    }

    # Helper function to filter cells by anomaly status
    .filterCellsByAnomalyStatus <- function(cell_indices, cell_names, anomaly_results,
                                            cell_type, dataset_type, keep_anomalous = TRUE) {
        if (is.null(anomaly_results) || !cell_type %in% names(anomaly_results)) {
            return(cell_indices)  # Return all cells if no anomaly data
        }

        # Get anomaly status for this cell type
        if (dataset_type == "reference") {
            anomaly_names <- names(anomaly_results[[cell_type]][["reference_anomaly"]])
            anomaly_status <- anomaly_results[[cell_type]][["reference_anomaly"]]
            # Strip "Reference_" prefix to match cell names
            anomaly_names_clean <- gsub("^Reference_", "", anomaly_names)
        } else {
            anomaly_names <- names(anomaly_results[[cell_type]][["query_anomaly"]])
            anomaly_status <- anomaly_results[[cell_type]][["query_anomaly"]]
            # Strip "Query_" prefix to match cell names
            anomaly_names_clean <- gsub("^Query_", "", anomaly_names)
        }

        if (is.null(anomaly_names) || length(anomaly_names) == 0) {
            return(cell_indices)  # Return all cells if no anomaly data
        }

        # Map cell indices to cell names and then to anomaly status
        current_cell_names <- cell_names[cell_indices]

        # Find which cells are anomalous
        cells_are_anomalous <- current_cell_names %in% anomaly_names_clean[anomaly_status]

        # Filter based on keep_anomalous parameter
        if (keep_anomalous) {
            filtered_indices <- cell_indices[cells_are_anomalous]
        } else {
            filtered_indices <- cell_indices[!cells_are_anomalous]
        }

        return(filtered_indices)
    }

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = TRUE,
                                  n_cell_types = NULL)

    # Ensure cell names exist for anomaly detection mapping
    # Store original cell names or create them if they don't exist
    original_ref_names <- colnames(reference_data)
    original_query_names <- colnames(query_data)

    if (is.null(original_ref_names) || any(is.na(original_ref_names)) ||
        length(unique(original_ref_names)) != length(original_ref_names)) {
        original_ref_names <- paste0("REF_CELL_", seq_len(ncol(reference_data)))
        colnames(reference_data) <- original_ref_names
    }

    if (is.null(original_query_names) || any(is.na(original_query_names)) ||
        length(unique(original_query_names)) != length(original_query_names)) {
        original_query_names <- paste0("QUERY_CELL_", seq_len(ncol(query_data)))
        colnames(query_data) <- original_query_names
    }

    # Get PCA results
    pca_attrs <- attributes(SingleCellExperiment::reducedDim(reference_data, "PCA"))
    pca_rotation <- pca_attrs[["rotation"]]
    percent_var <- pca_attrs[["percentVar"]]
    if (is.null(pca_rotation) || is.null(percent_var)) {
        stop("PCA rotation matrix and/or percentVar not found in reference_data's reducedDim 'PCA' attributes.")
    }
    if (max(pc_subset) > ncol(pca_rotation)) {
        stop("pc_subset contains indices greater than the number of PCs available in the reference.")
    }
    pc_percent_var <- percent_var[pc_subset]
    names(pc_percent_var) <- paste0("PC", pc_subset)

    # Perform anomaly detection FIRST (before any downsampling)
    # This ensures the rotation matrix is intact
    anomaly_results <- NULL
    if (detect_anomalies) {
        tryCatch({
            anomaly_results <- detectAnomaly(
                reference_data = reference_data,
                query_data = query_data,
                ref_cell_type_col = ref_cell_type_col,
                query_cell_type_col = query_cell_type_col,
                cell_types = cell_types,
                pc_subset = pc_subset,
                n_tree = n_tree,
                anomaly_threshold = anomaly_threshold,
                assay_name = assay_name,
                max_cells_ref = NULL,
                max_cells_query = NULL
            )
        }, error = function(e) {
            warning("Anomaly detection failed: ", e[["message"]], ". Continuing without anomaly detection.")
            anomaly_results <<- NULL
        })
    }

    # Now downsample the data (with cell type filtering)
    query_data <- downsampleSCE(sce_object = query_data,
                                max_cells = max_cells_query,
                                cell_types = cell_types,
                                cell_type_col = query_cell_type_col)
    reference_data <- downsampleSCE(sce_object = reference_data,
                                    max_cells = max_cells_ref,
                                    cell_types = cell_types,
                                    cell_type_col = ref_cell_type_col)

    # Check if cell type columns exist after downsampling
    if (!query_cell_type_col %in% names(SummarizedExperiment::colData(query_data))) {
        stop(paste("Column '", query_cell_type_col, "' not found in query_data colData"))
    }

    if (!ref_cell_type_col %in% names(SummarizedExperiment::colData(reference_data))) {
        stop(paste("Column '", ref_cell_type_col, "' not found in reference_data colData"))
    }

    # Get common genes
    common_genes <- intersect(rownames(query_data), rownames(reference_data))
    common_genes <- intersect(common_genes, rownames(pca_rotation))
    if (length(common_genes) == 0) {
        stop("No common genes found between query, reference, and PCA rotation matrix.")
    }
    pca_rotation <- pca_rotation[common_genes, ]

    # Prepare cell indices (data is already filtered by cell type and downsampled)
    query_cell_indices <- split(seq_len(ncol(query_data)), query_data[[query_cell_type_col]])
    ref_cell_indices <- split(seq_len(ncol(reference_data)), reference_data[[ref_cell_type_col]])
    query_cell_indices <- query_cell_indices[names(query_cell_indices) %in% cell_types]
    ref_cell_indices <- ref_cell_indices[names(ref_cell_indices) %in% cell_types]
    available_cell_types <- intersect(names(query_cell_indices), names(ref_cell_indices))
    if (length(available_cell_types) == 0) {
        warning("No common cell types with sufficient cells found between datasets.")
        return(list())
    }

    # Gene-level statistical analysis
    all_top_genes <- character(0)
    gene_metadata_list <- list()
    pc_results <- stats::setNames(vector("list", length(pc_subset)), paste0("PC", pc_subset))

    for (pc in pc_subset) {
        pc_name <- paste0("PC", pc)
        pc_loadings <- pca_rotation[, pc]
        top_loading_indices <- order(abs(pc_loadings),
                                     decreasing = TRUE)[1:min(n_top_loadings, length(pc_loadings))]
        top_genes <- names(pc_loadings)[top_loading_indices]
        top_loadings_vals <- pc_loadings[top_loading_indices]

        all_top_genes <- unique(c(all_top_genes, top_genes))
        gene_metadata_list[[pc_name]] <- data.frame(gene = top_genes,
                                                    pc = pc,
                                                    loading = top_loadings_vals,
                                                    stringsAsFactors = FALSE)

        pc_result_list <- list()
        for (ct in available_cell_types) {
            query_cells_ct <- query_cell_indices[[ct]]
            ref_cells_ct <- ref_cell_indices[[ct]]

            # Apply anomaly filtering if requested
            if (anomaly_comparison && !is.null(anomaly_results)) {
                # Filter reference cells: keep only NON-anomalous
                ref_cells_ct <- .filterCellsByAnomalyStatus(
                    ref_cells_ct,
                    colnames(reference_data),
                    anomaly_results,
                    ct,
                    "reference",
                    keep_anomalous = FALSE
                )

                # Filter query cells: keep only ANOMALOUS
                query_cells_ct <- .filterCellsByAnomalyStatus(
                    query_cells_ct,
                    colnames(query_data),
                    anomaly_results,
                    ct,
                    "query",
                    keep_anomalous = TRUE
                )

                # Check if we have enough cells after filtering
                if (length(query_cells_ct) < 3) {
                    warning("Cell type '", ct, "' has fewer than 3 anomalous query cells. Skipping statistical analysis.")
                    next
                }
                if (length(ref_cells_ct) < 3) {
                    warning("Cell type '", ct, "' has fewer than 3 non-anomalous reference cells. Skipping statistical analysis.")
                    next
                }
            } else {
                # Original check for minimum cell counts
                if (length(query_cells_ct) < 3 || length(ref_cells_ct) < 3) next
            }

            query_expr_ct <- SummarizedExperiment::assay(query_data,
                                                         assay_name)[top_genes,
                                                                     query_cells_ct, drop = FALSE]
            ref_expr_ct <- SummarizedExperiment::assay(reference_data,
                                                       assay_name)[top_genes,
                                                                   ref_cells_ct, drop = FALSE]

            gene_results <- processGenesSimple(top_genes,
                                               top_loadings_vals,
                                               query_expr_ct,
                                               ref_expr_ct,
                                               ct)
            if (length(gene_results) > 0)
                pc_result_list <- c(pc_result_list, gene_results)
        }

        if (length(pc_result_list) > 0) {
            df <- do.call(rbind, pc_result_list)
            df[["p_adjusted"]] <- stats::p.adjust(df[["p_value"]], method = adjust_method)
            df[["significant"]] <- df[["p_adjusted"]] <= p_value_threshold
            pc_results[[pc_name]] <- df[order(df[["p_adjusted"]]), ]
            rownames(pc_results[[pc_name]]) <- NULL
        } else {
            pc_results[[pc_name]] <- data.frame()
        }
    }
    gene_metadata <- do.call(rbind, gene_metadata_list)
    rownames(gene_metadata) <- NULL

    # Cell-type-specific variance explained by global PCs
    var_explained_list <- list()
    full_ref_assay <- SummarizedExperiment::assay(reference_data,
                                                  assay_name)[common_genes, ]
    full_query_assay <- SummarizedExperiment::assay(query_data,
                                                    assay_name)[common_genes, ]

    for (ct in available_cell_types) {
        ref_cells_ct <- ref_cell_indices[[ct]]
        query_cells_ct <- query_cell_indices[[ct]]

        # Apply the same anomaly filtering for variance calculation if anomaly_comparison is TRUE
        if (anomaly_comparison && !is.null(anomaly_results)) {
            ref_cells_ct <- .filterCellsByAnomalyStatus(
                ref_cells_ct,
                colnames(reference_data),
                anomaly_results,
                ct,
                "reference",
                keep_anomalous = FALSE
            )

            query_cells_ct <- .filterCellsByAnomalyStatus(
                query_cells_ct,
                colnames(query_data),
                anomaly_results,
                ct,
                "query",
                keep_anomalous = TRUE
            )
        }

        ref_expr_ct <- full_ref_assay[, ref_cells_ct, drop = FALSE]
        query_expr_ct <- full_query_assay[, query_cells_ct, drop = FALSE]

        total_var_ref <- if(ncol(ref_expr_ct) > 1) sum(apply(as.matrix(ref_expr_ct), 1, stats::var)) else 0
        total_var_query <- if(ncol(query_expr_ct) > 1) sum(apply(as.matrix(query_expr_ct), 1, stats::var)) else 0

        for (pc in pc_subset) {
            pc_name <- paste0("PC", pc)
            loadings_pc <- pca_rotation[, pc, drop = FALSE]

            if (total_var_ref > .Machine[["double.eps"]]) {
                ref_scores <- crossprod(ref_expr_ct, loadings_pc)
                var_scores_ref <- stats::var(as.vector(ref_scores))
                pct_var_ref <- (var_scores_ref / total_var_ref) * 100
                var_explained_list[[length(var_explained_list) + 1]] <- data.frame(
                    pc = pc_name, cell_type = ct, dataset = "Reference",
                    percent_variance = pct_var_ref, stringsAsFactors = FALSE)
            }
            if (total_var_query > .Machine[["double.eps"]]) {
                query_scores <- crossprod(query_expr_ct, loadings_pc)
                var_scores_query <- stats::var(as.vector(query_scores))
                pct_var_query <- (var_scores_query / total_var_query) * 100
                var_explained_list[[length(var_explained_list) + 1]] <- data.frame(
                    pc = pc_name, cell_type = ct, dataset = "Query",
                    percent_variance = pct_var_query, stringsAsFactors = FALSE)
            }
        }
    }
    cell_type_variance_df <- do.call(rbind, var_explained_list)
    rownames(cell_type_variance_df) <- NULL

    # Data for plotting (use all cells, not filtered by anomaly status)
    query_relevant_cells <- unlist(query_cell_indices[available_cell_types], use.names = FALSE)
    ref_relevant_cells <- unlist(ref_cell_indices[available_cell_types], use.names = FALSE)

    query_expr_plot <- SummarizedExperiment::assay(query_data,
                                                   assay_name)[all_top_genes,
                                                               query_relevant_cells, drop = FALSE]
    ref_expr_plot <- SummarizedExperiment::assay(reference_data,
                                                 assay_name)[all_top_genes,
                                                             ref_relevant_cells, drop = FALSE]
    expression_data <- cbind(ref_expr_plot, query_expr_plot)

    # Create cell metadata
    cell_metadata <- rbind(
        data.frame(cell_id = colnames(ref_expr_plot), dataset = "Reference",
                   cell_type = reference_data[[ref_cell_type_col]][ref_relevant_cells],
                   original_index = ref_relevant_cells, stringsAsFactors = FALSE),
        data.frame(cell_id = colnames(query_expr_plot), dataset = "Query",
                   cell_type = query_data[[query_cell_type_col]][query_relevant_cells],
                   original_index = query_relevant_cells, stringsAsFactors = FALSE)
    )

    # Map anomaly status using cell names
    if (!is.null(anomaly_results)) {
        # Initialize anomaly status
        cell_metadata[["anomaly_status"]] <- "Normal"

        # Process anomaly results for each cell type
        for (ct in available_cell_types) {
            if (ct %in% names(anomaly_results)) {

                # Map reference anomaly status using cell names
                if ("reference_anomaly" %in% names(anomaly_results[[ct]])) {
                    ref_anomaly_names <- names(anomaly_results[[ct]][["reference_anomaly"]])
                    ref_anomaly_status <- anomaly_results[[ct]][["reference_anomaly"]]

                    if (!is.null(ref_anomaly_names) && length(ref_anomaly_names) > 0) {
                        # Find which cells are anomalous and strip "Reference_" prefix
                        anomalous_ref_names <- ref_anomaly_names[ref_anomaly_status]
                        anomalous_ref_names_clean <- gsub("^Reference_", "", anomalous_ref_names)

                        ref_mask <- cell_metadata[["dataset"]] == "Reference" &
                            cell_metadata[["cell_id"]] %in% anomalous_ref_names_clean
                        cell_metadata[["anomaly_status"]][ref_mask] <- "Anomaly"
                    }
                }

                # Map query anomaly status using cell names
                if ("query_anomaly" %in% names(anomaly_results[[ct]])) {
                    query_anomaly_names <- names(anomaly_results[[ct]][["query_anomaly"]])
                    query_anomaly_status <- anomaly_results[[ct]][["query_anomaly"]]

                    if (!is.null(query_anomaly_names) && length(query_anomaly_names) > 0) {
                        # Find which cells are anomalous and strip "Query_" prefix
                        anomalous_query_names <- query_anomaly_names[query_anomaly_status]
                        anomalous_query_names_clean <- gsub("^Query_", "", anomalous_query_names)

                        query_mask <- cell_metadata[["dataset"]] == "Query" &
                            cell_metadata[["cell_id"]] %in% anomalous_query_names_clean
                        cell_metadata[["anomaly_status"]][query_mask] <- "Anomaly"
                    }
                }
            }
        }
    }

    # Assemble final results
    final_results <- c(
        pc_results,
        list(
            expression_data = expression_data,
            cell_metadata = cell_metadata,
            gene_metadata = gene_metadata,
            pca_rotation = pca_rotation,
            percent_var = pc_percent_var,
            cell_type_variance = cell_type_variance_df
        )
    )

    # Add analysis type information
    final_results[["analysis_type"]] <- if (anomaly_comparison && !is.null(anomaly_results)) {
        "non_anomalous_reference_vs_anomalous_query"
    } else {
        "all_reference_vs_all_query"
    }

    # Add anomaly results if available
    if (!is.null(anomaly_results)) {
        final_results[["anomaly_results"]] <- anomaly_results
    }

    class(final_results) <- c(class(final_results), "calculateTopLoadingGeneShiftsObject")
    return(final_results)
}

#' @title Process Genes for Statistical Analysis
#'
#' @description
#' Internal helper function that performs statistical testing on top loading genes
#' to compare their expression distributions between query and reference datasets.
#'
#' @param top_genes Character vector of gene names to analyze.
#' @param top_loadings Numeric vector of PC loading values corresponding to the genes.
#' @param query_expr_matrix Numeric matrix of expression values for query data (genes × cells).
#' @param ref_expr_matrix Numeric matrix of expression values for reference data (genes × cells).
#' @param cell_type Character string specifying the cell type being analyzed.
#'
#' @keywords internal
#'
#' @return A list of data frames, where each data frame contains results for one gene.
#'
# Helper function to process genes for statistical analysis
processGenesSimple <- function(top_genes,
                               top_loadings,
                               query_expr_matrix,
                               ref_expr_matrix,
                               cell_type) {

    result_list <- list()

    for (i in seq_along(top_genes)) {
        gene <- top_genes[i]
        loading <- top_loadings[i]

        query_expr <- query_expr_matrix[gene, ]
        ref_expr <- ref_expr_matrix[gene, ]

        query_expr <- query_expr[!is.na(query_expr)]
        ref_expr <- ref_expr[!is.na(ref_expr)]

        if (length(query_expr) < 3 || length(ref_expr) < 3) {
            next
        }

        mean_query <- mean(query_expr)
        mean_reference <- mean(ref_expr)

        test_result <- suppressWarnings(
            stats::wilcox.test(query_expr, ref_expr, alternative = "two.sided")
        )

        result_row <- data.frame(
            gene = gene,
            loading = loading,
            cell_type = cell_type,
            p_value = test_result[["p.value"]],
            mean_query = mean_query,
            mean_reference = mean_reference,
            stringsAsFactors = FALSE
        )

        result_list[[length(result_list) + 1]] <- result_row
    }

    return(result_list)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Test calculateTopLoadingGeneShifts function
test_that("calculateTopLoadingGeneShifts works with valid inputs", {

    # Test with default parameters
    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation"
    )

    # Test return structure
    expect_type(result, "list")
    expect_s3_class(result, "calculateTopLoadingGeneShiftsObject")

    # Test required elements exist
    expected_elements <- c("expression_data", "cell_metadata", "gene_metadata", "percent_var")
    expect_true(all(expected_elements %in% names(result)))

    # Test PC elements exist (default is 1:5)
    pc_names <- paste0("PC", 1:5)
    expect_true(all(pc_names %in% names(result)))

    # Test expression_data structure
    expect_true(nrow(result[["expression_data"]]) > 0)
    expect_true(ncol(result[["expression_data"]]) > 0)

    # Test cell_metadata structure
    expect_s3_class(result[["cell_metadata"]], "data.frame")
    expected_cell_cols <- c("cell_id", "dataset", "cell_type", "original_index")
    expect_true(all(expected_cell_cols %in% colnames(result[["cell_metadata"]])))
    expect_true(all(result[["cell_metadata"]][["dataset"]] %in% c("Reference", "Query")))

    # Test gene_metadata structure
    expect_s3_class(result[["gene_metadata"]], "data.frame")
    expected_gene_cols <- c("gene", "pc", "loading")
    expect_true(all(expected_gene_cols %in% colnames(result[["gene_metadata"]])))

    # Test percent_var structure
    expect_type(result[["percent_var"]], "double")
    expect_named(result[["percent_var"]], paste0("PC", 1:5))
    expect_true(all(result[["percent_var"]] >= 0))
    expect_true(all(result[["percent_var"]] <= 100))
})

test_that("calculateTopLoadingGeneShifts works with custom parameters", {

    # Test with custom pc_subset
    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:3,
        n_top_loadings = 20,
        p_value_threshold = 0.01,
        adjust_method = "bonferroni"
    )

    # Test correct PC subset
    expect_true(all(paste0("PC", 1:3) %in% names(result)))
    expect_false("PC4" %in% names(result))
    expect_false("PC5" %in% names(result))

    # Test percent_var has correct names
    expect_named(result[["percent_var"]], paste0("PC", 1:3))

    # Test gene metadata has correct PCs
    expect_true(all(result[["gene_metadata"]][["pc"]] %in% 1:3))
})

test_that("calculateTopLoadingGeneShifts works with specific cell types", {

    # Get available cell types
    common_cell_types <- intersect(
        unique(query_data[["SingleR_annotation"]]),
        unique(reference_data[["expert_annotation"]])
    )

    # Test with specific cell types
    selected_cell_types <- head(common_cell_types, 2)
    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_types = selected_cell_types,
        pc_subset = 1:2
    )

    # Test that only selected cell types are present
    unique_cell_types <- unique(result[["cell_metadata"]][["cell_type"]])
    expect_true(all(unique_cell_types %in% selected_cell_types))

    # Test PC results only contain selected cell types
    for (pc_name in paste0("PC", 1:2)) {
        if (nrow(result[[pc_name]]) > 0) {
            expect_true(all(result[[pc_name]][["cell_type"]] %in% selected_cell_types))
        }
    }
})

test_that("calculateTopLoadingGeneShifts handles edge cases", {

    # Test with single PC
    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1,
        n_top_loadings = 10
    )

    expect_true("PC1" %in% names(result))
    expect_false("PC2" %in% names(result))
    expect_length(result[["percent_var"]], 1)

    # Test with small n_top_loadings
    result_small <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:2,
        n_top_loadings = 5
    )

    expect_true(nrow(result_small[["gene_metadata"]]) <= 10)  # 5 genes * 2 PCs
})

test_that("calculateTopLoadingGeneShifts input validation works", {

    # Test invalid n_top_loadings
    expect_error(
        calculateTopLoadingGeneShifts(
            query_data = query_data,
            reference_data = reference_data,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            n_top_loadings = -5
        ),
        "n_top_loadings must be a positive integer"
    )

    expect_error(
        calculateTopLoadingGeneShifts(
            query_data = query_data,
            reference_data = reference_data,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            n_top_loadings = 0
        ),
        "n_top_loadings must be a positive integer"
    )

    # Test invalid p_value_threshold
    expect_error(
        calculateTopLoadingGeneShifts(
            query_data = query_data,
            reference_data = reference_data,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            p_value_threshold = -0.1
        ),
        "p_value_threshold must be between 0 and 1"
    )

    expect_error(
        calculateTopLoadingGeneShifts(
            query_data = query_data,
            reference_data = reference_data,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            p_value_threshold = 1.5
        ),
        "p_value_threshold must be between 0 and 1"
    )

    # Test invalid cell type columns
    expect_error(
        calculateTopLoadingGeneShifts(
            query_data = query_data,
            reference_data = reference_data,
            query_cell_type_col = "nonexistent_column",
            ref_cell_type_col = "expert_annotation"
        )
    )

    expect_error(
        calculateTopLoadingGeneShifts(
            query_data = query_data,
            reference_data = reference_data,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "nonexistent_column"
        )
    )
})

test_that("calculateTopLoadingGeneShifts handles different adjust methods", {

    methods <- c("fdr", "bonferroni", "holm", "none")

    for (method in methods) {
        result <- calculateTopLoadingGeneShifts(
            query_data = query_data,
            reference_data = reference_data,
            query_cell_type_col = "SingleR_annotation",
            ref_cell_type_col = "expert_annotation",
            pc_subset = 1:2,
            n_top_loadings = 10,
            adjust_method = method
        )

        expect_s3_class(result, "calculateTopLoadingGeneShiftsObject")

        # Test that p_adjusted values are reasonable
        for (pc_name in paste0("PC", 1:2)) {
            if (nrow(result[[pc_name]]) > 0) {
                expect_true(all(result[[pc_name]][["p_adjusted"]] >= 0))
                expect_true(all(result[[pc_name]][["p_adjusted"]] <= 1))

                # For most methods, p_adjusted should be >= p_value
                if (method != "none") {
                    expect_true(all(result[[pc_name]][["p_adjusted"]] >=
                                        result[[pc_name]][["p_value"]]))
                }
            }
        }
    }
})


test_that("processGenesSimple helper function works correctly", {

    # Create test data with more cells for robust statistical testing
    test_genes <- c("GENE1", "GENE2")
    test_loadings <- c(0.5, -0.3)

    # Create test expression matrices with more cells (5 instead of 3)
    set.seed(123)  # For reproducible results
    query_matrix <- matrix(c(1, 2, 3, 4, 5,
                             2, 3, 4, 5, 6), nrow = 2, ncol = 5, byrow = TRUE)
    ref_matrix <- matrix(c(2, 3, 4, 5, 6,
                           3, 4, 5, 6, 7), nrow = 2, ncol = 5, byrow = TRUE)

    rownames(query_matrix) <- test_genes
    rownames(ref_matrix) <- test_genes

    result <- scDiagnostics:::processGenesSimple(
        top_genes = test_genes,
        top_loadings = test_loadings,
        query_expr_matrix = query_matrix,
        ref_expr_matrix = ref_matrix,
        cell_type = "TestCellType"
    )

    expect_type(result, "list")
    expect_length(result, 2)

    # Test first gene result
    gene1_result <- result[[1]]
    expect_s3_class(gene1_result, "data.frame")
    expect_equal(gene1_result[["gene"]], "GENE1")
    expect_equal(gene1_result[["loading"]], 0.5)
    expect_equal(gene1_result[["cell_type"]], "TestCellType")
    expect_true(is.numeric(gene1_result[["p_value"]]))
    expect_true(gene1_result[["p_value"]] >= 0 && gene1_result[["p_value"]] <= 1)
    expect_true(is.numeric(gene1_result[["mean_query"]]))
    expect_true(is.numeric(gene1_result[["mean_reference"]]))

    # Test that means are calculated correctly
    expect_equal(gene1_result[["mean_query"]], mean(c(1, 2, 3, 4, 5)))
    expect_equal(gene1_result[["mean_reference"]], mean(c(2, 3, 4, 5, 6)))

    # Test second gene result
    gene2_result <- result[[2]]
    expect_equal(gene2_result[["gene"]], "GENE2")
    expect_equal(gene2_result[["loading"]], -0.3)
    expect_equal(gene2_result[["cell_type"]], "TestCellType")
})

test_that("calculateTopLoadingGeneShifts preserves gene order by significance", {

    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:2,
        n_top_loadings = 20
    )

    # Test that results are sorted by p_adjusted (ascending)
    for (pc_name in paste0("PC", 1:2)) {
        if (nrow(result[[pc_name]]) > 1) {
            p_values <- result[[pc_name]][["p_adjusted"]]
            expect_true(all(p_values == sort(p_values)))
        }
    }
})

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")


test_that("plot method works correctly", {

    # Use package data to create a proper calculateTopLoadingGeneShiftsObject
    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:3,
        n_top_loadings = 20
    )

    # Get available cell types for testing
    available_cell_types <- unique(result$cell_metadata$cell_type)
    test_cell_type <- available_cell_types[1]

    # Test input validation
    expect_error(
        plot(result),
        "cell_type must be specified and be a single character string"
    )

    expect_error(
        plot(result, cell_type = c("CD4", "CD8")),
        "cell_type must be specified and be a single character string"
    )

    expect_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot", n_genes = NULL),
        "For 'boxplot' plot type, 'n_genes' must be specified"
    )

    expect_error(
        plot(result, cell_type = test_cell_type, plot_type = "heatmap",
             n_genes = NULL, significance_threshold = NULL),
        "For 'heatmap' plot type, at least one of 'n_genes' or 'significance_threshold' must be specified"
    )

    expect_error(
        plot(result, cell_type = test_cell_type, n_genes = -5),
        "If not NULL, 'n_genes' must be a positive integer"
    )

    expect_error(
        plot(result, cell_type = test_cell_type, n_genes = 0),
        "If not NULL, 'n_genes' must be a positive integer"
    )

    expect_error(
        plot(result, cell_type = test_cell_type, significance_threshold = -0.1),
        "If not NULL, 'significance_threshold' must be a numeric value between 0 and 1"
    )

    expect_error(
        plot(result, cell_type = test_cell_type, significance_threshold = 1.5),
        "If not NULL, 'significance_threshold' must be a numeric value between 0 and 1"
    )

    # Test nonexistent cell type
    expect_error(
        plot(result, cell_type = "NonexistentType"),
        "Cell type 'NonexistentType' not found in results"
    )

    # Test no available PCs
    expect_error(
        plot(result, cell_type = test_cell_type, pc_subset = 10:15),
        "None of the requested PCs were found in the results object"
    )

    # Test boxplot PC limit warning
    expect_warning(
        {
            # Create result with more PCs to trigger the warning
            result_many_pcs <- calculateTopLoadingGeneShifts(
                query_data = query_data,
                reference_data = reference_data,
                query_cell_type_col = "SingleR_annotation",
                ref_cell_type_col = "expert_annotation",
                pc_subset = 1:8,  # Create with 8 PCs
                n_top_loadings = 20
            )
            plot(result_many_pcs,
                 cell_type = test_cell_type,
                 plot_type = "boxplot",
                 pc_subset = 1:8)  # Request all 8 PCs
        },
        "Boxplot type can display at most 5 PCs. Using the first 5"
    )
})

test_that("plot method argument matching works", {

    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:3,
        n_top_loadings = 20
    )

    test_cell_type <- unique(result$cell_metadata$cell_type)[1]

    # Test plot_type matching
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot")
    )

    # Test plot_by matching
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_by = "p_adjusted")
    )

    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_by = "top_loading")
    )
})

test_that("plot method returns appropriate objects for boxplot", {

    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:3,
        n_top_loadings = 20
    )

    test_cell_type <- unique(result$cell_metadata$cell_type)[1]

    # Test boxplot returns ggplot object
    plot_result <- plot(result, cell_type = test_cell_type, plot_type = "boxplot")
    expect_s3_class(plot_result, "ggplot")
})

test_that("plot method object structure validation works", {

    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:3,
        n_top_loadings = 20
    )

    test_cell_type <- unique(result$cell_metadata$cell_type)[1]

    # Test missing required elements
    incomplete_data <- result
    incomplete_data$expression_data <- NULL

    expect_error(
        plot(incomplete_data, cell_type = test_cell_type),
        "Input object 'x' is missing required elements"
    )
})

test_that("plot method works with different parameter combinations", {

    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:3,
        n_top_loadings = 20
    )

    test_cell_type <- unique(result$cell_metadata$cell_type)[1]

    # Test boxplot with different plot_by options
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot",
             plot_by = "top_loading")
    )

    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot",
             plot_by = "p_adjusted")
    )

    # Test different n_genes values
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot", n_genes = 5)
    )

    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot", n_genes = 15)
    )
})

test_that("plot method handles different pc_subset configurations", {

    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:5,
        n_top_loadings = 20
    )

    test_cell_type <- unique(result$cell_metadata$cell_type)[1]

    # Test single PC
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot", pc_subset = 1)
    )

    # Test multiple PCs
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot", pc_subset = 1:3)
    )

    # Test different PC combinations
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot", pc_subset = c(1, 3, 5))
    )
})

test_that("plot method handles significance_threshold correctly", {

    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:3,
        n_top_loadings = 20
    )

    test_cell_type <- unique(result$cell_metadata$cell_type)[1]

    # Test different significance thresholds
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot",
             significance_threshold = 0.01)
    )

    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot",
             significance_threshold = 0.1)
    )

    # Test with NULL significance_threshold
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot",
             significance_threshold = NULL)
    )
})

# Conditional tests for heatmap functionality (only if ComplexHeatmap is available)
test_that("plot method heatmap functionality works when ComplexHeatmap is available", {

    skip_if_not_installed("ComplexHeatmap")

    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:3,
        n_top_loadings = 20
    )

    test_cell_type <- unique(result$cell_metadata$cell_type)[1]

    # Test heatmap with n_genes only
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "heatmap", n_genes = 5)
    )

    # Test heatmap with significance_threshold only
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "heatmap",
             n_genes = NULL, significance_threshold = 0.05)
    )

    # Test heatmap with both n_genes and significance_threshold
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "heatmap",
             n_genes = 5, significance_threshold = 0.05)
    )
})

test_that("plot method handles empty gene selection gracefully", {

    result <- calculateTopLoadingGeneShifts(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:3,
        n_top_loadings = 20
    )

    test_cell_type <- unique(result$cell_metadata$cell_type)[1]

    # Test boxplot with very small n_genes (should still work)
    expect_no_error(
        plot(result, cell_type = test_cell_type, plot_type = "boxplot", n_genes = 1)
    )

    # This would only test heatmap empty selection if ComplexHeatmap is available
    skip_if_not_installed("ComplexHeatmap")

    # Test with very restrictive significance threshold for heatmap
    expect_message(
        heatmap_result <- plot(result, cell_type = test_cell_type, plot_type = "heatmap",
                               n_genes = NULL, significance_threshold = 0.0001),
        "No data available to generate a heatmap"
    )
    expect_null(heatmap_result)
})

#' @title Visualization of gene sets or pathway scores on dimensional reduction plot
#'
#' @description
#' Plot gene sets or pathway scores on PCA, TSNE, or UMAP. Single cells are color-coded by scores of gene sets or pathways.
#'
#' @details
#' This function plots gene set scores on reduced dimensions such as PCA, t-SNE, or UMAP.
#' It extracts the reduced dimensions from the provided \code{\linkS4class{SingleCellExperiment}} object.
#' Gene set scores are visualized as a scatter plot with colors indicating the scores.
#' For PCA, the function automatically includes the percentage of variance explained
#' in the plot's legend.
#'
#' @param sce_object An object of class \code{\linkS4class{SingleCellExperiment}} containing numeric expression matrix and other metadata.
#'        It can be either a reference or query dataset.
#' @param cell_type_col The column name in the \code{colData} of \code{sce_object} that identifies the cell types.
#' @param method A character string indicating the method for visualization ("PCA", "TSNE", or "UMAP").
#' @param score_col A character string representing the name of the score_col (score) in the colData(sce_object) to plot.
#' @param pc_subset An optional vector specifying the principal components (PCs) to include in the plot if method = "PCA".
#'        Default is 1:5.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.
#' @param max_cells Maximum number of cells to retain. If the object has fewer cells, it is returned unchanged.
#'                  Default is 2000.
#'
#' @return A ggplot2 object representing the gene set scores plotted on the specified reduced dimensions.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("query_data")
#'
#' # Plot gene set scores on PCA
#' plotGeneSetScores(sce_object = query_data,
#'                   method = "PCA",
#'                   score_col = "gene_set_scores",
#'                   pc_subset = 1:5,
#'                   cell_types = "CD8",
#'                   cell_type_col = "SingleR_annotation")
#'
# Function to plot the gene set scores
plotGeneSetScores <- function(sce_object,
                              cell_type_col,
                              method = c("PCA", "TSNE", "UMAP"),
                              score_col,
                              pc_subset = 1:5,
                              cell_types = NULL,
                              max_cells = 2000) {

    # Check standard input arguments
    argumentCheck(query_data = sce_object,
                  pc_subset_query = pc_subset,
                  query_cell_type_col = cell_type_col,
                  max_cells_query = max_cells)

    # Convert cell type columns to character if needed
    sce_object <- convertColumnsToCharacter(sce_object = sce_object,
                                            convert_cols = cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = sce_object,
                                  query_cell_type_col = cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = NULL)

    # Match method argument
    method <- match.arg(method)

    # Store PCA attributes BEFORE downsampling (if method is PCA)
    pca_percent_var <- NULL
    if (method == "PCA" && "PCA" %in% reducedDimNames(sce_object)) {
        pca_attrs <- attributes(reducedDim(sce_object, "PCA"))
        if (!is.null(pca_attrs[["percentVar"]])) {
            pca_percent_var <- pca_attrs[["percentVar"]]
        }
    }

    # Downsample SCE object
    sce_object <- downsampleSCE(sce_object = sce_object,
                               max_cells = max_cells,
                               cell_types = cell_types,
                               cell_type_col = cell_type_col)

    # Check if dimension reduction method is present in reference's reducedDims
    if (!(method %in% names(reducedDims(sce_object)))) {
        stop("\'sce_object\' must have pre-computed \'",
             method,
             "\' in \'reducedDims\'.")
    }

    # Check if score_col is a valid column name in sce_object
    if (!score_col %in% colnames(colData(sce_object))) {
        stop("score_col: '", score_col,
             "' is not a valid column name in sce_object.")
    }

    # Handle cell type filtering
    filtered_to_specific_types <- FALSE
    if (!is.null(cell_type_col)) {
        # Check if cell type column exists
        if (!cell_type_col %in% colnames(colData(sce_object))) {
            stop("Specified cell_type_col does not exist in colData.")
        }

        # Get available cell types
        available_cell_types <- unique(colData(sce_object)[[cell_type_col]])

        # If specific cell types are requested, filter to those
        if (!is.null(cell_types)) {
            # Check if requested cell types exist
            missing_types <- setdiff(cell_types, available_cell_types)
            if (length(missing_types) > 0) {
                warning("The following cell types were not found: ",
                        paste(missing_types, collapse = ", "))
            }

            # Filter to cells of specified types
            cell_mask <- colData(sce_object)[[cell_type_col]] %in%
                cell_types
            filtered_to_specific_types <- TRUE
        } else {
            # Use all cells if no specific types requested
            cell_mask <- rep(TRUE, ncol(sce_object))
            cell_types <- available_cell_types
        }

        # Filter the object
        sce_object <- sce_object[, cell_mask]
    }

    # Extract scores (after potential filtering)
    scores <- colData(sce_object)[[score_col]]

    if(method %in% c("TSNE", "UMAP")){

        # Extract dimension reduction coordinates from SingleCellExperiment object
        reduction <- reducedDim(sce_object, method)

        # Prepare data for plotting
        df <- data.frame(Dim1 = reduction[, 1], Dim2 = reduction[, 2],
                         Scores = scores)

        # Add cell type information if available
        if (!is.null(cell_type_col)) {
            df[["CellType"]] <- colData(sce_object)[[cell_type_col]]
        }

        # Create the plot object with the original color scheme
        plot_obj <- ggplot2::ggplot(df, ggplot2::aes(x = .data[["Dim1"]],
                                                     y = .data[["Dim2"]])) +
            ggplot2::geom_point(ggplot2::aes(color = .data[["Scores"]]),
                                alpha = 0.7, size = 1.2) +
            ggplot2::scale_color_gradientn(
                colors = c("#2171B5", "#8AABC1", "#FFEDA0", "#E6550D"),
                values = seq(0, 1, by = 1/3),
                limits = c(min(scores, na.rm = TRUE), max(scores, na.rm = TRUE)),
                name = "Scores") +
            ggplot2::xlab("Dimension 1") +
            ggplot2::ylab("Dimension 2") +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                strip.background = ggplot2::element_rect(fill = "grey85",
                                                         color = "grey70"),
                strip.text = ggplot2::element_text(size = 10,
                                                   face = "bold",
                                                   color = "black"),
                axis.title = ggplot2::element_text(size = 12),
                axis.text = ggplot2::element_text(size = 10),
                panel.grid = ggplot2::element_blank(),
                panel.background = ggplot2::element_rect(fill = "white",
                                                         color = "black"),
                legend.position = "right",
                plot.title = ggplot2::element_text(size = 14, hjust = 0.5),
                plot.background = ggplot2::element_rect(fill = "white"))

        # Add title with cell type info if filtered
        if (!is.null(cell_type_col) && !is.null(cell_types)) {
            plot_obj <- plot_obj +
                ggplot2::ggtitle(paste0("Gene Set Scores in ",
                                        paste(cell_types, collapse = ", "),
                                        " Cells"))
        }

    } else if (method == "PCA"){

        # Check input for pc_subset
        if(!all(pc_subset %in% seq_len(ncol(reducedDim(sce_object, "PCA")))))
            stop("\'pc_subset\' is out of range.")

        # PCA data
        plot_mat <- reducedDim(sce_object, "PCA")[, pc_subset]

        # Create PC column names with variance explained (always show percentages if available)
        if (!is.null(pca_percent_var) && length(pca_percent_var) >= max(pc_subset)) {
            plot_names <- paste0(
                "PC", pc_subset, " (",
                sprintf("%.1f%%", pca_percent_var[pc_subset]), ")")
        } else {
            # Fallback if percentVar is not available
            plot_names <- paste0("PC", pc_subset)
        }

        # Create a new data frame with selected PCs
        pc_df <- data.frame(matrix(0, nrow = nrow(plot_mat),
                                   ncol = length(pc_subset)))
        colnames(pc_df) <- plot_names

        for (i in 1:length(pc_subset)) {
            pc_df[, i] <- plot_mat[, i]
        }

        # Add scores data
        pc_df[["Scores"]] <- scores

        # Add cell type information if available
        if (!is.null(cell_type_col)) {
            pc_df[["CellType"]] <- colData(sce_object)[[cell_type_col]]
        }

        # Create a simple plot to extract the legend with original color scheme
        legend_plot <- ggplot2::ggplot(pc_df,
                                       ggplot2::aes(x = pc_df[,1],
                                                    y = pc_df[,2])) +
            ggplot2::geom_point(ggplot2::aes(color = .data[["Scores"]])) +
            ggplot2::scale_color_gradientn(
                colors = c("#2171B5", "#8AABC1", "#FFEDA0", "#E6550D"),
                values = seq(0, 1, by = 1/3),
                limits = c(min(scores, na.rm = TRUE), max(scores,
                                                          na.rm = TRUE)),
                name = "Scores") +
            ggplot2::theme(
                legend.position = "right",
                legend.box = "vertical",
                legend.key = ggplot2::element_rect(fill = "white"),
                legend.background = ggplot2::element_blank()
            )

        # Lower facet - always scatter with score coloring
        .scoresScatterFunc <- function(data, mapping, ...) {
            ggplot2::ggplot(data = data, mapping = mapping) +
                ggplot2::geom_point(alpha = 0.7, size = 1.2,
                                    ggplot2::aes(color = .data[["Scores"]])) +
                ggplot2::scale_color_gradientn(
                    colors = c("#2171B5", "#8AABC1", "#FFEDA0", "#E6550D"),
                    values = seq(0, 1, by = 1/3),
                    limits = c(min(scores, na.rm = TRUE), max(scores,
                                                              na.rm = TRUE))) +
                ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(
                        color = "black", fill = NA, linewidth = 0.5),
                    legend.position = "none")
        }

        # Blank facet function for diagonal and upper
        .blankFunc <- function(data, mapping, ...) {
            ggplot2::ggplot() +
                ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(
                        color = "black", fill = NA, linewidth = 0.5),
                    legend.position = "none",
                    axis.text = ggplot2::element_blank(),
                    axis.ticks = ggplot2::element_blank(),
                    axis.title = ggplot2::element_blank(),
                    panel.grid = ggplot2::element_blank())
        }

        # Create pairs plot using GGally
        plot_obj <- suppressMessages(
            GGally::ggpairs(
                pc_df,
                columns = seq_len(length(pc_subset)),
                mapping = ggplot2::aes(color = .data[["Scores"]]),
                lower = list(continuous = .scoresScatterFunc),
                upper = list(continuous = .blankFunc),
                diag = list(continuous = .blankFunc),
                progress = FALSE,
                legend = GGally::grab_legend(legend_plot)
            )
        )

        # Add consistent theming
        plot_obj <- plot_obj +
            ggplot2::theme(
                strip.background = ggplot2::element_rect(
                    fill = "white", color = "black", linewidth = 0.5),
                strip.text = ggplot2::element_text(color = "black")
            )

        # Add title with cell type info if filtered
        if (!is.null(cell_type_col) && !is.null(cell_types)) {
            plot_obj <- plot_obj +
                ggplot2::labs(title = paste0("Gene Set Scores in ",
                                             paste(cell_types,
                                                   collapse = ", "),
                                             " Cells"))
        }
    }

    return(plot_obj)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example dataset
data("query_data")

test_that("plotGeneSetScores generates plots correctly", {
    # Test PCA method - returns ggmatrix object
    p1 <- plotGeneSetScores(
        sce_object = query_data,
        cell_type_col = "SingleR_annotation",
        method = "PCA",
        score_col = "gene_set_scores",
        pc_subset = 1:5
    )

    # Check if output is a ggmatrix object (from GGally) for PCA
    expect_s3_class(p1, "ggmatrix")

    # Test UMAP method - should return ggplot object
    # (assuming query_data has UMAP coordinates)
    if ("UMAP" %in% reducedDimNames(query_data)) {
        p2 <- plotGeneSetScores(
            sce_object = query_data,
            cell_type_col = "SingleR_annotation",
            method = "UMAP",
            score_col = "gene_set_scores"
        )
        expect_s3_class(p2, "ggplot")
    }

    # Test TSNE method - should return ggplot object
    # (assuming query_data has TSNE coordinates)
    if ("TSNE" %in% reducedDimNames(query_data)) {
        p3 <- plotGeneSetScores(
            sce_object = query_data,
            cell_type_col = "SingleR_annotation",
            method = "TSNE",
            score_col = "gene_set_scores"
        )
        expect_s3_class(p3, "ggplot")
    }
})

test_that("plotGeneSetScores handles edge cases", {
    # Test with missing/NA values in scores (if applicable)
    # This would depend on how your function handles NA values in the score column

    # Test with single cell type
    available_cell_types <- unique(colData(query_data)[["SingleR_annotation"]])
    if (length(available_cell_types) > 0) {
        p1 <- plotGeneSetScores(
            sce_object = query_data,
            cell_type_col = "SingleR_annotation",
            method = "PCA",
            score_col = "gene_set_scores",
            pc_subset = 1:2,
            cell_types = available_cell_types[1]
        )
        expect_s3_class(p1, "ggmatrix")
    }
})

test_that("plotGeneSetScores validates score column data", {
    # Check that the score column contains numeric data
    # This assumes gene_set_scores column exists and contains numeric values
    scores <- colData(query_data)[["gene_set_scores"]]
    expect_true(is.numeric(scores))

    # Test plot generation works with actual score data
    p1 <- plotGeneSetScores(
        sce_object = query_data,
        cell_type_col = "SingleR_annotation",
        method = "PCA",
        score_col = "gene_set_scores",
        pc_subset = 1:3
    )
    expect_s3_class(p1, "ggmatrix")
})

#' @title Compute Wasserstein Distance Distributions Between Query and Reference Datasets
#'
#' @description
#' This function calculates distributions of Wasserstein distances between reference-reference
#' pairs and reference-query pairs for each specified cell type, after projecting them into
#' a shared PCA space. It then computes the probability of superiority to assess whether
#' reference-query distances tend to be larger than reference-reference distances.
#'
#' @details
#' The function projects the query dataset onto the PCA space defined by the reference dataset.
#' For each cell type, it computes two distributions: (1) Wasserstein distances between randomly
#' sampled pairs within the reference dataset (null distribution), and (2) Wasserstein distances
#' between reference and query dataset samples. It then calculates the probability of superiority,
#' which represents the probability that a randomly selected ref-query distance is larger than
#' a randomly selected ref-ref distance.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing a numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object with a numeric expression matrix for the reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies cell types.
#' @param cell_types A character vector specifying the cell types to include in the analysis. If NULL, all common cell types are included.
#' @param pc_subset A numeric vector specifying which principal components to use. Default is \code{1:5}.
#' @param n_resamples An integer specifying the number of resamples to generate each distribution. Default is \code{300}.
#' @param assay_name The name of the assay to use for computations. Default is \code{"logcounts"}.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#'
#' @return A list with the following components:
#' \item{ref_ref_dist}{A named list of numeric vectors containing Wasserstein distances computed from resampled pairs within the reference dataset for each cell type.}
#' \item{ref_query_dist}{A named list of numeric vectors containing Wasserstein distances between reference and query datasets for each cell type.}
#' \item{probability_superiority}{A named numeric vector showing the probability that ref-query distances are larger than ref-ref distances for each cell type.}
#' \item{cell_types}{A character vector containing the cell types analyzed.}
#'
#' @references
#' Schuhmacher, D., Bernhard, S., & Book, M. (2019). "A Review of Approximate Transport in Machine Learning".
#' In \emph{Journal of Machine Learning Research} (Vol. 20, No. 117, pp. 1-61).
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{plot.calculateWassersteinDistanceObject}}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Compute Wasserstein distance distributions for all cell types
#' wasserstein_data <- calculateWassersteinDistance(query_data = query_data,
#'                                                  reference_data = reference_data,
#'                                                  query_cell_type_col = "expert_annotation",
#'                                                  ref_cell_type_col = "expert_annotation",
#'                                                  pc_subset = 1:5,
#'                                                  n_resamples = 100)
#' plot(wasserstein_data)
#'
#' @importFrom stats quantile
#'
calculateWassersteinDistance <- function(query_data,
                                         reference_data,
                                         ref_cell_type_col,
                                         query_cell_type_col,
                                         cell_types = NULL,
                                         pc_subset = 1:5,
                                         n_resamples = 300,
                                         assay_name = "logcounts",
                                         max_cells_query = 5000,
                                         max_cells_ref = 5000){

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Check if n_resamples is a positive integer
    if (!inherits(n_resamples, "numeric")) {
        stop("\'n_resamples\' should be numeric.")
    } else if (any(!n_resamples == floor(n_resamples), n_resamples < 1)) {
        stop("\'n_resamples\' should be an integer, greater than zero.")
    }

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = TRUE,
                                  n_cell_types = NULL)

    # Get the projected PCA data
    pca_output <- projectPCA(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = query_cell_type_col,
                             ref_cell_type_col = ref_cell_type_col,
                             cell_types = cell_types,
                             pc_subset = pc_subset,
                             assay_name = assay_name,
                             max_cells_ref = max_cells_ref,
                             max_cells_query = max_cells_query)

    # Split by cell type
    cell_list <- split(pca_output, pca_output[["cell_type"]])

    # Extract variance explained for weighting
    weights <- attributes(reducedDim(
        reference_data, "PCA"))[["varExplained"]][pc_subset] /
        sum(attributes(reducedDim(
            reference_data, "PCA"))[["varExplained"]][pc_subset])

    # Initialize results
    ref_ref_dist <- list()
    ref_query_dist <- list()
    probability_superiority <- numeric(length(cell_types))
    names(probability_superiority) <- cell_types

    # Process each cell type
    for (cell_type in cell_types) {

        # Skip if cell type not present in data
        if (!cell_type %in% names(cell_list)) {
            warning(paste("Cell type", cell_type, "not found in data. Skipping."))
            next
        }

        cell_data <- cell_list[[cell_type]]
        ref_indices <- cell_data[["dataset"]] == "Reference"
        query_indices <- cell_data[["dataset"]] == "Query"

        # Check if we have both reference and query cells for this cell type
        if (sum(ref_indices) == 0 || sum(query_indices) == 0) {
            warning(paste("Cell type", cell_type, "missing in reference or query data. Skipping."))
            next
        }

        # Get sample size for Wasserstein distributions
        n_sample <- min(floor(sum(ref_indices)/2), sum(query_indices), 200)

        if (n_sample < 10) {
            warning(paste("Too few cells for cell type", cell_type, ". Skipping."))
            next
        }

        # Extract PCA data for this cell type
        pca_ref <- as.matrix(cell_data[ref_indices, paste0("PC", pc_subset)])
        pca_query <- as.matrix(cell_data[query_indices, paste0("PC", pc_subset)])

        # Apply variance weighting
        pca_ref_weighted <- t(apply(pca_ref, 1,
                                    function(x, weights) return(x * weights),
                                    weights = sqrt(weights)))
        pca_query_weighted <- t(apply(pca_query, 1,
                                      function(x, weights) return(x * weights),
                                      weights = sqrt(weights)))

        # Compute reference-reference weighted distances (full distance matrix)
        weighted_dist_ref <- as.matrix(dist(pca_ref_weighted))

        # Compute reference-query weighted distances (full distance matrix)
        weighted_dist_query <- sqrt(outer(rowSums(pca_ref_weighted^2),
                                          rowSums(pca_query_weighted^2), "+") -
                                        2 * pca_ref_weighted %*% t(pca_query_weighted))

        # Computing reference-reference Wasserstein distance distribution
        ref_ref_distances <- numeric(n_resamples)
        prob_masses <- rep(1/n_sample, n_sample)

        for(iter in seq_len(n_resamples)){
            sample_ref_1 <- sample(seq_len(nrow(pca_ref)), n_sample, replace = FALSE)
            sample_ref_2 <- sample(seq_len(nrow(pca_ref))[-sample_ref_1],
                                   n_sample, replace = FALSE)
            cost_mat <- weighted_dist_ref[sample_ref_1, sample_ref_2]
            opt_plan <- transport::transport(prob_masses, prob_masses,
                                             costm = cost_mat)
            ref_ref_distances[iter] <- transport::wasserstein(prob_masses,
                                                              prob_masses,
                                                              tplan = opt_plan,
                                                              costm = cost_mat)
        }

        # Computing reference-query Wasserstein distance distribution
        ref_query_distances <- numeric(n_resamples)

        for(iter in seq_len(n_resamples)){
            sample_ref <- sample(seq_len(nrow(pca_ref)), n_sample, replace = FALSE)
            sample_query <- sample(seq_len(nrow(pca_query)), n_sample, replace = FALSE)
            cost_mat <- weighted_dist_query[sample_ref, sample_query]
            opt_plan <- transport::transport(prob_masses, prob_masses,
                                             costm = cost_mat)
            ref_query_distances[iter] <- transport::wasserstein(prob_masses,
                                                                prob_masses,
                                                                tplan = opt_plan,
                                                                costm = cost_mat)
        }

        # Store distributions for this cell type
        ref_ref_dist[[cell_type]] <- ref_ref_distances
        ref_query_dist[[cell_type]] <- ref_query_distances

        # Calculate probability of superiority
        # P(ref_query > ref_ref) when sampling one value from each distribution
        n_comparisons <- 0
        n_superiority <- 0

        for (rq_val in ref_query_distances) {
            for (rr_val in ref_ref_distances) {
                n_comparisons <- n_comparisons + 1
                if (rq_val > rr_val) {
                    n_superiority <- n_superiority + 1
                }
            }
        }

        probability_superiority[cell_type] <- n_superiority / n_comparisons
    }

    # Filter out cell types that were skipped
    processed_cell_types <- names(ref_ref_dist)
    probability_superiority <- probability_superiority[processed_cell_types]

    # Return the results
    wasserstein_data <- list(
        ref_ref_dist = ref_ref_dist,
        ref_query_dist = ref_query_dist,
        probability_superiority = probability_superiority,
        cell_types = processed_cell_types
    )
    class(wasserstein_data) <- c(class(wasserstein_data),
                                 "calculateWassersteinDistanceObject")
    return(wasserstein_data)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Extract CD4 cells for subset analysis
ref_data_subset <- reference_data[, which(reference_data$expert_annotation == "CD4")]
query_data_subset <- query_data[, which(query_data$expert_annotation == "CD4")]

# Selecting highly variable genes (can be customized by the user)
ref_top_genes <- scran::getTopHVGs(ref_data_subset, n = 500)
query_top_genes <- scran::getTopHVGs(query_data_subset, n = 500)

# Intersect the gene symbols to obtain common genes
common_genes <- intersect(ref_top_genes, query_top_genes)
ref_data_subset <- ref_data_subset[common_genes, ]
query_data_subset <- query_data_subset[common_genes, ]

# Run PCA on reference data subset
ref_data_subset <- scater::runPCA(ref_data_subset)

# Test to ensure that the function returns a list with expected components
test_that("calculateWassersteinDistance returns correct object structure", {
    result <- calculateWassersteinDistance(
        query_data = query_data_subset,
        reference_data = ref_data_subset,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:5,
        n_resamples = 100
    )

    expect_true(is.list(result))
    expect_named(result, c("ref_ref_dist", "ref_query_dist", "probability_superiority", "cell_types"))
    expect_s3_class(result, "calculateWassersteinDistanceObject")

    # Check components of the result
    expect_type(result$ref_ref_dist, "list")
    expect_type(result$ref_query_dist, "list")
    expect_type(result$probability_superiority, "double")
    expect_type(result$cell_types, "character")
})

# Test to ensure that calculateWassersteinDistance correctly identifies a specific number of unique cell types
test_that("calculateWassersteinDistance finds the right number of cell types", {
    result <- calculateWassersteinDistance(
        query_data = query_data_subset,
        reference_data = ref_data_subset,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:5,
        n_resamples = 100
    )

    expect_equal(length(result$cell_types), length(unique(ref_data_subset$expert_annotation)))
    expect_equal(length(result$probability_superiority), length(result$cell_types))
})

# Test to ensure that the function can run with default parameters
test_that("calculateWassersteinDistance works with default parameters", {
    result <- calculateWassersteinDistance(
        query_data = query_data_subset,
        reference_data = ref_data_subset,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "expert_annotation"
    )

    expect_type(result$ref_ref_dist, "list")
    expect_type(result$ref_query_dist, "list")
    expect_type(result$probability_superiority, "double")
})

# Test parameter validation
test_that("calculateWassersteinDistance validates parameters correctly", {
    # Test invalid n_resamples
    expect_error(calculateWassersteinDistance(
        query_data = query_data_subset,
        reference_data = ref_data_subset,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "expert_annotation",
        n_resamples = -10
    ), "n_resamples.*should be an integer, greater than zero")

    expect_error(calculateWassersteinDistance(
        query_data = query_data_subset,
        reference_data = ref_data_subset,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "expert_annotation",
        n_resamples = "invalid"
    ), "n_resamples.*should be numeric")
})

# Test with specific cell types
test_that("calculateWassersteinDistance works with specific cell types", {
    result <- calculateWassersteinDistance(
        query_data = query_data_subset,
        reference_data = ref_data_subset,
        query_cell_type_col = "expert_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_types = "CD4",
        pc_subset = 1:5,
        n_resamples = 50
    )

    expect_equal(result$cell_types, "CD4")
    expect_equal(length(result$ref_ref_dist), 1)
    expect_equal(length(result$ref_query_dist), 1)
    expect_equal(length(result$probability_superiority), 1)
})

# Load necessary libraries
library(testthat)
library(scDiagnostics)
library(ggplot2)

# Load example datasets and prepare data as before
data("reference_data")
data("query_data")

# Generate Wasserstein distance data
wasserstein_data <- calculateWassersteinDistance(
    query_data = query_data,
    reference_data = reference_data,
    query_cell_type_col = "expert_annotation",
    ref_cell_type_col = "expert_annotation",
    pc_subset = 1:5,
    n_resamples = 100
)

test_that("plot.calculateWassersteinDistanceObject returns a ggplot object", {
    plot_obj <- plot(wasserstein_data)
    expect_s3_class(plot_obj, "ggplot")
})

test_that("plot creates ridge plots with correct structure", {
    plot_obj <- plot(wasserstein_data)

    # Check that it has ridge layers
    expect_true(length(plot_obj$layers) > 0)

    # Check correct labels
    expect_equal(plot_obj$labels$title, "Comparison of Wasserstein Distance Distributions by Cell Type")
    expect_equal(plot_obj$labels$x, "Wasserstein Distance")
})

test_that("plot works with specific cell types", {
    available_cell_types <- wasserstein_data$cell_types[1:min(2, length(wasserstein_data$cell_types))]
    plot_obj <- plot(wasserstein_data, cell_types = available_cell_types)
    expect_s3_class(plot_obj, "ggplot")
})

#' @title Downsample SingleCellExperiment Objects
#'
#' @description
#' This internal function downsamples \code{\linkS4class{SingleCellExperiment}} objects while preserving
#' reducedDims coordinate information (PCA, UMAP, t-SNE, etc.). Optionally, it can also subset by cell types
#' before downsampling.
#'
#' @details
#' The function can perform cell type filtering followed by random downsampling without replacement
#' when the number of cells exceeds the specified threshold. All reducedDims coordinates are preserved
#' through standard sce_object subsetting operations. This function does not preserve PCA rotation matrices or
#' other model-specific attributes.
#'
#' @param sce_object A \code{\linkS4class{SingleCellExperiment}} object to potentially downsample.
#' May contain PCA, UMAP, TSNE, or other reducedDims.
#' @param cell_type_col The column name in colData that contains cell type information.
#' Required if cell_types is not NULL. Default is NULL.
#' @param cell_types A character vector specifying which cell types to retain. If NULL,
#' no cell type filtering is performed. Default is NULL.
#' @param max_cells Maximum number of cells to retain. If the object has fewer cells,
#' it is returned unchanged. If NULL, no downsampling is performed (all cells are kept).
#' Default is 2500.
#' @param seed Random seed for reproducible downsampling. If NULL, no seed is set.
#' Default is NULL.
#'
#' @keywords internal
#'
#' @return A \code{\linkS4class{SingleCellExperiment}} object with at most max_cells cells
#' and optionally filtered by cell types. ReducedDims coordinates are preserved through
#' standard subsetting. If max_cells is NULL, the original object is returned unchanged.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
# Function to downsample sce_object objects
downsampleSCE <- function(sce_object,
                          cell_type_col = NULL,
                          cell_types = NULL,
                          max_cells = 2500,
                          seed = NULL) {

    # Check standard input arguments
    argumentCheck(query_data = sce_object,
                  query_cell_type_col = cell_type_col,
                  max_cells_query = max_cells)

    # Convert cell type columns to character if needed
    sce_object <- convertColumnsToCharacter(sce_object = sce_object,
                                            convert_cols = cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = sce_object,
                                  query_cell_type_col = cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = NULL)

    # Check seed argument
    if (!is.null(seed)) {
        if (!is.numeric(seed) || length(seed) != 1 || seed != as.integer(seed)) {
            stop("'seed' must be a single integer or NULL.")
        }
        seed <- as.integer(seed)
    }

    # Set seed if provided
    if (!is.null(seed)) {
        set.seed(seed)
    }

    # Subset by cell types if requested
    if (!is.null(cell_types)) {
        cell_type_mask <- sce_object[[cell_type_col]] %in% cell_types
        if (sum(cell_type_mask) == 0) {
            warning("No cells found for the specified cell types. Returning empty sce_object object.")
            return(sce_object[, FALSE])  # Return empty sce_object with same genes but no cells
        }
        sce_object <- sce_object[, cell_type_mask]
    }

    # Check if downsampling is needed
    n_cells <- ncol(sce_object)

    # If max_cells is NULL, return object without downsampling (but potentially cell-type filtered)
    if (is.null(max_cells)) {
        return(sce_object)
    }

    # If we already have fewer cells than max_cells, return as is
    if (n_cells <= max_cells) {
        return(sce_object)
    }

    # Step 3: Downsample cells
    cell_indices <- sample(n_cells, max_cells)
    sce_object_subset <- sce_object[, cell_indices]

    # Return the subsetted object (reducedDims coordinates are automatically preserved)
    return(sce_object_subset)
}

#' @title Create Faceted Scatter Plots for Specified PC Combinations From \code{detectAnomaly} Object
#'
#' @description
#' This S3 plot method generates faceted scatter plots for specified principal component (PC) combinations
#' within an anomaly detection object. It visualizes the relationship between specified PCs, highlights
#' anomalies detected by the Isolation Forest algorithm, and provides a background gradient representing
#' anomaly scores.
#'
#' @details
#' The function extracts the specified PCs from the given anomaly detection object and generates
#' scatter plots for each pair of PCs. It uses \code{GGally} to create a pairs plot where each facet
#' represents a pair of PCs. The plot includes:
#'
#' 1. Lower facets: Scatter plots with a background gradient representing anomaly scores from green (low) to red (high)
#' 2. Diagonal facets: Density, ridge, boxplot or blank visualizations showing the distribution of each PC, separated by anomaly status
#' 3. Upper facets: Blank panels by default, or contour/ellipse plots separated by anomaly status if specified
#'
#' @param x A list object containing the anomaly detection results from the \code{detectAnomaly} function.
#' Each element of the list should correspond to a cell type and contain \code{reference_mat_subset}, \code{query_mat_subset},
#' \code{var_explained}, and \code{anomaly}.
#' @param cell_type A character string specifying the cell type for which the plots should be generated. This should
#' be a name present in \code{x}. If NULL, the "Combined" cell type will be plotted. Default is NULL.
#' @param pc_subset A numeric vector specifying the indices of the PCs to be included in the plots. If NULL, all PCs
#' in \code{reference_mat_subset} will be included.
#' @param data_type A character string specifying whether to plot the "query" data or the "reference" data. Default is "query".
#' @param n_tree An integer specifying the number of trees for the isolation forest. Default is 500
#' @param upper_facet Either "blank" (default), "contour", or "ellipse" for the upper facet plots.
#' @param diagonal_facet Either "density" (default), "ridge", "boxplot" or "blank" for the diagonal plots.
#' @param max_cells_ref Maximum number of reference cells to include in the plot. If NULL,
#' all available reference cells are plotted. Default is NULL.
#' @param max_cells_query Maximum number of query cells to include in the plot. If NULL,
#' all available query cells are plotted. Default is NULL.
#' @param ... Additional arguments passed to the `isolation.forest` function.
#'
#' @return The S3 plot method returns a \code{GGally::ggpairs} object representing the PCA plots with anomalies highlighted.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{detectAnomaly}}
#'
#' @rdname detectAnomaly
#'
# Function to plot isolation forest anomaly regions
plot.detectAnomalyObject <- function(x,
                                     cell_type = NULL,
                                     pc_subset = NULL,
                                     data_type = c("query", "reference"),
                                     n_tree = 500,
                                     upper_facet = c("blank", "contour", "ellipse"),
                                     diagonal_facet = c("density", "ridge", "boxplot", "blank"),
                                     max_cells_ref = NULL,
                                     max_cells_query = NULL,
                                     ...) {

    # Check if PCA was used for computations
    if(!("var_explained" %in% names(x[[names(x)[1]]])))
        stop("The plot function can only be used if \'pc_subset\' is not NULL when using \'detectAnomaly\'.")

    # Check input for upper_facet and diagonal_facet
    upper_facet <- match.arg(upper_facet)
    diagonal_facet <- match.arg(diagonal_facet)

    # Check input for cell type
    if(is.null(cell_type)){
        cell_type <- "Combined"
    } else{
        if(!(cell_type %in% names(x)))
            stop("'cell_type' is not available in 'x'.")
    }

    # Check input for pc_subset
    if(!is.null(pc_subset)){
        if(!all(pc_subset %in% seq_len(
            length(x[[cell_type]][["var_explained"]]))))
            stop("'pc_subset' is out of range.")
    } else{
        pc_subset <- seq_len(length(x[[cell_type]][["var_explained"]]))
    }

    # Check input for data_type
    data_type <- match.arg(data_type)

    # Validate max_cells parameters
    if (!is.null(max_cells_ref)) {
        if (!is.numeric(max_cells_ref) || max_cells_ref <= 0 || max_cells_ref != as.integer(max_cells_ref)) {
            stop("'max_cells_ref' must be a positive integer.")
        }
    }

    if (!is.null(max_cells_query)) {
        if (!is.numeric(max_cells_query) || max_cells_query <= 0 || max_cells_query != as.integer(max_cells_query)) {
            stop("'max_cells_query' must be a positive integer.")
        }
    }

    # Filter and prepare data based on data type
    if(is.null(x[[cell_type]][["query_mat_subset"]]) && data_type == "query"){
        stop("There is no query data available in the 'detectAnomaly' object.")
    } else{
        if(data_type == "query"){
            data_subset <- x[[cell_type]][["query_mat_subset"]][, pc_subset,
                                                                drop = FALSE]
            anomaly <- x[[cell_type]][["query_anomaly"]]

            # Downsample query data if max_cells_query is specified
            if(!is.null(max_cells_query) && nrow(data_subset) > max_cells_query){
                sampled_indices <- sample(nrow(data_subset), max_cells_query)
                data_subset <- data_subset[sampled_indices, , drop = FALSE]
                anomaly <- anomaly[sampled_indices]
            }

        } else if(data_type == "reference"){
            data_subset <- x[[cell_type]][["reference_mat_subset"]][, pc_subset,
                                                                    drop = FALSE]
            anomaly <- x[[cell_type]][["reference_anomaly"]]

            # Downsample reference data if max_cells_ref is specified
            if(!is.null(max_cells_ref) && nrow(data_subset) > max_cells_ref){
                sampled_indices <- sample(nrow(data_subset), max_cells_ref)
                data_subset <- data_subset[sampled_indices, , drop = FALSE]
                anomaly <- anomaly[sampled_indices]
            }
        }
    }

    # Add variance explained to column names
    colnames(data_subset) <- paste0(
        "PC", pc_subset,
        " (", sprintf("%.1f%%",
                      x[[cell_type]][["var_explained"]][pc_subset]), ")")

    # Create a data frame with PC values and anomaly info
    pc_df <- data.frame(data_subset)
    colnames(pc_df) <- colnames(data_subset)
    pc_df[["anomaly"]] <- factor(anomaly)

    # Colors for anomalous and non-anomalous data
    anomaly_colors <- c("TRUE" = "red", "FALSE" = "black")
    anomaly_fill_colors <- c("TRUE" = "#FFB6B6", "FALSE" = "#DDDDDD")  # Light red and light gray

    # Train isolation forest for each PC combination
    isolation_forests <- list()
    for(i in seq_along(pc_subset)){
        for(j in seq_along(pc_subset)){
            if(i < j){
                pc_i <- paste0("PC", pc_subset[i])
                pc_j <- paste0("PC", pc_subset[j])

                if(data_type == "reference"){
                    train_data <-
                        x[[cell_type]][["reference_mat_subset"]][, c(pc_i, pc_j)]
                } else {
                    train_data <-
                        x[[cell_type]][["reference_mat_subset"]][, c(pc_i, pc_j)]
                }

                isolation_forests[[paste(pc_i, pc_j, sep="-")]] <-
                    isotree::isolation.forest(train_data, ntree = n_tree, ...)
            }
        }
    }

    # [All the plotting helper functions remain the same - they're unchanged from the original]
    # Function to create scatter plot with isolation forest background
    .anomalyScatterFunc <- function(data, mapping, ...){
        x_name <- rlang::as_name(mapping[["x"]])
        y_name <- rlang::as_name(mapping[["y"]])

        # Get PC numbers from column names
        pc_x <- as.numeric(sub("PC([0-9]+).*", "\\1", x_name))
        pc_y <- as.numeric(sub("PC([0-9]+).*", "\\1", y_name))

        # Get isolation forest for this PC combination
        if_key <- paste0("PC", min(pc_x, pc_y), "-PC", max(pc_x, pc_y))
        if_model <- isolation_forests[[if_key]]

        # Get ranges for grid
        x_range <- range(data[[x_name]])
        y_range <- range(data[[y_name]])
        x_buffer <- 0.1 * diff(x_range)
        y_buffer <- 0.1 * diff(y_range)

        # Create grid for background
        x_seq <- seq(x_range[1] - x_buffer, x_range[2] + x_buffer,
                     length.out = 150)
        y_seq <- seq(y_range[1] - y_buffer, y_range[2] + y_buffer,
                     length.out = 150)
        grid <- expand.grid(x = x_seq, y = y_seq)

        # Calculate step sizes for better tile width/height
        x_step <- diff(x_seq)[1]
        y_step <- diff(y_seq)[1]

        # Get anomaly scores for grid
        grid_data <- data.frame(grid)
        colnames(grid_data) <- c(paste0("PC", pc_x),
                                 paste0("PC", pc_y))

        scores <- predict(if_model, newdata = grid_data,
                          type = "score")

        # Create background data for plotting
        background_data <- data.frame(
            x_value = grid[["x"]],
            y_value = grid[["y"]],
            anomaly_score = scores,
            width = x_step,
            height = y_step
        )

        # Create gradient colors
        gradient_colors <- c(
            "#2CA25F",
            "#FFFFE5",
            "#FF8585"
        )
        stops <- c(0, 0.55, 1)

        # Create plot with background and points
        p <- ggplot2::ggplot() +
            ggplot2::geom_tile(
                data = background_data,
                ggplot2::aes(
                    x = .data[["x_value"]],
                    y = .data[["y_value"]],
                    fill = .data[["anomaly_score"]],
                    width = .data[["width"]],
                    height = .data[["height"]]
                )
            ) +
            ggplot2::scale_fill_gradientn(
                colors = gradient_colors,
                values = scales::rescale(stops),
                limits = c(0, 1),
                guide = "none"
            ) +
            ggplot2::geom_point(
                data = data,
                ggplot2::aes(
                    x = !!rlang::sym(x_name),
                    y = !!rlang::sym(y_name),
                    color = anomaly
                ),
                shape = 16,
                size = 1.5,
                alpha = 0.45
            ) +
            ggplot2::scale_color_manual(
                values = anomaly_colors,
                name = "Anomalous"
            ) +
            ggplot2::scale_x_continuous(expand = c(0, 0)) +
            ggplot2::scale_y_continuous(expand = c(0, 0)) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(color = "black",
                                                     fill = NA,
                                                     linewidth = 0.5),
                panel.grid = ggplot2::element_blank(),
                legend.position = "none"
            )

        return(p)
    }

    # Function for diagonal density plots with separate anomaly curves
    .densityDiagFunc <- function(data, mapping, ...){
        x_name <- rlang::as_name(mapping[["x"]])

        # Create separate density curves for anomalous and non-anomalous data
        ggplot2::ggplot(data, ggplot2::aes(x = !!rlang::sym(x_name))) +
            ggplot2::geom_density(
                ggplot2::aes(fill = anomaly, color = anomaly),
                alpha = 0.3,  # Semi-transparent fill
                linewidth = 0.8
            ) +
            ggplot2::scale_color_manual(values = anomaly_colors) +
            ggplot2::scale_fill_manual(values = anomaly_fill_colors) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(color = "black",
                                                     fill = NA,
                                                     linewidth = 0.5),
                panel.grid = ggplot2::element_blank(),
                axis.title.y = ggplot2::element_blank(),
                axis.text.y = ggplot2::element_blank(),
                axis.ticks.y = ggplot2::element_blank(),
                legend.position = "none"
            )
    }

    # Function for diagonal ridge plots with separate anomaly distributions
    .ridgeDiagFunc <- function(data, mapping, ...){
        x_name <- rlang::as_name(mapping[["x"]])

        # Create a long-format data frame for ridge plot
        plot_data <- data.frame(
            value = data[[x_name]],
            anomaly = data[["anomaly"]]
        )

        # Create ridge plot with ggridges
        suppressMessages({
            p <- ggplot2::ggplot(plot_data,
                                 ggplot2::aes(x = .data[["value"]],
                                              y = .data[["anomaly"]],
                                              fill = .data[["anomaly"]],
                                              color = .data[["anomaly"]])) +
                ggridges::geom_density_ridges(
                    alpha = 0.6,
                    scale = 1.8,
                    rel_min_height = 0.01,
                    quantile_lines = FALSE
                ) +
                ggplot2::scale_fill_manual(values = anomaly_fill_colors) +
                ggplot2::scale_color_manual(values = anomaly_colors) +
                ggplot2::scale_y_discrete(limits = rev(levels(data[["anomaly"]]))) +
                ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(color = "black",
                                                         fill = NA,
                                                         linewidth = 0.5),
                    axis.title = ggplot2::element_blank(),
                    axis.text.y = ggplot2::element_blank(),
                    axis.ticks.y = ggplot2::element_blank(),
                    legend.position = "none",
                    panel.grid = ggplot2::element_blank()
                )
        })

        return(p)
    }

    # Function for diagonal boxplot with separate anomaly distributions
    .boxplotDiagFunc <- function(data, mapping, ...){
        x_name <- rlang::as_name(mapping[["x"]])

        # Create a long-format data frame for boxplot
        plot_data <- data.frame(
            value = data[[x_name]],
            anomaly = data[["anomaly"]]
        )

        # Create horizontal boxplot
        ggplot2::ggplot(plot_data,
                        ggplot2::aes(x = .data[["value"]],
                                     y = .data[["anomaly"]],
                                     fill = .data[["anomaly"]])) +
            ggplot2::geom_boxplot(
                alpha = 0.7,
                outlier.size = 0.5,
                width = 0.6,
                color = "black"
            ) +
            ggplot2::scale_fill_manual(values = anomaly_fill_colors) +
            ggplot2::scale_y_discrete(limits = rev(levels(data[["anomaly"]]))) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(color = "black",
                                                     fill = NA,
                                                     linewidth = 0.5),
                panel.grid = ggplot2::element_blank(),
                axis.title = ggplot2::element_blank(),
                axis.text.y = ggplot2::element_blank(),
                axis.ticks.y = ggplot2::element_blank(),
                legend.position = "none"
            )
    }

    # Function for blank panels
    .blankFunc <- function(data, mapping, ...){
        ggplot2::ggplot() +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(color = "black",
                                                     fill = NA,
                                                     linewidth = 0.5),
                panel.grid = ggplot2::element_blank(),
                axis.title = ggplot2::element_blank(),
                axis.text = ggplot2::element_blank(),
                axis.ticks = ggplot2::element_blank()
            )
    }

    # Function for contour plots separated by anomaly status
    .contourFunc <- function(data, mapping, ...){
        x_name <- rlang::as_name(mapping[["x"]])
        y_name <- rlang::as_name(mapping[["y"]])

        # Start with empty plot
        p <- ggplot2::ggplot(data, ggplot2::aes(
            x = !!rlang::sym(x_name),
            y = !!rlang::sym(y_name))) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(color = "black",
                                                     fill = NA,
                                                     linewidth = 0.5),
                panel.grid = ggplot2::element_blank(),
                axis.title = ggplot2::element_blank(),
                axis.text = ggplot2::element_blank(),
                axis.ticks = ggplot2::element_blank(),
                legend.position = "none"
            )

        # Add contours for non-anomalous data (if there are enough points)
        data_normal <- data[data[["anomaly"]] == "FALSE",]
        if(nrow(data_normal) >= 10) {
            p <- p + ggplot2::stat_density_2d(
                data = data_normal,
                ggplot2::aes(
                    x = !!rlang::sym(x_name),
                    y = !!rlang::sym(y_name)
                ),
                color = anomaly_colors[["FALSE"]],
                linewidth = 0.5,
                contour = TRUE,
                bins = 5
            )
        }

        # Add contours for anomalous data (if there are enough points)
        data_anomaly <- data[data[["anomaly"]] == "TRUE",]
        if(nrow(data_anomaly) >= 10) {
            p <- p + ggplot2::stat_density_2d(
                data = data_anomaly,
                ggplot2::aes(
                    x = !!rlang::sym(x_name),
                    y = !!rlang::sym(y_name)
                ),
                color = anomaly_colors[["TRUE"]],
                linewidth = 0.5,
                contour = TRUE,
                bins = 5
            )
        }

        return(p)
    }

    # Function for ellipse plots separated by anomaly status
    .ellipseFunc <- function(data, mapping, ...){
        x_name <- rlang::as_name(mapping[["x"]])
        y_name <- rlang::as_name(mapping[["y"]])

        # Function to calculate and create ellipses
        createEllipse <- function(d) {
            if (nrow(d) < 10) return(NULL)

            # Extract variables
            x <- d[[x_name]]
            y <- d[[y_name]]

            # Calculate robust center and covariance to handle outliers well
            cov_mat <- cov(cbind(x, y), use = "pairwise.complete.obs")
            center <- c(mean(x, na.rm = TRUE), mean(y, na.rm = TRUE))

            # Use robust covariance estimation to minimize outlier impact
            ellipse <- MASS::cov.trob(cbind(x, y))

            # Get ellipse coordinates (95% confidence interval = 2.45 sigma)
            ev <- eigen(ellipse[["cov"]])
            a <- sqrt(ev[["values"]][1]) * 2.45  # major axis
            b <- sqrt(ev[["values"]][2]) * 2.45  # minor axis

            # Calculate rotation angle
            angle <- atan2(ev[["vectors"]][2,1], ev[["values"]][1,1])

            # Create ellipse coordinates
            theta <- seq(0, 2 * pi, length.out = 100)
            ellipse_x <- center[1] + a * cos(theta) * cos(angle) -
                b * sin(theta) * sin(angle)
            ellipse_y <- center[2] + a * cos(theta) * sin(angle) +
                b * sin(theta) * cos(angle)

            return(data.frame(x = ellipse_x, y = ellipse_y))
        }

        # Start with empty plot
        p <- ggplot2::ggplot() +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(color = "black",
                                                     fill = NA,
                                                     linewidth = 0.5),
                panel.grid = ggplot2::element_blank(),
                axis.title = ggplot2::element_blank(),
                axis.text = ggplot2::element_blank(),
                axis.ticks = ggplot2::element_blank(),
                legend.position = "none"
            )

        # Create ellipses for normal data
        data_normal <- data[data[["anomaly"]] == "FALSE",]
        ellipse_normal <- createEllipse(data_normal)
        if (!is.null(ellipse_normal)) {
            p <- p + ggplot2::geom_path(
                data = ellipse_normal,
                ggplot2::aes(x = .data[["x"]], y = .data[["y"]]),
                color = anomaly_colors[["FALSE"]],
                linewidth = 0.7
            )
        }

        # Create ellipses for anomalous data
        data_anomaly <- data[data[["anomaly"]] == "TRUE",]
        ellipse_anomaly <- createEllipse(data_anomaly)
        if (!is.null(ellipse_anomaly)) {
            p <- p + ggplot2::geom_path(
                data = ellipse_anomaly,
                ggplot2::aes(x = .data[["x"]], y = .data[["y"]]),
                color = anomaly_colors[["TRUE"]],
                linewidth = 0.7
            )
        }

        return(p)
    }

    # Select diagonal facet type based on input
    if (diagonal_facet == "density") {
        diag_func <- .densityDiagFunc
    } else if (diagonal_facet == "ridge") {
        diag_func <- .ridgeDiagFunc
    } else if (diagonal_facet == "boxplot") {
        diag_func <- .boxplotDiagFunc
    }  else if (diagonal_facet == "blank") {
        diag_func <- .blankFunc
    }

    # Select upper facet type based on input
    if (upper_facet == "blank") {
        upper_func <- .blankFunc
    } else if (upper_facet == "contour") {
        upper_func <- .contourFunc
    } else if (upper_facet == "ellipse") {
        upper_func <- .ellipseFunc
    }

    # Create a simple plot to extract the legend
    legend_plot <- ggplot2::ggplot(pc_df, ggplot2::aes(x = pc_df[,1],
                                                       y = pc_df[,2])) +
        ggplot2::geom_point(ggplot2::aes(color = anomaly)) +
        ggplot2::scale_color_manual(values = anomaly_colors,
                                    name = "Anomalous") +
        ggplot2::theme(
            legend.position = "right",
            legend.box = "vertical",
            legend.key = ggplot2::element_rect(fill = "white")
        )

    # Create pairs plot using GGally
    plot_obj <- suppressMessages(
        GGally::ggpairs(
            pc_df,
            columns = seq_len(length(pc_subset)),
            mapping = ggplot2::aes(color = anomaly),
            lower = list(continuous = .anomalyScatterFunc),
            diag = list(continuous = diag_func),
            upper = list(continuous = upper_func),
            progress = FALSE,
            legend = GGally::grab_legend(legend_plot)
        )
    )

    # Add black frame around facet titles and center the title
    plot_obj <- plot_obj +
        ggplot2::theme(
            strip.background = ggplot2::element_rect(fill = "white",
                                                     color = "black",
                                                     linewidth = 0.5),
            strip.text = ggplot2::element_text(color = "black"),
            plot.title = ggplot2::element_text(hjust = 0.5)
        ) +
        ggplot2::ggtitle(paste0("Isolation Forest Anomaly Plot: ",
                                cell_type))

    return(plot_obj)
}

#' @title Plot Projected Data on Unified Discriminant Space
#'
#' @description
#' The S3 plot method visualizes the projected reference and query data on the unified discriminant space.
#'
#' @details
#' The S3 plot method generates a pairs plot visualization of discriminant vectors, similar to PCA plot visualization.
#' Each panel shows the relationship between two discriminant vectors with customizable display options for lower,
#' diagonal, and upper panels. The visualization allows for comprehensive examination of the discriminant space
#' structure and cell type separability.
#'
#' @param x An object of class \code{calculateDiscriminantSpaceObject} containing the projected data on the discriminant space.
#' @param cell_types A character vector specifying the cell types to plot. If NULL (default), all cell types will be plotted.
#' @param dv_subset A numeric vector specifying which discriminant vectors to include in the plot.
#'                 Default is the number of cell types minus 1.
#' @param lower_facet Type of plot to use for the lower panels. Either "scatter" (default),
#'                   "contour", "ellipse", or "blank".
#' @param diagonal_facet Type of plot to use for the diagonal panels. Either "ridge" (default),
#'                      "density", "boxplot" or "blank".
#' @param upper_facet Type of plot to use for the upper panels. Either "blank" (default),
#'                   "scatter", "contour", or "ellipse".
#' @param max_cells_ref Maximum number of reference cells to include in the plot. If NULL,
#' all available reference cells are plotted. Default is NULL.
#' @param max_cells_query Maximum number of query cells to include in the plot. If NULL,
#' all available query cells are plotted. Default is NULL.
#' @param ... Additional arguments to be passed to the plotting functions.
#'
#' @return The S3 plot method returns a \code{GGally::ggpairs} object representing
#'         the visualization of the projected discriminant space.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{calculateDiscriminantSpace}}
#'
#' @rdname calculateDiscriminantSpace
#'
# Function to plot data projected onto discriminant space
plot.calculateDiscriminantSpaceObject <- function(
        x,
        cell_types = NULL,
        dv_subset = NULL,
        lower_facet = c("scatter", "contour", "ellipse", "blank"),
        diagonal_facet = c("ridge", "density", "boxplot", "blank"),
        upper_facet = c("blank", "scatter", "contour", "ellipse"),
        max_cells_ref = NULL,
        max_cells_query = NULL,
        ...){

    # Check if query data is available in the object
    if(!("query_proj" %in% names(x)))
        stop("There is no query data to plot.")

    # Match facet arguments
    lower_facet <- match.arg(lower_facet)
    diagonal_facet <- match.arg(diagonal_facet)
    upper_facet <- match.arg(upper_facet)

    # Extract all cell types if not specified
    if(is.null(cell_types)){
        cell_types <- unique(c(x[["ref_proj"]][["cell_type"]],
                               x[["query_proj"]][["cell_type"]]))
    }

    # Filter data to include only requested cell types
    ref_data <- x[["ref_proj"]][x[["ref_proj"]][["cell_type"]] %in%
                                    cell_types, ]
    query_data <- x[["query_proj"]][x[["query_proj"]][["cell_type"]] %in%
                                        cell_types, ]

    # Downsample reference data if max_cells_ref is specified
    if(!is.null(max_cells_ref)){
        # Input validation for max_cells_ref
        if (!is.numeric(max_cells_ref) || max_cells_ref <= 0 || max_cells_ref != as.integer(max_cells_ref)) {
            stop("'max_cells_ref' must be a positive integer.")
        }

        if(nrow(ref_data) > max_cells_ref){
            # Stratified sampling by cell type
            ref_data_list <- list()
            for(ct in cell_types){
                ct_data <- ref_data[ref_data[["cell_type"]] == ct, ]
                if(nrow(ct_data) > 0){
                    # Calculate proportional allocation
                    n_cells_ct <- min(nrow(ct_data),
                                      max(1, round(max_cells_ref * nrow(ct_data) / nrow(ref_data))))
                    if(nrow(ct_data) > n_cells_ct){
                        sampled_indices <- sample(nrow(ct_data), n_cells_ct)
                        ct_data <- ct_data[sampled_indices, ]
                    }
                    ref_data_list[[ct]] <- ct_data
                }
            }
            ref_data <- do.call(rbind, ref_data_list)
            rownames(ref_data) <- NULL
        }
    }

    # Downsample query data if max_cells_query is specified
    if(!is.null(max_cells_query)){
        # Input validation for max_cells_query
        if (!is.numeric(max_cells_query) || max_cells_query <= 0 || max_cells_query != as.integer(max_cells_query)) {
            stop("'max_cells_query' must be a positive integer.")
        }

        if(nrow(query_data) > max_cells_query){
            # Stratified sampling by cell type
            query_data_list <- list()
            for(ct in cell_types){
                ct_data <- query_data[query_data[["cell_type"]] == ct, ]
                if(nrow(ct_data) > 0){
                    # Calculate proportional allocation
                    n_cells_ct <- min(nrow(ct_data),
                                      max(1, round(max_cells_query * nrow(ct_data) / nrow(query_data))))
                    if(nrow(ct_data) > n_cells_ct){
                        sampled_indices <- sample(nrow(ct_data), n_cells_ct)
                        ct_data <- ct_data[sampled_indices, ]
                    }
                    query_data_list[[ct]] <- ct_data
                }
            }
            query_data <- do.call(rbind, query_data_list)
            rownames(query_data) <- NULL
        }
    }

    # Get discriminant vector columns
    dv_cols <- grep("^DV", colnames(ref_data), value = TRUE)
    total_dvs <- length(dv_cols)

    # Handle dv_subset parameter
    if(is.null(dv_subset)) {
        # Use all available discriminant vectors if dv_subset is NULL
        dv_subset <- seq_len(total_dvs)
    } else {
        # Check if specified dv_subset is valid
        if(max(dv_subset) > total_dvs) {
            stop(sprintf("Invalid \"dv_subset\".",
                         total_dvs))
        }
        if(min(dv_subset) < 1) {
            stop("Invalid dv_subset. Indices must be positive integers.")
        }
    }

    # Filter to the specified subset of discriminant vectors
    dv_cols <- dv_cols[dv_subset]

    # Check that we have at least one discriminant vector
    if(length(dv_cols) == 0) {
        stop("No valid discriminant vectors specified.")
    }

    # Combine reference and query data
    plot_data <- rbind(
        data.frame(ref_data, dataset = "Reference"),
        data.frame(query_data, dataset = "Query")
    )

    # Get the discriminant vector names for the plot
    plot_names <- paste0("DV", dv_subset)

    # Create a new data frame with selected DVs
    dv_df <- data.frame(matrix(0, nrow = nrow(plot_data),
                               ncol = length(dv_subset)))
    colnames(dv_df) <- plot_names

    for (i in 1:length(dv_subset)) {
        dv_df[, i] <- plot_data[, paste0("DV", dv_subset[i])]
    }

    # Create a cell type dataset column for coloring
    cell_type_dataset <- paste(plot_data[["dataset"]],
                               plot_data[["cell_type"]],
                               sep = " ")

    # Define the order of cell type and dataset combinations
    order_combinations <- paste(
        rep(c("Reference", "Query"), length(cell_types)),
        rep(sort(cell_types), each = 2))

    cell_type_dataset <- factor(cell_type_dataset, levels = order_combinations)

    # Generate colors for cell types and datasets
    cell_type_colors <- generateColors(order_combinations, paired = TRUE)

    # Add the cell type dataset column to the DV data frame
    dv_df[["cell_type_dataset"]] <- cell_type_dataset

    # Create a simple plot to extract the legend using GGally::grab_legend
    legend_plot <- ggplot2::ggplot(dv_df,
                                   ggplot2::aes(x = dv_df[,1],
                                                y = dv_df[,2])) +
        ggplot2::geom_point(ggplot2::aes(color = cell_type_dataset)) +
        ggplot2::scale_color_manual(values = cell_type_colors,
                                    name = "Cell Type") +
        ggplot2::theme(
            legend.position = "right",
            legend.box = "vertical",
            legend.key = ggplot2::element_rect(fill = "white"),
            legend.background = ggplot2::element_blank()
        ) +
        ggplot2::guides(color = ggplot2::guide_legend(ncol = 1))

    # Scatterplot facet function
    .scatterFunc <- function(data, mapping, ...) {
        ggplot2::ggplot(data = data, mapping = mapping) +
            ggplot2::geom_point(alpha = 0.5, size = 1,
                                ggplot2::aes(color = cell_type_dataset)) +
            ggplot2::scale_color_manual(values = cell_type_colors) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(
                    color = "black",
                    fill = NA,
                    linewidth = 0.5))
    }

    # Contour facet function
    .smoothContourFunc <- function(data, mapping, ...) {
        x_name <- rlang::as_name(mapping[["x"]])
        y_name <- rlang::as_name(mapping[["y"]])

        # Start with empty plot
        p <- ggplot2::ggplot() +
            ggplot2::theme_minimal()

        # Process each cell type separately
        for (ct in unique(data[["cell_type_dataset"]])) {
            subset_data <- data[data[["cell_type_dataset"]] == ct, ]

            # Skip if too few points
            if (nrow(subset_data) < 10) next

            # Use larger adjustment factor for smoother contours
            adjust_factor <- 1.5

            # Add smoother contours with fewer levels
            p <- p + ggplot2::stat_density_2d(
                data = subset_data,
                mapping = ggplot2::aes(
                    x = !!rlang::sym(x_name),
                    y = !!rlang::sym(y_name)
                ),
                contour = TRUE,
                adjust = adjust_factor,
                bins = 5,
                color = cell_type_colors[which(
                    levels(data[["cell_type_dataset"]]) == ct)],
                linewidth = 0.5,
                na.rm = TRUE
            )
        }

        # Apply theme elements
        p + ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(
                    color = "black", fill = NA, linewidth = 0.5),
                legend.position = "none",
                axis.text = ggplot2::element_blank(),
                axis.ticks = ggplot2::element_blank(),
                axis.title = ggplot2::element_blank()
            )
    }

    # [Continue with all other helper functions...]
    # Ellipse facet function
    .robustEllipseFunc <- function(data, mapping, ...) {
        # Function to calculate robust ellipses through bootstrapping
        createEllipse <- function(d) {
            if (nrow(d) < 10) return(NULL)

            x_var <- rlang::as_name(mapping[["x"]])
            y_var <- rlang::as_name(mapping[["y"]])

            # Extract variables
            x <- d[[x_var]]
            y <- d[[y_var]]

            # Calculate robust center and covariance
            cov_mat <- cov(cbind(x, y), use = "pairwise.complete.obs")
            center <- c(mean(x, na.rm = TRUE), mean(y, na.rm = TRUE))

            # Calculate ellipse points
            theta <- seq(0, 2 * pi, length.out = 100)
            ellipse <- MASS::cov.trob(cbind(x, y))

            # Get ellipse coordinates
            ev <- eigen(ellipse[["cov"]])
            a <- sqrt(ev[["values"]][1]) * 2.45
            b <- sqrt(ev[["values"]][2]) * 2.45

            # Create ellipse coordinates
            angle <- atan2(ev[["vectors"]][2,1], ev[["vectors"]][1,1])
            ellipse_x <- center[1] + a * cos(theta) * cos(angle) -
                b * sin(theta) * sin(angle)
            ellipse_y <- center[2] + a * cos(theta) * sin(angle) +
                b * sin(theta) * cos(angle)

            return(data.frame(x = ellipse_x, y = ellipse_y))
        }

        p <- ggplot2::ggplot(data, mapping)

        # Split by cell type and create robust ellipses
        for (ct in unique(data[["cell_type_dataset"]])) {
            subset_data <- data[data[["cell_type_dataset"]] == ct,]
            ellipse_data <- createEllipse(subset_data)

            if (!is.null(ellipse_data)) {
                p <- p + ggplot2::geom_path(
                    data = ellipse_data,
                    ggplot2::aes(x = .data[["x"]], y = .data[["y"]]),
                    color = cell_type_colors[
                        which(levels(data[["cell_type_dataset"]]) == ct)],
                    linewidth = 0.7
                )
            }
        }

        p + ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(
                    color = "black", fill = NA,
                    linewidth = 0.5),
                legend.position = "none",
                axis.text = ggplot2::element_blank(),
                axis.ticks = ggplot2::element_blank(),
                axis.title = ggplot2::element_blank()
            )
    }

    # Blank facet function
    .blankFunc <- function(data, mapping, ...) {
        ggplot2::ggplot() +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(
                    color = "black", fill = NA,
                    linewidth = 0.5),
                legend.position = "none",
                axis.text = ggplot2::element_blank(),
                axis.ticks = ggplot2::element_blank(),
                axis.title = ggplot2::element_blank(),
                panel.grid = ggplot2::element_blank()
            )
    }

    # Ridge diagonal facet
    .ridgeFunc <- function(data, mapping, ...) {
        # Get current mapping info
        x_var_name <- rlang::as_name(mapping[["x"]])

        # Create a long-format data frame for ridge plot
        plot_data <- data.frame(
            value = data[[x_var_name]],
            group = data[["cell_type_dataset"]]
        )

        # Create ridge plot with ggridges
        suppressMessages({
            p <- ggplot2::ggplot(plot_data,
                                 ggplot2::aes(x = .data[["value"]],
                                              y = .data[["group"]],
                                              fill = .data[["group"]])) +
                ggridges::geom_density_ridges(
                    alpha = 0.7,
                    scale = 2,
                    rel_min_height = 0.01,
                    quantile_lines = FALSE
                ) +
                ggplot2::scale_fill_manual(values = cell_type_colors) +
                ggplot2::scale_y_discrete(
                    limits = rev(levels(cell_type_dataset))) +
                ggplot2::theme_minimal() +
                ggplot2::theme(
                    panel.border = ggplot2::element_rect(color = "black",
                                                         fill = NA,
                                                         linewidth = 0.5),
                    axis.title = ggplot2::element_blank(),
                    axis.text.y = ggplot2::element_blank(),
                    axis.ticks.y = ggplot2::element_blank(),
                    legend.position = "none"
                )
        })

        return(p)
    }

    # Density diagonal facet
    .densityFunc <- function(data, mapping, ...) {
        # Get current mapping info
        x_var_name <- rlang::as_name(mapping[["x"]])

        ggplot2::ggplot(data = data, mapping = mapping) +
            ggplot2::geom_density(ggplot2::aes(fill = cell_type_dataset,
                                               color = cell_type_dataset),
                                  alpha = 0.5) +
            ggplot2::scale_fill_manual(values = cell_type_colors) +
            ggplot2::scale_color_manual(values = cell_type_colors) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(color = "black",
                                                     fill = NA,
                                                     linewidth = 0.5),
                axis.title.y = ggplot2::element_blank(),
                axis.text.y = ggplot2::element_blank(),
                axis.ticks.y = ggplot2::element_blank()
            )
    }

    # Boxplot diagonal facet
    .boxplotFunc <- function(data, mapping, ...) {
        # Extract the x variable name for the boxplot title
        x_name <- rlang::as_name(mapping[["x"]])

        # Create a long-format data frame for horizontal boxplot
        plot_data <- data.frame(
            value = data[[x_name]],
            group = data[["cell_type_dataset"]]
        )

        # Create horizontal boxplot
        ggplot2::ggplot(plot_data, ggplot2::aes(x = .data[["value"]],
                                                y = .data[["group"]],
                                                fill = .data[["group"]])) +
            ggplot2::geom_boxplot(alpha = 0.7,
                                  outlier.size = 0.5,
                                  width = 0.6) +
            ggplot2::scale_fill_manual(values = cell_type_colors) +
            ggplot2::scale_y_discrete(limits = rev(
                levels(cell_type_dataset))) +
            ggplot2::labs(x = "", y = "") +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                panel.border = ggplot2::element_rect(color = "black",
                                                     fill = NA,
                                                     linewidth = 0.5),
                axis.title.x = ggplot2::element_blank(),
                axis.text.x = ggplot2::element_text(angle = 45,
                                                    hjust = 1,
                                                    size = 7),
                legend.position = "none",
                axis.title.y = ggplot2::element_blank(),
                axis.text.y = ggplot2::element_blank(),
                axis.ticks.y = ggplot2::element_blank()
            )
    }

    # Select the lower facet plot type based on user input
    if (lower_facet == "scatter") {
        lower_plot <- .scatterFunc
    } else if (lower_facet == "contour") {
        lower_plot <- .smoothContourFunc
    } else if (lower_facet == "ellipse") {
        lower_plot <- .robustEllipseFunc
    } else if (lower_facet == "blank") {
        lower_plot <- .blankFunc
    }

    # Select the diagonal plot type based on user input
    if (diagonal_facet == "density") {
        diag_plot <- .densityFunc
    } else if (diagonal_facet == "boxplot") {
        diag_plot <- .boxplotFunc
    } else if (diagonal_facet == "ridge") {
        diag_plot <- .ridgeFunc
    }

    # Select the upper facet plot type based on user input
    if (upper_facet == "blank") {
        upper_plot <- .blankFunc
    } else if (upper_facet == "scatter") {
        upper_plot <- .scatterFunc
    } else if (upper_facet == "contour") {
        upper_plot <- .smoothContourFunc
    } else if (upper_facet == "ellipse") {
        upper_plot <- .robustEllipseFunc
    }

    # Create pairs plot using GGally
    plot_obj <- suppressMessages(
        GGally::ggpairs(
            dv_df,
            columns = seq_len(length(dv_subset)),
            mapping = ggplot2::aes(color = cell_type_dataset),
            lower = list(continuous = lower_plot),
            upper = list(continuous = upper_plot),
            diag = list(continuous = diag_plot),
            progress = FALSE,
            legend = GGally::grab_legend(legend_plot)
        )
    )

    # Add black frame around facet titles
    plot_obj <- plot_obj +
        ggplot2::theme(
            strip.background = ggplot2::element_rect(
                fill = "white", color = "black", linewidth = 0.5),
            strip.text = ggplot2::element_text(color = "black")
        )

    return(plot_obj)
}

#' @title Compute Cell Distances Between Reference and Query Data
#'
#' @description
#' This function computes the distances within the reference dataset and the distances from each query cell to all
#' reference cells for each cell type. It uses PCA for dimensionality reduction and Euclidean distance for distance calculation.
#'
#' @details
#' The function first performs PCA on the reference dataset and projects the query dataset onto the same PCA space.
#' It then computes pairwise Euclidean distances within the reference dataset for each cell type, as well as distances from each
#' query cell to all reference cells of a particular cell type. The results are stored in a list, with one entry per cell type.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data}
#' that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data}
#' that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.
#' @param pc_subset A numeric vector specifying which principal components to include in the plot. Default 1:5.
#' @param assay_name Name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#'
#' @return A list containing distance data for each cell type. Each entry in the list contains:
#' \describe{
#'   \item{ref_distances}{A vector of all pairwise distances within the reference subset for the cell type.}
#'   \item{query_to_ref_distances}{A matrix of distances from each query cell to all reference cells for the cell type.}
#' }
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{plot.calculateCellDistancesObject}}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Plot the PC data
#' distance_data <- calculateCellDistances(query_data = query_data,
#'                                         reference_data = reference_data,
#'                                         query_cell_type_col = "SingleR_annotation",
#'                                         ref_cell_type_col = "expert_annotation",
#'                                         pc_subset = 1:10)
#'
#' # Identify outliers for CD4
#' cd4_anomalies <- detectAnomaly(reference_data = reference_data,
#'                                query_data = query_data,
#'                                query_cell_type_col = "SingleR_annotation",
#'                                ref_cell_type_col = "expert_annotation",
#'                                pc_subset = 1:10,
#'                                n_tree = 500,
#'                                anomaly_threshold = 0.5)
#' cd4_top6_anomalies <- names(sort(cd4_anomalies$CD4$query_anomaly_scores, decreasing = TRUE)[1:6])
#'
#' # Plot the densities of the distances
#' plot(distance_data, ref_cell_type = "CD4", cell_names = cd4_top6_anomalies)
#' plot(distance_data, ref_cell_type = "CD8", cell_names = cd4_top6_anomalies)
#'
# Function to compute distances within reference data and between query data and reference cells
calculateCellDistances <- function(query_data,
                                   reference_data,
                                   query_cell_type_col,
                                   ref_cell_type_col,
                                   cell_types = NULL,
                                   pc_subset = 1:5,
                                   assay_name = "logcounts",
                                   max_cells_query = 5000,
                                   max_cells_ref = 5000) {

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = NULL)


    # Get the projected PCA data
    pca_output <- projectPCA(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = query_cell_type_col,
                             ref_cell_type_col = ref_cell_type_col,
                             cell_types = cell_types,
                             pc_subset = pc_subset,
                             assay_name = assay_name,
                             max_cells_ref = max_cells_ref,
                             max_cells_query = max_cells_query)

    # Create a list to store distance data for each cell type
    distance_data <- vector("list", length = length(cell_types))
    names(distance_data) <- cell_types

    # Function to compute Euclidean distance between a vector and each row of a matrix
    .computeDistances <- function(matrix, vector) {

        # Apply the distance function to each row of the matrix
        distances <- apply(matrix, 1, function(row) {
            sqrt(sum((row - vector) ^ 2))
        })

        return(distances)
    }

    for (cell_type in cell_types) {

        # Subset principal component scores for current cell type
        ref_subset_scores <- pca_output[which(
            pca_output[["dataset"]] == "Reference" &
                pca_output[["cell_type"]] == cell_type), pc_subset]
        query_subset_scores <- pca_output[pca_output[["dataset"]] == "Query",
                                          pc_subset]

        # Compute all pairwise distances within the reference subset
        ref_distances <- as.vector(dist(ref_subset_scores))

        # Compute distances from each query cell to all reference cells
        query_to_ref_distances <- apply(
            query_subset_scores, 1, function(query_cell,
                                             ref_subset_scores) {
                .computeDistances(ref_subset_scores,
                                   query_cell)
                }, ref_subset_scores = ref_subset_scores)

        # Store the distances
        distance_data[[cell_type]] <- list(
            ref_distances = ref_distances,
            query_to_ref_distances = t(query_to_ref_distances)
        )
    }

    # Add class of object
    class(distance_data) <- c(class(distance_data),
                              "calculateCellDistancesObject")

    # Return the distance data
    return(distance_data)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics) 

# Load example datasets
data("reference_data")
data("query_data")

# Test cases
test_that("calculateCellDistances computes distances correctly", {
    
    # Test with typical input
    distance_data <- calculateCellDistances(query_data = query_data, 
                                            reference_data = reference_data,
                                            query_cell_type_col = "SingleR_annotation",
                                            ref_cell_type_col = "expert_annotation",
                                            cell_types = c("CD4", "CD8", "B_and_plasma"),
                                            pc_subset = 1:5)
    
    # Check if the result is a list
    expect_true(is.list(distance_data))
    
    # Check if the list contains entries for each cell type
    expect_equal(length(distance_data), length(c("CD4", "CD8", "B_and_plasma")))
    
    # Check structure of each entry
    expect_named(distance_data[["CD4"]], c("ref_distances", "query_to_ref_distances"))
    expect_named(distance_data[["CD8"]], c("ref_distances", "query_to_ref_distances"))
    expect_named(distance_data[["B_and_plasma"]], c("ref_distances", "query_to_ref_distances"))
    
    # Check dimensions and types of data within each entry if needed
    
})

test_that("calculateCellDistances handles NULL cell_types gracefully", {
    
    # Test with NULL cell_types
    distance_data <- calculateCellDistances(query_data = query_data, 
                                            reference_data = reference_data,
                                            query_cell_type_col = "SingleR_annotation",
                                            ref_cell_type_col = "expert_annotation",
                                            pc_subset = 1:5)
    
    # Check if the result is a list
    expect_true(is.list(distance_data))
    
    # Check if the list contains entries for each cell type present in the data
    expect_equal(length(distance_data), length(na.omit(unique(c(reference_data$expert_annotation, 
                                                                query_data$SingleR_annotation)))))
    
})

test_that("calculateCellDistances handles single cell type gracefully", {
    
    # Test with a single cell type
    distance_data <- calculateCellDistances(query_data = query_data, 
                                            reference_data = reference_data,
                                            query_cell_type_col = "SingleR_annotation",
                                            ref_cell_type_col = "expert_annotation",
                                            cell_types = "CD4",
                                            pc_subset = 1:5)
    
    # Check if the result is a list
    expect_true(is.list(distance_data))
    
    # Check if the list contains an entry for the specified cell type
    expect_named(distance_data, "CD4")
    
    # Check structure of the CD4 entry
    expect_named(distance_data[["CD4"]], c("ref_distances", "query_to_ref_distances"))
    
})

#' @title Project Query Data Onto PCA Space of Reference Data
#'
#' @description
#' This function projects a query singleCellExperiment object onto the PCA space of a reference
#' singleCellExperiment object. The PCA analysis on the reference data is assumed to be pre-computed
#' and stored within the object. Optionally filters by cell types and downsamples the results.
#'
#' @details
#' This function assumes that the "PCA" element exists within the \code{reducedDims} of the reference data
#' (obtained using \code{reducedDim(reference_data)}) and that the genes used for PCA are present in both
#' the reference and query data. It performs centering and scaling of the query data based on the reference
#' data before projection using the FULL datasets to maintain proper mean centering. Cell type filtering
#' and downsampling are performed AFTER projection to preserve the statistical properties of the PCA space.
#' Cell names from the original SCE objects are preserved as rownames in the output.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix
#' for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix
#' for the reference cells.
#' @param query_cell_type_col character. The column name in the \code{colData} of \code{query_data}
#' that identifies the cell types.
#' @param ref_cell_type_col character. The column name in the \code{colData} of \code{reference_data}
#' that identifies the cell types.
#' @param cell_types A character vector specifying which cell types to retain in the output. If NULL,
#' no cell type filtering is performed. Default is NULL.
#' @param pc_subset A numeric vector specifying the subset of principal components (PCs) to compare. Default is 1:10.
#' @param assay_name Name of the assay on which to perform computations. Defaults to \code{"logcounts"}.
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is NULL.
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is NULL.
#'
#' @return A \code{data.frame} containing the projected data in rows (reference and query data combined),
#' optionally filtered by cell types and downsampled. Rownames preserve the original cell names from
#' the SCE objects.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Project the query data onto PCA space of reference
#' pca_output <- projectPCA(query_data = query_data,
#'                          reference_data = reference_data,
#'                          query_cell_type_col = "SingleR_annotation",
#'                          ref_cell_type_col = "expert_annotation",
#'                          pc_subset = 1:10)
#'
#' # Project with cell type filtering and balanced downsampling
#' pca_output_filtered <- projectPCA(query_data = query_data,
#'                                   reference_data = reference_data,
#'                                   query_cell_type_col = "SingleR_annotation",
#'                                   ref_cell_type_col = "expert_annotation",
#'                                   pc_subset = 1:5,
#'                                   cell_types = c("CD4", "CD8"),
#'                                   max_cells_ref = 1000,
#'                                   max_cells_query = 1000)
#'
# Function to project query data onto PCA space of reference data
projectPCA <- function(query_data,
                       reference_data,
                       query_cell_type_col,
                       ref_cell_type_col,
                       cell_types = NULL,
                       pc_subset = 1:10,
                       assay_name = "logcounts",
                       max_cells_query = NULL,
                       max_cells_ref = NULL){

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  pc_subset_ref = pc_subset,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Helper function to validate PCA integrity for reference data only
    .validateReferencePCA <- function(sce, assay_name) {
        # Check if PCA exists
        if (!"PCA" %in% reducedDimNames(sce)) {
            return(FALSE)
        }

        # Check if rotation matrix exists
        pca_attrs <- attributes(reducedDim(sce, "PCA"))
        rotation_mat <- pca_attrs[["rotation"]]
        if (is.null(rotation_mat)) {
            return(FALSE)
        }

        # Check if genes in rotation matrix exist in the assay
        pca_genes <- rownames(rotation_mat)
        assay_genes <- rownames(assay(sce, assay_name))
        if (!all(pca_genes %in% assay_genes)) {
            return(FALSE)
        }

        # Check dimension consistency
        pca_coords <- reducedDim(sce, "PCA")
        if (nrow(pca_coords) != ncol(sce)) {
            return(FALSE)
        }

        return(TRUE)
    }

    # Validate PCA integrity for reference data ONLY
    ref_pca_valid <- .validateReferencePCA(reference_data, assay_name)

    if (!ref_pca_valid) {
        message("Reference PCA is invalid or missing. Computing PCA for reference data only...")

        # Use processPCA to recompute PCA for REFERENCE DATA ONLY
        reference_data <- processPCA(sce_object = reference_data,
                                     assay_name = assay_name)

        # Validate that PCA was properly computed
        if (!.validateReferencePCA(reference_data, assay_name)) {
            stop("Failed to compute valid PCA for reference data. Please check your data and try again.")
        }

        message("Reference PCA computation completed. Proceeding with projection...")
    }

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = NULL)

    # Extract reference PCA components and rotation matrix
    ref_mat <- reducedDim(reference_data, "PCA")[, pc_subset, drop = FALSE]
    rotation_mat <- attributes(reducedDim(reference_data, "PCA"))[["rotation"]][, pc_subset, drop = FALSE]
    PCA_genes <- rownames(rotation_mat)

    # Check if genes used for PCA are available in query data
    if (!all(PCA_genes %in% rownames(assay(query_data, assay_name)))) {
        stop("Genes in reference PCA are not found in query data.")
    }

    # Center query data using full reference dataset (maintains original PCA centering)
    ref_assay <- assay(reference_data, assay_name)
    centering_vec <- Matrix::rowMeans(ref_assay)[PCA_genes]
    query_assay_subset <- assay(query_data, assay_name)[PCA_genes, , drop = FALSE]

    # Ensure gene order matches
    centering_vec <- centering_vec[rownames(rotation_mat)]
    query_assay_subset <- query_assay_subset[rownames(rotation_mat), , drop = FALSE]

    # Use sparse-compatible operations
    query_transposed <- Matrix::t(query_assay_subset)
    query_centered <- sweep(query_transposed, 2, centering_vec, "-")
    query_mat <- as.matrix(query_centered %*% rotation_mat)

    # Get cell names from the original SCE objects
    ref_cell_names <- colnames(reference_data)
    query_cell_names <- colnames(query_data)

    # Create full output dataframe with all cells
    full_output <- data.frame(
        rbind(ref_mat, query_mat),
        dataset = c(rep("Reference", nrow(ref_mat)),
                    rep("Query", nrow(query_mat))),
        cell_type = c(ifelse(rep(is.null(ref_cell_type_col), nrow(ref_mat)),
                             rep(NA, nrow(ref_mat)),
                             reference_data[[ref_cell_type_col]]),
                      ifelse(rep(is.null(query_cell_type_col), nrow(query_mat)),
                             rep(NA, nrow(query_mat)),
                             query_data[[query_cell_type_col]])),
        stringsAsFactors = FALSE
    )

    # Make cell names unique by prefixing with dataset name
    unique_ref_names <- paste0("Reference_", ref_cell_names)
    unique_query_names <- paste0("Query_", query_cell_names)
    rownames(full_output) <- c(unique_ref_names, unique_query_names)

    # Filter by cell types if specified
    if (!is.null(cell_types)) {
        # Check if specified cell types exist in the data
        available_types <- unique(full_output[["cell_type"]])
        available_types <- available_types[!is.na(available_types)]

        missing_types <- setdiff(cell_types, available_types)
        if (length(missing_types) > 0) {
            warning("Cell types not found in data: ", paste(missing_types, collapse = ", "))
        }

        # Filter to specified cell types
        cell_type_mask <- full_output[["cell_type"]] %in% cell_types
        if (sum(cell_type_mask) == 0) {
            warning("No cells found for the specified cell types. Returning empty data frame.")
            return(full_output[FALSE, ])
        }

        full_output <- full_output[which(cell_type_mask), ]
    }

    # Downsample each dataset separately if specified
    ref_indices <- which(full_output[["dataset"]] == "Reference")
    query_indices <- which(full_output[["dataset"]] == "Query")

    selected_indices <- c()

    # Downsample reference cells
    if (!is.null(max_cells_ref) && length(ref_indices) > max_cells_ref) {
        selected_ref_indices <- sample(ref_indices, max_cells_ref)
        selected_indices <- c(selected_indices, selected_ref_indices)
    } else {
        selected_indices <- c(selected_indices, ref_indices)
    }

    # Downsample query cells
    if (!is.null(max_cells_query) && length(query_indices) > max_cells_query) {
        selected_query_indices <- sample(query_indices, max_cells_query)
        selected_indices <- c(selected_indices, selected_query_indices)
    } else {
        selected_indices <- c(selected_indices, query_indices)
    }

    # Apply the selection (rownames are automatically preserved)
    full_output <- full_output[selected_indices, ]

    # Do NOT reset rownames - keep the original cell names
    return(full_output)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

test_that("projectPCA projects query data onto PCA space of reference data", {
    # Perform projection of query data onto reference PCA space
    pca_output <- projectPCA(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:10
    )

    # Check if output has expected number of rows
    expect_equal(nrow(pca_output), ncol(reference_data) + ncol(query_data))
    
    # Check if output has expected number of columns
    expect_equal(ncol(pca_output), length(1:10) + 2)  # +2 for dataset and cell_type columns
    
    # Check if dataset column has correct values
    expect_true(all(pca_output$dataset %in% c("Reference", "Query")))
    
    # Check if cell_type column has correct values
    expect_true(all(is.na(pca_output$cell_type) | pca_output$cell_type %in% unique(reference_data$expert_annotation)))
})

test_that("projectPCA handles incorrect parameters", {
    # Test for invalid query data column
    expect_error(projectPCA(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "invalid_column",
        ref_cell_type_col = "expert_annotation",
        pc_subset = 1:10
    ))
    
    # Test for invalid reference data column
    expect_error(projectPCA(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "invalid_column",
        pc_subset = 1:10
    ))
})

#' @title Plot Graph-Based Integration Diagnostics
#'
#' @description
#' The S3 plot method generates visualizations of annotation consistency diagnostics,
#' including query-only communities, cross-cell-type mixing, and local annotation inconsistencies.
#'
#' @details
#' The S3 plot method creates optimized visualizations showing different types of annotation
#' issues including community-level and local neighborhood-level inconsistencies.
#'
#' @param x An object of class \code{calculateGraphIntegrationObject} containing the diagnostic results.
#' @param plot_type Character string specifying visualization type. Options: "community_network" (default),
#'                  "cell_network", "community_data", "summary", "local_issues", or "annotation_issues".
#' @param color_by Character string specifying the variable to use for coloring points/elements if `plot_type` is
#'                 "community_network" or "cell_network". Default is "cell_type".
#' @param max_nodes Maximum number of nodes to display for performance. Default is 2000.
#' @param point_size Point size for graph nodes. Default is 0.8.
#' @param exclude_reference_only Logical indicating whether to exclude reference-only communities/cells from visualization.
#'                               Default is FALSE.
#' @param ... Additional arguments passed to ggplot2 functions.
#'
#' @return A \code{ggplot} object showing integration diagnostics.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{calculateGraphIntegration}}
#'
#' @rdname calculateGraphIntegration
#'
#' @importFrom stats reorder
#'
# Function to plot the graph integration diagnostics
plot.calculateGraphIntegrationObject <- function(
        x,
        plot_type = c("community_network", "cell_network", "community_data",
                      "summary", "local_issues", "annotation_issues"),
        color_by = c("cell_type", "community_type"),
        max_nodes = 2000,
        point_size = 0.8,
        exclude_reference_only = FALSE,
        ...) {

    plot_type <- match.arg(plot_type)
    color_by <- match.arg(color_by)

    colors <- list(
        high_query_prop = "#DC143C",
        cross_mixing = "#FF8C00",
        local_inconsistent = "#9932CC",
        well_integrated = "#2E8B57",
        reference_only = "#4169E1",
        small_community = "gray70",
        excellent = "#228B22",
        concerning = "#FF6347",
        problematic = "#DC143C"
    )

    pluralize <- function(count, singular, plural = paste0(singular, "s")) {
        if (count == 1) return(paste(count, singular))
        else return(paste(count, plural))
    }

    if (plot_type == "community_network") {
        community_composition <- x[["community_composition"]]

        if (nrow(community_composition) == 0) {
            p <- ggplot2::ggplot() +
                ggplot2::annotate("text", x = 0.5, y = 0.5,
                                  label = "No communities to display",
                                  size = 8, color = "gray50") +
                ggplot2::labs(title = "Community Network") +
                ggplot2::theme_void() +
                ggplot2::theme(plot.title = ggplot2::element_text(size = 14,
                                                                  face = "bold",
                                                                  hjust = 0.5))
        } else {
            # Filter out reference-only communities if requested
            if (exclude_reference_only) {
                reference_only_mask <- community_composition[["query_proportion"]] < 0.1
                community_composition <- community_composition[!reference_only_mask, ]
            }

            if (nrow(community_composition) == 0) {
                p <- ggplot2::ggplot() +
                    ggplot2::annotate("text", x = 0.5, y = 0.5,
                                      label = "No communities to display after filtering",
                                      size = 8, color = "gray50") +
                    ggplot2::labs(title = "Community Network") +
                    ggplot2::theme_void() +
                    ggplot2::theme(plot.title = ggplot2::element_text(size = 14,
                                                                      face = "bold",
                                                                      hjust = 0.5))
            } else {
                # Calculate community centroids in PCA space
                community_centroids <- data.frame(
                    community = integer(0),
                    x = numeric(0),
                    y = numeric(0),
                    stringsAsFactors = FALSE
                )

                for (comm in community_composition[["community"]]) {
                    comm_cells <- x[["cell_info"]][["community"]] == comm
                    if (sum(comm_cells) > 0) {
                        centroid_x <- mean(x[["graph_info"]][["layout"]][comm_cells, 1])
                        centroid_y <- mean(x[["graph_info"]][["layout"]][comm_cells, 2])
                        community_centroids <- rbind(community_centroids, data.frame(
                            community = comm,
                            x = centroid_x,
                            y = centroid_y,
                            stringsAsFactors = FALSE
                        ))
                    }
                }

                # Calculate adaptive threshold based on overall graph connectivity
                original_edges <- x[["graph_info"]][["edges"]]
                cell_communities <- x[["cell_info"]][["community"]]
                total_cells <- length(cell_communities)
                total_edges <- nrow(original_edges)

                # Base connectivity rate in the graph
                base_connectivity <- total_edges / choose(total_cells, 2)

                # Adaptive threshold: 2x the base rate, with bounds
                adaptive_threshold <- max(0.005, min(0.05, 2 * base_connectivity))

                # Create edge lookup for fast checking
                edge_lookup <- paste(original_edges[, 1],
                                     original_edges[, 2],
                                     sep = "_")
                edge_lookup <-
                    c(edge_lookup, paste(original_edges[, 2],
                                         original_edges[, 1],
                                         sep = "_"))
                edge_set <-
                    as.environment(as.list(setNames(rep(TRUE,
                                                        length(edge_lookup)),
                                                    edge_lookup)))

                edge_data <- data.frame(x = numeric(0), y = numeric(0),
                                        xend = numeric(0), yend = numeric(0),
                                        connection_strength = numeric(0))

                communities <- community_composition[["community"]]
                n_communities <- length(communities)

                # Check all pairs of communities
                if (n_communities > 1) {
                    for (i in seq_len(n_communities - 1)) {
                        for (j in seq(i + 1, n_communities)) {
                            comm1 <- communities[i]
                            comm2 <- communities[j]

                            cells_comm1 <- which(cell_communities == comm1)
                            cells_comm2 <- which(cell_communities == comm2)

                            if (length(cells_comm1) > 0 && length(cells_comm2) > 0) {
                                # Adaptive sampling based on community sizes
                                max_samples <- min(500, length(cells_comm1) * length(cells_comm2))
                                n_samples <- max(100, max_samples)

                                sampled_comm1 <- sample(cells_comm1, n_samples, replace = TRUE)
                                sampled_comm2 <- sample(cells_comm2, n_samples, replace = TRUE)

                                # Count connections
                                connections <- 0
                                for (k in seq_len(n_samples)) {
                                    pair_key <- paste(sampled_comm1[k], sampled_comm2[k], sep = "_")
                                    if (exists(pair_key, envir = edge_set)) {
                                        connections <- connections + 1
                                    }
                                }

                                connection_rate <- connections / n_samples

                                # Use adaptive threshold
                                if (connection_rate >= adaptive_threshold) {
                                    comm1_idx <- which(community_centroids$community == comm1)
                                    comm2_idx <- which(community_centroids$community == comm2)

                                    if (length(comm1_idx) > 0 && length(comm2_idx) > 0) {
                                        edge_data <- rbind(edge_data, data.frame(
                                            x = community_centroids$x[comm1_idx],
                                            y = community_centroids$y[comm1_idx],
                                            xend = community_centroids$x[comm2_idx],
                                            yend = community_centroids$y[comm2_idx],
                                            connection_strength = connection_rate
                                        ))
                                    }
                                }
                            }
                        }
                    }
                }

                # Prepare node data
                node_data <- merge(community_centroids,
                                   community_composition,
                                   by = "community")

                if (color_by == "cell_type") {
                    # Get all cell types and generate paired colors
                    all_cell_types <- sort(unique(x[["cell_info"]][["cell_type"]]))
                    cell_types_cases <- c()
                    for(cell_type_case in all_cell_types){
                        cell_types_cases <- c(cell_types_cases,
                                              paste(cell_type_case, "(Mixed)"),
                                              paste(cell_type_case, "(Pure)"))
                    }
                    paired_colors <- generateColors(cell_types_cases, paired = TRUE)

                    # Determine dominant cell type and whether community is mixed for each community
                    node_data[["dominant_cell_type"]] <- sapply(node_data[["community"]], function(comm) {
                        comm_cells <- x[["cell_info"]][x[["cell_info"]][["community"]] == comm, ]
                        cell_type_counts <- table(comm_cells[["cell_type"]])
                        names(cell_type_counts)[which.max(cell_type_counts)]
                    })
                    node_data[["is_mixed"]] <- node_data[["n_cell_types"]] > 1
                    node_data[["cell_type_case"]] <-
                        paste(node_data[["dominant_cell_type"]],
                              ifelse(node_data[["is_mixed"]], "(Mixed)", "(Pure)"))

                    # Assign colors based on dominant cell type and mixing status
                    node_data[["color"]] <- paired_colors[node_data[["cell_type_case"]]]

                    # Classify communities for shapes
                    node_data[["shape_type"]] <- "Well Integrated"
                    node_data[["shape_type"]][node_data[["community"]] %in%
                                                  x[["cross_type_mixing"]][["community"]]] <-
                        "Cross-Type Mixing"
                    node_data[["shape_type"]][node_data[["community"]] %in%
                                                  x[["high_query_prop_analysis"]][["community"]]] <-
                        "High Query Proportion"
                    node_data[["shape_type"]][node_data[["query_proportion"]] < 0.1 &
                                                  node_data[["shape_type"]] == "Well Integrated"] <-
                        "High Reference Proportion"

                    # Create color legend - only show colors that are actually used
                    used_types <- unique(node_data[["dominant_cell_type"]])
                    used_mixed <- any(node_data[["is_mixed"]])
                    used_pure <- any(!node_data[["is_mixed"]])

                    color_legend_labels <- names(paired_colors)
                    color_legend_values <- paired_colors

                    p <- ggplot2::ggplot() +
                        {if (nrow(edge_data) > 0) {
                            ggplot2::geom_segment(data = edge_data,
                                                  ggplot2::aes(
                                                      x = .data[["x"]], y = .data[["y"]],
                                                      xend = .data[["xend"]], yend = .data[["yend"]],
                                                      alpha = .data[["connection_strength"]]),
                                                  color = "gray60", linewidth = 0.8)
                        }} +
                        ggplot2::geom_point(data = node_data,
                                            ggplot2::aes(
                                                x = .data[["x"]], y = .data[["y"]],
                                                color = .data[["cell_type_case"]],
                                                shape = .data[["shape_type"]],
                                                size = .data[["total_cells"]]),
                                            alpha = 0.8) +
                        ggplot2::scale_color_manual(
                            values = color_legend_values,
                            name = "Dominant Cell Type"
                        ) +
                        ggplot2::scale_shape_manual(
                            values = c("Well Integrated" = 16,
                                       "Cross-Type Mixing" = 18,
                                       "High Query Proportion" = 17,
                                       "High Reference Proportion" = 15),
                            name = "Community Classification"
                        ) +
                        ggplot2::scale_size_continuous(
                            range = c(3, 10),
                            trans = "sqrt",
                            guide = "none"
                        ) +
                        ggplot2::scale_alpha_continuous(
                            range = c(0.3, 1.0),
                            guide = "none"
                        ) +
                        ggplot2::geom_text(
                            data = node_data,
                            ggplot2::aes(x = .data[["x"]], y = .data[["y"]],
                                         label = .data[["community"]]),
                            color = "black", size = 3, fontface = "bold") +
                        ggplot2::labs(
                            title = "Community Network: Cell Type Composition",
                            subtitle = paste0("Showing ", nrow(node_data),
                                              " communities with ",
                                              nrow(edge_data),
                                              " connections"),
                            caption = "Node size proportional to # cells; Shape indicates community classification; Edge opacity proportional to connection strength"
                        ) +
                        ggplot2::theme_void() +
                        ggplot2::theme(
                            plot.title = ggplot2::element_text(size = 14, face = "bold", hjust = 0.5),
                            plot.subtitle = ggplot2::element_text(size = 10, hjust = 0.5),
                            legend.position = "right"
                        )
                } else if (color_by == "community_type") {
                    # Original community type classification
                    node_data[["issue_type"]] <- "Well Integrated"
                    node_data[["issue_type"]][node_data[["community"]] %in%
                                                  x[["cross_type_mixing"]][["community"]]] <-
                        "Cross-Type Mixing"
                    node_data[["issue_type"]][node_data[["community"]] %in%
                                                  x[["high_query_prop_analysis"]][["community"]]] <-
                        "High Query Proportion"
                    node_data[["issue_type"]][node_data[["query_proportion"]] < 0.1 &
                                                  node_data[["issue_type"]] == "Well Integrated"] <-
                        "High Reference Proportion"

                    p <- ggplot2::ggplot() +
                        {if (nrow(edge_data) > 0) {
                            ggplot2::geom_segment(data = edge_data,
                                                  ggplot2::aes(
                                                      x = .data[["x"]], y = .data[["y"]],
                                                      xend = .data[["xend"]], yend = .data[["yend"]],
                                                      alpha = .data[["connection_strength"]]),
                                                  color = "gray60", linewidth = 0.8)
                        }} +
                        ggplot2::geom_point(data = node_data,
                                            ggplot2::aes(
                                                x = .data[["x"]], y = .data[["y"]],
                                                color = .data[["issue_type"]],
                                                size = .data[["total_cells"]]),
                                            alpha = 0.8) +
                        ggplot2::scale_color_manual(
                            values = c("High Query Proportion" = colors[["high_query_prop"]],
                                       "Cross-Type Mixing" = colors[["cross_mixing"]],
                                       "Well Integrated" = colors[["well_integrated"]],
                                       "High Reference Proportion" = colors[["reference_only"]]),
                            name = "Community Type"
                        ) +
                        ggplot2::scale_size_continuous(
                            range = c(3, 10),
                            trans = "sqrt",
                            guide = "none"
                        ) +
                        ggplot2::scale_alpha_continuous(
                            range = c(0.3, 1.0),
                            guide = "none"
                        ) +
                        ggplot2::geom_text(
                            data = node_data,
                            ggplot2::aes(x = .data[["x"]], y = .data[["y"]],
                                         label = .data[["community"]]),
                            color = "black", size = 3, fontface = "bold") +
                        ggplot2::labs(
                            title = "Community Network: Inter-Community Connection Strength",
                            subtitle = paste0("Showing ", nrow(node_data), " communities with ",
                                              nrow(edge_data), " connections"),
                            caption = "Node size proportional to # cells in community; Edge opacity proportional to connection strength"
                        ) +
                        ggplot2::theme_void() +
                        ggplot2::theme(
                            plot.title = ggplot2::element_text(size = 14, face = "bold", hjust = 0.5),
                            plot.subtitle = ggplot2::element_text(size = 10, hjust = 0.5),
                            legend.position = "right"
                        )

                }
            }
        }

    } else if (plot_type == "cell_network") {
        # Identify reference-only communities (communities with very low query proportion)
        reference_only_comms <- x[["community_composition"]][["community"]][
            x[["community_composition"]][["query_proportion"]] < 0.1
        ]

        # Filter out reference-only communities if requested
        if (exclude_reference_only) {
            keep_cells <- !(x[["cell_info"]][["community"]] %in% reference_only_comms)
            cell_info_filtered <- x[["cell_info"]][keep_cells, ]
            layout_filtered <- x[["graph_info"]][["layout"]][keep_cells, ]
            edges_filtered <- x[["graph_info"]][["edges"]][
                x[["graph_info"]][["edges"]][, 1] %in% which(keep_cells) &
                    x[["graph_info"]][["edges"]][, 2] %in% which(keep_cells),
            ]

            # Remap edge indices
            if (nrow(edges_filtered) > 0) {
                old_to_new <- rep(NA, nrow(x[["cell_info"]]))
                old_to_new[which(keep_cells)] <- seq_len(sum(keep_cells))
                edges_filtered[, 1] <- old_to_new[edges_filtered[, 1]]
                edges_filtered[, 2] <- old_to_new[edges_filtered[, 2]]
            }
        } else {
            cell_info_filtered <- x[["cell_info"]]
            layout_filtered <- x[["graph_info"]][["layout"]]
            edges_filtered <- x[["graph_info"]][["edges"]]
            keep_cells <- rep(TRUE, nrow(x[["cell_info"]]))
        }

        n_nodes <- nrow(layout_filtered)

        if (n_nodes > max_nodes) {
            local_inconsistent_cells <- x[["local_annotation_inconsistencies"]][["query_cell_idx"]]
            # Filter local inconsistent cells to only those remaining after reference_only filtering
            if (exclude_reference_only) {
                local_inconsistent_cells <- local_inconsistent_cells[
                    local_inconsistent_cells %in% which(keep_cells)
                ]
                # Remap indices
                old_to_new <- rep(NA, nrow(x[["cell_info"]]))
                old_to_new[which(keep_cells)] <- seq_len(sum(keep_cells))
                local_inconsistent_cells <- old_to_new[local_inconsistent_cells]
                local_inconsistent_cells <- local_inconsistent_cells[!is.na(local_inconsistent_cells)]
            }

            high_query_prop_comms <- x[["high_query_prop_analysis"]][["community"]]
            cross_mixing_comms <- x[["cross_type_mixing"]][["community"]]

            priority_1 <- which(cell_info_filtered[["community"]] %in%
                                    high_query_prop_comms &
                                    cell_info_filtered[["dataset"]] == "Query")
            priority_2 <- setdiff(local_inconsistent_cells, priority_1)
            priority_3 <- which(cell_info_filtered[["community"]] %in%
                                    cross_mixing_comms &
                                    cell_info_filtered[["dataset"]] == "Query" &
                                    !(seq_len(nrow(cell_info_filtered)) %in%
                                          c(priority_1, priority_2)))
            priority_4 <- which(cell_info_filtered[["community"]] %in%
                                    c(high_query_prop_comms, cross_mixing_comms) &
                                    cell_info_filtered[["dataset"]] == "Reference" &
                                    !(seq_len(nrow(cell_info_filtered)) %in% c(priority_1,
                                                                               priority_2,
                                                                               priority_3)))

            keep_nodes <- c()
            remaining_slots <- max_nodes
            min_slots_per_type <- 50

            if (length(priority_1) > 0) {
                take_p1 <- min(length(priority_1), remaining_slots)
                keep_nodes <- c(keep_nodes, priority_1[1:take_p1])
                remaining_slots <- remaining_slots - take_p1
            }

            if (length(priority_2) > 0 && remaining_slots > 0) {
                take_p2 <- min(length(priority_2), max(min_slots_per_type, remaining_slots))
                keep_nodes <- c(keep_nodes, priority_2[1:take_p2])
                remaining_slots <- remaining_slots - take_p2
            }

            if (length(priority_3) > 0 && remaining_slots > 0) {
                take_p3 <- min(length(priority_3), max(min_slots_per_type, remaining_slots))
                keep_nodes <- c(keep_nodes, priority_3[1:take_p3])
                remaining_slots <- remaining_slots - take_p3
            }

            if (length(priority_4) > 0 && remaining_slots > 0) {
                take_p4 <- min(length(priority_4), remaining_slots * 0.3)
                if (take_p4 > 0) {
                    keep_nodes <- c(keep_nodes, sample(priority_4, take_p4))
                    remaining_slots <- remaining_slots - take_p4
                }
            }

            if (remaining_slots > 0) {
                all_priority <- c(priority_1, priority_2,
                                  priority_3,
                                  priority_4)
                remaining_cells <- setdiff(seq_len(nrow(cell_info_filtered)),
                                           all_priority)
                if (length(remaining_cells) > 0) {
                    take_random <- min(length(remaining_cells), remaining_slots)
                    keep_nodes <- c(keep_nodes, sample(remaining_cells, take_random))
                }
            }

            cell_info_subset <- cell_info_filtered[keep_nodes, ]
            layout_subset <- layout_filtered[keep_nodes, ]

            edges_keep <- edges_filtered[, 1] %in% keep_nodes & edges_filtered[, 2] %in%
                keep_nodes
            edges_subset <- edges_filtered[edges_keep, ]

            if (nrow(edges_subset) > 0) {
                old_to_new <- rep(NA, n_nodes)
                old_to_new[keep_nodes] <- seq_along(keep_nodes)
                edges_subset[, 1] <- old_to_new[edges_subset[, 1]]
                edges_subset[, 2] <- old_to_new[edges_subset[, 2]]

                if (nrow(edges_subset) > 3000) {
                    edges_subset <- edges_subset[sample(nrow(edges_subset), 3000), ]
                }
            }
        } else {
            cell_info_subset <- cell_info_filtered
            layout_subset <- layout_filtered
            edges_subset <- edges_filtered
            keep_nodes <- seq_len(nrow(cell_info_filtered))
            if (nrow(edges_subset) > 3000) {
                edges_subset <- edges_subset[sample(nrow(edges_subset), 3000), ]
            }
        }

        node_data <- data.frame(
            x = layout_subset[, 1],
            y = layout_subset[, 2],
            dataset = cell_info_subset[["dataset"]],
            cell_type = cell_info_subset[["cell_type"]],
            community = cell_info_subset[["community"]],
            stringsAsFactors = FALSE
        )

        # Create edge data
        if (nrow(edges_subset) > 0) {
            edge_data <- data.frame(
                x = layout_subset[edges_subset[, 1], 1],
                y = layout_subset[edges_subset[, 1], 2],
                xend = layout_subset[edges_subset[, 2], 1],
                yend = layout_subset[edges_subset[, 2], 2]
            )
        } else {
            edge_data <- data.frame(x = numeric(0), y = numeric(0),
                                    xend = numeric(0), yend = numeric(0))
        }

        # Create plot based on color_by parameter
        if (color_by == "cell_type") {
            # Generate colors for cell types
            all_cell_types <- sort(unique(node_data[["cell_type"]]))
            cell_types_cases <- c()
            for(cell_type_case in all_cell_types){
                cell_types_cases <- c(cell_types_cases,
                                      paste(c("Reference", "Query"), cell_type_case))
            }
            cell_type_colors <- generateColors(cell_types_cases, paired = TRUE)

            # Set up data for node colors
            node_data[["cell_type_case"]] <- paste(node_data[["dataset"]],
                                                   node_data[["cell_type"]])

            p <- ggplot2::ggplot() +
                {if (nrow(edge_data) > 0) {
                    ggplot2::geom_segment(
                        data = edge_data,
                        ggplot2::aes(x = .data[["x"]], y = .data[["y"]],
                                     xend = .data[["xend"]], yend = .data[["yend"]]),
                        alpha = 0.2, color = "gray80", linewidth = 0.2)
                }} +
                ggplot2::geom_point(
                    data = node_data,
                    ggplot2::aes(x = .data[["x"]], y = .data[["y"]],
                                 color = .data[["cell_type_case"]]),
                    size = point_size, alpha = 0.7) +
                ggplot2::scale_color_manual(
                    values = cell_type_colors,
                    name = "Cell Type"
                )

            plot_title <- "Cell Network: Cell Type Distribution"

        } else if (color_by == "community_type") {
            # Add issue type classification
            node_data[["issue_type"]] <- "Well Integrated"

            # Identify reference-only communities for remaining nodes
            reference_only_comms_remaining <- reference_only_comms[
                reference_only_comms %in% unique(node_data[["community"]])
            ]

            node_data[["issue_type"]][node_data[["community"]] %in%
                                          reference_only_comms_remaining] <-
                "High Reference Proportion"
            node_data[["issue_type"]][node_data[["community"]] %in%
                                          x[["cross_type_mixing"]][["community"]]] <-
                "Cross-Type Mixing"
            node_data[["issue_type"]][node_data[["community"]] %in%
                                          x[["high_query_prop_analysis"]][["community"]]] <-
                "High Query Proportion"

            if (nrow(x[["local_annotation_inconsistencies"]]) > 0) {
                local_inconsistent_cells <- x[["local_annotation_inconsistencies"]][["query_cell_idx"]]

                if (exclude_reference_only) {
                    # Map original indices to current subset
                    original_keep_indices <- which(keep_cells)
                    subset_positions <- match(local_inconsistent_cells, original_keep_indices)
                    subset_positions <- subset_positions[!is.na(subset_positions)]

                    if (n_nodes > max_nodes) {
                        # Further map to final keep_nodes
                        final_positions <- match(subset_positions, keep_nodes)
                        valid_positions <- final_positions[!is.na(final_positions)]
                    } else {
                        valid_positions <- subset_positions
                    }
                } else {
                    if (n_nodes > max_nodes) {
                        subset_positions <- match(local_inconsistent_cells, keep_nodes)
                        valid_positions <- subset_positions[!is.na(subset_positions)]
                    } else {
                        valid_positions <- local_inconsistent_cells
                    }
                }

                if (length(valid_positions) > 0) {
                    # Only override if not already High Query Proportion
                    local_mask <- node_data[["issue_type"]][valid_positions] != "High Query Proportion"
                    node_data[["issue_type"]][valid_positions[local_mask]] <- "Local Inconsistent"
                }
            }

            p <- ggplot2::ggplot() +
                {if (nrow(edge_data) > 0) {
                    ggplot2::geom_segment(
                        data = edge_data,
                        ggplot2::aes(x = .data[["x"]], y = .data[["y"]],
                                     xend = .data[["xend"]], yend = .data[["yend"]]),
                        alpha = 0.2, color = "gray80", linewidth = 0.2)
                }} +
                ggplot2::geom_point(data = node_data,
                                    ggplot2::aes(x = .data[["x"]], y = .data[["y"]],
                                                 color = .data[["issue_type"]]),
                                    size = point_size, alpha = 0.7) +
                ggplot2::scale_color_manual(
                    values = c("High Query Proportion" = colors[["high_query_prop"]],
                               "Cross-Type Mixing" = colors[["cross_mixing"]],
                               "Local Inconsistent" = colors[["local_inconsistent"]],
                               "Well Integrated" = colors[["well_integrated"]],
                               "High Reference Proportion" = colors[["reference_only"]]),
                    name = "Annotation Status"
                )

            plot_title <- "Cell Network: Annotation Consistency Diagnostics"
        }

        # Create subtitle
        high_query_prop_comms <- nrow(x[["high_query_prop_analysis"]])
        cross_mixing_comms <- nrow(x[["cross_type_mixing"]])
        local_inconsistent_cells <- nrow(x[["local_annotation_inconsistencies"]])

        subtitle_parts <- c()
        if (high_query_prop_comms > 0) {
            subtitle_parts <- c(subtitle_parts,
                                paste0("Query-Only (",
                                       pluralize(high_query_prop_comms,
                                                 "community", "communities"), ")"))
        }
        if (cross_mixing_comms > 0) {
            subtitle_parts <- c(subtitle_parts,
                                paste0("Cross-Type Mixing (",
                                       pluralize(cross_mixing_comms,
                                                 "community", "communities"), ")"))
        }
        if (local_inconsistent_cells > 0) {
            subtitle_parts <- c(subtitle_parts,
                                paste0("Local Inconsistent (",
                                       pluralize(local_inconsistent_cells,
                                                 "cell"), ")"))
        }

        subtitle_text <- if (length(subtitle_parts) > 0) {
            paste(subtitle_parts, collapse = " | ")
        } else {
            "No annotation issues detected"
        }

        # Add titles and theme
        p <- p +
            ggplot2::labs(
                title = plot_title,
                subtitle = subtitle_text,
                caption = if(nrow(x[["graph_info"]][["layout"]]) >
                             max_nodes) paste0(
                                 "Showing ",
                                 nrow(node_data), " of ",
                                 nrow(x[["graph_info"]][["layout"]]),
                                 " cells (prioritizing problematic cells)") else ""
            ) +
            ggplot2::theme_void() +
            ggplot2::theme(
                plot.title = ggplot2::element_text(size = 14,
                                                   face = "bold",
                                                   hjust = 0.5),
                plot.subtitle = ggplot2::element_text(size = 10,
                                                      hjust = 0.5),
                legend.position = "right"
            )

    } else if (plot_type == "community_data") {
        community_composition <- x[["community_composition"]]

        if (nrow(community_composition) == 0) {
            p <- ggplot2::ggplot() +
                ggplot2::annotate("text", x = 0.5, y = 0.5,
                                  label = "No communities to display",
                                  size = 8, color = "gray50") +
                ggplot2::labs(title = "Community Overview") +
                ggplot2::theme_void() +
                ggplot2::theme(plot.title = ggplot2::element_text(size = 14,
                                                                  face = "bold",
                                                                  hjust = 0.5))
        } else {
            # Ensure n_cell_types is integer
            community_composition[["n_cell_types"]] <-
                as.integer(community_composition[["n_cell_types"]])

            # Filter out reference-only communities if requested
            if (exclude_reference_only) {
                # Identify reference-only communities (communities with very low query proportion)
                reference_only_mask <- community_composition[["query_proportion"]] < 0.1
                community_composition <- community_composition[!reference_only_mask, ]
            }

            if (nrow(community_composition) == 0) {
                p <- ggplot2::ggplot() +
                    ggplot2::annotate("text", x = 0.5, y = 0.5,
                                      label = "No communities to display after filtering",
                                      size = 8, color = "gray50") +
                    ggplot2::labs(title = "Community Overview") +
                    ggplot2::theme_void() +
                    ggplot2::theme(plot.title = ggplot2::element_text(size = 14, face = "bold", hjust = 0.5))
            } else {
                # Classify communities
                community_composition[["issue_type"]] <-
                    "Well Integrated"
                community_composition[["issue_type"]][community_composition[["community"]] %in%
                                                          x[["cross_type_mixing"]][["community"]]] <-
                    "Cross-Type Mixing"
                community_composition[["issue_type"]][community_composition[["community"]] %in%
                                                          x[["high_query_prop_analysis"]][["community"]]] <-
                    "High Query Proportion"

                # Add Reference Only communities (communities with very low query proportion)
                community_composition[["issue_type"]][community_composition[["query_proportion"]] < 0.1 &
                                                          community_composition[["issue_type"]] == "Well Integrated"] <-
                    "High Reference Proportion"

                p <- ggplot2::ggplot(community_composition, ggplot2::aes(x = .data[["query_proportion"]],
                                                                         y = .data[["total_cells"]])) +
                    ggplot2::geom_point(ggplot2::aes(color = .data[["issue_type"]],
                                                     size = .data[["n_cell_types"]]),
                                        alpha = 0.7) +
                    ggplot2::scale_color_manual(
                        values = c("High Query Proportion" = colors[["high_query_prop"]],
                                   "Cross-Type Mixing" = colors[["cross_mixing"]],
                                   "Well Integrated" = colors[["well_integrated"]],
                                   "High Reference Proportion" = colors[["reference_only"]]),
                        name = "Community Type"
                    ) +
                    ggplot2::scale_size_continuous(
                        range = c(2, 8), name = "# Cell Types",
                        breaks = function(x) {
                            pretty_breaks <- pretty(x, n = 5)
                            return(pretty_breaks[pretty_breaks == round(pretty_breaks)])
                        },
                        labels = function(x) as.character(as.integer(x))) +
                    ggplot2::geom_text(ggplot2::aes(
                        label = .data[["community"]],
                        vjust = -0.75 - (.data[["n_cell_types"]] - min(.data[["n_cell_types"]])) /
                            (max(.data[["n_cell_types"]]) - min(.data[["n_cell_types"]]) + 1) * 1.5),
                        size = 3) +
                    ggplot2::labs(
                        title = "Community Data Overview",
                        x = "Query Proportion",
                        y = "Total Cells in Community"
                    ) +
                    ggplot2::theme_bw() +
                    ggplot2::theme(
                        plot.title = ggplot2::element_text(size = 14, face = "bold", hjust = 0.5),
                        legend.position = "right"
                    )
            }
        }

    } else if (plot_type == "summary") {
        summary_data <- x[["annotation_consistency"]]
        summary_data[["total_issues"]] <- summary_data[["high_query_prop_cells"]] +
            summary_data[["true_cross_mixing_cells"]]

        local_summary <- x[["local_inconsistency_summary"]]
        summary_data <- merge(summary_data,
                              local_summary[, c("cell_type",
                                                "locally_inconsistent_cells")],
                              by = "cell_type")
        summary_data[["total_issues"]] <- summary_data[["total_issues"]] +
            summary_data[["locally_inconsistent_cells"]]
        summary_data[["total_issue_rate"]] <- summary_data[["total_issues"]] /
            summary_data[["total_query_cells"]]

        summary_data[["issue_category"]] <- cut(summary_data[["total_issue_rate"]],
                                                breaks = c(-Inf, 0.05, 0.15, Inf),
                                                labels = c("Excellent",
                                                           "Concerning",
                                                           "Problematic"))

        p <- ggplot2::ggplot(summary_data, ggplot2::aes(x = reorder(.data[["cell_type"]],
                                                                    .data[["total_issue_rate"]]))) +
            ggplot2::geom_col(ggplot2::aes(y = .data[["total_issue_rate"]],
                                           fill = .data[["issue_category"]]),
                              alpha = 0.8, color = "black", linewidth = 0.3) +
            ggplot2::scale_fill_manual(
                values = c("Excellent" = colors[["excellent"]],
                           "Concerning" = colors[["concerning"]],
                           "Problematic" = colors[["problematic"]]),
                name = "Issue Level"
            ) +
            ggplot2::geom_text(ggplot2::aes(y = .data[["total_issue_rate"]],
                                            label = paste0(.data[["total_issues"]], "/",
                                                           .data[["total_query_cells"]])),
                               vjust = -0.5, size = 3) +
            ggplot2::labs(
                title = "Total Annotation Issues by Cell Type",
                subtitle = "Includes query-only, cross-mixing, and local inconsistencies",
                x = "Cell Type",
                y = "Proportion of Query Cells with Issues",
                caption = paste0("Mean issue rate: ",
                                 round(mean(summary_data[["total_issue_rate"]]), 3))
            ) +
            ggplot2::theme_bw() +
            ggplot2::theme(
                axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
                plot.title = ggplot2::element_text(size = 14, face = "bold", hjust = 0.5),
                legend.position = "bottom"
            ) +
            ggplot2::coord_cartesian(ylim = c(0, max(1, max(summary_data[["total_issue_rate"]]) * 1.1)))

    } else if (plot_type == "local_issues") {
        if (nrow(x[["local_annotation_inconsistencies"]]) == 0) {
            p <- ggplot2::ggplot() +
                ggplot2::annotate("text", x = 0.5, y = 0.6,
                                  label = "No local annotation inconsistencies detected!",
                                  size = 8, color = colors[["excellent"]], fontface = "bold") +
                ggplot2::annotate("text", x = 0.5, y = 0.4,
                                  label = "All query annotations supported by local neighborhoods",
                                  size = 6, color = colors[["excellent"]]) +
                ggplot2::labs(title = "Local Annotation Consistency") +
                ggplot2::theme_void() +
                ggplot2::theme(plot.title = ggplot2::element_text(size = 14, face = "bold", hjust = 0.5))
        } else {
            local_data <- x[["local_inconsistency_summary"]]
            local_data <- local_data[local_data[["locally_inconsistent_cells"]] > 0, ]

            if (nrow(local_data) > 0) {
                local_data[["inconsistency_category"]] <- cut(
                    local_data[["local_inconsistency_rate"]],
                    breaks = c(-Inf, 0.1, 0.3, Inf),
                    labels = c("Minor", "Moderate", "Severe"))

                p <- ggplot2::ggplot(local_data, ggplot2::aes(x = reorder(
                    .data[["cell_type"]],
                    .data[["local_inconsistency_rate"]]))) +
                    ggplot2::geom_col(ggplot2::aes(
                        y = .data[["local_inconsistency_rate"]],
                        fill = .data[["inconsistency_category"]]),
                        alpha = 0.8, color = "black", linewidth = 0.3) +
                    ggplot2::scale_fill_manual(
                        values = c("Minor" = colors[["excellent"]],
                                   "Moderate" = colors[["concerning"]],
                                   "Severe" = colors[["problematic"]]),
                        name = "Severity"
                    ) +
                    ggplot2::geom_text(ggplot2::aes(
                        y = .data[["local_inconsistency_rate"]],
                        label = paste0(.data[["locally_inconsistent_cells"]], "/",
                                       .data[["total_query_cells"]])),
                        vjust = -0.5, size = 3) +
                    ggplot2::labs(
                        title = "Local Annotation Inconsistencies by Cell Type",
                        subtitle = "Query cells not supported by reference neighbors",
                        x = "Cell Type",
                        y = "Local Inconsistency Rate",
                        caption = paste0(
                            "Mean rate: ",
                            round(x[["overall_metrics"]][["mean_local_inconsistency_rate"]], 3))
                    ) +
                    ggplot2::theme_bw() +
                    ggplot2::theme(
                        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
                        plot.title = ggplot2::element_text(size = 14, face = "bold", hjust = 0.5),
                        legend.position = "bottom"
                    )
            } else {
                p <- ggplot2::ggplot() +
                    ggplot2::annotate("text", x = 0.5, y = 0.5, label = "No data to display", size = 6) +
                    ggplot2::theme_void()
            }
        }

    } else if (plot_type == "annotation_issues") {
        total_issues <- nrow(x[["high_query_prop_analysis"]]) + nrow(x[["cross_type_mixing"]]) +
            nrow(x[["local_annotation_inconsistencies"]])

        if (total_issues == 0) {
            p <- ggplot2::ggplot() +
                ggplot2::annotate("text", x = 0.5, y = 0.6,
                                  label = "No annotation issues detected!",
                                  size = 8, color = colors[["excellent"]], fontface = "bold") +
                ggplot2::annotate("text", x = 0.5, y = 0.4,
                                  label = "Excellent annotation consistency across all metrics",
                                  size = 6, color = colors[["excellent"]]) +
                ggplot2::labs(title = "Comprehensive Annotation Assessment") +
                ggplot2::theme_void() +
                ggplot2::theme(plot.title = ggplot2::element_text(size = 14, face = "bold", hjust = 0.5))
        } else {
            high_query_prop_comms <- nrow(x[["high_query_prop_analysis"]])
            cross_mixing_comms <- nrow(x[["cross_type_mixing"]])
            local_inconsistent_cells <- nrow(x[["local_annotation_inconsistencies"]])

            issue_summary <- data.frame(
                issue_type = character(0),
                count = numeric(0),
                affected_cells = numeric(0),
                label_text = character(0),
                stringsAsFactors = FALSE
            )

            if (high_query_prop_comms > 0) {
                issue_summary <- rbind(issue_summary, data.frame(
                    issue_type = "Query-Only Communities",
                    count = high_query_prop_comms,
                    affected_cells = sum(x[["high_query_prop_analysis"]][["total_query_cells"]]),
                    label_text = paste0(
                        "Query-Only Communities\n(", pluralize(high_query_prop_comms,
                                                               "community", "communities"), ")"),
                    stringsAsFactors = FALSE
                ))
            }

            if (cross_mixing_comms > 0) {
                issue_summary <- rbind(issue_summary, data.frame(
                    issue_type = "Cross-Type Mixing",
                    count = cross_mixing_comms,
                    affected_cells = sum(x[["cross_type_mixing"]][["n_query"]]),
                    label_text = paste0(
                        "Cross-Type Mixing\n(", pluralize(cross_mixing_comms,
                                                          "community", "communities"), ")"),
                    stringsAsFactors = FALSE
                ))
            }

            if (local_inconsistent_cells > 0) {
                issue_summary <- rbind(issue_summary, data.frame(
                    issue_type = "Local Inconsistencies",
                    count = local_inconsistent_cells,
                    affected_cells = local_inconsistent_cells,
                    label_text = paste0(
                        "Local Inconsistencies\n(",
                        pluralize(local_inconsistent_cells, "cell"), ")"),
                    stringsAsFactors = FALSE
                ))
            }

            p <- ggplot2::ggplot(issue_summary, ggplot2::aes(
                x = reorder(.data[["label_text"]], .data[["affected_cells"]]),
                y = .data[["affected_cells"]])) +
                ggplot2::geom_col(ggplot2::aes(fill = .data[["issue_type"]]),
                                  alpha = 0.8, color = "black", linewidth = 0.3) +
                ggplot2::scale_fill_manual(
                    values = c("Query-Only Communities" = colors[["high_query_prop"]],
                               "Cross-Type Mixing" = colors[["cross_mixing"]],
                               "Local Inconsistencies" = colors[["local_inconsistent"]]),
                    name = "Issue Type"
                ) +
                ggplot2::labs(
                    title = "Annotation Issues Summary",
                    subtitle = "Number of affected query cells by issue type",
                    x = "Issue Type",
                    y = "Number of Affected Query Cells"
                ) +
                ggplot2::theme_bw() +
                ggplot2::theme(
                    axis.text.x = ggplot2::element_text(angle = 0, hjust = 0.5, size = 10),
                    axis.text.y = ggplot2::element_text(size = 10),
                    plot.title = ggplot2::element_text(size = 14, face = "bold", hjust = 0.5),
                    legend.position = "none",
                    plot.margin = ggplot2::margin(20, 40, 20, 40)
                ) +
                ggplot2::coord_flip() +
                ggplot2::scale_y_continuous(expand = ggplot2::expansion(mult = c(0, 0.1)))
        }
    }

    return(p)
}

#' @title Reference Single-Cell RNA-Seq Dataset
#'
#' @description
#' This dataset contains the processed reference dataset from the HeOrganAtlas dataset for Marrow tissue.
#' It has been preprocessed to include log-normalized counts, specific metadata columns, and PCA, t-SNE, and UMAP results.
#'
#' @details
#' This dataset underwent the following steps:
#' \itemize{
#'   \item Loads the HeOrganAtlas dataset specifically for Marrow tissue from the \code{scRNAseq} package.
#'   \item Divides the loaded dataset into a reference dataset used for downstream analysis.
#'   \item Performs log normalization on the reference dataset using the function \code{logNormCounts} from the \code{scuttle} package.
#'   \item Selects the column \code{expert_annotation}) from the cell metadata for downstream analysis.
#'   \item Selects highly variable genes (HVGs) using the function \code{getTopHVGs} from the \code{scran} package on the reference dataset.
#'   \item Performs Principal Component Analysis (PCA) on the reference dataset using the function \code{runPCA} from the \code{scater} package.
#'   \item Performs t-Distributed Stochastic Neighbor Embedding (t-SNE) on the reference dataset using the function \code{runTSNE} from the \code{scater} package.
#'   \item Performs Uniform Manifold Approximation and Projection (UMAP) on the reference dataset using the function \code{runUMAP} from the \code{scater} package.
#' }
#'
#' @seealso Use \code{data("reference_data")} to load and access the resulting reference dataset.
#'
#' @source The HeOrganAtlas dataset, available through the scRNAseq package.

#' @references He, et al. (2020). HeOrganAtlas: a comprehensive human organ atlas based on single-cell RNA sequencing.
#'
#' @examples
#' # Load and explore the reference dataset
#' data("reference_data")
#'
#' @keywords internal
#'
"reference_data"

#' @title Query Single-Cell RNA-Seq Dataset
#'
#' @description
#' This dataset contains the processed query dataset from the HeOrganAtlas dataset for Marrow tissue.
#' It has been preprocessed to include log-normalized counts, specific metadata columns, annotations
#' based on SingleR cell type scoring, and PCA, t-SNE, and UMAP results.
#'
#' @details
#' This dataset underwent the following steps:
#' \itemize{
#'   \item Loads the HeOrganAtlas dataset specifically for Marrow tissue from the \code{scRNAseq} package.
#'   \item Divides the loaded dataset into a query dataset used for downstream analysis.
#'   \item Performs log normalization on the query dataset using the function \code{logNormCounts} from the \code{scuttle} package.
#'   \item Selects specific columns (\code{percent_mito}, \code{expert_annotation}) from the cell metadata for downstream analysis.
#'   \item Selects highly variable genes (HVGs) using the function \code{getTopHVGs} from the \code{scran} package on the query dataset.
#'   \item Computes AUC gene set scores using the function \code{AUCell_calcAUC} from the \code{AUCell} package based on a CD4 T cell
#'         signature containing 12 known CD4 T cell marker genes (IL7R, CCR7, SELL, LEF1, TCF7, LTB, KLF2, IL32, CD2, CD3D, CD3E, CD3G)
#'         and adds these scores to the query dataset as \code{gene_set_scores}.
#'   \item Intersects the highly variable genes between the query and reference datasets to obtain common genes for analysis.
#'   \item Performs Principal Component Analysis (PCA) on the query dataset using the function \code{runPCA} from the \code{scater}
#'         package.
#'   \item Performs t-Distributed Stochastic Neighbor Embedding (t-SNE) on the query dataset using the function \code{runTSNE} from the
#'         \code{scater} package.
#'   \item Performs Uniform Manifold Approximation and Projection (UMAP) on the query dataset using the function \code{runUMAP} from the
#'         \code{scater} package.
#'   \item Adds SingleR annotations (\code{SingleR_annotation}) and annotation scores (\code{annotation_scores}) to the query dataset
#'         using the function \code{SingleR} from the \code{SingleR} package.

#' }
#'
#' @seealso
#' Use \code{data("query_data")} to load and access the resulting query dataset and the
#' \code{data("reference_data")} for comparison with the reference dataset.
#'
#' @source The HeOrganAtlas dataset, available through the scRNAseq package.
#' @references He, et al. (2020). HeOrganAtlas: a comprehensive human organ atlas based on single-cell RNA sequencing.
#'
#' @examples
#' # Load and explore the query dataset
#' data("query_data")
#'
#' @keywords internal
#'
"query_data"

#' @title Quality Control Single-Cell RNA-Seq Dataset
#'
#' @description
#' This dataset contains the processed query dataset from the Bunis haematopoietic stem and progenitor cell data.
#' It has been preprocessed to include log-normalized counts, QC metrics, SingleR cell type predictions,
#' and annotation scores.
#'
#' @details
#' This dataset underwent the following steps:
#' \itemize{
#'   \item Loads the \code{hpca} reference dataset using \code{fetchReference} from the \code{celldex} package.
#'   \item Loads the QC dataset (Bunis haematopoietic stem and progenitor cell data) from Bunis DG et al. (2021).
#'   \item Adds QC metrics to the QC dataset using the function \code{addPerCellQCMetrics} from the \code{scuttle} package.
#'   \item Performs log normalization on the QC dataset using the function \code{logNormCounts} from the \code{scuttle} package.
#'   \item Runs SingleR to predict cell types and assigns predicted labels to the QC dataset using the function \code{SingleR} from the \code{SingleR} package.
#'   \item Assigns annotation scores to the QC dataset.
#'   \item Selects specific columns (\code{total}, \code{SingleR_annotation}, \code{annotation_scores}) from the cell metadata for downstream analysis.
#'   \item Selects highly variable genes (HVGs) using the function \code{getTopHVGs} from the \code{scran} package on the QC dataset.
#' }
#'
#' @seealso
#' Use \code{data("qc_data")} to load and access the resulting quality control dataset.
#'
#' @source Bunis DG et al. (2021). Single-Cell Mapping of Progressive Fetal-to-Adult Transition in Human Naive T Cells Cell Rep. 34(1): 108573
#' @references Bunis DG et al. (2021). Single-Cell Mapping of Progressive Fetal-to-Adult Transition in Human Naive T Cells Cell Rep. 34(1): 108573
#'
#' @examples
#' # Load and explore the quality control dataset
#' data("qc_data")
#'
#' @keywords internal
#'
"qc_data"

#' @title Generate Paired Colors for Cell Types
#'
#' @description
#' This function assigns paired colors (light and dark) to a list of cell type names. The colors are selected from various color
#' palettes in the `pals` package.
#'
#' @details
#' The function uses color palettes from the `pals` package to generate colors or pairs of colors (light and dark) for each cell
#' type name provided. It cycles through different color families (blues, greens, reds, oranges, purples, purd and greys) to create
#' the colors
#'
#' @param cell_type_names A character vector of cell type names that need to be assigned colors.
#' @param paired If TRUE, the colored returned should be paired. Default is FALSE.
#'
#' @keywords internal
#'
#' @return A named character vector where the names are the original cell type names, and the values are the assigned colors.
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
# Function to generate paired colors
generateColors <- function(cell_type_names, paired = FALSE){

    if (isTRUE(paired)) {

        if(length(cell_type_names) > 20)
            stop("There can be at most 10 cell types for plot function.")

        full_colors <- c(c("#BDD7E7", "#3182BD"),
                         c("#FCAE91", "#DE2D26"),
                         c("#BAE4B3", "#31A354"),
                         c("#FED766", "#E68E00"),
                         c("#CBC9E2", "#756BB1"),
                         c("#D7B5D8", "#DD1C77"),
                         c("#80e0d0", "#20b2aa"),
                         c("#D2B48C", "#8B4513"),
                         c("#FFD700", "#B8860B"),
                         c("#B7CE95", "#6B8E23"))[seq_len(
                             length(cell_type_names))]
        names(full_colors) <- cell_type_names

    } else if (isFALSE(paired)){

        if(length(cell_type_names) > 10)
            stop("There can be at most 10 cell types for plot function.")

        full_colors <- c("#74C476",
                         "#FB6A4A",
                         "#6BAED6",
                         "#FDB32C",
                         "#9E9AC8",
                         "#DF65B0",
                         "#50C9BD",
                         "#AF7D50",
                         "#FFC400",
                         "#A2B964")[seq_len(length(cell_type_names))]
        names(full_colors) <- cell_type_names
    }
    return(full_colors)
}

#' @title Calculate Sliced Inverse Regression (SIR) Space for Different Cell Types
#'
#' @description
#' This function calculates the SIR space projections for different cell types in the query and reference datasets.
#'
#' @details
#' The function projects the query dataset onto the SIR space of the reference dataset based on shared cell types.
#' It computes conditional means for the reference dataset, extracts the SVD components, and performs the projection
#' of both the query and reference data. It uses the `projectSIR` function to perform the actual projection and
#' allows the user to specify particular cell types for analysis.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing the numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing the numeric expression matrix for the reference cells.
#' @param query_cell_type_col A character string specifying the column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col A character string specifying the column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param cell_types A character vector specifying the cell types to include in the analysis. If NULL, all common cell types between the query and reference data will be used.
#' @param multiple_cond_means Logical. Whether to compute conditional means for multiple conditions in the reference dataset. Default is TRUE.
#' @param cumulative_variance_threshold A numeric value specifying the cumulative variance threshold for selecting principal components. Default is 0.7.
#' @param n_neighbor A numeric value specifying the number of neighbors for computing the SIR space. Default is 1.
#' @param assay_name A character string specifying the name of the assay on which to perform computations. Default is "logcounts".
#' @param max_cells_ref Maximum number of reference cells to retain after cell type filtering. If NULL,
#' no downsampling of reference cells is performed. Default is 5000.
#' @param max_cells_query Maximum number of query cells to retain after cell type filtering. If NULL,
#' no downsampling of query cells is performed. Default is 5000.
#'
#' @return A list containing the SIR projections, rotation matrix, and percentage of variance explained for the given cell types.
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{plot.calculateSIRSpaceObject}}
#'
#' @examples
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Compute important variables for all pairwise cell comparisons
#' sir_output <- calculateSIRSpace(reference_data = reference_data,
#'                                 query_data = query_data,
#'                                 query_cell_type_col = "expert_annotation",
#'                                 ref_cell_type_col = "expert_annotation",
#'                                 multiple_cond_means = TRUE,
#'                                 cumulative_variance_threshold = 0.9,
#'                                 n_neighbor = 1)
#'
#' # Generate plots SIR projections
#' plot(sir_output,
#'      sir_subset = 1:5,
#'      cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
#'      lower_facet = "scatter",
#'      diagonal_facet = "boxplot",
#'      upper_facet = "blank")
#'
#' # Plot top loadings
#' plot(sir_output,
#'      sir_subset = 1:5,
#'      plot_type = "loadings",
#'      n_top = 10)
#'
# Function to plot cell types in SIR space
calculateSIRSpace <- function(query_data,
                              reference_data,
                              query_cell_type_col,
                              ref_cell_type_col,
                              cell_types = NULL,
                              multiple_cond_means = TRUE,
                              cumulative_variance_threshold = 0.7,
                              n_neighbor = 1,
                              assay_name = "logcounts",
                              max_cells_query = 5000,
                              max_cells_ref = 5000){

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  assay_name = assay_name,
                  max_cells_query = max_cells_query,
                  max_cells_ref = max_cells_ref)

    # Check if cumulative_variance_threshold is between 0 and 1
    if (!is.numeric(cumulative_variance_threshold) ||
        cumulative_variance_threshold < 0 || cumulative_variance_threshold > 1) {
        stop("cumulative_variance_threshold must be a numeric value between 0 and 1.")
    }

    # Check if n_neighbor is a positive integer
    if (!is.numeric(n_neighbor) || n_neighbor <= 0 ||
        n_neighbor != as.integer(n_neighbor)) {
        stop("n_neighbor must be a positive integer.")
    }

    # Select cell types
    cell_types <- selectCellTypes(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = query_cell_type_col,
                                  ref_cell_type_col = ref_cell_type_col,
                                  cell_types = cell_types,
                                  dual_only = FALSE,
                                  n_cell_types = NULL)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Downsample query and reference data (with cell type filtering)
    query_data <- downsampleSCE(sce_object = query_data,
                                cell_types = cell_types,
                                cell_type_col = query_cell_type_col,
                                max_cells = max_cells_query)
    reference_data <- downsampleSCE(sce_object = reference_data,
                                    cell_types = cell_types,
                                    cell_type_col = ref_cell_type_col,
                                    max_cells = max_cells_ref)

    # Get the projected PCA data
    sir_output <- projectSIR(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = query_cell_type_col,
                             ref_cell_type_col = ref_cell_type_col,
                             cell_types = cell_types,
                             multiple_cond_means = multiple_cond_means,
                             assay_name = assay_name,
                             cumulative_variance_threshold = cumulative_variance_threshold,
                             n_neighbor = n_neighbor)

    # Return SIR projections output
    class(sir_output) <- c(class(sir_output), "calculateSIRSpaceObject")
    return(sir_output)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Compute important variables for all pairwise cell comparisons
sir_output <- calculateSIRSpace(reference_data = reference_data,
                                query_data = query_data,
                                query_cell_type_col = "SingleR_annotation",
                                ref_cell_type_col = "expert_annotation",
                                multiple_cond_means = TRUE,
                                cumulative_variance_threshold = 0.9,
                                n_neighbor = 1)

test_that("plot.calculateSIRSpaceObject generates scores plots correctly", {
    # Generate scores plot using the function (default plot_type)
    p1 <- plot(sir_output, plot_type = "scores", sir_subset = 1:3)

    # Check if output is a ggmatrix object (from GGally::ggpairs)
    expect_s3_class(p1, "ggmatrix")
})

test_that("plot.calculateSIRSpaceObject generates loadings plots correctly", {
    # Generate loadings plot using the function
    p1 <- plot(sir_output, plot_type = "loadings", sir_subset = 1:3, n_top = 5)

    # Check if output is a ggplot object
    expect_s3_class(p1, "ggplot")
})

test_that("plot.calculateSIRSpaceObject works with different facet options", {
    # Test different facet combinations for scores plot
    p1 <- plot(sir_output,
               plot_type = "scores",
               sir_subset = 1:2,
               lower_facet = "scatter",
               diagonal_facet = "density",
               upper_facet = "blank")

    expect_s3_class(p1, "ggmatrix")

    # Test with different options
    p2 <- plot(sir_output,
               plot_type = "scores",
               sir_subset = 1:2,
               lower_facet = "contour",
               diagonal_facet = "boxplot",
               upper_facet = "ellipse")

    expect_s3_class(p2, "ggmatrix")
})

test_that("plot.calculateSIRSpaceObject works with specific cell types", {
    # Test with specific cell types
    available_cell_types <- unique(sir_output$sir_projections$cell_type)
    selected_cell_types <- available_cell_types[1:min(2, length(available_cell_types))]

    p1 <- plot(sir_output,
               plot_type = "scores",
               cell_types = selected_cell_types,
               sir_subset = 1:2)

    expect_s3_class(p1, "ggmatrix")
})

test_that("plot.calculateSIRSpaceObject parameter validation works", {
    # Test invalid n_top for loadings
    expect_error(plot(sir_output, plot_type = "loadings", n_top = -1),
                 "n_top must be a positive integer")

    # Test invalid sir_subset (assuming there are fewer SIR components available)
    max_sir <- ncol(sir_output$rotation_mat)
    expect_error(plot(sir_output, sir_subset = 1:(max_sir + 5)),
                 "sir_subset contains values outside the range")
})

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

test_that("calculateSIRSpace returns correct structure", {
    # Run the calculateSIRSpace function
    result <- calculateSIRSpace(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation"
    )

    # Check that result is a list and contains expected components
    expect_type(result, "list")
    expect_named(result, c("cond_means", "rotation_mat", "sir_projections", "percent_var"))

    # Check that the class includes "calculateSIRSpace"
    expect_s3_class(result, "calculateSIRSpaceObject")
})

test_that("calculateSIRSpace uses the correct cumulative variance threshold", {
    # Run the function with a specific cumulative variance threshold
    result <- calculateSIRSpace(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        cumulative_variance_threshold = 0.9
    )

    # Ensure the cumulative variance threshold is respected
    expect_true(sum(result$percent_var) >= 0.9)
})

test_that("calculateSIRSpace works with specific cell types", {
    # Test with a specified subset of cell types
    cell_types <- c("CD4", "CD8")

    result <- calculateSIRSpace(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        cell_types = cell_types
    )

    # Check that the conditional means are computed only for the specified cell types
    expect_true(all(rownames(result$cond_means) %in% cell_types))
})

test_that("calculateSIRSpace handles multiple conditional means", {
    # Test with multiple_cond_means set to FALSE
    result <- calculateSIRSpace(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        multiple_cond_means = FALSE
    )

    # Ensure that the conditional means are calculated based on single condition
    expect_true(is.matrix(result$cond_means)) # Expect conditional means to be a matrix
})

test_that("calculateSIRSpace handles n_neighbor parameter", {
    # Test with n_neighbor = 3
    result <- calculateSIRSpace(
        query_data = query_data,
        reference_data = reference_data,
        query_cell_type_col = "SingleR_annotation",
        ref_cell_type_col = "expert_annotation",
        n_neighbor = 3
    )

    # No direct way to test n_neighbor impact, but ensure result structure is correct
    expect_type(result$sir_projections, "list")
})

test_that("calculateSIRSpace throws error for missing arguments", {
    # Test if function throws an error for missing required arguments
    expect_error(calculateSIRSpace(query_data = query_data), "argument \"reference_data\" is missing")
    expect_error(calculateSIRSpace(reference_data = reference_data), "argument \"query_data\" is missing")
    expect_error(calculateSIRSpace(query_data = query_data, reference_data = reference_data), "argument \"query_cell_type_col\" is missing")
})

#' @title Compare Principal Components Analysis (PCA) Results
#'
#' @description This function compares the principal components (PCs) obtained from separate PCA on reference and query
#' datasets for a single cell type using either cosine similarity or correlation.
#'
#' @details
#' This function compares the PCA results between the reference and query datasets by computing cosine
#' similarities or correlations between the loadings of top variables for each pair of principal components. It first
#' extracts the PCA rotation matrices from both datasets and identifies the top variables with highest loadings for
#' each PC. Then, it computes the cosine similarities or correlations between the loadings of top variables for each
#' pair of PCs using vectorized operations for improved performance. The resulting matrix contains the similarity values,
#' where rows represent reference PCs and columns represent query PCs.
#'
#' @param query_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
#' @param reference_data A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
#' @param query_cell_type_col The column name in the \code{colData} of \code{query_data} that identifies the cell types.
#' @param ref_cell_type_col The column name in the \code{colData} of \code{reference_data} that identifies the cell types.
#' @param pc_subset A numeric vector specifying the subset of principal components (PCs) to compare. Default is the first five PCs.
#' @param n_top_vars An integer indicating the number of top loading variables to consider for each PC. Default is 50.
#' @param metric The similarity metric to use. It can be either "cosine" or "correlation". Default is "cosine".
#' @param correlation_method The correlation method to use if metric is "correlation". It can be "spearman"
#' or "pearson". Default is "spearman".
#' @param n_permutations Number of permutations for statistical significance testing. If 0, no permutation test is performed. Default is 0.
#'
#' @return A list containing:
#'   \item{similarity_matrix}{A matrix comparing the principal components of the reference and query datasets.}
#'   \item{top_variables}{A list containing the top loading variables for each PC pair comparison.}
#'   \item{p_values}{A matrix of permutation p-values (if n_permutations > 0).}
#'   \item{metric}{The similarity metric used.}
#'   \item{n_top_vars}{Number of top variables used.}
#'
#' @export
#'
#' @author Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
#'
#' @seealso \code{\link{plot.comparePCAObject}}
#'
#' @examples
#' # Load libraries
#' library(scran)
#' library(scater)
#'
#' # Load data
#' data("reference_data")
#' data("query_data")
#'
#' # Extract CD4 cells
#' ref_data_subset <- reference_data[, which(reference_data$expert_annotation == "CD4")]
#' query_data_subset <- query_data[, which(query_data$expert_annotation == "CD4")]
#'
#' # Selecting highly variable genes (can be customized by the user)
#' ref_top_genes <- getTopHVGs(ref_data_subset, n = 500)
#' query_top_genes <- getTopHVGs(query_data_subset, n = 500)
#'
#' # Intersect the gene symbols to obtain common genes
#' common_genes <- intersect(ref_top_genes, query_top_genes)
#' ref_data_subset <- ref_data_subset[common_genes,]
#' query_data_subset <- query_data_subset[common_genes,]
#'
#' # Run PCA on datasets separately
#' ref_data_subset <- runPCA(ref_data_subset)
#' query_data_subset <- runPCA(query_data_subset)
#'
#' # Call the PCA comparison function
#' similarity_mat <- comparePCA(query_data = query_data_subset,
#'                              reference_data = ref_data_subset,
#'                              query_cell_type_col = "expert_annotation",
#'                              ref_cell_type_col = "expert_annotation",
#'                              pc_subset = 1:5,
#'                              n_top_vars = 50,
#'                              metric = c("cosine", "correlation")[1],
#'                              correlation_method = c("spearman", "pearson")[1],
#'                              n_permutation = 100)
#'
#' # Create the heatmap
#' plot(similarity_mat, show_significance = TRUE)
#'
# Function to compare PCA of the SCE objects
comparePCA <- function(query_data,
                       reference_data,
                       query_cell_type_col,
                       ref_cell_type_col,
                       pc_subset = 1:5,
                       n_top_vars = 50,
                       metric = c("cosine", "correlation"),
                       correlation_method = c("spearman", "pearson"),
                       n_permutations = 0) {

    # Check standard input arguments
    argumentCheck(query_data = query_data,
                  reference_data = reference_data,
                  query_cell_type_col = query_cell_type_col,
                  ref_cell_type_col = ref_cell_type_col,
                  unique_cell_type = TRUE,
                  pc_subset_query = pc_subset,
                  pc_subset_ref = pc_subset,
                  common_rotation_genes = TRUE)

    # Convert cell type columns to character if needed
    query_data <- convertColumnsToCharacter(sce_object = query_data,
                                            convert_cols = query_cell_type_col)
    reference_data <- convertColumnsToCharacter(sce_object = reference_data,
                                                convert_cols = ref_cell_type_col)

    # Match arguments
    metric <- match.arg(metric)
    correlation_method <- match.arg(correlation_method)

    # Enhanced input validation
    if (!is.numeric(n_top_vars) || n_top_vars <= 0 ||
        n_top_vars != as.integer(n_top_vars)) {
        stop("'n_top_vars' must be a positive integer.")
    }

    if (!is.numeric(n_permutations) || n_permutations < 0 ||
        n_permutations != as.integer(n_permutations)) {
        stop("'n_permutations' must be a non-negative integer.")
    }

    # Extract PCA data from reference and query data
    ref_rotation <-
        attributes(reducedDim(reference_data, "PCA"))[["rotation"]][, pc_subset]
    query_rotation <-
        attributes(reducedDim(query_data, "PCA"))[["rotation"]][, pc_subset]
    query_rotation <-
        query_rotation[match(rownames(ref_rotation), rownames(query_rotation)), ]

    # Check if n_top_vars is reasonable given the number of genes
    n_genes <- nrow(ref_rotation)
    if (n_top_vars > n_genes) {
        warning(sprintf("n_top_vars (%d) is greater than the number of genes (%d). Using all genes.",
                        n_top_vars, n_genes))
        n_top_vars <- n_genes
    }

    # Function to identify high-loading variables for each PC (vectorized)
    .getHighLoadingVars <- function(rotation_mat, n_top_vars) {
        abs_loadings <- abs(rotation_mat)
        high_loading_vars <- apply(abs_loadings, 2, function(pc_loadings) {
            names(sort(pc_loadings, decreasing = TRUE))[seq_len(n_top_vars)]
        })
        return(high_loading_vars)
    }

    # Get top variables for each PC
    top_ref <- .getHighLoadingVars(ref_rotation, n_top_vars)
    top_query <- .getHighLoadingVars(query_rotation, n_top_vars)

    # Store top variables information for each comparison
    top_variables_info <- list()

    # Initialize similarity matrix
    similarity_matrix <- matrix(NA,
                                nrow = length(pc_subset),
                                ncol = length(pc_subset))

    # Vectorized similarity computation function
    .computeSimilarityMatrix <- function(ref_rot, query_rot, top_ref, top_query,
                                         metric, correlation_method) {
        sim_mat <- matrix(NA, nrow = ncol(ref_rot), ncol = ncol(query_rot))
        top_vars_list <- list()

        for (i in seq_len(ncol(ref_rot))) {
            for (j in seq_len(ncol(query_rot))) {
                # Get union of top variables for this PC pair
                combination_union <- union(top_ref[, i], top_query[, j])
                top_vars_list[[paste(i, j, sep = "_")]] <- combination_union

                # Extract relevant loadings
                ref_loadings <- ref_rot[combination_union, i]
                query_loadings <- query_rot[combination_union, j]

                # Compute similarity based on metric
                if (metric == "cosine") {
                    # Vectorized cosine similarity
                    dot_product <- sum(ref_loadings * query_loadings)
                    norm_ref <- sqrt(sum(ref_loadings^2))
                    norm_query <- sqrt(sum(query_loadings^2))
                    sim_mat[i, j] <- dot_product / (norm_ref * norm_query)
                } else if (metric == "correlation") {
                    sim_mat[i, j] <- cor(ref_loadings, query_loadings,
                                         method = correlation_method)
                }
            }
        }

        return(list(similarity_matrix = sim_mat, top_variables = top_vars_list))
    }

    # Compute similarity matrix
    result <- .computeSimilarityMatrix(ref_rotation, query_rotation,
                                       top_ref, top_query,
                                       metric, correlation_method)
    similarity_matrix <- result[["similarity_matrix"]]
    top_variables_info <- result[["top_variables"]]

    # Perform permutation test if requested
    p_values <- NULL
    if (n_permutations > 0) {
        message(sprintf("Performing %d permutations for significance testing...",
                        n_permutations))

        # Store original similarities for comparison
        original_similarities <- similarity_matrix
        permuted_similarities <- array(NA, dim = c(nrow(similarity_matrix),
                                                   ncol(similarity_matrix),
                                                   n_permutations))

        for (perm in seq_len(n_permutations)) {
            # Permute gene labels
            permuted_genes <- sample(rownames(ref_rotation))
            ref_rotation_perm <- ref_rotation
            rownames(ref_rotation_perm) <- permuted_genes

            # Recompute similarities with permuted data
            perm_result <- .computeSimilarityMatrix(ref_rotation_perm, query_rotation,
                                                    top_ref, top_query,
                                                    metric, correlation_method)
            permuted_similarities[, , perm] <- perm_result[["similarity_matrix"]]
        }

        # Calculate p-values
        p_values <- matrix(NA, nrow = nrow(similarity_matrix),
                           ncol = ncol(similarity_matrix))
        for (i in seq_len(nrow(similarity_matrix))) {
            for (j in seq_len(ncol(similarity_matrix))) {
                observed_sim <- abs(original_similarities[i, j])
                permuted_sims <- abs(permuted_similarities[i, j, ])
                p_values[i, j] <- (sum(permuted_sims >= observed_sim) + 1) /
                    (n_permutations + 1)
            }
        }
    }

    # Add informative row and column names
    ref_var_explained <-
        attributes(reducedDim(reference_data, "PCA"))[["percentVar"]][pc_subset]
    query_var_explained <-
        attributes(reducedDim(query_data, "PCA"))[["percentVar"]][pc_subset]

    rownames(similarity_matrix) <-
        paste0("Ref PC", pc_subset, " (", round(ref_var_explained, 1), "%)")
    colnames(similarity_matrix) <-
        paste0("Query PC", pc_subset, " (", round(query_var_explained, 1), "%)")

    if (!is.null(p_values)) {
        rownames(p_values) <- rownames(similarity_matrix)
        colnames(p_values) <- colnames(similarity_matrix)
    }

    # Create comprehensive output object
    output <- list(
        similarity_matrix = similarity_matrix,
        top_variables = top_variables_info,
        p_values = p_values,
        metric = metric,
        correlation_method = if(metric == "correlation") correlation_method else NULL,
        n_top_vars = n_top_vars,
        n_permutations = n_permutations,
        pc_subset = pc_subset
    )

    # Update class
    class(output) <- c("comparePCAObject", class(output))

    return(output)
}

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Extract CD4 cells
reference_data_subset <- reference_data[, which(reference_data$expert_annotation == "CD4")]
query_data_subset <- query_data[, which(query_data$expert_annotation == "CD4")]

# Selecting highly variable genes (can be customized by the user)
ref_top_genes <- scran::getTopHVGs(reference_data_subset, n = 500)
query_top_genes <- scran::getTopHVGs(query_data_subset, n = 500)

# Intersect the gene symbols to obtain common genes
common_genes <- intersect(ref_top_genes, query_top_genes)
reference_data_subset <- reference_data_subset[common_genes,]
query_data_subset <- query_data_subset[common_genes,]

# Run PCA on datasets separately
reference_data_subset <- scater::runPCA(reference_data_subset)
query_data_subset <- scater::runPCA(query_data_subset)

# Unit tests for comparePCA
test_that("comparePCA returns expected output structure", {
    # Run the comparePCA function
    result <- comparePCA(query_data = query_data_subset,
                         reference_data = reference_data_subset,
                         query_cell_type_col = "expert_annotation",
                         ref_cell_type_col = "expert_annotation",
                         pc_subset = 1:5,
                         n_top_vars = 50,
                         metric = "cosine",
                         correlation_method = "spearman")

    # Check the class of the output
    expect_s3_class(result, "comparePCAObject")

    # Check the dimensions of the similarity matrix
    expect_equal(dim(result$similarity_matrix), c(5, 5))

    # Check that the matrix contains numeric values
    expect_type(result$similarity_matrix, "double")
})

test_that("comparePCA handles invalid n_top_vars argument", {
    # Check if function stops with invalid n_top_vars
    expect_error(comparePCA(query_data = query_data_subset,
                            reference_data = reference_data_subset,
                            query_cell_type_col = "expert_annotation",
                            ref_cell_type_col = "expert_annotation",
                            pc_subset = 1:5,
                            n_top_vars = -10,
                            metric = "cosine",
                            correlation_method = "spearman"))
})

test_that("comparePCA handles invalid metric argument", {
    # Check if function stops with invalid metric
    expect_error(comparePCA(query_data = query_data_subset,
                            reference_data = reference_data_subset,
                            query_cell_type_col = "expert_annotation",
                            ref_cell_type_col = "expert_annotation",
                            pc_subset = 1:5,
                            n_top_vars = 50,
                            metric = "invalid_metric",
                            correlation_method = "spearman"))
})

test_that("comparePCA handles invalid correlation_method argument", {
    # Check if function stops with invalid correlation method
    expect_error(comparePCA(query_data = query_data_subset,
                            reference_data = reference_data_subset,
                            query_cell_type_col = "expert_annotation",
                            ref_cell_type_col = "expert_annotation",
                            pc_subset = 1:5,
                            n_top_vars = 50,
                            metric = "correlation",
                            correlation_method = "invalid_method"))
})

test_that("comparePCA works with correlation metric", {
    # Run the comparePCA function with correlation metric
    result <- comparePCA(query_data = query_data_subset,
                         reference_data = reference_data_subset,
                         query_cell_type_col = "expert_annotation",
                         ref_cell_type_col = "expert_annotation",
                         pc_subset = 1:5,
                         n_top_vars = 50,
                         metric = "correlation",
                         correlation_method = "spearman")

    # Check the class of the output
    expect_s3_class(result, "comparePCAObject")

    # Check the dimensions of the similarity matrix
    expect_equal(dim(result$similarity_matrix), c(5, 5))

    # Check that the matrix contains numeric values
    expect_type(result$similarity_matrix, "double")
})

test_that("comparePCA works with different correlation methods", {
    # Run the comparePCA function with pearson correlation
    result <- comparePCA(query_data = query_data_subset,
                         reference_data = reference_data_subset,
                         query_cell_type_col = "expert_annotation",
                         ref_cell_type_col = "expert_annotation",
                         pc_subset = 1:5,
                         n_top_vars = 50,
                         metric = "correlation",
                         correlation_method = "pearson")

    # Check the class of the output
    expect_s3_class(result, "comparePCAObject")

    # Check the dimensions of the similarity matrix
    expect_equal(dim(result$similarity_matrix), c(5, 5))

    # Check that the matrix contains numeric values
    expect_type(result$similarity_matrix, "double")
})

test_that("comparePCA produces expected results", {
    # Run the comparePCA function with correlation similarity
    result <- comparePCA(query_data = query_data_subset,
                         reference_data = reference_data_subset,
                         query_cell_type_col = "expert_annotation",
                         ref_cell_type_col = "expert_annotation",
                         pc_subset = 1:5,
                         n_top_vars = 50,
                         metric = "correlation",
                         correlation_method = "spearman")

    # Check if the correlation values are in the range [-1, 1]
    expect_true(all(result$similarity_matrix >= -1 & result$similarity_matrix <= 1))
})

# Load necessary libraries
library(testthat)
library(scDiagnostics)

# Load example datasets
data("reference_data")
data("query_data")

# Extract CD4 cells
reference_data_subset <- reference_data[, which(reference_data$expert_annotation == "CD4")]
query_data_subset <- query_data[, which(query_data$expert_annotation == "CD4")]

# Selecting highly variable genes (can be customized by the user)
ref_top_genes <- scran::getTopHVGs(reference_data_subset, n = 500)
query_top_genes <- scran::getTopHVGs(query_data_subset, n = 500)

# Intersect the gene symbols to obtain common genes
common_genes <- intersect(ref_top_genes, query_top_genes)
reference_data_subset <- reference_data_subset[common_genes,]
query_data_subset <- query_data_subset[common_genes,]

# Run PCA on datasets separately
reference_data_subset <- scater::runPCA(reference_data_subset)
query_data_subset <- scater::runPCA(query_data_subset)

# Call the PCA comparison function
similarity_result <- comparePCA(reference_data = reference_data_subset,
                                query_data = query_data_subset,
                                query_cell_type_col = "expert_annotation",
                                ref_cell_type_col = "expert_annotation",
                                pc_subset = 1:5,
                                n_top_vars = 50,
                                metric = c("cosine", "correlation")[1],
                                correlation_method = c("spearman", "pearson")[1])

test_that("plot.comparePCAObject generates plots correctly", {
    # Generate plot using the function
    p1 <- plot(similarity_result)

    # Check if output is a ggplot object
    expect_s3_class(p1, "ggplot")
})

test_that("plot.comparePCAObject works with different parameters", {
    # Test with show_values = FALSE
    p1 <- plot(similarity_result, show_values = FALSE)
    expect_s3_class(p1, "ggplot")

    # Test with show_significance = FALSE
    p2 <- plot(similarity_result, show_significance = FALSE)
    expect_s3_class(p2, "ggplot")

    # Test with custom color limits
    p3 <- plot(similarity_result, color_limits = c(-1, 1))
    expect_s3_class(p3, "ggplot")
})

test_that("plot.comparePCAObject works with correlation metric", {
    # Create result with correlation metric
    similarity_result_corr <- comparePCA(reference_data = reference_data_subset,
                                         query_data = query_data_subset,
                                         query_cell_type_col = "expert_annotation",
                                         ref_cell_type_col = "expert_annotation",
                                         pc_subset = 1:5,
                                         n_top_vars = 50,
                                         metric = "correlation",
                                         correlation_method = "spearman")

    # Generate plot
    p1 <- plot(similarity_result_corr)
    expect_s3_class(p1, "ggplot")
})

---
title: "Detection and Analysis of Annotation Anomalies"
author:
    - name: Anthony Christidis
      affiliation:
        - &core_affiliation Core for Computational Biomedicine, Harvard Medical School
    - name: Andrew Ghazi
      affiliation:
        - *core_affiliation
    - name: Smriti Chawla
      affiliation:
        - *core_affiliation
    - name: Nitesh Turaga
      affiliation:
        - *core_affiliation
    - name: Ludwig Geistlinger
      affiliation:
        - *core_affiliation
    - name: Robert Gentleman
      affiliation:
        - *core_affiliation
package: scDiagnostics
output: 
  BiocStyle::html_document:
    toc: true
    toc_float: true
vignette: >
  %\VignetteIndexEntry{4. Detection and Analysis of Annotation Anomalies}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 72
---

```{r setup, include = FALSE, fig.show='hide'}
knitr::knit_hooks$set(pngquant = knitr::hook_pngquant)

knitr::opts_chunk$set(
collapse = TRUE,
comment = "#>",
dev = "ragg_png",
dpi = 72,
fig.retina = 2,
fig.align = "center",
out.width = "100%",
pngquant = "--speed=1 --quality=1-5"
)
```

# Introduction 

The `scDiagnostics` package provides powerful tools for anomaly detection in single-cell data, enabling researchers to identify and analyze outliers in complex datasets. Central to this process is the `detectAnomaly()` function, which integrates dimensionality reduction through Principal Component Analysis (PCA) with the robust capabilities of the isolation forest algorithm.

In single-cell analysis, detecting anomalies is crucial for identifying potential data issues, such as mislabeled cells, technical artifacts, or biologically distinct subpopulations. The `detectAnomaly()` function offers a versatile approach to anomaly detection by allowing users to project data onto a PCA space and apply isolation forests to uncover outliers. Whether working solely with a reference dataset or comparing a query dataset against a well-characterized reference, this function provides detailed insights into potential anomalies.

This vignette illustrates how to effectively use the `detectAnomaly()` function in various scenarios. We explore both cell-type-specific and global anomaly detection, demonstrate the utility of integrating query data with reference data, and offer guidance on interpreting the results. Additionally, we show how to extend this analysis by combining anomaly detection with PCA loadings using the `calculateCellSimilarityPCA()` function, providing a comprehensive toolkit for investigating the structure and quality of single-cell data.

This vignette also demonstrates the use of two functions, `calculateCellDistances()` and `calculateCellDistancesSimilarity()`, to analyze the distances between cells in query and reference datasets and to measure the similarity of density distributions. These functions are useful for investigating how cells in different datasets relate to each other, particularly in the context of identifying anomalies and understanding distribution overlaps.

# Preliminaries

In the context of the `scDiagnostics` package, the following datasets illustrate the application of these tools:

- `reference_data`: A curated and processed dataset containing expert-assigned cell type annotations. This dataset serves as a reference for comparison and can be used alone to detect anomalies within the reference annotations.

- `query_data`: A dataset that also includes expert-assigned cell type annotations, but additionally features annotations generated by the `r BiocStyle::Biocpkg("SingleR")` package. This allows for the comparison of expert annotations with those produced by an automated method to detect inconsistencies or anomalies.

```{r, message=FALSE, fig.show='hide'}
# Load library
library(scDiagnostics)

# Load datasets
data("reference_data")
data("query_data")

# Set seed for reproducibility
set.seed(0)
```


By using these datasets, you can leverage the package's tools to compare annotations between `reference_data` and `query_data`, or analyze `reference_data` alone to identify potential issues. The package’s flexibility supports various analysis scenarios, whether you need to assess overall annotation quality or focus on specific cell types.

Through these capabilities, `scDiagnostics` empowers you to perform thorough and nuanced assessments of cell type annotations, enhancing the accuracy and reliability of your analyses.


# The `detectAnomaly()` Function 

## Function Overview  

### Description

The `detectAnomaly()` function integrates dimensionality reduction via PCA with the isolation forest algorithm to detect anomalies in single-cell data. By projecting both reference and query datasets (if available) onto the PCA space of the reference data, the function trains an isolation forest model on reference data in PCA space to pinpoint anomalies in the reference or query data. This approach is highly versatile:

- **Reference Only**: Compute anomaly scores solely for the reference dataset to identify potential issues within the reference itself.
- **Reference and Query**: Compare the query dataset against the reference to find anomalies in the query data that may not align with the established reference.
- **Global and Specific Analysis**: Assess anomalies at a global level or focus on specific cell types to gain targeted insights into your data.
The function also provides detailed visualizations and statistical outputs to help you interpret the anomalies detected.

### Parameters

The function takes a `r BiocStyle::Biocpkg("SingleCellExperiment")` object as `reference_data` and trains an isolation forest model on the reference PCA-projected data, with an optional `query_data` for projecting onto this PCA space for anomaly detection. You can specify cell type annotations through `ref_cell_type_col` and `query_cell_type_col`, and limit the analysis to certain cell types using the `cell_types` parameter. The function allows you to select specific principal components to use to train the isolation forest via `pc_subset`, adjust the number of trees with `n_tree`, and set an `anomaly_threshold` for classifying anomalies.


### Return Value

The function returns several outputs: `anomaly_scores` indicating the likelihood of each cell being an anomaly, a logical vector (`anomaly`) identifying these anomalies, PCA projections for the reference data (`reference_mat_subset`) and optionally for the query data (`query_mat_subset`), and the proportion of variance explained by the selected principal components (`var_explained`).



## `detectAnomaly()` Examples

### Anomaly Detection with Reference and Query Data

This section demonstrates how to use the `detectAnomaly()` function when both reference and query datasets are provided. It includes examples of analyzing anomalies for specific cell types and globally across all data.


#### Example 1: Cell-Type Specific Anomaly Detection
In this example, we analyze anomalies specifically for the "CD4" cell type. The anomaly scores are trained on the PCA projections of the "CD4" cells from the reference dataset. If query data is provided, anomaly scores for the query data are predicted based on the PCA projections of the query data onto the reference PCA space for the "CD4" cell type.




```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Perform anomaly detection
anomaly_output <- detectAnomaly(reference_data = reference_data, 
                                query_data = query_data, 
                                ref_cell_type_col = "expert_annotation", 
                                query_cell_type_col = "SingleR_annotation",
                                pc_subset = 1:5,
                                n_tree = 500,
                                anomaly_threshold = 0.6)

# Plot the output for the "CD4" cell type
plot(anomaly_output, 
     cell_type = "CD8", 
     pc_subset = 1:5, 
     data_type = "query")
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/AnnotationAnomalies/detectAnomaly1.png)

In this example, we analyze anomalies specifically for the "CD4" cell type. The anomaly scores are trained on the PCA projections of the "CD4" cells from the reference dataset. If query data is provided, anomaly scores for the query data are predicted based on the PCA projections of the query data onto the reference PCA space for the "CD4" cell type.


#### Example 2: Global Anomaly Detection
Here, we perform global anomaly detection by setting `cell_type = NULL`. In this case, the isolation forest is trained on PCA projections of all cells in the reference data combined. The global anomaly scores are then computed for both reference and query datasets.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Perform anomaly detection
anomaly_output <- detectAnomaly(reference_data = reference_data, 
                                query_data = query_data, 
                                ref_cell_type_col = "expert_annotation", 
                                query_cell_type_col = "SingleR_annotation",
                                pc_subset = 1:5,
                                n_tree = 500,
                                anomaly_threshold = 0.6)

# Plot the global anomaly scores
plot(anomaly_output, 
     cell_type = NULL,  # Plot all cell types
     pc_subset = 1:5, 
     data_type = "query")
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/AnnotationAnomalies/detectAnomaly2.png)

Setting `cell_type = NULL` means that the anomaly detection is done globally. The isolation forest is trained on PCA projections of all cells from the reference dataset. Anomaly scores are then predicted for the query data based on these global PCA projections. The plot provides a comprehensive view of anomalies across all cell types in the query dataset.

#### Anomaly Detection on Reference Data

Note that if you do not provide a query dataset to the `detectAnomaly()` function, the function will only return the anomaly output data on the reference data.


# Integrating Anomaly Detection with Cell Similarity Analysis Using PCA Loadings

The `detectAnomaly()` function identifies anomalous cells in a dataset by detecting outliers based on PCA results. Once anomalies are detected, `calculateCellSimilarityPCA()` evaluates how these outliers influence principal component directions. This analysis helps determine if anomalous cells significantly impact later PCs, which capture finer variations in the data. By combining these functions, you can both pinpoint anomalies and understand their effect on PCA directions, providing deeper insights into the data structure.
```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Detect anomalies and select top anomalies for further analysis
anomaly_output <- detectAnomaly(reference_data = reference_data, 
                                query_data = query_data,
                                ref_cell_type_col = "expert_annotation", 
                                query_cell_type_col = "SingleR_annotation",
                                pc_subset = 1:10,
                                n_tree = 500,
                                anomaly_threshold = 0.5)

# Select the top 6 anomalies based on the anomaly scores
top6_anomalies <- names(sort(anomaly_output$Combined$reference_anomaly_scores, 
                             decreasing = TRUE)[1:6])

# Calculate cosine similarity between the top anomalies and top 25 PCs
cosine_similarities <- calculateCellSimilarityPCA(reference_data, 
                                                  cell_names = top6_anomalies, 
                                                  pc_subset = 1:25, 
                                                  n_top_vars = 50)

# Plot the cosine similarities across PCs
round(cosine_similarities[, paste0("PC", 15:25)], 2)
```

Note that there is also a plot method for the object return for `calculateCellSimilarityPCA()`. See reference manual for details.

# Analyzing Cell Distances 

## `calculateCellDistances()`

The `calculateCellDistances()` function calculates Euclidean distances between cells within the reference dataset and between query cells and reference cells. This helps in understanding how closely related cells are within and across datasets based on their principal component (PC) scores.


## Function Usage
```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Compute cell distances
distance_data <- calculateCellDistances(
    query_data = query_data,
    reference_data = reference_data,
    query_cell_type_col = "SingleR_annotation",
    ref_cell_type_col = "expert_annotation",
    pc_subset = 1:10
)
```

In the code above:

- `query_data` and reference_data: These are `r BiocStyle::Biocpkg("SingleCellExperiment")` objects containing the respective datasets for analysis.
- `query_cell_type_col` and ref_cell_type_col: These arguments specify the columns in the `colData` of each dataset that contain cell type annotations.
- `pc_subset`: Specifies which principal components (1 to 10) are used to compute distances. PCA is applied for dimensionality reduction before calculating distances.

## Output
The output `distance_data` is a list where each entry contains:

- `ref_distances`: Pairwise distances within the reference dataset for each cell type.
- `query_to_ref_distances`: Distances from each query cell to all reference cells for each cell type.


## Example Workflow

In the example workflow, the first step involves plotting the distributions of cell distances for selected query cells against various reference cell populations. This involves visualizing both the distances between the query cells and the reference cells as well as the distances among the reference cells themselves. For each cell type, such as CD4 and CD8, these plots illustrate how the selected query cells compare in terms of their proximity to different reference populations.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Identify top 6 anomalies for CD4 cells
cd4_anomalies <- detectAnomaly(
  reference_data = reference_data,
  query_data = query_data,
  query_cell_type_col = "SingleR_annotation",
  ref_cell_type_col = "expert_annotation",
  pc_subset = 1:10,
  n_tree = 500,
  anomaly_threshold = 0.5)

# Top 6 CD4 anomalies
cd4_top6_anomalies <- names(sort(cd4_anomalies$CD4$query_anomaly_scores, decreasing = TRUE)[1:6])

# Plot distances for CD4 and CD8 cells
plot(distance_data, ref_cell_type = "CD4", cell_names = cd4_top6_anomalies)
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/AnnotationAnomalies/calculateCellDistances1.png)

For CD4 cells, you might observe that the query cells are located much further away from the reference cells, indicating a significant divergence in their density distributions. This suggests that the CD4 query cells are less similar to the CD4 reference cells, showing greater variation or potential anomalies.


In contrast, for CD8 cells, the plots might reveal more overlap between the density distributions of query cells and reference cells. This overlap indicates that the CD8 query cells are more similar to the CD8 reference cells, suggesting better alignment or less divergence between their densities. Such insights are crucial for understanding how closely the query cells match with reference populations and can help in identifying cell types with notable differences or similarities.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Plot distances for CD8 cells
plot(distance_data, ref_cell_type = "CD8", cell_names = cd4_top6_anomalies)
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/AnnotationAnomalies/calculateCellDistances2.png)

## `calculateCellDistancesSimilarity()`

The `calculateCellDistancesSimilarity()` function measures how similar the density distributions of query cells are to those in the reference dataset. It computes **Bhattacharyya coefficients** and **Hellinger distances**, which quantify the overlap between the distributions.

The Bhattacharyya coefficient and Hellinger distance are two measures used to quantify the similarity between probability distributions. The Bhattacharyya coefficient ranges from 0 to 1, where a value closer to 1 indicates higher similarity between distributions, while a value closer to 0 suggests greater dissimilarity. The Hellinger distance, also ranging from 0 to 1, reflects the divergence between distributions, with lower values indicating greater similarity and higher values showing more distinct distributions. In the context of comparing query cells to reference cells, a high Bhattacharyya coefficient and a low Hellinger distance both suggest that the query cells have density profiles similar to those in the reference dataset, which can be desirable depending on the experimental objectives.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Compute similarity measures for top 6 CD4 anomalies
overlap_measures <- calculateCellDistancesSimilarity(
  query_data = query_data,
  reference_data = reference_data,
  cell_names = cd4_top6_anomalies,
  query_cell_type_col = "SingleR_annotation",
  ref_cell_type_col = "expert_annotation",
  pc_subset = 1:10)
overlap_measures
```


------------------------------------------------------------------------

# R Session Info

```{r SessionInfo, echo=FALSE, message=FALSE, warning=FALSE, comment=NA, fig.show='hide'}
options(width = 80) 
sessionInfo()
```

---
title: "Visualization of Cell Type Annotations"
author:
    - name: Anthony Christidis
      affiliation:
        - &core_affiliation Core for Computational Biomedicine, Harvard Medical School
    - name: Andrew Ghazi
      affiliation:
        - *core_affiliation
    - name: Smriti Chawla
      affiliation:
        - *core_affiliation
    - name: Nitesh Turaga
      affiliation:
        - *core_affiliation
    - name: Ludwig Geistlinger
      affiliation:
        - *core_affiliation
    - name: Robert Gentleman
      affiliation:
        - *core_affiliation
package: scDiagnostics
output: 
  BiocStyle::html_document:
    toc: true
    toc_float: true
vignette: >
  %\VignetteIndexEntry{2. Visualization of Cell Type Annotations}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 72
---

```{r setup, include = FALSE, fig.show='hide'}
knitr::knit_hooks$set(pngquant = knitr::hook_pngquant)

knitr::opts_chunk$set(
collapse = TRUE,
comment = "#>",
dev = "ragg_png",
dpi = 72,
fig.retina = 2,
fig.align = "center",
out.width = "100%",
pngquant = "--speed=1 --quality=5-10"
)
```

# Introduction 

This vignette demonstrates the use `plotCellTypeMDS()`, `plotCellTypePCA()`, `boxplotPCA()` and `calculateDiscriminantSpace()`, which are designed to help assess and compare cell types between query and reference datasets using Multidimensional Scaling (MDS) and Principal Component Analysis (PCA), and Fisher Discriminant Analysis (FDA) respectively.

This vignette also demonstrates how to visualize gene expression data from single-cell RNA sequencing (scRNA-seq) experiments using two functions: `plotGeneExpressionDimred()` and `plotMarkerExpression()`. These functions allow researchers to explore gene expression patterns across various dimensions and compare expression distributions between different datasets or cell types.

Lastly this vignette will demonstrate how to visualize quality control (QC) scores and annotation scores, potentially identifying relationship between QC statistics (e.g., total library size or percentage of mitochondrial genes) and cell type annotation scores.


# Preliminaries

In the context of the `scDiagnostics` package, this vignette illustrates how to evaluate cell type annotations using two distinct datasets:

- `reference_data`: This dataset consists of meticulously curated cell type annotations assigned by experts, providing a robust benchmark for assessing the quality of annotations. It serves as a standard reference to evaluate and detect anomalies or inconsistencies within the cell type annotations.

- `query_data`: This dataset includes annotations both from expert assessments and those generated by the `r BiocStyle::Biocpkg("SingleR")` package. By comparing these annotations, you can identify discrepancies between manual and automated results, thereby pinpointing potential inconsistencies or areas that may need further scrutiny.

-  `qc_data`: This dataset includes data from haematopoietic stem and progenitor cells, including QC metrics like library size and mitochondrial content, `r BiocStyle::Biocpkg("SingleR")`-predicted cell types and annotation scores indicating prediction confidence.


```{r, message=FALSE, fig.show='hide'}
# Load library
library(scDiagnostics)

# Load datasets
data("reference_data")
data("query_data")
data("qc_data")

# Set seed for reproducibility
set.seed(0)
```

# Visualization of Query vs. Reference Dataset

## Plot Reference and Query Cell Types Using MDS

The `plotCellTypeMDS()` function creates a scatter plot using Multidimensional Scaling (MDS) to visualize the similarity between cell types in query and reference datasets. This function generates a MDS plot that colors cells according to their types based on a dissimilarity matrix calculated from a specified gene set.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Generate the MDS scatter plot with cell type coloring
plotCellTypeMDS(query_data = query_data, 
                reference_data = reference_data, 
                cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                query_cell_type_col = "SingleR_annotation", 
                ref_cell_type_col = "expert_annotation")
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/VisualizationTools/plotCellTypeMDS.png)

## Plot Principal Components for Different Cell Types

The `plotCellTypePCA()` function projects the query dataset onto the PCA space of the reference dataset. It then plots specified principal components for different cell types, allowing comparison of PCA results between query and reference datasets.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Generate the MDS scatter plot with cell type coloring
plotCellTypePCA(query_data = query_data, 
                reference_data = reference_data,
                cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                query_cell_type_col = "SingleR_annotation", 
                ref_cell_type_col = "expert_annotation", 
                pc_subset = 1:3)
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/VisualizationTools/plotCellTypePCA.png)

Here, `plotCellTypePCA()` projects the query data into the PCA space of the reference data and plots the specified principal components. The `pc_subset` argument allows you to select which principal components to display.


## Plot Principal Components as Boxplots

The `boxplotPCA()` function provides a boxplot visualization of principal components for different cell types across query and reference datasets. This function helps in comparing the distributions of PCA scores for specified cell types and datasets.
```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Plot the PCA data as boxplots
boxplotPCA(query_data = query_data, 
           reference_data = reference_data,
           cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
           query_cell_type_col = "SingleR_annotation", 
           ref_cell_type_col = "expert_annotation", 
           pc_subset = 1:3)
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/VisualizationTools/boxplotPCA.png)

In this example, `boxplotPCA()` generates boxplots for the specified principal components, showing the distribution of PCA scores for different cell types and datasets.


## Project Query Data onto Discriminant Space of Reference Data

The `calculateDiscriminantSpace()` function projects query single-cell RNA-seq data onto a discriminant space defined by reference data. This process involves using the reference data to identify key variables, compute discriminant vectors, and project both the reference and query data onto this space. The function then assesses the similarity between the projections of the query data and the reference data using cosine similarity and Mahalanobis distance.

## Function Details
The function performs the following steps for each pairwise combination of cell types:

1. **Identify Important Variables**: Determine the most significant variables that distinguish the two cell types from the reference data.
2. **Compute Covariance Matrices**: Estimate the covariance matrices for each cell type using the Ledoit-Wolf shrinkage method.
3. **Construct Scatter Matrices**: Create within-class and between-class scatter matrices.
4. **Solve Eigenvalue Problem**: Solve the generalized eigenvalue problem to obtain discriminant vectors.
5. **Project Data**: Project both reference and query data onto the discriminant space.
6. **Assess Similarity**: Measure the similarity of the query data projections to the reference projections using cosine similarity and Mahalanobis distance.

### Example Application

First, applying the function to the `SingleR` annotation.
```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Compute important variables for all pairwise cell comparisons
disc_output <- calculateDiscriminantSpace(
  reference_data = reference_data,
  query_data = query_data, 
  query_cell_type_col = "SingleR_annotation", 
  ref_cell_type_col = "expert_annotation")

# Generate scatter and boxplot
plot(disc_output, plot_type = "scatterplot")
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/VisualizationTools/calculateDiscriminantSpace1.png)

### Using Mahalanobis Distance for Anomaly Detection in Single-Cell RNA-Seq Data

We aim to evaluate whether a new batch of query cells fits well within established reference cell types or if there are cells exhibiting unusual characteristics. We will use the Mahalanobis distance calculated by the `calculateDiscriminantSpace()` function to achieve this.
```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Perform discriminant analysis and projection
discriminant_results <- calculateDiscriminantSpace(
  reference_data = reference_data,
  query_data = query_data,
  ref_cell_type_col = "expert_annotation",
  query_cell_type_col = "SingleR_annotation",
  calculate_metrics = TRUE,  # Compute Mahalanobis distance and cosine similarity
  alpha = 0.01  # Significance level for Mahalanobis distance cutoff
)
```

You can extract and analyze the Mahalanobis distances of query cells relative to reference cell types. High Mahalanobis distances may indicate potential anomalies or novel cell states.
```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Extract Mahalanobis distances
mahalanobis_distances <- discriminant_results$query_mahalanobis_dist

# Identify anomalies based on a threshold
threshold <- discriminant_results$mahalanobis_crit  # Use the computed cutoff value
anomalies <- mahalanobis_distances[mahalanobis_distances > threshold]
```

You can now create visualizations to illustrate the distribution of Mahalanobis distances and highlight potential anomalies.
```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Load ggplot2 for visualization
library(ggplot2)

# Convert distances to a data frame
distances_df <- data.frame(Distance = mahalanobis_distances,
                           Anomaly = ifelse(mahalanobis_distances > threshold, "Anomaly", "Normal"))

# Plot histogram of Mahalanobis distances
ggplot(distances_df, aes(x = Distance, fill = Anomaly)) +
  geom_histogram(binwidth = 0.5, position = "identity", alpha = 0.7) +
  labs(title = "Histogram of Mahalanobis Distances",
       x = "Mahalanobis Distance",
       y = "Frequency") +
  theme_bw()
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/VisualizationTools/calculateDiscriminantSpace2.png)

- **Low Mahalanobis Distances**: These query cells align well with the reference cell types, suggesting they are similar to the established cell types.

- **High Mahalanobis Distances**: Cells with high distances may represent outliers or novel, previously uncharacterized cell types. Further investigation may be needed to understand these anomalies.

## Project Data onto Sliced Inverse Regression (SIR) Space of Reference Data

The `calculateSIRSpace()` function leverages Sliced Inverse Regression (SIR) to analyze high-dimensional cell type data by focusing on the directions that best separate different cell types and their subtypes. The function projects data onto a lower-dimensional SIR space, aiming to highlight the most informative features for distinguishing between cell types.

The process starts by handling each cell type as a distinct slice of the data, allowing for a more detailed analysis. When `multiple_cond_means` is set to `TRUE`, the function refines this by computing conditional means for each cell type, using clustering to capture finer distinctions within each type. It then performs Singular Value Decomposition (SVD) on these refined means to uncover the principal directions that most effectively differentiate the cell types.

Ultimately, the function provides projections that reveal how cell types and their subtypes relate to each other in a reduced space, facilitating a deeper understanding of cellular variability and distinctions in complex datasets.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Calculate the SIR space projections
sir_output <- calculateSIRSpace(
  reference_data = reference_data,
  query_data = query_data,
  query_cell_type_col = "SingleR_annotation",
  ref_cell_type_col = "expert_annotation",
  multiple_cond_means = TRUE
)

# Plot the SIR space projections
plot(sir_output,
     sir_subset = 1:5,
     cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
     lower_facet = "scatter",
     diagonal_facet = "boxplot",
     upper_facet = "blank")
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/VisualizationTools/calculateSIRSpace1.png)

In this example, we compute the SIR space projections using the `calculateSIRSpace()` function, and then we visualize the projections with a boxplot to analyze the distribution of the data in the SIR space. This example helps you understand how different cell types and their subtypes are projected into a common space, facilitating their comparison and interpretation.

The plot below visualizes the contribution of different markers to the variance along the SIR dimensions, helping identify which markers are most influential in differentiating e.g. between B cells (including plasma cells) from other cell types.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Plot the top contributing markers
plot(sir_output, 
     plot_type = "loadings", 
     sir_subset = 1:5,
     n_top = 10)
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/VisualizationTools/calculateSIRSpace2.png)

# Visualization of Marker Expressions

## Visualizing Gene Expression in Reduced Dimensions

The `plotGeneExpressionDimred()` function plots the expression of a specific gene on a dimensional reduction plot. This function is useful for exploring how gene expression varies across cells in the context of reduced dimensions, such as those derived from PCA, t-SNE, or UMAP. Each point on the plot represents a single cell, color-coded according to the expression level of the specified gene.


Let's visualize the expression of the gene **NKG7** for CD4 on a PCA plot:
```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Visualize VPREB3 gene on a PCA plot
plotGeneExpressionDimred(sce_object = query_data,
                         method = "PCA",
                         pc_subset = 1:5,
                         feature = "NKG7",
                         cell_types = "CD4",
                         cell_type_col = "SingleR_annotation")
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/VisualizationTools/plotGeneExpressionDimred.png)

In this example, we visualize the expression of the **NKG7** gene on a PCA plot derived from the `query_data` dataset. The `method` argument specifies the dimensional reduction technique, which in this case is "PCA." The `pc_subset` argument allows us to focus on specific principal components (PCs), here selecting the first five. The `feature` argument is the name of the gene we want to visualize.

When executed, this function generates a scatter plot with cells colored by their expression levels of **NKG7**. The resulting plot can help identify clusters of cells with similar expression patterns or highlight how expression varies across the dataset.

You can also visualize gene expression using other dimensional reduction methods, such as t-SNE or UMAP.


## Plotting Gene Expression Distribution

The `plotMarkerExpression()` function compares the distribution of a specific gene's expression across an overall dataset and within specific cell types. It generates density plots to visualize these distributions, providing a graphical means of assessing the similarity between reference and query datasets. The function also applies a z-rank normalization to make expression values comparable between datasets.
```{r, fig.height=5, fig.width=10, fig.show='hide'}
plotMarkerExpression(
    reference_data = reference_data,
    query_data = query_data,
    ref_cell_type_col = "expert_annotation",
    query_cell_type_col = "SingleR_annotation",
    gene_name = "NKG7",
    cell_type = "CD4",
    normalization = "z_score"
)
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/VisualizationTools/plotMarkerExpression.png)

In this example, the function compares the expression distribution of the **VPREB3** gene between the reference and query datasets. The `ref_cell_type_col` and `query_cell_type_col` arguments specify the columns in `colData` that identify the cell types in the reference and query datasets, respectively. The `cell_type` argument specifies the cell type of interest, and `gene_name` is the gene whose expression distribution we wish to compare.

When executed, this function generates two density plots: one showing the overall gene expression distribution across all cells and another focusing on the specified cell type. These plots help identify whether the gene's expression levels are similar between the reference and query datasets, providing insight into dataset alignment.

# Visualization of QC and Annotation Scores

## Scatter Plot: QC Stats vs Cell Type Annotation Scores

The scatter plot visualizes the relationship between QC statistics (e.g., total library size or percentage of mitochondrial genes) and cell type annotation scores. This plot helps in understanding how QC metrics influence or correlate with the predicted cell types.


```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Remove cell types with very few cells
qc_data_subset <- qc_data[, !(qc_data$SingleR_annotation 
                              %in% c("Chondrocytes", "DC", 
                                     "Neurons","Platelets"))]

# Generate scatter plot
p1 <- plotQCvsAnnotation(sce_object = qc_data_subset,
                         cell_type_col = "SingleR_annotation",
                         qc_col = "total",
                         score_col = "annotation_scores")
p1 + ggplot2::xlab("Library Size")
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/VisualizationTools/plotQCvsAnnotation.png)

A scatter plot can reveal patterns such as whether cells with higher library sizes or mitochondrial content tend to be associated with specific annotations. For instance, cells with unusually high mitochondrial content might be identified as low-quality or stressed, potentially affecting their annotations.

## Histograms: QC Stats and Annotation Scores Visualization

Histograms provide a distribution view of QC metrics and annotation scores. They help in evaluating the range, central tendency, and spread of these variables across cells.



```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Generate histograms
histQCvsAnnotation(sce_object = query_data, 
                   cell_type_col = "SingleR_annotation", 
                   qc_col = "percent_mito", 
                   score_col = "annotation_scores")
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/VisualizationTools/histQCvsAnnotation.png)

Histograms are useful for assessing the overall distribution of QC metrics and annotation scores. For example, if the majority of cells have high percent_mito, it might indicate that many cells are stressed or dying, which could impact the quality of the data.

# Visualization of Gene Sets or Pathway Scores on Dimensional Reduction Plots

Dimensional reduction plots (PCA, t-SNE, UMAP) are used to visualize the relationships between cells in reduced dimensions. Overlaying gene set scores on these plots provides insights into how specific gene activities are distributed across cell clusters.
```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Plot gene set scores on PCA
plotGeneSetScores(sce_object = query_data, 
                  cell_type_col = "SingleR_annotation",
                  cell_types = "CD8",
                  method = "PCA", 
                  score_col = "gene_set_scores", 
                  pc_subset = 1:3)
```
![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/VisualizationTools/plotGeneSetScores.png)

By visualizing gene set scores on PCA or UMAP plots, one can identify clusters of cells with high or low gene set activities. This can help in understanding the biological relevance of different gene sets or pathways in various cell states or types.


------------------------------------------------------------------------

# R Session Info

```{r SessionInfo, echo=FALSE, message=FALSE, warning=FALSE, comment=NA, fig.show='hide'}
options(width = 80) 
sessionInfo()
```

---
title: "Getting Started with `scDiagnostics`"
author:
    - name: Anthony Christidis
      affiliation:
        - &core_affiliation Core for Computational Biomedicine, Harvard Medical School
    - name: Andrew Ghazi
      affiliation:
        - *core_affiliation
    - name: Smriti Chawla
      affiliation:
        - *core_affiliation
    - name: Nitesh Turaga
      affiliation:
        - *core_affiliation
    - name: Ludwig Geistlinger
      affiliation:
        - *core_affiliation
    - name: Robert Gentleman
      affiliation:
        - *core_affiliation
package: scDiagnostics
output: 
  BiocStyle::html_document:
    toc: true
    toc_float: true
vignette: >
  %\VignetteIndexEntry{1. Getting Started with scDiagnostics}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 72
---

```{r setup, include = FALSE, fig.show='hide'}
knitr::knit_hooks$set(pngquant = knitr::hook_pngquant)

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dev = "ragg_png",
  dpi = 72,
  fig.retina = 2,
  fig.align = "center",
  out.width = "100%",
  pngquant = "--speed=1 --quality=1-5"
)
```

# Purpose

Annotation transfer from a reference dataset is a key process for
annotating cell types in new single-cell RNA-sequencing (scRNA-seq)
experiments. This approach provides a quick, automated, and reproducible
alternative to manual annotation based on marker gene expression.
Despite its advantages, challenges such as dataset imbalance and
unrecognized discrepancies between query and reference datasets can lead
to inaccurate annotations and affect subsequent analyses.

The `scDiagnostics` package is designed to address these issues by
offering a suite of diagnostic tools for the systematic evaluation of
cell type assignments in scRNA-seq data. It provides functionality to
assess whether query and reference datasets are well-aligned, which is
crucial for ensuring accurate annotation transfer. In addition,
`scDiagnostics` helps evaluate annotation ambiguity, cluster
heterogeneity, and marker gene alignment. By providing insights into
these aspects, `scDiagnostics` enables researchers to determine the
precision with which cells from a new scRNA-seq experiment can be
assigned to known cell types, thereby supporting more accurate and
reliable downstream analysis.

# Installation

## Installation from Bioconductor (Release)

Users interested in using the stable release version of the `scDiagnostics` package: please follow the installation instructions [**here**](https://bioconductor.org/packages/release/bioc/html/scDiagnostics.html). This is the recommended way of installing the package.

## Installation from GitHub (Development)

To install the development version of the package from Github, use the
following command:

```{r, eval = FALSE, fig.show='hide'}
BiocManager::install("ccb-hms/scDiagnostics")
```

To build the package vignettes upon installation use:

```{r, eval=FALSE, fig.show='hide'}
BiocManager::install("ccb-hms/scDiagnostics",
                     build_vignettes = TRUE,
                     dependencies = TRUE)
```

Once you have installed the package, you can load it with the following
code:

```{r, message=FALSE, fig.show='hide'}
library(scDiagnostics)
```

# Preliminaries

To explore the full capabilities of the `scDiagnostics` package, you
have the option to use your own data or leverage the datasets included
within the `scDiagnostics` package itself. In this guide, we will focus
on utilizing these built-in datasets, which provide a practical and
convenient resource for demonstrating the features of `scDiagnostics.`
These datasets are specifically designed to facilitate the exploration
of the package's functionalities and to help evaluate the accuracy of
cell type assignments. You can learn more about the datasets by looking
at the documentation of the datasets available in the reference manual.

## Loading Datasets

In these datasets available in the `scDiagnostics` package,
`reference_data`, `query_data`, and `qc_data` are all
`r BiocStyle::Biocpkg("SingleCellExperiment")` objects that include a
`logcounts` assay, which stores the log-transformed expression values
for the genes.

The `reference_data` and `query_data` objects both originate from
scRNA-seq experiments on hematopoietic tissues, specifically bone marrow
samples, as provided by the `r BiocStyle::Biocpkg("scRNAseq")` package.
These datasets have undergone comprehensive processing and cleaning,
ensuring high-quality data for downstream analysis. Log-normalized
counts were added to both datasets using the
`r BiocStyle::Biocpkg("scuttle")` package. The `query_data` object has
been further annotated with cell type assignments using the
`r BiocStyle::Biocpkg("SingleR")` package, and it includes
`annotation_scores` that reflect the confidence in these annotations.
Additionally, gene set scores were computed and incorporated into the
`query_data` using the `r BiocStyle::Biocpkg("AUCell")` package. For
feature selection, the top 500 highly variable genes (HVGs) common to
both datasets were identified and retained using the
`r BiocStyle::Biocpkg("scran")` package. Finally, dimensionality
reduction techniques including PCA, t-SNE, and UMAP were applied to both
datasets, with the results stored within each object using the
`r BiocStyle::Biocpkg("scater")` package.

The `qc_data` dataset in this package is derived from the `hpca` dataset
available in the `r BiocStyle::Biocpkg("celldex")` package. Like the
other datasets, `qc_data` has undergone significant cleaning and
processing to ensure high data quality. Quality control (QC) metrics
were added using the `r BiocStyle::Biocpkg("scuttle")` package. Cell
type annotations and associated annotation_scores were generated using
the `r BiocStyle::Biocpkg("SingleR")` package. Additionally, the top
highly variable genes were selected using the
`r BiocStyle::Biocpkg("scran")` package to enhance the dataset’s utility
for downstream analyses.

```{r, message=FALSE, fig.show='hide'}
# Load datasets
data("reference_data")
data("query_data")
data("qc_data")

# Set seed for reproducibility
set.seed(0)
```

The `reference_data` contains a column data labeled `expert_annotation`,
which provides cell type annotations assigned by experts. On the other
hand, `query_data` also includes `expert_annotation`, but it
additionally features `SingleR_annotation`, which is the cell type
annotation generated by the `r BiocStyle::Biocpkg("SingleR")` package, a
popular package for cell type assignment based on reference datasets.
The `qc_data` object contains a special column called
`annotation_scores`, which holds the scores from the `SingleR`
annotations, providing a measure of confidence or relevance for the
assigned cell types.

By working with these datasets, you can gain hands-on experience with
the various diagnostic tools and functions offered by `scDiagnostics`,
allowing you to better understand how well it aligns query and reference
datasets, assesses annotation ambiguity, and evaluates cluster
heterogeneity and marker gene alignment.

## Subsetting the Datasets

Some functions in the vignette are designed to work with
`r BiocStyle::Biocpkg("SingleCellExperiment")` objects that contain data
from only one cell type. We will create separate
`r BiocStyle::Biocpkg("SingleCellExperiment")` objects that only CD4
cells, to ensure compatibility with these functions.

```{r, message=FALSE, fig.show='hide'}
# Load library
library(scran)
library(scater)

# Subset to CD4 cells
ref_data_cd4 <- reference_data[, which(
    reference_data$expert_annotation == "CD4")]
query_data_cd4 <- query_data_cd4 <- query_data[, which(
    query_data$expert_annotation == "CD4")]

# Select highly variable genes
ref_top_genes <- getTopHVGs(ref_data_cd4, n = 500)
query_top_genes <- getTopHVGs(query_data_cd4, n = 500)
common_genes <- intersect(ref_top_genes, query_top_genes)

# Subset data by common genes
ref_data_cd4 <- ref_data_cd4[common_genes,]
query_data_cd4 <- query_data_cd4[common_genes,]

# Run PCA on both datasets
ref_data_cd4 <- runPCA(ref_data_cd4)
query_data_cd4 <- runPCA(query_data_cd4)
```

# Getting Started with `scDiagnostics`

**The functions introduced in this section represent just a subset of
the functions available in the `scDiagnostics` package.**

For a complete overview and detailed demonstrations of all the functions
included in the package, please refer to the designated vignettes which
you may browse from the [pkgdown site for
`scDiagnostics`](https://ccb-hms.github.io/scDiagnostics/). Each
vignette is designed to address specific aspects of `scDiagnostics`, and
this vignette highlights key functionalities to illustrate their
applications. These vignettes provide in-depth guidance and examples for
each function, helping users fully leverage the capabilities of
`scDiagnostics` in their single-cell analyses.

# Visualization of Cell Type Annotations

For a detailed example of all possible functions to visualize reference
and query datasets, please refer to the [Visualization of Cell Type
Annotations](https://ccb-hms.github.io/scDiagnostics/articles/VisualizationTools.html)
vignette.

## Visualization of Cell Type Annotations in Reduced Dimensions

### `plotCellTypePCA()`

The `plotCellTypePCA()` function provides a visual comparison of
principal components (PCs) for different cell types across query and
reference datasets. By projecting the query data onto the PCA space of
the reference dataset, it creates informative plots to help you
understand how various cell types are distributed in the principal
component space.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Plot PCA data
pc_plot <- plotCellTypePCA(
    query_data = query_data, 
    reference_data = reference_data,
    cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
    query_cell_type_col = "expert_annotation", 
    ref_cell_type_col = "expert_annotation",
    pc_subset = 1:3
)
# Display plot
pc_plot
```

![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/scDiagnostics/plotCellTypePCA.png)

The function returns a `ggplot` object featuring pairwise scatter plots
of the selected principal components. Each plot compares how different
cell types from the query and reference datasets project onto the PCA
space. This visualization aids in identifying how cell types distribute
across PCs and facilitates comparisons between datasets.

The `reference_data` argument contains the reference cell data, which
serves as the foundation for defining the PC space. The `query_data`
parameter includes the query cell data that will be projected. The
function uses the `ref_cell_type_col` and `query_cell_type_col` to
identify the relevant cell type annotations in the reference and query
datasets.

### `calculateDiscriminantSpace()`

Alternatively, you can also use the `calculateDiscriminantSpace()`
function, which projects query single-cell RNA-seq data onto a
discriminant space defined by a reference dataset. This approach helps
evaluate the similarity between the query and reference data, offering
insights into the classification of query cells.

```{r, fig.show='hide'}
disc_output <- calculateDiscriminantSpace(
    reference_data = reference_data,
    query_data = query_data, 
    ref_cell_type_col = "expert_annotation",
    query_cell_type_col = "SingleR_annotation"
)
```

The function returns a comprehensive output that includes discriminant
eigenvalues and eigenvectors, which represent the variance explained by
each discriminant axis and are used to project the data. It also
provides the projections of the reference and query data onto the
discriminant space. The Mahalanobis distances between the query and
reference cell types are calculated, offering insights into how close
the query projections are to the reference. The cosine similarity scores
provide another metric to assess the similarity between the datasets.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
plot(disc_output, plot_type = "scatterplot")
```

![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/scDiagnostics/calculateDiscriminantSpace.png)

Alternatively, you can create a boxplot comparing query and reference
projections for a specific cell type. See the reference manual for an
example.

## Visualization of Marker Expressions

Visualizing gene expression distributions is crucial for understanding
dataset similarity and cell type-specific expression patterns. The
`plotMarkerExpression()` function allows you to compare the expression
levels of a specific gene between a reference dataset and a query
dataset, both overall and within a specified cell type. This comparison
is done using density plots, which help in assessing the alignment and
potential discrepancies between datasets.

```{r, fig.height=5, fig.width=10, fig.show='hide', fig.show='hide'}
plotMarkerExpression(reference_data = reference_data, 
                     query_data = query_data, 
                     ref_cell_type_col = "expert_annotation", 
                     query_cell_type_col = "SingleR_annotation", 
                     gene_name = "VPREB3", 
                     cell_type = "B_and_plasma")
```

![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/scDiagnostics/plotMarkerExpression.png)

## Visualization of QC and Annotation Scores

The scatter plot illustrates the relationship between key QC metrics
(e.g., total library size, percentage of mitochondrial genes) and cell
type annotation scores. This visualization aids in assessing how QC
factors might influence or correlate with the assigned cell type
annotations.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Remove cell types with very few cells
qc_data_subset <- qc_data[, !(qc_data$SingleR_annotation 
                              %in% c("Chondrocytes", "DC", 
                                     "Neurons","Platelets"))]

# Generate scatter plot
library(ggplot2)
p1 <- plotQCvsAnnotation(sce_object = qc_data_subset,
                         cell_type_col = "SingleR_annotation",
                         qc_col = "total",
                         score_col = "annotation_scores")
p1 + xlab("Library Size")
```

![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/scDiagnostics/plotQCvsAnnotation.png)

This scatter plot can uncover patterns, such as whether cells with
larger library sizes or higher mitochondrial content are linked to
specific annotations. For example, cells exhibiting unusually high
mitochondrial content may be flagged as low-quality or stressed, which
could impact their annotation accuracy.

# Evaluation of Dataset and Marker Gene Alignment

For a detailed example of all possible functions to assess the alignment
of datasets and marker genes, please refer to the [Evaluation of Dataset
and Marker Gene
Alignment](https://ccb-hms.github.io/scDiagnostics/articles/https://ccb-hms.github.io/scDiagnostics/articles/DatasetAlignment.html)
vignette.

## `comparePCASubspace()`

In single-cell RNA-seq analysis, evaluating how closely the subspaces
defined by the leading principal components (PCs) of query and reference
datasets match is crucial. This evaluation is key to understanding how
each dataset captures and represents structure and variation. The
`comparePCASubspace()` function is specifically designed to assess this
alignment by calculating the cosine similarity between the loadings of
the most significant variables for each principal component. This
analysis is essential for measuring the extent of similarity between
datasets, which is vital for precise cell type annotation and effective
data integration.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Compare PCA subspaces between query and reference data
subspace_comparison <- comparePCASubspace(
    query_data = query_data_cd4,
    reference_data = ref_data_cd4, 
    query_cell_type_col = "expert_annotation", 
    ref_cell_type_col = "expert_annotation", 
    pc_subset = 1:5
)

# View weighted cosine similarity score
subspace_comparison$weighted_cosine_similarity

# Plot output for PCA subspace comparison (if a plot method is available)
plot(subspace_comparison)
```

![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/scDiagnostics/comparePCASubspace.png)

## `plotPairwiseDistancesDensity()`

The `plotPairwiseDistancesDensity()` function calculates and visualizes
pairwise distances or correlations between cell types in query and
reference datasets, aiding in the evaluation of cell type annotation
consistency in single-cell RNA sequencing (scRNA-seq) analysis.
Operating on `r BiocStyle::Biocpkg("SingleCellExperiment")` objects, it
allows users to specify cell types of interest and compute either
distances or correlation coefficients, with the option to project data
into PCA space for focused analysis. The function generates a density
plot using `ggplot2`, comparing cell relationships within and between
datasets.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Example usage of the function
plotPairwiseDistancesDensity(query_data = query_data, 
                             reference_data = reference_data, 
                             query_cell_type_col = "expert_annotation", 
                             ref_cell_type_col = "expert_annotation", 
                             cell_type = "CD8", 
                             pc_subset = 1:10,
                             distance_metric = "correlation",
                             correlation_method = "pearson")
```

![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/scDiagnostics/plotPairwiseDistancesDensity.png)

## `calculateWassersteinDistance()`

The code below illustrates how to use the
`calculateWassersteinDistance()` function to compare the Wasserstein
distances between CD4 cells in the reference and query datasets. The
resulting plot provides insight into whether the differences between the
datasets are statistically significant.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Generate the Wasserstein distance density plot
wasserstein_data <- calculateWassersteinDistance(
    query_data = query_data_cd4,
    reference_data = ref_data_cd4, 
    query_cell_type_col = "expert_annotation", 
    ref_cell_type_col = "expert_annotation", 
    pc_subset = 1:10,
)
plot(wasserstein_data)
```

![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/scDiagnostics/plotWassersteinDistance.png)

## `calculateVarImpOverlap()`

Imagine you have two sets of data: one called reference_data and another
called query_data. Both sets include information about gene expression
and cell types, with columns named `expert_annotation` for the reference
data and `SingleR_annotation` for the query data. You want to find out
which genes are most important for each dataset and then compare them.

Here’s how you can do it with the function:

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# RF function to compare (between datasets) which genes are best at differentiating cell types
rf_output <- calculateVarImpOverlap(reference_data = reference_data, 
                                    query_data = query_data, 
                                    query_cell_type_col = "SingleR_annotation", 
                                    ref_cell_type_col = "expert_annotation", 
                                    n_tree = 500,
                                    n_top = 50)

# Comparison table
rf_output$var_imp_comparison
```

## `calculateAveragePairwiseCorrelation()`

The `calculateAveragePairwiseCorrelation()` function computes the
average pairwise correlations between specified cell types in
single-cell gene expression data. It calculates pairwise correlations
between query and reference cells using a specified correlation method,
and then averages these correlations for each cell type pair. This
approach helps assess the similarity between cells in reference and
query datasets and provides insights into the reliability of cell type
annotations.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Compute pairwise correlations between specified cell types
cor_matrix_avg <- calculateAveragePairwiseCorrelation(
  query_data = query_data, 
  reference_data = reference_data, 
  query_cell_type_col = "SingleR_annotation", 
  ref_cell_type_col = "expert_annotation", 
  cell_types = c("CD4", "CD8", "B_and_plasma"), 
  pc_subset = 1:5,
  correlation_method = "spearman"
)

# Visualize the average pairwise correlation matrix
plot(cor_matrix_avg)
```

![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/scDiagnostics/calculateAveragePairwiseCorrelation.png)

# Detection and Analysis of Annotation Anomalies

For a detailed example of all possible functions to detect and analyze
potentially anomalous cells, please refer to the [Detection and Analysis
of Annotation
Anomalies](https://ccb-hms.github.io/scDiagnostics/articles/AnnotationAnomalies.html)
vignette.

## Detection of Annotation Anomalies

The `detectAnomaly()` function is designed to identify anomalies in
single-cell data by leveraging PCA projections and the Isolation Forest
algorithm. This method is useful for detecting anomalies or unusual
patterns in single-cell datasets, whether you're analyzing a reference
dataset or comparing a query dataset against it.

The function projects single-cell data onto a PCA space and builds an
Isolation Forest model on this PCA space to detect anomalies. If a query
dataset is provided, the function computes anomaly scores for the query
data based on its PCA projections relative to the reference data. If no
query data is provided, it computes anomaly scores for the reference
data itself.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Perform anomaly detection
anomaly_output <- detectAnomaly(reference_data = reference_data, 
                                query_data = query_data, 
                                ref_cell_type_col = "expert_annotation", 
                                query_cell_type_col = "SingleR_annotation",
                                pc_subset = 1:5)
# Plot the results for a specific cell type
plot(anomaly_output, 
     cell_type = "CD4", 
     data_type = "query",
     pc_subset = 1:5)
```

![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/scDiagnostics/detectAnomaly.png)

## Analysis of Annotation Anomalies

The `calculateCellDistances()` function computes distances both within a
reference dataset and between query cells and reference cells for each
specified cell type. By first projecting data onto a PCA space, the
function calculates Euclidean distances to quantify similarities and
dissimilarities. For each cell type, it generates pairwise distances
within the reference dataset and measures how far each query cell is
from all reference cells. This approach enables detailed analysis of
cell type-specific distances, aiding in the identification of outliers
and other patterns of interest.

To identify anomalous cells within the query data, we first use the
`detectAnomaly()` function, focusing specifically on the CD4 cell type.
This function will compute anomaly scores for each CD4 cell in the query
dataset based on their projection in the PCA space of the reference
data. Next, we will plot the distance distributions for the top 6 CD4
cells with the highest anomaly scores. These distances, computed using
the `calculateCellDistances()` function, will illustrate how these
anomalous cells differ from the reference cells, providing insight into
their potential outlier status and helping to visualize patterns within
the data.

```{r, fig.height=5, fig.width=10, fig.show='hide'}
# Identify outliers for CD4
cd4_anomalies <- detectAnomaly(reference_data = reference_data, 
                               query_data = query_data, 
                               query_cell_type_col = "SingleR_annotation", 
                               ref_cell_type_col = "expert_annotation")
cd4_top6_anomalies <- names(sort(cd4_anomalies$CD4$query_anomaly_scores, 
                                 decreasing = TRUE)[1:6])

# Plot the PC data
distance_data <- calculateCellDistances(
    query_data = query_data, 
    reference_data = reference_data, 
    query_cell_type_col = "SingleR_annotation", 
    ref_cell_type_col = "expert_annotation"
) 

# Plot the densities of the distances
plot(distance_data, ref_cell_type = "CD4", cell_names = cd4_top6_anomalies)
```

![](https://raw.githubusercontent.com/ccb-hms/scDiagnostics/main/inst/extdata/compressed/scDiagnostics/calculateCellDistances.png)

------------------------------------------------------------------------

# R Session Info

```{r SessionInfo, echo=FALSE, message=FALSE, warning=FALSE, comment=NA, fig.show='hide'}
options(width = 80) 
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCellDistances.R,
%   R/plot.calculateCellDistancesObject.R
\name{calculateCellDistances}
\alias{calculateCellDistances}
\alias{plot.calculateCellDistancesObject}
\title{Compute Cell Distances Between Reference and Query Data}
\usage{
calculateCellDistances(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  pc_subset = 1:5,
  assay_name = "logcounts",
  max_cells_query = 5000,
  max_cells_ref = 5000
)

\method{plot}{calculateCellDistancesObject}(x, ref_cell_type, cell_names, ...)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data}
that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data}
that identifies the cell types.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.}

\item{pc_subset}{A numeric vector specifying which principal components to include in the plot. Default 1:5.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}

\item{x}{A list containing the distance data computed by \code{calculatecellDistances}.}

\item{ref_cell_type}{A string specifying the reference cell type.}

\item{cell_names}{A string specifying the query cell name for which to plot the distances.}

\item{...}{Additional arguments passed to the plotting function.}
}
\value{
A list containing distance data for each cell type. Each entry in the list contains:
\describe{
  \item{ref_distances}{A vector of all pairwise distances within the reference subset for the cell type.}
  \item{query_to_ref_distances}{A matrix of distances from each query cell to all reference cells for the cell type.}
}

The S3 plot method returns a \code{ggplot} density plot comparing the reference distances and the distances from the specified cell to the reference cells.
}
\description{
This function computes the distances within the reference dataset and the distances from each query cell to all
reference cells for each cell type. It uses PCA for dimensionality reduction and Euclidean distance for distance calculation.

The S3 plot method plots the density functions for the reference data and the distances from a specified query cells
to all reference cell within a specified cell type.
}
\details{
The function first performs PCA on the reference dataset and projects the query dataset onto the same PCA space.
It then computes pairwise Euclidean distances within the reference dataset for each cell type, as well as distances from each
query cell to all reference cells of a particular cell type. The results are stored in a list, with one entry per cell type.

The S3 plot method first checks if the specified cell type and cell names are present in the object. If the
specified cell type or cell name is not found, an error is thrown. It then extracts the distances within the reference dataset
and the distances from the specified query cell to the reference cells The function creates a density plot using \code{ggplot2}
to compare the distance distributions. The density plot will show two distributions: one for the pairwise distances within the
reference dataset and one for the distances from the specified query cell to each reference cell. These distributions are
plotted in different colors to visually assess how similar the query cell is to the reference cells of the specified cell type.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Plot the PC data
distance_data <- calculateCellDistances(query_data = query_data,
                                        reference_data = reference_data,
                                        query_cell_type_col = "SingleR_annotation",
                                        ref_cell_type_col = "expert_annotation",
                                        pc_subset = 1:10)

# Identify outliers for CD4
cd4_anomalies <- detectAnomaly(reference_data = reference_data,
                               query_data = query_data,
                               query_cell_type_col = "SingleR_annotation",
                               ref_cell_type_col = "expert_annotation",
                               pc_subset = 1:10,
                               n_tree = 500,
                               anomaly_threshold = 0.5)
cd4_top6_anomalies <- names(sort(cd4_anomalies$CD4$query_anomaly_scores, decreasing = TRUE)[1:6])

# Plot the densities of the distances
plot(distance_data, ref_cell_type = "CD4", cell_names = cd4_top6_anomalies)
plot(distance_data, ref_cell_type = "CD8", cell_names = cd4_top6_anomalies)

}
\seealso{
\code{\link{plot.calculateCellDistancesObject}}

\code{\link{calculateCellDistances}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDocumentation.R
\docType{data}
\name{reference_data}
\alias{reference_data}
\title{Reference Single-Cell RNA-Seq Dataset}
\format{
An object of class \code{SingleCellExperiment} with 392 rows and 1500 columns.
}
\source{
The HeOrganAtlas dataset, available through the scRNAseq package.
}
\usage{
reference_data
}
\description{
This dataset contains the processed reference dataset from the HeOrganAtlas dataset for Marrow tissue.
It has been preprocessed to include log-normalized counts, specific metadata columns, and PCA, t-SNE, and UMAP results.
}
\details{
This dataset underwent the following steps:
\itemize{
  \item Loads the HeOrganAtlas dataset specifically for Marrow tissue from the \code{scRNAseq} package.
  \item Divides the loaded dataset into a reference dataset used for downstream analysis.
  \item Performs log normalization on the reference dataset using the function \code{logNormCounts} from the \code{scuttle} package.
  \item Selects the column \code{expert_annotation}) from the cell metadata for downstream analysis.
  \item Selects highly variable genes (HVGs) using the function \code{getTopHVGs} from the \code{scran} package on the reference dataset.
  \item Performs Principal Component Analysis (PCA) on the reference dataset using the function \code{runPCA} from the \code{scater} package.
  \item Performs t-Distributed Stochastic Neighbor Embedding (t-SNE) on the reference dataset using the function \code{runTSNE} from the \code{scater} package.
  \item Performs Uniform Manifold Approximation and Projection (UMAP) on the reference dataset using the function \code{runUMAP} from the \code{scater} package.
}
}
\examples{
# Load and explore the reference dataset
data("reference_data")

}
\references{
He, et al. (2020). HeOrganAtlas: a comprehensive human organ atlas based on single-cell RNA sequencing.
}
\seealso{
Use \code{data("reference_data")} to load and access the resulting reference dataset.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotGeneSetScores.R
\name{plotGeneSetScores}
\alias{plotGeneSetScores}
\title{Visualization of gene sets or pathway scores on dimensional reduction plot}
\usage{
plotGeneSetScores(
  sce_object,
  cell_type_col,
  method = c("PCA", "TSNE", "UMAP"),
  score_col,
  pc_subset = 1:5,
  cell_types = NULL,
  max_cells = 2000
)
}
\arguments{
\item{sce_object}{An object of class \code{\linkS4class{SingleCellExperiment}} containing numeric expression matrix and other metadata.
It can be either a reference or query dataset.}

\item{cell_type_col}{The column name in the \code{colData} of \code{sce_object} that identifies the cell types.}

\item{method}{A character string indicating the method for visualization ("PCA", "TSNE", or "UMAP").}

\item{score_col}{A character string representing the name of the score_col (score) in the colData(sce_object) to plot.}

\item{pc_subset}{An optional vector specifying the principal components (PCs) to include in the plot if method = "PCA".
Default is 1:5.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.}

\item{max_cells}{Maximum number of cells to retain. If the object has fewer cells, it is returned unchanged.
Default is 2000.}
}
\value{
A ggplot2 object representing the gene set scores plotted on the specified reduced dimensions.
}
\description{
Plot gene sets or pathway scores on PCA, TSNE, or UMAP. Single cells are color-coded by scores of gene sets or pathways.
}
\details{
This function plots gene set scores on reduced dimensions such as PCA, t-SNE, or UMAP.
It extracts the reduced dimensions from the provided \code{\linkS4class{SingleCellExperiment}} object.
Gene set scores are visualized as a scatter plot with colors indicating the scores.
For PCA, the function automatically includes the percentage of variance explained
in the plot's legend.
}
\examples{
# Load data
data("query_data")

# Plot gene set scores on PCA
plotGeneSetScores(sce_object = query_data,
                  method = "PCA",
                  score_col = "gene_set_scores",
                  pc_subset = 1:5,
                  cell_types = "CD8",
                  cell_type_col = "SingleR_annotation")

}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regressPC.R
\name{adjustPValues}
\alias{adjustPValues}
\title{Adjust P-Values in Regression Results}
\usage{
adjustPValues(
  regress_res,
  adjust_method = c("BH", "holm", "hochberg", "hommel", "bonferroni", "BY", "fdr",
    "none"),
  indep_var = c("cell_type", "cell_type_dataset_interaction",
    "cell_type_batch_interaction")
)
}
\arguments{
\item{regress_res}{A list containing regression results. The structure varies by analysis type.}

\item{adjust_method}{A character string specifying the method to adjust the p-values.
Options include "BH", "holm", "hochberg", "hommel", "bonferroni", "BY", "fdr", or "none".
Default is "BH" (Benjamini-Hochberg).}

\item{indep_var}{A character string specifying the independent variable for the adjustment.
Options are "cell_type", "cell_type_dataset_interaction", or "cell_type_batch_interaction".}
}
\value{
A list similar to \code{regress_res}, but with an added column for adjusted p-values
  in the coefficients tables.
}
\description{
Adjusts the p-values in the regression results using a specified adjustment method.
The adjustment is performed for different regression types including cell type,
dataset, and cell type-batch interaction analyses.
}
\details{
This function adjusts p-values from regression results stored in a list. The adjustment
can be applied across different regression structures depending on the analysis type.
The method for adjusting p-values can be selected from various options such as
Benjamini-Hochberg (BH), Holm, and others, which are supported by the `p.adjust` function in R.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateHotellingPValue.R
\name{hotellingT2}
\alias{hotellingT2}
\title{Calculate Hotelling's T^2 Statistic}
\usage{
hotellingT2(sample1, sample2)
}
\arguments{
\item{sample1}{A numeric matrix or data frame of multivariate observations for sample 1, where rows are observations and columns
are variables.}

\item{sample2}{A numeric matrix or data frame of multivariate observations for sample 2, with the same structure as sample 1.}
}
\value{
The Hotelling's T^2 statistic.
}
\description{
Calculates the Hotelling's T^2 statistic for comparing means of multivariate data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regressPC.R
\name{regressFastCustom}
\alias{regressFastCustom}
\title{Fast Custom Linear Regression for Principal Components}
\usage{
regressFastCustom(pc, indep_var, df)
}
\arguments{
\item{pc}{A character string specifying the principal component column name in the data frame.}

\item{indep_var}{A character string specifying the independent variable specification.
Options include "cell_type", "cell_type * batch", "cell_type * dataset", or other
interaction specifications.}

\item{df}{A data frame containing the principal component scores and categorical predictors.
Must include columns for the specified PC and predictor variables.}
}
\value{
A list containing:
  \item{coefficients}{A data frame with columns \code{coef}, \code{se}, \code{t}, and \code{p.value}
    containing regression coefficients, standard errors, t-statistics, and p-values.}
  \item{r_squared}{A numeric value representing the R-squared of the model.}
}
\description{
Performs efficient linear regression of principal component scores against categorical
predictors (cell types, batches, datasets, or their interactions) using QR decomposition
for numerical stability and computational efficiency.
}
\details{
This function implements a custom linear regression optimized for categorical predictors
commonly used in single-cell RNA sequencing analysis. It uses QR decomposition instead
of normal equations for improved numerical stability and handles rank-deficient design
matrices gracefully. The function supports various model specifications including:
\itemize{
  \item Simple cell type effects: \code{PC ~ cell_type}
  \item Cell type and batch interactions: \code{PC ~ cell_type * batch}
  \item Cell type and dataset interactions: \code{PC ~ cell_type * dataset}
}

The output format is compatible with \code{speedglm} results to maintain consistency
with existing plotting and analysis workflows.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCellSimilarityPCA.R,
%   R/plot.calculateCellSimilarityPCAObject.R
\name{calculateCellSimilarityPCA}
\alias{calculateCellSimilarityPCA}
\alias{plot.calculateCellSimilarityPCAObject}
\title{Calculate Cell Similarity Using PCA Loadings}
\usage{
calculateCellSimilarityPCA(
  sce_object,
  cell_names,
  pc_subset = 1:5,
  n_top_vars = 50,
  assay_name = "logcounts"
)

\method{plot}{calculateCellSimilarityPCAObject}(x, pc_subset = 1:5, ...)
}
\arguments{
\item{sce_object}{A \code{\linkS4class{SingleCellExperiment}} object containing expression data.}

\item{cell_names}{A character vector specifying the cell names for which to compute the similarity.}

\item{pc_subset}{A numeric vector specifying the subset of principal components to include in the plot. Default is 1:5.}

\item{n_top_vars}{An integer indicating the number of top loading variables to consider for each PC. Default is 50.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{x}{An object of class `calculateCellSimilarityPCA` containing a dataframe of cosine similarity values
between cells and PCs.}

\item{...}{Additional arguments passed to the plotting function.}
}
\value{
A data frame containing cosine similarity values between cells for each selected principal component.

The S3 plot method returns a \code{ggplot} object representing the cosine similarity heatmap.
}
\description{
This function calculates the cosine similarity between cells based on the principal components (PCs)
obtained from PCA (Principal Component Analysis) loadings.

The S3 plot method  creates a heatmap plot to visualize the cosine similarities between cells and principal components (PCs).
}
\details{
This function calculates the cosine similarity between cells based on the loadings of the selected
principal components obtained from PCA. It extracts the rotation matrix from the PCA results of the
\code{\linkS4class{SingleCellExperiment}} object and identifies the high-loading variables for each selected PC.
Then, it computes the cosine similarity between cells using the high-loading variables for each PC.

The S3 plot method reshapes the input data frame to create a long format suitable for plotting as a heatmap. It then
creates a heatmap plot using ggplot2, where the x-axis represents the PCs, the y-axis represents the cells, and the
color intensity represents the cosine similarity values.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Store PCA anomaly data and plots
anomaly_output <- detectAnomaly(reference_data = reference_data,
                                query_data = query_data,
                                ref_cell_type_col = "expert_annotation",
                                query_cell_type_col = "SingleR_annotation",
                                pc_subset = 1:10,
                                n_tree = 500,
                                anomaly_threshold = 0.5)
top6_anomalies <- names(sort(anomaly_output$Combined$reference_anomaly_scores,
                             decreasing = TRUE)[1:6])

# Compute cosine similarity between anomalies and top PCs
cosine_similarities <- calculateCellSimilarityPCA(reference_data,
                                                  cell_names = top6_anomalies,
                                                  pc_subset = 1:25,
                                                  n_top_vars = 50)
cosine_similarities

# Plot similarities
plot(cosine_similarities, pc_subset = 15:25)

}
\seealso{
\code{\link{plot.calculateCellSimilarityPCAObject}}

\code{\link{calculateCellSimilarityPCA}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateTopLoadingGeneShifts.R
\name{processGenesSimple}
\alias{processGenesSimple}
\title{Process Genes for Statistical Analysis}
\usage{
processGenesSimple(
  top_genes,
  top_loadings,
  query_expr_matrix,
  ref_expr_matrix,
  cell_type
)
}
\arguments{
\item{top_genes}{Character vector of gene names to analyze.}

\item{top_loadings}{Numeric vector of PC loading values corresponding to the genes.}

\item{query_expr_matrix}{Numeric matrix of expression values for query data (genes × cells).}

\item{ref_expr_matrix}{Numeric matrix of expression values for reference data (genes × cells).}

\item{cell_type}{Character string specifying the cell type being analyzed.}
}
\value{
A list of data frames, where each data frame contains results for one gene.
}
\description{
Internal helper function that performs statistical testing on top loading genes
to compare their expression distributions between query and reference datasets.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateMMDPValue.R
\name{computeMMDStatistic}
\alias{computeMMDStatistic}
\title{Compute Maximum Mean Discrepancy Statistic}
\usage{
computeMMDStatistic(X, Y, kernel_type = "gaussian", sigma = NULL)
}
\arguments{
\item{X}{A numeric matrix representing the first dataset, where rows are observations
and columns are features.}

\item{Y}{A numeric matrix representing the second dataset, where rows are observations
and columns are features.}

\item{kernel_type}{A character string specifying the kernel type. Options are
"gaussian" for RBF kernel or "linear" for linear kernel. Default is "gaussian".}

\item{sigma}{A numeric value specifying the bandwidth parameter for the Gaussian kernel.
If NULL, it is estimated using the median heuristic. Default is NULL.}
}
\value{
A numeric value representing the MMD^2 statistic between the two datasets.
}
\description{
Compute the Maximum Mean Discrepancy (MMD) statistic between two datasets
using either Gaussian or linear kernels for distribution comparison.
}
\details{
This function calculates the MMD statistic, which measures the distance between
two probability distributions by comparing their embeddings in a reproducing
kernel Hilbert space (RKHS). For the Gaussian kernel, an optimized median
heuristic is used to estimate the bandwidth parameter sigma when not provided.
The linear kernel provides a computationally faster alternative.

The MMD statistic is computed as:
MMD^2 = E[k(X,X')] + E[k(Y,Y')] - 2*E[k(X,Y)]
where k is the chosen kernel function.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateMMDPValue.R
\name{calculateMMDPValue}
\alias{calculateMMDPValue}
\title{Calculate Maximum Mean Discrepancy P-Values for Two-Sample Comparison}
\usage{
calculateMMDPValue(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  pc_subset = 1:5,
  n_permutation = 100,
  kernel_type = "gaussian",
  sigma = NULL,
  assay_name = "logcounts",
  max_cells_query = 5000,
  max_cells_ref = 5000
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.}

\item{pc_subset}{A numeric vector specifying which principal components to include in the plot. Default is PC1 to PC5.}

\item{n_permutation}{Number of permutations for p-value calculation. Default is 100.}

\item{kernel_type}{Type of kernel to use. Options are "gaussian" (default) or "linear".}

\item{sigma}{Bandwidth parameter for Gaussian kernel. If NULL, uses median heuristic.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}
}
\value{
A named vector of p-values from the MMD test for each cell type.
}
\description{
This function performs the Maximum Mean Discrepancy (MMD) test for comparing
distributions between two samples in PCA space using a custom implementation
with permutation testing for better sensitivity.
}
\details{
The function performs the following steps:
\enumerate{
  \item Projects the data into the PCA space.
  \item Subsets the data to the specified cell types and principal components.
  \item Performs a custom MMD test with permutation-based p-values for each cell type.
}
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Calculate MMD p-values (with query data)
mmd_test <- calculateMMDPValue(reference_data = reference_data,
                              query_data = query_data,
                              ref_cell_type_col = "expert_annotation",
                              query_cell_type_col = "SingleR_annotation",
                              cell_types = c("CD4", "CD8"),
                              pc_subset = 1:5,
                              n_permutation = 30)
mmd_test

}
\references{
Gretton, A., Borgwardt, K. M., Rasch, M. J., Schölkopf, B., & Smola, A. (2012).
"A kernel two-sample test". Journal of Machine Learning Research, 13(1), 723-773.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comparePCASubspace.R,
%   R/plot.comparePCASubspaceObject.R
\name{comparePCASubspace}
\alias{comparePCASubspace}
\alias{plot.comparePCASubspaceObject}
\title{Compare Subspaces Spanned by Top Principal Components}
\usage{
comparePCASubspace(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  pc_subset = 1:5,
  n_top_vars = 50
)

\method{plot}{comparePCASubspaceObject}(x, ...)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{pc_subset}{A numeric vector specifying the subset of principal components (PCs) to compare. Default is the first five PCs.}

\item{n_top_vars}{An integer indicating the number of top loading variables to consider for each PC. Default is 50.}

\item{x}{A numeric matrix output from the \code{comparePCASubspace} function, representing
cosine similarities between query and reference principal components.}

\item{...}{Additional arguments passed to the plotting function.}
}
\value{
A list containing the following components:
  \item{cosine_similarity}{A numeric vector of cosine values of principal angles.}
  \item{cosine_id}{A matrix showing which reference and query PCs were matched.}
  \item{var_explained_ref}{A numeric vector of variance explained by reference PCs.}
  \item{var_explained_query}{A numeric vector of variance explained by query PCs.}
  \item{var_explained_avg}{A numeric vector of average variance explained by each PC pair.}
  \item{weighted_cosine_similarity}{A numeric value representing the weighted cosine similarity.}

The S3 plot method returns a \code{ggplot} object representing the cosine similarities with variance information.
}
\description{
This function compares the subspace spanned by the top principal components (PCs) in a reference dataset to that
in a query dataset. It computes the cosine similarity between the loadings of the top variables for each PC in
both datasets and provides a weighted cosine similarity score.

The S3 plot method generates a visualization of the output from the \code{comparePCASubspace} function.
The plot shows the cosine of principal angles between reference and query principal components,
with point sizes representing the variance explained and colors showing the difference in variance between datasets.
}
\details{
This function compares the subspace spanned by the top principal components (PCs) in a reference dataset
to that in a query dataset. It first computes the cosine similarity between the loadings of the top variables
for each PC in both datasets. The top cosine similarity scores are then selected, and their corresponding PC
indices are stored. Additionally, the function calculates the average percentage of variance explained by the
selected top PCs. Finally, it computes a weighted cosine similarity score based on the top cosine similarities
and the average percentage of variance explained.

The S3 plot method converts the input list into a data frame suitable for plotting with \code{ggplot2}.
Each point in the scatter plot represents the cosine of a principal angle, with the size of the point
indicating the average variance explained by the corresponding principal components. The color represents
the difference in variance explained between reference and query datasets.
}
\examples{
# Load libraries
library(scran)
library(scater)

# Load data
data("reference_data")
data("query_data")

# Extract CD4 cells
ref_data_subset <- reference_data[, which(reference_data$expert_annotation == "CD4")]
query_data_subset <- query_data[, which(query_data$expert_annotation == "CD4")]

# Selecting highly variable genes (can be customized by the user)
ref_top_genes <- getTopHVGs(ref_data_subset, n = 500)
query_top_genes <- getTopHVGs(query_data_subset, n = 500)

# Intersect the gene symbols to obtain common genes
common_genes <- intersect(ref_top_genes, query_top_genes)
ref_data_subset <- ref_data_subset[common_genes,]
query_data_subset <- query_data_subset[common_genes,]

# Run PCA on datasets separately
ref_data_subset <- runPCA(ref_data_subset)
query_data_subset <- runPCA(query_data_subset)

# Compare PCA subspaces
subspace_comparison <- comparePCASubspace(query_data = query_data_subset,
                                          reference_data = ref_data_subset,
                                          query_cell_type_col = "expert_annotation",
                                          ref_cell_type_col = "expert_annotation",
                                          n_top_vars = 50,
                                          pc_subset = 1:5)

# Plot output for PCA subspace comparison
plot(subspace_comparison)

}
\seealso{
\code{\link{plot.comparePCASubspaceObject}}

\code{\link{comparePCASubspace}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/projectPCA.R
\name{projectPCA}
\alias{projectPCA}
\title{Project Query Data Onto PCA Space of Reference Data}
\usage{
projectPCA(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  pc_subset = 1:10,
  assay_name = "logcounts",
  max_cells_query = NULL,
  max_cells_ref = NULL
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix
for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix
for the reference cells.}

\item{query_cell_type_col}{character. The column name in the \code{colData} of \code{query_data}
that identifies the cell types.}

\item{ref_cell_type_col}{character. The column name in the \code{colData} of \code{reference_data}
that identifies the cell types.}

\item{cell_types}{A character vector specifying which cell types to retain in the output. If NULL,
no cell type filtering is performed. Default is NULL.}

\item{pc_subset}{A numeric vector specifying the subset of principal components (PCs) to compare. Default is 1:10.}

\item{assay_name}{Name of the assay on which to perform computations. Defaults to \code{"logcounts"}.}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is NULL.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is NULL.}
}
\value{
A \code{data.frame} containing the projected data in rows (reference and query data combined),
optionally filtered by cell types and downsampled. Rownames preserve the original cell names from
the SCE objects.
}
\description{
This function projects a query singleCellExperiment object onto the PCA space of a reference
singleCellExperiment object. The PCA analysis on the reference data is assumed to be pre-computed
and stored within the object. Optionally filters by cell types and downsamples the results.
}
\details{
This function assumes that the "PCA" element exists within the \code{reducedDims} of the reference data
(obtained using \code{reducedDim(reference_data)}) and that the genes used for PCA are present in both
the reference and query data. It performs centering and scaling of the query data based on the reference
data before projection using the FULL datasets to maintain proper mean centering. Cell type filtering
and downsampling are performed AFTER projection to preserve the statistical properties of the PCA space.
Cell names from the original SCE objects are preserved as rownames in the output.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Project the query data onto PCA space of reference
pca_output <- projectPCA(query_data = query_data,
                         reference_data = reference_data,
                         query_cell_type_col = "SingleR_annotation",
                         ref_cell_type_col = "expert_annotation",
                         pc_subset = 1:10)

# Project with cell type filtering and balanced downsampling
pca_output_filtered <- projectPCA(query_data = query_data,
                                  reference_data = reference_data,
                                  query_cell_type_col = "SingleR_annotation",
                                  ref_cell_type_col = "expert_annotation",
                                  pc_subset = 1:5,
                                  cell_types = c("CD4", "CD8"),
                                  max_cells_ref = 1000,
                                  max_cells_query = 1000)

}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateGraphIntegration.R,
%   R/plot.calculateGraphIntegrationObject.R
\name{calculateGraphIntegration}
\alias{calculateGraphIntegration}
\alias{plot.calculateGraphIntegrationObject}
\title{Calculate Graph Community Integration Diagnostics}
\usage{
calculateGraphIntegration(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  pc_subset = 1:10,
  k_neighbors = 30,
  assay_name = "logcounts",
  resolution = 0.1,
  min_cells_per_community = 10,
  min_cells_per_celltype = 20,
  high_query_prop_threshold = 0.9,
  cross_type_threshold = 0.15,
  local_consistency_threshold = 0.6,
  local_confidence_threshold = 0.2,
  max_cells_query = 5000,
  max_cells_ref = 5000
)

\method{plot}{calculateGraphIntegrationObject}(
  x,
  plot_type = c("community_network", "cell_network", "community_data", "summary",
    "local_issues", "annotation_issues"),
  color_by = c("cell_type", "community_type"),
  max_nodes = 2000,
  point_size = 0.8,
  exclude_reference_only = FALSE,
  ...
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{A character string specifying the column name in the query dataset containing cell type annotations.}

\item{ref_cell_type_col}{A character string specifying the column name in the reference dataset containing cell type annotations.}

\item{cell_types}{A character vector specifying the cell types to include in the analysis. If NULL, all cell types are included.}

\item{pc_subset}{A vector specifying the subset of principal components to use in the analysis. Default is 1:10.}

\item{k_neighbors}{An integer specifying the number of nearest neighbors for graph construction. Default is 30.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{resolution}{Resolution parameter for Leiden clustering. Default is 0.15 for fewer, larger communities.}

\item{min_cells_per_community}{Minimum number of cells required for a community to be analyzed. Default is 10.}

\item{min_cells_per_celltype}{Minimum number of cells required per cell type for inclusion. Default is 20.}

\item{high_query_prop_threshold}{Minimum proportion of query cells to consider a community "query-only". Default is 0.9.}

\item{cross_type_threshold}{Minimum proportion needed to flag cross-cell-type mixing. Default is 0.1.}

\item{local_consistency_threshold}{Minimum proportion of reference neighbors that should support a query cell's annotation. Default is 0.6.}

\item{local_confidence_threshold}{Minimum confidence difference needed to suggest re-annotation. Default is 0.2.}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}

\item{x}{An object of class \code{calculateGraphIntegrationObject} containing the diagnostic results.}

\item{plot_type}{Character string specifying visualization type. Options: "community_network" (default),
"cell_network", "community_data", "summary", "local_issues", or "annotation_issues".}

\item{color_by}{Character string specifying the variable to use for coloring points/elements if `plot_type` is
"community_network" or "cell_network". Default is "cell_type".}

\item{max_nodes}{Maximum number of nodes to display for performance. Default is 2000.}

\item{point_size}{Point size for graph nodes. Default is 0.8.}

\item{exclude_reference_only}{Logical indicating whether to exclude reference-only communities/cells from visualization.
Default is FALSE.}

\item{...}{Additional arguments passed to ggplot2 functions.}
}
\value{
A list containing:
\item{high_query_prop_analysis}{Analysis of communities with only query cells}
\item{cross_type_mixing}{Analysis of communities with true query-reference cross-cell-type mixing}
\item{local_annotation_inconsistencies}{Local neighborhood-based annotation inconsistencies}
\item{local_inconsistency_summary}{Summary of local inconsistencies by cell type}
\item{community_composition}{Detailed composition of each community}
\item{annotation_consistency}{Summary of annotation consistency issues}
\item{overall_metrics}{Overall diagnostic metrics}
\item{graph_info}{Graph structure information for plotting}
\item{parameters}{Analysis parameters used}
The list is assigned the class \code{"calculateGraphIntegration"}.

A \code{ggplot} object showing integration diagnostics.
}
\description{
This function performs graph-based community detection to identify annotation inconsistencies
by detecting query-only communities, true cross-cell-type mixing patterns, and local
annotation inconsistencies based on immediate neighborhood analysis.

The S3 plot method generates visualizations of annotation consistency diagnostics,
including query-only communities, cross-cell-type mixing, and local annotation inconsistencies.
}
\details{
The function performs three types of analysis: (1) Communities containing only query cells,
(2) Communities where query cells are mixed with reference cells of different cell types
WITHOUT any reference cells of the same type, and (3) Local analysis of each query cell's
immediate neighbors to detect annotation inconsistencies even within mixed communities.

The S3 plot method creates optimized visualizations showing different types of annotation
issues including community-level and local neighborhood-level inconsistencies.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Remove a cell type (Myeloid)
library(scater)
library(SingleR)
reference_data <- reference_data[, reference_data$expert_annotation != "Myeloid"]
reference_data <- runPCA(reference_data, ncomponents = 50)
SingleR_annotation <- SingleR(query_data, reference_data,
                              labels = reference_data$expert_annotation)
query_data$SingleR_annotation <- SingleR_annotation$labels

# Check annotation data
table(Expert = query_data$expert_annotation, SingleR = query_data$SingleR_annotation)

# Run comprehensive annotation consistency diagnostics
graph_diagnostics <- calculateGraphIntegration(
    query_data = query_data,
    reference_data = reference_data,
    query_cell_type_col = "SingleR_annotation",
    ref_cell_type_col = "expert_annotation",
   pc_subset = 1:10,
   k_neighbors = 30,
    resolution = 0.1,
    high_query_prop_threshold = 0.9,
    cross_type_threshold = 0.15,
    local_consistency_threshold = 0.6,
    local_confidence_threshold = 0.2
)

# Look at main output
graph_diagnostics$overall_metrics

# Network graph showing all issue types (color by cell type)
plot(graph_diagnostics, plot_type = "community_network", color_by = "cell_type")

# Network graph showing all issue types
plot(graph_diagnostics, plot_type = "cell_network",
     max_nodes = 2000, color_by = "community_type")

# Network graph showing all issue types
plot(graph_diagnostics, plot_type = "community_data")

# Summary bar chart of all issues by cell type
plot(graph_diagnostics, plot_type = "summary")

# Focus on local annotation inconsistencies
plot(graph_diagnostics, plot_type = "local_issues")

# Overall annotation issues overview
plot(graph_diagnostics, plot_type = "annotation_issues")

}
\seealso{
\code{\link{calculateGraphIntegration}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateAveragePairwiseCorrelation.R,
%   R/plot.calculateAveragePairwiseCorrelationObject.R
\name{calculateAveragePairwiseCorrelation}
\alias{calculateAveragePairwiseCorrelation}
\alias{plot.calculateAveragePairwiseCorrelationObject}
\title{Compute Average Pairwise Correlation between Cell Types}
\usage{
calculateAveragePairwiseCorrelation(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  pc_subset = 1:10,
  correlation_method = c("spearman", "pearson"),
  assay_name = "logcounts",
  max_cells_query = 5000,
  max_cells_ref = 5000
)

\method{plot}{calculateAveragePairwiseCorrelationObject}(x, ...)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.}

\item{pc_subset}{A numeric vector specifying which principal components to use in the analysis. Default is 1:10.
If set to \code{NULL} then no dimensionality reduction is performed and the assay data is used directly for computations.}

\item{correlation_method}{The correlation method to use for calculating pairwise correlations.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000}

\item{x}{Output matrix from `calculateAveragePairwiseCorrelation` function.}

\item{...}{Additional arguments to be passed to the plotting function.}
}
\value{
A matrix containing the average pairwise correlation values.
        Rows and columns are labeled with the cell types. Each element
        in the matrix represents the average correlation between a pair
        of cell types.

The S3 plot method returns a \code{ggplot} object representing the heatmap plot.
}
\description{
Computes the average pairwise correlations between specified cell types
in single-cell gene expression data.

The S3 plot method takes the output of the `calculateAveragePairwiseCorrelation` function,
which should be a matrix of pairwise correlations, and plots it as a heatmap.
}
\details{
This function operates on \code{\linkS4class{SingleCellExperiment}} objects,
ideal for single-cell analysis workflows. It calculates pairwise correlations between query and
reference cells using a specified correlation method, then averages these correlations for each
cell type pair. This function aids in assessing the similarity between cells in reference and query datasets,
providing insights into the reliability of cell type annotations in single-cell gene expression data.

The S3 plot method converts the correlation matrix into a dataframe, creates a heatmap using ggplot2,
and customizes the appearance of the heatmap with updated colors and improved aesthetics.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Compute pairwise correlations
cor_matrix_avg <- calculateAveragePairwiseCorrelation(query_data = query_data,
                                                      reference_data = reference_data,
                                                      query_cell_type_col = "SingleR_annotation",
                                                      ref_cell_type_col = "expert_annotation",
                                                      cell_types = c("CD4", "CD8", "B_and_plasma"),
                                                      pc_subset = 1:10,
                                                      correlation_method = "spearman")

# Visualize correlation output
plot(cor_matrix_avg)

}
\seealso{
\code{\link{plot.calculateAveragePairwiseCorrelationObject}}

\code{\link{calculateAveragePairwiseCorrelation}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateTopLoadingGeneShifts.R,
%   R/plot.calculateTopLoadingGeneShiftsObject.R
\name{calculateTopLoadingGeneShifts}
\alias{calculateTopLoadingGeneShifts}
\alias{plot.calculateTopLoadingGeneShiftsObject}
\title{Calculate Top Loading Gene Expression Shifts}
\usage{
calculateTopLoadingGeneShifts(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  pc_subset = 1:5,
  n_top_loadings = 50,
  p_value_threshold = 0.05,
  adjust_method = "fdr",
  assay_name = "logcounts",
  detect_anomalies = FALSE,
  anomaly_comparison = FALSE,
  anomaly_threshold = 0.6,
  n_tree = 500,
  max_cells_query = 5000,
  max_cells_ref = 5000
)

\method{plot}{calculateTopLoadingGeneShiftsObject}(
  x,
  cell_type,
  pc_subset = 1:3,
  plot_type = c("heatmap", "barplot", "boxplot"),
  plot_by = c("p_adjusted", "top_loading"),
  n_genes = 10,
  significance_threshold = 0.05,
  show_anomalies = FALSE,
  pseudo_bulk = FALSE,
  cluster_cols = FALSE,
  draw_plot = TRUE,
  show_all_query = TRUE,
  max_cells_ref = NULL,
  max_cells_query = NULL,
  ...
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{cell_types}{A character vector specifying the cell types to analyze. If NULL, all common cell types are used.}

\item{pc_subset}{A numeric vector specifying which principal components to plot. Default is 1:3.}

\item{n_top_loadings}{Number of top loading genes to analyze per PC. Default is 50.}

\item{p_value_threshold}{P-value threshold for statistical significance. Default is 0.05.}

\item{adjust_method}{Method for multiple testing correction. Default is "fdr".}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{detect_anomalies}{Logical indicating whether to perform anomaly detection using isolation forests.
Default is FALSE.}

\item{anomaly_comparison}{Logical indicating whether to perform statistical comparisons
between non-anomalous reference cells and anomalous query cells instead of all-vs-all
comparisons. When TRUE, only non-anomalous reference cells are compared against only
anomalous query cells for each cell type. Requires detect_anomalies = TRUE. Default is FALSE.}

\item{anomaly_threshold}{A numeric value specifying the threshold for identifying anomalies when
\code{detect_anomalies} is TRUE. Default is 0.6.}

\item{n_tree}{An integer specifying the number of trees for the isolation forest when
\code{detect_anomalies} is TRUE. Default is 500.}

\item{max_cells_query}{Maximum number of query cells to include in the plot. If NULL,
all available query cells are plotted. Default is NULL.}

\item{max_cells_ref}{Maximum number of reference cells to include in the plot. If NULL,
all available reference cells are plotted. Default is NULL.}

\item{x}{An object of class \code{calculateTopLoadingGeneShiftsObject}.}

\item{cell_type}{A character string specifying the cell type to plot (must be exactly one).}

\item{plot_type}{A character string specifying visualization type. Either "heatmap", "barplot", or "boxplot".
Default is "heatmap".}

\item{plot_by}{A character string specifying gene selection method when `n_genes` is not NULL.
Either "top_loading" or "p_adjusted". Default is "p_adjusted".}

\item{n_genes}{Number of top genes to show per PC. Can be NULL if `significance_threshold` is set.
Default is 10.}

\item{significance_threshold}{If not NULL, a numeric value between 0 and 1. Used for gene
selection or annotation. Default is 0.05.}

\item{show_anomalies}{Logical indicating whether to display anomaly status annotations.
Default is FALSE. Requires anomaly results to be present in the object.}

\item{pseudo_bulk}{Logical indicating whether to create pseudo-bulk profiles instead of
showing individual cells. When TRUE, expression values are averaged within groups
(dataset and optionally anomaly status). Not compatible with boxplot visualization.
Required for barplot visualization. Default is FALSE.}

\item{cluster_cols}{Logical indicating whether to cluster columns in the heatmap when
`pseudo_bulk = TRUE`. When TRUE, columns (pseudo-bulk profiles) will be
hierarchically clustered. When FALSE, columns maintain their original ordering
(Query groups followed by Reference groups). Only applicable when
`pseudo_bulk = TRUE` and `plot_type = "heatmap"`. Default is FALSE.}

\item{draw_plot}{Logical indicating whether to draw the plot immediately (TRUE) or return
the undrawn plot object (FALSE). For heatmaps, FALSE returns a ComplexHeatmap
object that can be further customized before drawing. Default is TRUE.}

\item{show_all_query}{Logical indicating whether to show the yellow bar for all query vs reference
comparison. Default is TRUE. When FALSE, only green and red bars are shown.}

\item{...}{Additional arguments passed to \code{\link[ComplexHeatmap]{draw}} or not used for other plot types.}
}
\value{
A list containing:
\itemize{
  \item PC results: Named elements for each PC (e.g., "PC1", "PC2") containing data frames with gene-level analysis results.
  \item expression_data: Matrix of expression values for all analyzed genes (genes × cells).
  \item cell_metadata: Data frame with columns: cell_id, dataset, cell_type, original_index, and optionally anomaly_status.
  \item gene_metadata: Data frame with columns: gene, pc, loading for all analyzed genes.
  \item percent_var: Named numeric vector of global percent variance explained for each analyzed PC.
  \item cell_type_variance: A data frame detailing the percent of variance a global PC explains within specific cell types for both query and reference datasets.
  \item anomaly_results: If \code{detect_anomalies} is TRUE, contains the full output from \code{detectAnomaly}.
}

The `cell_type_variance` data frame contains columns: pc, cell_type, dataset, percent_variance.
When anomaly detection is enabled, `cell_metadata` includes an additional `anomaly_status` column.

A plot object. For heatmaps when \code{draw_plot = FALSE}, returns a ComplexHeatmap object.
For boxplots and barplots, returns a ggplot2 object.
}
\description{
This function identifies genes with the highest loadings for specified principal components
and performs statistical tests to detect distributional differences between query and reference data.
It also calculates the proportion of variance explained by each principal component within
specific cell types. Optionally, it can detect anomalous cells using isolation forests.

This function creates visualizations showing expression distributions for top loading genes
that exhibit distributional differences between query and reference datasets. Can display
results as elegant complex heatmaps, information-rich summary boxplots, or pseudo-bulk fold
change barplots. Optionally displays anomaly status when available.
}
\details{
This function extracts the top loading genes for each specified principal component from the reference
PCA space and performs distributional comparisons between query and reference data. For each gene,
it performs statistical tests to identify genes that may be causing PC-specific alignment issues
between datasets. A key feature is the calculation of cell-type-specific variance explained by
global PCs, providing a more nuanced view of how major biological axes affect individual populations.
When anomaly detection is enabled, isolation forests are used to identify anomalous cells based on
their PCA projections.

When \code{anomaly_comparison = TRUE}, the statistical analysis focuses specifically on
comparing non-anomalous reference cells against anomalous query cells. This can help
identify genes that are differentially expressed between "normal" reference cells and
potentially problematic query cells, providing insights into what makes certain query
cells anomalous.

This function visualizes the results from \code{calculateTopLoadingGeneShifts}.
The "heatmap" option displays a hierarchically clustered set of genes.
The "boxplot" option creates a two-panel plot using `ggplot2`: the left panel shows
horizontal expression boxplots for up to 5 PCs, while the right panel displays their
corresponding PC loadings and adjusted p-values.
The "barplot" option creates horizontal barplots showing log2 fold changes between
pseudo-bulk expression profiles (query vs reference), with genes ordered identically
to the heatmap clustering. Bars show comparisons for query non-anomaly (green),
optionally all query cells (yellow), and query anomaly cells (red) versus reference.
When anomaly detection results are available and \code{show_anomalies} is TRUE,
additional annotation bars or visual cues highlight anomalous cells.
}
\seealso{
\code{\link{plot.calculateTopLoadingGeneShiftsObject}}, \code{\link{detectAnomaly}}

\code{\link{calculateTopLoadingGeneShifts}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareMarkers.R, R/plot.compareMarkersObject.R
\name{compareMarkers}
\alias{compareMarkers}
\alias{plot.compareMarkersObject}
\title{Compare Marker Gene Expression between Query and Reference Data}
\usage{
compareMarkers(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  n_markers = 50,
  min_cells = 10,
  anomaly_filter = c("none", "anomalous_only", "non_anomalous_only"),
  assay_name = "logcounts",
  max_cells_query = 5000,
  max_cells_ref = 5000,
  ...
)

\method{plot}{compareMarkersObject}(x, cell_types = NULL, ...)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing reference cells.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{cell_types}{Character vector specifying which cell types to plot. If NULL, all cell types are plotted.}

\item{n_markers}{Number of top marker genes to consider for each cell type. Default is 50.}

\item{min_cells}{Minimum number of cells required per cell type for marker identification. Default is 10.}

\item{anomaly_filter}{Character string specifying how to filter query cells based on anomaly detection.
Options: "none" (default), "anomalous_only", "non_anomalous_only".}

\item{assay_name}{Name of the assay to use for computations. Default is "logcounts".}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}

\item{...}{Additional arguments passed to the plotting function.}

\item{x}{A list containing the output from the \code{compareMarkers} function.}
}
\value{
A list containing the following elements:
\describe{
  \item{marker_overlap}{Matrix showing overlap of top markers between query and reference for each cell type.}
  \item{expression_consistency}{Matrix showing expression consistency of reference markers in query data.}
  \item{quality_scores}{Named vector of quality assessments for each cell type.}
  \item{markers_query}{List of marker gene results for each cell type in query data.}
  \item{markers_ref}{List of marker gene results for each cell type in reference data.}
  \item{common_cell_types}{Vector of cell types present in both datasets.}
  \item{n_cells_query}{Named vector of cell counts per type in query data.}
  \item{n_cells_ref}{Named vector of cell counts per type in reference data.}
  \item{anomaly_filter_used}{Character string indicating the anomaly filter applied.}
  \item{selected_cell_types}{Character vector of cell types analyzed.}
  \item{anomaly_output}{Output from anomaly detection if performed.}
}

The S3 plot method returns a \code{ggplot} object representing the marker gene comparison results.
}
\description{
This function identifies marker genes for each cell type in both query and reference datasets
using the standard Bioconductor approach (Wilcoxon rank-sum test), and compares their expression
patterns to assess annotation quality. It can optionally filter query cells based on anomaly detection
results and restrict analysis to specific cell types.

The S3 plot method generates a comprehensive visualization of the output from the `compareMarkers` function.
The plot shows marker gene overlap and expression consistency between query and reference cell types,
with quality assessment and detailed annotations.
}
\details{
The function performs the following steps:
1. Optionally performs anomaly detection and filters query cells based on results.
2. Identifies marker genes for each cell type in both datasets using \code{findMarkers} approach.
3. Reference markers are always computed using all reference cells for each cell type.
4. Query markers are computed using filtered cells (anomalous/non-anomalous) if specified.
5. Compares the overlap of top marker genes between corresponding cell types.
6. Evaluates the expression consistency of reference markers in query data.
7. Provides quality scores based on marker gene concordance.

Marker genes are identified using Wilcoxon rank-sum tests comparing each cell type against all others.
High overlap and consistent expression of markers indicate good annotation quality.

The S3 plot method creates a scatter plot showing the relationship between marker overlap (x-axis)
and expression consistency (y-axis) for each cell type. Points are colored by quality score and
sized by the minimum number of cells. Quality zones provide visual guidance for interpretation.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Compare marker genes
marker_comparison <- compareMarkers(query_data = query_data,
                                    reference_data = reference_data,
                                    query_cell_type_col = "expert_annotation",
                                    ref_cell_type_col = "expert_annotation")

# With anomaly filtering
marker_comparison_filtered <- compareMarkers(query_data = query_data,
                                            reference_data = reference_data,
                                            query_cell_type_col = "expert_annotation",
                                            ref_cell_type_col = "expert_annotation",
                                            anomaly_filter = "non_anomalous_only")

# Visualize results
plot(marker_comparison)

}
\seealso{
\code{\link{plot.compareMarkersObject}}, \code{\link{detectAnomaly}}

\code{\link{compareMarkers}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.calculateTopLoadingGeneShiftsObject.R
\name{extractGeneOrder}
\alias{extractGeneOrder}
\title{Extract Gene Order from ComplexHeatmap Object}
\usage{
extractGeneOrder(heatmap_object)
}
\arguments{
\item{heatmap_object}{A ComplexHeatmap Heatmap object (undrawn or drawn).}
}
\value{
A character vector of gene names in the clustered order (top to bottom as they appear in heatmap).
}
\description{
Extracts the hierarchically clustered gene order from a ComplexHeatmap Heatmap object.
}
\details{
This function initializes a ComplexHeatmap object (without displaying it) to perform
hierarchical clustering, then extracts the resulting gene order. This allows matching
gene ordering between heatmap and other plot types. The function uses a null graphics
device to initialize the heatmap clustering without actually drawing the plot.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selectCellTypes.R
\name{selectCellTypes}
\alias{selectCellTypes}
\title{Cell Type Selection and Validation for SingleCellExperiment Analysis}
\usage{
selectCellTypes(
  query_data = NULL,
  reference_data = NULL,
  query_cell_type_col = NULL,
  ref_cell_type_col = NULL,
  cell_types = NULL,
  dual_only = FALSE,
  n_cell_types = NULL
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
Can be \code{NULL} if only reference data is available.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
Can be \code{NULL} if only query data is available.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data}
that identifies the cell types. Should be \code{NULL} if \code{query_data} is \code{NULL}.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data}
that identifies the cell types. Should be \code{NULL} if \code{reference_data} is \code{NULL}.}

\item{cell_types}{A character vector specifying the cell types to validate. If \code{NULL},
cell types will be automatically selected based on dataset availability and \code{dual_only} setting.}

\item{dual_only}{A logical value indicating whether cell types must be present in both datasets.
If \code{TRUE}, both \code{query_data} and \code{reference_data} must be provided, and only
cell types present in both datasets will be considered. Default is \code{FALSE}.}

\item{n_cell_types}{An integer specifying the maximum number of cell types to select based on
highest cell count. If \code{NULL}, all valid cell types are returned. Default is \code{NULL}.}
}
\value{
A character vector of selected cell types that meet the specified criteria.
}
\description{
This function selects and validates cell types for functions that analyze \code{\linkS4class{SingleCellExperiment}}
objects. It determines which cell types to include based on availability in datasets, applies filtering
criteria, and optionally selects the top cell types by cell count.
}
\details{
The function performs the following selection and validation steps:
\itemize{
 \item Validates that at least one of \code{query_data} or \code{reference_data} is provided.
 \item When \code{dual_only} is TRUE, ensures both datasets are provided.
 \item Determines available cell types based on dataset availability and \code{dual_only} setting.
 \item If \code{cell_types} is NULL and both datasets are available, includes cell types based on \code{dual_only}.
 \item If \code{cell_types} is NULL and only one dataset is available, includes all cell types from that dataset.
 \item If \code{cell_types} is provided, filters to include only valid types.
 \item If \code{n_cell_types} is specified, selects the top cell types by total cell count.
 \item Returns the selected and validated cell types as character strings.
}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.regressPCObject.R
\name{plotCoefficientHeatmap}
\alias{plotCoefficientHeatmap}
\title{Generate Regression Coefficients Heatmap}
\usage{
plotCoefficientHeatmap(x, alpha = 0.05, coefficients_include = NULL, ...)
}
\arguments{
\item{x}{An object of class \code{regressPCObject} containing regression results
from \code{regressPC} function.}

\item{alpha}{Numeric value specifying the significance threshold for p-value
adjustment. Default is 0.05.}

\item{coefficients_include}{Character vector specifying which coefficient types to include.
Options are \code{c("cell_type", "batch", "interaction")}. Default is \code{NULL},
which includes all available coefficient types.}

\item{...}{Additional arguments passed to the plotting function (currently unused).}
}
\value{
A \code{ggplot2} object representing the regression coefficients heatmap
  with faceted organization and significance annotations.
}
\description{
Creates a heatmap visualization of regression coefficients from principal component
regression analysis, organized by coefficient type (cell type, batch, interaction)
and annotated with significance indicators.
}
\details{
This function generates a comprehensive heatmap showing regression coefficients
for each principal component and model term. The visualization includes:
\itemize{
  \item Color-coded coefficient values (blue = negative, red = positive)
  \item Significance indicators (asterisks) for adjusted p-values below threshold
  \item Faceted organization by coefficient category (Cell Type, Batch, Interaction)
  \item Clean term labels with proper formatting and reference category information
}

The function handles different model types automatically:
\itemize{
  \item Simple cell type models: \code{PC ~ cell_type}
  \item Batch interaction models: \code{PC ~ cell_type * batch}
  \item Dataset interaction models: \code{PC ~ cell_type * dataset}
}

Term labels are cleaned and formatted for better readability, with batch/dataset
terms converted to consistent "Query Batch" terminology. The plot includes
comprehensive subtitle information showing model specification, significance
threshold, and reference categories.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCramerPValue.R
\name{calculateCramerPValue}
\alias{calculateCramerPValue}
\title{Calculate Cramer Test P-Values for Two-Sample Comparison of Multivariate ECDFs}
\usage{
calculateCramerPValue(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  pc_subset = 1:5,
  assay_name = "logcounts",
  max_cells_query = 5000,
  max_cells_ref = 5000
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.}

\item{pc_subset}{A numeric vector specifying which principal components to include in the plot. Default is PC1 to PC5.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}
}
\value{
A named vector of p-values from the Cramer test for each cell type.
}
\description{
This function performs the Cramer test for comparing multivariate empirical cumulative distribution functions (ECDFs)
between two samples.
}
\details{
The function performs the following steps:
\enumerate{
  \item Projects the data into the PCA space.
  \item Subsets the data to the specified cell types and principal components.
  \item Performs the Cramer test for each cell type using the \code{cramer.test} function in the \code{cramer} package.
}
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Plot the PC data (with query data)
cramer_test <- calculateCramerPValue(reference_data = reference_data,
                                     query_data = query_data,
                                     ref_cell_type_col = "expert_annotation",
                                     query_cell_type_col = "SingleR_annotation",
                                     cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                                     pc_subset = 1:5)
cramer_test

}
\references{
Baringhaus, L., & Franz, C. (2004). "On a new multivariate two-sample test".
Journal of Multivariate Analysis, 88(1), 190-206.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convertColumnsToCharacter.R
\name{convertColumnsToCharacter}
\alias{convertColumnsToCharacter}
\title{Convert Specified Columns to Character in SingleCellExperiment Objects}
\usage{
convertColumnsToCharacter(sce_object, convert_cols)
}
\arguments{
\item{sce_object}{A \code{\linkS4class{SingleCellExperiment}} object containing single-cell data.}

\item{convert_cols}{A character vector specifying the column names in \code{colData} to convert
to character type. All specified columns must exist in the \code{colData}.}
}
\value{
A \code{\linkS4class{SingleCellExperiment}} object with the specified columns converted
to character type in the \code{colData}.
}
\description{
This function converts specified columns in the \code{colData} of a \code{\linkS4class{SingleCellExperiment}}
object to character type. It checks that the specified columns exist and only performs conversion
when necessary (i.e., when columns are not already character type).
}
\details{
The function performs the following operations:
\itemize{
 \item Validates that the input is a \code{\linkS4class{SingleCellExperiment}} object.
 \item Checks that all specified columns exist in the \code{colData} of the object.
 \item Converts each specified column to character type if it is not already character.
 \item Returns the modified \code{\linkS4class{SingleCellExperiment}} object.
 \item If all specified columns are already character type, returns the object unchanged.
}

This function is particularly useful for handling factor columns that need to be converted
to character for downstream analysis functions that expect character input.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotQCvsAnnotation.R
\name{plotQCvsAnnotation}
\alias{plotQCvsAnnotation}
\title{Scatter plot: QC stats vs Cell Type Annotation Scores}
\usage{
plotQCvsAnnotation(
  sce_object,
  cell_type_col,
  cell_types = NULL,
  qc_col,
  score_col,
  max_cells = 5000
)
}
\arguments{
\item{sce_object}{A \code{\linkS4class{SingleCellExperiment}} containing the single-cell
expression data and metadata.}

\item{cell_type_col}{The column name in the \code{colData} of \code{sce_object}
that contains the cell type labels.}

\item{cell_types}{A vector of cell type labels to plot (e.g., c("T-cell", "B-cell")).
Defaults to \code{NULL}, which will include all the cells.}

\item{qc_col}{A column name in the \code{colData} of \code{sce_object} that
contains the QC stats of interest.}

\item{score_col}{The column name in the \code{colData} of \code{sce_object} that
contains the cell type annotation scores.}

\item{max_cells}{Maximum number of cells to retain. If the object has fewer cells, it is returned unchanged.
Default is 5000.}
}
\value{
A ggplot object displaying a scatter plot of QC stats vs annotation scores,
        where each point represents a cell, color-coded by its cell type.
}
\description{
Creates a scatter plot to visualize the relationship between QC stats (e.g., library size)
and cell type annotation scores for one or more cell types.
}
\details{
This function generates a scatter plot to explore the relationship between various quality
control (QC) statistics, such as library size and mitochondrial percentage, and cell type
annotation scores. By examining these relationships, users can assess whether specific QC
metrics, systematically influence the confidence in cell type annotations,
which is essential for ensuring reliable cell type annotation.
}
\examples{
# Load data
data("qc_data")

# Remove cell types with very few cells
qc_data_subset <- qc_data[, !(qc_data$SingleR_annotation
                              \%in\% c("Chondrocytes", "DC",
                                     "Neurons","Platelets"))]

p1 <- plotQCvsAnnotation(sce_object = qc_data_subset,
                         cell_type_col = "SingleR_annotation",
                         cell_types = NULL,
                         qc_col = "total",
                         score_col = "annotation_scores")
p1 + ggplot2::xlab("Library Size")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMarkerExpression.R
\name{plotMarkerExpression}
\alias{plotMarkerExpression}
\title{Plot gene expression distribution from overall and cell type-specific perspective}
\usage{
plotMarkerExpression(
  query_data,
  reference_data,
  ref_cell_type_col,
  query_cell_type_col,
  cell_type,
  gene_name,
  assay_name = "logcounts",
  normalization = c("z_score", "min_max", "rank", "none"),
  max_cells_query = NULL,
  max_cells_ref = NULL
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{cell_type}{A cell type to plot (e.g., c("T-cell", "B-cell")).}

\item{gene_name}{The gene name for which the distribution is to be visualized.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{normalization}{Method for normalizing expression values. Options: "z_score" (default), "min_max", "rank", "none".}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is NULL.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is NULL.}
}
\value{
A ggplot object containing density plots comparing reference and query distributions.
}
\description{
This function generates density plots to visualize the distribution of gene expression values
for a specific gene across the overall dataset and within a specified cell type.
}
\details{
This function generates density plots to compare the distribution of a specific marker
gene between reference and query datasets. The aim is to inspect the alignment of gene expression
levels as a surrogate for dataset similarity. Similar distributions suggest a good alignment,
while differences may indicate discrepancies or incompatibilities between the datasets.

Multiple normalization options are available:
- "z_score": Standard z-score normalization within each dataset
- "min_max": Min-max scaling to [0,1] range within each dataset
- "rank": Maps values to quantile ranks (0-100 scale)
- "none": No transformation (preserves original scale differences)
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Note: Users can use SingleR or any other method to obtain the cell type annotations.
plotMarkerExpression(reference_data = reference_data,
                     query_data = query_data,
                     ref_cell_type_col = "expert_annotation",
                     query_cell_type_col = c("expert_annotation", "SingleR_annotation")[1],
                     gene_name = "CD8A",
                     cell_type = "CD4",
                     normalization = "z_score")

}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDocumentation.R
\docType{data}
\name{qc_data}
\alias{qc_data}
\title{Quality Control Single-Cell RNA-Seq Dataset}
\format{
An object of class \code{SingleCellExperiment} with 500 rows and 750 columns.
}
\source{
Bunis DG et al. (2021). Single-Cell Mapping of Progressive Fetal-to-Adult Transition in Human Naive T Cells Cell Rep. 34(1): 108573
}
\usage{
qc_data
}
\description{
This dataset contains the processed query dataset from the Bunis haematopoietic stem and progenitor cell data.
It has been preprocessed to include log-normalized counts, QC metrics, SingleR cell type predictions,
and annotation scores.
}
\details{
This dataset underwent the following steps:
\itemize{
  \item Loads the \code{hpca} reference dataset using \code{fetchReference} from the \code{celldex} package.
  \item Loads the QC dataset (Bunis haematopoietic stem and progenitor cell data) from Bunis DG et al. (2021).
  \item Adds QC metrics to the QC dataset using the function \code{addPerCellQCMetrics} from the \code{scuttle} package.
  \item Performs log normalization on the QC dataset using the function \code{logNormCounts} from the \code{scuttle} package.
  \item Runs SingleR to predict cell types and assigns predicted labels to the QC dataset using the function \code{SingleR} from the \code{SingleR} package.
  \item Assigns annotation scores to the QC dataset.
  \item Selects specific columns (\code{total}, \code{SingleR_annotation}, \code{annotation_scores}) from the cell metadata for downstream analysis.
  \item Selects highly variable genes (HVGs) using the function \code{getTopHVGs} from the \code{scran} package on the QC dataset.
}
}
\examples{
# Load and explore the quality control dataset
data("qc_data")

}
\references{
Bunis DG et al. (2021). Single-Cell Mapping of Progressive Fetal-to-Adult Transition in Human Naive T Cells Cell Rep. 34(1): 108573
}
\seealso{
Use \code{data("qc_data")} to load and access the resulting quality control dataset.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCategorizationEntropy.R
\name{inverseNormalTransformation}
\alias{inverseNormalTransformation}
\title{Inverse Normal Transformation}
\usage{
inverseNormalTransformation(X, constant = 3/8)
}
\arguments{
\item{X}{A numeric matrix or vector.}

\item{constant}{A numeric value used in the transformation. Default is \code{3 / 8}.}
}
\value{
A matrix or vector with the same dimensions as \code{X}, with values transformed using the inverse normal transformation.
}
\description{
This function performs an inverse normal transformation on a matrix or vector.
}
\details{
The function ranks the elements of \code{X} and then applies the inverse normal transformation using the formula \eqn{qnorm((rank - constant) / (n - 2 * constant + 1))}.
}
\author{
Andrew Ghazi, \email{andrew_ghazi@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateDiscriminantSpace.R,
%   R/plot.calculateDiscriminantSpaceObject.R
\name{calculateDiscriminantSpace}
\alias{calculateDiscriminantSpace}
\alias{plot.calculateDiscriminantSpaceObject}
\title{Project Query Data onto a Unified Discriminant Space of Reference Data}
\usage{
calculateDiscriminantSpace(
  reference_data,
  query_data = NULL,
  ref_cell_type_col,
  query_cell_type_col = NULL,
  cell_types = NULL,
  n_tree = 500,
  n_top = 20,
  eigen_threshold = 0.1,
  calculate_metrics = FALSE,
  alpha = 0.01,
  assay_name = "logcounts",
  max_cells_ref = NULL,
  max_cells_query = NULL
)

\method{plot}{calculateDiscriminantSpaceObject}(
  x,
  cell_types = NULL,
  dv_subset = NULL,
  lower_facet = c("scatter", "contour", "ellipse", "blank"),
  diagonal_facet = c("ridge", "density", "boxplot", "blank"),
  upper_facet = c("blank", "scatter", "contour", "ellipse"),
  max_cells_ref = NULL,
  max_cells_query = NULL,
  ...
)
}
\arguments{
\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
If NULL, only the projected reference data is returned. Default is NULL.}

\item{ref_cell_type_col}{The column name in \code{reference_data} indicating cell type labels.}

\item{query_cell_type_col}{The column name in \code{query_data} indicating cell type labels.}

\item{cell_types}{A character vector specifying the cell types to plot. If NULL (default), all cell types will be plotted.}

\item{n_tree}{An integer specifying the number of trees for the random forest used in variable importance calculation.}

\item{n_top}{An integer specifying the number of top variables to select based on importance scores from each pairwise comparison.}

\item{eigen_threshold}{A numeric value specifying the threshold for retaining eigenvalues in discriminant analysis.}

\item{calculate_metrics}{Parameter to determine if cosine similarity and Mahalanobis distance metrics should be computed. Default is FALSE.}

\item{alpha}{A numeric value specifying the significance level for Mahalanobis distance cutoff.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_ref}{Maximum number of reference cells to include in the plot. If NULL,
all available reference cells are plotted. Default is NULL.}

\item{max_cells_query}{Maximum number of query cells to include in the plot. If NULL,
all available query cells are plotted. Default is NULL.}

\item{x}{An object of class \code{calculateDiscriminantSpaceObject} containing the projected data on the discriminant space.}

\item{dv_subset}{A numeric vector specifying which discriminant vectors to include in the plot.
Default is the number of cell types minus 1.}

\item{lower_facet}{Type of plot to use for the lower panels. Either "scatter" (default),
"contour", "ellipse", or "blank".}

\item{diagonal_facet}{Type of plot to use for the diagonal panels. Either "ridge" (default),
"density", "boxplot" or "blank".}

\item{upper_facet}{Type of plot to use for the upper panels. Either "blank" (default),
"scatter", "contour", or "ellipse".}

\item{...}{Additional arguments to be passed to the plotting functions.}
}
\value{
A list with the following components:
\item{discriminant_eigenvalues}{Eigenvalues from the discriminant analysis.}
\item{discriminant_eigenvectors}{Eigenvectors from the discriminant analysis.}
\item{ref_proj}{Reference data projected onto the discriminant space.}
\item{query_proj}{Query data projected onto the discriminant space (if query_data is provided).}
\item{query_mahalanobis_dist}{Mahalanobis distances of query projections (if calculate_metrics is TRUE).}
\item{mahalanobis_crit}{Cutoff value for Mahalanobis distance significance (if calculate_metrics is TRUE).}
\item{query_cosine_similarity}{Cosine similarity scores of query projections (if calculate_metrics is TRUE).}

The S3 plot method returns a \code{GGally::ggpairs} object representing
        the visualization of the projected discriminant space.
}
\description{
This function projects query single-cell RNA-seq data onto a unified discriminant space defined by reference data. The reference data
is used to identify important variables across all cell types and compute discriminant vectors, which are then used to project both reference and query
data. Similarity between the query and reference projections can be assessed using cosine similarity and Mahalanobis distance.

The S3 plot method visualizes the projected reference and query data on the unified discriminant space.
}
\details{
The function performs the following steps:
\itemize{
  \item Identifies the top important variables to distinguish cell types from the reference data by taking the union of important variables from pairwise comparisons.
  \item Computes the Ledoit-Wolf shrinkage estimate of the covariance matrix for each cell type using these important genes.
  \item Constructs within-class and between-class scatter matrices.
  \item Solves the generalized eigenvalue problem to obtain discriminant vectors.
  \item Projects both reference and query data onto the unified discriminant space.
  \item Assesses similarity of the query data projection to the reference data using cosine similarity and Mahalanobis distance.
}

The S3 plot method generates a pairs plot visualization of discriminant vectors, similar to PCA plot visualization.
Each panel shows the relationship between two discriminant vectors with customizable display options for lower,
diagonal, and upper panels. The visualization allows for comprehensive examination of the discriminant space
structure and cell type separability.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Compute discriminant space using unified model across all cell types
disc_output <- calculateDiscriminantSpace(reference_data = reference_data,
                                          query_data = query_data,
                                          query_cell_type_col = "SingleR_annotation",
                                          ref_cell_type_col = "expert_annotation",
                                          n_tree = 500,
                                          n_top = 50,
                                          eigen_threshold  = 1e-1,
                                          calculate_metrics = FALSE,
                                          alpha = 0.01)

# Generate scatter and boxplot
plot(disc_output, plot_type = "scatterplot")
plot(disc_output, cell_types = c("CD4", "CD8"), plot_type = "boxplot")

# Check comparison
table(Expert_Annotation = query_data$expert_annotation,
      SingleR = query_data$SingleR_annotation)

}
\references{
\itemize{
\item Fisher, R. A. (1936). "The Use of Multiple Measurements in Taxonomic Problems". *Annals of Eugenics*. 7 (2): 179–188. doi:10.1111/j.1469-1809.1936.tb02137.x.
\item Hastie, T., Tibshirani, R., & Friedman, J. (2009). *The Elements of Statistical Learning: Data Mining, Inference, and Prediction*. Springer. Chapter 4: Linear Methods for Classification.
\item Ledoit, O., & Wolf, M. (2004). "A well-conditioned estimator for large-dimensional covariance matrices". *Journal of Multivariate Analysis*. 88 (2): 365–411. doi:10.1016/S0047-259X(03)00096-4.
\item De Maesschalck, R., Jouan-Rimbaud, D., & Massart, D. L. (2000). "The Mahalanobis distance". *Chemometrics and Intelligent Laboratory Systems*. 50 (1): 1–18. doi:10.1016/S0169-7439(99)00047-7.
\item Breiman, L. (2001). "Random Forests". *Machine Learning*. 45 (1): 5–32. doi:10.1023/A:1010933404324.
}
}
\seealso{
\code{\link{plot.calculateDiscriminantSpaceObject}}

\code{\link{calculateDiscriminantSpace}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.calculateTopLoadingGeneShiftsObject.R
\name{plotHeatmap}
\alias{plotHeatmap}
\title{Plot Heatmaps for Top Loading Gene Shifts (Simplified Single Heatmap)}
\usage{
plotHeatmap(
  x,
  cell_type,
  available_pcs,
  plot_by,
  n_genes,
  significance_threshold,
  show_anomalies,
  pseudo_bulk = FALSE,
  cluster_cols = FALSE
)
}
\arguments{
\item{x}{An object of class \code{calculateTopLoadingGeneShiftsObject} containing
expression data and analysis results.}

\item{cell_type}{A character string specifying the cell type to visualize.}

\item{available_pcs}{A character vector of principal components to include in analysis.}

\item{plot_by}{A character string indicating gene selection criterion ("top_loading" or "p_adjusted").}

\item{n_genes}{An integer specifying the number of top genes to display per PC. Can be NULL.}

\item{significance_threshold}{A numeric value between 0 and 1 for significance filtering. Can be NULL.}

\item{show_anomalies}{Logical indicating whether to show anomaly annotations.}

\item{pseudo_bulk}{Logical indicating whether to create pseudo-bulk profiles instead of
showing individual cells. When TRUE, expression values are averaged within groups
(dataset and optionally anomaly status). Not compatible with boxplot visualization.
Default is FALSE.}

\item{cluster_cols}{Logical indicating whether to cluster columns in the heatmap when
`pseudo_bulk = TRUE`. When TRUE, columns (pseudo-bulk profiles) will be
hierarchically clustered. When FALSE, columns maintain their original ordering
(Query groups followed by Reference groups). Only applicable when
`pseudo_bulk = TRUE` and `plot_type = "heatmap"`. Default is FALSE.}
}
\value{
A ComplexHeatmap object ready for plotting, or NULL if no genes meet selection criteria.
}
\description{
This internal helper function creates a single, hierarchically clustered heatmap
displaying expression data for top loading genes from principal component analysis.
The function handles gene selection, data preprocessing, and visualization formatting.
Optionally includes anomaly status annotations with proper cell ordering.
}
\details{
This function generates a ComplexHeatmap visualization showing scaled gene expression
data across cells. Genes are selected based on either their loading values or
statistical significance. The function automatically handles data filtering,
scaling, and visual formatting including color schemes and annotations.

Cell ordering (left to right):
1. Query Anomalous cells (leftmost)
2. Query Normal cells
3. Reference Anomalous cells
4. Reference Normal cells (rightmost)

This ensures clear dataset separation while grouping anomalous cells together
within each dataset for easy visual identification.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotPairwiseDistancesDensity.R
\name{plotPairwiseDistancesDensity}
\alias{plotPairwiseDistancesDensity}
\title{Ridgeline Plot of Pairwise Distance Analysis}
\usage{
plotPairwiseDistancesDensity(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_type,
  pc_subset = 1:5,
  distance_metric = c("correlation", "euclidean"),
  correlation_method = c("spearman", "pearson"),
  bandwidth = 0.25,
  assay_name = "logcounts",
  max_cells_query = 5000,
  max_cells_ref = 5000
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} containing the single-cell
expression data and metadata.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing the single-cell
expression data and metadata.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{cell_type}{The cell type for which distances or correlations are calculated.}

\item{pc_subset}{A numeric vector specifying which principal components to use in the analysis. Default is 1:5.
If set to \code{NULL}, the assay data is used directly for computations without dimensionality reduction.}

\item{distance_metric}{The distance metric to use for calculating pairwise distances, such as euclidean, manhattan, etc.
Set to "correlation" to calculate correlation coefficients.}

\item{correlation_method}{The correlation method to use when \code{distance_metric} is "correlation".
Possible values are "pearson" and "spearman".}

\item{bandwidth}{Numeric value controlling the smoothness of the density estimate; smaller values create more detailed curves. Default is 0.25.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}
}
\value{
A ggplot2 object showing ridgeline plots of calculated distances or correlations.
}
\description{
This function calculates pairwise distances or correlations between query and reference cells of a specified cell type
and visualizes the results using ridgeline plots, displaying the density distribution for each comparison.
}
\details{
Designed for \code{\linkS4class{SingleCellExperiment}} objects, this function subsets data for the specified cell type,
computes pairwise distances or correlations, and visualizes these measurements through ridgeline plots.
The plots help evaluate the consistency and differentiation of annotated cell types within single-cell datasets.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Example usage of the function
plotPairwiseDistancesDensity(query_data = query_data,
                             reference_data = reference_data,
                             query_cell_type_col = "SingleR_annotation",
                             ref_cell_type_col = "expert_annotation",
                             cell_type = "CD8",
                             pc_subset = 1:5,
                             distance_metric = "euclidean",
                             correlation_method = "pearson")

}
\seealso{
\code{\link{calculateWassersteinDistance}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/argumentCheck.R
\name{argumentCheck}
\alias{argumentCheck}
\title{Argument Validation for SingleCellExperiment Analysis}
\usage{
argumentCheck(
  query_data = NULL,
  reference_data = NULL,
  query_cell_type_col = NULL,
  ref_cell_type_col = NULL,
  unique_cell_type = FALSE,
  plot_function = FALSE,
  cell_names_query = NULL,
  cell_names_ref = NULL,
  pc_subset_query = NULL,
  pc_subset_ref = NULL,
  common_rotation_genes = FALSE,
  assay_name = NULL,
  max_cells_ref = NULL,
  max_cells_query = NULL
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
If `NULL`, no check is performed.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
If `NULL`, no check is performed.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data}
that identifies the cell types. If `NULL`, no check is performed.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data}
that identifies the cell types. If `NULL`, no check is performed.}

\item{unique_cell_type}{If `TRUE`, there should only be one cell type in the provided \code{\linkS4class{SingleCellExperiment}} objects.
Default is `FALSE`.}

\item{plot_function}{A logical value indicating whether the function is being called to generate a plot. Default is `FALSE`.}

\item{cell_names_query}{A character vector of cell names in query data to be analyzed. If `NULL`, no check is performed.}

\item{cell_names_ref}{A character vector of cell names in reference data to be analyzed. If `NULL`, no check is performed.}

\item{pc_subset_query}{A numeric vector specifying the principal components to be used for the query data. If `NULL`, no check is performed.}

\item{pc_subset_ref}{A numeric vector specifying the principal components to be used for the reference data. If `NULL`, no check is performed.}

\item{common_rotation_genes}{If TRUE, check the rotation matrices of the reference and query data and ensure they have the same genes.
Default is FALSE.}

\item{assay_name}{Name of the assay on which to perform computations. If `NULL`, no check is performed.}

\item{max_cells_ref}{Maximum number of reference cells to retain. If `NULL`, no check is performed.}

\item{max_cells_query}{Maximum number of query cells to retain. If `NULL`, no check is performed.}
}
\value{
None.
}
\description{
This function validates the input arguments for functions that analyze \code{\linkS4class{SingleCellExperiment}}
objects. It checks that the inputs are of the correct types and formats, and that required columns and cell types
are present in the data.
}
\details{
The function performs a series of checks to ensure that:
\itemize{
 \item `query_data` and `reference_data` are \code{\linkS4class{SingleCellExperiment}} objects.
 \item `query_cell_type_col` and `ref_cell_type_col` exist in the column data of their respective \code{\linkS4class{SingleCellExperiment}} objects.
 \item If `unique_cell_type` is `TRUE`, there should only be one cell type in the \code{\linkS4class{SingleCellExperiment}} objects.
 \item `cell_names_query` are valid cell names in the provided query dataset.
 \item `cell_names_ref` are valid cell names in the provided reference dataset.
 \item The PCA subsets specified by `pc_subset_query` and `pc_subset_ref` are valid.
 \item `max_cells_ref` and `max_cells_query` are positive integers when not NULL.
}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.regressPCObject.R, R/regressPC.R
\name{plot.regressPCObject}
\alias{plot.regressPCObject}
\alias{regressPC}
\title{Plot Regression Results on Principal Components}
\usage{
\method{plot}{regressPCObject}(
  x,
  plot_type = c("r_squared", "variance_contribution", "coefficient_heatmap"),
  alpha = 0.05,
  coefficients_include = NULL,
  ...
)

regressPC(
  query_data,
  reference_data = NULL,
  query_cell_type_col,
  ref_cell_type_col = NULL,
  query_batch_col = NULL,
  cell_types = NULL,
  pc_subset = 1:10,
  adjust_method = c("BH", "holm", "hochberg", "hommel", "bonferroni", "BY", "fdr",
    "none"),
  assay_name = "logcounts",
  max_cells_ref = 5000,
  max_cells_query = 5000
)
}
\arguments{
\item{x}{An object of class \code{regressPCObject} containing the output of the \code{regressPC} function.}

\item{plot_type}{Type of plot to generate. Available options:
"r_squared", "variance_contribution", "coefficient_heatmap".}

\item{alpha}{Significance threshold for p-values. Default is 0.05.}

\item{coefficients_include}{Character vector specifying which coefficient types to include
in the coefficient heatmap. Options are \code{c("cell_type", "batch", "interaction")}.
Default is \code{NULL}, which includes all available coefficient types. Only applies
to \code{plot_type = "coefficient_heatmap"}.}

\item{...}{Additional arguments to be passed to the plotting functions.}

\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.
If NULL, the PC scores are regressed against the cell types of the query data.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{query_batch_col}{The column name in the \code{colData} of \code{query_data} that identifies the batch or sample.
If provided, performs interaction analysis with cell types. Default is NULL.}

\item{cell_types}{A character vector specifying the cell types to include in the analysis. If NULL, all cell types are included.}

\item{pc_subset}{A numeric vector specifying which principal components to include in the analysis. Default is PC1 to PC10.}

\item{adjust_method}{A character string specifying the method to adjust the p-values.
Options include "BH", "holm", "hochberg", "hommel", "bonferroni", "BY", "fdr", or "none".
Default is "BH" (Benjamini-Hochberg).}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000.}
}
\value{
The S3 plot method returns a \code{ggplot} object representing the specified plot type.

A \code{list} containing \itemize{ \item summaries of the linear
regression models for each specified principal component, \item the
corresponding R-squared (R2) values, \item the variance contributions for
each principal component, and \item the total variance explained.}
}
\description{
The S3 plot method generates plots to visualize the results of regression analyses
performed on principal components (PCs) against cell types, datasets, or their interactions.

This function performs linear regression of a covariate of interest onto one
or more principal components, based on the data in a \code{\linkS4class{SingleCellExperiment}}
object.
}
\details{
Principal component regression, derived from PCA, can be used to quantify the
variance explained by a covariate of interest. Applications for single-cell
analysis include quantification of batch effects, assessing clustering
homogeneity, and evaluating alignment of query and reference datasets in cell
type annotation settings.

The function supports multiple regression scenarios:
\itemize{
  \item Query only, no batch: PC  cell_type
  \item Query only, with batch: PC  cell_type * batch
  \item Query + Reference, no batch: PC  cell_type * dataset
  \item Query + Reference, with batch: PC  cell_type * batch (where batch includes Reference)
}

When batch information is provided with reference data, batches are labeled as
"Reference" for reference data and "Query_BatchName" for query batches, with
Reference set as the first factor level for interpretation.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Query only analysis
regress_res <- regressPC(query_data = query_data,
                         query_cell_type_col = "expert_annotation",
                         cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                         pc_subset = 1:10)
# Visualize results
plot(regress_res, plot_type = "r_squared")
plot(regress_res, plot_type = "variance_contribution")
plot(regress_res, plot_type = "coefficient_heatmap")

# Query + Reference analysis
regress_res <- regressPC(query_data = query_data,
                         reference_data = reference_data,
                         query_cell_type_col = "SingleR_annotation",
                         ref_cell_type_col = "expert_annotation",
                         cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                         pc_subset = 1:10)
# Visualize results
plot(regress_res, plot_type = "r_squared")
plot(regress_res, plot_type = "variance_contribution")
plot(regress_res, plot_type = "coefficient_heatmap")

}
\references{
Luecken et al. Benchmarking atlas-level data integration in
single-cell genomics. Nature Methods, 19:41-50, 2022.
}
\seealso{
\code{\link{regressPC}}

\code{\link{plot.regressPCObject}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateHotellingPValue.R
\name{calculateHotellingPValue}
\alias{calculateHotellingPValue}
\title{Perform Hotelling's T-squared Test on PCA Scores for Single-cell RNA-seq Data}
\usage{
calculateHotellingPValue(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  pc_subset = 1:5,
  n_permutation = 500,
  assay_name = "logcounts",
  max_cells_query = 5000,
  max_cells_ref = 5000
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{character. The column name in the \code{colData} of \code{query_data}
that identifies the cell types.}

\item{ref_cell_type_col}{character. The column name in the \code{colData} of \code{reference_data}
that identifies the cell types.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.}

\item{pc_subset}{A numeric vector specifying which principal components to include in the plot. Default is PC1 to PC5.}

\item{n_permutation}{Number of permutations to perform for p-value calculation. Default is 500.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}
}
\value{
A named numeric vector of p-values from Hotelling's T-squared test for each cell type.
}
\description{
Computes Hotelling's T-squared test statistic and p-values for each specified cell type
based on PCA-projected data from query and reference datasets.
}
\details{
This function calculates Hotelling's T-squared statistic for comparing multivariate means
between reference and query datasets, projected onto a subset of principal components (PCs).
It performs a permutation test to obtain p-values for each cell type specified.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Get the p-values
p_values <- calculateHotellingPValue(query_data = query_data,
                                     reference_data = reference_data,
                                     query_cell_type_col = "SingleR_annotation",
                                     ref_cell_type_col = "expert_annotation",
                                     pc_subset = 1:10)
round(p_values, 5)

}
\references{
Hotelling, H. (1931). "The generalization of Student's ratio". *Annals of Mathematical Statistics*. 2 (3): 360–378.
doi:10.1214/aoms/1177732979.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/projectSIR.R
\name{conditionalMeans}
\alias{conditionalMeans}
\title{Compute Conditional Means for Cell Types}
\usage{
conditionalMeans(
  reference_data,
  ref_cell_type_col,
  cell_types,
  multiple_cond_means = FALSE,
  assay_name = "logcounts",
  cumulative_variance_threshold = 0.7,
  n_neighbor = 1
)
}
\arguments{
\item{reference_data}{A \code{SingleCellExperiment} object containing the reference data, where rows
represent genes and columns represent cells.}

\item{ref_cell_type_col}{A character string specifying the column in \code{colData(reference_data)} that contains the cell type labels.}

\item{cell_types}{A character vector of cell types for which to compute conditional means.}

\item{multiple_cond_means}{A logical value indicating whether to compute multiple conditional means per cell type.
Defaults to \code{FALSE}.}

\item{assay_name}{A character string specifying the name of the assay to use for the computation. Defaults to \code{"logcounts"}.}

\item{cumulative_variance_threshold}{A numeric value between 0 and 1 specifying the variance threshold
for PCA when computing multiple conditional means. Defaults to \code{0.7}.}

\item{n_neighbor}{An integer specifying the number of nearest neighbors for clustering when computing multiple conditional means.
Defaults to \code{1}.}
}
\value{
A numeric matrix with the conditional means for each cell type. If \code{multiple_cond_means = TRUE}, the matrix
will contain multiple rows for each cell type, representing the different conditional means computed via clustering.
}
\description{
This function computes conditional means for each cell type in the reference data. It can compute
either a single conditional mean per cell type or multiple conditional means, depending on the
specified settings. Principal component analysis (PCA) is used for dimensionality reduction before
clustering when computing multiple conditional means.
}
\details{
The function offers two modes of operation:
- **Single conditional mean per cell type**: For each cell type, it computes the mean expression
  across all observations.
- **Multiple conditional means per cell type**: For each cell type, the function performs PCA to
  reduce dimensionality, followed by clustering to compute multiple conditional means.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regressPC.R
\name{decomposeR2}
\alias{decomposeR2}
\title{Decompose R-squared by Model Components}
\usage{
decomposeR2(pc, indep_var, df)
}
\arguments{
\item{pc}{A character string specifying the principal component column name in the data frame.}

\item{indep_var}{A character string specifying the independent variable specification.
Options are "cell_type", "cell_type * batch", or "cell_type * dataset".}

\item{df}{A data frame containing the principal component scores and categorical predictors.
Must include columns for the specified PC and predictor variables.}
}
\value{
A named list containing R-squared components:
  \item{cell_type}{Numeric value representing the R-squared explained by cell type main effect.}
  \item{batch/dataset}{Numeric value representing the R-squared explained by batch or dataset
    main effect (only for interaction models).}
  \item{interaction}{Numeric value representing the R-squared explained by the interaction term
    (only for interaction models).}
}
\description{
Decomposes the total R-squared from a linear model into individual components
representing the variance explained by main effects (cell type, batch/dataset)
and their interaction using sequential sum of squares.
}
\details{
This function performs R-squared decomposition using a sequential sum of squares approach,
which partitions the total explained variance into additive components. The decomposition
follows the hierarchical structure:
\enumerate{
  \item Cell type main effect
  \item Batch/dataset main effect (after accounting for cell type)
  \item Cell type × batch/dataset interaction (after accounting for main effects)
}

For simple cell type models, only the cell type component is returned. For interaction
models, all three components are computed. The method uses group means and residual
analysis to avoid computationally expensive matrix operations while maintaining
mathematical accuracy equivalent to ANOVA decomposition.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCategorizationEntropy.R
\name{calculateEntropy}
\alias{calculateEntropy}
\title{Calculate Entropy}
\usage{
calculateEntropy(p)
}
\arguments{
\item{p}{A numeric vector representing a probability distribution. The elements should sum to 1.}
}
\value{
A numeric value representing the entropy of the probability distribution.
}
\description{
This function calculates the entropy of a probability distribution.
}
\details{
The entropy is calculated using the formula \eqn{-\sum p \log(p)}, where the sum is over all non-zero elements of \code{p}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateHVGOverlap.R
\name{calculateHVGOverlap}
\alias{calculateHVGOverlap}
\title{Calculate the Overlap Coefficient for Highly Variable Genes}
\usage{
calculateHVGOverlap(reference_genes, query_genes)
}
\arguments{
\item{reference_genes}{A character vector of highly variable genes from the reference dataset.}

\item{query_genes}{A character vector of highly variable genes from the query dataset.}
}
\value{
Overlap coefficient, a value between 0 and 1, where 0 indicates no overlap
and 1 indicates complete overlap of highly variable genes between datasets.
}
\description{
Calculates the overlap coefficient between the sets of highly variable genes from a reference dataset and a query dataset.
}
\details{
The overlap coefficient measures the similarity between two gene sets, indicating how well-aligned
reference and query datasets are in terms of their highly variable genes. This metric is
useful in single-cell genomics to understand the correspondence between different datasets.

The coefficient is calculated using the formula:

\deqn{Coefficient(X, Y) = \frac{|X \cap Y|}{min(|X|, |Y|)}}

where X and Y are the sets of highly variable genes from the reference and query datasets, respectively,
\eqn{|X \cap Y|} is the number of genes common to both \eqn{X} and \eqn{Y}, and \eqn{min(|X|, |Y|)} is the size of the
smaller set among \eqn{X} and \eqn{Y}.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Selecting highly variable genes
ref_var <- scran::getTopHVGs(reference_data, n = 500)
query_var <- scran::getTopHVGs(query_data, n = 500)
overlap_coefficient <- calculateHVGOverlap(reference_genes = ref_var,
                                           query_genes = query_var)
overlap_coefficient

}
\references{
Luecken et al. Benchmarking atlas-level data integration in single-cell genomics. Nature Methods, 19:41-50, 2022.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCategorizationEntropy.R
\name{nElements}
\alias{nElements}
\title{Number of Elements}
\usage{
nElements(X)
}
\arguments{
\item{X}{A matrix or vector.}
}
\value{
An integer representing the number of elements in \code{X}.
}
\description{
This function returns the number of elements in a matrix or vector.
}
\details{
If \code{X} is a matrix, the function returns the product of its dimensions. If \code{X} is a vector, the function returns
its length.
}
\author{
Andrew Ghazi, \email{andrew_ghazi@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downsampleSCE.R
\name{downsampleSCE}
\alias{downsampleSCE}
\title{Downsample SingleCellExperiment Objects}
\usage{
downsampleSCE(
  sce_object,
  cell_type_col = NULL,
  cell_types = NULL,
  max_cells = 2500,
  seed = NULL
)
}
\arguments{
\item{sce_object}{A \code{\linkS4class{SingleCellExperiment}} object to potentially downsample.
May contain PCA, UMAP, TSNE, or other reducedDims.}

\item{cell_type_col}{The column name in colData that contains cell type information.
Required if cell_types is not NULL. Default is NULL.}

\item{cell_types}{A character vector specifying which cell types to retain. If NULL,
no cell type filtering is performed. Default is NULL.}

\item{max_cells}{Maximum number of cells to retain. If the object has fewer cells,
it is returned unchanged. If NULL, no downsampling is performed (all cells are kept).
Default is 2500.}

\item{seed}{Random seed for reproducible downsampling. If NULL, no seed is set.
Default is NULL.}
}
\value{
A \code{\linkS4class{SingleCellExperiment}} object with at most max_cells cells
and optionally filtered by cell types. ReducedDims coordinates are preserved through
standard subsetting. If max_cells is NULL, the original object is returned unchanged.
}
\description{
This internal function downsamples \code{\linkS4class{SingleCellExperiment}} objects while preserving
reducedDims coordinate information (PCA, UMAP, t-SNE, etc.). Optionally, it can also subset by cell types
before downsampling.
}
\details{
The function can perform cell type filtering followed by random downsampling without replacement
when the number of cells exceeds the specified threshold. All reducedDims coordinates are preserved
through standard sce_object subsetting operations. This function does not preserve PCA rotation matrices or
other model-specific attributes.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.calculateTopLoadingGeneShiftsObject.R
\name{plotBoxplot}
\alias{plotBoxplot}
\title{Plot Boxplots for Top Loading Gene Shifts (Two-Panel Summary using ggplot2)}
\usage{
plotBoxplot(
  x,
  cell_type,
  available_pcs,
  plot_by,
  n_genes,
  significance_threshold,
  show_anomalies
)
}
\arguments{
\item{x}{An object of class \code{calculateTopLoadingGeneShiftsObject} containing
expression data and analysis results.}

\item{cell_type}{A character string specifying the cell type to visualize.}

\item{available_pcs}{A character vector of principal components to include in analysis.}

\item{plot_by}{A character string indicating gene selection criterion ("top_loading" or "p_adjusted").}

\item{n_genes}{An integer specifying the number of top genes to display per PC.}

\item{significance_threshold}{A numeric value between 0 and 1 for significance annotation.}

\item{show_anomalies}{Logical indicating whether to show anomaly annotations using
line type differences in boxplot borders.}
}
\value{
A ggplot2 object ready for display, or NULL if no genes meet selection criteria.
  The returned plot contains:
  \itemize{
    \item Left panel: Expression boxplots with dataset-specific fill colors
    \item Right panel: PC loading scatter points with PC-specific colors/shapes
    \item Gene labels on y-axis with significance indicators (*)
    \item P-value annotations on secondary y-axis
    \item Legend showing dataset, PC information, and anomaly status (if applicable)
  }
}
\description{
This internal helper function creates a comprehensive two-panel summary plot
displaying gene expression distributions and principal component loadings.
The visualization uses ggplot2 faceting to create side-by-side panels.
Optionally includes anomaly status information using visual cues.
}
\details{
This function generates a dual-panel ggplot2 visualization where the left panel
shows horizontal boxplots of gene expression distributions comparing Reference
and Query datasets, while the right panel displays PC loading values as points
with adjusted p-values. Gene selection is based on the union of top genes
across specified principal components.

When anomaly information is available and requested, anomalous cells are
distinguished using dashed boxplot borders (normal cells have solid borders).
This approach avoids color conflicts between dataset identification (fill colors)
and PC identification (point colors/shapes).

Visual encoding:
\itemize{
  \item Dataset: Fill colors (Reference = blue, Query = red)
  \item Anomaly status: Line types (Normal = solid, Anomaly = dashed borders)
  \item PC identity: Point colors and shapes in loading panel
}
}
\seealso{
\code{\link{plotHeatmap}}, \code{\link{plot.calculateTopLoadingGeneShiftsObject}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDocumentation.R
\docType{data}
\name{query_data}
\alias{query_data}
\title{Query Single-Cell RNA-Seq Dataset}
\format{
An object of class \code{SingleCellExperiment} with 392 rows and 503 columns.
}
\source{
The HeOrganAtlas dataset, available through the scRNAseq package.
}
\usage{
query_data
}
\description{
This dataset contains the processed query dataset from the HeOrganAtlas dataset for Marrow tissue.
It has been preprocessed to include log-normalized counts, specific metadata columns, annotations
based on SingleR cell type scoring, and PCA, t-SNE, and UMAP results.
}
\details{
This dataset underwent the following steps:
\itemize{
  \item Loads the HeOrganAtlas dataset specifically for Marrow tissue from the \code{scRNAseq} package.
  \item Divides the loaded dataset into a query dataset used for downstream analysis.
  \item Performs log normalization on the query dataset using the function \code{logNormCounts} from the \code{scuttle} package.
  \item Selects specific columns (\code{percent_mito}, \code{expert_annotation}) from the cell metadata for downstream analysis.
  \item Selects highly variable genes (HVGs) using the function \code{getTopHVGs} from the \code{scran} package on the query dataset.
  \item Computes AUC gene set scores using the function \code{AUCell_calcAUC} from the \code{AUCell} package based on a CD4 T cell
        signature containing 12 known CD4 T cell marker genes (IL7R, CCR7, SELL, LEF1, TCF7, LTB, KLF2, IL32, CD2, CD3D, CD3E, CD3G)
        and adds these scores to the query dataset as \code{gene_set_scores}.
  \item Intersects the highly variable genes between the query and reference datasets to obtain common genes for analysis.
  \item Performs Principal Component Analysis (PCA) on the query dataset using the function \code{runPCA} from the \code{scater}
        package.
  \item Performs t-Distributed Stochastic Neighbor Embedding (t-SNE) on the query dataset using the function \code{runTSNE} from the
        \code{scater} package.
  \item Performs Uniform Manifold Approximation and Projection (UMAP) on the query dataset using the function \code{runUMAP} from the
        \code{scater} package.
  \item Adds SingleR annotations (\code{SingleR_annotation}) and annotation scores (\code{annotation_scores}) to the query dataset
        using the function \code{SingleR} from the \code{SingleR} package.
}
}
\examples{
# Load and explore the query dataset
data("query_data")

}
\references{
He, et al. (2020). HeOrganAtlas: a comprehensive human organ atlas based on single-cell RNA sequencing.
}
\seealso{
Use \code{data("query_data")} to load and access the resulting query dataset and the
\code{data("reference_data")} for comparison with the reference dataset.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCategorizationEntropy.R
\name{calculateCategorizationEntropy}
\alias{calculateCategorizationEntropy}
\title{Calculate Categorization Entropy}
\usage{
calculateCategorizationEntropy(
  X,
  inverseNormalTransformationform = FALSE,
  plot = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{X}{A matrix of category scores.}

\item{inverseNormalTransformationform}{If TRUE, apply inverse normal transformation to X. Default is FALSE.}

\item{plot}{If TRUE, plot a histogram of the entropies. Default is TRUE.}

\item{verbose}{If TRUE, display messages about the calculations. Default is TRUE.}
}
\value{
A vector of entropy values for each column in X.
}
\description{
This function takes a matrix of category scores (cell type by
cells) and calculates the entropy of the category probabilities for each
cell. This gives a sense of how confident the cell type assignments are.
High entropy = lots of plausible category assignments = low confidence. Low
entropy = only one or two plausible categories = high confidence. This is
confidence in the vernacular sense, not in the "confidence interval"
statistical sense. Also note that the entropy tells you nothing about
whether or not the assignments are correct -- see the other functionality
in the package for that. This functionality can be used for assessing how
comparatively confident different sets of assignments are (given that the
number of categories is the same).
}
\details{
The function checks if X is already on the probability scale.
Otherwise, it applies softmax columnwise.

You can think about entropies on a scale from 0 to a maximum that depends
on the number of categories. This is the function for entropy (minus input
checking): \code{entropy(p) = -sum(p*log(p))} . If that input vector p is a
uniform distribution over the \code{length(p)} categories, the entropy will
be a high as possible.
}
\examples{
# Simulate 500 cells with scores on 4 possible cell types
X <- rnorm(500 * 4) |> matrix(nrow = 4)
X[1, 1:250] <- X[1, 1:250] + 5 # Make the first category highly scored in the first 250 cells

# The function will issue a message about softmaxing the scores, and the entropy histogram will be
# bimodal since we made half of the cells clearly category 1 while the other half are roughly even.
entropy_scores <- calculateCategorizationEntropy(X)

}
\author{
Andrew Ghazi, \email{andrew_ghazi@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateColors.R
\name{generateColors}
\alias{generateColors}
\title{Generate Paired Colors for Cell Types}
\usage{
generateColors(cell_type_names, paired = FALSE)
}
\arguments{
\item{cell_type_names}{A character vector of cell type names that need to be assigned colors.}

\item{paired}{If TRUE, the colored returned should be paired. Default is FALSE.}
}
\value{
A named character vector where the names are the original cell type names, and the values are the assigned colors.
}
\description{
This function assigns paired colors (light and dark) to a list of cell type names. The colors are selected from various color
palettes in the `pals` package.
}
\details{
The function uses color palettes from the `pals` package to generate colors or pairs of colors (light and dark) for each cell
type name provided. It cycles through different color families (blues, greens, reds, oranges, purples, purd and greys) to create
the colors
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scDiagnostics-package.R
\docType{package}
\name{scDiagnostics-package}
\alias{scDiagnostics-package}
\alias{scDiagnostics}
\title{scDiagnostics: Single-Cell Diagnostics Package}
\description{
`scDiagnostics` is a comprehensive toolkit designed for the analysis and diagnostics of single-cell RNA sequencing (scRNA-seq) data. This package provides functionalities for comparing principal components, visualizing canonical correlation analysis (CCA) outputs, and plotting cell type-specific MDS and PCA projections.
}
\details{
The package includes the following key functionalities, organized by their specific purposes:
}
\section{Visualization of Cell Type Annotations}{

Functions for visualizing differences between query and reference datasets across multiple cell types.
\itemize{
  \item \code{\link{boxplotPCA}}: Boxplots of PCA scores for cell types.
  \item \code{\link{calculateDiscriminantSpace}}: Calculates discriminant space, with a plot method for visualization.
  \item \code{\link{plotCellTypeMDS}}: Creates MDS plots for cell types using query and reference datasets.
  \item \code{\link{plotCellTypePCA}}: Plots principal components for different cell types.
}
}

\section{Evaluation of Dataset Alignment}{

Functions for visualizing differences between query and reference datasets for a specific cell type.
\itemize{
  \item \code{\link{calculateGraphIntegration}}: Calculate a graph network and form communities of cells.
  \item \code{\link{calculateWassersteinDistance}}: Wasserstein distance for different cell types.
  \item \code{\link{comparePCA}}: Compares PCA results, with a plot method for visualization.
  \item \code{\link{comparePCASubspace}}: Compares PCA subspace, with a plot method for visualization.
  \item \code{\link{plotPairwiseDistancesDensity}}: Plots the density of pairwise distances.
}
}

\section{Calculation of Statistical Measures to Compare Two Datasets}{

Functions to compute statistical measures to compare two datasets.
\itemize{
  \item \code{\link{calculateAveragePairwiseCorrelation}}: Calculates average pairwise correlation, with a plot method for visualization.
  \item \code{\link{calculateCramerPValue}}: Calculates the p-value using Cramer's test.
  \item \code{\link{calculateHotellingPValue}}: Calculates the p-value using Hotelling's T-squared test.
  \item \code{\link{calculateMMDPValue}}: Calculates the p-value using maximum mean discrepancy.
  \item \code{\link{regressPC}}: Performs regression on principal components, with a plot method for visualization.
}
}

\section{Evaluation of Marker Gene Alignment}{

Functions for calculating overlap measures of genes between two datasets.
\itemize{
  \item \code{\link{calculateHVGOverlap}}: Calculates overlap of highly variable genes (HVG) between datasets.
  \item \code{\link{calculateVarImpOverlap}}: Calculates overlap of variable importance measures between datasets.
  \item \code{\link{compareMarkers}}: Compares marker genes between datasets.
}
}

\section{Visualization of Marker Expressions}{

Functions for to visualize and compare the expression of markers between a reference and a query dataset.
\itemize{
  \item \code{\link{plotGeneExpressionDimred}}: Plots gene expression in a dimensionality reduction space.
  \item \code{\link{plotMarkerExpression}}: Plots marker expression levels.
}
}

\section{Anomaly Detection (Global and Cell Type-Specific)}{

Functions for detecting anomalies at both the global and cell type-specific levels.
\itemize{
  \item \code{\link{detectAnomaly}}: Detects anomalies in the data, with a plot method for visualization.
  \item \code{\link{calculateCellSimilarityPCA}}: Calculates cell similarity in PCA space, with a plot method for visualization.
}
}

\section{Calculation of Distances Between Specific Cells and Cell Populations}{

Functions for calculating distances between specific cells and cell populations.
\itemize{
  \item \code{\link{calculateCellDistances}}: Calculates distances between cells, with a plot method for visualization.
  \item \code{\link{calculateCellDistancesSimilarity}}: Calculates similarity based on cell distances, with a plot method for visualization.
}
}

\section{Visualization of QC and Annotation Scores}{

Functions for visualizing quality control (QC) metrics or other characteristics of the data.
\itemize{
  \item \code{\link{histQCvsAnnotation}}: Plots histograms of QC metrics versus annotations.
  \item \code{\link{plotGeneSetScores}}: Plots scores of gene sets.
  \item \code{\link{plotQCvsAnnotation}}: Plots QC metrics versus annotations.
}
}

\section{Misc}{

Miscellaneous functions for various tasks.
\itemize{
  \item \code{\link{processPCA}}: Process \code{\linkS4class{SingleCellExperiment}} objects to compute PCA.
  \item \code{\link{projectPCA}}: Projects new data into PCA space.
  \item \code{\link{projectSIR}}: Projects new data into SIR space.
  \item \code{\link{calculateCategorizationEntropy}}: Calculates categorization entropy for clusters.
}

The package is built to facilitate in-depth analysis and visualization of single-cell data, enhancing the understanding of cell type similarities and differences across datasets.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ccb-hms/scDiagnostics}
  \item Report bugs at \url{https://github.com/ccb-hms/scDiagnostics/issues}
}

}
\author{
\strong{Maintainer}: Anthony Christidis \email{anthony-alexander_christidis@hms.harvard.edu} (\href{https://orcid.org/0000-0002-4565-6279}{ORCID})

Authors:
\itemize{
  \item Andrew Ghazi
  \item Smriti Chawla
  \item Ludwig Geistlinger
  \item Robert Gentleman
}

Other contributors:
\itemize{
  \item Nitesh Turaga [contributor]
}

}
\keyword{internal}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/histQCvsAnnotation.R
\name{histQCvsAnnotation}
\alias{histQCvsAnnotation}
\title{Histograms: QC Stats and Annotation Scores Visualization}
\usage{
histQCvsAnnotation(
  sce_object,
  cell_type_col,
  cell_types = NULL,
  qc_col,
  score_col,
  max_cells = NULL
)
}
\arguments{
\item{sce_object}{A \code{\linkS4class{SingleCellExperiment}} containing the single-cell
expression data and metadata.}

\item{cell_type_col}{The column name in the \code{colData} of \code{sce_object}
that contains the cell type labels.}

\item{cell_types}{A vector of cell types to plot (e.g., c("T-cell", "B-cell")).
Defaults to \code{NULL}, which will include all the cells.}

\item{qc_col}{A column name in the \code{colData} of \code{sce_object} that
contains the QC stats of interest.}

\item{score_col}{The column name in the \code{colData} of \code{sce_object} that
contains the cell type scores.}

\item{max_cells}{Maximum number of cells to retain. If the object has fewer cells, it is returned unchanged.
Default is NULL.}
}
\value{
A object containing two histograms displayed side by side.
The first histogram represents the distribution of QC stats,
and the second histogram represents the distribution of annotation scores.
}
\description{
This function generates histograms for visualizing the distribution of quality control (QC) statistics and
annotation scores associated with cell types in single-cell genomic data.
}
\details{
The particularly useful in the analysis of data from single-cell experiments,
where understanding the distribution of these metrics is crucial for quality assessment and
interpretation of cell type annotations.
}
\examples{
data("query_data")

# Generate histograms
histQCvsAnnotation(sce_object = query_data,
                   cell_type_col = "SingleR_annotation",
                   cell_types = c("CD4", "CD8"),
                   qc_col = "percent_mito",
                   score_col = "annotation_scores")

histQCvsAnnotation(sce_object = query_data,
                   cell_type_col = "SingleR_annotation",
                   cell_types = NULL,
                   qc_col = "percent_mito",
                   score_col = "annotation_scores")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/projectSIR.R
\name{projectSIR}
\alias{projectSIR}
\title{Project Query Data Onto SIR Space of Reference Data}
\usage{
projectSIR(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  multiple_cond_means = TRUE,
  cumulative_variance_threshold = 0.7,
  n_neighbor = 1,
  assay_name = "logcounts",
  max_cells_query = 5000,
  max_cells_ref = 5000
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{A character string specifying the column in the \code{colData} of \code{query_data}
that identifies the cell types.}

\item{ref_cell_type_col}{A character string specifying the column in the \code{colData} of \code{reference_data}
that identifies the cell types.}

\item{cell_types}{A character vector of cell types for which to compute conditional means in the reference data.}

\item{multiple_cond_means}{A logical value indicating whether to compute multiple conditional means per cell type
(through PCA and clustering). Defaults to \code{TRUE}.}

\item{cumulative_variance_threshold}{A numeric value between 0 and 1 specifying the variance threshold for PCA
when computing multiple conditional means. Defaults to \code{0.7}.}

\item{n_neighbor}{An integer specifying the number of nearest neighbors for clustering when computing multiple
conditional means. Defaults to \code{1}.}

\item{assay_name}{A character string specifying the assay name on which to perform computations. Defaults to \code{"logcounts"}.}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}
}
\value{
A list containing:
\item{cond_means}{A matrix of the conditional means computed for the reference data.}
\item{rotation_mat}{The rotation matrix obtained from the SVD of the conditional means.}
\item{sir_projections}{A \code{data.frame} containing the SIR projections for both the reference and query datasets.}
\item{percent_var}{The percentage of variance explained by each component of the SIR projection.}
}
\description{
This function projects a query \code{SingleCellExperiment} object onto the SIR (supervised independent
component) space of a reference \code{SingleCellExperiment} object. The SVD of the reference data is
computed on conditional means per cell type, and the query data is projected based on these reference
components.
}
\details{
The genes used for the projection (SVD) must be present in both the reference and query datasets.
The function first computes conditional means for each cell type in the reference data, then performs
SVD on these conditional means to obtain the rotation matrix used for projecting both the reference
and query datasets. The query data is centered and scaled based on the reference data.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Project the query data onto SIR space of reference
sir_output <- projectSIR(query_data = query_data,
                         reference_data = reference_data,
                         query_cell_type_col = "SingleR_annotation",
                         ref_cell_type_col = "expert_annotation")

}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.calculateTopLoadingGeneShiftsObject.R
\name{plotBarplot}
\alias{plotBarplot}
\title{Plot Barplots for Top Loading Gene Fold Changes (Pseudo-Bulk)}
\usage{
plotBarplot(
  x,
  cell_type,
  available_pcs,
  plot_by,
  n_genes,
  significance_threshold,
  show_anomalies,
  show_all_query = TRUE
)
}
\arguments{
\item{x}{An object of class \code{calculateTopLoadingGeneShiftsObject} containing
expression data and analysis results.}

\item{cell_type}{A character string specifying the cell type to visualize.}

\item{available_pcs}{A character vector of principal components to include in analysis.}

\item{plot_by}{A character string indicating gene selection criterion ("top_loading" or "p_adjusted").}

\item{n_genes}{An integer specifying the number of top genes to display per PC.}

\item{significance_threshold}{A numeric value between 0 and 1 for significance annotation.}

\item{show_anomalies}{Logical indicating whether to show anomaly-related bars.}

\item{show_all_query}{Logical indicating whether to show the yellow bar for all query vs reference
comparison. Default is TRUE. When FALSE, only green and red bars are shown.}
}
\value{
A ggplot2 object ready for display, or NULL if no genes meet selection criteria.
}
\description{
This internal helper function creates a barplot visualization showing pseudo-bulk fold changes
between different cell groups for top loading genes.
}
\details{
This function generates a ggplot2 barplot where genes are arranged vertically
in hierarchically clustered order (identical to heatmap gene ordering), and horizontal bars show log2 fold changes
for different pseudo-bulk comparisons vs reference. The function creates an internal heatmap
object with the same parameters to extract the exact gene clustering order, ensuring
consistency between plot types.

Bar colors:
\itemize{
  \item Green: Query non-anomaly (pseudo-bulk) vs Reference (pseudo-bulk)
  \item Yellow: All Query (pseudo-bulk) vs Reference (pseudo-bulk) (shown when show_all_query = TRUE and anomaly data available)
  \item Red: Query anomaly (pseudo-bulk) vs Reference (pseudo-bulk)
}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxplotPCA.R
\name{boxplotPCA}
\alias{boxplotPCA}
\title{Plot Principal Components for Different Cell Types}
\usage{
boxplotPCA(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  pc_subset = 1:5,
  shape = c("box", "violin"),
  assay_name = "logcounts",
  max_cells_query = NULL,
  max_cells_ref = NULL
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the
query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for
the reference cells.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are
included.}

\item{pc_subset}{A numeric vector specifying which principal components to include in the plot. Default is PC1 to PC5.}

\item{shape}{Character string indicating the plot type: "box" for boxplots or "violin" for violin plots.
Default is "box".}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is NULL.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is NULL.}
}
\value{
A ggplot object representing the boxplots or violin plots of specified principal components for the given
        cell types and datasets.
}
\description{
This function generates a \code{ggplot2} visualization of principal components (PCs) for different
cell types across two datasets (query and reference), using either boxplots or violin plots.
}
\details{
The function \code{boxplotPCA} is designed to provide a visualization of principal component analysis (PCA) results. It projects
the query dataset onto the principal components obtained from the reference dataset. The results are then visualized
as boxplots or violin plots, grouped by cell types and datasets (query and reference). This allows for a comparative analysis of the
distributions of the principal components across different cell types and datasets. The function internally calls \code{projectPCA}
to perform the PCA projection. It then reshapes the output data into a long format suitable for ggplot2 plotting.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Plot the PC data with boxplots (default)
pc_plot <- boxplotPCA(query_data = query_data,
                      reference_data = reference_data,
                      cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                      query_cell_type_col = "SingleR_annotation",
                      ref_cell_type_col = "expert_annotation",
                      pc_subset = 1:6)
pc_plot

# Plot the PC data with violin plots
pc_violin <- boxplotPCA(query_data = query_data,
                        reference_data = reference_data,
                        cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
                        query_cell_type_col = "SingleR_annotation",
                        ref_cell_type_col = "expert_annotation",
                        pc_subset = 1:6,
                        shape = "violin")
pc_violin

}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detectAnomaly.R, R/plot.detectAnomalyObject.R
\name{detectAnomaly}
\alias{detectAnomaly}
\alias{plot.detectAnomalyObject}
\title{PCA Anomaly Scores via Isolation Forests with Visualization}
\usage{
detectAnomaly(
  reference_data,
  query_data = NULL,
  ref_cell_type_col,
  query_cell_type_col = NULL,
  cell_types = NULL,
  pc_subset = 1:5,
  n_tree = 500,
  anomaly_threshold = 0.6,
  assay_name = "logcounts",
  max_cells_query = 5000,
  max_cells_ref = 5000,
  ...
)

\method{plot}{detectAnomalyObject}(
  x,
  cell_type = NULL,
  pc_subset = NULL,
  data_type = c("query", "reference"),
  n_tree = 500,
  upper_facet = c("blank", "contour", "ellipse"),
  diagonal_facet = c("density", "ridge", "boxplot", "blank"),
  max_cells_ref = NULL,
  max_cells_query = NULL,
  ...
)
}
\arguments{
\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_data}{An optional \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
If NULL, then the isolation forest anomaly scores are computed for the reference data. Default is NULL.}

\item{ref_cell_type_col}{A character string specifying the column name in the reference dataset containing cell type annotations.}

\item{query_cell_type_col}{A character string specifying the column name in the query dataset containing cell type annotations.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.}

\item{pc_subset}{A numeric vector specifying the indices of the PCs to be included in the plots. If NULL, all PCs
in \code{reference_mat_subset} will be included.}

\item{n_tree}{An integer specifying the number of trees for the isolation forest. Default is 500}

\item{anomaly_threshold}{A numeric value specifying the threshold for identifying anomalies, Default is 0.6.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_query}{Maximum number of query cells to include in the plot. If NULL,
all available query cells are plotted. Default is NULL.}

\item{max_cells_ref}{Maximum number of reference cells to include in the plot. If NULL,
all available reference cells are plotted. Default is NULL.}

\item{...}{Additional arguments passed to the `isolation.forest` function.}

\item{x}{A list object containing the anomaly detection results from the \code{detectAnomaly} function.
Each element of the list should correspond to a cell type and contain \code{reference_mat_subset}, \code{query_mat_subset},
\code{var_explained}, and \code{anomaly}.}

\item{cell_type}{A character string specifying the cell type for which the plots should be generated. This should
be a name present in \code{x}. If NULL, the "Combined" cell type will be plotted. Default is NULL.}

\item{data_type}{A character string specifying whether to plot the "query" data or the "reference" data. Default is "query".}

\item{upper_facet}{Either "blank" (default), "contour", or "ellipse" for the upper facet plots.}

\item{diagonal_facet}{Either "density" (default), "ridge", "boxplot" or "blank" for the diagonal plots.}
}
\value{
A list containing the following components for each cell type and the combined data:
\item{anomaly_scores}{Anomaly scores for each cell in the query data.}
\item{anomaly}{Logical vector indicating whether each cell is classified as an anomaly.}
\item{reference_mat_subset}{PCA projections of the reference data.}
\item{query_mat_subset}{PCA projections of the query data (if provided).}
\item{var_explained}{Proportion of variance explained by the retained principal components.}

The S3 plot method returns a \code{GGally::ggpairs} object representing the PCA plots with anomalies highlighted.
}
\description{
This function detects anomalies in single-cell data by projecting the data onto a PCA space and using an isolation forest
algorithm to identify anomalies.

This S3 plot method generates faceted scatter plots for specified principal component (PC) combinations
within an anomaly detection object. It visualizes the relationship between specified PCs, highlights
anomalies detected by the Isolation Forest algorithm, and provides a background gradient representing
anomaly scores.
}
\details{
This function projects the query data onto the PCA space of the reference data. An isolation forest is then built on the
reference data to identify anomalies in the query data based on their PCA projections. If no query dataset is provided by the user,
the anomaly scores are computed on the reference data itself. Anomaly scores for the data with all combined cell types are also
provided as part of the output.

The function extracts the specified PCs from the given anomaly detection object and generates
scatter plots for each pair of PCs. It uses \code{GGally} to create a pairs plot where each facet
represents a pair of PCs. The plot includes:

1. Lower facets: Scatter plots with a background gradient representing anomaly scores from green (low) to red (high)
2. Diagonal facets: Density, ridge, boxplot or blank visualizations showing the distribution of each PC, separated by anomaly status
3. Upper facets: Blank panels by default, or contour/ellipse plots separated by anomaly status if specified
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Store PCA anomaly data
anomaly_output <- detectAnomaly(reference_data = reference_data,
                                query_data = query_data,
                                ref_cell_type_col = "expert_annotation",
                                query_cell_type_col = "SingleR_annotation",
                                pc_subset = 1:5,
                                n_tree = 500,
                                anomaly_threshold = 0.6)

# Plot the output for a cell type
plot(anomaly_output,
     cell_type = "CD4",
     pc_subset = 1:3,
     data_type = "query")

}
\references{
\itemize{
  \item Liu, F. T., Ting, K. M., & Zhou, Z. H. (2008). Isolation forest. In 2008 Eighth IEEE International Conference on Data Mining (pp. 413-422). IEEE.
  \item \href{https://cran.r-project.org/web/packages/isotree/isotree.pdf}{isotree: Isolation-Based Outlier Detection}
}
}
\seealso{
\code{\link{plot.detectAnomalyObject}}

\code{\link{detectAnomaly}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comparePCA.R, R/plot.comparePCAObject.R
\name{comparePCA}
\alias{comparePCA}
\alias{plot.comparePCAObject}
\title{Compare Principal Components Analysis (PCA) Results}
\usage{
comparePCA(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  pc_subset = 1:5,
  n_top_vars = 50,
  metric = c("cosine", "correlation"),
  correlation_method = c("spearman", "pearson"),
  n_permutations = 0
)

\method{plot}{comparePCAObject}(
  x,
  show_values = TRUE,
  show_significance = TRUE,
  significance_threshold = 0.05,
  color_limits = NULL,
  ...
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{pc_subset}{A numeric vector specifying the subset of principal components (PCs) to compare. Default is the first five PCs.}

\item{n_top_vars}{An integer indicating the number of top loading variables to consider for each PC. Default is 50.}

\item{metric}{The similarity metric to use. It can be either "cosine" or "correlation". Default is "cosine".}

\item{correlation_method}{The correlation method to use if metric is "correlation". It can be "spearman"
or "pearson". Default is "spearman".}

\item{n_permutations}{Number of permutations for statistical significance testing. If 0, no permutation test is performed. Default is 0.}

\item{x}{A \code{comparePCAObject} output from the \code{comparePCA} function.}

\item{show_values}{Logical, whether to display similarity values on the heatmap. Default is TRUE.}

\item{show_significance}{Logical, whether to display significance indicators (requires permutation test). Default is TRUE.}

\item{significance_threshold}{Numeric, p-value threshold for significance. Default is 0.05.}

\item{color_limits}{Numeric vector of length 2 specifying color scale limits. If NULL, uses data range.}

\item{...}{Additional arguments passed to the plotting function.}
}
\value{
A list containing:
  \item{similarity_matrix}{A matrix comparing the principal components of the reference and query datasets.}
  \item{top_variables}{A list containing the top loading variables for each PC pair comparison.}
  \item{p_values}{A matrix of permutation p-values (if n_permutations > 0).}
  \item{metric}{The similarity metric used.}
  \item{n_top_vars}{Number of top variables used.}

A \code{ggplot} object representing the heatmap of similarities.
}
\description{
This function compares the principal components (PCs) obtained from separate PCA on reference and query
datasets for a single cell type using either cosine similarity or correlation.

The S3 plot method generates a heatmap to visualize the similarities between
principal components from the output of the \code{comparePCA} function.
}
\details{
This function compares the PCA results between the reference and query datasets by computing cosine
similarities or correlations between the loadings of top variables for each pair of principal components. It first
extracts the PCA rotation matrices from both datasets and identifies the top variables with highest loadings for
each PC. Then, it computes the cosine similarities or correlations between the loadings of top variables for each
pair of PCs using vectorized operations for improved performance. The resulting matrix contains the similarity values,
where rows represent reference PCs and columns represent query PCs.

The S3 plot method creates an enhanced heatmap visualization with options to display
statistical significance and similarity values. The heatmap uses a blue-white-red
color gradient for similarity values, and optionally overlays significance indicators.
}
\examples{
# Load libraries
library(scran)
library(scater)

# Load data
data("reference_data")
data("query_data")

# Extract CD4 cells
ref_data_subset <- reference_data[, which(reference_data$expert_annotation == "CD4")]
query_data_subset <- query_data[, which(query_data$expert_annotation == "CD4")]

# Selecting highly variable genes (can be customized by the user)
ref_top_genes <- getTopHVGs(ref_data_subset, n = 500)
query_top_genes <- getTopHVGs(query_data_subset, n = 500)

# Intersect the gene symbols to obtain common genes
common_genes <- intersect(ref_top_genes, query_top_genes)
ref_data_subset <- ref_data_subset[common_genes,]
query_data_subset <- query_data_subset[common_genes,]

# Run PCA on datasets separately
ref_data_subset <- runPCA(ref_data_subset)
query_data_subset <- runPCA(query_data_subset)

# Call the PCA comparison function
similarity_mat <- comparePCA(query_data = query_data_subset,
                             reference_data = ref_data_subset,
                             query_cell_type_col = "expert_annotation",
                             ref_cell_type_col = "expert_annotation",
                             pc_subset = 1:5,
                             n_top_vars = 50,
                             metric = c("cosine", "correlation")[1],
                             correlation_method = c("spearman", "pearson")[1],
                             n_permutation = 100)

# Create the heatmap
plot(similarity_mat, show_significance = TRUE)

}
\seealso{
\code{\link{plot.comparePCAObject}}

\code{\link{comparePCA}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.regressPCObject.R
\name{plotVarianceContribution}
\alias{plotVarianceContribution}
\title{Generate Variance Contribution Bar Plot with Component Breakdown}
\usage{
plotVarianceContribution(x, ...)
}
\arguments{
\item{x}{An object of class \code{regressPCObject} containing regression results
from \code{regressPC} function.}

\item{...}{Additional arguments passed to the plotting function (currently unused).}
}
\value{
A \code{ggplot2} object representing the variance contribution bar plot
  with optional component breakdown.
}
\description{
Creates a bar plot visualization of variance contributions for each principal component,
showing how much total dataset variance is explained by the regression model.
When available, displays stacked bars showing individual component contributions
(cell type, batch/dataset, and interaction effects).
}
\details{
This function visualizes the variance contribution of each principal component,
calculated as the product of PC variance and R-squared values. The variance
contribution represents the percentage of total dataset variance explained by
the regression model for each PC.

When component decomposition is available, the function creates stacked bars with:
\itemize{
  \item Cell type main effect contribution (blue)
  \item Batch or dataset main effect contribution (orange)
  \item Interaction effect contribution (green)
}

The plot subtitle includes the total variance explained across all principal
components. PC labels show the individual variance percentage for each component.
The function automatically selects appropriate PCA variance values based on
analysis type (query-only vs. query+reference).
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/processPCA.R
\name{processPCA}
\alias{processPCA}
\title{Process PCA for SingleCellExperiment Objects}
\usage{
processPCA(
  sce_object,
  assay_name = "logcounts",
  n_hvgs = 2000,
  max_cells = NULL
)
}
\arguments{
\item{sce_object}{A \code{\linkS4class{SingleCellExperiment}} object to process.}

\item{assay_name}{Name of the assay to use for HVG selection and PCA computation.
Should contain log-normalized expression values. Default is "logcounts".}

\item{n_hvgs}{Number of highly variable genes to select for PCA computation. Default is 2000.}

\item{max_cells}{Maximum number of cells to retain if downsampling is needed for PCA computation.
If NULL, no downsampling is performed. Default is NULL.}
}
\value{
A \code{\linkS4class{SingleCellExperiment}} object with valid PCA in the reducedDims slot,
including rotation matrix and percentVar attributes. Will have original cell count if PCA was valid,
or at most max_cells if PCA was computed.
}
\description{
This function ensures that a \code{\linkS4class{SingleCellExperiment}} object has valid PCA computed using
highly variable genes when needed. It only performs downsampling when PCA computation
is required, preserving existing valid PCA computations without modification.
}
\details{
The function performs the following operations:
\itemize{
  \item Checks if PCA exists and is valid in the provided \code{\linkS4class{SingleCellExperiment}} object
  \item Validates PCA integrity including rotation matrix, percentVar, gene consistency, and dimensions
  \item If PCA is valid, returns the object unchanged (no downsampling)
  \item If PCA is missing or invalid and dataset is large, downsamples before computing PCA
  \item Computes PCA using highly variable genes when PCA is missing or invalid
  \item Utilizes scran for HVG selection and scater for PCA computation (soft dependencies)
}

The downsampling strategy uses random sampling without replacement and only occurs
when PCA computation is necessary. This preserves expensive pre-computed PCA results
while ensuring computational efficiency for new PCA computations.

PCA validation includes checking for:
\itemize{
  \item Presence of PCA in reducedDims
  \item Existence of rotation matrix and percentVar attributes
  \item Gene consistency between rotation matrix and current assay
  \item Dimension consistency between PCA coordinates and cell count
}
}
\note{
This function requires the scran and scater packages for HVG selection and PCA computation.
These packages should be installed via BiocManager::install(c("scran", "scater")).

Objects with existing valid PCA are returned unchanged to preserve expensive pre-computations.
Only datasets requiring PCA computation are subject to downsampling.
}
\examples{
library(SingleCellExperiment)

# Load data
data("reference_data")
data("query_data")

# Example 1: Dataset without PCA (will compute PCA)
query_no_pca <- query_data
reducedDims(query_no_pca) <- list()  # Remove existing PCA

processed_query <- processPCA(sce_object = query_no_pca, n_hvgs = 500)
"PCA" \%in\% reducedDimNames(processed_query)  # Should be TRUE
ncol(processed_query)  # Should be 503 (unchanged)

# Example 2: Dataset with existing valid PCA (will be preserved)
processed_existing <- processPCA(sce_object = query_data, n_hvgs = 500)
ncol(processed_existing)  # Should be 503 (unchanged, no downsampling)

# Example 3: Large dataset requiring downsampling for PCA computation
ref_no_pca <- reference_data
reducedDims(ref_no_pca) <- list()  # Remove existing PCA

processed_large <- processPCA(sce_object = ref_no_pca,
                              n_hvgs = 800,
                              max_cells = 1000)
ncol(processed_large)  # Should be 1000 (downsampled for PCA computation)

# Example 4: Large dataset with existing PCA (no downsampling)
processed_large_existing <- processPCA(sce_object = reference_data,
                                       n_hvgs = 800,
                                       max_cells = 1000)
ncol(processed_large_existing)  # Should be 1500 (preserved, no downsampling)

}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotCellTypePCA.R
\name{plotCellTypePCA}
\alias{plotCellTypePCA}
\title{Plot Principal Components for Different Cell Types}
\usage{
plotCellTypePCA(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  pc_subset = 1:5,
  assay_name = "logcounts",
  lower_facet = c("scatter", "contour", "ellipse", "blank"),
  diagonal_facet = c("ridge", "density", "boxplot"),
  upper_facet = c("blank", "scatter", "contour", "ellipse"),
  max_cells_query = 2000,
  max_cells_ref = 2000
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.}

\item{pc_subset}{A numeric vector specifying which principal components to include in the plot. Default is 1:5.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{lower_facet}{Type of plot to use for the lower panels. Either "scatter" (default), "contour", "ellipse", or "blank".}

\item{diagonal_facet}{Type of plot to use for the diagonal panels. Either "ridge" (default), "density", or "boxplot".}

\item{upper_facet}{Type of plot to use for the upper panels. Either "blank" (default), "scatter", "contour", or "ellipse".}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 2000.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 2000.}
}
\value{
A ggmatrix object representing a pairs plot of specified principal components for the given cell types and datasets.
}
\description{
This function plots the principal components for different cell types in the query and reference datasets.
}
\details{
This function projects the query dataset onto the principal component space of the reference dataset and then plots the
specified principal components for the specified cell types.
It uses the `projectPCA` function to perform the projection and \code{GGally} to create the pairs plot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateWassersteinDistance.R,
%   R/plot.calculateWassersteinDistanceObject.R
\name{calculateWassersteinDistance}
\alias{calculateWassersteinDistance}
\alias{plot.calculateWassersteinDistanceObject}
\title{Compute Wasserstein Distance Distributions Between Query and Reference Datasets}
\usage{
calculateWassersteinDistance(
  query_data,
  reference_data,
  ref_cell_type_col,
  query_cell_type_col,
  cell_types = NULL,
  pc_subset = 1:5,
  n_resamples = 300,
  assay_name = "logcounts",
  max_cells_query = 5000,
  max_cells_ref = 5000
)

\method{plot}{calculateWassersteinDistanceObject}(x, cell_types = NULL, bandwidth = NULL, ...)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing a numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object with a numeric expression matrix for the reference cells.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies cell types.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies cell types.}

\item{cell_types}{A character vector specifying which cell types to plot. If NULL, all cell types are plotted.}

\item{pc_subset}{A numeric vector specifying which principal components to use. Default is \code{1:5}.}

\item{n_resamples}{An integer specifying the number of resamples to generate each distribution. Default is \code{300}.}

\item{assay_name}{The name of the assay to use for computations. Default is \code{"logcounts"}.}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}

\item{x}{A list object containing the Wasserstein distance results from the \code{calculateWassersteinDistance} function.}

\item{bandwidth}{A numeric value specifying the bandwidth for density estimation. If NULL (default), automatic bandwidth selection is used.}

\item{...}{Additional arguments for future extensions.}
}
\value{
A list with the following components:
\item{ref_ref_dist}{A named list of numeric vectors containing Wasserstein distances computed from resampled pairs within the reference dataset for each cell type.}
\item{ref_query_dist}{A named list of numeric vectors containing Wasserstein distances between reference and query datasets for each cell type.}
\item{probability_superiority}{A named numeric vector showing the probability that ref-query distances are larger than ref-ref distances for each cell type.}
\item{cell_types}{A character vector containing the cell types analyzed.}

A ggplot2 object representing the comparison of Wasserstein distance distributions.
}
\description{
This function calculates distributions of Wasserstein distances between reference-reference
pairs and reference-query pairs for each specified cell type, after projecting them into
a shared PCA space. It then computes the probability of superiority to assess whether
reference-query distances tend to be larger than reference-reference distances.

This function generates ridge plots comparing reference-reference
and reference-query Wasserstein distance distributions for each cell type.
}
\details{
The function projects the query dataset onto the PCA space defined by the reference dataset.
For each cell type, it computes two distributions: (1) Wasserstein distances between randomly
sampled pairs within the reference dataset (null distribution), and (2) Wasserstein distances
between reference and query dataset samples. It then calculates the probability of superiority,
which represents the probability that a randomly selected ref-query distance is larger than
a randomly selected ref-ref distance.

The function creates faceted ridge plots showing two clearly separated density curves for each
cell type: one for the reference-reference distribution (null) and one for the
reference-query distribution.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Compute Wasserstein distance distributions for all cell types
wasserstein_data <- calculateWassersteinDistance(query_data = query_data,
                                                 reference_data = reference_data,
                                                 query_cell_type_col = "expert_annotation",
                                                 ref_cell_type_col = "expert_annotation",
                                                 pc_subset = 1:5,
                                                 n_resamples = 100)
plot(wasserstein_data)

}
\references{
Schuhmacher, D., Bernhard, S., & Book, M. (2019). "A Review of Approximate Transport in Machine Learning".
In \emph{Journal of Machine Learning Research} (Vol. 20, No. 117, pp. 1-61).
}
\seealso{
\code{\link{plot.calculateWassersteinDistanceObject}}

\code{\link{calculateWassersteinDistance}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCellDistancesSimilarity.R
\name{calculateCellDistancesSimilarity}
\alias{calculateCellDistancesSimilarity}
\title{Function to Calculate Bhattacharyya Coefficients and Hellinger Distances}
\usage{
calculateCellDistancesSimilarity(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  cell_names_query,
  pc_subset = 1:5,
  assay_name = "logcounts",
  max_cells_ref = 5000
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data}
that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data}
that identifies the cell types.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are
included.}

\item{cell_names_query}{A character vector specifying the names of the query cells for which to compute distance measures.}

\item{pc_subset}{A numeric vector specifying which principal components to include in the plot. Default is 1:5.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}
}
\value{
A list containing distance data for each cell type. Each entry in the list contains:
\describe{
  \item{ref_distances}{A vector of all pairwise distances within the reference subset for the cell type.}
  \item{query_to_ref_distances}{A matrix of distances from each query cell to all reference cells for the cell type.}
}
}
\description{
This function computes Bhattacharyya coefficients and Hellinger distances to quantify the similarity of density
distributions between query cells and reference data for each cell type.
}
\details{
This function first computes distance data using the \code{calculateCellDistances} function, which calculates
pairwise distances between cells within the reference data and between query cells and reference cells in the PCA space.
Bhattacharyya coefficients and Hellinger distances are calculated to quantify the similarity of density distributions between query
cells and reference data for each cell type. Bhattacharyya coefficient measures the similarity of two probability distributions,
while Hellinger distance measures the distance between two probability distributions.

Bhattacharyya coefficients range between 0 and 1. A value closer to 1 indicates higher similarity between distributions, while a value
closer to 0 indicates lower similarity

Hellinger distances range between 0 and 1. A value closer to 0 indicates higher similarity between distributions, while a value
closer to 1 indicates lower similarity.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Plot the PC data
distance_data <- calculateCellDistances(query_data = query_data,
                                        reference_data = reference_data,
                                        query_cell_type_col = "SingleR_annotation",
                                        ref_cell_type_col = "expert_annotation",
                                        pc_subset = 1:10)

# Identify outliers for CD4
cd4_anomalies <- detectAnomaly(reference_data = reference_data,
                               query_data = query_data,
                               query_cell_type_col = "SingleR_annotation",
                               ref_cell_type_col = "expert_annotation",
                               pc_subset = 1:10,
                               n_tree = 500,
                               anomaly_threshold = 0.5)
cd4_top6_anomalies <- names(sort(cd4_anomalies$CD4$query_anomaly_scores, decreasing = TRUE)[1:6])

# Get overlap measures
overlap_measures <- calculateCellDistancesSimilarity(query_data = query_data,
                                                     reference_data = reference_data,
                                                     cell_names_query = cd4_top6_anomalies,
                                                     query_cell_type_col = "SingleR_annotation",
                                                     ref_cell_type_col = "expert_annotation",
                                                     pc_subset = 1:10)
overlap_measures

}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotCellTypeMDS.R
\name{plotCellTypeMDS}
\alias{plotCellTypeMDS}
\title{Plot Reference and Query Cell Types using MDS}
\usage{
plotCellTypeMDS(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  assay_name = "logcounts",
  max_cells_query = 5000,
  max_cells_ref = 5000
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} containing the single-cell
expression data and metadata.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing the single-cell
expression data and metadata.}

\item{query_cell_type_col}{The column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{The column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}
}
\value{
A ggplot object representing the MDS scatter plot with cell type coloring.
}
\description{
This function facilitates the assessment of similarity between reference and query datasets
through Multidimensional Scaling (MDS) scatter plots. It allows the visualization of cell types,
color-coded with user-defined custom colors, based on a dissimilarity matrix computed from a
user-selected gene set. If MDS coordinates are precomputed in reducedDims, they will be used;
otherwise, MDS will be computed from scratch.
}
\details{
The function first checks if MDS coordinates are available in the reducedDims of both datasets.
If precomputed MDS is found, it uses those coordinates directly for visualization.

If MDS is not precomputed, the function selects specific subsets of cells from
both reference and query datasets. It then calculates Spearman correlations between gene expression profiles,
deriving a dissimilarity matrix. This matrix undergoes Classical Multidimensional Scaling (MDS) for
visualization, presenting cell types in a scatter plot, distinguished by colors defined by the user.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Generate the MDS scatter plot with cell type coloring
mds_plot <- plotCellTypeMDS(query_data = query_data,
                            reference_data = reference_data,
                            cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid")[1:4],
                            query_cell_type_col = "SingleR_annotation",
                            ref_cell_type_col = "expert_annotation")
mds_plot

}
\references{
\itemize{
\item Kruskal, J. B. (1964). "Multidimensional scaling by optimizing goodness of fit to a nonmetric hypothesis". *Psychometrika*, 29(1), 1-27. doi:10.1007/BF02289565.
\item Borg, I., & Groenen, P. J. F. (2005). *Modern multidimensional scaling: Theory and applications* (2nd ed.). Springer Science & Business Media. doi:10.1007/978-0-387-25975-1.
}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.regressPCObject.R
\name{plotRSquared}
\alias{plotRSquared}
\title{Generate R-squared Bar Plot with Component Breakdown}
\usage{
plotRSquared(x, ...)
}
\arguments{
\item{x}{An object of class \code{regressPCObject} containing regression results
from \code{regressPC} function.}

\item{...}{Additional arguments passed to the plotting function (currently unused).}
}
\value{
A \code{ggplot2} object representing the R-squared bar plot with optional
  component breakdown.
}
\description{
Creates a bar plot visualization of R-squared values for each principal component,
with optional stacked bars showing the contribution of individual model components
(cell type, batch/dataset, and interaction effects) when component decomposition
is available.
}
\details{
This function generates either a simple bar plot or a stacked bar plot depending
on the availability of component-wise R-squared decomposition in the input object.
When component breakdown is available, the bars are stacked to show:
\itemize{
  \item Cell type main effect (blue)
  \item Batch or dataset main effect (orange)
  \item Interaction effect (green)
}

Principal component labels include the percentage of total variance explained by
each PC. The total R-squared value is displayed above each bar. For query-only
analyses, the function uses query PCA variance; for query+reference analyses,
it uses reference PCA variance.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateDiscriminantSpace.R
\name{ledoitWolf}
\alias{ledoitWolf}
\title{Ledoit-Wolf Covariance Matrix Estimation}
\usage{
ledoitWolf(class_data)
}
\arguments{
\item{class_data}{A numeric matrix or data frame containing the data for covariance estimation,
where rows represent observations and columns represent variables.}
}
\value{
A numeric matrix representing the Ledoit-Wolf estimated covariance matrix.
}
\description{
Estimate the covariance matrix using the Ledoit-Wolf shrinkage method.
}
\details{
This function computes the Ledoit-Wolf shrinkage covariance matrix estimator,
which improves the accuracy of the sample covariance matrix by shrinking it
towards a structured estimator, typically the diagonal matrix with the mean
of variances as its diagonal elements.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.regressPCObject.R
\name{.getAvailableCoefficients}
\alias{.getAvailableCoefficients}
\title{Determine Available Coefficient Types}
\usage{
.getAvailableCoefficients(x)
}
\arguments{
\item{x}{An object of class \code{regressPCObject} containing regression results
from \code{regressPC} function.}
}
\value{
A character vector of available coefficient types.
}
\description{
Helper function to determine which coefficient types are available in a regressPCObject
based on the model type and data structure.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateSIRSpace.R,
%   R/plot.calculateSIRSpaceObject.R
\name{calculateSIRSpace}
\alias{calculateSIRSpace}
\alias{plot.calculateSIRSpaceObject}
\title{Calculate Sliced Inverse Regression (SIR) Space for Different Cell Types}
\usage{
calculateSIRSpace(
  query_data,
  reference_data,
  query_cell_type_col,
  ref_cell_type_col,
  cell_types = NULL,
  multiple_cond_means = TRUE,
  cumulative_variance_threshold = 0.7,
  n_neighbor = 1,
  assay_name = "logcounts",
  max_cells_query = 5000,
  max_cells_ref = 5000
)

\method{plot}{calculateSIRSpaceObject}(
  x,
  plot_type = c("scores", "loadings"),
  cell_types = NULL,
  sir_subset = 1:5,
  lower_facet = c("scatter", "contour", "ellipse", "blank"),
  diagonal_facet = c("ridge", "density", "boxplot", "blank"),
  upper_facet = c("blank", "scatter", "contour", "ellipse"),
  n_top = 10,
  max_cells_ref = NULL,
  max_cells_query = NULL,
  ...
)
}
\arguments{
\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing the numeric expression matrix for the query cells.}

\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing the numeric expression matrix for the reference cells.}

\item{query_cell_type_col}{A character string specifying the column name in the \code{colData} of \code{query_data} that identifies the cell types.}

\item{ref_cell_type_col}{A character string specifying the column name in the \code{colData} of \code{reference_data} that identifies the cell types.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are included. Only used when plot_type = "scores".}

\item{multiple_cond_means}{Logical. Whether to compute conditional means for multiple conditions in the reference dataset. Default is TRUE.}

\item{cumulative_variance_threshold}{A numeric value specifying the cumulative variance threshold for selecting principal components. Default is 0.7.}

\item{n_neighbor}{A numeric value specifying the number of neighbors for computing the SIR space. Default is 1.}

\item{assay_name}{A character string specifying the name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells_query}{Maximum number of query cells to include in the plot. If NULL,
all available query cells are plotted. Default is NULL. Only used when plot_type = "scores".}

\item{max_cells_ref}{Maximum number of reference cells to include in the plot. If NULL,
all available reference cells are plotted. Default is NULL. Only used when plot_type = "scores".}

\item{x}{An object of class \code{calculateSIRSpaceObject} containing SIR projections.}

\item{plot_type}{A character string specifying the type of plot. Either "scores" (default) for SIR projections or "loadings" for variable loadings.}

\item{sir_subset}{A numeric vector specifying which SIR components to include in the plot. Default is 1:5.}

\item{lower_facet}{Type of plot to use for the lower panels. Either "scatter" (default), "contour", "ellipse", or "blank". Only used when plot_type = "scores".}

\item{diagonal_facet}{Type of plot to use for the diagonal panels. Either "ridge" (default), "density", "boxplot" or "blank". Only used when plot_type = "scores".}

\item{upper_facet}{Type of plot to use for the upper panels. Either "blank" (default), "scatter", "contour", or "ellipse". Only used when plot_type = "scores".}

\item{n_top}{A numeric value specifying the number of n_top variables (by absolute loading value) to display. Default is 10 Only used when plot_type = "loadings".}

\item{...}{Additional arguments passed to the plotting function.}
}
\value{
A list containing the SIR projections, rotation matrix, and percentage of variance explained for the given cell types.

A ggmatrix object representing a pairs plot of specified SIR components for the given cell types and datasets when plot_type = "scores", or a ggplot object showing loadings when plot_type = "loadings".
}
\description{
This function calculates the SIR space projections for different cell types in the query and reference datasets.

This function plots the Sliced Inverse Regression (SIR) components for different cell types in query and reference datasets.
}
\details{
The function projects the query dataset onto the SIR space of the reference dataset based on shared cell types.
It computes conditional means for the reference dataset, extracts the SVD components, and performs the projection
of both the query and reference data. It uses the `projectSIR` function to perform the actual projection and
allows the user to specify particular cell types for analysis.

This function visualizes the SIR projections for specified cell types, providing a pairs plot of the SIR components.
It offers various visualization options for different facets of the plot including scatter plots, contours, ellipses, and density plots.
When plot_type is "loadings", it creates horizontal bar plots showing the n_top contributing variables for each SIR component.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Compute important variables for all pairwise cell comparisons
sir_output <- calculateSIRSpace(reference_data = reference_data,
                                query_data = query_data,
                                query_cell_type_col = "expert_annotation",
                                ref_cell_type_col = "expert_annotation",
                                multiple_cond_means = TRUE,
                                cumulative_variance_threshold = 0.9,
                                n_neighbor = 1)

# Generate plots SIR projections
plot(sir_output,
     sir_subset = 1:5,
     cell_types = c("CD4", "CD8", "B_and_plasma", "Myeloid"),
     lower_facet = "scatter",
     diagonal_facet = "boxplot",
     upper_facet = "blank")

# Plot top loadings
plot(sir_output,
     sir_subset = 1:5,
     plot_type = "loadings",
     n_top = 10)

}
\seealso{
\code{\link{plot.calculateSIRSpaceObject}}

\code{\link{calculateSIRSpace}}
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateVarImpOverlap.R
\name{calculateVarImpOverlap}
\alias{calculateVarImpOverlap}
\title{Compare Gene Importance Across Datasets Using Random Forest}
\usage{
calculateVarImpOverlap(
  reference_data,
  query_data = NULL,
  ref_cell_type_col,
  query_cell_type_col = NULL,
  cell_types = NULL,
  n_tree = 500,
  n_top = 50,
  assay_name = "logcounts",
  max_cells_ref = 5000,
  max_cells_query = 5000
)
}
\arguments{
\item{reference_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the reference cells.}

\item{query_data}{A \code{\linkS4class{SingleCellExperiment}} object containing numeric expression matrix for the query cells.
If NULL, then the variable importance scores are only computed for the reference data. Default is NULL.}

\item{ref_cell_type_col}{A character string specifying the column name in the reference dataset containing cell type annotations.}

\item{query_cell_type_col}{A character string specifying the column name in the query dataset containing cell type annotations.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.}

\item{n_tree}{An integer specifying the number of trees to grow in the Random Forest. Default is 500.}

\item{n_top}{An integer specifying the number of top genes to consider when comparing variable importance scores. Default is 50.}

\item{assay_name}{Name of the assay on which to perform computations. Defaults to \code{"logcounts"}.}

\item{max_cells_ref}{Maximum number of reference cells to retain after cell type filtering. If NULL,
no downsampling of reference cells is performed. Default is 5000.}

\item{max_cells_query}{Maximum number of query cells to retain after cell type filtering. If NULL,
no downsampling of query cells is performed. Default is 5000.}
}
\value{
A list containing three elements:
\item{var_imp_ref}{A list of data frames containing variable importance scores for each combination of cell types in the reference
dataset.}
\item{var_imp_query}{A list of data frames containing variable importance scores for each combination of cell types in the query
dataset.}
\item{var_imp_comparison}{A named vector indicating the proportion of top genes that overlap between the reference and query
datasets for each combination of cell types.}
}
\description{
This function identifies and compares the most important genes for differentiating cell types between a query dataset
and a reference dataset using Random Forest.
}
\details{
This function uses the Random Forest algorithm to calculate the importance of genes in differentiating between cell types
within both a reference dataset and a query dataset. The function then compares the top genes identified in both datasets to determine
the overlap in their importance scores.
}
\examples{
# Load data
data("reference_data")
data("query_data")

# Compute important variables for all pairwise cell comparisons
rf_output <- calculateVarImpOverlap(reference_data = reference_data,
                                    query_data = query_data,
                                    query_cell_type_col = "SingleR_annotation",
                                    ref_cell_type_col = "expert_annotation",
                                    n_tree = 500,
                                    n_top = 50)

# Comparison table
rf_output$var_imp_comparison

}
\references{
Breiman, L. (2001). "Random forests". *Machine Learning*, 45(1), 5-32. doi:10.1023/A:1010933404324.
}
\author{
Anthony Christidis, \email{anthony-alexander_christidis@hms.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotGeneExpressionDimred.R
\name{plotGeneExpressionDimred}
\alias{plotGeneExpressionDimred}
\title{Visualize gene expression on a dimensional reduction plot}
\usage{
plotGeneExpressionDimred(
  sce_object,
  method = c("TSNE", "UMAP", "PCA"),
  pc_subset = 1:5,
  feature,
  cell_type_col,
  cell_types = NULL,
  assay_name = "logcounts",
  max_cells = 2000
)
}
\arguments{
\item{sce_object}{An object of class \code{\linkS4class{SingleCellExperiment}} containing log-transformed expression matrix and other metadata.
It can be either a reference or query dataset.}

\item{method}{The reduction method to use for visualization. It should be one of the supported methods: "TSNE", "UMAP", or "PCA".}

\item{pc_subset}{An optional vector specifying the principal components (PCs) to include in the plot if method = "PCA".
Default is 1:5.}

\item{feature}{A character string representing the name of the gene or feature to be visualized.}

\item{cell_type_col}{The column name in the \code{colData} of \code{sce_object} that identifies the cell types.}

\item{cell_types}{A character vector specifying the cell types to include in the plot. If NULL, all cell types are included.}

\item{assay_name}{Name of the assay on which to perform computations. Default is "logcounts".}

\item{max_cells}{Maximum number of cells to retain. If the object has fewer cells, it is returned unchanged.
Default is 2000.}
}
\value{
A ggplot object representing the dimensional reduction plot with gene expression.
}
\description{
This function plots gene expression on a dimensional reduction plot using methods like t-SNE, UMAP, or PCA. Each single cell is color-coded based on the expression of a specific gene or feature.
}
\examples{
# Load data
data("query_data")

# Plot gene expression on PCA plot
plotGeneExpressionDimred(sce_object = query_data,
                         cell_type_col = "SingleR_annotation",
                         method = "PCA",
                         pc_subset = 1:5,
                         feature = "CD8A",
                         cell_types = "CD4")

}
